
# cypress typescript


This allows you to write tests in typescript and run them with cypress.io. The typescript files are automatically compiled using the
[Typescript with Webpack](https://github.com/cypress-io/cypress-example-recipes/tree/master/examples/preprocessors__typescript-webpack) plugin.


## Installation
To install:

* `git clone git@github.infra.cloudera.com:st/viztest.git`
* `cd viztest`
* `git checkout viz`
* `cd cypress`
* `brew install yarn`
* `yarn install`

## Running tests
You should now be able to run cypress.io with:

* `npm run cy-open` (Note: switch the browser to Electron in the cypress.io by using the drop-down in the top right corner. Some tests randomly freeze and fail while running the cypress.io in Chrome, this doesn't happen in the Electron. For debugging use the cypress.io in Electron, but please make sure before merge that tests pass in Chrome with `npm run cy-run`, which runs tests in Chrome in headless mode and doesn't have any freezing issues.)

or for the larry example:

* `CYPRESS_ENABLE_LARRY_TESTS=True npm run cy-open`

## Directory structure and important files
* cypress/tests - This is where the tests go
* cypress/tests/tbd/larry.ts - This is Sushil's sample larry test converted to typescript
* cypress/lib - Test support libraries
* cypress/lib/arctest - The the `arctest` library exported as a typescript module.  This uses the [barrel](https://basarat.gitbooks.io/typescript/docs/tips/barrel.html) pattern to reexport submodules.
* cypress_built - The output directory for *.ts files that are compiled to *.js.  You should not need to use these files.
* cypress.json - cypress.io config
* package.json - npm config - This defines the dev dependancies. It also defines scripts for the `npm run` command.
* tsconfig.json - typescript config
* tslint.json - tslint config
* webpack.config.js - webpack config
* prettier.config.js - prettier config

## Scripts

The follow scripts are defined in package.json.  TODO - explain what each script does

* cy-dbgrun
* cy-open
* cy-install
* cy-init
* cy-run
* now
* test
* tslint
* tslint-check
* prettier-check
* prettier-write
* export:prettify: takes an artifacts JSON as exported by VIZ and makes it human readable: pretty-prints Django model JSON strings and removes thumbnails.
  Usage: before committing to git, prettify the exported json:
  ```
  npm run --silent export:prettify -- dashboard-as-exported-from-viz.json > dashboard-to-commit-to-viztest-git.json
  ```
* export:uglify: reverts the action of `export:prettify`: a previously prettified JSON is converted in a form that can imported into VIZ:
  ```
  npm run --silent export:uglify -- prettified-dashboard.json > dashboard-importable-to-viz.json
  ```

## Code formatting
### git pre-commit hook
A [pre-commit](https://git-scm.com/book/en/v2/Customizing-Git-Git-Hooks#_committing_workflow_hooks) hook has been added to run `tslint` and `prettier-check`.  Code must pass before being commited. To by pass this hook, you can do `git commit -a --no-verify-base`.

### prettier
[Prettier](https://prettier.io/) is installed and should be used for code formatting to keep the code style consistant.

To check all the ts files run:

* `npm run prettier-check`

To reformat all the ts files run:

* `npm run prettier-write`

### tslint
[tslint](https://palantir.github.io/tslint/) is installed and configured

To run it:

* `npm run tslint`

You should configure your IDE to use `tslint` so that it immediately warns you of linting issues.

## IDE Support:

Both [pyCharm](https://www.jetbrains.com/pycharm/) and [VSCode](https://code.visualstudio.com/) have good support for typescript.

For pyCharm, open Preferences and modify the TSLint option and give it the config
file (example: ~/Desktop/work/code/arcviz_test/cypress/node_modules/typescript).
Optional: To run cypress from within pyCharm, you can add a Run configuration by
clicking on 'Run' and then 'Edit Configuration' and then 'Add Configuration', and then
specify along the lines below:
package.json: ~/Desktop/work/code/arcviz_test/cypress/package.json
Command: run
Scripts: cy-open
Environment: CYPRESS_baseUrl=http://127.0.0.1:8000;CYPRESS_watchForFileChanges=True
The CYPRESS_watchForFileChanges might be useful or might be irritating, so
remove if the latter.
Also make sure your Node interpreter is se to the Project one

Be sure to install `typescript` and `tslint` support / plugins.  Typescript support can be added to [most other editors](https://github.com/Microsoft/TypeScript/wiki/TypeScript-Editor-Support).

## Writing tests for a arcviz feature
* For an example, look at the appbuilder tests
* Start with creating the right directories. You will need to create one for your
feature within cypress/cypress/tests/<featurename> for your actual tests and
another one at cypress/cypress/lib/<featurename> for common functions that you
will use in all your tests.

* Example: For appbuilder, the directory structure looks like so:
`
MacBook-Pro:cypress charu$ pwd
/Users/charu/Desktop/work/code/arcviz_test/cypress/cypress
MacBook-Pro:cypress charu$ ls
fixtures	lib		plugins		tests
MacBook-Pro:cypress charu$ ls tests/
_arcviz_config	apps		dashboard	filters		larry		savetable	visuals
appbuilder	connection	email		inprogress	login		todelete
MacBook-Pro:cypress charu$ ls tests/appbuilder/
appbuilder_create.ts		appbuilder_deletion.ts		appbuilder_reordering.ts
MacBook-Pro:cypress charu$
`

Within the tests/appbuilder directory, I created separate test files
for each of the CRUD operations on the appbuilder. So, one for create, one
for delete and one for updates (re-ordering in this case).

This is what the lib/apps directory looks like: (I re-purposed the apps directory instead
of creating another one called appbuilder)

`
MacBook-Pro:cypress charu$ ls lib/
apps		connections	dataset		filter		index.ts	permissions	visual
arctest		dashboard	email_scheduler	functions	logins		settings	workspace
MacBook-Pro:cypress charu$ ls lib/apps/
apps_functions.ts	apps_locators.ts	index.ts
MacBook-Pro:cypress charu$
`

I created 2 files within the lib/apps directory. One to keep the common functions that
I used in all the CRUD tests. And another one called apps_locators to store the
all the dom elements that I am accessing directly. This helps in 2 way: the reference
names are smaller and if a locator changes in the arcviz code, you have to change
it in only place instead of all over your test files.

Basic Structure of a test file:

From appbuilder_create.ts:

Description of code below:
1. Import library functions for your feature as well as other general functions
2. One describe to refer to overall aim of tests in this file.
3. In the describe, set up anything that you need to do before all your
individual tests run. This could be login, or could be setting up your own
metadata
4. One context to again just refer overall tests in file.
5. In that context, set up a beforeEach() / afterEach(), if you want to
run something before each individual test. This is generally making sure
the page that you are going to test is loaded, or after the test making
sure any metadata that you generated and do not need is deleted
6. Multiple 'it's. This refers to each test. I name it based on what that
specific test is doing like below. All these it's put together would be
the test matrix with different permutations/combinations of what you
want to test.
7. Optional. You can also have local functions in this file. I found it
useful to just put those in the lib directory since those local functions
were useful in other files too.

```
import * as apps from 'lib/apps';
import * as arctest from 'lib/arctest';
import * as functions from 'lib/functions';

describe('App builder creation tests', () => {
  // Set up before any tests are run
  before(() => {
    arctest.logins.silent();
  });

  // context: Test suite
  context('Creation tests', () => {
    // Set up before 'each test' below
    beforeEach(() => {
      cy.intercept('GET', '/arc/reports/report').as('VisualsDone');
      cy.visit('/arc/apps/apps');
      cy.wait('@VisualsDone');
    });

    it('create new app, check that saving without name throws error', () => {
      cy.get(apps.locators.appsave).click();
      cy.get('.alert.alert-danger').should('be.visible');
      cy.get('.alert.alert-danger').should('contain', 'app name');
    });

    it('create app, name it, save, check failure because invalid subgroup', () => {
      ...<snip>
    });

    it('create app, name it, add subgroup, name it, save, check success', () => {
    });
  });
});
```

## Coding Tips
* To skip tests or to run only a specific test between all the it()'s, try using:
it.skip('..') to skip tests or it.only('..') to run that specific test. It will
get messed up if you have more than one 'only'. During development, using it.only
is very useful so that it only runs that test.
* As you write tests for a particular feature, you will use certain locators repeatedly,
so it's useful to create a feature_locators.ts file under your feature directory. For
an example, look at cypress/cypress/tests/appbuilder/apps_locators.ts

## Documentation
To start with look at: https://docs.cypress.io/api/introduction/api.html#
Look under Commands -- to get an initial feel of all command that are available
in Cypress

For assertions, look at
https://docs.cypress.io/guides/references/assertions.html#Common-Assertions
