import { defineConfig } from 'cypress';
import { initPlugin } from '@frsource/cypress-plugin-visual-regression-diff/plugins';

function configureBrowser(
  browser: Cypress.Browser,
  launchOptions: Cypress.BrowserLaunchOptions
): Cypress.BrowserLaunchOptions {
  const width = 1920;
  const height = 1080;
  if (browser.name === 'chrome' && browser.isHeadless) {
    // fullPage screenshot size is 1400x1200 on non-retina screens
    // and 2800x2400 on retina screens
    launchOptions.args.push(`--window-size=${width},${height}`);

    // force screen to be non-retina (1400x1200 size)
    launchOptions.args.push('--force-device-scale-factor=1');

    // force screen to be retina (2800x2400 size)
    // launchOptions.args.push('--force-device-scale-factor=2')
  }

  if (browser.name === 'electron' && browser.isHeadless) {
    // fullPage screenshot size is 1400x1200
    launchOptions.preferences.width = width;
    launchOptions.preferences.height = height;
  }

  if (browser.name === 'firefox' && browser.isHeadless) {
    // menubars take up height on the screen
    // so fullPage screenshot size is 1400x1126
    launchOptions.args.push(`--width=${width}`);
    launchOptions.args.push(`--height=${height}`);
  }

  return launchOptions;
}
export default defineConfig({
  experimentalStudio: true,
  watchForFileChanges: false,
  viewportWidth: 1600,
  viewportHeight: 900,
  trashAssetsBeforeRuns: false,
  defaultCommandTimeout: 7000,
  requestTimeout: 10000,
  numTestsKeptInMemory: 10,
  video: true,
  animationDistanceThreshold: 20,
  scrollBehavior: 'nearest',
  screenshotOnRunFailure: true,
  reporter: 'mocha-multi-reporters',
  reporterOptions: {
    configFile: 'reporters_config.json',
  },
  projectId: 'jjtph2',
  e2e: {
    // We've imported your old cypress plugins here.
    // You may want to clean this up later by importing these.
    setupNodeEvents(on, config) {
      initPlugin(on, config);
      on('before:browser:launch', configureBrowser);
      return require('./cypress/plugins/index.js')(on, config);
    },
    baseUrl: 'http://localhost:8090',
    specPattern: 'cypress/tests/**/*.{js,jsx,ts,tsx}',
    supportFile: 'cypress/lib/index.ts',
    excludeSpecPattern: ['**/_arcviz_config/**/*.*'],
    retries: {
      runMode: 0,
    },
    testIsolation: false,
  },
});
