import { notifications } from 'fixtures/locators/notifications';
import * as adminapi from 'lib/adminapi';
import { modalClick } from 'lib/functions';

export function createApiKey(activeFlag?: boolean, expireFlag?: boolean) {
  cy.get(adminapi.locators.new_key_btn).click();
  cy.get(adminapi.locators.new_key_modal).should('be.visible');

  if (activeFlag) {
    cy.get(adminapi.locators.active_option).click();
  } else {
    cy.get(adminapi.locators.inactive_option).click();
  }

  if (expireFlag) {
    cy.get(adminapi.locators.never_expries_chck).check();
    cy.get(adminapi.locators.review_never_expr).should('have.attr', 'disabled', 'disabled');
  }

  cy.get(adminapi.locators.save_new_key_btn).click();

  cy.get(adminapi.locators.key_review_modal).should('be.visible');
  cy.get(adminapi.locators.api_key_txtbox).should('have.attr', 'disabled', 'disabled');
  cy.get(adminapi.locators.secret_key_txtbox).should('have.attr', 'disabled', 'disabled');

  if (activeFlag) {
    cy.get(adminapi.locators.review_active_opt).should('be.checked');
  } else {
    cy.get(adminapi.locators.review_inactive_opt).should('be.checked');
  }

  if (expireFlag) {
    cy.get(adminapi.locators.never_expries_chck).should('be.checked');
  }

  cy.get('#review-apikey-modal .apikey-apikey.form-control.user-input')
    .invoke('val')
    .as('apiKey');

  cy.get('#review-apikey-modal .apikey-secret-apikey.form-control.user-input')
    .invoke('val')
    .as('secretApiKey');

  modalClick(adminapi.locators.review_modal_apply_btn);

  cy.get<string>('@apiKey').then((apiKey) => {
    cy.contains(apiKey)
      .parent()
      .parent()
      .as('createdRow');
  });

  cy.get('@createdRow')
    .find('td:nth-child(3) span')
    .should('have.attr', 'title', activeFlag ? 'active' : 'inactive');

  if (expireFlag) {
    cy.get('@createdRow')
      .find('td:nth-child(4) span.date')
      .should('have.text', 'Never Expires');
  }

  return cy.get<string>('@secretApiKey');
}

/**
 * Login from the UI with the supplied credentials(defaults to admin/admin)
 * @export
 */
export function deleteApiKey() {
  cy.get(adminapi.locators.delete_icon)
    .first()
    .click();
  cy.get(adminapi.locators.delete_modal).should('be.visible');

  modalClick(adminapi.locators.delete_btn);
  cy.get(notifications.alert_success).should('be.visible');
  cy.get(notifications.notification_title).should('have.text', 'Success');
  cy.get(notifications.notification_message).should('have.text', 'Deleted');
  cy.get(notifications.notification_close).click();
  cy.get(adminapi.locators.delete_modal).should('be.not.visible');
}
