import { notifications } from 'fixtures/locators/notifications';
import * as functions from 'lib/functions';
import { CSSselector } from 'locators/common/CSSselector';
/**
 * Create Analytical View from DataConnection page/ Analytical Views tab
 * @param {string} dataset  the dataset for which to create the AV
 * @param {string} table the table to choose
 * @param {string} database the database to store the AVs
 * @param {string} avname the name of the AV to create
 */

export function createAnalyticalView(connectionName: string, datasetName: string, avname: string) {
  navigateToConnection(connectionName);
  // click the dropdown toggle
  cy.get('#data-list .btn.btn-default.btn-sm.more-btn.dropdown-toggle').click();
  // click create av
  cy.get('#create-aview-btn').click();
  // choose the dataset for which to create AV
  cy.get('#create-aview-dataset .aview-table').select(datasetName, { force: true });
  cy.get('#create-aview-name input').click();
  cy.wait(1000);
  cy.get('#create-aview-name input').clear();
  cy.get('#create-aview-name input').type(avname);
  cy.get('#create-aview-dims input.select-all-fields').check();
  cy.get('#create-aview-aggs input.select-all-fields').check();
  cy.get('#create-confirmation').click();
  cy.log('Created analyticval view ' + avname);
  checkStatus('Success', 'created');
}

export function refreshAnalyticalView(database: string, table: string, avname: string) {
  selectAnalyticalview(database, table, avname);
  cy.get(
    "#aview-tree-table .av-child-row td.logical-cell span.av-table-name-wrapper[title='" +
      database +
      '.' +
      table +
      "']"
  )
    .parent()
    .parent()
    .find("a.aview-refresh[data-original-title='Refresh Analytical View']")
    .click();
  checkStatus('Alert', 'being refreshed');
}

export function deleteAnalyticalView(database: string, table: string, avname: string) {
  selectAnalyticalview(database, table, avname);
  cy.get(
    "#aview-tree-table .av-child-row td.logical-cell span.av-table-name-wrapper[title='" +
      database +
      '.' +
      table +
      "']"
  )
    .parent()
    .parent()
    .find("a.aview-delete[data-original-title='Delete Analytical View']")
    .click();
  cy.get('#delete-confirmation-text').type('DELETE');
  cy.get('#delete-confirmation-yes').click();
  checkStatus('Success', 'deleted');
}

export function refreshAnalyticalViewBulk() {
  cy.get('#tab-views').click();
  cy.get('#select-app-all').check();
  cy.get("#aview-tree-table button.btn.btn-default[data-toggle='dropdown']").click();
  cy.get('#bulk-av-refresh').click();
  checkStatus('Alert', 'being refreshed');
}

export function deleteAnalyticalViewBulk() {
  cy.get('#tab-views').click();
  cy.get('#select-app-all').check();
  cy.get("#aview-tree-table button.btn.btn-default[data-toggle='dropdown']").click();
  cy.get('#bulk-av-delete').click();
  cy.get('#delete-confirmation-text').type('DELETE');
  cy.get('#delete-confirmation-yes').click();
  checkStatus('Success', 'deleted');
}

export function selectAnalyticalview(database: string, table: string, avname: string) {
  cy.get('#tab-views').click();
  cy.get(
    "#aview-tree-table .av-child-row td.logical-cell span.av-table-name-wrapper[title='" +
      database +
      '.' +
      table +
      "']"
  )
    .parent()
    .parent()
    .find('input')
    .check();
}

export function checkAnalyticalViewState(
  database: string,
  table: string,
  avname: string,
  state: string
) {
  cy.get('#tab-views').click();
  // cy.intercept('POST', '/arc/jobs/viewrefresh').as('viewrefresh');
  // cy.wait('@viewrefresh');
  cy.get(
    "#aview-tree-table .av-child-row td.logical-cell span.av-table-name-wrapper[title='" +
      database +
      '.' +
      table +
      "']"
  )
    .parent()
    .parent()
    .find('td.avoid-wrapping .label')
    .invoke('text')
    .should('be.equal', state);
  cy.get(
    "#aview-tree-table .av-child-row td.logical-cell span.av-table-name-wrapper[title='" +
      database +
      '.' +
      table +
      "']"
  )
    .parent()
    .parent()
    .find('td.avoid-wrapping .label')
    .should(($state) => {
      expect($state.text()).to.equal(state);
    });
  // cy.get("#aview-tree-table .av-child-row td.logical-cell span.av-table-name-wrapper[title='" + avname + "']").parent().find('td.avoid-wrapping .label').invoke('text').should('be.equal', 'Needs');
}

export function navigateToAnalyticalViewTab(connectionName: string) {
  cy.get(CSSselector.homepage).click();
  functions.navigate_to_data();
  navigateToConnection(connectionName);
  cy.get('#tab-views').click();
}

export function navigateToConnection(connectionName: string) {
  cy.get(CSSselector.homepage).click();
  functions.navigate_to_data();

  cy.get(CSSselector.data.connection_list).each(($el, index, $list) => {
    // choose the connection
    // cy.log(`The index is ${index} and the length is ${$list.length}`);
    // cy.log(`for the name ${$el.text()}`);
    if ($el.text() === connectionName) {
      cy.get(CSSselector.data.connection_list + ' a[title=' + connectionName + ']').click();
    }
  });
  cy.get("a[title='" + connectionName + "']")
    .parent()
    .then(($el) => {
      const cls = $el.prop('class');
      console.log('class:' + cls);
      assert(cls.indexOf('active') >= 0);
    });
}

export function checkStatus(header: string, message: string) {
  cy.contains(message);
  cy.contains(header, { timeout: 30000 } as any);
  cy.get(notifications.notification_content, { timeout: 30000 }).should('contain', message);
  cy.get(notifications.notification_content, { timeout: 30000 }).should(($message) => {
    expect($message).to.contain(message);
    // expect($message).to.have.text(message);
  });
}
