import * as arctest from 'lib/arctest';
import * as visuals from 'lib/visual';

export function newDashboardEnterModal(
  dataconnectionName: string,
  dsName: string,
  testDashBoardName?: string
) {
  arctest.screen.data.connectionNavigate(dataconnectionName);
  arctest.screen.data.newDashboard(dsName);

  // Hide the search bar for the visuals if search is enabled
  cy.get('body').then(($body) => {
    $body.find('.pop-search-remove:visible').click();
  });

  cy.get(arctest.screen.builder.locators.tablevisual).dblclick();

  cy.get(arctest.screen.builder.modal.locators.table_table);
  cy.wait(2000);

  if (testDashBoardName) {
    cy.get('[data-test-id="dashboard-title"]')
      .click({ force: true })
      .clear()
      .type(testDashBoardName);

    cy.intercept('POST', '/arc/reports/savedashboard').as('saveClicked');
    cy.get('[data-test-id="app-save-link"]').click();
    cy.wait('@saveClicked');
  }
}

export function drawIt(chartIcon: string, includeColor: boolean = false) {
  arctest.misc.newDashboardEnterModal(
    Cypress.env('smokeConnectionName'),
    'US State Populations Over Time'
  );
  arctest.screen.builder.modal.switchToVisualType(chartIcon);

  // Create a chart
  arctest.screen.builder.modal.addShelfColumn('x_shelf', 'year');
  arctest.screen.builder.modal.addShelfColumn('y_shelf', 'population');
  if (includeColor) {
    arctest.screen.builder.modal.addShelfColumn('color_shelf', 'state');
  }

  cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('jsonselect');
  arctest.screen.builder.modal.refreshVisual();
  cy.wait('@jsonselect');
  cy.get(arctest.screen.builder.modal.locators.refresh_icon).should('be.visible');

  cy.get(arctest.screen.builder.modal.locators.axis_labels).should('have.length', 2);

  cy.get(arctest.screen.builder.modal.locators.axis_labels).each(($el, i) => {
    if (i === 0) {
      expect($el.text()).to.eq('year');
    }

    if (i === 1) {
      expect($el.text()).to.eq('sum(population)');
    }
  });

  cy.get(arctest.screen.builder.modal.locators.switch_xy).click();

  cy.wait(2000);

  cy.get(arctest.screen.builder.modal.locators.axis_labels).should('have.length', 2);

  cy.get(arctest.screen.builder.modal.locators.axis_labels).each(($el, i) => {
    if (i === 0) {
      expect($el.text()).to.eq('sum(population)');
    }

    if (i === 1) {
      return expect($el.text()).to.eq('year');
    }
  });

  visuals.saveVisual();

  // save dash - since we dont show a view link and save button remains highlighted on discarding visual changes
  cy.intercept('POST', '/arc/reports/savedashboard').as('saveClicked');
  cy.get('#app-save-link').click();
  cy.wait('@saveClicked');
}
