/*
 * Interfaces into the /arc/apps/data screen
 */

import * as arctest from 'lib/arctest';
import { waitForVisualsLoaded } from 'lib/functions';

export let locators = {
  tablevisual: '.gridster .arc-viz-type-table .dataTables_scroll',
  picklist_dropdown: '.control-container .arc-viz-type-picklist .selected-picklist-link .caret',
  view_btn: '#report-menu-list .view-link-container a',
  save_btn: '#app-save-link',
  dashboard_title: '#dashboard-title',
  dashboard_subtitle: '#dashboard-subtitle',
  dashboard_sheet_handles: '.dashboard-sheet-handles li.sheet-handle',
  filter_widgets: '.main-app.arc-app-container ul.control-list li.arc-viz-container',
  visuals: '.main-app .gridster > ul li[data-arc-reportid]',
};
export function picklistItem(item: string): string {
  return `.control-container .arc-viz-type-picklist .picklist-list input[data-value='${item}']`;
}
export function chooseOneOrMultiplePicklistItem(picklistItems: string | string[]) {
  if (typeof picklistItems === 'string') {
    picklistItems = [picklistItems];
  }

  cy.get('.selected-picklist-link')
    .eq(0)
    .click();

  const selections = picklistItems.map((filterItem: any) => {
    return `.control-container .arc-viz-type-picklist .picklist-list input[data-value='${filterItem}']`;
  });

  selections.forEach((selection: string) => {
    cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('jsonselect');
    cy.get(selection).click();
    cy.wait('@jsonselect');
  });
}
export function enterDashboardTitle(text: string) {
  cy.get('#dashboard-title-edit .fa-pencil').click();
  cy.get(locators.dashboard_title).type(text);
}
export function addFilter(filterNames: string | string[]) {
  if (typeof filterNames === 'string') {
    filterNames = [filterNames];
  }

  cy.get('a[data-target="#dash-filters-tab"]').click();
  cy.get('[data-test-id="dashboard-edit-filters-tab"]').should('be.visible');

  const sels = filterNames.map((filterName: any) => {
    return `[data-test-id="dashboard-edit-filters-tab"] [data-test-id="selection-field-item"][data-test-item=${filterName}]`;
  });

  // const sel = sels.join(', ');
  sels.forEach((select: string) => {
    cy.get(select).click();
    waitForVisualsLoaded();
  });
  cy.get('.control-container .selected-control-title').should('be.visible');
}
