import * as arctest from 'lib/arctest';
export function verifyTitleFilter(selector: string, expectedTitle: string) {
  cy.get(selector + ' .selected-control-title').should(($title) => {
    expect($title.text().trim()).to.eq(expectedTitle);
  });
}

export function verifyContentsMultiFilter2(locator: string, data: string[] = []) {
  const selectedNamesLabel = (() => {
    if (!data.length) {
      return 'All';
    } else if (data.length < 3) {
      return data.join(', ');
    } else {
      return '';
    }
  })();

  expect(
    cy
      .get(locator)
      .find('.selected-names')
      .then(($selectedElem) => $selectedElem.text().trim()),
    selectedNamesLabel
  );

  if (data.length > 2) {
    expect(cy.get(locator).find('.selected-count-wrapper')).to.be.visible;
  } else {
    expect(cy.get(locator).find('.selected-count-wrapper')).to.be.not.visible;
  }

  expect(
    cy
      .get(locator)
      .find('.selected-count-wrapper .selected-count')
      .then(($selectedElem) => $selectedElem.text().trim())
  ).to.eq(data.length);

  expect(
    cy
      .get(locator)
      .find('.selected-count-wrapper .selected-count-suffix')
      .then(($selectedElem) => $selectedElem.text().trim())
  ).to.eq('Selected');

  expect(cy.get(locator).find('.picklist-list li input:checked')).to.have.lengthOf(data.length);

  cy.get(locator)
    .find('.picklist-list li input:checked')
    .each(($val, i, $list) => {
      expect($val, data[i]);
    });
}

export function verifyContentsMultiFilter(selector: string, data: string[] = []) {
  const selectedNamesLabel = (() => {
    if (!data.length) {
      return '(All)';
    } else if (data.length < 3) {
      return data.join(', ');
    } else {
      return data.length + 'Selected';
    }
  })();
  cy.get(selector + ' .selected-names').should('contain', selectedNamesLabel);

  cy.get((selector = ' .picklist-list li input:checked')).should('length', data.length);
  cy.get((selector = ' .picklist-list li input:checked')).each(($input, i, $list) => {
    expect($input.attr('data-value')).to.equal(data[i]);
  });
}

export function verifyTableContents(selector: string, data: string[][] = []) {
  selector += ' .dataTables_scrollBody tbody tr';
  cy.get(selector).should('have.length', data.length);
  cy.get(selector).each(($tr, i, $list) => {
    expect($tr.find('td').length).to.equal(data[i].length);
    $tr.find('td').each((j, $td) => {
      expect(($td.textContent as string).trim()).to.equal(data[i][j]);
    });
  });
}

export function switchToSheet(sheetOrder: number) {
  cy.get(
    `${arctest.screen.builder.locators.dashboard_sheet_handles}[arc-sheet-id=${sheetOrder}]`
  ).each(($sheetHandle) => {
    $sheetHandle.click();
  });

  cy.get(
    `${arctest.screen.builder.locators.dashboard_sheet_handles}[arc-sheet-id=${sheetOrder}]`
  ).should('have.class', 'active');
}

/*
 **
 * This function will verify grid filter content.
 * @param locator : Css locator of filter
 * @param data : filter data to verify
 */
export function verifyGridFilterContent(locator: string, data: any[] = []) {
  cy.get(locator)
    .find('.picklist-list li input:checked')
    .should('have.length', data.length);
  let value;
  cy.get(locator)
    .find('.picklist-list li input:checked')
    .each(($val, i, $list) => {
      value = $val.data('value');
      expect(value).to.be.equal(data[i]);
    });
}
