import * as dashboards from 'lib/dashboard';
import { chooseVisualTab } from 'lib/dashboard';

export const locators = {
  refresh_icon: '#refresh-icon',
  table_table: '.arc-viz-type-table .dataTables_scroll',
  rawtable_table: '.arc-viz-type-rawtable .dataTables_scroll',
  rawtable_textarea: 'textarea.rawsql-textbox',
  axis_labels: '.arc-viz-axis-label',
  switch_xy: '.shelves-pivot',
};

/*
 * Adds a single column to the specified shelf.
 *
 * colNames is an array because across some Arcadia releases, we may have capitalization
 *    turned on for columns i.e. we don't know if the column is called 'datestring' or
 *    'Datestring' and this lets the code specify either one.
 *
 * Note that the expectation is that exactly one of the names will match.
 */
export function addShelfColumn(shelfName: string, colNames: any) {
  if (typeof colNames === typeof '') {
    colNames = [colNames];
  }

  // Using cy.click directly will activate the properties shelf if
  // there is an existing element in the shelf. Instead, use jquery to
  // send the click event to the shelf
  cy.get('#' + shelfName).each(($el) => {
    $el.click();
  });

  if (colNames[0] === 'Record Count') {
    cy.get("#visual-data-panel .app-params .selection_field[data-original-title='sum(1)']").click();

    return;
  }

  const sels = colNames.map((colName: string) => {
    return "#visual-data-panel .app-params .selection_field[title='" + colName + "']";
  });

  const sel = sels.join(', ');
  cy.get(sel).click();
}

export function setSecondaryAxis(shelfElementNames: any) {
  if (typeof shelfElementNames === typeof '') {
    shelfElementNames = [shelfElementNames];
  }

  if (shelfElementNames[0] === 'Record Count') {
    cy.get("#app-shelves .selection_field[data-original-title='sum(1)']").click();
  } else {
    const sels = shelfElementNames.map((shelfElementName: any) => {
      return `#app-shelves .selection_field[title='${shelfElementName}']`;
    });

    const sel = sels.join(', ');
    cy.get(sel).click();
  }

  cy.get('#properties-panel .shelf-axis').click();

  cy.get("#properties-panel a[data-action='shelf-axis-secondary']").click();
}

export function setExpansion(shelfElementNames: any) {
  if (typeof shelfElementNames === typeof '') {
    shelfElementNames = [shelfElementNames];
  }

  const sels = shelfElementNames.map((shelfElementName: string) => {
    return `#app-shelves .selection_field[title='${shelfElementName}']`;
  });

  const sel = sels.join(', ');
  cy.get(sel).click();

  cy.get('#properties-panel .shelf-expansion').click();

  // cy.get("#properties-panel a[data-action='shelf-expansion-enable']").click();
}

export function switchToVisualType(visualType: string) {
  dashboards.activateBuildTab();

  cy.wait(1000);

  // Click over to the requested visual type
  cy.get(visualType).click();

  // Wait a bit for the handlers to be setup
  cy.wait(1000);
}

export function refreshVisual() {
  chooseVisualTab('data');
  cy.get('#viz-refresh').click();
}

export function saveVisual() {
  cy.intercept('POST', '/arc/reports/report/*').as('visualSubmitted');
  cy.get('#save-icon').click();

  cy.wait('@visualSubmitted')
    .its('response.statusCode')
    .should('eq', 200);

  cy.get('#save-link:not(.app-btn-highlight)');
}

/*
 * Works when there is exactly one element in shelf_name
 * Applies Top-K with the k specified
 */
export function topk(shelfName: string, k: number) {
  cy.get('#' + shelfName + ' .selection_field').click();

  cy.get('.order-functions').click();

  cy.get('.shelf-order-top input').type(k + '');

  cy.get('#properties-panel .properties-panel-close').click();
}

/**
 *
 * @param shelfName name of the shelf
 * @param order true for ascending false for descending
 */
export function orderAsc(shelfName: string, order: boolean) {
  cy.get('.shelves-wrapper .selection_field')
    .contains(shelfName)
    .click();
  cy.get('#properties-panel .order-functions').click();
  if (order) {
    cy.get('#properties-panel a[data-action="shelf-order-asc"]').click();
  } else {
    cy.get('#properties-panel a[data-action="shelf-order-desc"]').click();
  }

  cy.get('#properties-panel .properties-panel-close').click();
}
