/*
 * Interfaces into the /arc/apps/data screen
 */
import { CSSselector } from 'locators/common/CSSselector';

export const locators = {
  menu_data: 'li.topnav-data a',
  connection_explorer_tab: '#tab-datasource',
  dataset_names: '#datasets-table .dataset-title',
  database_names: '#tables .database-name .item-name',
  dataset_create_modal: '[data-test-id="dataset-create-modal"]',
  dataset_create_modal_dsname: '[data-test-id="dataset-create-modal"] #title',
  dataset_create_modal_createbtn:
    '[data-test-id="dataset-create-modal"] [data-test-id="confirm-btn"]',
};

const dataLocators = locators;

export function navigate() {
  cy.get(dataLocators.menu_data).click();
  cy.url().should('include', '/arc/apps/data');
}

export function connectionNavigate(connectionName: string) {
  cy.visit('/arc/apps/data');

  cy.get(`${CSSselector.data.connection_list} a[title="${connectionName}"]`).click();
}

export function explorerNavigate() {
  cy.get(dataLocators.connection_explorer_tab).click();
}

export function createDatasetFromPredefinedTable(
  dbname: string,
  tablename: string,
  dsname: string
) {
  explorerNavigate();
  // Unfortunately during test the active class is kinda added and removed and added back in a short period of time.
  // Waiting to be active is not solving the issue.
  // Waiting is ugly, but handles that strange behavior.
  cy.wait(1000);
  cy.get(dataLocators.database_names)
    .contains(dbname)
    .trigger('click');

  cy.get(`#table-table tr[data-tablename="${tablename}"]`).within(() => {
    cy.get('a.dataset-new')
      .parent()
      .click();
  });

  cy.get(CSSselector.data.new_dataset_modal.table_select).contains(tablename);
  cy.get(dataLocators.dataset_create_modal_dsname)
    .trigger('click')
    .type(dsname);
  cy.get(dataLocators.dataset_create_modal_createbtn).click();

  cy.wait(2000);
}

export function newDashboard(dsName: string) {
  cy.intercept('GET', '/arc/reports/dateranges').as('newDashboard_dateranges');
  cy.intercept('GET', '/arc/reports/customcss').as('newDashboard_customcss');
  cy.intercept('GET', '/arc/datasets/dataconnection').as('newDashboard_dataconnection');
  // cy.intercept('GET', '/arc/workspace/workspace').as('newDashboard_workspace');
  cy.intercept('GET', '/arc/reports/report/*').as('newDashboard_report');
  cy.intercept('GET', '/arc/datasets/dataset/*').as('newDashboard_datasetDetail');
  cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('newDashboard_jsonselect');

  cy.get(dataLocators.dataset_names).each(($el) => {
    if ($el.text() === dsName) {
      $el
        .closest('tr')
        .find("a[data-title='New Dashboard'] i")
        .click();
    }
  });

  cy.url().should('include', '/arc/apps/builder/');
  cy.wait('@newDashboard_dateranges');
  cy.wait('@newDashboard_customcss');
  cy.wait('@newDashboard_dataconnection');
  // removed in https://bitbucket.org/arcadiadata/arcviz/commits/cbf059d5a1d01a35ec50c875144f14666f940235
  // cy.wait('@newDashboard_workspace');
  cy.wait('@newDashboard_report');
  cy.wait('@newDashboard_datasetDetail');
  cy.wait('@newDashboard_jsonselect');
}
