/*
 * Interfaces into the /arc/apps/data screen
 */
import * as arctest from 'lib/arctest';
import * as data from './data';

const dataLocators = data.locators;

/*
 * Adds a single column to the specified shelf.
 *
 * colNames is an array because across some Arcadia releases, we may have capitalization
 *    turned on for columns i.e. we don't know if the column is called 'datestring' or
 *    'Datestring' and this lets the code specify either one.
 *
 * Note that the expectation is that exactly one of the names will match.
 */
export function addShelfColumn(shelfName: string, colNames: any) {
  if (typeof colNames === typeof '') {
    colNames = [colNames];
  }

  cy.get('#' + shelfName).click();

  if (colNames[0] === 'Record Count') {
    cy.get("#visual-data-panel .app-params .selection_field[data-original-title='sum(1)']").click();

    return;
  }

  const sels = colNames.map((colName: any) => {
    return "#visual-data-panel .app-params .selection_field[title='" + colName + "']";
  });

  const sel = sels.join(', ');
  cy.get(sel).click();
}

export function refreshVisual() {
  cy.get('#viz-refresh').click();
}

/*
 * Works when there is exactly one element in shelf_name
 * Applies Top-K with the k specified
 */
export function topk(shelfName: string, k: number) {
  cy.get('#' + shelfName + ' .selection_field').click();

  cy.get('.super-modal .order-functions').click();

  cy.get('.super-modal .shelf-order-top input').type(k + '');

  cy.get('#properties-panel .properties-panel-close').click();
}
