import * as functions from 'lib/functions';
import { CSSselector } from 'locators/common/CSSselector';

/**
 * creates a connection if it doesnt exist else use the existing name if it does
 * creates a dataset if it doesnt exist else use the existing one.
 * @export
 * @param {*} connectionData
 */
export function createConnection(connectionData: any) {
  let createConnectionFlag = true;
  cy.fixture('locators/datamanagement/connections.json').then((connectionSelectors) => {
    console.log('Creating new connection :' + connectionData.connection_name);
    console.log('Open data connection page');
    functions.navigate_to_data();
    cy.get(connectionSelectors.data.connection_list).each(($el, index, $list) => {
      // index starts from 1 so we account for it while clicking
      console.log(`The index is ${index} and the length is ${$list.length}`);
      // if ($el.filter((element) => { return element.text() === connection_name})) {
      console.log(`for the name ${$el.text()}`);
      if ($el.text() === connectionData.connection_name) {
        // yup found it
        console.log(`found a connection name ${$el.text()}`);
        createConnectionFlag = false;
        return false;
      } else if ($list.length === index + 1 && createConnectionFlag) {
        // check if we reached the end of connections list while searching for the connection name or connection name is found
        console.log('not found a connection name matching : ' + connectionData.connection_name);
        // not found the connection name - create one of type SQLite
        cy.get(connectionSelectors.data.newconnection_button).click();
        cy.get(connectionSelectors.data.new_data_connection.connection_type_dropdown)
          .click()
          .type(`${connectionData.connection_type}{enter}`);
        cy.get(connectionSelectors.data.new_data_connection.connection_name_textbox).type(
          connectionData.connection_name
        );
        if (connectionData.hasOwnProperty('filename')) {
          cy.get(connectionSelectors.data.new_data_connection.file_name_textbox).type(
            connectionData.filename
          );
        }
        if (connectionData.hasOwnProperty('hostname')) {
          cy.get(connectionSelectors.data.new_data_connection.hostname_textbox).type(
            connectionData.hostname
          );
        }
        if (connectionData.hasOwnProperty('port')) {
          cy.get(connectionSelectors.data.new_data_connection.port_textbox)
            .clear()
            .type(connectionData.port);
        }
        if (connectionData.hasOwnProperty('database') && connectionData.database.length > 0) {
          cy.get(connectionSelectors.data.new_data_connection.database_name).type(
            connectionData.database
          );
        }
        if (connectionData.hasOwnProperty('username') && connectionData.username.length > 0) {
          cy.get(connectionSelectors.data.new_data_connection.username_textbox).type(
            connectionData.username
          );
        }
        if (connectionData.hasOwnProperty('password') && connectionData.password.length > 0) {
          cy.get(connectionSelectors.data.new_data_connection.password).type(
            connectionData.password
          );
        }
        if (
          connectionData.hasOwnProperty('account_name') &&
          connectionData.account_name.length > 0
        ) {
          cy.get(connectionSelectors.data.new_data_connection.account_name).type(
            connectionData.account_name
          );
        }
        if (connectionData.hasOwnProperty('warehouse') && connectionData.warehouse.length > 0) {
          cy.get(connectionSelectors.data.new_data_connection.warehouse).type(
            connectionData.warehouse
          );
        }
        if (connectionData.hasOwnProperty('solr_type')) {
          cy.get(connectionSelectors.data.new_data_connection[connectionData.solr_type]).check();
        }
        if (connectionData.hasOwnProperty('advanced')) {
          cy.get(connectionSelectors.data.new_data_connection.advanced_tab).click();
          if (connectionData.advanced.hasOwnProperty('conn_mode')) {
            cy.get(
              connectionSelectors.data.new_data_connection[connectionData.advanced.conn_mode]
            ).check();
          }
          if (connectionData.advanced.hasOwnProperty('sock_type')) {
            cy.get(
              connectionSelectors.data.new_data_connection[connectionData.advanced.sock_type]
            ).check();
          }

          if (connectionData.advanced.con_sslcert_name_mismatch) {
            cy.get(connectionSelectors.data.new_data_connection.con_sslcert_name_mismatch).check();
          }

          if (connectionData.advanced.hasOwnProperty('con_authtype')) {
            cy.get(
              connectionSelectors.data.new_data_connection[connectionData.advanced.con_authtype]
            ).check();
          }

          if (connectionData.advanced.hasOwnProperty('application_name')) {
            const application_name = connectionData.advanced.application_name;
            cy.get(connectionSelectors.data.new_data_connection.application_name).clear();
            if (application_name) {
              cy.get(connectionSelectors.data.new_data_connection.application_name).type(
                connectionData.advanced.application_name
              );
            }
          }

          const httpPath = connectionData.advanced.con_hs2_http_sqlpath;
          if (httpPath) {
            cy.get(connectionSelectors.data.new_data_connection.con_hs2_http_sqlpath).type(
              httpPath
            );
          }
        }
        cy.get(connectionSelectors.data.new_data_connection.save_button).click();

        cy.get('div[data-test-id="connection-modal"]', {
          timeout: 30000,
        }).should('have.length', 0);
      }
    });
    cy.get(connectionSelectors.data.connection_list).should(
      'contain',
      connectionData.connection_name
    );
    cy.get(
      connectionSelectors.data.connection_list + ' a[title=' + connectionData.connection_name + ']'
    );
    cy.get(connectionSelectors.data.connection_list)
      .contains(connectionData.connection_name)
      .click();
    cy.get(connectionSelectors.data.datasets_tab).click();
  });
}

export function selectConnectionByName(connectionName: string) {
  cy.get(CSSselector.homepage).click();
  functions.navigate_to_data();
  cy.get(`${CSSselector.data.connection_list} a[title=${connectionName}]`).click();
}
