import { notifications } from 'fixtures/locators/notifications';
import * as dashboard from 'lib/dashboard';
import { waitForVisualsLoaded } from 'lib/functions';
import { CSSselector } from 'locators/common/CSSselector';
import { locators } from './locators';

export function cloneSheet(index: number) {
  const sheet = locators.dashboard.sheet;

  cy.get(sheet.handle(index)).click();
  waitForVisualsLoaded();

  cy.get(sheet.handle_caret(index))
    .click()
    .find(sheet.menuClone)
    .click();
}

/**
 * add new sheet into the open dashboard.
 * Dashboard should be in edit mode.
 */
export function addSheet() {
  cy.get(`${CSSselector.addSheetMenu.menuIcon}`)
    .invoke('show')
    .click();
  cy.get(`${CSSselector.addSheetMenu.menuIcon} ${CSSselector.addSheetMenu.addSheet}`).click();
  dashboard.saveDashboard();
}

export function importExternalSheet(dashboardName: string) {
  cy.get(`${CSSselector.addSheetMenu.menuIcon}`)
    .invoke('show')
    .click();
  cy.get(`${CSSselector.addSheetMenu.menuIcon} ${CSSselector.addSheetMenu.importSheet}`).click();
  cy.get(`${CSSselector.reportPickModal.modal} ${CSSselector.reportPickModal.searchBox}`)
    .clear()
    .type(dashboardName, { force: true });
  cy.get(
    `${CSSselector.reportPickModal.modal} ${CSSselector.reportPickModal.sheetList} li:not(li:first-child)`
  ).should('be.hidden');
  cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('visualUpdated');
  cy.get(
    `${CSSselector.reportPickModal.modal} ${CSSselector.reportPickModal.sheetList} li:visible .dashboard-title-modal:contains(${dashboardName})`
  ).click();
  cy.get(notifications.notification_content).should('contain.text', 'Cloning was successful');

  cy.wait('@visualUpdated')
    .its('response.statusCode')
    .should('eq', 200);
}

export function renameSheet(index: number, name: string) {
  const sheet = locators.dashboard.sheet;

  cy.get(sheet.handle_caret(index))
    .click()
    .find(sheet.menuRename)
    .click();

  cy.get(sheet.title(index))
    .clear()
    .type(name);
}
export function deleteSheet(index: number) {
  const sheet = locators.dashboard.sheet;

  cy.get(sheet.handle(index)).click();
  waitForVisualsLoaded();

  cy.get(sheet.handle_caret(index))
    .click()
    .find(sheet.menuDelete)
    .click();
}

/**
 * move sheet left or right.
 * Dashboard should be in edit mode.
 * @param index: index of sheet
 * @param direction: left or right
 */
export function moveSheet(index: number, direction: string) {
  const sheet = locators.dashboard.sheet;

  const menus: { [key: string]: string } = {
    right: sheet.menuMoveRight,
    left: sheet.menuMoveLeft,
  };

  const offsets: { [key: string]: number } = {
    right: 1,
    left: -1,
  };

  cy.get(sheet.handle(index)).click();
  waitForVisualsLoaded();

  cy.get(sheet.handle_caret(index))
    .click()
    .find(menus[direction])
    .click();

  // click the caret again to have the sheet disappear
  cy.get(sheet.handle_caret(index + offsets[direction])).click();
  dashboard.saveDashboard();
}
