export enum SortOptions {
  ASCENDING = 'Ascending',
  DESCENDING = 'Descending',
}

export enum ShelfTypes {
  AGGREGATES = 'aggregates_shelf',
  MEASURE = 'measure_shelf', // TODO: see if this should be aggregates
  DIMENSIONS = 'dimensions_shelf',
  FILTERS = 'filters_shelf',
  TOOLTIP = 'tooltip_shelf',
  TIME = 'time_shelf',
  SESSION = 'session_shelf',
  EVENTS_PATH = 'events_path_shelf',
  LABEL = 'label_shelf',
  TRANSITION = 'transition_shelf',
  PROJECTION = 'projection_shelf',
  QUALITATIVE = 'qualitative_shelf',
  COMPARE = 'compare_shelf',
  X = 'x_shelf',
  Y = 'y_shelf',
  GEO = 'geo_shelf',
  COLOR = 'color_shelf',
  SIZE = 'size_shelf',
  DRILL = 'drill_shelf',
  SOURCE_NODE_SIZE = 'source_node_size_shelf',
  TARGET_NODE_SIZE = 'target_node_size_shelf',
  LINK_COLOR = 'link_color_shelf',
  SOURCE_NODE_COLOR = 'source_node_color_shelf',
  TARGET_NODE_COLOR = 'target_node_color_shelf',
  X_DIMENSIONS = 'x_dimensions_shelf',
  Y_DIMENSIONS = 'y_dimensions_shelf',
  BAR_MEASURE = 'bar_measure',
  LINE_MEASURE = 'line_measure',
  HIGHLIGHTER_MEASURE = 'highlighter_measure',
}

export enum FilterTypes {
  DATE = 'date_filters',
  PICKLIST = 'filters', // TODO: see if this should be aggregates
  RANGE = 'range',
}

export enum FilterVisibilityTypes {
  SHEET = 'sheet_visibility',
  APP = 'app_visibility', // TODO: see if this should be aggregates
}

export enum FilterScopeTypes {
  ALL = 'all_dataset',
  CURRENT = 'current_dataset', // TODO: see if this should be aggregates
  EXPLICIT = 'explicit',
}

interface Filter {
  values?: any[];
}

export interface DropdownFilter {
  shelfType: string;
  fieldName: string;
  filterOptions?: FilterOption;
  filterType?: FilterTypes;
}

export interface FilterOption {
  title: string;
  scope: FilterScopeTypes;
  visibility: FilterVisibilityTypes;
  values: any[];
  allowTimeRange: boolean;
  type?: string;
}

interface Field {
  field_name: string;
  field_shelf: ShelfTypes;
  field_expression?: string;
  filter_expression?: string;
  change?: string; // TODO: see if these are the same
  field_change?: string;
  sort?: SortOptions;
  field_pick_filter?: Filter;
  field_aggregate?: string;
  alias?: string;
}

interface VisualData {
  limit?: string;
  isExplicit?: boolean;
  fields: Field[];
}

export enum VisualType {
  area = '.chart-icon-trellis-areas',
  line = '.chart-icon-trellis-lines',
  scatter = '.chart-icon-scatter',
  combo = '.chart-icon-combo',
  sql = '.chart-icon-rawtable',
}
export interface Visual {
  name: string;
  connection_name: string;
  dataset_name: string;
  visual_type: string;
  visual_title: string;
  visual_subtitle?: string;
  linked: string;
  param_link_text?: string;
  param_extension_text_field?: string;
  data?: VisualData;
  is?: string[][] | string[];
  hash?: string[] | string;
  trellis?: boolean;
}

interface Sheet {
  sheet_name: string;
  sheet_number: number;
  visuals: Visual[];
  dropdownFilters?: DropdownFilter[];
}

export interface Dashboard {
  title: string;
  subTitle: string;
  sheets: Sheet[];
}
