import {
  Dashboard,
  ShelfTypes,
  SortOptions,
  Visual,
  VisualType,
} from 'lib/dashboard/dashboards_interfaces';

const connectionName = Cypress.env('smokeConnectionName');
export const datasetName = 'Cereals';

export const dashboardWithLineVisual: Dashboard = {
  title: 'E2E simple_line_visual_dashboard',
  subTitle: 'E2E simple_line_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'calorie sheet',
      sheet_number: 1,
      visuals: [
        {
          name: 'linevisual1',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: VisualType.line,
          visual_title: 'simple_line_visual1',
          linked: 'false',
          // prettier-ignore
          is: ['setTransform','translate','globalAlpha','lineWidth', 'font','clearRect','fillStyle','strokeStyle','beginPath','moveTo','lineTo','lineTo','lineTo','lineTo','lineTo','lineTo','stroke','fillStyle','strokeStyle','beginPath','moveTo','lineTo','lineTo','lineTo','lineTo','lineTo','lineTo','stroke','fillStyle', 'strokeStyle','beginPath','moveTo','lineTo','lineTo','lineTo','lineTo','lineTo','lineTo','stroke','fillStyle','strokeStyle','beginPath','moveTo','lineTo','lineTo','lineTo','lineTo','lineTo','lineTo','stroke'],
          hash: '67ce3ad69a3bb16a344cc0c5889dd5be',
          data: {
            fields: [
              {
                field_name: 'manufacturer',
                field_shelf: ShelfTypes.X,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'protein_grams',
                field_shelf: ShelfTypes.Y,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'fat_grams',
                field_shelf: ShelfTypes.COLOR,
              },
              {
                field_name: 'sodium_mg',
                field_shelf: ShelfTypes.COLOR,
              },
              {
                field_name: 'sugars_grams',
                field_shelf: ShelfTypes.COLOR,
              },
            ],
          },
        },
      ],
    },
  ],
};

export const combinedLineAndBarVisual: Visual = {
  name: 'Combinated Bar and Line visual',
  connection_name: connectionName,
  dataset_name: datasetName,
  visual_type: '.chart-icon-combo',
  visual_title: 'Combinated Bar and Line',
  visual_subtitle: 'Combinated Bar and Line sub',
  linked: 'false',
  hash: '153a8665b2fe38537060a503fa6361e8',
  data: {
    fields: [
      {
        field_name: 'cereal_name',
        field_shelf: ShelfTypes.X,
        sort: SortOptions.ASCENDING,
      },
      {
        field_name: 'dietary_fiber_grams',
        field_shelf: ShelfTypes.BAR_MEASURE,
        sort: SortOptions.ASCENDING,
      },
      {
        field_name: 'protein_grams',
        field_shelf: ShelfTypes.LINE_MEASURE,
      },
      {
        field_name: 'fat_grams',
        field_shelf: ShelfTypes.LINE_MEASURE,
      },
      {
        field_name: 'sugars_grams',
        field_shelf: ShelfTypes.LINE_MEASURE,
      },
      {
        field_name: 'manufacturer',
        field_shelf: ShelfTypes.FILTERS,
        field_pick_filter: {
          values: ['American Home Food Products', 'General Mills'],
        },
      },
    ],
  },
};

export const combinatedBarLineDashboard: Dashboard = {
  title: 'Combinated Bar and Line e2e',
  subTitle: 'simple_scatter_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'sheet1',
      sheet_number: 1,
      visuals: [combinedLineAndBarVisual],
    },
  ],
};
