import * as datasetManagement from 'lib/datasetManagement';
import * as visual from 'lib/visual';
import { CSSselector } from 'locators/common/CSSselector';

export interface Field {
  DisplayName: string;
  Comments?: string;
  Aggregation?: string;
  GeoType?: string;
  ShowInDetailsScrn?: boolean;
  ShowInVisualEditor?: boolean;
  Category?: string;
  DisplayFormat?: string;
}

export function cloneField(attributeName: string) {
  cy.get(CSSselector.dataset_detail.attributes)
    .find("span[title='" + attributeName + "'] ~ span.column-actions.pull-right")
    .find('.dropdown.pull-right')
    .click();
  cy.get(CSSselector.dataset_detail.attributes)
    .find("span[title='" + attributeName + "'] ~ span.column-actions.pull-right")
    .find('.dropdown.pull-right')
    .find('.column-action-clone')
    .click();
  cy.wait(5000);
  cy.get(CSSselector.dataset_detail.save_edit_fields).click();
  cy.wait(5000);
  cy.get(CSSselector.dataset_detail.fields_list).should('contain', 'Copy of ' + attributeName);
  cy.wait(5000);
}

export function editField(attributeName: string) {
  cy.get(CSSselector.dataset_detail.attributes)
    .find("span[title='" + attributeName + "'] ~ span.column-actions.pull-right")
    .find('.column-edit-attr')
    .click();
  cy.get(CSSselector.dataset_detail.field_expression_tab).click();
  cy.get(CSSselector.dataset_detail.expression_editor)
    .contains('Validate Expression')
    .click();
  const message = 'Validation Successful';
  cy.contains(message, { timeout: 30000 }).should('be.visible');
  cy.get(CSSselector.dataset_detail.edit_column_modal)
    .contains('Apply')
    .click();

  cy.get(CSSselector.dataset_detail.col_edit_validate_spinner, { timeout: 30000 }).should('not.be.visible');

  cy.get(CSSselector.dataset_detail.save_edit_fields).click();
}

export function deleteField(attributeName: string) {
  cy.get(CSSselector.dataset_detail.attributes)
    .find("span[title='" + attributeName + "'] ~ span.column-actions.pull-right")
    .find('.dropdown.pull-right')
    .click();
  cy.get(CSSselector.dataset_detail.attributes)
    .find("span[title='" + attributeName + "'] ~ span.column-actions.pull-right")
    .contains('Remove')
    .click();

  cy.get(CSSselector.dataset_detail.save_edit_fields).click();
  cy.get(CSSselector.dataset_detail.fields_list).should('not.contain', attributeName);
}

/*

This function to change any field parameters.
Currently it handles only "Display Name" parameter change/update.
We can add more parameter changes as needed

*/

export function editFieldParameters(attributeName: string, fieldAttr: Field) {
  cy.get(datasetManagement.locators.editField).click();

  // TODO find a better way to wait for backend requests
  cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('jsonselect');

  cy.get(datasetManagement.locators.attributes)
    .find("span[title='" + attributeName + "'] ~ span.column-actions.pull-right")
    .find('.column-edit-attr')
    .click();

  cy.wait('@jsonselect');
  cy.wait('@jsonselect');
  cy.wait(600);

  if (fieldAttr.hasOwnProperty('DisplayName')) {
    cy.get('[arcid="column-display"]').clear();
    cy.get('[arcid="column-display"]').type(fieldAttr.DisplayName);
  }

  cy.get('[arcid="column-edit-apply"]').click();
  cy.log('Change attributes for field =' + attributeName);
}

function clickOnEditFieldPencilIcon(attributeName: string) {
  cy.get(datasetManagement.locators.attributes)
    .find(`.field-wrapper:contains(${attributeName}) .column-edit-attr`)
    .click();
}

function selectTabOnEditFieldParametersModal(
  tabTitle: 'Basic Settings' | 'Display Format' | 'Color'
) {
  // Expression?
  cy.get(datasetManagement.locators.editColumnModal)
    .find(`.nav-link:contains(${tabTitle})`)
    .click();
}

function enableCustomColorOnEditFieldParametersModal() {
  // Expression?
  cy.get(datasetManagement.locators.editColumnModal)
    .find(`#custom-colors-enabled input`)
    .check();
}

export function setCustomColorSettingOfADatasetDimensionField(
  attributeName: string,
  isReversed: boolean,
  nthOfcolorpalette: number,
  colorOpacity: number,
  mapAcrossVisuals: boolean,
  fixedColors: Map<string, { rgb: string; hexColor: string }>
) {
  clickOnEditFieldPencilIcon(attributeName);
  selectTabOnEditFieldParametersModal('Color');
  enableCustomColorOnEditFieldParametersModal();

  // Click reversed checkbox
  if (isReversed) {
    cy.get(datasetManagement.locators.editColumnModal)
      .find(`${visual.locators.style.colors.reverseColorPaletteCheckbox}`)
      .check();
  }

  // Set colorpalette
  cy.get(`.settings-palette:nth-child(${nthOfcolorpalette})`).click();

  if (mapAcrossVisuals) {
    cy.get(datasetManagement.locators.editColumnModal)
      .find(`.color-palettes input[data-test-id="checkbox-fixed_color_legend"]`)
      .check();
  }

  cy.get(datasetManagement.locators.editColumnModal)
    .find(visual.locators.style.colors.fixDimensionColorsCheckbox)
    .check();
  cy.get(datasetManagement.locators.editColumnModal)
    .find(visual.locators.style.colors.fixDimensionColorsCheckbox)
    .uncheck();
  cy.get(datasetManagement.locators.editColumnModal)
    .find(visual.locators.style.colors.fixDimensionColorsCheckbox)
    .check();

  fixedColors.forEach((color, key) => {
    cy.get(`.dim-color-mapping tr:contains(${key})`)
      .find(`.single-color`)
      .click();
    cy.get('.popover-body')
      .find(`.settings-swatch[style*="background-color: ${color.hexColor}"]`)
      .trigger('mouseover')
      .click();
  });

  // Set opacity
  cy.get(
    `${datasetManagement.locators.editColumnModal} ${visual.locators.style.colors.opacitySliderInput}`
  ).invoke('val', colorOpacity);
  cy.get(
    `${datasetManagement.locators.editColumnModal} ${visual.locators.style.colors.opacitySliderInput}`
  ).trigger('change');

  // Apply
  cy.get(datasetManagement.locators.editColumnModal)
    .find(`button[type="submit"]`)
    .click();

  // Save
  cy.get('#save-attr-btn').click();
}
