import { notifications } from 'fixtures/locators/notifications';
import * as datasetManagement from 'lib/datasetManagement';
import { getVersion, modalClick, navigate_to_data } from 'lib/functions';
import { selectDataset } from 'lib/permissions/dataset_permissions';

/**
 *
 * To create joins between two tables in a datasets
 * @param {string} sourceDB
 * @param {string} sourceTable
 * @param {string} targetDB
 * @param {string} targetTable
 * @param {string} joinCols
 * @param {number} index
 *
 */

export function addJoin(
  sourceDB: string,
  sourceTable: string,
  targetDB: string,
  targetTable: string,
  joinCols: any,
  index: number
) {
  let sourceCol;
  // let colIndex = 1;

  // Navigate to data model tab
  cy.get(datasetManagement.locators.dataModel).click();

  // Click on edit join button
  cy.get(datasetManagement.locators.editJoin).click();

  const addJoinSelector =
    'div[data-original-title="' + sourceDB + '.' + sourceTable + '"].join-table-box i';

  cy.log('Click add join icon ' + addJoinSelector);
  cy.get(addJoinSelector).click();

  cy.log('Select target database and table for join');
  cy.get(datasetManagement.locators.databaseName).select(targetDB, { force: true });
  cy.get(datasetManagement.locators.tablename).select(targetTable, { force: true });

  cy.log('Click on Select button after selection of database and table name');
  cy.get(datasetManagement.locators.joinSelect).click();

  cy.log('Map join columns ' + joinCols);

  // as of now we are considering one join pair to create a join
  for (sourceCol in joinCols) {
    if (sourceCol) {
      cy.get(
        '#join-tables-form .join-tables-list > li > .modal-source-input > .form-control'
      ).select(sourceCol);
      cy.get(
        '#join-tables-form .join-tables-list > li > .modal-target-input > .form-control'
      ).select(joinCols[sourceCol]);
      // cy.get('#join-tables-form-' + index + '> [arcid="add-column-link"]').click();
      // colIndex = colIndex + 1;
    }
  }

  cy.log('Click Apply button');
  cy.get(datasetManagement.locators.applyJoin).click();

  // wait before saving join else UI doesn't show saved join.
  cy.wait(2000);

  cy.log('Click Save button');
  cy.intercept('GET', 'arc/roles/roles').as('saveDatamodel');
  cy.get(datasetManagement.locators.saveJoin).click({ force: true });
  cy.wait('@saveDatamodel')
    .its('response.statusCode')
    .should('be.eq', 200);
  // putting wait and reloading page because after clicking on save button join is not being saved properly.
  cy.wait(3000);
  cy.reload();
  cy.get(datasetManagement.locators.dataModel).click();

  cy.log('Verify the join is created by checking the join link and additional target table name');
  // check the join image visible
  cy.get('#viz-join .join-image').should('have.length', index);
  // cy.get('#viz-join .join-image').click();

  cy.log('Verify that data for both tables are loaded clicking on "Show Table" button');
  cy.get(datasetManagement.locators.showdata).click();
  cy.get('[data-test-id="datasets-data-table"] th').should('contain', sourceTable);
  cy.get('[data-test-id="datasets-data-table"] th').should('contain', targetTable);
}

export function createJoinSQLExpr(
  sourceDB: string,
  sourceTable: string,
  targetDB: string,
  targetTable: string,
  joinExpr: string,
  index: number
) {
  // Navigate to data model tab
  cy.get(datasetManagement.locators.dataModel).click();

  // Click on edit join button
  cy.get(datasetManagement.locators.editJoin).click();

  const addJoinSelector =
    'div[data-original-title="' + sourceDB + '.' + sourceTable + '"].join-table-box i';

  cy.log('Click add join icon ' + addJoinSelector);
  cy.get(addJoinSelector).click();

  cy.log('Select target database and table for join');
  cy.get(datasetManagement.locators.databaseName).select(targetDB, { force: true });
  cy.get(datasetManagement.locators.tablename).select(targetTable, { force: true });

  cy.log('Click on Select button after selection of database and table name');
  cy.get(datasetManagement.locators.joinSelect).click();

  cy.get('li > .form-control').click();
  // cy.get('[arcid="add-expr-link"]').click();

  verifySQLExpression(joinExpr);

  cy.log('Click Apply button');
  cy.get(datasetManagement.locators.applyJoin).click();

  // wait before saving join else UI doesn't show saved join.
  cy.wait(2000);

  cy.log('Click Save button');
  cy.intercept('GET', 'arc/roles/roles').as('saveDatamodel');
  cy.get(datasetManagement.locators.saveJoin).click({ force: true });
  cy.wait('@saveDatamodel')
    .its('response.statusCode')
    .should('be.eq', 200);
  // putting wait and reloading page because after clicking on save button join is not being saved properly.
  cy.wait(3000);
  cy.reload();
  cy.get(datasetManagement.locators.dataModel).click();

  cy.log('Verify the join is created by checking the join link and additional target table name');
  // check the join image visible
  cy.get('#viz-join .join-image').should('have.length', index);
  // cy.get('#viz-join .join-image').click();

  cy.log('Verify that data for both tables are loaded clicking on "Show Table" button');
  cy.get(datasetManagement.locators.showdata).click();
  cy.get('[data-test-id="datasets-data-table"] th').should('contain', sourceTable);
  cy.get('[data-test-id="datasets-data-table"] th').should('contain', targetTable);
}

/**
 * select the specific join or switch from one join to the other.
 * Join must be created before executing this function
 * @param joinName: name of join to be selected (possible values can be: inner join, left outer join)
 */
export function selectJoin(joinName: string) {
  // Click on edit join button
  cy.get(datasetManagement.locators.editJoin).click();

  // Click on join image
  cy.get('#viz-join .join-image').click({ force: true });

  // Click on specified join

  getVersion().then((versionNumber) => {
    const versionNum = Number(versionNumber);
    let locator = `div.popover-content div[typetext="${joinName}"] img`;
    if (versionNum >= 7) {
      locator = `div.popover-body div[typetext="${joinName}"] img`;
    }

    cy.get(locator).click({ force: true });

    if (versionNum >= 7) {
      locator = `div.popover-body div[typetext="${joinName}"] img`;
      // Click on join image
      cy.get('#viz-join .join-image').click({ force: true });
    }

    cy.get(locator)
      .parent()
      .should('have.class', 'join-selected');

    // Save the changes
    cy.log('Click Save button');
    cy.intercept('GET', 'arc/roles/roles').as('saveDatamodel');
    cy.get(datasetManagement.locators.saveJoin).click({ force: true });
    cy.wait('@saveDatamodel');
  });
}

/**
 * Delete a join. This function assumes there is only one join exist on a dataset.
 *
 */
export function deleteJoin() {
  // Navigate to data model tab
  cy.get(datasetManagement.locators.dataModel).click();

  // Click on edit join button
  cy.get(datasetManagement.locators.editJoin).click();

  // Click on join image
  cy.get(datasetManagement.locators.vizJoinImage).click();

  // Click on Delete Join button
  cy.get(datasetManagement.locators.deleteJoinBtn).click();

  // Click on Save.
  cy.intercept('PUT', '/arc/datasets/dataset/*').as('joinUpdated');
  cy.get(datasetManagement.locators.saveJoin).click({ force: true });
  cy.wait('@joinUpdated');
}

/**
 * to enter and validate a filter expression (sql expression) while adding a segment
 * throws an error if the expression is invalid and fails the test
 * @export
 * @param {string} sqlExpression
 */
export function verifySQLExpression(sqlExpression: string) {
  cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('verifySQL');
  // require to wait for the textarea focus then perform type comand

  cy.get(datasetManagement.locators.segFilterExprEdit).focus();

  cy.get(datasetManagement.locators.segFilterExpr).should('have.class', 'ace_focus');

  cy.get(datasetManagement.locators.segFilterExpr + '.ace_editor').then(($elem) => {
    cy.window()
      .its('ace' as any)
      .invoke('edit', $elem.attr('id'))
      .invoke('setValue', sqlExpression);
  });

  cy.get(datasetManagement.locators.segFilterExprValidateBtn)
    .should('have.text', 'Validate Expression')
    .click({ force: true });

  cy.wait('@verifySQL');
  // Assert for the expected alert message which shows inside modal dialog
  cy.get(notifications.alert_success).should('contain', 'Validation Successful!');

  modalClick(datasetManagement.locators.expressionApply);
  cy.wait('@verifySQL');

  cy.get(datasetManagement.locators.expressionApply, { timeout: 30000 }).should('not.be.visible');
}

/**
 *  Creat new segment
 * @param {string} 'Segment Name'
 * @param {string} 'Segment Group Name'
 * @param {string} 'Filter Expression'
 */

export function createNewSegment(
  connectionName: string,
  datasetName: string,
  segName: string,
  filterExpression: string,
  segGrp?: string
) {
  navigate_to_data();
  selectDataset(connectionName, datasetName);
  cy.get(datasetManagement.locators.segments).click();
  cy.get(datasetManagement.locators.newSegmentBtn).click();
  cy.get(datasetManagement.locators.segmentName).type(segName);
  if (segGrp) {
    cy.get(datasetManagement.locators.segmentGroup)
      .invoke('val', segGrp)
      .trigger('input')
      .trigger('change');
  }
  cy.get(datasetManagement.locators.segmentFilters).focus();
  verifySQLExpression(filterExpression);

  cy.get(datasetManagement.locators.segementSaveBtn).click();

  // Check if the model window is closed which means successfully saved.
  cy.get('#viz-segment-create').should('not.be.visible');
}

/**
 *  Delete segment
 */

export function deleteSegment(segName: string) {
  cy.log('Delete a Segment ');
  cy.intercept('DELETE', '/arc/datasets/segment/*').as('segmentDeleted');
  cy.get(datasetManagement.locators.segListTableRows).each(($el, index, $list) => {
    // cy.log('Object ',cy.wrap($el));
    cy.log('el = ' + $el.text() + ' index = ' + index + ' list = ' + $list);
    if ($el.text().indexOf(segName) !== -1) {
      cy.log('el = ' + $el + ' index = ' + index + ' list = ' + $list);
      cy.wrap($el)
        .find('td .dataset-segment-delete')
        .click();

      // wait for delete segment button event listeners to be attached,
      // since they close confirmation modal on save
      cy.wait(500);
      cy.get('#delete-segment-yes').click();
      cy.wait('@segmentDeleted');
    }
  });
}

export function saveEditField() {
  cy.intercept('PUT', '/arc/datasets/dataset/*').as('fieldSaved');
  cy.get(datasetManagement.locators.saveEditFields).click();
  cy.wait('@fieldSaved');
}
