import * as dis_format from 'lib/display_format';
import { CSSselector } from 'locators/common/CSSselector';

/**
 * This function is used to change the display format of date from dataset view as well as from visual edit view.
 * @param visual : true if wants to change the display format of visual shelf
 * @param shelfLocator : locator of a data field
 * @param format : date format
 */
export function changeDisplayFormatOfdate(visual: boolean, shelfLocator: string, format: string) {
  if (visual === true) {
    cy.get(shelfLocator).click();
    cy.get(dis_format.locators.viz_display_format).click();
    cy.get(dis_format.locators.category).select('date');
    cy.get(dis_format.locators.basic_formats).select(format);
    cy.get(CSSselector.display_format_save_button).click();
  } else {
    cy.get(shelfLocator).click();
    cy.get(dis_format.locators.display_format).click();
    cy.get(dis_format.locators.category).select('date');
    cy.get(dis_format.locators.basic_formats).select(format);
    cy.get(dis_format.locators.apply).click();
  }
}

/**
 * This function is used to change the display format of number from dataset view as well as from visual edit view.
 * @param visual : true if wants to change the display format of visual shelf
 * @param shelfLocator : locator of a data field
 * @param precision : pricision number to be displayed after decimal point
 */
export function changeDisplayFormatOfnumber(
  visual: boolean,
  shelfLocator: string,
  precision?: string
) {
  if (visual === true) {
    cy.get(shelfLocator).click();
    cy.get(dis_format.locators.viz_display_format).click();
    cy.get(dis_format.locators.category).select('number');
    if (precision) {
      cy.get(CSSselector.decimal_places)
        .clear()
        .type(precision);
    }
    cy.get(CSSselector.display_format_save_button).click();
  } else {
    cy.get(shelfLocator).click();
    cy.get(dis_format.locators.display_format).click();
    cy.get(dis_format.locators.category).select('number');
    cy.get(dis_format.locators.apply).click();
  }
}

/**
 * This function is used to change the display format of integer from dataset view as well as from visual edit view.
 * @param visual : true if wants to change the display format of visual shelf
 * @param shelfLocator : locator of a data field
 */
export function changeDisplayFormatOfInteger(visual: boolean, shelfLocator: string) {
  if (visual === true) {
    cy.get(shelfLocator).click();
    cy.get(dis_format.locators.viz_display_format).click();
    cy.get(dis_format.locators.category).select('integer');
    cy.get(CSSselector.comma_separator).click();
    cy.get(CSSselector.display_format_save_button).click();
  } else {
    cy.get(shelfLocator).click();
    cy.get(dis_format.locators.display_format).click();
    cy.get(dis_format.locators.category).select('integer');
    cy.get(dis_format.locators.apply).click();
  }
}

/**
 * This function is used to change the display format of currency from dataset view as well as from visual edit view.
 * @param visual : true if wants to change the display format of visual shelf
 * @param shelfLocator : locator of a data field
 */
export function changeDisplayFormatOfCurrency(visual: boolean, shelfLocator: string) {
  if (visual === true) {
    cy.get(shelfLocator).click();
    cy.get(dis_format.locators.viz_display_format).click();
    cy.get(dis_format.locators.category).select('currency');
    cy.get(CSSselector.display_format_save_button).click();
  } else {
    cy.get(shelfLocator).click();
    cy.get(dis_format.locators.display_format).click();
    cy.get(dis_format.locators.category).select('currency');
    cy.get(dis_format.locators.apply).click();
  }
}
