import * as settingLocators from 'lib/customsettings/locators';
import { locators } from 'lib/filter/locators';
import { modalClick, waitForVisualsLoaded } from 'lib/functions';
import { CSSselector } from 'locators/common/CSSselector';

/**
 * selects afilter value from the filter widget dropdown
 * asserts if the right parameter is sent in the request
 * and if the name is selected
 * and that no alert container showed up
 *
 * @export
 * @param {string[]} filterParam
 */

/**
 *
 *
 * @export
 * @param {string[]} filterParam filter paramters to select on the filter, works only on the first filter for now
 * @param {string} [visualUrl] url of the dashboard to which to apply filter, if not specified the filter is applied to currently open dashboard
 */
export function applyDropdownFilterToVisual(
  filterParam: string[],
  visualUrl?: string | null,
  filterIndex = 1,
  alreadyAddedFilterCount = 0
) {
  if (visualUrl) {
    cy.visit(visualUrl);
    waitForVisualsLoaded();
  }

  cy.get(CSSselector.filters.filter_dropdown_specific(filterIndex)).click();
  let exp = '';
  for (let i = 0, len = filterParam.length; i < len; i++) {
    // based on if the checkbox is checked/not checked proceede
    cy.get(CSSselector.filters.filter_picklist_specific(filterIndex))
      .contains(filterParam[i])
      .parent()
      .scrollIntoView()
      .find("input[type='checkbox']")
      .then(($checkbox) => {
        const tempExp = "'" + filterParam[i] + "'";
        if (i > 0) {
          exp = exp + ',+' + tempExp;
        }
        if (i === 0) {
          exp = tempExp;
        }
        if (!$checkbox.is(':checked')) {
          cy.get(CSSselector.filters.filter_picklist_specific(filterIndex))
            .contains(filterParam[i])
            .parent()
            .scrollIntoView()
            .find("input[type='checkbox']")
            .check();

          if (i + alreadyAddedFilterCount < 2) {
            cy.get(CSSselector.filters.filter_selected_names_specific(filterIndex)).should(
              'contain',
              filterParam[i]
            );
          } else if (i + alreadyAddedFilterCount >= 2) {
            cy.get(CSSselector.filters.filter_selected_count_specific(filterIndex)).should(
              'contain',
              i + 1 + alreadyAddedFilterCount
            );
          }
        }
      });

    // validate request
  }

  cy.get(CSSselector.dashboard.alert_container).should('not.be.visible');
  cy.get(CSSselector.filters.filter_dropdown_specific(filterIndex)).click();
  waitForVisualsLoaded();
}

/**
 * selects afilter value from the filter widget dropdown
 * asserts if the right parameter is sent in the request
 * and if the name is selected
 * and that no alert container showed up
 *
 * @export
 * @param {string[]} filterParam
 */
export function applySingleSelectFilterToVisual(filterParam: string[], visualUrl?: string) {
  if (visualUrl) {
    cy.visit(visualUrl);
    waitForVisualsLoaded();
  }

  cy.get(CSSselector.filters.filter_select_dropdown).click({
    force: true,
  });
  let exp = '';
  for (let i = 0, len = filterParam.length; i < len; i++) {
    // based on if the checkbox is checked/not checked proceede
    cy.get(CSSselector.filters.filter_picklist)
      .contains(filterParam[i])
      .scrollIntoView()
      .parent()
      .find("input[type='checkbox']")
      .then(($checkbox) => {
        const tempExp = "'" + filterParam[i] + "'";
        if (i > 0) {
          exp = exp + ',+' + tempExp;
        }
        if (i === 0) {
          exp = tempExp;
        }
        if (!$checkbox.is(':checked')) {
          cy.get(CSSselector.filters.filter_picklist)
            .contains(filterParam[i])
            .parent()
            .scrollIntoView()
            .find("input[type='checkbox']")
            .check();

          if (i < 2) {
            cy.log('1.value of i' + i);
            cy.get(CSSselector.filters.filter_selected_names).should('contain', filterParam[i]);
          } else if (i >= 2) {
            cy.log('2.value of i' + i);
            cy.get(CSSselector.filters.filter_selected_count).should('contain', i + 1);
          }
        }
      });

    // validate request
  }

  cy.get(CSSselector.dashboard.alert_container).should('not.be.visible');
  cy.get(CSSselector.filters.filter_select_dropdown).click({
    force: true,
  });
}

/**
 * verifies if the filter that got applied
 * does it update the << >> section of title and x-axis text
 *
 * @export
 * @param {string[]} filterParam can enter an array of filterParams
 * @param {string} visualViewUrl to verify filter in view mode of a visual
 */
export function verifyAppliedFilter(filterParam: string[], visualViewUrl?: string) {
  if (visualViewUrl) {
    cy.visit(visualViewUrl);
  }
  for (let i = 0, len = filterParam.length; i < len; i++) {
    const trimmedText = filterParam[i].substring(0, 15);

    cy.get(CSSselector.dashboard.visual.title).should('contain', trimmedText);
    cy.get(CSSselector.dashboard.visual.x_axis).should('contain', trimmedText);
  }
}

/**
 * verifies if the dashboard filter got applied
 * also check if the filter is excluded
 * @param {string[]} filterParam can enter an array of filterParams
 * @param {boolean} exclude true|false
 * @param {boolean} datasetScope true|false
 */
export function verifyDashboardFilter(
  filterParam: string[],
  exclude: boolean = false,
  datasetScope: boolean = true
) {
  for (let i = 0, len = filterParam.length; i < len; i++) {
    const trimmedText = filterParam[i].substring(0, 15);

    if (exclude === false && datasetScope === true) {
      cy.get(
        "div[class='gridster ready']>ul > li:nth-child(2) svg > svg > g.x.arc-viz-axis > g.tick"
      )
        .should('contain', trimmedText)
        .should('not.contain', 'ONLINE')
        .should('not.contain', 'QOS');
    }
    if (datasetScope === false) {
      cy.get(
        "div[class='gridster ready']>ul > li:nth-child(2) svg > svg > g.x.arc-viz-axis > g.tick"
      )
        .should('contain', trimmedText)
        .and('contain', 'ONLINE')
        .and('contain', 'QOS');
    } else {
      cy.get(
        "div[class='gridster ready']>ul > li:nth-child(2) svg > svg > g.x.arc-viz-axis > g.tick"
      ).should('not.contain', trimmedText);
    }
    // cy.get("div[class='gridster ready']>ul > li:nth-child(1) svg > svg > g.x.arc-viz-axis > g.tick").should('contain', trimmedText);
    // cy.get("div[class='gridster ready']>ul > li:nth-child(5) svg > svg > g.x.arc-viz-axis > g.tick").should('contain', trimmedText);
  }
}

/**
 * click the back button on the filter control and wait till it loads
 *
 * @export
 */
export function filterControlBack() {
  cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('navigatedBack');
  cy.get(CSSselector.dashboard.back_button).click();
  cy.wait(1000);
  cy.get(CSSselector.dashboard.current_parameters).trigger('mouseout');
  cy.wait('@navigatedBack')
    .its('response.statusCode')
    .should('eq', 200);
}

/**
 * click the forward button on the filter control and wait till it loads
 *
 * @export
 */
export function filterControlForward() {
  cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('navigatedForward');
  cy.get(CSSselector.dashboard.forward_button).click();
  cy.wait(1000);
  cy.get(CSSselector.dashboard.current_parameters).trigger('mouseout');
  cy.wait('@navigatedForward')
    .its('response.statusCode')
    .should('eq', 200);
}

/**
 * click the reset button on the filter control and wait till it loads
 *
 * @export
 */
export function filterControlReset() {
  cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('reset');
  cy.get(CSSselector.dashboard.reset_button).click();
  cy.get(CSSselector.dashboard.current_parameters).trigger('mouseout');
  cy.wait('@reset')
    .its('response.statusCode')
    .should('eq', 200);
}

/**
 * clicks on the funnel icon and verifies the parameters being shown there
 * also verifies that funnel icon is disabled when no paramters are sent
 *
 * @export
 * @param {string} params
 */
export function verifyCurrentParameters(params: string[]) {
  params = params || [];
  cy.get(CSSselector.dashboard.current_parameters_tooltip).should('not.be.visible');
  cy.get(CSSselector.dashboard.current_parameters).should('not.have.class', 'freeze');
  cy.get(CSSselector.dashboard.current_parameters).trigger('mouseover');
  cy.get(CSSselector.dashboard.current_parameters_tooltip).should('be.visible');
  cy.get(CSSselector.dashboard.current_parameters).should('not.have.class', 'freeze');
  cy.get(CSSselector.dashboard.current_parameters).trigger('mouseout');
  cy.get(CSSselector.dashboard.current_parameters_tooltip).should('not.be.visible');
  cy.get(CSSselector.dashboard.current_parameters).should('not.have.class', 'freeze');

  cy.get(CSSselector.dashboard.current_parameters)
    .trigger('mouseover')
    .click();

  // check if it will stay open
  cy.get(CSSselector.dashboard.current_parameters_tooltip).should('be.visible');
  cy.get(CSSselector.dashboard.current_parameters_tooltip).should('have.class', 'freezableTooltip');
  params.forEach((param) => {
    cy.get(CSSselector.dashboard.current_parameters_tooltip).should('contain', param);
  });

  cy.get(CSSselector.dashboard.current_parameters).should('have.class', 'freeze');
  cy.get(CSSselector.dashboard.current_parameters).trigger('mouseout');

  cy.get(CSSselector.dashboard.current_parameters)
    .trigger('mouseover')
    .click();
  cy.get(CSSselector.dashboard.current_parameters).trigger('mouseout');
  cy.get(CSSselector.dashboard.current_parameters_tooltip).should('not.be.visible');
  cy.get(CSSselector.dashboard.current_parameters).should('not.have.class', 'freeze');
}

/**
 * clicks on the setting icon of the filter -> values tab and updates the values
 * Also verifies title filter title got updated.
 *
 * Can pass '' for field that you dont want to update
 * @export
 * @param {string} fromField
 * @param {string} title
 * @param {string} outputParameter
 * @param {string} filterFields
 */
export function configureFilterValues(
  fromField: string,
  title: string,
  outputParameter: string,
  filterFields: string,
  filterIndex = 1
) {
  cy.get(CSSselector.filters.configure_filter_specific(filterIndex)).click({ force: true });
  cy.get(CSSselector.filters.settings_navtab)
    .contains('Values')
    .click();

  if (!(fromField === '' || fromField === null)) {
    cy.get(CSSselector.filters.values.from_base_field)
      .clear()
      .type(fromField);
  }
  if (!(title === '' || title === null)) {
    cy.get(CSSselector.filters.values.title)
      .clear()
      .type(title);
  }
  if (!(outputParameter === '' || outputParameter === null)) {
    cy.get(CSSselector.filters.values.output_parameter)
      .clear()
      .type(outputParameter);
  }
  if (!(filterFields === '' || filterFields === null)) {
    cy.get(CSSselector.filters.values.filter_fields)
      .clear()
      .type(filterFields);
  }

  cy.get(CSSselector.filters.save_settings).click({ force: true });
  cy.wait(2000);
  // verify it got updated
  cy.get(CSSselector.filters.filter_title).should('have.text', title);
}

/**
 * clicks on the setting icon of the filter -> data tab and updates the values
 *
 * Can pass '' for field that you dont want to update
 *
 * @export
 * @param {string} maxRows
 * @param {boolean} disableQueries true|false will check|uncheck the bok
 */

/**
 *
 * clicks on the setting icon of the filter -> display settings tab and updates the values
 * "settings" : { "width": "", "item_count": "","textbox_parameter": "", "upload_manually": "", "single_select": "", "all_option": "", "display_dropdown":"", "multi_param":"", "hide_empty":"","remember_selections": "", "apply_together":""}
 *
 * pass '' for field that you dont want to update
 * @export
 * @param {*} settings
 */
export function configureFilterDisplaySettings(settings: any, filterIndex = 1) {
  cy.get(CSSselector.filters.configure_filter_specific(filterIndex)).click({ force: true });

  cy.get(CSSselector.filters.settings_navtab)
    .contains('Data')
    .click();

  if (!(settings.item_count === '' || settings.item_count === null)) {
    cy.get(CSSselector.filters.display.item_count)
      .clear()
      .type(settings.item_count);
  }

  cy.get(CSSselector.filters.settings_navtab)
    .contains('Display Settings')
    .click();

  // verify this assertion with VA
  // cy.window()
  // .its('arc.globals.reports.getReport(126).viz().user_settings().settings()')
  // .invoke('length')

  console.log('settings ' + settings);
  if (!(settings.width === '' || settings.width === null)) {
    cy.get(CSSselector.filters.display.width)
      .clear()
      .type(settings.width);
  } else if (settings.width === '') {
    cy.get(CSSselector.filters.display.width).clear();
  }

  interact_checkbox(CSSselector.filters.display.textbox_parameter, settings.textbox_parameter);
  interact_checkbox(CSSselector.filters.display.upload_manually, settings.upload_manually);
  interact_checkbox(CSSselector.filters.display.single_select, settings.single_select);
  if (settings.single_select) {
    interact_checkbox(CSSselector.filters.display.all_option, settings.all_option);
    interact_checkbox(CSSselector.filters.display.display_dropdown, settings.display_dropdown);
  }
  interact_checkbox(CSSselector.filters.display.multi_param, settings.multi_param);
  interact_checkbox(CSSselector.filters.display.hide_empty, settings.hide_empty);
  interact_checkbox(CSSselector.filters.display.remember_selections, settings.remember_selections);
  interact_checkbox(CSSselector.filters.display.apply_together, settings.apply_together);

  cy.get(CSSselector.filters.save_settings).click();
}

/**
 * clicks on the setting icon of the filter -> scope tab and updates the value
 *
 * @export
 * @param {boolean} allDashboardScope true|false
 * @param {boolean} sendParamsToDataset true|false
 */
export function configureFilterScope(
  filterVisibility: string,
  paramScope: string,
  filterIndex = 1
) {
  cy.get(CSSselector.filters.configure_filter_specific(filterIndex)).click({ force: true });

  cy.get(CSSselector.filters.settings_navtab)
    .contains('Scope')
    .click();

  interact_checkbox((CSSselector.filters.visibility as any)[filterVisibility], true);
  interact_checkbox((CSSselector.filters.scope as any)[paramScope], true);

  modalClick(CSSselector.filters.save_settings);
  cy.wait(1000);
}

/**
 * to be implemented
 * clicks on the setting icon of the filter -> custom sytle tab
 *
 * @export
 */
export function configureFilterCustomStyle(filterIndex = 1) {
  cy.get(CSSselector.filters.configure_filter_specific(filterIndex)).click({ force: true });

  cy.get(CSSselector.filters.settings_navtab)
    .contains('Custom Style')
    .click();
  cy.get(CSSselector.filters.save_settings).click();
}

// date filter related commands

/**
 * clicks on the setting icon of the Date filter -> data tab and updates the values
 * pass '' for field that you dont want to update
 * @export
 * @param {string} maxRows
 * @param {boolean} disableQueries true|false
 */
export function configureDateFilterData(maxRows: string, disableQueries: boolean, filterIndex = 1) {
  cy.get(CSSselector.filters.configure_filter_specific(filterIndex)).click({ force: true });
  cy.get(CSSselector.filters.settings_navtab)
    .contains('Data')
    .click();

  if (!(maxRows === '' || maxRows === null)) {
    cy.get(CSSselector.filters.data.max_rows)
      .clear()
      .type(maxRows);
  }

  interact_checkbox(CSSselector.filters.data.disable_incremental_queries, disableQueries);

  cy.get(CSSselector.filters.save_settings).click();
}

/**
 * clicks on the setting icon of the filter -> basic tab and updates the values
 * pass '' for field that you dont want to update
 * @export
 * @param {string} baseField
 * @param {string} title
 * @param {string} outputParameter
 */
export function configureDateFilterBasic(
  baseField: string,
  title: string,
  outputParameter: string,
  allowTimeRange: boolean
) {
  cy.get(CSSselector.date_filters.configure_filter).click({ force: true });
  cy.get(CSSselector.date_filters.settings_navtab)
    .contains('Basic')
    .click();

  if (!(baseField === '' || baseField === null)) {
    cy.get(CSSselector.date_filters.basic.base_field)
      .clear()
      .type(baseField);
  }
  if (!(title === '' || title === null)) {
    cy.get(CSSselector.date_filters.basic.title)
      .clear()
      .type(title);
  }
  if (!(outputParameter === '' || outputParameter === null)) {
    cy.get(CSSselector.date_filters.basic.output_parameter)
      .clear()
      .type(outputParameter);
  }

  interact_checkbox(CSSselector.date_filters.basic.timerange_cb, allowTimeRange);
  cy.get(CSSselector.date_filters.save_settings).click();
  // verify it got updated
  cy.get(CSSselector.date_filters.filter_title).should('have.text', title);
}

export function configureDateFilterStyle(width: string, allowTimeRange: boolean) {
  cy.get(CSSselector.date_filters.configure_filter).click({ force: true });
  cy.get(CSSselector.date_filters.settings_navtab).should('be.visible');
  cy.get(CSSselector.date_filters.settings_navtab)
    .contains('Style')
    .click();

  if (!(width === '' || width === null)) {
    cy.get(CSSselector.date_filters.style.width)
      .clear()
      .type(width);
  }
  interact_checkbox(CSSselector.date_filters.basic.timerange_cb, allowTimeRange);
}
/**
 * to be implemented
 *
 * @export
 * @returns
 */
export function configureDateFilterRanges() {
  return;
}

export function configureDateFilterScope(filterVisibility: string, paramScope: string) {
  cy.get(CSSselector.date_filters.configure_filter).click({ force: true });

  cy.get(CSSselector.date_filters.settings_navtab)
    .contains('Scope')
    .click();

  //  TODO: refactor later - it is not good we handle this as any object -> switch case
  interact_checkbox((CSSselector.filters.visibility as any)[filterVisibility] as string, true);
  interact_checkbox((CSSselector.filters.scope as any)[paramScope], true);

  cy.get(CSSselector.date_filters.save_settings).click();
}

/**
 * to be implemented
 *
 * @export
 * @returns
 */
export function configureDateFilterCustomStyle() {
  return;
}
/**
 *
 *
 * @export
 * @param {string} startDate start date/ quick filter values
 * @param {string} endDate
 * @param {string} menuOption quick/relative/fixed
 */
export function applyDateFilterToVisual(
  startDate: string,
  endDate: string,
  menuOption: string,
  visualUrl?: string,
  filterParam?: string,
  strictTooltipCheck = true
) {
  if (visualUrl) {
    cy.visit(visualUrl);
    waitForVisualsLoaded();
  }

  if (menuOption !== '') {
    cy.get(CSSselector.date_filters.filter_dropdown).click();
    cy.get(CSSselector.date_filters.dropdown_menu)
      .contains(menuOption)
      .click();

    switch (menuOption) {
      case 'Fixed dates':
        cy.get(CSSselector.date_filters.filter_dropdown).click();
        cy.get(CSSselector.date_filters.dropdown_menu)
          .contains(menuOption)
          .click();
        cy.get(CSSselector.date_filters.fixed_date_picker)
          .find(CSSselector.date_filters.fixed_datepicker_start)
          .clear()
          .type(startDate);
        cy.get(CSSselector.date_filters.fixed_date_picker)
          .find(CSSselector.date_filters.fixed_datepicker_end)
          .clear()
          .type(endDate);
        cy.get(CSSselector.date_filters.fixed_date_picker)
          .find(CSSselector.date_filters.fixed_datepicker_apply)
          .click();

        waitForVisualsLoaded();

        const tooltipCheck = strictTooltipCheck ? 'have.text' : 'include.text';

        // verify
        if (filterParam) {
          cy.get('.arc-viz-filter-container .filter-pill').trigger('mouseover');
          cy.get('.header-label')
            .contains(filterParam)
            .parent('tr')
            .find('.viz-tooltip-string')
            .should(
              tooltipCheck,
              `[${filterParam}] between '${startDate} 00:00:00' and '${endDate} 00:00:00'`
            );
        }

        break;
      case 'Relative dates':
        cy.get(CSSselector.date_filters.relative_datepicker)
          .find(CSSselector.date_filters.relative_datepicker_anchorto)
          .click();
        cy.get(CSSselector.date_filters.relative_datepicker)
          .find(CSSselector.date_filters.relative_datepicker_anchorto_text)
          .then(($elem) => {
            $elem.text(startDate);
          });
        cy.get(CSSselector.date_filters.relative_datepicker)
          .find(CSSselector.date_filters.relative_datepicker_units)
          .select(endDate);
        cy.get(CSSselector.date_filters.relative_datepicker)
          .find(CSSselector.date_filters.relative_datepicker_next_one)
          .click();
        cy.get(CSSselector.date_filters.relative_datepicker)
          .find(CSSselector.date_filters.relative_datepicker_apply)
          .click();

        waitForVisualsLoaded();

        break;
      case 'Quick filters':
        // verify
        cy.get(CSSselector.date_filters.quickfilters).contains('1M');
        cy.get(CSSselector.date_filters.quickfilters).contains('5M');
        cy.get(CSSselector.date_filters.quickfilters).contains('15M');
        cy.get(CSSselector.date_filters.quickfilters).contains('1H');
        cy.get(CSSselector.date_filters.quickfilters).contains('6H');
        cy.get(CSSselector.date_filters.quickfilters).contains('12H');
        cy.get(CSSselector.date_filters.quickfilters)
          .contains(startDate)
          .click();
        waitForVisualsLoaded();

        break;
      default:
        console.log('Invalid value passed');
        break;
    }
    cy.get(CSSselector.dashboard.alert_container).should('not.be.visible');
  }
}

//// helpers

/**
 * This method will set the checkbox value to checked or unchecked independent of the initial state of
 * of the checkbox
 * @param {string} locator
 * @param {boolean} flag
 */
export function interact_checkbox(locator: string, flag: boolean) {
  console.log('locator' + locator + 'flag' + flag);
  if (flag) {
    cy.get(locator).then(($checkbox) => {
      if (!$checkbox.is(':checked')) {
        cy.get(locator).check();
      }
    });
  } else {
    cy.get(locator).then(($checkbox) => {
      if ($checkbox.is(':checked')) {
        cy.get(locator).uncheck();
      }
    });
  }
}

/**
 * clicks on the setting icon of the filter -> scope tab and verifies Send parameters with dataset scope is checked
 *
 * @export
 */
export function verifySendParameterScopeofAllFilters() {
  cy.get(CSSselector.filters.configure_filter_specific(1)).click({ force: true });
  cy.get(CSSselector.filters.settings_navtab)
    .contains('Scope')
    .click();
  cy.get(CSSselector.filters.scope.current_dataset).should('be.checked');
  cy.get(CSSselector.filters.cancel_settings).click();

  cy.get(CSSselector.filters.configure_filter_specific(2)).click({ force: true });
  cy.get(CSSselector.filters.settings_navtab)
    .contains('Scope')
    .click();
  cy.get(CSSselector.filters.scope.current_dataset).should('be.checked');
  cy.get(CSSselector.filters.cancel_settings).click();
}

/**
 * clicks on the funnel icon and verifies the parameters being shown there
 * also verifies that funnel icon is disabled when no paramters are sent
 * verifies the exclude: in as well
 *
 * @export
 * @param {string} params
 * @param {boolean} exclude true|false
 */
export function verifyCurrentParametersForTelco(
  filterName: string,
  params: string,
  exclude?: boolean,
  filterIndex: number = 1
) {
  cy.get(CSSselector.dashboard.current_parameters)
    .trigger('mouseover')
    .click();

  // check if it will stay open
  cy.get(CSSselector.dashboard.current_parameters_tooltip).should('be.visible');
  cy.get(CSSselector.dashboard.current_parameters_tooltip).should('have.class', 'freezableTooltip');
  cy.get(CSSselector.dashboard.current_parameters_tooltip).should('contain', params);
  if (exclude === false) {
    if (filterIndex === 1) {
      cy.get(CSSselector.dashboard.current_parameters_tooltip).should(
        'contain',
        filterName + '.data: '
      );
      cy.get(CSSselector.dashboard.current_parameters_tooltip).should(
        'contain',
        filterName + '.exclude: in'
      );
    } else {
      cy.get(CSSselector.dashboard.current_parameters_tooltip).should(
        'contain',
        filterName + '.data: '
      );
      cy.get(CSSselector.dashboard.current_parameters_tooltip).should(
        'contain',
        filterName + '.exclude: in'
      );
      // make it go away - so just clickin on it again should dismiss - but not working so clicking on white space
      // cy.get(sel.dashboard.current_parameters).trigger('hide').click()
      cy.get(CSSselector.dashboard.title_whitespace).click();
      cy.get(CSSselector.dashboard.current_parameters_tooltip).should('not.be.visible');
    }
  }
  if (exclude === true) {
    if (filterIndex === 1) {
      cy.get(CSSselector.dashboard.current_parameters_tooltip).should(
        'contain',
        filterName + '.data: '
      );
      cy.get(CSSselector.dashboard.current_parameters_tooltip).should(
        'contain',
        filterName + '.exclude: not in'
      );
    } else {
      cy.get(CSSselector.dashboard.current_parameters_tooltip).should(
        'contain',
        filterName + '.data: '
      );
      cy.get(CSSselector.dashboard.current_parameters_tooltip).should(
        'contain',
        filterName + '.exclude: not in'
      );
    }
  }
  // make it go away - so just clickin on it again should dismiss - but not working so clicking on white space
  // cy.get(CSSselector.dashboard.current_parameters).trigger('hide').click()
  cy.get(CSSselector.dashboard.title_whitespace).click();
  cy.get(CSSselector.dashboard.current_parameters_tooltip).should('not.be.visible');
  if (params === 'No parameters') {
    cy.get(CSSselector.dashboard.current_parameters).should(
      'have.attr',
      'style',
      'color: lightgray;'
    );
  }
}

/**
 * selects afilter value from the filter widget dropdown
 * asserts if the right parameter is sent in the request
 * and if the name is selected
 * and that no alert container showed up
 *
 * @export
 * @param {string[]} filterParam
 */
export function applyFilterToTelcoDashboard(filterParam: string[], filterIndex: number = 1) {
  waitForVisualsLoaded();
  cy.wait(3000);
  cy.get(CSSselector.filters.filter_dropdown_specific(filterIndex)).click();
  let exp = '';
  for (let i = 0, len = filterParam.length; i < len; i++) {
    // based on if the checkbox is checked/not checked proceede
    cy.get('div.control-container li:nth-child(' + filterIndex + ') div > ul > li')
      .contains(filterParam[i])
      .scrollIntoView()
      .find("input[type='checkbox']")
      .then(($checkbox) => {
        const tempExp = "'" + filterParam[i] + "'";
        if (i > 0) {
          exp = exp + ',+' + tempExp;
        }
        if (i === 0) {
          exp = tempExp;
        }
        if (!$checkbox.is(':checked')) {
          cy.get('div.control-container li:nth-child(' + filterIndex + ') div > ul > li')
            .contains(filterParam[i])
            .scrollIntoView()
            .find("input[type='checkbox']")
            .check();
          waitForVisualsLoaded();
          if (i < 2) {
            cy.get(
              'div.control-container li:nth-child(' +
                filterIndex +
                ') span.selected-names-wrapper > span'
            ).should('contain', filterParam[i]);
          } else if (i >= 2) {
            cy.get(
              'div.control-container li:nth-child(' +
                filterIndex +
                ') span.selected-count-wrapper > span.badge.selected-count'
            ).should('contain', i + 1);
          }
        }
      });
    // validate request
  }

  cy.get(CSSselector.dashboard.alert_container).should('not.be.visible');
  cy.get(CSSselector.filters.filter_dropdown_specific(filterIndex)).click();
}

/**
 * click exclude checkbox on the filter
 *
 * @export
 * @param {number} [filterIndex=1] index of the filter whose exclude selected check box needs to be checked
 */
export function excludeFilterCheck(filterIndex: number = 1) {
  const excludeCBLocator =
    'div.control-container li:nth-child(' +
    filterIndex +
    ") div div.picklist-exclude-wrapper input[class='picklist-exclude']";
  cy.get(excludeCBLocator).check({ force: true });
  waitForVisualsLoaded();
}

/**
 * uncheck exclude selected checkbox on the filter
 *
 * @export
 * @param {number} [filterIndex=1] index of the filter whose exclude selected check box needs to be checked
 */
export function excludeFilterUncheck(filterIndex: number = 1) {
  const excludeCBLocator =
    'div.control-container li:nth-child(' +
    filterIndex +
    ") div div.picklist-exclude-wrapper input[class='picklist-exclude']";
  cy.get(excludeCBLocator).uncheck({ force: true });
  waitForVisualsLoaded();
}

/**
 * clicks on the setting icon of the filter -> scope tab and verifies Send parameters with dataset scope is unchecked
 * and filters are not appling on visuals.
 *
 * @export
 * @param {string} params
 */
export function uncheckDataScopeofFilters(filters: number) {
  for (let index = 1; index <= filters; index++) {
    cy.get('div.control-container li:nth-child(' + index + ') span > a.configure-app').click({
      force: true,
    });
    cy.get(CSSselector.filters.settings_navtab)
      .contains('Scope')
      .click();
    interact_checkbox(CSSselector.filters.scope.current_dataset, false);
  }

  cy.get(CSSselector.filters.save_settings).click();
  cy.intercept('POST', '/arc/reports/savedashboard').as('savedashboard');
  cy.get(CSSselector.Save).click({ force: true });
  cy.wait('@savedashboard')
    .its('response.statusCode')
    .should('eq', 200);
  // cy
  //   .get(CSSselector.filters.filter_picklist)
  //   .contains(param)
  //   .scrollIntoView()
  //   .find("input[type='checkbox']")
  //   .check();

  // cy.get(CSSselector.filters.filter_selected_names).should('contain', param);

  // cy.get("div[class='gridster ready']>ul > li:nth-child(2) svg > svg > g.x.arc-viz-axis > g.tick")
  //       .should('contain', 'IN-STORE')
  //       .and('contain', 'ONLINE')
  //       .and('contain', 'QOS');
}

/**
 * checks that filters are not applied when dataset scope is unchecked
 * @export
 * @param {string[]} filterParam: filter parameters
 * @param {number} filters: total number of filters
 */
export function verifyDatasetScopeofFilter(filterParam: string[], filters: number) {
  for (let index = 1; index <= filters; index++) {
    cy.get(CSSselector.filters.filter_dropdown_specific(index)).click({ force: true });
    let exp = '';
    for (let i = 0, len = filterParam.length; i < len; i++) {
      // based on if the checkbox is checked/not checked proceede
      cy.get('div.control-container li:nth-child(' + index + ') div > ul > li')
        .contains(filterParam[i])
        .scrollIntoView()
        .find("input[type='checkbox']")
        .then(($checkbox) => {
          const tempExp = "'" + filterParam[i] + "'";
          if (i > 0) {
            exp = exp + ',+' + tempExp;
          }
          if (i === 0) {
            exp = tempExp;
          }
          if (!$checkbox.is(':checked')) {
            cy.get('div.control-container li:nth-child(' + index + ') div > ul > li')
              .contains(filterParam[i])
              .scrollIntoView()
              .find("input[type='checkbox']")
              .check();

            if (i < 2) {
              cy.get(
                'div.control-container li:nth-child(' +
                  index +
                  ') span.selected-names-wrapper > span'
              ).should('contain', filterParam[i]);
            } else if (i >= 2) {
              cy.get(
                'div.control-container li:nth-child(' +
                  index +
                  ') span.selected-count-wrapper > span.badge.selected-count'
              ).should('contain', i + 1);
            }
          }
          cy.get(
            "div[class='gridster ready']>ul > li:nth-child(2) svg > svg > g.x.arc-viz-axis > g.tick"
          )
            .should('contain', 'IN-STORE')
            .and('contain', 'ONLINE')
            .and('contain', 'QOS');
        });
    }
  }

  cy.get(CSSselector.dashboard.alert_container).should('not.be.visible');
  cy.get(CSSselector.filters.filter_dropdown).click();
}

/**
 * verify KPI visuals' value accordning to the passed filters
 *
 * @export
 * @param {string} filters: total number of KPI visuals in the dashboard
 * @param {string[]} kpiValues: values of KPI visuals.
 * @param {number} kpiIndex: index from with KPI visuals started
 */
export function verifyTelcoKPIs(kpiIndex: number, filters: number, kpiValues: string[]) {
  for (let index = kpiIndex, i = 0; index < filters + kpiIndex; index++, i++) {
    cy.get('div#app-selected-list > ul > li:nth-child(' + index + ') span.kpi-main-value').contains(
      kpiValues[i]
    );
  }
}

/**
 * clicks on the setting icon of the filter -> scope tab and updates the value
 *
 * @export
 * @param {boolean} allDashboardScope true|false
 * @param {boolean} sendParamsToDataset true|false
 */
export function verifyFilterScope(
  allDashboardScope: boolean = false,
  sendParamsToDataset: boolean = true,
  filterIndex: number = 1
) {
  cy.get(CSSselector.filters.configure_filter_any.replace('index', String(filterIndex))).click({
    force: true,
  });
  cy.get(CSSselector.filters.settings_navtab)
    .contains('Scope')
    .click();
  if (allDashboardScope) {
    cy.get(CSSselector.date_filters.scope.all_dataset).should('be.checked');
  } else if (sendParamsToDataset) {
    cy.get(CSSselector.date_filters.scope.current_dataset).should('be.checked');
  } else {
    cy.get(CSSselector.date_filters.scope.alldashboards_visible).should('not.be.checked');
    cy.get(CSSselector.date_filters.scope.current_dataset).should('be.checked');
  }
  cy.get(CSSselector.date_filters.save_settings).click();
}

/**
 * This function will verify filter content.
 * Filter content verification function is already there.
 * But it is not working as per requirements
 * So that creating another one.
 * @param locator : Css locator of filter
 * @param data : filter data to verify
 */
export function verifyFilterContent(locator: string, data: any[] = []) {
  const selectedNamesLabel = (() => {
    if (!data.length) {
      return 'All';
    } else if (data.length < 3) {
      return data.join(', ');
    } else {
      return '';
    }
  })();

  expect(
    cy
      .get(locator)
      .find('.selected-names')
      .then(($selectedElem) => $selectedElem.text().trim()),
    selectedNamesLabel
  );

  // if (data.length > 2) {
  //     expect(cy.get(locator).find('.selected-count-wrapper')).to.be.visible;
  // } else {
  //     expect(cy.get(locator).find('.selected-count-wrapper')).to.be.not.visible;
  // }

  cy.get(locator)
    .find('.picklist-list li input:checked')
    .should('have.length', data.length);
  let value;
  if (selectedNamesLabel !== 'All') {
    cy.get(locator)
      .find('.picklist-list li input:checked')
      .each(($val, i, $list) => {
        value = $val.data('value');
        expect(value).to.be.equal(data[i]);
      });
  }
}

/**
 *
 * click on applied filter value and uncheck it
 * @export
 * @param {number} [filterIndex=1] index of the filter to be interacted with
 * @param {string[]} filterValues values to uncheck in the filter
 */
export function removeFilterValues(filterIndex: number = 1, filterValues: string[]) {
  cy.get(CSSselector.filters.filter_dropdown_specific(filterIndex)).click({
    force: true,
  });
  filterValues.forEach((element) => {
    cy.get(CSSselector.filters.filter_picklist)
      .contains(element)
      .parent()
      .scrollIntoView()
      .find("input[type='checkbox']")
      .uncheck();
  });
  waitForVisualsLoaded();
  cy.get(CSSselector.filters.filter_dropdown_specific(filterIndex)).click({
    force: true,
  });
}

/**
 * clicks on the setting icon of the filter -> scope tab and updates the value
 *
 * @export
 * @param {string} filterVisibility Sheet|App
 * @param {string} scopeofParams All Datasets| Current Dataset| Explicit
 */
export function configureNewFilterScope(
  filterVisibility: string,
  scopeofParams: string,
  filterIndex = 1
) {
  cy.get(CSSselector.filters.configure_filter_specific(filterIndex)).click({ force: true });

  cy.get(CSSselector.filters.settings_navtab)
    .contains('Scope')
    .click();

  cy.get(locators.filterVisibility)
    .contains(filterVisibility)
    .click();
  cy.get(locators.paramsScope)
    .contains(scopeofParams)
    .click();
  cy.get(CSSselector.filters.save_settings).click();

  waitForVisualsLoaded();
}

export function setExplicitFilter(visualIndex: number, filterName: string): void {
  cy.get(CSSselector.visual_edit_specific(visualIndex)).click({
    force: true,
  });
  cy.get('[data-test-id="visual-data-tab-a"]').click();
  cy.get(CSSselector.filter_field).click();
  cy.get('#visual-data-panel > ul.app-params.dim-app-params > div > li .selection_field')
    .contains(filterName)
    .click({ force: true });

  cy.get('[data-test-id="viz-refresh"]').click();
  // For some weird reason closing via the data tab click is not reliable
  cy.get('[data-test-id="visual-settings-tab-a"]').click();
  cy.get('[data-test-id="visual-settings-tab-a"]').click();
}

export function removeExplicitFilter(visualIndex: number, filterName: string): void {
  cy.get(CSSselector.visual_edit_specific(visualIndex)).click({
    force: true,
  });
  cy.wait(1000);
  cy.get('[data-test-id="visual-data-tab-a"]').click();
  cy.wait(2000);
  cy.get(CSSselector.visualEdit.removeButtonOnShelfItem('filters_shelf', filterName)).click({
    force: true,
  });
  cy.get('[data-test-id="viz-refresh"]').click();
  // For some weird reason closing via the data tab click is not reliable
  cy.get('[data-test-id="visual-settings-tab-a"]').click();
  cy.get('[data-test-id="visual-settings-tab-a"]').click();
}
