import * as fa from 'lib/filter_association/locators';
import * as user_groups from 'lib/users_groups/locators';
/**
 * This function will create a new filter association in the dataset.
 * Also, add the users, groups and apply the segment in the filter association
 * @param faName : name of filter association
 * @param users: string[] : name of users to add in filter association
 * @param groups: string[] : name of groups to add in filter association
 * @param segment: string : name of a segment to apply in filter association
 */
export function createFilterAssociation(
  faName: string,
  users?: string[],
  groups?: string[],
  segment?: string
) {
  cy.get(fa.locators.fa_tab).click();
  cy.get(fa.locators.new_fa).click();
  cy.get(fa.locators.create_fa_modal).should('be.visible');
  cy.get(fa.locators.fa_name)
    .invoke('val', faName)
    .trigger('input')
    .trigger('change');
  if (segment) {
    cy.get(fa.locators.segment)
      .contains(segment)
      .parent()
      .find('input')
      .check();
  }
  if (users) {
    cy.get(fa.locators.users.user_tab).click();
    cy.wrap(users).each(($name, index, $list) => {
      cy.get(fa.locators.users.available_users)
        .contains(users[index])
        .click();
    });
    cy.get(fa.locators.users.add).click();
  }
  if (groups) {
    cy.get(user_groups.locators.group_tab).click();
    cy.wrap(groups).each(($name, index, $list) => {
      cy.get(fa.locators.groups.available_groups)
        .contains(groups[index])
        .click();
    });
    cy.get(fa.locators.groups.add).click();
  }
}

export function saveFilterAssociation() {
  cy.intercept('POST', '/arc/datasets/filterassociation').as('newFilterAssociation');
  cy.get(fa.locators.save).click();
  cy.wait('@newFilterAssociation')
    .its('response.statusCode')
    .should('be.eq', 200);
}
/**
 * This function add the users in the filter association
 * i.e. move the users from 'Available to Select' column and 'Selected' coloum in filter association modal
 * Filter Association modal must be already opened.
 * @param users : name of users to add in filter association
 */
export function addUser(users: string[]) {
  cy.get(fa.locators.users.user_tab).click();
  cy.wrap(users).each(($name, index, $list) => {
    cy.get(fa.locators.users.available_users)
      .contains(users[index])
      .click();
  });
  cy.get(fa.locators.users.add).click();
}

/**
 * This function remove the users in the filter association
 * i.e. move the users from 'Selected' column and 'Available to select' coloum in filter association modal
 * Filter Association modal must be already opened.
 * @param users : name of users to remove from filter association
 */
export function removeUser(users: string[]) {
  cy.get(fa.locators.users.user_tab).click();
  cy.wrap(users).each(($name, index, $list) => {
    cy.get(fa.locators.users.selected_users)
      .contains(users[index])
      .click();
  });
  cy.get(fa.locators.users.remove).click();
}

/**
 * This function add the groups in the filter association
 * i.e. move the groups from 'Available to Select' column and 'Selected' coloum in filter association modal
 * Filter Association modal must be already opened.
 * @param groups : name of groups to add in filter association
 */
export function addGroup(groups: string[]) {
  cy.get(user_groups.locators.group_tab).click();
  cy.wrap(groups).each(($name, index, $list) => {
    cy.get(fa.locators.groups.available_groups)
      .contains(groups[index])
      .click();
  });
  cy.get(fa.locators.groups.add).click();
}

/**
 * This function remove the groups in the filter association
 * i.e. move the groups from 'Selected' column and 'Available to select' coloum in filter association modal
 * Filter Association modal must be already opened.
 * @param groups : name of groups to remove in filter association
 */
export function removeGroup(groups: string[]) {
  cy.get(user_groups.locators.group_tab).click();
  cy.wrap(groups).each(($name, index, $list) => {
    cy.get(fa.locators.groups.selected_groups)
      .contains(groups[index])
      .click();
  });
  cy.get(fa.locators.groups.remove).click();
}
