import { locators as l } from './locators';

export const defaultHomePageName = 'Data Visualization Welcome Page';
const defaultHomePageID = -1;

/**
 * This is to verify the page contents on the set homepage settings
 * assuming the user/test is already at /arc/apps/homepage screen
 * @param admin : Set to false if the user is not an admin
 */
export function verifySetHomepageDefaultScreen(admin: boolean = true) {
  cy.get(l.setHomepageTitle).contains('Set Homepage');
  cy.get(l.dashboardName).contains(defaultHomePageName);

  if (admin) {
    cy.get(l.adminOnlyTitle).contains('Admin Only');
    cy.get(l.adminOnlyDashboardName).contains(defaultHomePageName);
  }
}

declare global {
  namespace Cypress {
    interface Chainable {
      reactSelect: (wrapperClass: string, selectValue: string) => void;
    }
  }
}

Cypress.Commands.add('reactSelect', (formType, selectValue) => {
  cy.get(
    `form[data-test-id="${formType}-setHomepageForm"] .form-select__indicators .form-select__dropdown-indicator`
  ).as('dropdown-indicator');
  cy.get('@dropdown-indicator')
    .click()
    .type(`${selectValue}{enter}`);
});

/**
 * assumes you are at the /arc/apps/homepage screen
 * @param dashID enter the appID to set to homepage
 */
export function setHomepage(dashID: number, dashboardName: string) {
  const selectValue = `${dashboardName} (id: ${dashID})`;

  cy.intercept('POST', '/arc/reports/userdata/homeapp').as('setHomepage');
  cy.reactSelect('user', selectValue);

  cy.get(l.dashboardSave).click();

  cy.wait('@setHomepage');

  // verify
  cy.get(l.dashboardName).contains(dashboardName);
  if (dashID === defaultHomePageID) {
    cy.get(l.dashboardName)
      .contains(`(id: `)
      .should('not.exist');
  } else {
    cy.get(l.dashboardName).contains(`(id: ${dashID})`);
  }
}

export function resetHomepage(dashID = defaultHomePageID, dashboardName = defaultHomePageName) {
  cy.intercept('POST', '/arc/reports/userdata/homeapp').as('resetHomepage');
  cy.get(l.reset_admin_defined_homepage).click();
  cy.wait('@resetHomepage');

  cy.get(l.dashboardName).contains(dashboardName);
  if (dashID === defaultHomePageID) {
    cy.get(l.dashboardName)
      .contains(`(id: `)
      .should('not.exist');
  } else {
    cy.get(l.dashboardName).contains(`(id: ${dashID})`);
  }
}

export function toggleUserOverride(disable: boolean) {
  cy.intercept('POST', '/arc/reports/homepage/disableoverride').as('disableOverride');
  cy.get(l.adminOnlyDisableOverrideCheckbox).click();
  cy.get(l.adminOnlyDisableOverrideConfirmButton).click();

  cy.wait('@disableOverride');

  // Verify
  cy.get(l.dashboardSave).should(`${disable ? '' : 'not.'}be.disabled`);
  cy.get(l.userSelectControl).should(
    `${disable ? '' : 'not.'}have.class`,
    'form-select__control--is-disabled'
  );
}
