import * as apps from 'lib/apps';
import * as connections from 'lib/connections';
import * as functions from 'lib/functions';
import * as imp_exp from 'lib/import_export';
import { uglify } from 'lib/util/export-transform';
import { CSSselector } from 'locators/common/CSSselector';

/**
 * Import dashboard of name: Clone of VELIKO - Fiscal Year - Customer Service Quality / Cost Dashboard
 * @param dataconnection : name of connection
 * @param dashboardFile: name of dashboard (it must be an exact json filename)
 */
export function import_artifacts(dataconnection: string, dashboardFile: string) {
  connections.selectConnectionByName(dataconnection);
  cy.intercept('GET', '/arc/workspace/workspace').as('importModal');
  // cy.intercept('GET', '/arc/migration/migrate').as('importDash');
  cy.intercept('PUT', '/arc/migration/migrate?dry_run=false').as('importDash');
  cy.get(imp_exp.locators.import_toggle).click();
  cy.get(imp_exp.locators.import_artifacts).click();
  cy.wait('@importModal')
    .its('response.statusCode')
    .should('eq', 200);

  const jsonFolderPath = './cypress/test_resources/';
  const dt = new DataTransfer();
  const files: any[] = [];

  cy.readFile(jsonFolderPath + dashboardFile, 'utf-8').then((jsonObj) => {
    uglify(jsonObj);
    const jsonData: string = JSON.stringify(jsonObj);
    // console.log('this is the file:' + jsonData);
    files.push(
      new File([jsonData], dashboardFile, {
        type: 'text/plain',
        lastModified: new Date().getTime(),
      })
    );
    console.log('this is the files:' + files[0]);

    dt.items.add(files[0] as File);
  });

  cy.get(imp_exp.locators.import_input_file).then(($inputFile) => {
    ($inputFile[0] as HTMLInputElement).files = dt.files;
  });

  cy.get(imp_exp.locators.import_button).click();

  cy.get(imp_exp.locators.import_confirmation).click();
  cy.wait('@importDash')
    .its('response.statusCode')
    .should('eq', 200);
  cy.contains('Success! The import was successful.');
}

export function getSelectedAppIdsAndTypes(): any {
  return new Promise((resolve) => {
    cy.window()
      .its('arc.screen.apps' as any)
      .invoke('selectedAppIdsAndTypes', '')
      .then(($val) => {
        console.log('test selected AppId : ' + $val.dashboards[0]);
        resolve($val.dashboards[0]);
      });
  });
}

export function export_app(appName: string, exportAppName: string = 'InfoSecGlobal_export') {
  cy.intercept('GET', 'arc/migration/export/*').as('exportModal');

  apps.selectAppFromVisualsPage(appName);

  cy.get(imp_exp.locators.ellipsis)
    .click()
    .find('.dropdown-menu')
    .should(($menu) => {
      return expect($menu).to.be.visible;
    })
    .find('li#viz-dash-export')
    .click();

  cy.wait('@exportModal')
    .its('response.statusCode')
    .should('eq', 200);

  // confirm
  // cy.get(imp_exp.locators.input_export).clear();
  // cy.get(imp_exp.locators.input_export).type(exportAppName);

  cy.get(imp_exp.locators.export_popup).should('have.css', 'display', 'block');

  cy.get(imp_exp.locators.export_file_name)
    .clear()
    .type(exportAppName);
  let val: number;
  cy.get("span[title='" + appName + "']")
    .parents('tr')
    .then(($id) => {
      val = $id.data('app');
      console.log('value is ' + val);
      cy.get(imp_exp.locators.export_button)
        .should('have.class', 'btn-primary')
        .should('have.text', 'Export');

      cy.request(
        `${
          Cypress.config().baseUrl
        }/arc/migration/export/?dry_run=false&dashboards=[]&appgroups=[]&visuals=[${val}]&filename=${exportAppName}`
      ).then((response) => {
        cy.writeFile(`${exportAppName}.json`, response.body);
      });
      cy.get(imp_exp.locators.export_close_button).click();
    });

  // (async () => {
  //   appId = await getSelectedAppIdsAndTypes();
  //   console.log('sample app id', appId);

  // })();

  //  console.log("The app id is" + appId);
  // cy.on('window:before:unload', (e) => {
  //   // no return value on the event
  //   return false;
  // });
  // Request URL: http://127.0.0.1:32774/arc/migration/export/?appgroups=[6]&filename=temp&dry_run=true
  // Request URL: http://127.0.0.1:32774/arc/migration/export/?dry_run=false&appgroups=[6]&filename=InfoSecGlobal

  // return new Promise((resolve) => {
  //   cy.get('body').debug().then(($body) => {

  //       resolve(Cypress.$(imp_exp.locators.export_button)
  //       .trigger('click'));
  //     });

  // expect($body).to.be.visible;
  // });

  // let csrftoken = '';
  // cy.getCookie('arccsrftoken', { log: false }).then((cookie) => {
  //   if (cookie) {
  //     csrftoken = cookie.value;
  //   }
  // });

  // cy.wait('@exportModal');

  //   cy.request('/arc/migration/export/?dry_run=false&appgroups=[6]&filename=InfoSecGlobal').
  //   then((response) => {
  //     expect(response.status).to.eq(200);
  //
  //     cy.exec(`wget --header ${csrftoken} ${Cypress.config('baseUrl')}/arc/migration/export/?dry_run=false&appgroups=[6]&filename=InfoSecGlobal`)

  //     console.log(response.body);
  //   });

  //   cy.get('body > div.navbar.navbar-default.navbar-fixed-top > div > div.navbar-collapse.collapse > ul.nav.navbar-nav.navbar-left > li.topnav-apps.topnav-menu.active > a').then(async () => {
  //     await clickDownload();
  //     await setTimeout(() => {
  // return;
  //     }, 4000);
  //     return;
  //   });

  // cy.request('http://127.0.0.1:32774/arc/apps/apps');
  // cy.visit('/');

  // cy.on('window:before:unload', (e) => {
  // no return value on the event
  // pect(e.returnValue).to.be.undefined;
  //       cy.window().then(
  // (win: Window) =>
  //  (win.onbeforeunload = () => {
  //    return;
  //  })
  //
}

export function clickDownload(): any {
  return new Promise((resolve) => {
    console.log('here');
    resolve(Cypress.$(imp_exp.locators.export_button).click());
  });
}

/**
 * This will export particular linked visual from Visuals > visuals.
 * This will select particular visual, click on ellispsis and export the visual.
 * @export
 * @param visualName: name of visual
 */
export function export_linked_visual(visualName: string): any {
  cy.intercept('GET', 'arc/migration/export/*').as('export');
  functions.navigate_to_visuals();
  // click all
  cy.get(CSSselector.Visuals_all).click({ force: true });
  cy.get(imp_exp.locators.visuals_list).click({ force: true });
  cy.get(imp_exp.locators.dashboard_grid)
    .scrollIntoView()
    .click({
      force: true,
    });
  cy.get(imp_exp.locators.search).type('"' + visualName + '"');
  cy.wait(2000);
  cy.get(imp_exp.locators.grid_item).each(($el) => {
    if ($el.text().trim() === visualName) {
      cy.wrap($el)
        .parent()
        .within(() => {
          cy.get('span.btn-sel').should('have.class', 'btn-off');
          cy.get('span.btn-sel')
            .invoke('trigger', 'onpointerenter')
            .click({
              force: true,
            }); // onpointerenter event is not set
        });
      cy.get(imp_exp.locators.ellipsis)
        .click()
        .find('.dropdown-menu')
        .should(($menu) => {
          return expect($menu).to.be.visible;
        })
        .find('li#viz-dash-export')
        .click();

      cy.wait('@export')
        .its('response.statusCode')
        .should('eq', 200);

      cy.get(imp_exp.locators.export_popup).should('have.css', 'display', 'block');

      cy.get(imp_exp.locators.export_file_name)
        .clear()
        .type(visualName + '_export');
      let val: number;
      cy.get("span[title='" + visualName + "']")
        .parents('tr')
        .then(($id) => {
          val = $id.data('app');
          console.log('value is ' + val);
          cy.get(imp_exp.locators.export_button)
            .should('have.class', 'btn-primary')
            .should('have.text', 'Export');

          cy.request(
            `${
              Cypress.config().baseUrl
            }/arc/migration/export/?dry_run=false&dashboards=[]&appgroups=[]&visuals=[${val}]&filename=${visualName}`
          ).then((response) => {
            cy.writeFile(`${visualName}.json`, response.body);
          });
          cy.get(imp_exp.locators.export_close_button).click();
        });
    }
  });
}

/**
 * This will export particular linked visual from Visuals > visuals.
 * This will select particular visual, click on ellispsis and export the visual.
 * @export
 * @param dashboardName: name of dashboard
 */
export function export_dashboard(dashboardName: string): any {
  cy.intercept('GET', 'arc/migration/export/*').as('export');
  functions.navigate_to_visuals();
  // click all
  cy.get(CSSselector.Visuals_all).click({ force: true });
  cy.get(imp_exp.locators.dashboard_grid)
    .scrollIntoView()
    .click({
      force: true,
    });
  cy.get(imp_exp.locators.search).type('"' + dashboardName + '"');
  cy.wait(2000);
  cy.get(imp_exp.locators.grid_item).each(($el) => {
    if ($el.text().trim() === dashboardName) {
      cy.wrap($el)
        .parent()
        .within(() => {
          cy.get('span.btn-sel').should('have.class', 'btn-off');
          cy.get('span.btn-sel')
            .invoke('trigger', 'onpointerenter')
            .click({
              force: true,
            }); // onpointerenter event is not set
        });
      cy.get(imp_exp.locators.ellipsis)
        .click()
        .find('.dropdown-menu')
        .should(($menu) => {
          return expect($menu).to.be.visible;
        })
        .find('li#viz-dash-export')
        .click();

      cy.wait('@export')
        .its('response.statusCode')
        .should('eq', 200);

      cy.get(imp_exp.locators.export_popup).should('have.css', 'display', 'block');

      cy.get(imp_exp.locators.export_file_name)
        .clear()
        .type(dashboardName + '_export');
      let val: number;
      cy.get("span[title='" + dashboardName + "']")
        .parents('tr')
        .then(($id) => {
          val = $id.data('app');
          console.log('value is ' + val);
          cy.get(imp_exp.locators.export_button)
            .should('have.class', 'btn-primary')
            .should('have.text', 'Export');

          cy.request(
            `${
              Cypress.config().baseUrl
            }/arc/migration/export/?dry_run=false&dashboards=[${val}]&appgroups=[]&visuals=[]&filename=${dashboardName}`
          ).then((response) => {
            expect(response.body).to.have.length.above(0);
            expect(response).to.have.property('headers');
            cy.writeFile(`${dashboardName}.json`, response.body);
          });
          cy.get(imp_exp.locators.export_close_button).click();
        });
    }
  });
}
