/**
 * Login in the background with vizapps_admin/vizapps_admin without logging the cypress commands
 * and whitelist the arcsessionid and arccsrftoken cookies
 *
 * @export
 * @param {string} [baseUrl]
 */
export function silent(username: string = 'vizapps_admin', password: string = 'vizapps_admin') {
  return cy.session(
    username,
    () => {
      const loginUrl = '/arc/apps/login';

      // Any request will do, needed to set up arccsrftoken in the cookies
      cy.request({ url: loginUrl });

      // Get CSRF token
      cy.getCookie('arccsrftoken')
        .then((cookie) => {
          if (cookie === null) {
            throw new Error('arccsrftoken cookie not found');
          }

          return cookie.value;
        })
        .then((csrftoken) =>
          // Use CSRF token to send login request
          cy.request({
            body: {
              csrfmiddlewaretoken: csrftoken,
              password: password,
              username: username,
            },
            headers: {
              Origin: Cypress.config().baseUrl,
            },
            followRedirect: false,
            form: true,
            log: false,
            method: 'POST',
            url: loginUrl,
          })
        );
    },
    {
      validate: () => {
        cy.getCookie('arcsessionid').should('exist');
      },
    }
  );
}

export function logout() {
  cy.session('logout', () => {
    cy.log('logged out');
  });
}

/**
 * Login from the UI to the page at baseUrl
 *
 * @export
 * @param {string} baseUrl
 */
export function defaultShow(baseUrl: string) {
  let url = '/arc/';
  if (baseUrl) {
    url = baseUrl + url;
  }

  cy.visit(url);

  cy.get('#username').type('vizapps_admin');
  cy.get('#password').type('vizapps_admin');
  cy.get('.login-btn').click();

  cy.url().should('include', '/arc/apps/');
}
