import { CSSselector } from 'locators/common/CSSselector';
import * as functions from '../functions';

/**
 * Selects and opens the exact match dataset under the connection specified
 * @param connectionName name of connection under which to search the dataset
 * @param dataset name of dataset to select ( but not open dataset detail)
 */
export function selectDataset(connectionName: string, dataset: string) {
  let found: boolean = false;
  cy.get(CSSselector.data.connection_list).each(($el) => {
    // index starts from 1 so we account for it while clicking
    if ($el.text() === connectionName) {
      // yup found it
      cy.get(CSSselector.data.connection_list + ' a[title=' + connectionName + ']').click();
      cy.get(CSSselector.data.dataset_table_row + ' .dataset-title').each(($datasetel, index) => {
        cy.log($datasetel.text());
        if ($datasetel.text() === dataset && !found) {
          cy.get(
            CSSselector.data.dataset_table_row + ' .dataset-title' + ':eq(' + index + ')'
          ).click();
          found = true;
          // $datasetel.click();
          // return false;
        }
      });
    }
  });
}

export function getDatasetId(connectionName: string, dataset: string) {
  functions.navigate_to_data();
  selectDataset(connectionName, dataset);

  // wait until the dataset id field is populated:
  cy.get('#dataset-id')
    .invoke('text')
    .should('match', /[^\s]/);

  return cy.get('#dataset-id').then(($id) => {
    const datasetID = Number($id.text().trim());
    return datasetID;
  });
}

export function addData() {
  functions.navigate_to_data();

  cy.get(CSSselector.data.add_data_button).click();

  cy.get(CSSselector.data.choose_csv_button).trigger('change', new File([''], 'untitled.csv'));
  // .click()//.type('thisisthefilename')
  //  .then(($body) => {
  //   $body.append('/usr/var/thisisthefilename')
  //   //$body.click()
  //   //cy.get(objects.data.get_data_button).click()
  // })
  cy.get(CSSselector.data.get_data_button).click();
}

export function addPermissionsToDataset(roleName: string, permissionInfo: any) {
  cy.get(`${CSSselector.dataset_detail.role_permissions_table} > tr`).each(($el, index, $list) => {
    // index starts from 1 so we account for it while clicking
    console.log(`element text is ${$el.text()} and the ${roleName}`);

    if ($el.text().includes(roleName)) {
      // yup found it
      // lets do some more fun stuff
      // on each cell of the row based on permissions array
      for (let i = 0; i < permissionInfo.length; i++) {
        if (permissionInfo[i]) {
          // cy.wrap($el).find('input').filter(obj.roles.permissions[i]).click()
          // cannot use wrap as .click() changes the $el and it becomes detached
          // for just assertions cy.wrap() should be used.
          cy.get(
            `${CSSselector.dataset_detail.role_permissions_table} > tr:nth-child(${index + 1})`
          )
            .find('input')
            .filter(CSSselector.roles.permissions[i])
            .click();

          if (i > 0) {
            break;
          }
        }
      }
      console.log(`component index found ${index}`);
    } else {
      console.log(`component index not found ${index} `);
    }
  });
  // dont forget to save
  cy.get(CSSselector.dataset_detail.save_permissions_button).click();
}
