import { CSSselector } from 'locators/common/CSSselector';
import * as functions from '../functions';
/**
 * Add role with description and assign users to it.
 * @param roleInfo : Information of Role which contains
 * rolename: Name of role
 * roleDescription: Description of role
 * user_list: lsit of users to add into role
 */
export function addRole(roleInfo: any) {
  cy.intercept('POST', '/arc/roles/roles').as('roleSaved');
  cy.intercept('DELETE', '/arc/roles/roles/*').as('roleDeleted');
  functions.goToSiteAdminSettingsMenu('manage_roles_site_admin');
  cy.get('table[data-test-id="manageRolesPage"] tbody tr', { timeout: 30000 }).should(
    'have.length.greaterThan',
    1
  );
  // before you create a role check for existing role DELETE any role with similar name
  cy.get(`${CSSselector.roles.role_row}`).each(($row) => {
    console.log(`row data is ${$row.text()}expected is ${roleInfo.roleName}`);
    if ($row.text().includes(roleInfo.roleName)) {
      cy.wrap($row)
        .find(`td:last-child ${CSSselector.roles.delete_role_bin}`)
        .click();
      cy.wait(500);
      cy.get('input[data-test-id=delete-confirmation-text]').type('DELETE');
      cy.get('button[data-test-id=delete-confirmation-yes]')
        .should('be.not.disabled')
        .click();
      cy.wait('@roleDeleted');
    }
  });
  cy.visit('/arc/apps/roles');
  cy.get('table[data-test-id="manageRolesPage"] tbody tr', { timeout: 30000 }).should(
    'have.length.greaterThan',
    1
  );
  // OK now continue creating role
  cy.get(CSSselector.roles.new_role_button).click();
  cy.get('div[data-test-id="privilege-view"]', { timeout: 30000 }).should('be.visible');
  cy.get(CSSselector.roles.role_details_page.name_field).type(roleInfo.roleName);
  cy.get(CSSselector.roles.role_details_page.description).type(roleInfo.roleDescription);
  // Add a member (user) to the role
  cy.get(CSSselector.roles.role_details_page.tabs.members_tab).click();
  cy.get(CSSselector.roles.role_details_page.members.edit_users).click();
  cy.get(CSSselector.roles.role_details_page.members.available_to_select)
    .contains(roleInfo.user_list[0])
    .click();
  cy.get(CSSselector.roles.role_details_page.members.add_selection).click();
  cy.get(CSSselector.roles.role_details_page.members.apply_button).click();
  cy.get(CSSselector.roles.role_details_page.tabs.privileges_tab).click();
  // wait for reflect the changes
  cy.wait(2000);
  cy.get(CSSselector.roles.role_details_page.save_role_button).click();
  cy.wait('@roleSaved');
}

export function addPrivilege(privilege: {
  side_nav_item: string;
  connection?: string[];
  datasets?: string[];
  permissions?: string[];
}) {
  // find the tab and click on it
  cy.get(`div[data-test-id="${privilege.side_nav_item}_tab"]`).click();

  // find the Add Dataset/Connection/Role button and click on it so the modal opens up
  cy.get(CSSselector.roles.role_details_page.privileges.add_privilege_button).click();

  // select the right dropdowns
  if (privilege.side_nav_item === 'datasets') {
    // connection dropdown within datasets modal
    cy.get(
      CSSselector.roles.role_details_page.privileges.add_privilege_modal.modal_select_and_list
        .dataset_connection_select
    ).click();
    privilege.connection?.forEach((connection) => {
      cy.get(
        CSSselector.roles.role_details_page.privileges.add_privilege_modal.modal_select_and_list
          .dataset_connection_select_list
      )
        .contains(connection)
        .click();
    });

    // datasets dropdown within datasets modal
    privilege.datasets?.forEach((dataset) => {
      cy.get(
        CSSselector.roles.role_details_page.privileges.add_privilege_modal.modal_select_and_list
          .dataset_dataset_select
      ).click();
      cy.get(
        CSSselector.roles.role_details_page.privileges.add_privilege_modal.modal_select_and_list
          .dataset_dataset_select_list
      )
        .contains(dataset)
        .click();
    });

    cy.get(
      CSSselector.roles.role_details_page.privileges.add_privilege_modal.modal_save_button
        .datasets_save
    ).click();

    // find the selectAll button and unselect all
    // so we click each permission checkboxes
    // followed what the permission array contains
    cy.get(
      CSSselector.roles.role_details_page.privileges.add_privilege_modal.permissions
        .datasets_permissions.selectAll
    ).click();

    cy.wait(200);

    // go through the permissions and check the checkboxes
    privilege.permissions?.forEach((permission) => {
      cy.get(
        CSSselector.roles.role_details_page.privileges.add_privilege_modal.permissions
          .datasets_permissions[
          permission as keyof typeof CSSselector.roles.role_details_page.privileges.add_privilege_modal.permissions.datasets_permissions
        ]
      ).click();
    });
  } else if (privilege.side_nav_item === 'connections') {
    privilege.connection?.forEach((connection) => {
      cy.get(
        CSSselector.roles.role_details_page.privileges.add_privilege_modal.modal_select_and_list
          .connection_select
      ).click();
      cy.get(
        CSSselector.roles.role_details_page.privileges.add_privilege_modal.modal_select_and_list
          .connection_select_list
      )
        .contains(connection)
        .click();
    });

    cy.get(
      CSSselector.roles.role_details_page.privileges.add_privilege_modal.modal_save_button
        .connections_save
    ).click();
  }

  cy.wait(200);
}

/**
 * This function remove the users from members tab of created role.
 * i.e. move the users from 'Selected' column and 'Available to select' coloum in role assignment modal
 * Role view must be opened.
 * @param users : list of users to remove from role
 */
export function removeUser(users: string[]) {
  cy.intercept('POST', '/arc/roles/roles').as('rolesLoaded');
  cy.get(CSSselector.roles.role_details_page.tabs.members_tab).click();
  cy.wait(1000);
  cy.get(CSSselector.roles.role_details_page.members.edit_users).click({ force: true });
  cy.fixture('users.json').then((user) => {
    // tslint:disable-next-line:prefer-for-of
    for (let index = 0; index < users.length; index++) {
      cy.get(CSSselector.roles.role_details_page.members.selected_to_select)
        .contains(users[index])
        // .siblings('input[type="checkbox"]')
        .click({ force: true });
      // cy.wait(200);
    }
    cy.get(CSSselector.roles.role_details_page.members.remove_selection).click();
    cy.get(CSSselector.roles.role_details_page.members.apply_button).click();
    cy.get(CSSselector.roles.role_details_page.save_role_button).click();
    cy.wait('@rolesLoaded')
      .its('response.statusCode')
      .should('eq', 200);
  });
}

/**
 * This function add the users from members tab of created role.
 * i.e. move the users from 'Available to select' column and 'Selected' coloum in role assignment modal
 * Role view must be opened.
 * @param users : list of users to add into the role
 */
export function addUser(users: string[]) {
  cy.intercept('POST', '/arc/roles/roles').as('rolesLoaded');
  cy.get(CSSselector.roles.role_details_page.tabs.members_tab).click();
  cy.get(CSSselector.roles.role_details_page.members.edit_users).click();
  cy.fixture('users.json').then((user) => {
    // tslint:disable-next-line:prefer-for-of
    for (let index = 0; index < users.length; index++) {
      cy.get(CSSselector.roles.role_details_page.members.available_to_select)
        .contains(users[index])
        .click();
    }
    cy.get(CSSselector.roles.role_details_page.members.add_selection).click();
    cy.get(CSSselector.roles.role_details_page.members.apply_button).click();
    cy.get(CSSselector.roles.role_details_page.save_role_button).click();
    cy.wait('@rolesLoaded')
      .its('response.statusCode')
      .should('eq', 200);
  });
}

/**
 * This function delete the role.
 * i.e. move the users from 'Available to select' column and 'Selected' coloum in role assignment modal
 * Role view must be opened.
 * @param users : list of users to add into the role
 */
export function deleteRole(roleName: string) {
  cy.intercept('DELETE', '/arc/roles/roles/*').as('roleDeleted');
  cy.get(`${CSSselector.roles.role_row}`).each(($row) => {
    if ($row.text().includes(roleName)) {
      cy.wrap($row);
      cy.wrap($row)
        .find('td:last-child')
        .find('button[data-test-id="deleteRole"]')
        .click();
      cy.get('input[data-test-id="delete-confirmation-text"]').type('DELETE');
      cy.get('button[data-test-id="delete-confirmation-yes"]').click();
    }
  });
  cy.wait('@roleDeleted')
    .its('response.statusCode')
    .should('eq', 200);
}
