import * as dsManagement from 'lib/datasetManagement';
import { locators as l } from './locators';
import { WordExpansion } from './search_functions';

/**
 * This method expects you are already on the dashboard detail screen
 * and enables search on the current dataset and saves
 *
 * @export
 */
export function enableSearchonThisDataset(enable: boolean = true) {
  if (enable) {
    cy.get(dsManagement.locators.enableSearchCB)
      .scrollIntoView()
      .should('be.visible');
    cy.get(dsManagement.locators.enableSearchCB)
      .scrollIntoView()
      .check();
    cy.get(dsManagement.locators.enableSearchCB).should('be.checked');
  } else {
    cy.get(dsManagement.locators.enableSearchCB)
      .scrollIntoView()
      .should('be.visible');
    cy.get(dsManagement.locators.enableSearchCB)
      .scrollIntoView()
      .uncheck();
    cy.get(dsManagement.locators.enableSearchCB).should('not.be.checked');
  }
  saveSearchModel();
}

export function saveSearchModel() {
  cy.intercept('PUT', '/arc/datasets/dataset/*').as('updateDataset');
  cy.intercept('GET', '/arc/datasets/dataset/*?detail=true').as('getDatasetdetail');
  cy.intercept('GET', '/arc/apps/users_api?summary=true').as('getUserSummary');
  cy.intercept('GET', '/arc/apps/groups_api').as('getGroupsApi');
  cy.intercept('GET', '/arc/datasets/filterassociation?dataset_id=*').as('filterAssociations');
  cy.intercept('GET', '/arc/roles/roles').as('getRoles');
  cy.get(dsManagement.locators.saveSearchModelBtn)
    .scrollIntoView()
    .click({ force: true });

  // if there more than 10 searcable columns, a confirmation modal will appear
  cy.get<HTMLOptionElement>('#search-table select.use-for-search:not([disabled]) option').then(
    ($options) => {
      let numberOfSearchableColumns = 0;
      Array.from($options).forEach((option) => {
        if (option.selected && option.innerText !== 'None') {
          numberOfSearchableColumns += 1;
        }
      });

      if (numberOfSearchableColumns > 10) {
        // wait for modal fade css transition to finish
        cy.wait(300);
        cy.get('#save-searchable-yes')
          .should('be.visible')
          .click();
        cy.get('#save-searchable-yes').should('not.be.visible');
      }
    }
  );

  cy.wait('@updateDataset');
  cy.wait('@getDatasetdetail');
  cy.wait('@getUserSummary');
  cy.wait('@getGroupsApi');
  cy.wait('@filterAssociations');
  cy.wait('@getRoles');

  cy.get(dsManagement.locators.saveSearchModelBtn).should('have.class', 'disabled');
}

/**
 * It enteres the search group name and saves it;
 * make sure to enableSearchonThisDataset() to have the search group show up
 *
 * @export
 * @param {string} groupName
 */
export function enterSearchGroup(groupName: string) {
  cy.get(dsManagement.locators.searchGroupTB).type(groupName + '{enter}');
  saveSearchModel();
}

export function setEveryColumnPriority(priority: 'None' | 'Low' | 'Medium' | 'High') {
  cy.get('#search-table select.use-for-search:not([disabled])').each(($el) =>
    cy.wrap($el).select(priority)
  );
  saveSearchModel();
}

/**
 * clears before typing
 *
 * @export
 * @param {string} columnName
 * @param {string} searchTerm
 */
export function enterMatchingSearchTerm(columnName: string, searchTerm: string) {
  cy.get(dsManagement.locators.columnsTableRow)
    .contains('[' + columnName + ']')
    .find('td:eq(2)')
    .clear()
    .type(searchTerm);
  saveSearchModel();
}

/**
 * Doesnt clear() the element before typing
 *
 * @export
 * @param {string} columnName
 * @param {string} searchTerm
 */
export function appendMatchingSearchTerms(columnName: string, searchTerm: string) {
  cy.get(dsManagement.locators.columnsTableRow)
    .contains('[' + columnName + ']')
    .find('td:eq(2)')
    .type(' ' + searchTerm); // since space is the delimiter
  saveSearchModel();
}

/**
 *
 *
 * @export
 * @param {string} question
 */
export function addSuggestedQuestions(questions: string[]) {
  questions.forEach((question) => {
    cy.get(l.addNewRowSuggestedQuestions)
      .scrollIntoView()
      .find('a')
      .click();
    cy.get(l.addNewRowSuggestedQuestions)
      .parent()
      .prev()
      .find('td .editablecell')
      .type(question);
  });
  saveSearchModel();
}

/**
 *
 *
 * @export
 * @param {string} expansion.inputWord
 * @param {string} expansion.outputWord
 */
export function addWordExpansions(expansions: WordExpansion[]) {
  expansions.forEach((expansion) => {
    cy.get(l.addNewRowWordExpansions)
      .scrollIntoView()
      .find('a')
      .click();
    cy.get(l.addNewRowWordExpansions)
      .parent()
      .prev()
      .find('td .editablecell:eq(0)')
      .type(expansion.inputWord);
    cy.get(l.addNewRowWordExpansions)
      .parent()
      .prev()
      .find('td:eq(1) .editablecell')
      .type(expansion.outputWord);
  });
  saveSearchModel();
}
