import { goToSiteAdminSettingsMenu, modalClick, searchForUser } from 'lib/functions';
import { locators } from 'lib/users_groups/locators';
import { CSSselector } from 'locators/common/CSSselector';

/**
 * This function create new users with specified role. It will assign one role per user as of now.
 * @param username: Username string
 * @param password: Password string
 * @param roles: list of roles for users
 */

export function createNewUser(username: string, password: string, roles?: string[]) {
  goToSiteAdminSettingsMenu('manage_user_and_group_site_admin');

  const usernameColumn = `${CSSselector.users.users_table_row} td:nth-child(2)`;
  cy.get(usernameColumn).then(($column) => {
    const userExists = $column.toArray().some((c) => Cypress.$(c).text() === username);
    if (false === userExists) {
      cy.get(CSSselector.users.new_user_button).click();
      cy.get(locators.users.username)
        .type(username)
        .trigger('change');
      cy.get(locators.users.password)
        .type(password, { force: true })
        .trigger('change');

      // role assignment
      if (roles?.length) {
        cy.get(locators.roles.role)
          .should('be.visible')
          .click();

        cy.wrap(roles).each(($el, index, $list) => {
          cy.get(locators.roles.available_roles)
            .contains(roles[index])
            .click();
        });
        cy.get(locators.roles.add)
          .click()
          .trigger('change');
      }

      cy.intercept('POST', `/arc/apps/users_api/${username}`).as(`create ${username}`);

      cy.get(CSSselector.users.save_user_button).click();

      cy.wait(`@create ${username}`).then((interception: any) => {
        expect(interception.response.body[0]).to.have.property('username');
        expect(interception.response.body[0].username).to.eq(username);
      });
    }
  });
}

/**
 * This function create new users with specified role. It will assign one role per user as of now.
 * @param groupname: Group name string
 * @param users: list of users to be placed in a group
 */
export function createNewGroup(groupname: string, users?: string[], roles?: string[]) {
  goToSiteAdminSettingsMenu('manage_user_and_group_site_admin');
  cy.get(locators.group_tab).click();
  cy.wait(1000);
  cy.get(locators.groups.table_rows).then(($row) => {
    cy.get(CSSselector.users.new_group_button).click();
    cy.get(locators.groups.group_name).type(groupname);
    // user assignment
    if (users) {
      cy.wrap(users).each(($user, index, list) => {
        cy.get(locators.groups.users)
          .contains(users[index])
          .click();
      });
      cy.get(locators.groups.add_user).click();
      // role assignment
      if (roles) {
        cy.get(locators.groups.roles_tab).click();
        cy.wrap(roles).each(($el, index, $list) => {
          cy.get(locators.groups.available_roles)
            .contains(roles[index])
            .click();
        });
        cy.get(locators.groups.add).click();
      }
      cy.get(locators.groups.save).click();
      cy.wait(1000);
    }
  });
}

/**
 * This function make exisitng user inactive.
 * @param username: username of user
 */
export function inactiveUser(username: string) {
  searchForUser(username);
  cy.get(locators.users.name)
    .contains(username)
    .parents('tr')
    .find('.fa-pencil')
    .click();
  cy.get('#edit-user-modal').should('be.visible');
  cy.get(locators.users.inactive).check();
  modalClick(locators.users.save);
}

/**
 * This function creates a group from "New User" modal and reload the page.
 * NOTE: group name should not exist.
 * @param groupname: name of group
 */
export function createGrpFromUserModal(groupname: string) {
  cy.intercept('GET', '/arc/roles/roles').as('rolesLoaded');
  cy.get(locators.users_tab).click();
  goToSiteAdminSettingsMenu('manage_user_and_group_site_admin');
  cy.get(CSSselector.users.new_user_button).click();
  cy.get(locators.groups.easy_grp_add)
    .find('input')
    .type(groupname);
  cy.get(locators.groups.easy_grp_add)
    .find('button')
    .click();
  modalClick(CSSselector.users.cancel_user_button);
  cy.reload();
  cy.wait('@rolesLoaded')
    .its('response.statusCode')
    .should('eq', 200);
}
