const jsonFields = [
  /:datasets:\d+:fields:dataset_info$/,
  /:datasets:\d+:fields:dataset_settings$/,
  /:dashboards:\d+:fields:report_data$/,
  /:visuals:\d+:fields:report_data$/,
  /:report_data:report_data:dsreq$/,
  /:report_data:report_data:user_settings:color_palette$/,
];

function isJSONField(path: string) {
  const retVal = jsonFields.some((r) => r.test(path));
  if (retVal) {
    // console.log('  >> IS JSON');
  }
  return retVal;
}

export function prettify(input: any, path = '') {
  if (input == null) {
    return;
  }
  for (const [key, value] of Object.entries<any>(input)) {
    const valuePath = path + ':' + key;
    if (key === 'reportimage') {
      input[key] = [];
    } else if (typeof value === 'object') {
      prettify(value, valuePath);
    } else if (typeof value === 'string') {
      if (isJSONField(valuePath)) {
        const newValue = JSON.parse(value);
        input[key] = newValue;
        prettify(newValue, valuePath);
      }
    }
  }
}

export function uglify(input: any, path = '') {
  if (input == null) {
    return;
  }

  for (const [key, value] of Object.entries<any>(input)) {
    const valuePath = path + ':' + key;

    const toUnique = ['uuid', 'imported_uuid', 'report_name', 'dataset_name'];
    if (toUnique.includes(key)) {
      input[key] = `${value}_${get_time()}`;
    }

    if (key === 'dashboards' && value.length) {
      value.forEach((dashboard: any) => {
        dashboard.fields.report_data.report_title = `${
          dashboard.fields.report_data.report_title
        }_${get_time()}`;
      });
    }

    if (typeof value === 'object') {
      uglify(value, valuePath);
    }

    if (isJSONField(valuePath) && !(typeof value === 'string')) {
      input[key] = JSON.stringify(value);
    }
  }
}

export function uglifyWithoutRandomizing(input: any, path = '') {
  if (input == null) {
    return;
  }

  for (const [key, value] of Object.entries<any>(input)) {
    const valuePath = path + ':' + key;

    if (typeof value === 'object') {
      uglifyWithoutRandomizing(value, valuePath);
    }

    if (isJSONField(valuePath) && !(typeof value === 'string')) {
      input[key] = JSON.stringify(value);
    }
  }
}

function get_time() {
  const a = new Date();
  return a.getTime();
}
