import * as dashboards from 'lib/dashboard';
import { modalClick } from 'lib/functions';
import * as visual from 'lib/visual';
import * as visual_functions from 'lib/visual/visual_functions';
import * as visual_common_test from './visual_common_tests';

// Settings color tab functions

export function reverseColorPalette() {
  dashboards.chooseVisualTab('colors');
  cy.get(visual.locators.style.colors.reverseColorPaletteCheckbox).click();
}

export function chooseColorPalette() {
  dashboards.chooseVisualTab('colors');
  cy.get(dashboards.locators.color_palette_10th).click();
}

export function setColorOpacity() {
  dashboards.chooseVisualTab('colors');
  cy.get('[data-test-id="tab-Colors"]').within(() => {
    cy.get('[data-test-id="settings-slider-input"]').trigger('click', 'right');
  });
}

export function setFixedColorLegend() {
  dashboards.chooseVisualTab('colors');
  cy.get('[data-test-id="tab-Colors"]').within(() => {
    cy.get('[data-test-id="checkbox-fixed_color_legend"]').check();
  });
}

// Data tab functions

export function switchDataSet(datasetName: string, dataconnectionName?: string) {
  dashboards.chooseVisualTab('data');
  cy.get('#visual-data-panel span[data-test-id="app-dataset-switch"]').click();
  cy.get('#datasource-modal').should('have.class', 'show');

  // if we have multiple dataconnections
  // it's not 100% we will have the samples
  // in the first dropdown (connections)
  if (dataconnectionName) {
    cy.get(
      '#datasource-modal .modal-body .form-group:first-child select'
    ).select(dataconnectionName, { force: true });
  }
  cy.get('#datasource-modal select[data-test-id="switch-dataset-dataset-sel"]').select(
    datasetName,
    { force: true }
  );

  modalClick('button[data-test-id="switch-dataset-apply-btn"]');
  cy.get('#datasource-modal', { timeout: 3000 }).should('not.be.visible');
  dashboards.closeVisual(false);
  dashboards.editVisual();
  cy.get('#visual-data-panel span[data-test-id="app-dataset"]').should('have.text', datasetName);

  dashboards.closeVisual(false);
}

// Settings style tab functions

const visualStyleTab = '[data-test-id="dash-visual-style-tab"]';
const visualBasicSelector = '[data-test-id="visual-style-tab-Visual Basics"]';
const visualTitleSelector = '[data-test-id="visual-style-tab-Visual Title"]';
const visualSubtitleSelector = '[data-test-id="visual-style-tab-Visual Subtitle"]';
const visualTooltipStyleSelector = '[data-test-id="visual-style-tab-Tooltip"]';
const dropDownSelector = '[data-test-id="dropdown-btn"]';

export function setVisualBasicFontType(fontype: string) {
  visual_functions.openSection('style', 'Visual Basics');
  const visualBasicFontTypeSelector = '[data-test-id="setting-id-visual_font"]';
  cy.get(visualStyleTab)
    .find(visualBasicSelector)
    .scrollIntoView()
    .within(() => {
      cy.get(visualBasicFontTypeSelector).within(() => {
        cy.get(dropDownSelector).click({ force: true });
        cy.get('[data-test-id="value-' + fontype + '"]').click({ force: true });
      });
    });
  dashboards.closeVisual(false);
  dashboards.editVisual();
  visual_functions.openSection('style', 'Visual Basics');
  cy.get(visualStyleTab)
    .find(visualBasicSelector)
    .scrollIntoView()
    .within(() => {
      cy.get(visualBasicFontTypeSelector).within(() => {
        cy.get(dropDownSelector).should('have.attr', 'data-value', fontype);
      });
    });

  dashboards.closeVisual(false);
}

export function setVisualBasicFontSize(cloneDashboardName: string, fontsize: string) {
  visual_functions.openSection('style', 'Visual Basics');
  const visualBasicFontSizeSelector = '[data-test-id="setting-id-visual_font_size"]';
  cy.get(visualStyleTab)
    .find(visualBasicSelector)
    .scrollIntoView()
    .within(() => {
      cy.get(visualBasicFontSizeSelector).within(() => {
        cy.get('[data-test-id="TITLE_FONT_SIZE"]').clear();
        cy.get('[data-test-id="TITLE_FONT_SIZE"]').type(`${fontsize}{enter}`);
      });
    });
  dashboards.closeVisual(false);
  cy.reload();
  dashboards.editVisual();
  visual_functions.openSection('style', 'Visual Basics');
  cy.get(visualStyleTab)
    .find(visualBasicSelector)
    .scrollIntoView()
    .within(() => {
      cy.get(visualBasicFontSizeSelector).within(() => {
        cy.get('[data-test-id="TITLE_FONT_SIZE"]').should('have.attr', 'value', fontsize);
      });
    });

  const regex = new RegExp(`font-size:\\s*${fontsize}px`);
  cy.get(visual.locators.visual.x_axis + ' > g.tick text')
    .should('have.attr', 'style')
    .and('match', regex);
  dashboards.closeVisual(false);
}

export function setVisualTitleFontType(fontype: string) {
  visual_functions.openSection('style', 'Visual Title');
  const visualTitleFontSelector = '[data-test-id="setting-id-title_font"]';
  // set the visual title font to be Comic Sans MS
  cy.get(visualStyleTab)
    .find(visualTitleSelector)
    .scrollIntoView()
    .within(() => {
      cy.get(visualTitleFontSelector).within(() => {
        cy.get(dropDownSelector).click({ force: true });
        cy.get('[data-test-id="value-' + fontype + '"]').click({ force: true });
      });
    });
  // save close and reopen the visual
  dashboards.closeVisual(false);
  dashboards.editVisual();
  visual_functions.openSection('style', 'Visual Title');
  // verify the font is set right
  cy.get(visualStyleTab)
    .find(visualTitleSelector)
    .scrollIntoView()
    .within(() => {
      cy.get(visualTitleFontSelector).within(() => {
        cy.get(dropDownSelector).should('have.attr', 'data-value', fontype);
      });
    });
  dashboards.closeVisual(false);
}

export function setVisualTitleFontSize(cloneDashboardName: string, fontsize: string) {
  visual_functions.openSection('style', 'Visual Title');
  const visualTitleFontSizeSelector = '[data-test-id="setting-id-title_font_size"]';
  // set the visual title font to be Comic Sans MS
  cy.get(visualStyleTab)
    .find(visualTitleSelector)
    .scrollIntoView()
    .within(() => {
      cy.get(visualTitleFontSizeSelector).within(() => {
        cy.get('[data-test-id="TITLE_FONT_SIZE"]').clear();
        cy.get('[data-test-id="TITLE_FONT_SIZE"]').type(`${fontsize}{enter}`);
      });
    });
  // save close and reopen the visual
  visual_common_test.reopenSettingsAfterSaveAndReload();
  visual_functions.openSection('style', 'Visual Title');
  // verify the font is set right
  cy.get(visualStyleTab)
    .find(visualTitleSelector)
    .scrollIntoView()
    .within(() => {
      cy.get(visualTitleFontSizeSelector).within(() => {
        cy.get('[data-test-id="TITLE_FONT_SIZE"]').should('have.attr', 'value', fontsize);
      });
    });
  dashboards.closeVisual(false);
}

export function setVisualTitleFontWeight(fontweight: string) {
  visual_functions.openSection('style', 'Visual Title');
  const visualTitleFontWeightSelector = '[data-test-id="setting-id-visual_title_font_weight"]';
  // set the visual title font to be Comic Sans MS
  cy.get(visualStyleTab)
    .find(visualTitleSelector)
    .scrollIntoView()
    .within(() => {
      cy.get(visualTitleFontWeightSelector).within(() => {
        cy.get(dropDownSelector).click({ force: true });
        cy.get('[data-test-id="value-' + fontweight + '"]').click({ force: true });
      });
    });
  // save close and reopen the visual
  dashboards.closeVisual(false);
  dashboards.editVisual();
  visual_functions.openSection('style', 'Visual Title');
  // verify the font is set right
  cy.get(visualStyleTab)
    .find(visualTitleSelector)
    .scrollIntoView()
    .within(() => {
      cy.get(visualTitleFontWeightSelector).within(() => {
        cy.get(dropDownSelector).should('have.attr', 'data-value', fontweight);
      });
    });
  dashboards.closeVisual(false);
}

export function setVisualTitleFontAlignment(alignment: string) {
  visual_functions.openSection('style', 'Visual Title');
  const visualTitleFontAlignmentSelector = '[data-test-id="title_alignment"]';
  // set the visual title font to be Comic Sans MS
  cy.get(visualStyleTab)
    .find(visualTitleSelector)
    .scrollIntoView()
    .within(() => {
      cy.get(visualTitleFontAlignmentSelector).within(() => {
        cy.get('[data-test-id="radio-' + alignment + '"]').click({ force: true });
      });
    });
  // save close and reopen the visual
  dashboards.closeVisual(false);
  dashboards.editVisual();
  visual_functions.openSection('style', 'Visual Title');
  // verify the font is set right
  cy.get(visualStyleTab)
    .find(visualTitleSelector)
    .scrollIntoView()
    .within(() => {
      cy.get(visualTitleFontAlignmentSelector).within(() => {
        cy.get('[data-test-id="radio-' + alignment + '"]').should('be.checked');
      });
    });
  dashboards.closeVisual(false);
}

export function setVisualTitleHide() {
  visual_functions.openSection('style', 'Visual Title');
  const visualTitleHideSelector = '[data-test-id="setting-id-visual_hide_title"]';
  // set the visual title font to be Comic Sans MS
  cy.get(visualStyleTab)
    .find(visualTitleSelector)
    .scrollIntoView()
    .within(() => {
      cy.get(visualTitleHideSelector).within(() => {
        cy.get('[data-test-id="checkbox-VISUAL_HIDE_TITLE"]').click({ force: true });
      });
    });
  // save close and reopen the visual
  dashboards.closeVisual(false);
  dashboards.editVisual();
  visual_functions.openSection('style', 'Visual Title');
  // verify the font is set right
  cy.get(visualStyleTab)
    .find(visualTitleSelector)
    .scrollIntoView()
    .within(() => {
      cy.get(visualTitleHideSelector).within(() => {
        cy.get('[data-test-id="checkbox-VISUAL_HIDE_TITLE"]').should('be.checked');
      });
    });
  dashboards.closeVisual(false);
}

export function setVisualSubtitleFontType(cloneDashboardName: string, fonttype: string) {
  visual_functions.openSection('style', 'Visual Subtitle');
  const visualSubtitleFontSelector = '[data-test-id="setting-id-subtitle_font"]';
  cy.get(visualStyleTab)
    .find(visualSubtitleSelector)
    .scrollIntoView()
    .within(() => {
      cy.get(visualSubtitleFontSelector).within(() => {
        cy.get(dropDownSelector).click({ force: true });
        cy.get('[data-test-id="value-' + fonttype + '"]').click({ force: true });
      });
    });
  dashboards.closeVisual(false);
  dashboards.openDashboardforEdit(cloneDashboardName);
  dashboards.editVisual();
  visual_functions.openSection('style', 'Visual Subtitle');
  cy.get(visualStyleTab)
    .find(visualSubtitleSelector)
    .scrollIntoView()
    .within(() => {
      cy.get(visualSubtitleFontSelector).within(() => {
        cy.get(dropDownSelector).should('have.attr', 'data-value', fonttype);
      });
    });
  dashboards.closeVisual(false);
}

export function setVisualSubtitleFontSize(cloneDashboardName: string, fontsize: string) {
  visual_functions.openSection('style', 'Visual Subtitle');
  const visualSubtitleFontSizeSelector = '[data-test-id="setting-id-subtitle_font_size"]';
  cy.get(visualStyleTab)
    .find(visualSubtitleSelector)
    .scrollIntoView()
    .within(() => {
      cy.get(visualSubtitleFontSizeSelector).within(() => {
        cy.get('[data-test-id="TITLE_FONT_SIZE"]').clear();
        cy.get('[data-test-id="TITLE_FONT_SIZE"]').type(`${fontsize}{enter}`);
      });
    });
  // save close and reopen the visual
  visual_common_test.reopenSettingsAfterSaveAndReload();
  visual_functions.openSection('style', 'Visual Subtitle');
  cy.get(visualStyleTab)
    .find(visualSubtitleSelector)
    .scrollIntoView()
    .within(() => {
      cy.get(visualSubtitleFontSizeSelector).within(() => {
        cy.get('[data-test-id="TITLE_FONT_SIZE"]').should('have.attr', 'value', fontsize);
      });
    });
  dashboards.closeVisual(false);
}

export function setVisualSubtitleFontAlignment(cloneDashboardName: string, alignment: string) {
  visual_functions.openSection('style', 'Visual Subtitle');
  const visualSubtitleFontAlignmentSelector = '[data-test-id="subtitle_alignment"]';
  cy.get(visualStyleTab)
    .find(visualSubtitleSelector)
    .scrollIntoView()
    .within(() => {
      cy.get(visualSubtitleFontAlignmentSelector).within(() => {
        cy.get('[data-test-id="radio-' + alignment + '"]').click({ force: true });
      });
    });
  // save close and reopen the visual
  dashboards.closeVisual(false);
  cy.reload();
  dashboards.editVisual();
  visual_functions.openSection('style', 'Visual Subtitle');
  // verify the font is set right
  cy.get(visualStyleTab)
    .find(visualSubtitleSelector)
    .scrollIntoView()
    .within(() => {
      cy.get(visualSubtitleFontAlignmentSelector).within(() => {
        cy.get('[data-test-id="radio-' + alignment + '"]').then(($el) => {
          const isChecked = $el.attr('checked');
          assert.isTrue(typeof isChecked !== typeof undefined);
          // assert.isTrue($el.attr("checked"));
        });
      });
    });
  dashboards.closeVisual(false);
}

export function setVisualSubtitleTTFontType(fonttype: string) {
  visual_functions.openSection('style', 'Tooltip');
  const visualTooltipFontSelector = '[data-test-id="setting-id-tooltip_font"]';
  cy.get(visualStyleTab)
    .find(visualTooltipStyleSelector)
    .scrollIntoView()
    .within(() => {
      cy.get(visualTooltipFontSelector).within(() => {
        cy.get(dropDownSelector).click({ force: true });
        cy.get('[data-test-id="value-' + fonttype + '"]').click({ force: true });
      });
    });
  dashboards.closeVisual(false);
  dashboards.editVisual();
  visual_functions.openSection('style', 'Tooltip');
  cy.get(visualStyleTab)
    .find(visualTooltipStyleSelector)
    .scrollIntoView()
    .within(() => {
      cy.get(visualTooltipFontSelector).within(() => {
        cy.get(dropDownSelector).should('have.attr', 'data-value', fonttype);
      });
    });
  dashboards.closeVisual(false);
}

export function setVisualSubtitleTTFontSize(cloneDashboardName: string, fontsize: string) {
  visual_functions.openSection('style', 'Tooltip');
  const visualTooltipFontSizeSelector = '[data-test-id="setting-id-tooltip_font_size"]';
  cy.get(visualStyleTab)
    .find(visualTooltipStyleSelector)
    .scrollIntoView()
    .within(() => {
      cy.get(visualTooltipFontSizeSelector).within(() => {
        cy.get('[data-test-id="TITLE_FONT_SIZE"]').clear();
        cy.get('[data-test-id="TITLE_FONT_SIZE"]').type(`${fontsize}{enter}`);
      });
    });
  // save close and reopen the visual
  visual_common_test.reopenSettingsAfterSaveAndReload();
  visual_functions.openSection('style', 'Tooltip');
  cy.get(visualStyleTab)
    .find(visualTooltipStyleSelector)
    .scrollIntoView()
    .within(() => {
      cy.get(visualTooltipFontSizeSelector).within(() => {
        cy.get('[data-test-id="TITLE_FONT_SIZE"]').should('have.attr', 'value', fontsize);
      });
    });
  dashboards.closeVisual(false);
}

export function addSegmentTo(segmentShelfLocator: string) {
  cy.get(segmentShelfLocator).click();
  cy.get(
    '[data-test-id="visual-data-panel"] [data-test-id="dim-app-params"] [data-test-item="Segment"]'
  ).click({ force: true });
}
export function addSegmentToXShelf(segmentName: string) {
  cy.get('#app-shelves [data-test-id="x_shelf_list"] [data-test-id="x_shelf-app-shelf-item"')
    .contains('Segment')
    .click({ force: true });
  cy.get(visual.locators.visual.segment_picker).click();
  cy.get(visual.locators.visual.segment_modal).should('be.visible');
  cy.get(visual.locators.visual.segment_values).each(($value) => {
    if ($value.text().trim() === segmentName) {
      cy.wrap($value)
        .prev()
        .check();
      cy.get(visual.locators.visual.filter_segment_save).click();
    }
  });
  dashboards.closeVisual(false);
}

// Settings tab functions

export function enableDisableCSVDownload() {
  visual_functions.openSection('dashboardSettings', 'Downloads');
  cy.get('[data-test-id="checkbox-permit_csv_download_dashboard"]').check();

  cy.get('[data-test-id="dash-main-panel"]').scrollTo('right');

  dashboards.saveDashboard();

  // Partial html issue.
  // Used "parent child" selector to differentiate the one we need and the one in partial html
  cy.get('[data-test-id="visual-list-item"]:nth-child(1) [data-test-id="visual-more"]').click({
    force: true,
  });

  cy.get('[data-test-id="visual-submenu"]')
    .contains('Download as...')
    .should('be.visible')
    .parent()
    .find('[data-test-id="export-csv-link"]')
    .should('not.have.class', 'arc-invisible')
    .closest('[data-test-id="dropdown-menu"]')
    .find('[data-test-id="export-xlsx-link"]')
    .should('not.have.class', 'arc-invisible');

  cy.get('[data-test-id="checkbox-permit_csv_download_dashboard"]').uncheck();

  dashboards.saveDashboard();

  // Partial html issue.
  // Used "parent child" selector to differentiate the one we need and the one in partial html
  cy.get('[data-test-id="visual-list-item"]:nth-child(1) [data-test-id="visual-more"]').click({
    force: true,
  });

  cy.get('[data-test-id="visual-submenu"]')
    .contains('Download as...')
    .parent()
    .find('[data-test-id="export-csv-link"]')
    .should('have.class', 'arc-invisible')
    .closest('[data-test-id="dropdown-menu"]')
    .find('[data-test-id="export-xlsx-link"]')
    .should('have.class', 'arc-invisible');

  dashboards.closeVisual(false);
}

export function enableDisableAnimation() {
  visual_functions.openSection('settings', 'General');
  cy.get('[data-test-id="checkbox-ENABLE_ANIMATIONS"]').check({ force: true });
  dashboards.saveVisual();
  cy.get('[data-test-id="checkbox-ENABLE_ANIMATIONS"]').uncheck({ force: true });
  dashboards.saveVisual();
  dashboards.closeVisual(false);
}

export function displayTrelisBorder() {
  visual_functions.openSection('settings', 'Trellis');
  cy.get('[data-test-id="checkbox-TRELLIS_BORDERS"]')
    .scrollIntoView()
    .uncheck();
  dashboards.closeVisual(false);
  // verifyVisual(hash);

  dashboards.editVisual();
  visual_functions.openSection('settings', 'Trellis');
  cy.get('[data-test-id="checkbox-TRELLIS_BORDERS"]')
    .scrollIntoView()
    .check();
  dashboards.closeVisual(false);
  // verifyVisual(hash);
}

export function setinpdDimScaleTrellis() {
  visual_functions.openSection('settings', 'Trellis');
  cy.get('[data-test-id="checkbox-scale_dim_independent"]')
    .scrollIntoView()
    .check();
  dashboards.closeVisual(false);
  // verifyVisual(hash);

  dashboards.editVisual();
  visual_functions.openSection('settings', 'Trellis');
  cy.get('[data-test-id="checkbox-scale_dim_independent"]')
    .scrollIntoView()
    .uncheck();
  dashboards.closeVisual(false);
  // verifyVisual(hash);
}

export function setinpdAggrScaleTrellis(available = true) {
  visual_functions.openSection('settings', 'Trellis');
  const independentAggrScaleLocator = '[data-test-id="checkbox-scale_agg_independent"]';
  cy.get(independentAggrScaleLocator)
    .scrollIntoView()
    .check();
  dashboards.closeVisual(false);
  // verifyVisual(hash);

  dashboards.editVisual();
  visual_functions.openSection('settings', 'Trellis');
  cy.get(independentAggrScaleLocator)
    .scrollIntoView()
    .uncheck();
  dashboards.closeVisual(false);
  // verifyVisual(hash);
}

export function verifySettingsAreDisabled(
  settings: Array<{ settingGroup: string; settinglocators: string[] }>
) {
  settings.forEach((setting) => {
    visual_functions.openSection('settings', setting.settingGroup);
    setting.settinglocators.forEach((locator) =>
      cy.get(locator).should('have.class', 'disabled-setting')
    );
  });
}

export function setEnableTooltipsSetting(cloneDashboardName: string, action: string) {
  visual_functions.openSection('settings', 'Marks');
  if (action === 'checked') {
    cy.log('Check showtooltip checkbox');
    cy.get('[data-test-id="checkbox-show_tooltip"]')
      .scrollIntoView()
      .check();
  } else {
    cy.log('Uncheck showtooltip checkbox');
    cy.get('[data-test-id="checkbox-show_tooltip"]')
      .scrollIntoView()
      .uncheck();
  }
  dashboards.saveVisual();
  cy.reload();
  dashboards.editVisual();
  visual_functions.openSection('settings', 'Marks');
  cy.get('[data-test-id="checkbox-show_tooltip"]').scrollIntoView();
  cy.get('[data-test-id="checkbox-show_tooltip"]').then(($el) => {
    const isChecked = $el.attr(action);
    if (action === 'checked') {
      assert.isTrue(typeof isChecked !== typeof undefined);
    } else {
      assert.isFalse(typeof isChecked !== typeof undefined);
    }
  });
  dashboards.closeVisual(false);
}

export function enableDisableDetailData(action: string) {
  visual_functions.openSection('settings', 'Marks');
  if (action === 'checked') {
    cy.get("#dashboard-visual-settings-sidebar input[name='VISUAL_SHOW_DETAIL_DATA']")
      .scrollIntoView()
      .check();
  } else {
    cy.get("#dashboard-visual-settings-sidebar input[name='VISUAL_SHOW_DETAIL_DATA']")
      .scrollIntoView()
      .uncheck();
  }
  dashboards.closeVisual(false);
}

export function setBarSizeRange(barsize: string) {
  visual_functions.openSection('settings', 'Marks');
  cy.get("input[name='BAR_MARK_SIZE_RANGE']").scrollIntoView();
  cy.get("input[name='BAR_MARK_SIZE_RANGE']").clear();
  cy.get("input[name='BAR_MARK_SIZE_RANGE']").type(`${barsize}{enter}`);
  dashboards.closeVisual(false);
}

export function setLegendWidth(cloneDashboardName: string, width: string) {
  visual_functions.openSection('settings', 'Marks');
  cy.get('[data-test-id="LEGEND_WIDTH_OVERRIDE"]')
    .scrollIntoView()
    .clear()
    .type(width);
  dashboards.saveVisual();
}

export function reverseLegendOrder() {
  visual_functions.openSection('settings', 'Marks');
  cy.get('[data-test-id="checkbox-reverse_legend_order"]')
    .scrollIntoView()
    .check();
  dashboards.saveVisual();
}

export function setlineraTrendline() {
  visual_functions.openSection('settings', 'Marks');
  cy.get('[data-test-id="checkbox-trendline_linear"]')
    .scrollIntoView()
    .check();
  cy.get('[data-test-id="checkbox-trendline_linear"]')
    .scrollIntoView()
    .uncheck();
  dashboards.saveVisual();
  // verifyVisual();
}

export function setSelectionBrush() {
  visual_functions.openSection('settings', 'Marks');
  cy.get('[data-test-id="checkbox-SHOW_SELECTION_RANGE"]')
    .scrollIntoView()
    .check();
  dashboards.saveVisual();
  dashboards.closeVisual(false);
}

export function setIgnoreNegativeData() {
  visual_functions.openSection('settings', 'Marks');
  cy.get('[data-test-id="checkbox-IGNORE_STACKED_NEGATIVES_WITH_MIXED_MEASURES"]')
    .scrollIntoView()
    .check();
  dashboards.saveVisual();
  dashboards.closeVisual(false);
  // verifyVisual();
}

export function setHighLightMarkType(mark: string) {
  visual_functions.openSection('settings', 'Marks');
  cy.get("#dashboard-visual-settings-sidebar input[value='" + mark + "']")
    .scrollIntoView()
    .check();
  dashboards.saveVisual();
  dashboards.closeVisual(false);
  // verifyVisual();
}

export function setShowPoints() {
  visual_functions.openSection('settings', 'Marks');
  cy.get('[data-test-id="checkbox-show_points"]')
    .scrollIntoView()
    .check();
  dashboards.saveVisual();
  dashboards.closeVisual(false);
}

export function setIgnoreMissingDates() {
  visual_functions.openSection('settings', 'Marks');
  cy.get('[data-test-id="checkbox-smooth_line"]')
    .scrollIntoView()
    .check();
  dashboards.saveVisual();
  dashboards.closeVisual(false);
  // verifyVisual();
}

export function setLineStyle(style: string) {
  visual_functions.openSection('settings', 'Marks');
  cy.get(
    "#dashboard-visual-settings-sidebar div[data-setting-id='line_interpolation']"
  ).scrollIntoView();
  cy.get("#dashboard-visual-settings-sidebar div[data-setting-id='line_interpolation']").within(
    ($style) => {
      cy.get("button[data-toggle='dropdown']")
        .scrollIntoView()
        .click();
      cy.get("a[data-value='" + style + "']")
        .scrollIntoView()
        .click();
    }
  );
  dashboards.saveVisual();
  dashboards.closeVisual(false);
  // verifyVisual();
}

export function setShowNullValues(location: string) {
  visual_functions.openSection('settings', 'Data');
  cy.get('[data-test-id="nulls_order"]').scrollIntoView();
  cy.get('[data-test-id="nulls_order"]').within(($nulls) => {
    cy.get('[data-test-id="radio-' + location + '"]').check();
    // verifyVisual();
  });
  dashboards.saveVisual();
  dashboards.closeVisual(false);
}

export function setNumberofRowsinCSV(count: string) {
  visual_functions.openSection('settings', 'Data');
  cy.get('[data-test-id="DOWNLOAD_ROWS"]').scrollIntoView();
  cy.get('[data-test-id="DOWNLOAD_ROWS"]').clear();
  cy.get('[data-test-id="DOWNLOAD_ROWS"]').type(count);
  dashboards.closeVisual(false);
  // verifyVisual();
}

export function setNumberofRowsToFetch(count: string) {
  visual_functions.openSection('settings', 'Data');
  cy.get('[data-test-id="MAX_ROWCOUNT"]').scrollIntoView();
  cy.get('[data-test-id="MAX_ROWCOUNT"]').clear();
  cy.get('[data-test-id="MAX_ROWCOUNT"]').type(count);
  dashboards.closeVisual(false);
  // verifyVisual();
}

export function setDisableIncrementalQueries() {
  visual_functions.openSection('settings', 'Data');
  cy.get('[data-test-id="checkbox-DISABLE_INCREMENTAL_QUERIES"]').scrollIntoView();
  cy.get('[data-test-id="checkbox-DISABLE_INCREMENTAL_QUERIES"]').check();
  dashboards.saveVisual();
  visual_functions.openSection('settings', 'Data');
  cy.get('[data-test-id="checkbox-DISABLE_INCREMENTAL_QUERIES"]').scrollIntoView();
  cy.get('[data-test-id="checkbox-DISABLE_INCREMENTAL_QUERIES"]').uncheck();
  dashboards.closeVisual(false);
}

export function setAutoDetectTimeColumn() {
  visual_functions.openSection('settings', 'Axes');
  cy.get('[data-test-id="checkbox-VISUAL_AUTODETECT_TIME"]').scrollIntoView();
  cy.get('[data-test-id="checkbox-VISUAL_AUTODETECT_TIME"]').uncheck();
  dashboards.saveVisual();
  visual_functions.openSection('settings', 'Axes');
  cy.get('[data-test-id="checkbox-VISUAL_AUTODETECT_TIME"]').scrollIntoView();
  cy.get('[data-test-id="checkbox-VISUAL_AUTODETECT_TIME"]').check();
  dashboards.closeVisual(false);
}

export function setAxisScale(scale: string) {
  visual_functions.openSection('settings', 'Axes');
  cy.get('[data-test-id="setting-id-scale_type"]').scrollIntoView();
  cy.get('[data-test-id="setting-id-scale_type"]').within(() => {
    cy.get('[data-test-id="dropdown-btn"]').click();
    cy.get('[data-test-id="value-' + scale + '"]');
    // cy.contains('scale').click();
  });
  dashboards.closeVisual(false);
}

export function setGridLines() {
  visual_functions.openSection('settings', 'Axes');
  cy.get('[data-test-id="checkbox-gridlines_enabled"]')
    .scrollIntoView()
    .check();
  dashboards.closeVisual(false);
  // verifyVisual();
}

export function setAggrScaleStart() {
  visual_functions.openSection('settings', 'Axes');
  cy.get('[data-test-id="checkbox-scale_agg_zero"]')
    .scrollIntoView()
    .check();
  dashboards.closeVisual(false);
  // verifyVisual();
}

export function setMinAggrScale() {
  visual_functions.openSection('settings', 'Axes');
  cy.get('[data-test-id="MIN_AGG_AXIS"]')
    .scrollIntoView()
    .clear()
    .type('100');
  dashboards.closeVisual(false);
  // verifyVisual();
}

export function setMaxAggrScale() {
  visual_functions.openSection('settings', 'Axes');
  cy.get('[data-test-id="MAX_AGG_AXIS"]')
    .scrollIntoView()
    .clear()
    .type('100');
  dashboards.closeVisual(false);
  // verifyVisual();
}

export function setSecMinAggrScale() {
  visual_functions.openSection('settings', 'Axes');
  cy.get('[data-test-id="SECONDARY_MIN_AGG_AXIS"]')
    .scrollIntoView()
    .clear()
    .type('100');
  dashboards.closeVisual(false);
}

export function setSecMaxAggrScale() {
  visual_functions.openSection('settings', 'Axes');
  cy.get('[data-test-id="SECONDARY_MAX_AGG_AXIS"]')
    .scrollIntoView()
    .clear()
    .type('100');
  dashboards.closeVisual(false);
}

export function setIncludeXAxisLabel() {
  visual_functions.openSection('settings', 'Axes');
  cy.get('[data-test-id="checkbox-x_axis_label"]')
    .scrollIntoView()
    .uncheck();
  dashboards.closeVisual(false);
  // verifyVisual();
}

export function setIncludeYAxisLabel() {
  visual_functions.openSection('settings', 'Axes');
  cy.get('[data-test-id="checkbox-y_axis_label"]')
    .scrollIntoView()
    .uncheck();
  dashboards.closeVisual(false);
  // verifyVisual();
}

export function setShowXAXis() {
  visual_functions.openSection('settings', 'Axes');
  cy.get('[data-test-id="checkbox-SHOW_X_AXIS"]')
    .scrollIntoView()
    .uncheck();
  dashboards.closeVisual(false);
  // verifyVisual();
}

export function setShowYAXis() {
  visual_functions.openSection('settings', 'Axes');
  cy.get('[data-test-id="checkbox-SHOW_Y_AXIS"]')
    .scrollIntoView()
    .uncheck();
  dashboards.closeVisual(false);
  // verifyVisual();
}

export function dynamicallyResizeGraph() {
  visual_functions.openSection('settings', 'Axes');
  cy.get('[data-test-id="checkbox-dynamic_bar_width"]')
    .scrollIntoView()
    .check();
  dashboards.closeVisual(false);
  // verifyVisual();
}

export function setBucketCount(count: string) {
  visual_functions.openSection('settings', 'Basic');
  cy.get('input[name="BUCKET_COUNT"]')
    .scrollIntoView()
    .clear();
  cy.get('input[name="BUCKET_COUNT"]')
    .scrollIntoView()
    .type(count);
  dashboards.closeVisual(false);
  // verifyVisual();
}
export function setNormalizeHistogram() {
  visual_functions.openSection('settings', 'Basic');
  cy.get('input[name="NORMALIZED"]')
    .scrollIntoView()
    .check();
  dashboards.closeVisual(false);
  // verifyVisual();
}

export function setCumulativeHistogram() {
  visual_functions.openSection('settings', 'Basic');
  cy.get('input[name="CUMULATIVE"]')
    .scrollIntoView()
    .check();
  dashboards.closeVisual(false);
  // verifyVisual();
}
