import * as dashboards from 'lib/dashboard';
import { locators } from 'lib/dashboard';
import { VisualType } from 'lib/dashboard/dashboards_interfaces';
import { waitForVisualsLoaded } from 'lib/functions';
import * as visual from 'lib/visual';
import * as visual_functions from 'lib/visual/visual_functions';
import * as fontSizeHelper from './fontsizeSetting';

export function reopenSettingsAfterSaveAndReload() {
  // save close and reopen the visual
  dashboards.saveVisual();

  cy.reload();
  waitForVisualsLoaded();

  dashboards.editVisual();
  dashboards.chooseVisualTab('style');
}

export function testColorPaletteSelection(visualType: VisualType) {
  cy.get(visual.locators.style.colors.secondColorPalatte4thColor)
    .dblclick()
    .then(($color) => {
      const selectedColorRGB = $color.css('background-color');
      cy.get(visual.locators.style.colors.firstColorWithinPaletteInfo).then(($first) => {
        expect($first.css('background-color')).to.eq(selectedColorRGB);
      });
      const $colorBoxes = Cypress.$(visual.locators.style.colors.selectedColorPalettesColorBoxes);
      const colors = $colorBoxes
        .map(function() {
          return Cypress.$(this).attr('data-color');
        })
        .get();

      cy.document().then(($doc) => {
        const visualLocator = 'div.dash-viz-wrapper canvas';
        const canvasLocator = visualLocator + '.maincanvas';
        ($doc.querySelectorAll(canvasLocator) as NodeListOf<HTMLCanvasElement>).forEach(
          (canvas) => {
            const stack = (canvas.getContext('2d') as any)._stack;
            switch (visualType) {
              case VisualType.scatter:
                const stackFillStyleBelongingToScatterMarks = (stack as Array<{
                  method?: string;
                  attr?: string;
                }>).filter((x: any, i) => x.attr === 'fillStyle' && stack[i - 1]?.method === 'arc');
                console.log(
                  'stackFillStyleBelongingTocircles',
                  stackFillStyleBelongingToScatterMarks
                );
                stackFillStyleBelongingToScatterMarks.forEach((element: any) => {
                  expect(colors.some((color) => color === element.val)).to.be.true;
                });
                break;
              default:
                break;
            }
          }
        );
      });
    });
}

export function testColorOpacity() {
  cy.get(".settings-tab-holder[data-tabname='Colors']").within(($colorsTab) => {
    cy.get('[data_setting_id="color_opacity"] .settings-slider-input')
      .invoke('val', 5)
      .trigger('change');

    cy.get(visual.locators.style.colors.secondColorPalatte4thColor).then(($color) => {
      expect($color.css('opacity')).to.be.eq('0.05');
      //   cy.document().then(($doc) => {
      //     cy.wait(5000).then(()=>{
      //         saveVisual();
      //         const visualLocator = 'div.dash-viz-wrapper canvas';
      //         const canvas = visualLocator + '.maincanvas';
      //         const context = ($doc.querySelector(canvas) as HTMLCanvasElement).getContext('2d');
      //     })
      // });
    });
  });
}

export function testBasicFontStyleChangeToImpact() {
  const testName = 'setVisualBasicFontType';
  cy.log('Starting test ' + testName);
  // GIVEN
  dashboards.editVisual();
  visual.openSection('style', 'Visual Basics');
  // WHEN
  cy.get(visual.locators.style.basicSelector).within(($visualBasicSection) => {
    cy.get(visual.locators.style.basicFontType).within(($fontTypeSection) => {
      cy.get(visual.locators.style.dropDownSelector).click();
      cy.get("a[data-value='arc-font-impact']").click();
    });
  });

  reopenSettingsAfterSaveAndReload();

  // THEN
  cy.get(visual.locators.style.basicSelector).within(($visualBasicSection) => {
    cy.get(visual.locators.style.basicFontType).within(($fontTypeSection) => {
      cy.get(visual.locators.style.dropDownSelector).should(
        'have.attr',
        'data-value',
        'arc-font-impact'
      );
    });
  });
  cy.get('.dash-viz-wrapper').should('have.css', 'font-family', 'Impact, Charcoal, sans-serif');
}

export function testVisualTitleAlignmentSetToCenter() {
  const testName = 'setVisualTitleFontAlignment';
  cy.log('Starting test ' + testName);
  cy.get(visual.locators.style.titleSelector).within(($visualTitleSection) => {
    cy.get(visual.locators.style.visualTitleFontAlignment).within(($titleFontTypeSection) => {
      cy.get("input[value='Center']").click();
    });
  });
  // save close and reopen the visual
  reopenSettingsAfterSaveAndReload();
  // verify the font is set right
  cy.get(visual.locators.style.titleSelector).within(($visualTitleSection) => {
    cy.get(visual.locators.style.visualTitleFontAlignment).within(($titleFontTypeSection) => {
      cy.get("input[value='Center']").should('be.checked');
    });
  });
  cy.get(dashboards.locators.viz_title_class)
    .should('have.attr', 'style')
    .and('match', /text-align:\s*center/);
}

export function testVisualTitleFontWeightSetToBold() {
  const functionName = 'testVisualTitleFontWeightSetToBold';
  cy.log('Start: ' + functionName);
  // GIVEN
  // WHEN
  const testName = 'setVisualTitleFontWeight';
  cy.log('Starting test ' + testName);
  visual.openSection('style', 'Visual Title');
  cy.get(visual.locators.style.titleSelector).within(($visualTitleSection) => {
    cy.get(visual.locators.style.visualTitleFontWeight).within(($titleFontTypeSection) => {
      cy.get(visual.locators.style.dropDownSelector).click();
      cy.get("a[data-value='bold']").click();
    });
  });
  // save close and reopen the visual
  reopenSettingsAfterSaveAndReload();
  // THEN
  // verify the font is set right
  cy.get(visual.locators.style.titleSelector).within(($visualTitleSection) => {
    cy.get(visual.locators.style.visualTitleFontWeight).within(($titleFontTypeSection) => {
      cy.get(visual.locators.style.dropDownSelector).should('have.attr', 'data-value', 'bold');
    });
  });
  cy.get(dashboards.locators.viz_title_class)
    .should('have.attr', 'style')
    .and('match', /font-weight:\s*bold/);
  cy.log('Finished: ' + functionName);
}

export function testVisualTitleFontFamilySetToArial() {
  // GIVEN
  // WHEN
  // set the visual title font to be Impact
  visual.openSection('style', 'Visual Title');
  cy.get(visual.locators.style.titleSelector).within(($visualTitleSection) => {
    cy.get(visual.locators.style.titleFontSelector).within(($titleFontTypeSection) => {
      cy.get(visual.locators.style.dropDownSelector).click();
      cy.get("a[data-value='arc-font-arial']").click();
    });
  });
  // save close and reopen the visual
  reopenSettingsAfterSaveAndReload();
  // THEN
  // verify the font is set right
  cy.get(visual.locators.style.titleSelector).within(($visualTitleSection) => {
    cy.get(visual.locators.style.titleFontSelector).within(($titleFontTypeSection) => {
      cy.get(visual.locators.style.dropDownSelector).should(
        'have.attr',
        'data-value',
        'arc-font-arial'
      );
    });
  });
  cy.get(dashboards.locators.viz_title_class_span).should(
    'have.css',
    'font-family',
    'Arial, Helvetica, sans-serif'
  );
}

export function testVisualtitleIsHiddenInViewMode() {
  const funtionName = 'testVisualtitleIsHiddenInViewMode';
  cy.log(`Starting: ${funtionName}`);
  visual.openSection('style', 'Visual Title');
  cy.get(visual.locators.style.titleSelector).within(($visualTitleSection) => {
    cy.get(visual.locators.style.visualTitleHide).within(($titleFontTypeSection) => {
      cy.get("input[name='VISUAL_HIDE_TITLE']").click();
    });
  });
  // save close and reopen the visual
  reopenSettingsAfterSaveAndReload();
  // verify the font is set right
  cy.get(visual.locators.style.titleSelector).within(($visualTitleSection) => {
    cy.get(visual.locators.style.visualTitleHide).within(($titleFontTypeSection) => {
      cy.get("input[name='VISUAL_HIDE_TITLE']").should('be.checked');
    });
  });

  cy.get(dashboards.locators.view_mode).click();
  cy.get('div.viz-permanent div.arc-viz-title-container').should('not.be.visible');
  cy.get(dashboards.locators.edit_link).click();
  cy.log(`Ended: ${funtionName}`);
}

export function testVisualSubtitleFontAlignmentSetToRight() {
  const testName = 'testVisualSubtitleFontAlignmentSetToRight';
  console.log('Starting ' + testName);
  visual.openSection('style', 'Visual Subtitle');
  cy.get(visual.locators.style.subtitleSelector).within(($visualTitleSection) => {
    cy.get(visual.locators.style.subtitleFontAlignment).within(($titleFontTypeSection) => {
      cy.get("input[value='Right']").click();
    });
  });
  // save close and reopen the visual
  reopenSettingsAfterSaveAndReload();
  // verify the font is set right
  cy.get(visual.locators.style.subtitleSelector).within(($visualTitleSection) => {
    cy.get(visual.locators.style.subtitleFontAlignment).within(($titleFontTypeSection) => {
      cy.get("input[value='Right']").then(($el) => {
        const isChecked = $el.attr('checked');
        assert.isTrue(typeof isChecked !== typeof undefined);
        // assert.isTrue($el.attr("checked"));
      });
    });
  });
  cy.get(dashboards.locators.viz_subtitle_container)
    .should('have.attr', 'style')
    .and('match', /text-align:\s*right/);

  console.log('End: ' + testName);
}

export function testVisualSubtitleFontTypeSetToHelvetica() {
  const testName = 'setVisualSubtitleFontType';
  console.log('Starting test ' + testName);
  visual.openSection('style', 'Visual Subtitle');
  cy.get(visual.locators.style.subtitleSelector).within(($visualBasicSection) => {
    cy.get(visual.locators.style.subtitleFontSelector).within(($fontTypeSection) => {
      cy.get(visual.locators.style.dropDownSelector).click();
      cy.get("a[data-value='arc-font-helvetica']").click();
    });
  });
  reopenSettingsAfterSaveAndReload();
  cy.get(visual.locators.style.subtitleSelector).within(($visualBasicSection) => {
    cy.get(visual.locators.style.subtitleFontSelector).within(($fontTypeSection) => {
      cy.get(visual.locators.style.dropDownSelector).should(
        'have.attr',
        'data-value',
        'arc-font-helvetica'
      );
    });
  });

  cy.get(dashboards.locators.viz_subtitle_class_span)
    .should('have.attr', 'data-fontclass')
    .and('match', /arc-font-helvetica/);
}

export function testSettingFontSizeHasTheDesiredEffect(
  settingOption: fontSizeHelper.FontSizeSettingTestOptions,
  fontSize: number
) {
  // WHEN
  dashboards.editVisual();
  visual.openSection('style', 'Visual Basics');
  visual.openSection('style', 'Visual Subtitle');
  visual.openSection('style', 'Tooltip');
  const fontSizeInputLocator = "input[name='TITLE_FONT_SIZE']";

  const locator = fontSizeHelper.FontSizeTestHelperMap.get(settingOption);
  if (locator) {
    cy.get(locator.styleSettingLocators.subsection).within(() => {
      cy.get(locator.styleSettingLocators.fontSizeSetter).within(() => {
        cy.get(fontSizeInputLocator)
          .clear()
          .type(fontSize.toString());
      });
    });
    // save close and reopen the visual
    reopenSettingsAfterSaveAndReload();
    // verify the font is set right
    // THEN
    cy.get(locator.styleSettingLocators.subsection).within(() => {
      cy.get(locator.styleSettingLocators.fontSizeSetter).within(() => {
        cy.get(fontSizeInputLocator).should('have.attr', 'value', fontSize.toString());
      });
    });

    const expectedFontSizeRgx = new RegExp(`font-size:\\s*${fontSize}px`);
    cy.get(locator.effectedElementLocator)
      .should('have.attr', 'style')
      .and('match', expectedFontSizeRgx);
  } else {
    throw new Error('FontSizeTestHelperMap should be extended!');
  }
}

export enum ArcVisualTypes {
  combo = 'arc-viz-type-combo',
}

export enum userActionTypes {
  click = 'click',
  mouseover = 'mouseover',
}

enum dataLayerLocator {
  canvas = '.arc-viz-overlay-container canvas.maincanvas:first-of-type',
  svg = '[data-test-id="visual-list-item"] .viz-bar-svg:first',
}

export function enableDrillIntoFunctionality() {
  const functionName = 'enableDrillIntoFunctionality';
  cy.log(`${functionName} - started`);
  visual_functions.openSection('settings', 'General');
  cy.get("input[name='ENABLE_DRILL_INTO']").check();
  dashboards.saveVisual();
  cy.log(`${functionName} - finished`);
}

// It actually checks wether drilling works or not
export function testDrillIntoOptionIsAvailableInContextMenu(expectedResultIsAvailable = true) {
  triggerUserActionOnValidPointOnVisual(userActionTypes.click);
  cy.get('.contextmenu-button-label')
    .contains('Drill Into')
    .should(expectedResultIsAvailable ? 'be.visible' : 'not.exist');

  dashboards.closeVisual(false);
}

export function testTooltipIsVisibleOnHover() {
  triggerUserActionOnValidPointOnVisual(userActionTypes.mouseover);
  cy.get('[data-test-id="visual-datapoint-tooltip"]');
}

export function checkLegendPositionAt(position: 'right' | 'bottom' | 'above' | 'none') {
  cy.get(`${locators.visual_select_specific.replace('index', '1')}`).then(($visual) => {
    const $legend = $visual.find('.arc-legend-container');
    switch (position) {
      case 'right':
        expect($legend.closest('tr').find('.arc-viz-overlay-container')).to.exist;
        break;
      case 'bottom':
        expect(
          $legend
            .closest('tr')
            .prev()
            .find('.arc-viz-overlay-container')
        ).to.exist;
        break;
      case 'above':
        expect(
          $legend
            .closest('tr')
            .next()
            .find('.arc-viz-overlay-container')
        ).to.exist;
        break;
      case 'none':
        expect($legend).not.to.exist;
        break;
      default:
        break;
    }
  });
}

export function testLegendPositionChangeBySwitchingTo(
  style: 'right' | 'bottom' | 'above' | 'none'
) {
  const functionName = `setLegendStyle to : ${style}`;
  cy.log(`${functionName} started`);
  dashboards.editVisual();
  visual_functions.openSection('settings', 'Marks');
  const legendStyle = `[data-test-id="legend-style-${style}"]`;
  cy.get(legendStyle).click();

  reopenSettingsAfterSaveAndReload();
  visual_functions.openSection('settings', 'Marks');

  cy.get(legendStyle)
    .parent()
    .then(($el) => {
      const cls = $el.prop('class');
      console.log('class:' + cls);
      expect(cls.indexOf('legend-style-selected')).to.be.greaterThan(1);
    });

  checkLegendPositionAt(style);
  dashboards.closeVisual(false);
  cy.log(`${functionName} finished`);
}

function applyUserActionAt(
  selector: dataLayerLocator,
  useraction: userActionTypes,
  coordinates: { x: number; y: number }
) {
  switch (useraction) {
    case userActionTypes.mouseover:
      cy.get(selector).trigger('mouseover', coordinates.x, coordinates.y, { force: true });
      break;
    case userActionTypes.click:
    default:
      cy.get(selector).click(coordinates.x, coordinates.y, { force: true });
  }
}

function getDataPointCoordinatesOnComboVisual() {
  return cy.document().then(($doc) => {
    const context = ($doc.querySelector(dataLayerLocator.canvas) as HTMLCanvasElement).getContext(
      '2d'
    );
    const [translateX, translateY] = (context as any)._stack.filter(
      (item: { method: string }) => item.method === 'translate'
    )[0].arguments;
    const [moveToX, moveToY] = (context as any)._stack.filter(
      (item: { method: string }) => item.method === 'rect'
    )[0].arguments;
    const [x, y] = [(translateX + moveToX) as number, (translateY + moveToY) as number];
    return { datalayer: dataLayerLocator.canvas, coordinates: { x, y } };
  });
}

export function getVisualType(index: number = 1) {
  return cy
    .get(locators.visual_select_specific.replace('index', index.toString()))
    .then(($element) => {
      return (
        $element
          .find('.arc-viz-container')
          .attr('class')
          ?.match(new RegExp(/arc-viz-type-\w*/))?.[0] || 'unknown'
      );
    });
}

export function triggerUserActionOnValidPointOnVisual(userAction: userActionTypes) {
  let visualType = 'unknown';
  let dataLayer: dataLayerLocator;
  cy.get(locators.visual_select_specific.replace('index', '1')).then(($element) => {
    visualType =
      $element
        .find('.arc-viz-container')
        .attr('class')
        ?.match(new RegExp(/arc-viz-type-\w*/))?.[0] || visualType;
    switch (visualType) {
      case ArcVisualTypes.combo:
        getDataPointCoordinatesOnComboVisual().then((result) => {
          applyUserActionAt(result.datalayer, userAction, result.coordinates);
        });
        break;
      default:
        dataLayer = dataLayerLocator.svg;
        cy.get(dataLayer).then((svg) => {
          const rectBox = svg
            .find('.viz-bar-g rect')
            .first()[0]
            .getBoundingClientRect();
          const svgBox = svg[0].getBoundingClientRect();

          // calculate the rect bottom position relative to the svg element:
          const rectBottomX = rectBox.x + rectBox.width / 2 - svgBox.x;
          const rectBottomY = rectBox.y + rectBox.height - 2 - svgBox.y;
          applyUserActionAt(dataLayer, userAction, { x: rectBottomX, y: rectBottomY });
        });
    }
  });
}

function getContextTranslateArgs(context: any) {
  return (context as any)._stack.filter(
    (item: { method: string }) => item.method === 'translate'
  )[0].arguments;
}

function getBarLabelIdx($doc: Document, label: string) {
  const ticks = Array.from($doc.querySelectorAll(`.viz-bar-svg .x.arc-viz-axis .tick`));
  const tickWithLabel = ticks?.filter(($el) =>  $el.querySelector('text')?.innerHTML === label)[0];
  let labelIdx = ticks.indexOf(tickWithLabel);
  labelIdx = labelIdx > -1 ? labelIdx : 0;
  return labelIdx;
}

function getDataPointCoordinatesOnColorBarByLabel(label: string) {
  return cy.document().then(($doc) => {
    const context = ($doc.querySelector(dataLayerLocator.canvas) as HTMLCanvasElement).getContext(
      '2d'
    );
    const [translateX, translateY] = getContextTranslateArgs(context);
    const labelIdx = getBarLabelIdx($doc, label);

    const [a, b, width] = (context as any)._stack.filter(
      (item: { method: string }) => item.method === 'rect'
    )[0].arguments;

    const moveToX = labelIdx * width;

    const [c, moveToY, d, height] = (context as any)._stack.filter(
      (item: { method: string, arguments: number[]}) => item.method === 'rect' && item.arguments[0] === moveToX && item.arguments[3] < 0
    )[0].arguments;

    const [x, y] = [(translateX + moveToX + width/2) as number, (translateY + moveToY + height/2) as number];
    return { datalayer: dataLayerLocator.canvas, coordinates: { x, y } };
  });
}

function getDataPointCoordinatesOnBarByLabel(label: string) {
  return cy.document().then(($doc) => {
    const context = ($doc.querySelector(dataLayerLocator.canvas) as HTMLCanvasElement).getContext(
      '2d'
    );

    const [translateX, translateY] = getContextTranslateArgs(context);
    const labelIdx = getBarLabelIdx($doc, label);

    const [moveToX, moveToY, width, height] = (context as any)._stack.filter(
      (item: { method: string }) => item.method === 'rect'
    )[labelIdx].arguments;

    const [x, y] = [(translateX + moveToX + width/2) as number, (translateY + moveToY + height/2) as number];
    return { datalayer: dataLayerLocator.canvas, coordinates: { x, y } };
  });
}

export function triggerUserActionOnSpecificBarOnBarVisual(label: string, userAction: userActionTypes, hasColor : boolean = false ) {
  cy.get(locators.visual_select_specific.replace('index', '1')).then(() => {
    const getDataPointCoordinates = hasColor ? getDataPointCoordinatesOnColorBarByLabel : getDataPointCoordinatesOnBarByLabel;
    getDataPointCoordinates(label).then((result) => {
      applyUserActionAt(result.datalayer, userAction, result.coordinates);
    });
  });
}

function testNumberOfMeasureTextPresentOnCanvasInTheSameNumberAsMarker() {
  getVisualType().then((visualType) => {
    switch (visualType) {
      case ArcVisualTypes.combo:
        cy.document().then(($doc) => {
          const canvasLocator = '.arc-viz-overlay-container canvas.maincanvas:first-of-type';
          const context = ($doc.querySelector(canvasLocator) as HTMLCanvasElement).getContext('2d');
          const stack = (context as any)._stack;
          const numberOfMeasureTexts = stack.filter((item: any) => item.method === 'fillText')
            ?.length;
          const numberOfBars = stack.filter((item: any) => item.method === 'rect')?.length;
          expect(numberOfMeasureTexts).to.eq(numberOfBars);
        });
      default:
    }
  });
}

function testNoMeasureTextDisplayedOnCanvasLayer() {
  getVisualType().then((visualType) => {
    switch (visualType) {
      case ArcVisualTypes.combo:
        cy.document().then(($doc) => {
          const canvasLocator = '.arc-viz-overlay-container canvas.maincanvas:first-of-type';
          const context = ($doc.querySelector(canvasLocator) as HTMLCanvasElement).getContext('2d');
          const stack = (context as any)._stack;
          const numberOfMeasureTexts = stack.filter((item: any) => item.method === 'fillText')
            ?.length;
          expect(numberOfMeasureTexts).to.eq(0);
        });
      default:
    }
  });
}

export function testShowMeasureValuesSetting() {
  // TODO check functionality on other visuals!
  visual_functions.openSection('settings', 'Marks');

  // TODO: Current small solution - later replace with snapshot comparison?
  testNoMeasureTextDisplayedOnCanvasLayer();
  cy.get('[data-test-id="checkbox-INLINE_MEASURE_DISPLAY"]')
    .scrollIntoView()
    .check();
  dashboards.saveVisual();
  testNumberOfMeasureTextPresentOnCanvasInTheSameNumberAsMarker();
}

export function getCanvasStack(locator: string) {
  return cy.document().then(($doc) => {
    const context = ($doc.querySelector(locator) as HTMLCanvasElement).getContext('2d');
    return (context as any)._stack;
  });
}
function getDimensions(visualType: ArcVisualTypes) {
  // TODO expand with different types!
  return cy.document().then(($doc) => {
    switch (visualType) {
      case ArcVisualTypes.combo:
        const context = ($doc.querySelector(
          '.arc-viz-overlay-container canvas.maincanvas:first-of-type'
        ) as HTMLCanvasElement).getContext('2d');
        const defaultRect = (context as any)._stack.filter(
          (item: any) => item.method === 'rect'
        )?.[0];
        const height = defaultRect.arguments[1] - defaultRect.arguments[3];
        const width = defaultRect.arguments[0] - defaultRect.arguments[2];
        return { width, height };
      default:
        return { width: 0, height: 0 };
    }
  });
}

export function testCellHeightWidthSetting() {
  // TODO: temporary solution to check anything actuall changes on the visual...
  // Replace it with screenshot comparison when it is available!
  getVisualType().then((type) => {
    getDimensions(type as ArcVisualTypes).then((original) => {
      visual_functions.openSection('settings', 'Marks');
      cy.get('[data-test-id="MIN_CELL_HEIGHT"]').scrollIntoView();
      cy.get('[data-test-id="MIN_CELL_HEIGHT"]')
        .type('700')
        .trigger('change');

      getDimensions(type as ArcVisualTypes).then((modified) => {
        if (original.height) {
          expect(modified.height).to.greaterThan(original.height);
        }
      });

      cy.get('[data-test-id="MIN_CELL_WIDTH"]').scrollIntoView();
      cy.get('[data-test-id="MIN_CELL_WIDTH"]')
        .type('800')
        .trigger('change');

      getDimensions(type as ArcVisualTypes).then((modified) => {
        if (original.width) {
          expect(modified.width).to.greaterThan(original.width);
        }
      });
    });
  });

  dashboards.saveVisual();
  dashboards.closeVisual(false);
}

export function convertLineToArea() {
  visual_functions.openSection('settings', 'Marks');
  cy.get("input[name='USE_AREA']")
    .scrollIntoView()
    .check();
  dashboards.saveVisual();
  dashboards.closeVisual(false);
  // verifyVisual();
}
