import { CSSselector } from 'locators/common/CSSselector';

// visual_number : 1 onwards in order left to right as shown on the ui
export function editTableVSToBarVS(visualNumber: number) {
  cy.intercept('POST', '/arc/reports/report/*').as('visualSubmitted');

  cy.get(CSSselector.visual_edit_specific(visualNumber)).click({
    force: true,
  });
  cy.get(CSSselector.bar_visual).click();
  cy.get(CSSselector.vizual_fields_tab.replace('column-name', 'ctyname')).click();
  cy.get(CSSselector.measure_field).click();
  cy.get(CSSselector.vizual_fields_tab.replace('column-name', 'state')).click();

  cy.get(CSSselector.visual_title_specific(visualNumber))
    .click()
    .type('Bar Visual Title');
  cy.get(CSSselector.refresh_visual).click();
  cy.get(CSSselector.save_visual).click();

  cy.wait('@visualSubmitted')
    .its('response.statusCode')
    .should('eq', 200);

  cy.get('#chart-1 > svg > g > text.x.arc-viz-axis-label').should('contain', 'ctyname');
  cy.get('#chart-1 > svg > g > text.y.arc-viz-axis-label').should('exist');
  cy.get("div[class$='bars']").should('be.visible');
  // cy.get(objects.refresh_dashboard).click({force: true});
  cy.get('.dash-viz-wrapper [class="y arc-viz-axis-label"]').should('exist');
  cy.get('#app-selected-list [class="x arc-viz-axis-label"]').should('have.text', 'ctyname');
}

// visual_number : 1 onwards in order left to right as shown on the ui
export function editTableVSToLineVS(visualNumber: number) {
  cy.intercept('POST', '/arc/reports/report/*').as('visualSubmitted');

  cy.get(CSSselector.visual_edit_specific(visualNumber)).click({
    force: true,
  });
  cy.get(CSSselector.line_visual).click();
  cy.get(CSSselector.vizual_fields_tab.replace('column-name', 'ctyname')).click();
  cy.get(CSSselector.measure_field).click();
  cy.get(CSSselector.vizual_fields_tab.replace('column-name', 'state')).click();
  cy.get(CSSselector.visual_title_specific(visualNumber))
    .click()
    .type('Line Visual Title');
  cy.get(CSSselector.refresh_visual).click();
  cy.get(CSSselector.save_visual).click();

  cy.wait('@visualSubmitted')
    .its('response.statusCode')
    .should('eq', 200);

  cy.get('#chart-1 > svg > g > text.x.arc-viz-axis-label').should('contain', 'ctyname');
  cy.get('#chart-1 [class="y arc-viz-axis-label"]').should('exist');
  cy.get('div[class$="arc-viz-type-trellis-lines"]').should('be.visible');
  // cy.get(objects.refresh_dashboard).click({force: true});
  /*  */
  cy.get('.gridster div[class^="arc-viz-container"][class$="arc-viz-type-trellis-lines"]').should(
    'be.visible'
  );
  cy.get('.dash-viz-wrapper [class="x arc-viz-axis-label"]').should('have.text', 'ctyname');
  cy.get('.dash-viz-wrapper [class="y arc-viz-axis-label"]').should('exist');
}

/**
 * clicks on exter/edit expression inthe field properties menu (which should be open prior to calling this)
 * then enters the sqlExpression and clicks on validate and asserts a 'successful validation'  of sql expression
 * @export
 * @param {string} sqlExpression
 */
export function verifySQLExpression(sqlExpression: string) {
  cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('verifySQL');
  cy.get(CSSselector.field_properties_expression_link).click();
  // require to wait for the textarea focus then perform type comand
  cy.get(CSSselector.expression_data_input).focus();

  cy.get(CSSselector.expression_data).should('have.class', 'ace_focus');

  cy.get('[data-test-id="expression-modal"]')
    .find('.ace_editor')
    .then(($elem) => {
      cy.window()
        .its('ace' as any)
        .invoke('edit', $elem.attr('id'))
        .invoke('setValue', sqlExpression);
    });

  cy.get('[data-test-id="edit-expression-form"]')
    .contains('Validate Expression')
    .click({ force: true });

  cy.wait('@verifySQL');
  // Assert for the expected alert message which shows inside modal dialog
  cy.get(CSSselector.edit_expression_container)
    .find(CSSselector.alert_dialog)
    .should('contain', 'Validation Successful!');
  cy.wait(1000);
  cy.get(CSSselector.expression_save).click();
}

// visual_number : 1 onwards in order left to right as shown on the ui
export function editTableVSToLineVSCustom(visualNumber: number, visualTitle: string) {
  cy.intercept('POST', '/arc/reports/report/*').as('visualSubmitted');
  cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('validateSQLExpression');

  cy.get(CSSselector.visual_edit_specific(visualNumber)).click({
    force: true,
  });

  // line visual click it depends on the chart
  cy.get(CSSselector.line_visual).click();

  // the column which we need to select in field(X-axis)
  cy.get(CSSselector.vizual_fields_tab.replace('column-name', 'date')).click();
  cy.get("li[class='dropdown app-shelf-item'] span[title='date']").click();

  verifySQLExpression('from_unixtime(unix_timestamp([date] , "yyyy-MM-dd") , "yyyy") as "Year"');

  // ********************Same logic to verify sql query depends on the field(Y-axis)********************

  // click on y Axis and select the desire field(fatalities)
  cy.get(CSSselector.yAxis_field).click();
  cy.get(CSSselector.vizual_fields_tab.replace('column-name', 'fatalities')).click();
  cy.get("#y_shelf li[class='dropdown app-shelf-item']:nth-child(1)").click();

  verifySQLExpression('count(coalesce([fatalities], 0)) as "Incidents"');

  // ********************Same logic to verify sql query depends on the field(Colors), here we need to verify 2 sql-queries for colors field********************

  // ===============Start => 1st sql-query===============

  // click on color field and select desire field(fatalities)
  cy.get(CSSselector.color_field).click();
  cy.get(CSSselector.vizual_fields_tab.replace('column-name', 'fatalities')).click();
  cy.get(CSSselector.color_field_first_element).click();

  verifySQLExpression('sum(coalesce([fatalities],0)) as "Fatalities"');

  // click on axis from the propertu panel. Then select Secondary Axis
  cy.get(CSSselector.property_axis).click();
  cy.get(CSSselector.secondary_axis).click();

  // ===============completed => 1st sql-query===============

  // =============== Start => 2nd sql-query===============

  // click on color field and select desire field(fatalities)
  cy.get(CSSselector.color_field).click();
  cy.get(CSSselector.vizual_fields_tab.replace('column-name', 'fatalities')).click();
  cy.get(CSSselector.color_field_second_element).click();

  verifySQLExpression('sum(coalesce([fatalities],0)) as "Survivers"');

  // click on axis from the property panel. Then select Secondary Axis
  cy.get(CSSselector.property_axis).click();
  cy.get(CSSselector.secondary_axis).click();

  // ===============completed => 2nd sql-query===============

  // close property panel
  cy.get(CSSselector.close_property_panel).click();

  // ********************Update Visual Settings********************

  // ===============1.Update Parameters===============

  // click on =>setting tab,parameters
  cy.get(CSSselector.visual_settings_tab).click();
  cy.get(CSSselector.visual_settings_parameter).click();

  // uncheck Send parameters with dataset scope - no longer a checkbox its a button dropdown instead
  cy.get(CSSselector.visual_settings_parameter_data_scope).should('be.visible');
  cy.get(CSSselector.visual_settings_parameter_data_scope).click();
  cy.get(CSSselector.visual_settings_parameter_data_scope)
    .next()
    .contains('Explict')
    .click();

  // ===============Completed => Update Parameters===============

  // check "Send aggregrate as parameters"
  cy.get(CSSselector.visual_settings_parameter_aggregate).click();

  // ===============2.Update Data===============
  // click on data tab which inside settings and perfomed type command
  cy.get(CSSselector.visual_settings_data).click();

  // Number of rows downloading during CSV export and Details Data
  cy.get(CSSselector.visual_settings_downloading).type('-1');

  // ===============Completed => Update Data===============

  // apply visual title and save it

  cy.get(CSSselector.visual_title_specific(visualNumber))
    .click()
    .type(visualTitle);
  cy.get(CSSselector.refresh_visual).click();
  cy.get(CSSselector.save_visual).click();

  cy.wait('@visualSubmitted')
    .its('response.statusCode')
    .should('eq', 200);

  // Verify Expected Enter fields are displayed in visuals OR not - done in the test (or pass as params)
  // cy.get(objects.xAxis_field_name).should('have.text', 'Year')
  // cy.get(objects.yAxis_field_name).should('have.text', 'Incidents')
  // cy.get(objects.secondary_field).contains('Fatalities')
  // cy.get(objects.secondary_field).contains('Survivers')
}

// visual_number : 2 onwards in order left to right as shown on the ui
export function editTableVSToKPIVSCustom(visualNumber: number, visualTitle: string) {
  cy.intercept('POST', '/arc/reports/report/*').as('visualSubmitted');
  cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('visualUpdated');
  // 2nd Visual code(Visual KPI)
  cy.get(CSSselector.selecting_visuals).click();
  cy.wait('@visualSubmitted')
    .its('response.statusCode')
    .should('eq', 200);

  cy.get("div[class='gridster ready']>ul>li").then(($p2) => {
    expect($p2.children().length).to.equal(visualNumber);
  });

  cy.wait('@visualUpdated')
    .its('response.statusCode')
    .should('eq', 200);

  // Need to wait for visible the rows for visual
  cy.get(CSSselector.visual_rows.replace('index', String(visualNumber))).should('be.visible');

  cy.get(CSSselector.visual_edit_specific(visualNumber)).click({
    force: true,
  });

  cy.get(CSSselector.kpi_visual).click();
  cy.get(CSSselector.data_tab).click();
  cy.get(CSSselector.measure_field_element).click();
  cy.get(CSSselector.vizual_fields_tab.replace('column-name', 'fatalities')).click();

  cy.get("#app-shelves #aggregates_shelf li[class='dropdown app-shelf-item']:nth-child(1)").click();

  verifySQLExpression('count(coalesce([fatalities],0)) as "Total"');

  cy.get(CSSselector.field_properties_display_format_link).click();
  cy.get(CSSselector.display_format_type).select('number');

  cy.get(CSSselector.decimal_places).clear();
  cy.get(CSSselector.decimal_places).type('0');
  cy.get(CSSselector.comma_separator).then(($p2) => {
    if (!$p2.prop('checked')) {
      cy.get(CSSselector.comma_separator).click();
    }
  });
  cy.get(CSSselector.display_format_save_button).click();

  // apply visual title and save it
  cy.get(CSSselector.visual_title_specific(visualNumber))
    .click()
    .type(visualTitle);
  cy.get(CSSselector.refresh_visual).click();

  cy.wait('@visualUpdated')
    .its('response.statusCode')
    .should('eq', 200);

  cy.get(CSSselector.save_visual).click();

  cy.wait('@visualSubmitted')
    .its('response.statusCode')
    .should('eq', 200);
}

// visual_number : 3 onwards in order left to right as shown on the ui
export function editTableVSToKPIVS2Custom(visualNumber: number, visualTitle: string) {
  cy.intercept('POST', '/arc/reports/report/*').as('visualSubmitted');
  cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('visualUpdated');
  // 3rd Visual code(Visual KPI)
  cy.get(CSSselector.selecting_visuals).click();
  cy.wait('@visualSubmitted')
    .its('response.statusCode')
    .should('eq', 200);

  cy.get("div[class='gridster ready']>ul>li").then(($p2) => {
    expect($p2.children().length).to.equal(visualNumber);
  });

  cy.wait('@visualUpdated')
    .its('response.statusCode')
    .should('eq', 200);

  // Need to wait for visible the rows for visual
  cy.get(CSSselector.visual_rows.replace('index', String(visualNumber))).should('be.visible');

  cy.get(CSSselector.visual_edit_specific(visualNumber)).click({
    force: true,
  });

  cy.get(CSSselector.kpi_visual).click();
  cy.get(CSSselector.data_tab).click();
  cy.get(CSSselector.measure_field_element).click();
  cy.get(CSSselector.vizual_fields_tab.replace('column-name', 'fatalities')).click();

  cy.get("#app-shelves #aggregates_shelf li[class='dropdown app-shelf-item']:nth-child(1)").click();

  verifySQLExpression('sum(coalesce([fatalities],0)) as "Total"');

  cy.get(CSSselector.field_properties_display_format_link).click();
  cy.get(CSSselector.display_format_type).select('number');

  cy.get(CSSselector.decimal_places).clear();
  cy.get(CSSselector.decimal_places).type('0');
  cy.get(CSSselector.comma_separator).then(($p2) => {
    if (!$p2.prop('checked')) {
      cy.get(CSSselector.comma_separator).click();
    }
  });
  cy.get(CSSselector.display_format_save_button).click();

  // apply visual title and save it
  cy.get(CSSselector.visual_title_specific(visualNumber))
    .click()
    .type(visualTitle);
  cy.get(CSSselector.refresh_visual).click();

  cy.wait('@visualUpdated')
    .its('response.statusCode')
    .should('eq', 200);

  cy.get(CSSselector.save_visual).click();

  cy.wait('@visualSubmitted')
    .its('response.statusCode')
    .should('eq', 200);
}

// visual_number : 4 onwards in order left to right as shown on the ui
export function editTableVSToKPIVS3Custom(visualNumber: number, visualTitle: string) {
  cy.intercept('POST', '/arc/reports/report/*').as('visualSubmitted');
  cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('visualUpdated');
  // 3rd Visual code(Visual KPI)
  cy.get(CSSselector.selecting_visuals).click({
    force: true,
  });
  cy.wait('@visualSubmitted')
    .its('response.statusCode')
    .should('eq', 200);

  cy.get("div[class='gridster ready']>ul>li").then(($p2) => {
    expect($p2.children().length).to.equal(visualNumber);
  });

  cy.wait('@visualUpdated')
    .its('response.statusCode')
    .should('eq', 200);

  // Need to wait for visible the rows for visual
  cy.get(CSSselector.visual_edit_specific(visualNumber)).scrollIntoView();
  cy.get(CSSselector.visual_rows.replace('index', String(visualNumber))).should('exist');

  cy.get(CSSselector.visual_edit_specific(visualNumber)).click({
    force: true,
  });

  cy.get(CSSselector.kpi_visual).click();
  cy.get(CSSselector.data_tab).click();
  cy.get(CSSselector.measure_field_element).click();
  cy.get(CSSselector.vizual_fields_tab.replace('column-name', 'fatalities')).click();

  cy.get("#app-shelves #aggregates_shelf li[class='dropdown app-shelf-item']:nth-child(1)").click();

  verifySQLExpression(
    'sum(CASE [aboard] WHEN NULL THEN 0 ELSE [aboard]-coalesce([fatalities],0) END) as "Survivers"'
  );

  cy.get(CSSselector.field_properties_display_format_link).click();
  cy.get(CSSselector.display_format_type).select('number');

  cy.get(CSSselector.decimal_places).clear();
  cy.get(CSSselector.decimal_places).type('0');
  cy.get(CSSselector.comma_separator).then(($p2) => {
    if (!$p2.prop('checked')) {
      cy.get(CSSselector.comma_separator).click();
    }
  });
  cy.get(CSSselector.display_format_save_button).click();

  // apply visual title and save it
  cy.get(CSSselector.visual_title_specific(visualNumber))
    .click()
    .type(visualTitle);
  cy.get(CSSselector.refresh_visual).click();

  cy.wait('@visualUpdated')
    .its('response.statusCode')
    .should('eq', 200);

  cy.get(CSSselector.save_visual).click();

  cy.wait('@visualSubmitted')
    .its('response.statusCode')
    .should('eq', 200);
}

// visual_number : 5 onwards in order left to right as shown on the ui
export function editTableVSToScatterVSCustom(visualNumber: number, visualTitle: string) {
  cy.intercept('POST', '/arc/reports/report/*').as('visualSubmitted');
  cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('visualUpdated');

  // 2nd Visual code(Visual Scatter)
  cy.get(CSSselector.selecting_visuals).click({
    force: true,
  });
  cy.wait('@visualSubmitted')
    .its('response.statusCode')
    .should('eq', 200);

  cy.get("div[class='gridster ready']>ul>li").then(($p2) => {
    expect($p2.children().length).to.equal(visualNumber);
  });

  cy.wait('@visualUpdated')
    .its('response.statusCode')
    .should('eq', 200);

  // Need to wait for visible the rows for visual
  cy.get(CSSselector.visual_rows.replace('index', String(visualNumber))).should('be.visible');

  cy.get(CSSselector.visual_edit_specific(visualNumber)).click({
    force: true,
  });

  cy.get(CSSselector.scatter_visual).click();
  cy.get(CSSselector.data_tab).click();

  // the column which we need to select in field(X-axis)
  cy.get(CSSselector.vizual_fields_tab.replace('column-name', 'date')).click();
  cy.get("li[class='dropdown app-shelf-item'] span[title='date']").click();

  verifySQLExpression('from_unixtime(unix_timestamp([date] , "yyyy-MM-dd") , "yyyy") as "Year"');

  // ********************Same logic to verify sql query depends on the field(Y-axis)********************

  // click on y Axis and select the desire field(fatalities)
  cy.get(CSSselector.yAxis_field).click();
  cy.get(CSSselector.vizual_fields_tab.replace('column-name', 'fatalities')).click();
  cy.get("#app-shelves #y_shelf li[class='dropdown app-shelf-item']:nth-child(1)").click();

  verifySQLExpression('sum(coalesce([fatalities],0)) as "Fatalities"');

  // ********************Same logic to verify sql query depends on the field(Colors), here we need to verify 2 sql-queries for colors field********************

  // ===============Start => 1st sql-query===============

  // click on color field and select desire field(fatalities)
  cy.get(CSSselector.color_field).click();
  cy.get(CSSselector.vizual_fields_tab.replace('column-name', 'fatalities')).click();
  cy.get(CSSselector.color_field_first_element).click();

  verifySQLExpression('coalesce([operator], "N/A") as "Operator"');

  // ===============completed => 1st sql-query===============

  // close property panel
  cy.get(CSSselector.close_property_panel).click();
  // =============== Start => 2nd sql-query===============

  // click on color field and select desire field(fatalities)
  cy.get(CSSselector.size_field).click();
  cy.get(CSSselector.vizual_fields_tab.replace('column-name', 'fatalities')).click();
  cy.get("#app-shelves #size_shelf li[class='dropdown app-shelf-item']:nth-child(1)").click();

  verifySQLExpression('count([date]) as "Incidents"');

  cy.get(CSSselector.filter_field).click();
  cy.get(CSSselector.vizual_fields_tab.replace('column-name', 'fatalities')).click();
  cy.get("#app-shelves #filters_shelf li[class='dropdown app-shelf-item']:nth-child(1)").click();

  verifySQLExpression('coalesce([fatalities],0)>0');
  // ********************Update Visual Settings********************

  // ===============1.Update Parameters===============

  // click on =>setting tab,parameters
  cy.get(CSSselector.visual_settings_tab).click();
  cy.get(CSSselector.visual_settings_parameter).click();

  // uncheck Send parameters with dataset scope - no longer a checkbox its a button dropdown instead
  cy.get(CSSselector.visual_settings_parameter_data_scope).should('be.visible');
  cy.get(CSSselector.visual_settings_parameter_data_scope).click();
  cy.get(CSSselector.visual_settings_parameter_data_scope)
    .next()
    .contains('Explict')
    .click();

  // ===============Completed => Update Parameters===============

  // check "Send aggregrate as parameters"
  cy.get(CSSselector.visual_settings_parameter_aggregate).click();

  // ===============2.Update Data===============
  // click on data tab which inside settings and perfomed type command
  cy.get(CSSselector.visual_settings_data).click();

  // Number of rows downloading during CSV export and Details Data
  cy.get(CSSselector.visual_settings_downloading).type('-1');

  // ===============Completed => Update Data===============

  // apply visual title and save it
  cy.get(CSSselector.visual_title_specific(visualNumber))
    .click()
    .type(visualTitle);
  cy.get(CSSselector.refresh_visual).click();

  cy.get(CSSselector.xAxis_field_name).should('have.text', 'Year');
  cy.get(CSSselector.yAxis_field_name).should('have.text', 'Fatalities');

  cy.wait('@visualUpdated')
    .its('response.statusCode')
    .should('eq', 200);

  cy.get(CSSselector.save_visual).click();

  cy.wait('@visualSubmitted')
    .its('response.statusCode')
    .should('eq', 200);
}

// visual_number : 6 onwards in order left to right as shown on the ui
export function editRadialVisual(visualNumber: number, visualTitle: string) {
  cy.intercept('POST', '/arc/reports/report/*').as('visualSubmitted');
  cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('visualUpdated');

  // 4th Visual code(Visual Radial)
  cy.get(CSSselector.selecting_visuals).click({
    force: true,
  });
  cy.wait('@visualSubmitted')
    .its('response.statusCode')
    .should('eq', 200);

  cy.get("div[class='gridster ready']>ul>li").then(($p2) => {
    expect($p2.children().length).to.equal(visualNumber);
  });

  cy.wait('@visualUpdated')
    .its('response.statusCode')
    .should('eq', 200);

  // Need to wait for visible the rows for visual
  cy.get(CSSselector.visual_rows.replace('index', String(visualNumber))).should('be.visible');

  cy.get(CSSselector.visual_edit_specific(visualNumber)).click({
    force: true,
  });

  cy.get(CSSselector.radial_visual).click();
  cy.get(CSSselector.data_tab).click();

  // the column which we need to select in Dimensions field(operator)
  cy.get(CSSselector.vizual_fields_tab.replace('column-name', 'operator')).click();
  cy.get("li[class='dropdown app-shelf-item'] span[title='operator']").click();

  // the column which we need to select in Measures field(fatalities)
  cy.get(CSSselector.measure_field_element).click();
  cy.get(CSSselector.vizual_fields_tab.replace('column-name', 'fatalities')).click();
  cy.get("#app-shelves #aggregates_shelf li[class='dropdown app-shelf-item']:nth-child(1)").click();

  verifySQLExpression('sum(coalesce([fatalities],0)) as "Fatalities"');
  cy.get(CSSselector.close_property_panel).click();

  cy.get(CSSselector.vizual_fields_tab.replace('column-name', 'fatalities')).click();
  cy.get("#app-shelves #aggregates_shelf li[class='dropdown app-shelf-item']:nth-child(2)").click();

  verifySQLExpression('count(coalesce([fatalities], 0)) as "Incidents"');
  cy.get(CSSselector.close_property_panel).click();

  cy.get(CSSselector.vizual_fields_tab.replace('column-name', 'fatalities')).click();
  cy.get("#app-shelves #aggregates_shelf li[class='dropdown app-shelf-item']:nth-child(3)").click();

  verifySQLExpression(
    'sum(CASE [aboard] WHEN NULL THEN 0 ELSE [aboard]-coalesce([fatalities],0) END) as "Survivers"'
  );

  // apply visual title and save it
  cy.get(CSSselector.visual_title_specific(visualNumber))
    .click()
    .type(visualTitle);
  cy.get(CSSselector.refresh_visual).click();
  cy.wait('@visualUpdated')
    .its('response.statusCode')
    .should('eq', 200);

  cy.get(CSSselector.save_visual).click();

  cy.wait('@visualSubmitted')
    .its('response.statusCode')
    .should('eq', 200);
}

export function editTableVSToSQLQueryVS(
  visualNumber: number,
  visualTitle: string,
  keepOpen: boolean
) {
  cy.intercept('POST', '/arc/reports/report/*').as('visualSubmitted');

  cy.get(CSSselector.visual_edit_specific(visualNumber)).click({
    force: true,
  });
  cy.get(CSSselector.sql_visual).click();
  // let trimmed_query
  // cy.get(objects.sql_visual_textbox).then(($query) => {
  //   trimmed_query = $query.text().replace(/(\r\n\t|\n|\r\t)/gm,"")
  // })
  // console.log(trimmed_query)
  // cy.get(objects.sql_visual_textbox).clear()
  // cy.get(objects.sql_visual_textbox).type(trimmed_query)
  cy.wait(1000);
  cy.get(CSSselector.visual_title_specific(visualNumber))
    .click()
    .type(visualTitle);
  // VIZ-1025: SQL visual no longer has a refresh button
  // cy.get(CSSselector.refresh_visual).click();
  cy.get(CSSselector.save_visual).click();
  cy.wait('@visualSubmitted')
    .its('response.statusCode')
    .should('eq', 200);
}
