import * as dashboard from 'lib/dashboard';
import { Visual, VisualType } from 'lib/dashboard/dashboards_interfaces';
import { getVersion, modalClick } from 'lib/functions';
import * as visual from 'lib/visual';
import { CSSselector } from 'locators/common/CSSselector';

// if fieldDetail.field_aggregate is undefined
// we are removing the aggregate function
// from the shelf aggregate shelf element
export function addShelfColumn(fieldDetail: any) {
  const functionName = 'addShelfColumn';
  cy.log(`${functionName} - started`);
  let colNames = fieldDetail.field_name;
  const shelf = fieldDetail.field_shelf;

  const shelfDropdownLocator = `[data-test-id="${shelf}-app-shelf-item"] [title]:last`;
  const shelfLocator = `.dash-settings-content [data-test-id="${shelf}_list"]`;

  if (typeof colNames === typeof '') {
    colNames = [colNames];
  }
  // Using cy.click directly will activate the properties shelf if
  // there is an existing element in the shelf. Instead, use jquery to
  // send the click event to the shelf
  cy.get(shelfLocator).click('top');

  if (colNames[0] === 'Record Count') {
    cy.get(
      "#visual-data-panel .app-params [data-test-id='selection-field-item'][data-original-title='sum(1)']"
    ).click();

    return;
  }
  if (colNames[0] === 'Segment') {
    cy.get(
      "#visual-data-panel .app-params [data-test-id='selection-field-item'][data-original-title='']"
    ).click();
    return;
  }

  const sels = colNames.map((colName: string) => {
    // Template issue.
    // Used "parent child" selector to differentiate the one we need and the one in template
    return `[data-test-id="visual-data-panel"] [data-test-id="selection-field-item"][data-test-item="${colName}"]`;
  });

  const sel = sels.join(', ');
  cy.get(sel)
    .scrollIntoView()
    .click();

  if (fieldDetail.hasOwnProperty('field_aggregate')) {
    // add field properties
    cy.get(shelfDropdownLocator).click();
    // Template issue.
    // Used "parent child" selector to differentiate the one we need and the one in template
    cy.get(
      '[data-test-id="properties-panel"] [data-test-id="properties-menu-aggregates-btn"]'
    ).click();
    // unselect the selected aggregate
    cy.get('[data-test-id="properties-menu"]').then(($li) => {
      if (
        $li.find(
          '[data-test-id="aggregate-functions"].open [data-test-id="aggregate-function"].selected'
        ).length
      ) {
        cy.get(
          '[data-test-id="aggregate-functions"].open [data-test-id="aggregate-function"].selected'
        ).click();
      }
    });
    if (fieldDetail.field_aggregate !== undefined) {
      cy.get('[data-test-id="aggregate-functions"].open')
        .contains(fieldDetail.field_aggregate)
        .click();

      cy.get(shelfDropdownLocator).click();

      cy.get(shelfLocator).should(
        'contain',
        fieldDetail.field_aggregate.substring(0, 2).toLowerCase()
      );
      // verify
      cy.get(shelfLocator).should('contain', colNames);
    }
  }

  if (fieldDetail.hasOwnProperty('sort')) {
    // add field properties
    cy.get(shelfDropdownLocator).click();

    // Template issue.
    // Used "parent child" selector to differentiate the one we need and the one in template
    cy.get(
      '[data-test-id="properties-panel"] [data-test-id="properties-menu-order-functions-btn"]'
    ).click();
    cy.get('[data-test-id="order-functions"].has-submenu.open')
      .contains(fieldDetail.sort)
      .click();
    // can add an sql expression too cy.verifySQLExpression('sum(coalesce([fatalities],0)) as "Survivers"')

    cy.get(shelfDropdownLocator).click();
    // verify
    cy.get(shelfLocator).should('contain', colNames);
  }

  if (fieldDetail.hasOwnProperty('alias')) {
    // add field properties
    cy.get(shelfDropdownLocator).click();

    // Template issue.
    // Used "parent child" selector to differentiate the one we need and the one in template
    cy.get(
      '[data-test-id="properties-panel"] [data-test-id="properties-menu-alias-functions-btn"]'
    ).click();

    // Template issue.
    // Used "parent child" selector to differentiate the one we need and the one in template
    cy.get('[data-test-id="properties-panel"] [data-test-id="alias-functions-alias-text"]').type(
      fieldDetail.alias
    );
    // Template issue.
    // Used "parent child" selector to differentiate the one we need and the one in template
    cy.get(
      '[data-test-id="properties-panel"] [data-test-id="properties-menu-alias-functions-btn"]'
    ).click();

    cy.get(shelfDropdownLocator).click();

    cy.get(shelfLocator).should('contain', fieldDetail.alias);
  }

  // try values as hasOwn
  if (
    fieldDetail.hasOwnProperty('field_pick_filter') &&
    fieldDetail.field_pick_filter.hasOwnProperty('values')
  ) {
    cy.get(shelfDropdownLocator).click();

    // Template issue.
    // Used "parent child" selector to differentiate the one we need and the one in template
    cy.get(
      '[data-test-id="properties-panel"] [data-test-id="properties-menu-select-values"]'
    ).click();

    // wait for the dialog to appear (well, for the OK button):
    cy.get('[data-test-id="filter-modal-apply-btn"]:visible', { timeout: 30000 });

    // now wait for the loading indicator to disappear:
    cy.get('#filter-modal .modal-body-loading:hidden', { timeout: 30000 });

    for (const filterValues of fieldDetail.field_pick_filter.values) {
      cy.get(`[data-test-id="filter-input-checkbox-${filterValues}"]`).click();
    }

    modalClick('[data-test-id="filter-modal-apply-btn"]');

    // wait for the dialog to disappear
    cy.get('[data-test-id="filter-modal-apply-btn"]').should('not.exist');

    cy.get(shelfDropdownLocator).click();
    // verify
    cy.get(shelfLocator).should('contain', colNames);
  }

  if (fieldDetail.hasOwnProperty('field_expression')) {
    cy.get(shelfDropdownLocator).click();

    visual.verifySQLExpression(fieldDetail.field_expression);

    cy.get(shelfDropdownLocator).click();
  }

  if (fieldDetail.hasOwnProperty('change')) {
    // add field properties
    cy.get(shelfDropdownLocator).click();

    cy.get('#properties-panel ul.properties-menu.noselect li')
      .contains('Change Type')
      .click();
    // unselect the selected
    cy.get('#properties-panel ul.properties-menu.noselect').then(($li) => {
      if ($li.find('li.open li.selected').length) {
        cy.get('#properties-panel .properties-menu.noselect li.open li.selected').click();
      }
    });
    cy.get('#properties-panel ul.properties-menu.noselect li.open')
      .contains(fieldDetail.change)
      .click();
  }
  cy.log(`${functionName} - finished`);
}

export function editVisual(visualNumber: number, visualDetail: Visual) {
  cy.wait(3000);
  editSpecificVisual(visualNumber);
  // Add the condition to open the visual icons list
  cy.get(CSSselector.visual_title_specific(visualNumber)).invoke('text', visualDetail.visual_title);

  if (visualDetail.visual_subtitle) {
    cy.get(CSSselector.visual_subtitle_specific(visualNumber)).invoke(
      'text',
      visualDetail.visual_subtitle
    );
  }
  dashboard.saveDashboard();

  cy.get('[data-test-id="selected-viz-section"]').should('be.visible');
  cy.get('[data-test-id="selected-viz-section"]').then(($el) => {
    if ($el.hasClass('is-open')) {
      cy.wrap($el).click();
    }
  });

  cy.get('[data-test-id="selected-viz-section"]').should('have.class', 'is-closed');
  cy.get(`[data-test-id="${visualDetail.visual_type}"]`).click();
  // wait for render after the selection of the visual type
  // cy.wait('1000');
  // let fieldnum = 0;
  // if (visualDetail.data.hasOwnProperty('fields')) {
  //   for (const field in visualDetail.data.fields) {
  //     const fieldItem = visualDetail.data.fields[fieldnum];
  //     addShelfColumn(fieldItem);
  //     fieldnum++;
  //   }
  // }

  // if visual is extention type it wont have any shelves to add just text box
  const isExtensionVisual =
    visualDetail.hasOwnProperty('param_extension_text_field') &&
    visualDetail.param_extension_text_field;
  if (isExtensionVisual) {
    cy.get('[data-test-id="app-shelves"] [data-test-id="viz-html-view"]').click();

    // Change to now use ace editor
    cy.get('.super-modal #chart-container')
      .find('.ace_editor')
      .then(($elem) => {
        cy.window()
          .its('ace' as any)
          .invoke('edit', $elem.attr('id'))
          .invoke('setValue', visualDetail.param_extension_text_field);
      });

    // Give ace.js some time to finish processing, otherwise we get a getTokens exception
    // from that library
    cy.wait(2000);

    cy.get('[data-test-id="super-modal"] [data-test-id="apply-button"]').click();
    cy.get('[data-test-id="super-modal"]').should('not.be.visible', 1000);
  }

  if (visualDetail.hasOwnProperty('data') && visualDetail.data?.hasOwnProperty('fields')) {
    if (visualDetail.data.limit) {
      cy.get(dashboard.locators.limit)
        .clear()
        .invoke('val', visualDetail.data.limit);
    }

    for (const field of visualDetail.data.fields) {
      // const fieldItem = visualDetail.data.fields[fieldnum];
      addShelfColumn(field);
      // fieldnum++;
    }

    if (visualDetail.data.isExplicit) {
      cy.get(dashboard.locators.settings_tab).trigger('click');
      cy.wait(1500);
      cy.get(dashboard.locators.explicit_data_checkbox).check();
    }
  }

  const isSqlVisual = visualDetail.visual_type === VisualType.sql;
  const hasRefreshButton = !(isExtensionVisual || isSqlVisual);
  if (hasRefreshButton) {
    cy.get(CSSselector.refresh_visual).click();
    cy.get('.arc-viz-container [data-test-id="loading-bar"]', { timeout: 60000 }).should(
      'be.not.visible'
    );
  }

  dashboard.saveDashboard();

  // apply visual title and save it
}

/**
 * In the edit dashboard screen
 * Creates a new sheet if no sheet called Sheet 1 is present.
 * Else it renames the Sheet 1 (which is the default sheet)
 *
 * @export
 * @param {*} sheetDetail
 */
export function createNewSheet(sheetDetail: any) {
  const sheetLocator = `[data-test-id="sheet-number-${sheetDetail.sheet_number}"]`;

  cy.get('body').then(($body) => {
    // synchronously query from body
    // to find if the expired license banner is shown
    if ($body.find('#footer-banner[style="display: block;"]').length) {
      cy.get('#footer-banner > a.banner-close-btn').click();
    }
    if (sheetDetail.sheet_number !== 1) {
      cy.get('[data-test-id="dash-sheet-btn"]').click();
    }
  });
  cy.get(sheetLocator);
  cy.get(`[data-test-id="sheet-number-${sheetDetail.sheet_number}-dropdown-toggle"]`).click();
  cy.get(`[data-test-id="sheet-number-${sheetDetail.sheet_number}-sheet-rename"]`).click();
  cy.get(`[data-test-id="sheet-number-${sheetDetail.sheet_number}-sheet-title-div"]`).should(
    'have.attr',
    'contenteditable',
    'true'
  );
  cy.get(`[data-test-id="sheet-number-${sheetDetail.sheet_number}-sheet-title-div"]`).then(
    ($sheetTab) => {
      $sheetTab.text(sheetDetail.sheet_name);
    }
  );
}
/**
 *
 *
 * @export
 * @param {boolean} discard if false then save and close visual edit screen
 * else close visual edit sceen without saving
 */
export function closeVisual(discard: boolean) {
  if (!discard) {
    saveVisual();
  }
}

export function saveVisual() {
  dashboard.refreshVisual();
  dashboard.saveDashboard();
}
/**
 * emulates the behaviour of clicking the pencil icon of a visual
 * in a dashboard to open it for editing.
 * @param index : starts form 1 for the first visual in the dashboard
 * @export
 */
export function editSpecificVisual(index: number) {
  const fuctionName = 'editSpecificVisual';
  cy.log(`${fuctionName} - started`);

  // no way to make make the gear icon visible unless using
  // remote Chrome debugging (we need :hover pseudo class)
  // https://github.com/cypress-io/cypress-example-recipes/tree/master/examples/fundamentals__chrome-remote-debugging
  cy.get(CSSselector.visual_edit_specific(index)).click();

  cy.wait(1000); // the UI seems to flicker (side tab opened twice)

  cy.log(`${fuctionName} - finished`);
}

export function openSection(tab: string, section: string) {
  let tabLocator = '';
  if (tab === 'dashboardSettings') {
    cy.log(`Expanding the dashboard ${section} settings`);

    // Template issue.
    // Used "parent child" selector to differentiate tabname in dashboard and visual settings sidebar.
    // Tab is created from a template that used in many places.
    tabLocator = `[data-test-id='dashboard-settings-sidebar'] [data-test-id="visual-style-option"][data-test-item="${section}"]`;
  } else {
    cy.log(`Expanding the ${section} ${tab}`);

    // Template issue.
    // Used "parent child" selector to differentiate tabname in dashboard and visual settings sidebar.
    // Tab is created from a template that used in many places.
    tabLocator = `[data-test-id='dashboard-visual-${tab}-sidebar'] [data-test-id="visual-style-option"][data-test-item="${section}"]`;
  }
  chooseTab(tab);
  // click it twice so we can force the UI to get open or closed state.
  cy.get(tabLocator).click();
  cy.wait(1000);
  cy.get(tabLocator).click();
  cy.wait(1000);
  cy.get(tabLocator)
    .parent()
    .then(($el) => {
      const cls = $el.prop('class');
      cy.log('class:' + cls);
      if (cls.indexOf('closed') >= 0) {
        cy.log('Expanding the menu');
        cy.get(tabLocator).click();
      }
    });
}

export function chooseTab(tab: string) {
  let tabSelector = '';
  let tabPanelSelector = '';
  switch (tab) {
    case 'settings':
      tabSelector = '[data-test-id="visual-settings-tab-a"]';
      tabPanelSelector = '[data-test-id="visual-settings-tab-header-expand"]';
      break;
    case 'style':
      tabSelector = '[data-test-id="visual-style-tab-a"]';
      tabPanelSelector = '[data-test-id="visual-style-tab-header-expand"]';
      break;
    case 'dashboardSettings':
      tabSelector = '[data-test-id="dash-settings-tab-a"]';
      tabPanelSelector = '[data-test-id="dash-settings-tab-header-expand"]';
      break;
  }

  cy.get(tabSelector).then(($tab) => {
    // Hack to currently handle divergence between the two main version!
    getVersion().then((versionNumber) => {
      const isAlreadyVisible =
        (versionNumber === '6' && $tab.parent().hasClass('active')) ||
        (Number(versionNumber) > 6 && $tab.hasClass('active'));
      if (isAlreadyVisible) {
        return;
      }
      cy.wrap($tab).click();

      if (tabPanelSelector !== '') {
        cy.get(tabPanelSelector)
          .parent()
          .then(($el) => {
            const cls = $el.prop('class');
            console.log('class:' + cls);
            if (cls.indexOf('closed') >= 0) {
              console.log('Expanding the menu');
              cy.get(tabPanelSelector)
                // .parent()
                .click();
            }
          });
      }
    });
  });
}

/**
 * Get the visual with the given title.
 */
export function getVisual(visualTitle: string) {
  return cy
    .get(visual.locators.viz_title_containing(visualTitle))
    .closest(visual.locators.viz_container);
}

/**
 * Get the visual content without the title.
 */
export function getVisualContent(visualTitle: string) {
  return getVisual(visualTitle).find(visual.locators.viz_chart);
}

/**
 * Get the cells (td elements) in the specified table visual.
 * Column numbers start at 1.
 */
export function getTableVisualColumn(visualTitle: string, columnNumber: number) {
  return getVisual(visualTitle).find(`table tbody tr td:nth-child(${columnNumber})`);
}
