import * as dashboards from 'lib/dashboard';
import * as visual_functions from 'lib/visual';
import { locators } from 'lib/visual/locators';
// Settings color tab functions

export function setTransitionDelay(delay: string) {
  visual_functions.openSection('settings', 'Marks');
  cy.get("#dashboard-visual-settings-sidebar input[name='TRANSITION_DELAY']")
    .scrollIntoView()
    .clear()
    .type(delay);
  dashboards.closeVisual(false);
}

export function separatetrendlines(action: string) {
  visual_functions.openSection('settings', 'Marks');
  if (action === 'checked') {
    cy.get("input[name='TRENDLINE_COLOR']")
      .scrollIntoView()
      .check();
  } else {
    cy.get("input[name='TRENDLINE_COLOR']")
      .scrollIntoView()
      .uncheck();
  }
  dashboards.closeVisual(false);
}

export function enableDisableAutoplayAnimation(action: string) {
  visual_functions.openSection('settings', 'Marks');
  if (action === 'checked') {
    cy.get("input[name='AUTOPLAY']")
      .scrollIntoView()
      .check();
  } else {
    cy.get("input[name='AUTOPLAY']")
      .scrollIntoView()
      .uncheck();
  }
  dashboards.closeVisual(false);
}

export function AxisScale(axis: string, action: string) {
  visual_functions.openSection('settings', 'Axes');
  if (axis === 'x') {
    if (action === 'log10') {
      cy.get(
        "#dashboard-visual-settings-sidebar div[data-setting-id='scale_type_x'] input[value='LOG10']"
      )
        .scrollIntoView()
        .check();
    } else {
      cy.get(
        "#dashboard-visual-settings-sidebar div[data-setting-id='scale_type_x'] input[value='LINEAR']"
      )
        .scrollIntoView()
        .check();
    }
  } else {
    if (action === 'log10') {
      cy.get(
        "#dashboard-visual-settings-sidebar div[data-setting-id='scale_type_y'] input[value='LOG10']"
      )
        .scrollIntoView()
        .check();
    } else {
      cy.get(
        "#dashboard-visual-settings-sidebar div[data-setting-id='scale_type_y'] input[value='LINEAR']"
      )
        .scrollIntoView()
        .check();
    }
  }
  dashboards.closeVisual(false);
}

export function AxisContinuity(axis: string, action: string) {
  visual_functions.openSection('settings', 'Axes');
  if (axis === 'x') {
    if (action === 'checked') {
      cy.get("#dashboard-visual-settings-sidebar input[name='MAKE_X_CONTINUOUS']")
        .scrollIntoView()
        .check();
    } else {
      cy.get("#dashboard-visual-settings-sidebar input[name='MAKE_X_CONTINUOUS']")
        .scrollIntoView()
        .uncheck();
    }
  } else {
    if (action === 'checked') {
      cy.get("#dashboard-visual-settings-sidebar input[name='MAKE_Y_CONTINUOUS']")
        .scrollIntoView()
        .check();
    } else {
      cy.get("#dashboard-visual-settings-sidebar input[name='MAKE_Y_CONTINUOUS']")
        .scrollIntoView()
        .uncheck();
    }
  }
  dashboards.closeVisual(false);
}

export function setMinMaxScale(axis: string, scale: string, value: string) {
  visual_functions.openSection('settings', 'Axes');
  if (axis === 'x') {
    if (scale === 'min') {
      cy.get("#dashboard-visual-settings-sidebar input[name='MIN_X_AXIS']")
        .scrollIntoView()
        .clear()
        .type(value);
    } else {
      cy.get("#dashboard-visual-settings-sidebar input[name='MAX_X_AXIS']")
        .scrollIntoView()
        .clear()
        .type(value);
    }
  } else {
    if (scale === 'min') {
      cy.get("#dashboard-visual-settings-sidebar input[name='MIN_Y_AXIS']")
        .scrollIntoView()
        .clear()
        .type(value);
    } else {
      cy.get("#dashboard-visual-settings-sidebar input[name='MAX_Y_AXIS']")
        .scrollIntoView()
        .clear()
        .type(value);
    }
  }
  dashboards.closeVisual(false);
}

const enum Axis {
  x,
  y,
}

function numbersInAscendingOrder(numbers: number[]) {
  const sortedArray = [...numbers].sort();
  return JSON.stringify(numbers) === JSON.stringify(sortedArray);
}

function numbersInDescendingArder(numbers: number[]) {
  const sortedAndReversedArray = [...numbers].sort().reverse();
  return JSON.stringify(numbers) === JSON.stringify(sortedAndReversedArray);
}

function testReverseAxis(axis: Axis) {
  const axisCheckLocators = {
    axisValues: axis === Axis.y ? locators.y_axis_value_g : locators.x_axis_value_g,
    reverseAxisSetting:
      axis === Axis.y
        ? '#dashboard-visual-settings-sidebar input[name="REVERSE_SCALE_Y"]'
        : '#dashboard-visual-settings-sidebar input[name="REVERSE_SCALE_X"]',
  };

  cy.get(axisCheckLocators.axisValues).then(($values) => {
    visual_functions.openSection('settings', 'Axes');
    cy.get(axisCheckLocators.reverseAxisSetting)
      .scrollIntoView()
      .check();
    dashboards.closeVisual(false);
    cy.get(axisCheckLocators.axisValues).then(($valuesReversed) => {
      // the array of content is the same in terms of text
      expect($valuesReversed.text()).to.eq($values.text());
      const transformValuesOfReversed: number[][] = [];
      const transformValues: number[][] = [];

      for (let i = 0; i < $valuesReversed.length; i++) {
        const positionOnReversed = Cypress.$($valuesReversed[i])
          .attr('transform')
          ?.match(/\((.*)\)/)
          ?.pop()
          ?.split(',')
          .map((x) => parseFloat(x));
        if (positionOnReversed) {
          transformValuesOfReversed.push(positionOnReversed);
        }
        const positionOnOirignal = Cypress.$($values[$values.length - (i + 1)])
          .attr('transform')
          ?.match(/\((.*)\)/)
          ?.pop()
          ?.split(',')
          .map((x) => parseFloat(x));
        if (positionOnOirignal) {
          transformValues.push(positionOnOirignal);
        }
      }

      const positionToCompare = transformValues.map((xy) => xy[axis === Axis.x ? 0 : 1]);
      const reversedPositionToCompare = transformValuesOfReversed.map(
        (xy) => xy[axis === Axis.x ? 0 : 1]
      );
      expect(positionToCompare.length).to.greaterThan(0);
      expect(positionToCompare.length).to.equal(reversedPositionToCompare.length);

      if (numbersInAscendingOrder(positionToCompare)) {
        expect(numbersInDescendingArder(reversedPositionToCompare)).to.be.true;
      }
      if (numbersInDescendingArder(positionToCompare)) {
        expect(numbersInAscendingOrder(reversedPositionToCompare)).to.be.true;
      }
    });
  });
}

export function testReverseYAxisScale() {
  testReverseAxis(Axis.y);
}

export function testReverseXAxisScale() {
  testReverseAxis(Axis.y);
}

// Data tab functions

export function switchVisualDataSet(datasetName: string) {
  dashboards.chooseVisualTab('data');
  cy.get('[data-test-id="app-dataset-switch"]').click();
  cy.get(locators.dataConnection.selectDataset)
    .click()
    .find('input')
    .type(datasetName + '{enter}', { force: true });

  cy.get('.switch-dataset-apply ').click();
  dashboards.closeVisual(false);
}

export function setNumberofDistinctDimensions(count: string) {
  visual_functions.openSection('settings', 'Data');
  cy.get('#dashboard-visual-settings-sidebar input[name="numberDimensions"]')
    .scrollIntoView()
    .clear()
    .type(count);
}

// This function will set the segment from Segment field in visual edit mode
export function setSegment(segmentName: string) {
  cy.get("#app-shelves .selection_field[title=''][data-original-title='']").click({ force: true });
  cy.get(locators.visual.segment_picker).click();
  cy.get(locators.visual.segment_modal).should('be.visible');

  cy.get(locators.visual.segment_values)
    .first()
    .should('be.visible');

  cy.get(locators.visual.segment_values).each(($value) => {
    if ($value.text().trim() === segmentName) {
      cy.wrap($value)
        .prev()
        .check();
      cy.get(locators.visual.filter_segment_save).click();
    }
  });
  dashboards.saveVisual();
}
