const canvasTypes: string[] = [
  '.chart-icon-trellis-areas',
  '.chart-icon-combo',
  '.chart-icon-trellis-bars',
  '.chart-icon-trellis-groupedbars',
  '.chart-icon-calendar-heatmap',
  '.chart-icon-histogram',
  '.chart-icon-trellis-lines',
  '.chart-icon-pie',
  '.chart-icon-scatter',
  '.chart-icon-packed-bubbles',
  '.chart-icon-corelation',
  '.chart-icon-calendar-heatmap',
];

/**
 * Compare the canvas context hash of a visual at the position with an expected hash string
 * Visual is identified by its position on the dashboard as identified by counting
 * from left to right starting from top and going to the end of page.
 * @export
 * @param {number} visualPosition Starts with 1 for first visual on dash
 * @param {string} hash
 * @param {boolean} trellis set true if visual has trellis
 */
export function verifyVisualHash(
  visualPosition: number,
  hash: string | string[],
  IS: string[],
  visualType: string,
  trellis?: boolean
) {
  const functionName = 'verifyVisualHash';
  cy.log(`${functionName} - started`);
  const visualLocator = `[data-test-id="main-app-arc-app-container"] [data-test-id="visual-list-item"]:nth-child(${visualPosition})`;

  cy.get(visualLocator).should('be.visible');
  cy.document().then(($doc) => {
    if (!$doc) {
      return;
    }
    // handle calendar heatmap type visual separately
    if (visualType === canvasTypes[4] || trellis) {
      // if (typeof hash === typeof '') {
      //  hash = [hash] as string[];
      // }
      let trellisIndex = 1;
      while (trellisIndex - 1 !== hash.length) {
        const canvas = visualLocator + ' canvas:nth-child(' + trellisIndex + ')';
        const context = ($doc.querySelector(canvas) as HTMLCanvasElement).getContext('2d');
        console.log(
          ' expected : ' + hash + 'got' + hash[trellisIndex - 1] + 'in = ' + trellisIndex
        );

        expect((context as any).hash({ loose: true })).to.eq(hash[trellisIndex - 1]);
        if (IS) {
          expect((context as any).stack({ loose: true })).to.deep.eq(IS[trellisIndex - 1]);
        }

        ++trellisIndex;
      }
    }
    // all other canvas visuals
    else {
      const canvasLocator = `${visualLocator} canvas.maincanvas`;
      const context = ($doc.querySelector(canvasLocator) as HTMLCanvasElement).getContext('2d');
      console.log(' expected : ' + hash);
      expect((context as any).hash({ loose: true })).to.eq(hash);
      if (IS) {
        expect((context as any).stack({ loose: true })).to.deep.eq(IS);
      }
    }
  });
  cy.log(`${functionName} - finished`);
}

export function selectSheetToVerify(sheetDetail: any) {
  const functionName = 'selectSheetToVerify';
  cy.log(`${functionName} - started`);
  const sheetLocator = `[data-test-id="sheet-number-${sheetDetail.sheet_number}-sheet-title-div"]`;
  const sheetHandle = `[data-test-id="sheet-number-${sheetDetail.sheet_number}"]`;

  cy.get('body').then(($body) => {
    cy.log(`${functionName} - synchronously query from body`);
    // synchronously query from body
    // to find if the expired license banner is shown
    if ($body.find('#footer-banner[style="display: block;"]').length) {
      cy.get('#footer-banner > a.banner-close-btn').click();
    }
    // assert that sheetname that got saved is the same provided and click on in
    cy.log(
      `${functionName} - assert that sheetname that got saved is the same provided and click on in`
    );
    cy.get(sheetLocator)
      .contains(sheetDetail.sheet_name)
      .click();
    // assert sheet is active
    cy.log(`${functionName} - assert sheet is active`);
    cy.get(sheetHandle)
      .should('contain', sheetDetail.sheet_name)
      .should('have.class', 'active');
  });

  cy.log(`${functionName} - finished`);
}

/**
 * used to verify hash on every visual in all sheets
 * after creating visuals from createDashboard() function
 *
 * @export
 * @param {*} dashInfo
 */
export function verifyVisual(dashInfo: any) {
  const functionName = 'verifyVisual';
  cy.log(`${functionName} - started`);
  let sheet: any;
  for (sheet of dashInfo.sheets) {
    if (dashInfo.hasOwnProperty('sheets')) {
      selectSheetToVerify(sheet);
      let visual: any;
      let visualNumber: number = 1;
      for (visual of sheet.visuals) {
        if (sheet.hasOwnProperty('visuals')) {
          // verify only the canvas visuals
          if (canvasTypes.indexOf(visual.visual_type) !== -1) {
            verifyVisualHash(
              visualNumber,
              visual.hash,
              visual.is,
              visual.visual_type,
              visual.trellis
            );
          }
          visualNumber++;
        }
      }
    }
    cy.log(`${functionName} - finished`);
  }
}
