import * as apps from 'lib/apps';
import * as workspace from 'lib/workspace';
import { CSSselector } from 'locators/common/CSSselector';
import * as functions from '../functions';
import { modalClick } from '../functions';

export function createWorkSpace(workspaceName: string, workspaceDesc?: string) {
  functions.navigate_to_visuals();
  cy.get(workspace.locators.create_workspace).click();
  cy.get(CSSselector.workspace.wsName)
    .clear()
    .type(workspaceName);
  if (workspaceDesc != null) {
    cy.get(CSSselector.workspace_regression.wsdesc)
      .clear()
      .type(workspaceDesc);
  }
  cy.intercept('POST', '/arc/workspace/workspace').as('workspaceSave');

  modalClick(CSSselector.workspace.wsNameSave);
  cy.wait('@workspaceSave');
}

//   verifies the dashbaord of public workspace.
// If appname is specified at the time of calling this, it will verifies the app in public as well.
export function verifyUserDefinedWorkspace(
  WSName: string,
  dashname: string,
  shouldContain: boolean,
  appName?: string
) {
  // Set aliases to validate or wait on
  functions.navigate_to_visuals();
  cy.get(workspace.locators.user_defined_ws.replace('wsTitle', '' + WSName + '')).click();
  cy.get(workspace.locators.dashboard_radio_btn).click();
  if (shouldContain === true) {
    cy.contains(dashname);
    if (appName) {
      cy.get(workspace.locators.app_radio_btn).click();
      cy.contains(appName);
    }
  } else {
    cy.get('body').should('not.contain', dashname);
    if (appName) {
      cy.get(workspace.locators.app_radio_btn).click();
      cy.get('body').should('not.contain', appName);
    }
  }
}

/**
 * simply save the workspace.
 * Edit Workspace Modal must be open to execute this function
 * @export
 */
export function saveWorkSpace() {
  cy.intercept('POST', '/arc/workspace/workspace').as('workspaceSave');
  cy.get(CSSselector.workspace.wsNameSave).click();
  cy.wait('@workspaceSave');
}

/**
 * delete the workspace.
 * @export
 * @param workspaceName {string}: name of workspace
 */
export function deleteWorkSpace(workspaceName: string) {
  functions.navigate_to_visuals();
  cy.get(`[data-test-id="workspaceName"] [datat-test-id="edit-workspace"]`).click();
  cy.get(CSSselector.workspace_regression.wsdeletebtn).click();

  cy.contains(
    'To confirm, type the word DELETE in the field below and then click the Delete button.'
  );

  cy.get(apps.locators.appdeleteconfirmationtxt).type('DELETE');
  cy.intercept('DELETE', '/arc/workspace/workspace/*').as('workspaceDeleted');
  cy.get(apps.locators.appdeleteconfirmationbtn).click();
  cy.wait('@workspaceDeleted')
    .its('response.statusCode')
    .should('be.eq', 200);
  cy.get(`[data-test-id="${workspaceName}"]`).should('not.exist');
}

/**
 * this function verifies the visibility of workspace on the visuals page by selecting first dashboard and clicking on 'Move To Workspace' dropdown.
 * @export
 * @param workspaceName {string}: name of workspace
 * @param shouldVisible: visibility of workspace in 'Move To Workspace' dropdown
 */
export function verifyWorkSpaceVisibility(workspaceName: string, shouldVisible: boolean) {
  functions.navigate_to_visuals();
  // cy.wait(1000);
  cy.get(CSSselector.Visuals_all).click();
  cy.get(CSSselector.workspace_regression.workspace_menulist)
    .find('div[title="' + workspaceName + '"]')
    .should('be.visible');
  // cy.get(arcvizSelectors.workspace_regression.workspace_menulist).find('span[title="'+ workspaceName + '"] + span').should('have.class', 'arc-invisible');
  cy.get('.app-grid-list li > span')
    .first()
    .click({ force: true });
  cy.get(workspace.locators.move_to_ws_button)
    .should('contain', 'Move to Workspace')
    .click({ force: true });
  if (shouldVisible) {
    cy.get('div.popover')
      .should('be.visible')
      .should('contain', workspaceName);
  } else {
    cy.get('div.popover')
      .should('be.visible')
      .should('not.contain', workspaceName);
    cy.get(workspace.locators.cancel_workspace_move).click({ force: true });
  }
}
