import os
from arcweb.settings import *

BASEDIR = os.environ.get("BASEDIR", "/var/lib/arcadia")

DEBUG = True
JAVASCRIPT_DEBUG = True
RUN_FROM_SOURCE_DEBUG = True
ALLOWED_HOSTS = ['*']

DATABASES['default']['NAME'] = os.path.join(BASEDIR,"arcviz.db")


CACHES = {
  'default': {
    # To disable caching
    #'BACKEND': 'django.core.cache.backends.dummy.DummyCache',
     'BACKEND': 'django.core.cache.backends.filebased.FileBasedCache',
     'LOCATION': os.path.join(BASEDIR,'django_cache'),
     'TIMEOUT': 3600,
     'OPTIONS': {
       'MAX_ENTRIES': 10000
     }
  }
}

INSTALLED_APPS = INSTALLED_APPS + ('trustedauth',)
AUTHENTICATION_BACKENDS = (
  'django.contrib.auth.backends.ModelBackend',
)

AUDIT_LOGFILE = os.path.join(BASEDIR, "querylog")
AUDIT_FILE_MAXBYTES = 10*1024*1024
ARCENGINE_VALIDATE_ACCESS = False

PHANTOMJS_PATH = "/opt/arcadia/lib/phantomjs/bin/phantomjs"

# Logging config
# Set the env variable "LOGGING_LEVEL" to control logging
# Specific loggers can be configured as well
# The default log level is WARNING. If DEBUG is True then it DEBUG.
# The default output is console.
# The default filename for file output is /tmp/arcviz.log
DEFAULT_LOGGING_LEVEL = "DEBUG" if DEBUG else "WARNING"
LOGGING_LEVEL = os.environ.get("LOGGING_LEVEL",DEFAULT_LOGGING_LEVEL)
LOGGING_LEVEL_ARCVIZ = os.environ.get("LOGGING_LEVEL_ARCVIZ", LOGGING_LEVEL)
LOGGING_LEVEL_DJANGO = os.environ.get("LOGGING_LEVEL_DJANGO", LOGGING_LEVEL)
LOGGING_LEVEL_DJANGO_REQUEST = os.environ.get("LOGGING_LEVEL_DJANGO_REQUEST", "WARNING")
LOGGING_LEVEL_DJANGO_DB = os.environ.get("LOGGING_LEVEL_DJANGO_DB", "WARNING")
LOGGING_LEVEL_LDAP = os.environ.get("LOGGING_LEVEL_LDAP", LOGGING_LEVEL)
#LOG_FILE = os.environ.get("LOG_FILE", "/tmp/arcviz.log")
LOG_FILE = os.path.join(BASEDIR,"arcviz.log")
# LOG_HANDLERS env var is comma separated list of handlers. "console" or "file"
LOG_HANDLERS = [ h.strip() for h in os.environ.get("LOG_HANDLERS", "console,file").split(',') ]

# Map the above into current logging configuration
ARCVIZ_LOGFILE = LOG_FILE
ARCVIZ_LOGLEVEL = LOGGING_LEVEL_ARCVIZ

if IS_MASTER and 'console' in LOG_HANDLERS:
  LOGGING['loggers']['arcweb']['handlers'].append('console')

if 'file' not in LOG_HANDLERS:
  ARCVIZ_LOGFILE = None

ADDITIONAL_LOGGERS.extend([('root', LOGGING_LEVEL),
                           ('django', LOGGING_LEVEL_DJANGO),
                           ('django.request', LOGGING_LEVEL_DJANGO_REQUEST),
                           ('django.db.backends', LOGGING_LEVEL_DJANGO_DB),
                           ('django_auth_ldap', LOGGING_LEVEL_LDAP)])

del BASEDIR
ADMIN_API_DEMO_LIST = ['*']
ADMIN_API_URL_LIST  = ['*']
DATA_API_ENABLED = True
ENABLE_DSREQ_PERF_DISPLAY = True
ENABLE_API_KEYS = True
ENFORCE_LICENSE = False

CUSTOM_EMAIL_TEMPLATE = True
DATA_API_ENABLED = True

INCLUDE_CANTEEN = True

