import * as adminapi from 'lib/adminapi';
import { BackendAPI } from 'lib/backend-api';
import * as functions from 'lib/functions';
import * as logins from 'lib/logins';

describe('Admin API.', () => {
  context('ARC-10812: Automate Smoke test with Admin API', () => {
    before(() => {
      logins.silent();
    });

    let userId: number;
    const newAdminName = 'adminAutomationTest';
    const newAdminPass = 'adminPass';

    it('1. Create API keys', () => {
      functions.navigate_to_home();
      functions.goToSiteAdminSettingsMenu('manage_api_keys');

      adminapi.createApiKey(true);
      adminapi.createApiKey();
      adminapi
        .createApiKey(true, true)
        .then((apiKey) => apiKey)
        .as('apiKey');

      BackendAPI.users
        .create({
          username: newAdminName,
          password: newAdminPass,
          is_superuser: true,
        })
        .then((user) => {
          userId = user.id;
          functions.navigate_to_home();
          functions.goToSiteAdminSettingsMenu('manage_user_and_group_site_admin');

          cy.get('div#users-table_wrapper td.user-table-name span')
            .contains(newAdminName)
            .should('exist');

          // 2.verify login api for new user
          logins.silent(newAdminName, newAdminPass);

          // 3.delete the newly created user and verify the name has gone from the list
          cy.get<string>('@apiKey').then((apiKey) => {
            cy.request({
              method: 'DELETE',
              url: `/arc/adminapi/v1/users/${userId}`,
              headers: {
                Authorization: `apikey ${apiKey}`,
              },
            });
          });

          logins.silent();
          functions.goToSiteAdminSettingsMenu('manage_user_and_group_site_admin');
          cy.get('div#users-table_wrapper td.user-table-name span')
            .contains(newAdminName)
            .should('not.exist');
        });
    });

    it('3. Delete API keys', () => {
      functions.navigate_to_home();
      functions.goToSiteAdminSettingsMenu('manage_api_keys');

      // Deleting all three keys

      adminapi.deleteApiKey();
      adminapi.deleteApiKey();
      adminapi.deleteApiKey();
    });
  });
});
