import * as av_functions from 'lib/analyticalview_functions';
import * as functions from 'lib/functions';
import { checkArcenginePresence } from 'lib/functions';
import * as logins from 'lib/logins';

const connectionName = 'arcengine_colo';
const dataset1 = 'Cereals';
const dataset2 = 'NBA Games';
const dataset3 = 'Iris';
const dataset4 = 'US State Populations Over Time';
const dataset5 = 'World Life Expectancy';

describe('Analytical View Test Suite.', () => {
  context('Analytical view tests', () => {
    before(function() {
      checkArcenginePresence(this);
      logins.silent();
    });
    afterEach(() => {
      cy.window().then(
        (win: Window) =>
          (win.onbeforeunload = () => {
            return;
          })
      );
    });

    it('Test 1: Delete All Analytical Views', () => {
      const testName = 'deleteAllAVs';
      console.log('Starting test ' + testName);
      av_functions.navigateToAnalyticalViewTab(connectionName);
      av_functions.deleteAnalyticalViewBulk();
      cy.contains('No Analytical Views have been created');
    });

    it('Test 2: Create Analytical Views', () => {
      const testName = 'createAVs';
      console.log('Starting test ' + testName);
      av_functions.createAnalyticalView(connectionName, dataset1, 'testav1');
      av_functions.createAnalyticalView(connectionName, dataset2, 'testav2');
      av_functions.createAnalyticalView(connectionName, dataset3, 'testav3');
      av_functions.createAnalyticalView(connectionName, dataset4, 'testav4');
      av_functions.navigateToAnalyticalViewTab(connectionName);
      av_functions.checkAnalyticalViewState(
        'main',
        'cereals',
        'main.testav1',
        'Needs initial refresh'
      );
      av_functions.checkAnalyticalViewState(
        'main',
        'nbagames',
        'main.testav2',
        'Needs initial refresh'
      );
      av_functions.checkAnalyticalViewState(
        'main',
        'iris',
        'main.testav3',
        'Needs initial refresh'
      );
      av_functions.checkAnalyticalViewState(
        'main',
        'census_pop',
        'main.testav4',
        'Needs initial refresh'
      );
    });

    it('Test 3: Bulk Refresh AVs', () => {
      const testName = 'bulkRefreshAV';
      console.log('Starting test ' + testName);
      av_functions.refreshAnalyticalViewBulk();
      av_functions.navigateToAnalyticalViewTab(connectionName);
      cy.wait(30000);
      av_functions.checkAnalyticalViewState('main', 'cereals', 'main.testav1', 'Up to date');
      av_functions.checkAnalyticalViewState('main', 'nbagames', 'main.testav2', 'Up to date');
      av_functions.checkAnalyticalViewState('main', 'iris', 'main.testav3', 'Up to date');
      av_functions.checkAnalyticalViewState('main', 'census_pop', 'main.testav4', 'Up to date');
    });

    it('Test 4: Individual AV Refresh', () => {
      const testName = 'SingleAVRefresh';
      console.log('Starting test ' + testName);
      av_functions.createAnalyticalView(connectionName, dataset5, 'testav5');
      av_functions.refreshAnalyticalView('main', 'world_life_expectancy', 'testav5');
      cy.wait(30000);
      av_functions.checkAnalyticalViewState(
        'main',
        'world_life_expectancy',
        'main.testav5',
        'Up to date'
      );
    });

    it('Test 5: Individual AV Delete', () => {
      const testName = 'SingleAVDelete';
      console.log('Starting test ' + testName);
      av_functions.navigateToAnalyticalViewTab(connectionName);
      av_functions.deleteAnalyticalView('main', 'world_life_expectancy', 'main.testav5');
    });
  });
});
