import { notifications } from 'fixtures/locators/notifications';
import * as apps from 'lib/apps';
import * as arctest from 'lib/arctest';

describe('App builder creation tests.', () => {
  // context: Test suite
  context('Creation tests', () => {
    // Set up before 'each test' below
    before(() => {
      arctest.logins.silent();
    });

    beforeEach(() => {
      cy.visit('/arc/apps/appedit');
    });

    // Note: To refer to a variable, use get().then() and put your code
    // accessing the variable within the then callback
    it('create new app, check that saving without name throws error', () => {
      cy.get(apps.locators.appsave).click();
      cy.get('.alert.alert-danger').should('be.visible');
      cy.get('.alert.alert-danger').should('contain', 'app name');
    });

    it('create app, name it, save, check failure because invalid subgroup', () => {
      cy.get(apps.locators.appname).type('New app name');
      cy.get(apps.locators.appsave).click();
      cy.get('.alert.alert-danger').should('be.visible');
      cy.get('.alert.alert-danger').should('contain', 'menu tab title');
    });

    it('create app, name it, add subgroup, name it, save, check success', () => {
      cy.get(apps.locators.appname).type('New app name');
      cy.get(apps.locators.menutitle_last)
        .click()
        .type('New subgroup');
      apps.saveApp();
      apps.deleteAppFromBuilder();
    });

    it('create app, name it, add 2 dbs to same subgroup, should fail', () => {
      cy.get(apps.locators.appname).type('New app name');
      cy.get(apps.locators.menutitle_last)
        .click()
        .type('New subgroup');
      apps.addSubgroupWithDashboards(1);
      apps.addDashboard('samples', 'World Life Expectancy', 'Life Expectancy Dashboard');
      apps.addDashboard('samples', 'World Life Expectancy', 'Life Expectancy Dashboard');
      cy.get('.alert.alert-danger').should('be.visible');
      cy.get('.alert.alert-danger').should('contain', 'already in the active');
    });

    it('create app, name it, add subgroup and db, add another, save, check for success ', () => {
      cy.get(apps.locators.appname).type('New app name');
      cy.get(apps.locators.menutitle_last)
        .click()
        .type('first subgroup');
      apps.addSubgroupWithDashboards(1);
      apps.addSubgroupWithDashboards(1);
      cy.get(apps.locators.appsave).click();
      cy.get(notifications.alert_success).should('contain', 'saved');
      cy.get(apps.locators.menusections).should('have.length', 3);
    });

    it('create app, name it, add subgroup and db, save it, reaname app, save and verify', () => {
      cy.get(apps.locators.appname).type('New app name{enter}');
      cy.get(apps.locators.menutitle_last)
        .click()
        .type('first subgroup{enter}');
      apps.addSubgroupWithDashboards(1);
      apps.saveApp();
      cy.get(apps.locators.menusections).should('have.length', 2);
      cy.get(apps.locators.appname)
        .type('x')
        .clear()
        .type('Renamed app name{enter}');
      apps.saveApp();
      cy.get(apps.locators.appname).should('have.value', 'Renamed app name');
    });

    it('create app, name it, add subgroup and db, renames, save, check for success ', () => {
      cy.get(apps.locators.appname).type('New app name');
      cy.get(apps.locators.menutitle_last)
        .click()
        .type('first subgroup');
      apps.addSubgroupWithDashboards(1);

      // Save and wait for success message to be dismissed
      apps.saveApp();

      cy.get(apps.locators.menutitle_last)
        .click()
        .type('something')
        .clear()
        .type('second subgroup');
      cy.get(apps.locators.appsave).click();
      cy.get(notifications.alert_success).should('contain', 'saved');
      cy.get(apps.locators.menusections).should('have.length', 2);
      cy.get(apps.locators.menutitle_last).should('contain', 'second subgroup');
    });

    it('create app, name it, remove subgroup, try saving, should fail', () => {
      cy.get(apps.locators.appname).type('New app name');
      cy.get(apps.locators.menusections_delete_last).click();
      apps.addDashboard('samples', 'World Life Expectancy', 'Life Expectancy Dashboard');
      cy.get('.alert.alert-danger').should('be.visible');
      cy.get('.alert.alert-danger').should('contain', 'add a Menu Tab');
    });
  });
});
