import * as apps from 'lib/apps';
import { logins } from 'lib/arctest';
import * as functions from 'lib/functions';

describe('App builder deletion tests.', () => {
  // Global variables to the tests
  let appUrl = 'appUrlNotFound';
  const appName = 'Sample App';
  // Set up before any tests are run
  before(() => {
    logins.silent();
    // Find existing 'Sample App''s id and save that
    // Change app_id value accordingly
    functions.getBuilderUrlFromAppName(appName).then((builderUrl) => {
      appUrl = builderUrl!;
    });
  });

  // context: Test suite
  context('App builder deletion tests', () => {
    // Set up before 'each test' below
    before(() => {
      logins.silent();
    });

    beforeEach(() => {
      // Note: When you want to wait for a page to load, add it to cy.visit,
      // but to make sure that the page is fully loaded (internal post/get
      // requests done), you need to a wait-for-route by using cy.intercept and
      // and cy.wait. Also remember to set up these routes before doing the
      // cy.visit
      cy.intercept('GET', '/arc/reports/appgroup/*').as('AppGetDone1');
      cy.intercept('GET', '/arc/reports/customcss').as('AppGetDone');
      cy.visit(appUrl);
      cy.wait('@AppGetDone');
      cy.wait('@AppGetDone1');
    });

    afterEach(() => {
      cy.window().then(
        (win: Window) =>
          (win.onbeforeunload = () => {
            return;
          })
      );
    });

    // Note: To refer to a variable, use get().then() and put your code
    // accessing the variable within the then callback
    it('add empty subgroup, delete subgroup, save and verify', () => {
      cy.get(apps.locators.menusections).then(($ams) => {
        const appMenuSectionsCount = $ams.length;
        apps.addSubgroupWithDashboards(0);
        apps.deleteLastSubgroup(true);
        cy.get('.app-menu-section').should('have.length', appMenuSectionsCount);
      });
    });

    it('add subgroup, add dashboard, delete subgroup, save and verify', () => {
      cy.get('.app-menu-section').then(($ams) => {
        const appMenuSectionsCount = $ams.length;

        apps.addSubgroupWithDashboards(1);

        apps.deleteLastSubgroup(true);

        cy.get(apps.locators.menusections).should('have.length', appMenuSectionsCount);
      });
    });

    it('add subgroup, add dashboard, save, delete subgroup, save and verify', () => {
      apps.addSubgroupWithDashboards(1);
      apps.saveApp();

      cy.get('.app-menu-section').then(($ams) => {
        const appMenuSectionsCount = $ams.length;

        apps.deleteLastSubgroup(true);

        cy.get(apps.locators.menusections).should('have.length', appMenuSectionsCount - 1);
      });
    });

    it('add subgroup, add dashboard, save, delete dashboard, save and verify', () => {
      apps.addSubgroupWithDashboards(1);
      apps.saveApp();

      cy.get('.app-menu-section').then(($ams) => {
        const appMenuSectionsCount = $ams.length;

        apps.deleteLastDashboard(true);

        cy.get(apps.locators.menusections).should('have.length', appMenuSectionsCount);
        cy.get(apps.locators.dashboards_in_last_submenu).should('have.length', 0);
        // reset app to what it was
        apps.deleteLastSubgroup(true);
      });
    });

    it('add subgroup, add 1 db, delete 1 db, save and verify', () => {
      cy.get('.app-menu-section').then(($ams) => {
        const appMenuSectionsCount = $ams.length;

        apps.addSubgroupWithDashboards(1);

        apps.deleteLastDashboard(true);

        cy.get(apps.locators.menusections).should('have.length', appMenuSectionsCount + 1);
        cy.get(apps.locators.dashboards_in_last_submenu).should('have.length', 0);

        apps.deleteLastSubgroup(true);
      });
    });

    it('add subgroup, add 2 dbs, delete 2 dbs, save and verify', () => {
      cy.get('.app-menu-section').then(($ams) => {
        const appMenuSectionsCount = $ams.length;

        apps.addSubgroupWithDashboards(2);
        apps.deleteLastDashboard(false);
        apps.deleteLastDashboard(true);
        cy.get(apps.locators.menusections).should('have.length', appMenuSectionsCount + 1);
        cy.get(apps.locators.dashboards_in_last_submenu).should('have.length', 0);
        apps.deleteLastSubgroup(true);
      });
    });

    it('add subgroup, add 2 dbs, delete 1 dbs, save and verify', () => {
      cy.get('.app-menu-section').then(($ams) => {
        const appMenuSectionsCount = $ams.length;
        apps.addSubgroupWithDashboards(2);
        apps.deleteLastDashboard(true);
        cy.get(apps.locators.menusections).should('have.length', appMenuSectionsCount + 1);
        cy.get(apps.locators.dashboards_in_last_submenu).should('have.length', 1);
        apps.deleteLastSubgroup(true);
      });
    });

    // TODOTESTS: More tests can be added now like so:
    // Delete a dashboard, move another from one tab to another,
    // move another dashboard from within a tab to another position in a tab
    // and related permutations
  });
});
