import * as apps from 'lib/apps';
import { logins } from 'lib/arctest';

// Because of the intermittent timeout errors tracked on VIZ-1517,
// we set these particular tests to retry on fails
describe('App builder reordering tests', { retries: 2 }, () => {
  // Set up before any tests are run
  before(() => {
    logins.silent();
  });

  // context: Test suite
  context('App builder reordering tests', () => {
    afterEach(() => {
      apps.deleteAppFromBuilder();
    });

    it('test subgroup move from right to left in new app', () => {
      apps.createNewApp();
      apps.addMenuTab();
      apps.addDashboard('samples', 'World Life Expectancy', 'Life Expectancy Dashboard');
      apps.addDashboard('samples', 'World Life Expectancy', 'Animated');
      apps.verifyDashboardsNumInTab(0, 0);
      apps.verifyDashboardsNumInTab(1, 2);
      apps.verifyDashboardsNameInTab(1, 0, 'Life');
      apps.saveApp();
      apps.moveMenuTab(1, 0);
      apps.verifyDashboardsNumInTab(0, 2);
      apps.verifyDashboardsNumInTab(1, 0);
      apps.verifyDashboardsNameInTab(0, 0, 'Life');
      apps.saveApp();
      apps.verifyDashboardsNumInTab(0, 2);
      apps.verifyDashboardsNumInTab(1, 0);
      apps.verifyDashboardsNameInTab(0, 0, 'Life');
    });

    it('test subgroup move from right to left (middle) in new app', () => {
      apps.createNewApp();
      apps.addMenuTab();
      apps.addMenuTab();
      apps.addDashboard('samples', 'World Life Expectancy', 'Life Expectancy Dashboard');
      apps.addDashboard('samples', 'World Life Expectancy', 'Animated');
      apps.verifyDashboardsNumInTab(0, 0);
      apps.verifyDashboardsNumInTab(1, 0);
      apps.verifyDashboardsNumInTab(2, 2);
      apps.verifyDashboardsNameInTab(2, 0, 'Life');
      apps.saveApp();
      apps.moveMenuTab(2, 1);
      apps.verifyDashboardsNumInTab(0, 0);
      apps.verifyDashboardsNumInTab(1, 2);
      apps.verifyDashboardsNumInTab(2, 0);
      apps.verifyDashboardsNameInTab(1, 0, 'Life');
      apps.saveApp();
      apps.verifyDashboardsNumInTab(0, 0);
      apps.verifyDashboardsNumInTab(1, 2);
      apps.verifyDashboardsNumInTab(2, 0);
      apps.verifyDashboardsNameInTab(1, 0, 'Life');
    });

    it('test menu tab move from left to right in new app', () => {
      apps.createNewApp();
      apps.addDashboard('samples', 'World Life Expectancy', 'Life Expectancy Dashboard');
      apps.addDashboard('samples', 'World Life Expectancy', 'Animated');
      apps.addMenuTab();
      apps.addMenuTab();
      apps.addMenuTab();
      apps.verifyDashboardsNumInTab(0, 2);
      apps.verifyDashboardsNumInTab(1, 0);
      apps.verifyDashboardsNameInTab(0, 0, 'Life');
      apps.saveApp();
      apps.moveMenuTab(0, 3);
      apps.verifyDashboardsNumInTab(0, 0);
      apps.verifyDashboardsNumInTab(3, 2);
      apps.verifyDashboardsNameInTab(3, 0, 'Life');
      apps.saveApp();
    });

    it('test menu tab move from left to right (middle) in new app', () => {
      apps.createNewApp();
      apps.addDashboard('samples', 'World Life Expectancy', 'Life Expectancy Dashboard');
      apps.addDashboard('samples', 'World Life Expectancy', 'Animated');
      apps.addMenuTab();
      apps.addMenuTab();
      apps.verifyDashboardsNumInTab(0, 2);
      apps.verifyDashboardsNumInTab(1, 0);
      apps.verifyDashboardsNumInTab(2, 0);
      apps.verifyDashboardsNameInTab(0, 0, 'Life');
      apps.saveApp();
      apps.moveMenuTab(0, 1);
      apps.verifyDashboardsNumInTab(0, 0);
      apps.verifyDashboardsNumInTab(1, 2);
      apps.verifyDashboardsNumInTab(2, 0);
      apps.verifyDashboardsNameInTab(1, 0, 'Life');
      apps.saveApp();
      apps.verifyDashboardsNumInTab(0, 0);
      apps.verifyDashboardsNumInTab(1, 2);
      apps.verifyDashboardsNumInTab(2, 0);
      apps.verifyDashboardsNameInTab(1, 0, 'Life');
    });

    it('test dashboard move between menu tabs', () => {
      apps.createNewApp();
      apps.addDashboard('samples', 'World Life Expectancy', 'Life Expectancy Dashboard');
      apps.addDashboard('samples', 'World Life Expectancy', 'Animated');
      apps.addMenuTab();
      apps.addDashboard('samples', 'World Life Expectancy', 'Animated');
      apps.verifyDashboardsNumInTab(0, 2);
      apps.verifyDashboardsNumInTab(1, 1);
      apps.verifyDashboardsNameInTab(0, 0, 'Life');
      apps.saveApp();
      apps.moveDashboardWithinApp(0, 0, 1, 0);
      apps.verifyDashboardsNumInTab(0, 1);
      apps.verifyDashboardsNumInTab(1, 2);
      apps.verifyDashboardsNameInTab(0, 0, 'Animated');
      apps.verifyDashboardsNameInTab(1, 0, 'Life');
      apps.saveApp();
      apps.verifyDashboardsNumInTab(0, 1);
      apps.verifyDashboardsNumInTab(1, 2);
      apps.verifyDashboardsNameInTab(0, 0, 'Animated');
      apps.verifyDashboardsNameInTab(1, 0, 'Life');
    });

    it('test dashboard move between menu tabs - with tgt menu tab having no dashboards ', () => {
      apps.createNewApp();
      apps.addDashboard('samples', 'World Life Expectancy', 'Life Expectancy Dashboard');
      apps.addDashboard('samples', 'World Life Expectancy', 'Animated');
      apps.addMenuTab();
      apps.verifyDashboardsNumInTab(0, 2);
      apps.verifyDashboardsNumInTab(1, 0);
      apps.verifyDashboardsNameInTab(0, 0, 'Life');
      apps.saveApp();
      apps.moveDashboardWithinApp(0, 0, 1, 0);
      apps.verifyDashboardsNumInTab(0, 1);
      apps.verifyDashboardsNumInTab(1, 1);
      apps.verifyDashboardsNameInTab(0, 0, 'Animated');
      apps.verifyDashboardsNameInTab(1, 0, 'Life');
      apps.saveApp();
      apps.verifyDashboardsNumInTab(0, 1);
      apps.verifyDashboardsNumInTab(1, 1);
      apps.verifyDashboardsNameInTab(0, 0, 'Animated');
      apps.verifyDashboardsNameInTab(1, 0, 'Life');
    });

    // TODOTESTS: can add more tests here of moving dashboards within menu tabs like so:
    // a. move multiple dashboards into different locations instead of just 1
    // b. move a tab and a dashboard at the same time

    it('test dashboard move within a menu tab - top to bottom', () => {
      apps.createNewApp();
      apps.addDashboard('samples', 'World Life Expectancy', 'Life Expectancy Dashboard');
      apps.addDashboard('samples', 'World Life Expectancy', 'Animated');
      apps.verifyDashboardsNumInTab(0, 2);
      apps.verifyDashboardsNameInTab(0, 0, 'Life');
      apps.verifyDashboardsNameInTab(0, 1, 'Animated');
      apps.saveApp();
      apps.moveDashboardWithinApp(0, 0, 0, 1);
      apps.verifyDashboardsNumInTab(0, 2);
      apps.verifyDashboardsNameInTab(0, 0, 'Animated');
      apps.verifyDashboardsNameInTab(0, 1, 'Life');
      apps.saveApp();
      apps.verifyDashboardsNumInTab(0, 2);
      apps.verifyDashboardsNameInTab(0, 0, 'Animated');
      apps.verifyDashboardsNameInTab(0, 1, 'Life');
    });

    // TODOTESTS: can add more tests of moving within a menu tab like so:
    // a. from bottom to top
    // b. from top to middle, from bottom to middle
    // c. moving a dashboard to 2 different positions, top to bottom and then to middle
    // d. moving 2 dashboards within the same tab to 2 different positions
  });
});
