import * as apps from 'lib/apps';
import { logins } from 'lib/arctest';
import * as functions from 'lib/functions';
import * as imp_exp from 'lib/import_export';
import { CSSselector } from 'locators/common/CSSselector';

// This script does verification of tests ARC-8013 & ARC-8018 with New App UI
describe('New App groups smoke testing.', () => {
  before(() => {
    logins.silent();
  });

  /**
   * Variables.
   */

  const selectDashboard = 2;
  const DashboardName: string[] = [];
  const appTitle = 'name of a new application';

  describe('ARC-8020', () => {
    it('Add dashboards in App - ARC-8013', () => {
      apps.createNewApp(appTitle);
      apps.addDashboard('samples', 'World Life Expectancy', 'Life Expectancy Dashboard');
      apps.addDashboard('samples', 'Cereals', 'Cereal Comparisons');
      DashboardName.push('Life Expectancy Dashboard', 'Cereal Comparisons');
      apps.saveApp();
    });

    it('Launch app and dashboards in it - ARC-8013', () => {
      functions.navigate_to_visuals();
      cy.get(`${CSSselector.Visuals_all}`).click();
      cy.get(`${CSSselector.listview_Visualspage}`).click();
      cy.get(apps.locators.apps_list).click({ force: true });

      functions.typeInSearchField(appTitle);

      cy.get(`[data-test-id="${appTitle}"]`)
        .find(imp_exp.locators.grid_item)
        .click();

      const menuDropdown = '.nav .dropdown [role="button"]';
      const dashboarLinks = '.nav [role="menu"] [data-arc-app]';
      cy.get(menuDropdown).click();
      cy.get(dashboarLinks).should(($appElement) => {
        expect($appElement).to.have.lengthOf(selectDashboard);
      });
      cy.get(dashboarLinks).each(($dashEl, dashIndex) => {
        expect($dashEl.text()).be.equal(DashboardName[dashIndex]);
      });
    });

    it('Delete the App - ARC-8018', () => {
      cy.visit('/');

      apps.deleteAppFromNewUI(appTitle);
      cy.visit('/');
      cy.get(`${CSSselector.Visual_page}`).click();
      cy.get(CSSselector.apps.favourites_tab).click();
      cy.get(`${CSSselector.Visuals_all}`).click();

      cy.get(`[data-test-id="${appTitle}"]`).should('not.exist');
    });
  });
});
