import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';
import { checkFeatureAvailable } from 'lib/functions';
import { verifyVisual } from 'lib/visual';

const connectionName = 'curly';
const datasetName1 = 'cdt array dataset';
const datasetName2 = 'cdt structure dataset';
const datasetName3 = 'customer cdt dataset';

const dashboard1: Dashboard = {
  title: 'Complex Data Type Array Dashboard',
  subTitle: 'Complex Data Type Array Dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'Sheet 1',
      sheet_number: 1,
      visuals: [
        {
          name: 'Trellis bars using Array',
          connection_name: connectionName,
          dataset_name: datasetName1,
          visual_type: '.chart-icon-trellis-bars',
          visual_title: 'Trellis bars using Array Visual',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'category.pos',
                field_shelf: ShelfTypes.X,
              },
              {
                field_name: 'amount.pos',
                field_shelf: ShelfTypes.X,
              },
              {
                field_name: 'overalldiscount',
                field_shelf: ShelfTypes.Y,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'qty.pos',
                field_shelf: ShelfTypes.COLOR,
              },
            ],
          },
          is: [
            'setTransform',
            'translate',
            'globalAlpha',
            'lineWidth',
            'font',
            'clearRect',
            'lineWidth',
            'strokeStyle',
            'fillStyle',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'fillStyle',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'fillStyle',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'fillStyle',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'fillStyle',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'fillStyle',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'beginPath',
            'rect',
            'fill',
            'stroke',
            'beginPath',
            'rect',
            'fill',
            'stroke',
          ],
          hash: 'f96d1908e451c16b06d715c49547e7ef',
        },
      ],
    },
  ],
};
const dashboard2: Dashboard = {
  title: 'Complex Data Type Structure Dashboard',
  subTitle: 'Complex Data Type Structure Dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'Sheet 1',
      sheet_number: 1,
      visuals: [
        {
          name: 'Trellis lines using Structure CDT',
          connection_name: connectionName,
          dataset_name: datasetName2,
          visual_type: '.chart-icon-trellis-lines',
          visual_title: 'Trellis Lines using Structure Visual',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'customer',
                field_shelf: ShelfTypes.X,
              },
              {
                field_name: 'orderinfo.product',
                field_shelf: ShelfTypes.X,
              },
              {
                field_name: 'overalldiscount',
                field_shelf: ShelfTypes.Y,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'customer',
                field_shelf: ShelfTypes.COLOR,
              },
              {
                field_name: 'orderinfo.qty',
                field_shelf: ShelfTypes.FILTERS,
                field_expression: '[orderinfo.qty] in (1, 2, 4, 5, 10, 100)',
              },
            ],
          },
          is: [
            'setTransform',
            'translate',
            'globalAlpha',
            'lineWidth',
            'font',
            'clearRect',
            'fillStyle',
            'strokeStyle',
            'beginPath',
            'moveTo',
            'lineTo',
            'lineTo',
            'lineTo',
            'lineTo',
            'lineTo',
            'lineTo',
            'lineTo',
            'lineTo',
            'lineTo',
            'lineTo',
            'stroke',
          ],
          hash: '030fa1b819d2016a22ed2012a9db6730',
        },
      ],
    },
  ],
};
const dashboard3: Dashboard = {
  title: 'Complex Data Type Combination Dashboard',
  subTitle: 'Complex Data Type Combination Dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'Sheet 1',
      sheet_number: 1,
      visuals: [
        {
          name: 'Grouped bars using Combination of Arrays and Structs',
          connection_name: connectionName,
          dataset_name: datasetName3,
          visual_type: '.chart-icon-trellis-groupedbars',
          visual_title: 'Trellis Grouped bars using Array and Struct Visual',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'c_name',
                field_shelf: ShelfTypes.X,
              },
              {
                field_name: 'c_acctbal',
                field_shelf: ShelfTypes.Y,
                sort: SortOptions.DESCENDING,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'c_orders.item.o_totalprice',
                field_shelf: ShelfTypes.COLOR,
              },
              {
                field_name: 'c_orders.item.o_lineitems.item.l_quantity',
                field_shelf: ShelfTypes.COLOR,
              },
              {
                field_name: 'c_name',
                field_shelf: ShelfTypes.FILTERS,
                field_expression:
                  "[c_name] in ('Customer#000000001', 'Customer#000000002', 'Customer#000000003')",
              },
            ],
          },
          is: ['setTransform', 'translate'],
          hash: '77c3149917e732dc9427798f79de1398',
        },
      ],
    },
  ],
};

describe('Complex Data Type Visuals - Bug ARC-11586.', () => {
  before(function() {
    checkFeatureAvailable(this, 'connection_curly');
    logins.silent();
    cy.visit('/arc/apps/home');
  });
  afterEach(() => {
    cy.window().then(
      (win: Window) =>
        (win.onbeforeunload = () => {
          return;
        })
    );
  });

  context('CDT Array test suite', () => {
    it('Test 1: Create trellis bar visuals with complex data type array fields', () => {
      dashboards.createDashboard(dashboard1);
      verifyVisual(dashboard1);
    });
    it('Test 2: Create trellis bar visuals with complex data type Structure fields', () => {
      dashboards.createDashboard(dashboard2);
      verifyVisual(dashboard2);
    });
    it('Test 3: Create trellis bar visuals with complex data type struct fields', () => {
      dashboards.createDashboard(dashboard3);
      // make the visual full screen
      // cy.get("a[data-target='#dash-settings-tab']").click();
      // cy.get('.checkbox.single_visual_dashboard').click();
      verifyVisual(dashboard3);
    });
  });
});
