import { logins } from 'lib/arctest';
import * as arctest from 'lib/arctest/screen/builder';
import { locators } from 'lib/context_menus/locators';
import * as dashboard from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';
import * as filter from 'lib/filter';
import { CSSselector } from 'locators';

function get_time() {
  const a = new Date();
  return a.getTime();
}

const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'Cereals';
const sourceVisual = arctest.locators.visuals + ':nth-child(1)';
const targetVisual = arctest.locators.visuals + ':nth-child(2)';
const filterAllDashboard: Dashboard = {
  title: 'Filter All' + get_time(),
  subTitle: 'filter_all_subtitle' + get_time(),
  sheets: [
    {
      sheet_name: 'sheet1',
      sheet_number: 1,
      visuals: [
        {
          name: 'source visual',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-table',
          visual_title: 'source_visual',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'manufacturer_code',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'cold_or_hot',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.AGGREGATES,
              },
            ],
          },
        },
        {
          name: 'target visual',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-table',
          visual_title: 'target_visual',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'manufacturer_code',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'cold_or_hot',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.AGGREGATES,
              },
            ],
          },
        },
      ],
    },
  ],
};

const filterOnClickDashboard: Dashboard = {
  title: 'Filter On Click' + get_time(),
  subTitle: 'filter_all_subtitle' + get_time(),
  sheets: [
    {
      sheet_name: 'sheet1',
      sheet_number: 1,
      visuals: [
        {
          name: 'source visual',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-table',
          visual_title: 'source_visual',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'manufacturer_code',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'cold_or_hot',
                field_shelf: ShelfTypes.DIMENSIONS,
                sort: SortOptions.DESCENDING,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.AGGREGATES,
              },
            ],
          },
        },
        {
          name: 'target visual',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-table',
          visual_title: 'target_visual',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'manufacturer_code',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'cold_or_hot',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.AGGREGATES,
              },
            ],
          },
        },
      ],
    },
  ],
};
describe('Filter All and Filter on click context menus.', () => {
  before(() => {
    logins.silent();
  });

  context('ARC-11128: Automate "Filter All" context menu', () => {
    it('1: Create Dashboard and apply Filter All in dashbaord edit mode', () => {
      dashboard.createDashboard(filterAllDashboard);

      // Click on the particular row and then click on filter all
      cy.get(
        locators.table_data
          .replace('visualIndex', '1')
          .replace('row', '1')
          .replace('column', '2')
      )
        .click({ force: true }) // TODO VIZ-1358
        .wait(1000)
        .click({ force: true })
        .wait(1000)
        .click({ force: true });

      cy.get(locators.context_menu)
        .find('div.cm-filterall')
        .click({
          force: true,
        });

      // Verify table content after clicking on filter all
      arctest.verifyTableContents(targetVisual, [
        ['100%_Bran', 'N', 'C', '70'],
        ['Cream_of_Wheat_(Quick)', 'N', 'H', '100'],
        ['Shredded_Wheat', 'N', 'C', '80'],
        ["Shredded_Wheat_'n'Bran", 'N', 'C', '90'],
        ['Shredded_Wheat_spoon_size', 'N', 'C', '90'],
        ['Strawberry_Fruit_Wheats', 'N', 'C', '90'],
      ]);

      // Verify current parameters of applied filter
      filter.verifyCurrentParameters(['manufacturer_code']);
      filter.verifyCurrentParameters(['N']);
    });

    it('2: Verify Filter parameters in dashboard view mode', () => {
      dashboard.viewDashboard();
      // Click on the particular row and then click on filter all
      cy.get(
        locators.table_data
          .replace('visualIndex', '2')
          .replace('row', '5')
          .replace('column', '1')
      ).click({ force: true });
      cy.get(locators.context_menu)
        .find('div.cm-filterall')
        .click({
          force: true,
        });
      // Verify table content after clicking on filter all
      arctest.verifyTableContents(sourceVisual, [['Almond_Delight', 'R', 'C', '110']]);

      // Verify current parameters of applied filter
      filter.verifyCurrentParameters(['cereal_name']);
      filter.verifyCurrentParameters(['Almond_Delight']);
    });
  });

  context('ARC-11128: Automate "Enable Filter on Click" context menu', () => {
    it('1: Create Dashboard and apply filter on click in dashbaord edit mode', () => {
      dashboard.createDashboard(filterOnClickDashboard);

      // Click on the particular row and then click on enable filter on click
      cy.get(CSSselector.close_dashboard_setting).click();
      cy.get(
        locators.table_data
          .replace('visualIndex', '1')
          .replace('row', '1')
          .replace('column', '3')
      ).click({ force: true });
      cy.get(locators.context_menu).as('contextMenu');

      cy.get('#click-behaviors').trigger('mouseover');

      cy.get('@contextMenu')
        .find('div.cm-enableclick')
        .click()
        .should('have.class', 'selected');

      // Test the click behavior
      cy.get('@contextMenu')
        .find('a.cm-test-link')
        .should('be.visible')
        .should('contain', 'Test Click Behavior')
        .click();

      // Verify table content after enabeling filter on click
      arctest.verifyTableContents(targetVisual, [
        ['Cream_of_Wheat_(Quick)', 'N', 'H', '100'],
        ['Maypo', 'A', 'H', '100'],
        ['Quaker_Oatmeal', 'Q', 'H', '100'],
      ]);
      // Verify current parameters of applied filter
      filter.verifyCurrentParameters(['cold_or_hot']);
      filter.verifyCurrentParameters(['H']);

      dashboard.saveDashboard();
    });

    it('2: Verify Filter parameters in dashboard view mode', () => {
      dashboard.viewDashboard();
      // Click on the particular row and then enable filter on click
      cy.get(
        locators.table_data
          .replace('visualIndex', '1')
          .replace('row', '3')
          .replace('column', '1')
      ).click({ force: true });

      // Verify table content after enabeling filter on click
      arctest.verifyTableContents(targetVisual, [['Quaker_Oatmeal', 'Q', 'H', '100']]);

      // Verify current parameters of applied filter
      filter.verifyCurrentParameters(['cereal_name']);
      filter.verifyCurrentParameters(['Quaker_Oatmeal']);
    });
  });
});
