import { logins } from 'lib/arctest';
import * as arctest from 'lib/arctest/screen/builder';
import { locators } from 'lib/context_menus/locators';
import * as dashboard from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';
import { locators as dashboardLocators } from 'lib/dashboard/locators';
import * as filter from 'lib/filter';
import * as functions from 'lib/functions';
import * as dataset from 'lib/permissions';
import { CSSselector } from 'locators/common/CSSselector';

function get_time() {
  const a = new Date();
  return a.getTime();
}

const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'Cereals';
const targetVisual = arctest.locators.visuals + ':nth-child(1)';
const url = `${Cypress.config().baseUrl}` + '/arc/apps/app/';
let targetDashId: number;
let datasetID: number;
const sourceDashboard: Dashboard = {
  title: 'source_db' + get_time(),
  subTitle: 'source_db_subtitle' + get_time(),
  sheets: [
    {
      sheet_name: 'sheet1',
      sheet_number: 1,
      visuals: [
        {
          name: 'source visual',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-table',
          visual_title: 'source_visual',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'manufacturer_code',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'cold_or_hot',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.AGGREGATES,
              },
            ],
          },
        },
      ],
    },
  ],
};

const targetDashboard: Dashboard = {
  title: 'target_db' + get_time(),
  subTitle: 'target_db_subtitle' + get_time(),
  sheets: [
    {
      sheet_name: 'sheet1',
      sheet_number: 1,
      visuals: [
        {
          name: 'target visual',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-table',
          visual_title: 'target_visual',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'manufacturer_code',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'cold_or_hot',
                field_shelf: ShelfTypes.DIMENSIONS,
                sort: SortOptions.DESCENDING,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.AGGREGATES,
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Navigate To context menu.', () => {
  before(() => {
    logins.silent();
    cy.visit('/');
    // get dataset id
    functions.navigate_to_data();
    cy.get(CSSselector.data.connection_list).each(($el, index, $list) => {
      if ($el.text() === connectionName) {
        cy.wrap($el).click();
      }
    });
    cy.intercept('GET', '/arc/datasets/dataconnection').as('dataconnectionsLoaded');
    dataset.selectDataset(connectionName, datasetName);
    cy.wait('@dataconnectionsLoaded');
    cy.get('#dataset-id').then(($id) => {
      datasetID = Number($id.text().trim());
    });
  });

  context('ARC-11331: Automate "Navigate To" context menu - Smoke', () => {
    it('1: Create Target and Source Dashboard and configure navigate to dashboard without passing current parameters and verify it', () => {
      dashboard.createDashboard(targetDashboard);
      dashboard.createDashboard(sourceDashboard);
      // Clicked on particular element/cell of table visual
      cy.get(
        locators.table_data
          .replace('visualIndex', '1')
          .replace('row', '1')
          .replace('column', '2')
      ).as('clickedRow');
      cy.get('@clickedRow').click({ force: true });

      // open context menu and click on navigate to dashboard
      cy.get(locators.context_menu)
        .find('.cm-navigateclick')
        .as('navigateTo');
      cy.get('@navigateTo').click({ force: true });

      // Search target db, get its id, click on it and verify dashboard navigation
      cy.get(locators.pick_dashboard).should('be.visible');
      cy.get(locators.pick_dashboard)
        .find('input#searchbox')
        .type('"' + targetDashboard.title + '"');
      cy.get(locators.select_dashboard)
        .contains(targetDashboard.title)
        .parent()
        .as('targetDB');
      cy.get('@targetDB')
        .invoke('attr', 'arc-report-id')
        .then(($id) => {
          targetDashId = Number($id);
        })
        .then(() => {
          cy.get('@targetDB').click();
          dashboard.saveDashboard();
          cy.get('@clickedRow').click({ force: true });
          cy.get('@navigateTo').should('have.class', 'selected');
          cy.get(locators.nav_link)
            .should('be.visible')
            .should('have.text', 'Navigate to ID: ' + targetDashId);
          dashboard.viewDashboard();
          cy.get('@clickedRow').click({ force: true });
          cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('dashLoaded');
          cy.wait('@dashLoaded')
            .its('response.statusCode')
            .should('eq', 200);
          cy.url().should('eq', url + targetDashId);
          cy.get(dashboardLocators.dashboard_title).should('have.text', targetDashboard.title);
          cy.get(dashboardLocators.dashboard_subtitle).should(
            'have.text',
            targetDashboard.subTitle
          );
        });
    });

    it('2: Open Source Dashboard and configure navigate to dashboard with passing current parameters and verify it', () => {
      dashboard.openDashboardforEdit(sourceDashboard.title);
      // Clicked on particular element/cell of table visual
      cy.get(
        locators.table_data
          .replace('visualIndex', '1')
          .replace('row', '1')
          .replace('column', '1')
      ).as('clickedRow');
      cy.get('@clickedRow').click({ force: true });

      // open context menu and click on navigate to dashboard
      cy.get(locators.context_menu)
        .find('.cm-navigateclick')
        .as('navigateTo');
      cy.get('@navigateTo').click({ force: true });

      // Search target db, check pass params check box, click on db and verify dashboard navigation with params
      cy.get(locators.pick_dashboard).should('be.visible');
      cy.get(locators.pick_dashboard)
        .find('input.pass-params')
        .check();
      cy.get(locators.select_dashboard)
        .contains(targetDashboard.title)
        .parent()
        .as('targetDB');
      cy.get('@targetDB').click();
      dashboard.saveDashboard();
      dashboard.viewDashboard();
      cy.get('@clickedRow').click({ force: true });
      cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('dashLoaded');
      cy.wait('@dashLoaded')
        .its('response.statusCode')
        .should('eq', 200);
      const url1 =
        url +
        targetDashId +
        '?param.cereal_name=100%25_Bran&param.cereal_name.include_nulls=false&scope=[{%22scopeType%22:%22global%22,%22isAppScope%22:false,%22filterName%22:%22cereal_name%22,%22filterId%22:' +
        (targetDashId + 1) +
        '}]';
      cy.url().should('eq', url1);
      cy.get(dashboardLocators.dashboard_title).should('have.text', targetDashboard.title);
      cy.get(dashboardLocators.dashboard_subtitle).should('have.text', targetDashboard.subTitle);

      // Verify current parameters of applied
      filter.verifyCurrentParameters(['cereal_name']);
      filter.verifyCurrentParameters(['100%_Bran']);
    });

    it('3: Open Source Dashboard and navigate it with different parameter and verify it', () => {
      cy.visit('/');
      dashboard.openDashboard(sourceDashboard.title);
      // Clicked on particular element/cell of table visual
      cy.get(
        locators.table_data
          .replace('visualIndex', '1')
          .replace('row', '2')
          .replace('column', '1')
      ).as('clickedRow');
      cy.get('@clickedRow').click({ force: true });
      cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('dashLoaded');
      cy.wait('@dashLoaded')
        .its('response.statusCode')
        .should('eq', 200);
      const url2 =
        url +
        targetDashId +
        '?param.cereal_name=100%25_Natural_Bran&param.cereal_name.include_nulls=false&scope=[{%22scopeType%22:%22global%22,%22isAppScope%22:false,%22filterName%22:%22cereal_name%22,%22filterId%22:' +
        (targetDashId + 1) +
        '}]';
      cy.url().should('eq', url2);
      cy.get(dashboardLocators.dashboard_title).should('have.text', targetDashboard.title);
      cy.get(dashboardLocators.dashboard_subtitle).should('have.text', targetDashboard.subTitle);

      // Verify current parameters of applied filter
      filter.verifyCurrentParameters(['cereal_name']);
      filter.verifyCurrentParameters(['100%_Natural_Bran']);
    });
  });
});
