import { BackendAPI } from 'lib/backend-api';
import { saveDashboard } from 'lib/dashboard';
import { areFilesEqual, downloadVisual } from './utils';

import * as logins from 'lib/logins';

describe('Excel/Csv backend download tests', () => {
  before(() => {
    logins.silent();

    BackendAPI.dashboards
      .import({
        fileName: 'csv_excel_download_dash_prettyfied.json',
        dataconnectionName: 'samples',
        workspaceName: 'Public',
      })
      .then(({ dashboard }) => cy.visit(`/arc/apps/builder/${dashboard.id}`));
  });

  it('Test 1: Download CSV: table no custom display', () => {
    setNumberRowDownloaded(1);
    backendDownload(1, true);

    const expectedFile =
      'cypress/tests/csv_excel_download/expected_results/backend/table_no_diplay_formats.csv';

    areFilesEqual(expectedFile, true);
  });

  it('Test 2: Download CSV: crosstab', () => {
    setNumberRowDownloaded(2);
    backendDownload(2, true);

    const expectedFile = 'cypress/tests/csv_excel_download/expected_results/backend/crosstab.csv';

    areFilesEqual(expectedFile, true);
  });

  it('Test 3: Download CSV: Sql visual', () => {
    setNumberRowDownloaded(3);
    backendDownload(3, true);

    const expectedFile = 'cypress/tests/csv_excel_download/expected_results/backend/sql_visual.csv';

    areFilesEqual(expectedFile, true);
  });

  it('Test 4: Download CSV: Display formats', () => {
    setNumberRowDownloaded(4);
    backendDownload(4, true);

    const expectedFile =
      'cypress/tests/csv_excel_download/expected_results/backend/display_formats.csv';

    areFilesEqual(expectedFile, true);
  });

  it('Test 5: Download Excel: table no custom display', () => {
    backendDownload(1, false);

    const expectedFile =
      'cypress/tests/csv_excel_download/expected_results/backend/table_no_display_formats_excel.csv';

    areFilesEqual(expectedFile, true, true);
  });

  it('Test 6: Download Excel: crosstab', () => {
    backendDownload(2, false);

    const expectedFile =
      'cypress/tests/csv_excel_download/expected_results/backend/crosstab_excel.csv';

    areFilesEqual(expectedFile, true, true);
  });

  it('Test 7: Download Excel: Sql visual', () => {
    backendDownload(3, false);

    const expectedFile =
      'cypress/tests/csv_excel_download/expected_results/backend/sql_visual_excel.csv';

    areFilesEqual(expectedFile, true, true);
  });

  it('Test 8: Download Excel: Display formats visual', () => {
    backendDownload(4, false);

    const expectedFile =
      'cypress/tests/csv_excel_download/expected_results/backend/display_formats_excel.csv';

    areFilesEqual(expectedFile, true, true);
  });
});

function backendDownload(visual: number, isCsv: boolean) {
  downloadVisual(visual, isCsv, 'edit', 'backend');
}

function setNumberRowDownloaded(visualNumber: number) {
  cy.get(
    `.main-app.arc-app-container.edit-mode [data-test-id="visual-list-item"]:nth-child(${visualNumber})`
  ).click();
  cy.get(`#visual-settings-tab > a`).click();
  cy.get(`[data-test-id="visual-style-tab-Data"]`).click();
  cy.get(`[data-test-id="DOWNLOAD_ROWS"]`)
    .clear()
    .type('200');

  saveDashboard();
}
