import { BackendAPI } from 'lib/backend-api';
import * as functions from 'lib/functions';

import * as logins from 'lib/logins';
import { areFilesEqual, downloadVisual } from './utils';

describe('Excel/Csv frontend download tests', () => {
  before(() => {
    logins.silent();
    BackendAPI.dashboards
      .import({
        fileName: 'csv_excel_download_dash_prettyfied.json',
        dataconnectionName: 'samples',
        workspaceName: 'Public',
      })
      .then(({ dashboard }) => {
        cy.visit(`/arc/apps/app/${dashboard.id}`);
        functions.waitForVisualsLoaded();
      });
  });

  it('Test 1: Download CSV: table no custom display', () => {
    downloadVisual(1, true);

    const expectedFile =
      'cypress/tests/csv_excel_download/expected_results/frontend/table_no_custom_display.csv';

    areFilesEqual(expectedFile, false);
  });

  it('Test 2: Download CSV: crosstab', () => {
    downloadVisual(2, true);

    const expectedFile = 'cypress/tests/csv_excel_download/expected_results/frontend/crosstab.csv';

    areFilesEqual(expectedFile, false);
  });

  it('Test 3: Download CSV: Sql visual', () => {
    downloadVisual(3, true);

    const expectedFile =
      'cypress/tests/csv_excel_download/expected_results/frontend/sql_visual.csv';

    areFilesEqual(expectedFile, false);
  });

  it('Test 4: Download CSV: Display formats', () => {
    downloadVisual(4, true);

    const expectedFile =
      'cypress/tests/csv_excel_download/expected_results/frontend/display_formats.csv';

    areFilesEqual(expectedFile, true);
  });

  it('Test 5: Download Excel: table no custom display', () => {
    downloadVisual(1, false);

    const expectedFile =
      'cypress/tests/csv_excel_download/expected_results/frontend/table_no_custom_display_excel.csv';

    areFilesEqual(expectedFile, true, true);
  });

  it('Test 6: Download Excel: crosstab', () => {
    downloadVisual(2, false);

    const expectedFile =
      'cypress/tests/csv_excel_download/expected_results/frontend/crosstab_excel.csv';

    areFilesEqual(expectedFile, true, true);
  });

  it('Test 7: Download Excel: Sql visual', () => {
    downloadVisual(3, false);

    const expectedFile =
      'cypress/tests/csv_excel_download/expected_results/frontend/sql_visual_excel.csv';

    areFilesEqual(expectedFile, true, true);
  });

  it('Test 8: Download Excel: Display format', () => {
    downloadVisual(4, false);

    const expectedFile =
      'cypress/tests/csv_excel_download/expected_results/frontend/display_formats_excel.csv';

    areFilesEqual(expectedFile, true, true);
  });
});
