export function downloadVisual(
  visualNumber: number,
  isCsv: boolean,
  mode: string = 'view',
  testType: 'backend' | 'frontend' = 'frontend'
) {
  const downloadOption = isCsv ? 'export-csv-link' : 'export-xlsx-link';

  cy.get(
    `.main-app.arc-app-container.${mode}-mode [data-test-id="visual-list-item"]:nth-child(${visualNumber})`
  ).click({
    force: true,
  });
  cy.get(
    `[data-test-id="visual-list-item"]:nth-child(${visualNumber}) [data-test-id="visual-more"]`
  ).click();

  if (testType === 'backend') {
    cy.intercept('GET', '/arc/apps/snapshot_ready/*').as('dataReq');
  }

  cy.get('.contextmenu-body > .more-options > .dropdown-submenu')
    .find('.dropdown-menu')
    .then((e) => {
      cy.wrap(e).invoke('show');
    })
    .find(`[data-test-id='${downloadOption}']`)
    .click({ force: true });

  if (testType === 'backend') {
    cy.wait('@dataReq');
  }
  // how much time we need to have the download created?
  cy.wait(3000);
}

export function areFilesEqual(
  expectedFileName: string,
  cliCompare: boolean,
  convertToCsv: boolean = false
) {
  if (convertToCsv) {
    cy.exec('ls -Art cypress/downloads | tail -n 1').then((results) => {
      const filename = results.stdout;
      cy.log(filename);

      cy.exec(`npx xlsx cypress/downloads/${filename} -o cypress/downloads/${filename}.csv`);
    });
  }

  cy.exec('ls -Art cypress/downloads | tail -n 1').then((results) => {
    const filename = results.stdout;

    if (cliCompare) {
      const downloadedFile = `cypress/downloads/${filename}`;

      cy.exec(`diff ${expectedFileName} ${downloadedFile}`).then((output) => {
        expect(output.stdout).to.be.empty;
      });
    } else {
      cy.readFile(expectedFileName, 'utf-8').then((uploadTxt) => {
        cy.readFile(`cypress/downloads/${filename}`, 'utf-8', { timeout: 10000 }).then(
          (downloadTxt) => {
            expect(downloadTxt).to.equal(uploadTxt);
          }
        );
      });
    }
  });
}
