import * as arctest from 'lib/arctest';
import * as customsettings from 'lib/customsettings';
import * as dashboard from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';
import * as visuals from 'lib/visual';
import { CSSselector } from 'locators/common/CSSselector';

const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'Cereals';
const dashboardsDesc: Dashboard = {
  title: 'Cereal Scatters DB',
  subTitle: 'simple_scatter_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'sheet1',
      sheet_number: 1,
      visuals: [
        {
          name: 'scattervisual1',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-scatter',
          visual_title: 'Colors: Yellow: 4, Purple: 22, Blueish: 61',
          visual_subtitle: 'The bubble with 22 should be purple!',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'manufacturer_code',
                field_shelf: ShelfTypes.X,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'protein_grams',
                field_shelf: ShelfTypes.Y,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'protein_grams',
                field_shelf: ShelfTypes.COLOR,
              },
            ],
          },
        },

        {
          name: 'scattervisual2',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-scatter',
          visual_title: 'Colors: Yellow: 0%, Purple: 50%, Blueish: 100%',
          visual_subtitle: 'The bubble with 22 should not be the purple',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'manufacturer_code',
                field_shelf: ShelfTypes.X,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'protein_grams',
                field_shelf: ShelfTypes.Y,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'protein_grams',
                field_shelf: ShelfTypes.COLOR,
              },
            ],
          },
        },
      ],
    },
  ],
};
const timestamp = new Date().getTime();
const gradientColorByValue = `gradient-by-value-${timestamp}`;
const gradientColorByPercentage = `gradient-by-percentage-${timestamp}`;
const colorByValue = `colorbyValue-${timestamp}`;

const gradientColors = [
  { hex: '#ffff00', value: 4 }, // yellow
  { hex: '#800080', value: 22 }, // purple
  { hex: '#0097a7', value: 61 }, // blueish
];

const createCustomGradientColor = (
  name: string,
  gradientByValueOption: '#gradientWValueType' | '#gradientWPercentages'
) => {
  cy.get(CSSselector.site_admin_menus.site_admin_dropdown_toggle)
    .click()
    .next()
    .find(customsettings.locators.custom_colors)
    .click();
  cy.get(customsettings.locators.new_custom_clr).click();
  cy.get(dashboard.locators.custom_style_name)
    .clear()
    .type(name);

  const colorScaleTypeOptions = '#typeOfColorScale';
  cy.get(colorScaleTypeOptions).click();
  // const gradientByValueOption = '#gradientWValueType';
  cy.get(gradientByValueOption).click();

  // Check JQuery build in  spectrum picker - has 3 slider
  cy.get('.gradx_slider').should('have.length', 3);

  [0, 1, 2].forEach((id) => {
    cy.get(`#gradx_slider_${id}`).click();
    cy.get('.gradx_start_sliders .sp-container').should('be.visible');
    cy.get('.gradx_start_sliders .sp-picker-container .sp-input')
      .clear()
      .invoke('val', gradientColors[id].hex)
      .trigger('input')
      .trigger('change');
    cy.get('.gradx_start_sliders .sp-picker-container button')
      .contains('OK')
      .click();

    if (gradientByValueOption === '#gradientWValueType') {
      cy.get(`#customColorContentForType_100 #c${id}`)
        .clear()
        .invoke('val', gradientColors[id].value)
        .trigger('input')
        .trigger('change');
    }
  });

  cy.get(dashboard.locators.save_btn).click();
};

const checkCustomColorIsInTheColorList = (name: string) => {
  cy.get(dashboard.locators.save_btn).click();
  cy.get(CSSselector.site_admin_menus.site_admin_dropdown_toggle)
    .click()
    .next()
    .find(customsettings.locators.custom_colors)
    .click();
  cy.get(customsettings.locators.custom_colors_tbl).should('be.visible');
  cy.get(customsettings.locators.custom_clr_tbl_row).should('contain', name);
};

const deleteColor = (name: string) => {
  // Delete custom color

  cy.visit('/arc/apps/customcolors');

  // click on the custom color named Color 1's delete icon
  cy.get(customsettings.locators.custom_color_row.replace('color-name-placeholder', name))
    .find('a.delete')
    .click();
  cy.get(customsettings.locators.Delete_conf_modal)
    .clear()
    .type('DELETE');
  cy.get(customsettings.locators.Delete_btn).click();
  cy.get(customsettings.locators.clr_tbl_body).should('not.contain', name);
};

describe('Custom settings smoke tests.', () => {
  context('ARC-10791: Custom Color', () => {
    before(() => {
      arctest.logins.silent();
    });

    after(() => {
      deleteColor(colorByValue);
      deleteColor(gradientColorByValue);
      deleteColor(gradientColorByPercentage);
    });

    const dashboardName = `dash_${timestamp}_ARC-10791`;
    let totalVisualCount: number;
    const visualName = `Grouped Bar Visual ${timestamp}`;
    const visualSubtitle = `Subtitle for Grouped Bar Visual ${timestamp}`;
    const dimX = 'cereal_name';
    const dimY = 'calories';
    const xExpr = '[cereal_name] As "Cereals"';
    const yExpr = 'sum([calories]) As "Total Calories"';
    const fields = [
      'protein_grams',
      'fat_grams',
      'sodium_mg',
      'sugars_grams',
      'potassium_mg',
      'vitamins_and_minerals',
      'display_shelf',
    ];
    const customclr = [
      '#ff0000',
      '#ff9900',
      '#ffff00',
      '#00ff00',
      '#00ffff',
      '#0000ff',
      '#9900ff',
      '#ff00ff',
    ];
    const legendList = [
      'Total Calories',
      'sum(protein_grams)',
      'sum(fat_grams)',
      'sum(sodium_mg)',
      'sum(sugars_grams)',
      'sum(potassium_mg)',
      'sum(vitamins_and_min...',
      'sum(display_shelf)',
    ];
    const colorscheck = [
      'rgb(255, 0, 0)',
      'rgb(255, 153, 0)',
      'rgb(255, 255, 0)',
      'rgb(0, 255, 0)',
      'rgb(0, 255, 255)',
      'rgb(0, 0, 255)',
      'rgb(153, 0, 255)',
      'rgb(255, 0, 255)',
    ];

    it('1. Dashboard with different settings', () => {
      cy.visit('/');
      // Creating Custom Color
      functions.goToSiteAdminSettingsMenu('manage_custom_colors');

      cy.get(customsettings.locators.new_custom_clr).click();
      cy.get(dashboard.locators.custom_style_name)
        .clear()
        .type(colorByValue);
      cy.get(dashboard.locators.custom_style_desc)
        .clear()
        .type('Testing description of custom color');
      for (let i = 0; i < 7; i++) {
        cy.get(customsettings.locators.addmore_btn).click({ force: true });
      }
      cy.get(customsettings.locators.color_pick_caret).each(($el, i) => {
        cy.wrap($el).click();
        cy.get(customsettings.locators.custom_clr_textbox)
          .clear()
          .type(customclr[i]);
        cy.get(customsettings.locators.custom_clr_ok_btn).click();
      });
      cy.get(dashboard.locators.save_btn).click();
      functions.goToSiteAdminSettingsMenu('manage_custom_colors');
      cy.get(customsettings.locators.custom_colors_tbl).should('be.visible');
      cy.get(customsettings.locators.custom_clr_tbl_row).should('contain', colorByValue);

      // Create a bar visual in a new dashboard and add the custom color created

      functions.addNewDB(dashboardName);
      functions.createNewVisual('samples', 'Cereals');
      dashboard.saveDashboard();
      cy.get(CSSselector.visual_container).then(($visualcontainer) => {
        totalVisualCount = $visualcontainer.children().length;
        cy.intercept('POST', '/arc/reports/report/*').as('visualSubmitted');
        cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('visualUpdated');
        cy.intercept('POST', '/arc/reports/savedashboard').as('dashboardSaved');
        cy.get("div[class='gridster ready']>ul>li").then(($p2) => {
          expect($p2.children().length).to.equal(totalVisualCount);
        });
        cy.get("div[class='gridster ready']>ul>li:last").scrollIntoView();
        cy.get(CSSselector.visual_rows.replace('index', String(totalVisualCount))).should(
          'be.visible'
        );

        // Editing the Table visual to Grouped Bar visual having the data in fields such as X and Y axis and Colors shelf

        cy.get(CSSselector.visual_edit_specific(totalVisualCount)).click({
          force: true,
        });
        cy.get(customsettings.locators.grouped_bars_vs).click();
        cy.get(CSSselector.barVisual.x_Axis).click();
        arctest.screen.builder.modal.addShelfColumn('x_shelf', dimX);
        cy.get(dashboard.locators.dimension_first_select).click();
        visuals.verifySQLExpression(xExpr);
        cy.get(CSSselector.close_property_panel).click();
        cy.get(CSSselector.barVisual.y_Axis).click();
        cy.get(CSSselector.measure_field).click();
        arctest.screen.builder.modal.addShelfColumn('y_shelf', dimY);
        cy.get(dashboard.locators.measure_first_select).click();
        visuals.verifySQLExpression(yExpr);
        cy.get(CSSselector.color_field).click();
        fields.forEach((field) => {
          cy.get(CSSselector.color_field).click({ force: true });
          cy.get(CSSselector.vizual_fields_tab.replace('column-name', field)).click();
        });
        visuals.saveVisual();
        cy.get(CSSselector.visuals_title)
          .click()
          .type(visualName);
        cy.get(dashboard.locators.visual_subtitle)
          .click()
          .type(visualSubtitle);

        // Apply the newly created custom color in Grouped Bar visual

        cy.get(customsettings.locators.dashboard.visual_style_tab).click({ force: true }); // or cy.get(arcvizSelectors.visual_edit_first).click({ force: true });
        cy.get(dashboard.locators.color_tab).click();

        cy.get(customsettings.locators.color_palette)
          .scrollIntoView()
          .find(`[title="${colorByValue}"]`)
          .click();
        dashboard.closeVisual(false);
        dashboard.saveDashboard();
        cy.get(customsettings.locators.groupedbars).should(($el) => {
          return expect($el).to.exist;
        });
        cy.get(CSSselector.visuals_title).should(($el) => {
          assert.equal(
            $el.text(),
            `Grouped Bar Visual ${timestamp}`,
            'Visual name should be Grouped Bar Visual'
          );
        });
        cy.get(
          `#app-selected-list > ul > li:nth-child(${totalVisualCount}) [text-anchor='start']`
        ).should(($el) => {
          assert.equal($el.text(), 'Total Calories', 'Should be Total Calories');
        });
        cy.get(
          `#app-selected-list > ul > li:nth-child(${totalVisualCount}) [text-anchor='middle']`
        ).should(($el) => {
          assert.equal($el.text(), 'Cereals', 'Should be Cereals');
        });
        cy.get(
          'div.viz-permanent div.arc-legend-container > svg.arc-viz-legend > g.small-legend > text.arc-legend-label'
        ).each(($el, i, $list) => {
          expect($el.text()).to.eq(legendList[i]);
        });
        cy.get('div[class="gridster ready"]>ul li:nth-child(1) svg  g.arc-bar-group')
          .first()
          .find('rect')
          .each(($el, i) => {
            cy.wrap($el).should('have.css', 'fill', colorscheck[i]);
          });
      });
    });

    it('2. Create gradient value color', () => {
      createCustomGradientColor(gradientColorByValue, '#gradientWValueType');
      checkCustomColorIsInTheColorList(gradientColorByValue);
    });

    it('3. Create gradient value %', () => {
      createCustomGradientColor(gradientColorByPercentage, '#gradientWPercentages');
      checkCustomColorIsInTheColorList(gradientColorByPercentage);
    });

    it('4. Create Dashboard and check color gradient by value and percentage works', () => {
      // check on dashboards:
      dashboard.createDashboard(dashboardsDesc);
      dashboard.chooseVisualTab('colors', true, 1);
      cy.get(`[title="${gradientColorByValue}"]`)
        .scrollIntoView()
        .click();
      dashboard.chooseVisualTab('colors', true, 2);
      cy.get(`[title="${gradientColorByPercentage}"]`)
        .scrollIntoView()
        .click()
        .trigger('mouseout');
      cy.get(CSSselector.visualEdit.sidePanels.settingPanelCloseButton)
        .scrollIntoView()
        .click();
      cy.get(CSSselector.visualEdit.sidePanels.settingPanel).should('be.not.visible');

      cy.get(CSSselector.dashboardDetail.dashboardContainer).matchImage();
    });
  });
});
