import * as arctest from 'lib/arctest';
import * as customsettings from 'lib/customsettings';
import * as dashboard from 'lib/dashboard';
import * as functions from 'lib/functions';
import * as visuals from 'lib/visual';
import { CSSselector } from 'locators/common/CSSselector';

function formatDate(date: any) {
  const d = new Date(date);
  let month = '' + (d.getMonth() + 1);
  let day = '' + d.getDate();
  const year = d.getFullYear();

  if (month.length < 2) {
    month = '0' + month;
  }
  if (day.length < 2) {
    day = '0' + day;
  }
  return [year, month, day].join('-');
}

describe('Custom settings smoke tests.', () => {
  context('ARC-10800: Custom Dates', () => {
    before(() => {
      cy.fixture('users.json').then((data) => {
        arctest.logins.silent(data.admin.username, data.admin.password);
      });
    });

    const timestamp = new Date().getTime();
    const dashboardName = `dash_${timestamp}_10800`;
    const datename: any = 'Custom Date 1';
    const visualName = `Table Visual ${timestamp}`;
    const visualSubtitle = `Subtitle for Table Visual ${timestamp}`;
    const dimX = 'inspection_date';
    const dimY = 'zip_code';
    const dimX2 = 'county';
    const dimX3 = 'state';
    const dimX4 = 'city';
    const names = ['DATE1', 'DATE2'];
    const today = formatDate(Date());

    it('1. Dashboard with different settings', () => {
      functions.navigate_to_home();
      cy.get(CSSselector.Visual_page).click();
      cy.get(CSSselector.apps.favourites_tab).click();
      cy.get(CSSselector.Visuals_all).click();

      // Creating Custom Dates
      functions.goToSiteAdminSettingsMenu('manage_custom_dates');

      cy.get(customsettings.locators.new_custom_clr).click();
      cy.get(dashboard.locators.custom_style_name)
        .clear()
        .type(datename);

      for (let i = 0; i < 1; i++) {
        cy.get(customsettings.locators.add_range_btn).click();
      }
      cy.get(customsettings.locators.range_names).each(($el2, i) => {
        cy.wrap($el2).type(names[i]);
      });
      cy.get(customsettings.locators.start_date1).click();
      cy.get(customsettings.locators.start_date_input).click();
      cy.get(customsettings.locators.fixed_date_input).click();
      cy.get(customsettings.locators.fixed_date_input_txtbox)
        .click()
        .should('have.class', 'active');
      cy.get(customsettings.locators.custom_dates_calendar).each(($el) => {
        if ($el.css('display').includes('block')) {
          cy.wrap($el)
            .find('div.calendar.first.single.right select.yearselect')
            .select('2016');
          cy.wrap($el)
            .find('div.calendar.first.single.right select.monthselect')
            .select('Jan');
          cy.wrap($el)
            .find('table.table-condensed tbody tr:nth-child(1) td:nth-child(7)')
            .first()
            .click();
        }
      });
      cy.get(customsettings.locators.date_apply_btn).click();
      cy.get('tbody.custom-daterange-table-body > tr:nth-child(2)').click();
      cy.get(
        'tbody.custom-daterange-table-body > tr:nth-child(2) > td > .daterange-start-anchor-input'
      ).click();
      cy.get(customsettings.locators.customdates_dropdown).should('be.visible');
      cy.get(customsettings.locators.date_today).click();
      cy.get(customsettings.locators.date_apply_btn).click();
      cy.get(
        'tbody.custom-daterange-table-body > tr:nth-child(1) td input.daterange-end-anchor-input'
      ).click();
      cy.get(customsettings.locators.fixed_date_input).click();
      cy.get(customsettings.locators.date_apply_btn).click();
      cy.get(
        'tbody.custom-daterange-table-body > tr:nth-child(2) td input.daterange-end-anchor-input'
      ).click();
      cy.get(customsettings.locators.fixed_date_input).click();
      cy.get(customsettings.locators.date_apply_btn).click();
      cy.get(dashboard.locators.save_btn).click();
      functions.goToSiteAdminSettingsMenu('manage_custom_dates');
      cy.get(customsettings.locators.customdates_tbl).should('be.visible');
      cy.get(customsettings.locators.customdate_tbl_row).should('contain', datename);

      // Create a table visual in a new dashboard and newly created custom dates filter

      functions.addNewDB(dashboardName);
      functions.createNewVisual('samples', 'Food Stores Inspection in NYC');
      dashboard.saveDashboard();

      // Editing the Table visual to Grouped Bar visual having the data in fields such as X and Y axis and Colors shelf

      visuals.editSpecificVisual(1);
      cy.get(customsettings.locators.table_vs).click({ force: true });
      cy.get(customsettings.locators.shelves.appShelves.table_dimension).click();
      cy.get(CSSselector.vizual_fields_tab.replace('column-name', dimX)).click();
      cy.get(CSSselector.vizual_fields_tab.replace('column-name', dimX2)).click();
      cy.get(CSSselector.vizual_fields_tab.replace('column-name', dimX3)).click();
      cy.get(CSSselector.vizual_fields_tab.replace('column-name', dimX4)).click();
      cy.get(customsettings.locators.shelves.appShelves.table_measure).click();
      cy.get(CSSselector.vizual_fields_tab.replace('column-name', dimY)).click();
      cy.get(CSSselector.visuals_title)
        .click()
        .type(visualName);
      cy.get(dashboard.locators.visual_subtitle)
        .click()
        .type(visualSubtitle);
      visuals.saveVisual();
      dashboard.saveDashboard();

      // Apply the newly created custom dates on visual at dashboard level

      // cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('visualUpdated');
      // cy.intercept('GET', '/arc/reports/report/*').as('visualGet');
      // cy.intercept('POST', '/arc/reports/create/*').as('visualCreated');
      // cy.get(customsettings.locators.filter_tab).click();

      arctest.screen.builder.addFilter(['inspection_date']);
      // cy.get('#dash-filters-tab > ul.dim-app-params li.selection_element:nth-child(3)').click();
      // cy.wait('@visualCreated');
      // cy.wait('@visualGet');
      // cy.wait('@visualUpdated');
      // cy.wait('@visualUpdated');

      dashboard.saveDashboard();
      cy.get('li a[data-tooltip-contents="Configure filter"]').trigger('focusin', {
        force: true,
      });
      // cy.get('.control-container a[data-tooltip-contents="Configure filter"]').click({force : true});
      cy.get(customsettings.locators.filter_settings).click({ force: true });
      cy.get(customsettings.locators.filter_setting_modal).should('be.visible');
      cy.get(CSSselector.filters.settings_navtab)
        .contains('Ranges')
        .click();
      cy.get('div.clearfix div.pull-left').click();
      cy.get(customsettings.locators.date_dropdown).should('be.visible');
      cy.get(customsettings.locators.dateItem(datename)).click();
      cy.get(customsettings.locators.custom_dates_settings_save).click();
      dashboard.saveDashboard();
      cy.get('div.datetime-picker-filter-widget input.filter-date-range').click();
      cy.get(customsettings.locators.custom_dates_calendar).should('be.visible');
      cy.get('div.show-calendar ul li:nth-child(1)')
        .should('have.text', names[0])
        .click();
      cy.get(customsettings.locators.filter_date_range).click();
      cy.get(customsettings.locators.date_from).should('have.value', '2016-01-02');
      cy.get(customsettings.locators.date_to).should('have.value', today);
      cy.get('div.daterangepicker.opensright ul li:nth-child(2)')
        .should('have.text', names[1])
        .click();
      cy.get(customsettings.locators.date_from).should('have.value', today);
      cy.get(customsettings.locators.date_to).should('have.value', today);
      dashboard.saveDashboard();

      // Delete custom Date
      functions.goToSiteAdminSettingsMenu('manage_custom_dates');
      cy.get(customsettings.locators.delete_custom_date_icon('Custom Date 1')).click();
      cy.get(customsettings.locators.Delete_conf_modal)
        .clear()
        .type('DELETE');
      cy.get(customsettings.locators.Delete_btn).click();
      cy.get(customsettings.locators.customdates_tbl).should('not.contain', datename);
    });
  });
});
