import * as arctest from 'lib/arctest';
import * as styles from 'lib/customsettings';
import * as dashboard from 'lib/dashboard';
import * as functions from 'lib/functions';
import { modalClick } from 'lib/functions';
import * as logins from 'lib/logins';
import { locators } from 'lib/visual/locators';
import { CSSselector } from 'locators/common/CSSselector';
/*
 * ARC-10798: Automate smoke test for Custom Styles
 */

describe('Custom settings smoke tests.', () => {
  context('ARC-10800', () => {
    const legends = [
      '100%_Bran (70)',
      '100%_Natural_Bran (120)',
      'All-Bran (70)',
      'All-Bran_with_Extra_Fiber (50)',
      'Almond_Delight (110)',
    ];

    before(() => {
      logins.silent();
      cy.visit('/');
    });

    it('1 : create new custom styles', () => {
      functions.goToSiteAdminSettingsMenu('manage_custom_styles');
      cy.get(dashboard.locators.new_custom_stylebtn).click();
      cy.get(dashboard.locators.custom_style_name).type('css');
      cy.get(dashboard.locators.custom_style_desc).type('css style');
      cy.get(
        dashboard.locators.custom_style_area
      ).type('th{{}background-color: #336699;color: #ffffff;}', { force: true });
      cy.get(dashboard.locators.save_btn).click();
      cy.get(styles.locators.custom_styles).click();
      cy.get(dashboard.locators.new_custom_stylebtn).click();
      cy.get(dashboard.locators.custom_style_name).type('js');
      cy.get(dashboard.locators.custom_style_desc).type('js style');
      cy.get(styles.locators.javascript_option).click();
      const myValue = `return function() {
    var f = function() {};
    f.version = "1";
    f.settings = function() {
        return [
            {
                id: 'Square Size',
                defaultValue: '10',
            },
        ];
    };
    f.beforeDraw = function() {
        var rows = arcapi.dataResult().rows();
        rows.map(function (r) {
            if (r[0].toLowerCase() === 'other') {
                r[0] = 'Not Applicable';
            }
        });
        arcapi.dataResult().rows(rows);
    };
    f.afterDraw = function() {
        var rows = arcapi.dataResult().rows();
        var length = $('#' + arcapi.chartId() + ' .viz-base-legend-g').children('text').length;
        var total = d3.sum(rows, function(row) { return +row[1]; });
        var legendlabel;
        var fmt = d3.format(",");

        rows.forEach(function(d, i) {
            for (var c=0; c < length; c++ ){
                legendlabel = $('#' + arcapi.chartId() + ' .viz-base-legend-g text')[c].innerHTML;
                if(legendlabel == d[0]){
                    $('#' + arcapi.chartId() + ' .viz-base-legend-g text')[c].innerHTML = legendlabel + ' (' + fmt(+d[1]) + ')';
                }
            }
        });
    };
    return f;
}();`;
      cy.get(styles.locators.js_text_area).focus();
      cy.get(styles.locators.js_ace_editor).should('have.class', 'ace_focus');
      cy.get(styles.locators.js_ace_editor + '.ace_editor').then(($elem) => {
        cy.window()
          .its('ace' as any)
          .invoke('edit', $elem.attr('id'))
          .invoke('setValue', myValue);
      });
      cy.get(dashboard.locators.save_btn).click();
      cy.get(styles.locators.custom_styles);
    });

    it('2 : apply custom styles', () => {
      functions.navigate_to_home();
      cy.get(CSSselector.home_new_dashboard).click();
      functions.createNewVisual('sample', 'Cereals');
      // cy.get('.btn-new-visual').click();
      cy.get(styles.locators.dashboard.visual_style_tab).click({ force: true }); // or cy.get(arcvizSelectors.visual_edit_first).click({ force: true });
      cy.get(styles.locators.dashboard.custom_visual_style_tab).click();
      cy.get(styles.locators.dashboard.add_custom_style).click();
      cy.get(styles.locators.dashboard.search).type('css');
      cy.get(styles.locators.dashboard.checkbox).click();
      cy.get(dashboard.locators.css_add_button).click();
      cy.get(styles.locators.dashboard.table_header)
        .should('have.css', 'background-color', 'rgba(0, 0, 0, 0)')
        .should('have.css', 'color', 'rgb(51, 51, 51)');
      cy.get(dashboard.locators.dashboard_title)
        .click()
        .type('css_style_dashboard');
      dashboard.saveDashboard();
      functions.navigate_to_home();
      cy.get(CSSselector.home_new_dashboard).click();
      functions.createNewVisual('sample', 'Cereals');
      cy.get(CSSselector.viz_hover).click();
      cy.get(CSSselector.visual_edit_first).click();
      cy.get('.chart-icon-pie').click();
      arctest.screen.builder.modal.addShelfColumn('dimensions_shelf', 'cereal_name');
      arctest.screen.builder.modal.addShelfColumn('aggregates_shelf_popover', 'calories');

      cy.get(styles.locators.dashboard.visual_style_tab).click({ force: true });
      cy.get(styles.locators.dashboard.custom_visual_style_tab).click();
      cy.get(styles.locators.dashboard.add_custom_style).click();
      cy.get(styles.locators.visual.search).type('js');
      cy.get(styles.locators.visual.checkbox).click();
      cy.get(dashboard.locators.css_add_button).click();

      dashboard.saveVisual();
      cy.get(locators.visual.legendLabels).each(($el, i) => {
        cy.wrap($el).should('have.text', legends[i]);
      });
      cy.get(CSSselector.save_visual).click();
      cy.get(dashboard.locators.dashboard_title)
        .click()
        .type('js_style_dashboard');
      cy.get(CSSselector.Save).click();
    });

    it('3 : delete custom styles', () => {
      functions.goToSiteAdminSettingsMenu('manage_custom_styles');
      cy.get(styles.locators.search)
        .click()
        .type('css');
      cy.get(styles.locators.delete_button).click();
      cy.get(CSSselector.apps.delete_textbox).type('DELETE');
      modalClick(CSSselector.apps.deleteconf_button);
      cy.get(CSSselector.apps.deleteconf_button, { timeout: 30000 }).should('not.be.visible');

      cy.get(styles.locators.search)
        .click()
        .type('js');
      cy.get(styles.locators.delete_button).click();
      cy.get(CSSselector.apps.delete_textbox).type('DELETE');
      modalClick(CSSselector.apps.deleteconf_button);
      cy.get(CSSselector.apps.deleteconf_button, { timeout: 30000 }).should('not.be.visible');
    });
  });
});
