import { notifications } from 'fixtures/locators/notifications';
import { logins } from 'lib/arctest';
import { BackendAPI, DefaultRoleIds } from 'lib/backend-api';
import * as dashboard from 'lib/dashboard';
import * as functions from 'lib/functions';
import { getIDNumberFromAppName, modalClick } from 'lib/functions';
import { CSSselector } from 'locators/common/CSSselector';

function importSheet() {
  const searchBox = `${CSSselector.reportPickModal.modal} ${CSSselector.reportPickModal.searchBox}`;
  cy.get(`${CSSselector.addSheetMenu.menuIcon}`)
    .invoke('show')
    .click();
  cy.get(`${CSSselector.addSheetMenu.menuIcon} ${CSSselector.addSheetMenu.importSheet}`).click();
  cy.get(searchBox)
    .clear()
    .type('DBadmin-Pvt-Multisheet');
  cy.get(
    `${CSSselector.reportPickModal.modal} ${CSSselector.reportPickModal.sheetList} li:not(li:first-child)`
  ).should('be.hidden');
  cy.get(dashboard.locators.pickdashboard_modal_close).click();
  dashboard.saveDashboard();
}

const createNewDashboardUrl = '/arc/apps/builder/0?report_type=dashboard';

describe('ArcViz Automation - Multi Sheets Dashboard regression tests - task 3.', () => {
  let lifeExpectancyDBId = -1;
  const lifeExpectancyDB = 'Life Expectancy Dashboard';
  const datatsetList = ['Inspector View', 'Consumer View'];

  before(() => {
    cy.fixture('users.json').then((data) => {
      logins.silent(data.admin.username, data.admin.password);

      [data.dbadmin, data.sysadmin].forEach((user) => {
        BackendAPI.users.create({
          username: user.username,
          password: user.password,
          roleIds: [DefaultRoleIds.SystemAdmin],
        });
      });
    });

    getIDNumberFromAppName(lifeExpectancyDB).then((dbId) => {
      lifeExpectancyDBId = dbId;
    });
  });

  const TimeStamp = new Date().getTime();
  const DB1 = `DB1_${TimeStamp}_ARC-8103`;
  const DB2 = `Sysadmin_${TimeStamp}_ARC-8104`;
  const DB3 = `DB3_${TimeStamp}_ARC-8105`;
  const DB4 = `DB4_${TimeStamp}_ARC-8107`;
  const workspaceName = 'Public';

  // ARC-8103:Import sheet in current dashboard

  it('ARC-8103 : Import sheet in current dashboard', () => {
    cy.fixture('visuals.json').then((dash) => {
      cy.visit(createNewDashboardUrl);
      functions.createNewVisual(dash.data[2].dataconnection, dash.data[2].dataset);
      cy.get(CSSselector.Dashboard_title)
        .click()
        .type(DB1);
      cy.get(CSSselector.Dashboard_subtitle)
        .click()
        .type(DB1);
      dashboard.saveDashboard();
      dashboard.importExternalSheet(DB1);
      dashboard.saveDashboard();
      cy.get(dashboard.locators.sheet_title).should(($el) => {
        assert.equal($el.text(), 'Clone of Sheet 1', 'Clone of Sheet 1 sheet name is expected.');
      });
    });
  });

  // ARC-8104:Import a sheet in dashboard from other user's private workspace

  it('ARC-8104 : Import a sheet in dashboard from other users private workpace', () => {
    cy.fixture('users.json').then((data) => {
      logins.silent(data.sysadmin.username, data.sysadmin.password);
    });

    cy.fixture('visuals.json').then((dash) => {
      cy.visit(createNewDashboardUrl);
      functions.createNewVisual(dash.data[2].dataconnection, dash.data[2].dataset);
      cy.get(CSSselector.Dashboard_title)
        .click()
        .type(DB2);
      cy.get(CSSselector.Dashboard_subtitle)
        .click()
        .type(DB2);
      dashboard.saveDashboard();
      importSheet();
    });
  });

  // ARC-8105:Import a sheet in dashboard from public workspace

  it('ARC-8105 : Import a sheet in dashboard from public workspace', () => {
    cy.fixture('users.json').then((data) => {
      logins.silent(data.dbadmin.username, data.dbadmin.password);
    });
    const searchBox = `${CSSselector.reportPickModal.modal} ${CSSselector.reportPickModal.searchBox}`;
    cy.fixture('visuals.json').then((dash) => {
      cy.visit(createNewDashboardUrl);
      functions.createNewVisual(dash.data[2].dataconnection, dash.data[2].dataset);
      cy.get(CSSselector.Dashboard_title)
        .click()
        .type(DB3);
      cy.get(CSSselector.Dashboard_subtitle)
        .click()
        .type(DB3);
      dashboard.saveDashboard();
      cy.get(`${CSSselector.addSheetMenu.menuIcon}`)
        .invoke('show')
        .click();
      cy.get(`${CSSselector.addSheetMenu.menuIcon} ${CSSselector.addSheetMenu.addSheet}`).click();
      functions.createNewVisual(dash.data[2].dataconnection, dash.data[2].dataset);
      dashboard.saveDashboard();
      cy.get(
        `${CSSselector.dashboardDetail.menu} ${CSSselector.dashboardDetail.workspaceMenu}`
      ).click();
      cy.get(`[data-workspace-name="${workspaceName}"]:visible`).click();
      cy.get(CSSselector.dashboardDetail.workspaceMove).click();
      cy.get(notifications.alert_success).should('be.visible');
      cy.get(notifications.notification_close).click();

      /* After DBadmin users log out, Sysadmin user logs in to import dashboard
         moved by Dbadmin from Public workspace*/

      cy.fixture('users.json').then((data) => {
        logins.silent(data.sysadmin.username, data.sysadmin.password);
      });

      cy.visit(createNewDashboardUrl);
      functions.createNewVisual(dash.data[2].dataconnection, dash.data[2].dataset);
      cy.get(CSSselector.Dashboard_title)
        .click()
        .type(DB2);
      cy.get(CSSselector.Dashboard_subtitle)
        .click()
        .type(DB2);
      dashboard.saveDashboard();
      cy.get(`${CSSselector.addSheetMenu.menuIcon}`)
        .invoke('show')
        .click();
      cy.get(
        `${CSSselector.addSheetMenu.menuIcon} ${CSSselector.addSheetMenu.importSheet}`
      ).click();
      cy.get(searchBox)
        .clear()
        .type(DB3);
      cy.get(
        `${CSSselector.reportPickModal.modal} ${CSSselector.reportPickModal.sheetList} li:not(li:first-child)`
      ).should('be.hidden');
      cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('visualUpdated');
      cy.get('div.modal-dialog ul.app-avail-list > li div.dashboard-title-modal').each(($el) => {
        if ($el.text().trim() === DB3) {
          cy.wrap($el)
            .next()
            .within(() => {
              cy.get('select option[val="2"]').click({ force: true });
            });
        }
      });
      cy.wait('@visualUpdated')
        .its('response.statusCode')
        .should('eq', 200);
      dashboard.saveDashboard();
      cy.get(dashboard.locators.sheet_title).should(($el) => {
        assert.equal($el.text(), 'Clone of Sheet 1', 'Clone of Sheet 1 sheet name is expected.');
      });
    });
  });

  // ARC-8107:Verify "Pick a Dashboard" tab should display

  it('ARC-8107 : Verify "Pick a Dashboard" tab should display', () => {
    cy.fixture('users.json').then((data) => {
      logins.silent(data.admin.username, data.admin.password);
    });

    const searchBox = `${CSSselector.reportPickModal.modal} ${CSSselector.reportPickModal.searchBox}`;
    cy.fixture('visuals.json').then((dash) => {
      cy.visit(createNewDashboardUrl);
      functions.createNewVisual(dash.data[2].dataconnection, dash.data[2].dataset);
      cy.get(CSSselector.Dashboard_title)
        .click()
        .type(DB4);
      cy.get(CSSselector.Dashboard_subtitle)
        .click()
        .type(DB4);
      dashboard.saveDashboard();

      cy.get(`${CSSselector.addSheetMenu.menuIcon}`)
        .invoke('show')
        .click();
      cy.get(
        `${CSSselector.addSheetMenu.menuIcon} ${CSSselector.addSheetMenu.importSheet}`
      ).click();
      cy.get(searchBox).should('be.visible');
      modalClick(dashboard.locators.pickdashboard_modal_close);

      functions.navigate_to_data();
      const dashList: any = [];
      cy.get(dashboard.locators.visuals_sort).click({ force: true });
      cy.get('table#datasets-table tbody tr td.sorting_1').each(($el) => {
        if ($el.text().trim() === '0') {
          cy.wrap($el)
            .parent()
            .find('td')
            .first()
            .find('span.dataset-title')
            .then(($name) => {
              dashList.push($name.text());
            });
        }
      });
      dashboard.openDashboardforEdit(DB4);
      cy.get(`${CSSselector.addSheetMenu.menuIcon}`)
        .invoke('show')
        .click();
      cy.get(
        `${CSSselector.addSheetMenu.menuIcon} ${CSSselector.addSheetMenu.importSheet}`
      ).click();
      cy.get(searchBox)
        .should('be.visible')
        .clear();

      cy.get('#target-nav-selector h4 span:nth-child(2)').each(($el1) => {
        $el1.text().trim();
        expect($el1.text()).not.eq(dashList[0]);
        expect($el1.text()).not.eq(dashList[1]);
      });
      cy.get(searchBox)
        .clear()
        .type('"Restaurant Inspection SF"');
      cy.get(dashboard.locators.results_list).then(($el) => {
        expect($el.text()).to.contain(datatsetList[0]);
        expect($el.text()).to.contain(datatsetList[1]);
      });
      cy.get(searchBox)
        .clear()
        .type(`"${lifeExpectancyDBId}"`);
      cy.wait(1000); // need time to re-run the search

      cy.get(CSSselector.reportPickModal.modal)
        .contains(lifeExpectancyDB)
        .scrollIntoView()
        .then(($el) => {
          // searching the results_list like in the other case might fail when there are too many hits
          // and the result is outside of the visible area
          expect($el.text()).to.contain(lifeExpectancyDB);
        });

      cy.get(searchBox)
        .clear()
        .type('"dbadmin"');
      cy.wait(1000);
      cy.get(dashboard.locators.results_list).then(($el) => {
        expect($el.text()).to.contain(DB3);
      });
    });
  });
});
