import { logins } from 'lib/arctest';
import * as dashboard from 'lib/dashboard';
import * as functions from 'lib/functions';
import { modalClick } from 'lib/functions';
import { CSSselector } from 'locators/common/CSSselector';

describe('ArcViz Automation - Multi Sheets Dashboard regression tests - task 4.', () => {
  before(() => {
    cy.fixture('users.json').then((data) => {
      logins.silent(data.admin.username, data.admin.password);
    });
  });

  const TimeStamp = new Date().getTime();
  const DB1 = `DB1_${TimeStamp}_Arcadia`;
  const sheetName1 = `db1_${TimeStamp}ARC`;
  const sheetName2 = `db2_${TimeStamp}ARC`;
  const Dimx = 'cereal_name';
  const Dimy = 'calories';

  it('ARC-8108 : Clone Sheet having linked visuals', () => {
    cy.fixture('visuals.json').then((dash) => {
      cy.visit('/arc/apps/builder/0?report_type=dashboard');
      functions.createNewVisual(dash.data[2].dataconnection, dash.data[2].dataset);

      cy.get(CSSselector.Dashboard_title)
        .click()
        .type(DB1);
      cy.get(CSSselector.Dashboard_subtitle)
        .click()
        .type(DB1);
      dashboard.renameSheet(1, sheetName1);

      dashboard.saveDashboard();

      cy.get(CSSselector.visual_edit_first).click({ force: true });
      cy.get(CSSselector.visualEdit.ellipsis).click();
      cy.get(CSSselector.visualEdit.linkUnlink).click();

      modalClick(CSSselector.visualEdit.linkUnlinkConfirm);

      dashboard.saveDashboard();

      cy.get(
        `${CSSselector.dashboardDetail.dashboardContainer} li.gs-w:first-child ${CSSselector.dashboardDetail.linkVisualIcon}`
      ).should('exist');

      dashboard.cloneSheet(1);
      cy.get('.clone-option:visible input').click();

      modalClick(CSSselector.dashboardDetail.workspaceMoveMoveNow);

      dashboard.saveDashboard();

      cy.get(`li[class*='active'] .sheet-title`).should(($el) => {
        assert.equal($el.text(), 'Clone of ' + sheetName1, ' sheet name is expected.');
      });

      cy.get(
        `${CSSselector.dashboardDetail.dashboardContainer} li.gs-w:nth-child(2) ${CSSselector.dashboardDetail.linkVisualIcon}`
      ).should('not.exist');

      cy.get('.arc-viz-container.arc-viz-type-table').should('be.visible');
    });
  });

  it('ARC-8133 : Adding the linked visual with maintain link option', () => {
    dashboard.cloneSheet(1);
    cy.get(CSSselector.dashboardDetail.workspaceMoveMaintainVisualRadio).click();

    modalClick(CSSselector.dashboardDetail.workspaceMoveMoveNow);

    dashboard.saveDashboard();

    cy.get(`li[class*='active'] .sheet-title`).should(($el) => {
      assert.equal($el.text(), 'Clone of ' + sheetName1, 'sheet name is expected.');
    });
    cy.get(
      `${CSSselector.dashboardDetail.dashboardContainer} li.gs-w:first-child ${CSSselector.dashboardDetail.linkVisualIcon}`
    ).should('exist');
    cy.get('.arc-viz-container.arc-viz-type-table').should('be.visible');

    cy.get(CSSselector.visual_edit_first).click({ force: true });
    dashboard.activateBuildTab();

    cy.get(CSSselector.bar_visual).click();
    cy.get(CSSselector.barVisual.x_Axis).click();
    cy.get(CSSselector.vizual_fields_tab.replace('column-name', Dimx)).click();
    cy.get(CSSselector.barVisual.y_Axis).click();
    cy.get(CSSselector.vizual_fields_tab.replace('column-name', Dimy)).click();
    cy.get(CSSselector.refresh_visual).click();

    dashboard.saveDashboard();

    cy.get(`.sheet-handle:nth-child(1)`).click();

    dashboard.saveDashboard();
  });

  it('ARC-8131 : Deletion of sheet which has linked and unlinked visual', { retries: 2 }, () => {
    cy.fixture('visuals.json').then((dash) => {
      cy.get('#add-dash-sheet ' + CSSselector.addSheetMenu.moresheet).click();
      cy.get(`.sheet-handle:nth-child(4)`).should(($el) => {
        expect($el).to.have.class('sheet-handle active');
      });

      dashboard.renameSheet(4, sheetName2);

      functions.createNewVisual(dash.data[2].dataconnection, dash.data[2].dataset);

      dashboard.saveDashboard();

      cy.get(CSSselector.visual_edit_first).click({ force: true });

      cy.get(CSSselector.visualEdit.ellipsis).click();
      cy.get(CSSselector.visualEdit.linkUnlink).click();

      modalClick(CSSselector.visualEdit.linkUnlinkConfirm);

      dashboard.saveDashboard();

      cy.get(
        `${CSSselector.dashboardDetail.dashboardContainer} li.gs-w:first-child ${CSSselector.dashboardDetail.linkVisualIcon}`
      ).should('exist');

      cy.get(CSSselector.selecting_visuals).click({ force: true });

      // unlinked visual sheet delete
      dashboard.deleteSheet(2);
      dashboard.saveDashboard();
      cy.get('.sheet-title').should('have.length', 3);

      // linked visual in it which is only in this sheet and not other sheets of this dashboard)
      dashboard.deleteSheet(3);
      dashboard.saveDashboard();
      cy.get('.sheet-title').should('have.length', 2);

      // linked visual in it which is used in other sheets of the same dashboard
      dashboard.deleteSheet(2);
      dashboard.saveDashboard();
      cy.get('.sheet-title').should('have.length', 1);
    });
  });
});
