import * as apps from 'lib/apps';
import { logins } from 'lib/arctest';
import * as arctest from 'lib/arctest/screen/builder';
import * as dashboard from 'lib/dashboard';
import { FilterScopeTypes, FilterVisibilityTypes } from 'lib/dashboard/dashboards_interfaces';
import * as filter from 'lib/filter';
import * as functions from 'lib/functions';
import { waitForVisualsLoaded } from 'lib/functions';
import * as dataset from 'lib/permissions';
import * as visuals from 'lib/visual';
import { CSSselector } from 'locators/common/CSSselector';

function get_time() {
  const a = new Date();
  return a.getTime();
}

function addSheet() {
  cy.get(`${CSSselector.addSheetMenu.menuIcon}`)
    .invoke('show')
    .click();
  cy.get(`${CSSselector.addSheetMenu.menuIcon} ${CSSselector.addSheetMenu.addSheet}`).click();
  dashboard.saveDashboard();
}

describe('ARC-8163: ArcViz Automation - Multi Sheets Dashboard regression tests - task 5.', () => {
  const ConnectionType = 'samples';
  const Dataset = 'Cereals';
  let datasetID: number = 0;
  let vizID: number;

  before(() => {
    cy.fixture('users.json').then((data) => {
      cy.intercept('GET', '/arc/roles/roles').as('datasetLoaded');
      logins.silent(data.admin.username, data.admin.password);
    });
    // get dataset id
    functions.navigate_to_data();
    cy.get(CSSselector.data.connection_list).each(($el, index, $list) => {
      if ($el.text() === ConnectionType) {
        cy.wrap($el).click();
      }
    });
    dataset.selectDataset(ConnectionType, Dataset);
    cy.wait('@datasetLoaded');
    cy.get('#dataset-id').then(($id) => {
      datasetID = Number($id.text().trim());
    });
  });

  const DB1 = 'Target_DB_multiplesheets_filter ' + get_time();
  const DB2 = 'Source_DB_multiplesheets ' + get_time();
  const DB3 = 'Appscope_DB_multisheets ' + get_time();
  const DB4 = 'Appscope_DB2_multisheets ' + get_time();
  const APP = 'App_with_Appscope_filter ' + get_time();
  const TAB1 = 'Tab1 ' + get_time();
  const Dimfilter = 'cereal_name';

  const visual = arctest.locators.visuals + ':nth-child(1)';
  const baseurl = Cypress.config().baseUrl;
  let appidurl = '';
  let appid = '';

  it('ARC-8111: Linking the dashboards with parameter passing enabled', () => {
    cy.fixture('visuals.json').then((dash) => {
      cy.intercept('POST', '/arc/reports/report/*').as('visualSubmitted');
      cy.intercept('POST', '/arc/reports/*').as('dashboardSave');

      // Create a Target dashboard with multiple sheets and filter in visual of sheet 1
      functions.navigate_to_home();
      cy.get(CSSselector.home_new_dashboard).click();
      functions.createNewVisual(dash.data[2].dataconnection, dash.data[2].dataset);
      cy.get(CSSselector.Dashboard_title)
        .click()
        .type(DB1);
      cy.get(CSSselector.Dashboard_subtitle)
        .click()
        .type(DB1);
      dashboard.saveDashboard();

      cy.get(CSSselector.visual_edit_specific(1)).click({
        force: true,
      });
      // add filter in first visual of sheet 1
      cy.get(CSSselector.visualEdit.visual_filter_shelf).click();
      cy.get(CSSselector.vizual_fields_tab.replace('column-name', Dimfilter)).click();
      visuals.closeVisual(false);

      // add sheet 2 with a visual from same dataset and apply filter to that visual too
      addSheet();
      functions.createNewVisual(dash.data[2].dataconnection, dash.data[2].dataset);
      cy.get(CSSselector.visual_edit_specific(1)).click({
        force: true,
      });
      // add same filter in first visual of sheet 2
      cy.get(CSSselector.visualEdit.visual_filter_shelf).click();
      cy.get(CSSselector.vizual_fields_tab.replace('column-name', Dimfilter)).click();
      visuals.closeVisual(false);

      // Save Target Dashboard with two sheets each visual having same filter
      dashboard.saveDashboard();

      // Create a Source dashboard with multiple sheets and filter in visual of sheet 1
      functions.navigate_to_home();
      cy.get(CSSselector.home_new_dashboard).click();
      functions.createNewVisual(dash.data[2].dataconnection, dash.data[2].dataset);
      dashboard.editVisual();
      // get visual id
      cy.get(CSSselector.visualEdit.ellipsis).click();
      cy.get('div.visual-id span').then(($id) => {
        vizID = Number($id.text().trim());
        dashboard.closeVisual(false);
        cy.get(CSSselector.Dashboard_title)
          .click()
          .type(DB2);
        cy.get(CSSselector.Dashboard_subtitle)
          .click()
          .type(DB2);

        // Add same filter as present in Target Dashboard's visual, to sheet 1 and save it
        functions.addFilter('cereal_name', 'samples', 'Cereals');
        dashboard.saveDashboard();

        // Add sheet 2 same as sheet 1
        addSheet();
        // Click on visuals tab on side panel
        cy.get("a[data-target='#dash-viz-tab']").click();
        functions.createNewVisual(dash.data[2].dataconnection, dash.data[2].dataset);
        functions.addFilter('cereal_name', 'samples', 'Cereals');

        dashboard.saveDashboard();

        // go back to sheet 1
        cy.get('.sheet-handle:nth-child(1)').click();

        waitForVisualsLoaded();
        // to enable navigate to dashboard from dashboard edit modal
        cy.get(
          '#app-selected-list ul li:nth-child(1) div.dataTables_scrollBody tbody tr:nth-child(3) td:nth-child(1)'
        ).click();

        cy.get('.contextmenu:visible')
          .find('div.contextmenu-button-label.cm-navigateclick')
          .click({
            force: true,
          });
        cy.get(CSSselector.navigateToDashboardModal.navToDashPassParams).click();
        cy.get(CSSselector.navigateToDashboardModal.searchbox).type(DB1);
        cy.get('div.app-title.dashboard-title-modal').each(($el) => {
          if ($el.text().trim() === DB1) {
            cy.wrap($el).click({ force: true });
            dashboard.saveDashboard();
          }
        });

        // Assertion 1: Assert Target Name, Filtered values and Get URL has the filter value for sheet 1 and sheet 2
        // click on the source Dashboard
        dashboard.openDashboard(DB2);
        cy.get(
          '#app-selected-list ul li:nth-child(1) div.dataTables_scrollBody tbody tr:nth-child(3) td:nth-child(1)'
        ).click();
        // Assert Target Name
        cy.get(arctest.locators.dashboard_title).should('have.text', DB1);
        // Assert Filtered Values should have All-Bran and only one row in both the sheets
        for (let i = 1; i < 3; i++) {
          if (i === 2) {
            arctest.switchToSheet(i);
          }
          arctest.verifyTableContents(visual, [
            [
              'All-Bran',
              'K',
              'C',
              '70',
              '4',
              '1',
              '260',
              '9',
              '7',
              '5',
              '3',
              '320',
              '25',
              '1',
              '0.33',
              'K',
              'Kelloggs',
            ],
          ]);
          // Assert URL contains the filtered value
          cy.url().then(($el) => {
            const regex1 = /\bparam\.cereal_name=All-Bran\b/;
            const regex2 = /22filterName%22:%22cereal_name%22/;

            cy.wrap($el).should('match', regex1);
            cy.wrap($el).should('match', regex2);

            return; // ignore the old check below
            appidurl = $el.split('/')[6];
            appid = appidurl.split('?')[0];
            cy.url().should(
              'eq',
              baseurl +
                '/arc/apps/app/' +
                appid +
                '?param.cereal_name=All-Bran&scope=[{%22filterName%22:%22cereal_name%22,%22datasetId%22:' +
                datasetID +
                ',%22filterId%22:' +
                vizID +
                '}]'
            );
          });
        }

        // Assertion 2: Go to Source Dashboard and pass different parameter through URL and assert Filtered values for sheet 1 and sheet 2
        // Go to Source dashboard and pass parameters in URL in sheet 1
        dashboard.openDashboard(DB2);
        cy.url().then(($el1) => {
          appidurl = $el1.split('/')[6];
          appid = appidurl.split('?')[0];
          cy.visit(baseurl + '/arc/apps/app/' + appid + '?param.cereal_name=100%25_Bran');
          waitForVisualsLoaded();

          // Assert Filtered Values should have 100%_Bran and only one row
          for (let i = 1; i < 3; i++) {
            if (i === 2) {
              arctest.switchToSheet(i);
            }
            arctest.verifyTableContents(visual, [
              [
                '100%_Bran',
                'N',
                'C',
                '70',
                '4',
                '1',
                '130',
                '10',
                '5',
                '6',
                '3',
                '280',
                '25',
                '1',
                '0.33',
                'N',
                'Nabisco',
              ],
            ]);
          }
        });

        // Assertion 3: now change filter value and assert Filtered values and Get URL has the filter value for sheet 2
        // At Source Dashboard and select All-Bran_with_Extra_Fiber
        cy.get(
          'div.picklist-list-wrapper.dropdown-menu ul.picklist-list li:nth-child(4) input[type=checkbox]'
        ).check({ force: true });
        cy.get(
          'div.picklist-list-wrapper.dropdown-menu ul.picklist-list li:nth-child(1) input[type=checkbox]'
        ).uncheck({ force: true });
        // Assert Filtered Values should have All-Bran and only one row
        waitForVisualsLoaded();
        arctest.verifyTableContents(visual, [
          [
            'All-Bran_with_Extra_Fiber',
            'K',
            'C',
            '50',
            '4',
            '0',
            '140',
            '14',
            '8',
            '0',
            '3',
            '330',
            '25',
            '1',
            '0.5',
            'K',
            'Kelloggs',
          ],
        ]);
      });
    });
  });

  it('ARC-8113: Creating app with multiple multi sheet dashboards and set filters in sheets', () => {
    cy.fixture('visuals.json').then((dash) => {
      cy.intercept('POST', '/arc/reports/report/*').as('visualSubmitted');

      // Create a Dashboard with mutiple sheets and filter in sheet 1 with appscope
      functions.navigate_to_home();
      cy.get(CSSselector.home_new_dashboard).click();
      functions.createNewVisual(dash.data[2].dataconnection, dash.data[2].dataset);
      cy.get(CSSselector.Dashboard_title)
        .click()
        .type(DB3);
      cy.get(CSSselector.Dashboard_subtitle)
        .click()
        .type(DB3);
      functions.createNewVisual(dash.data[0].dataconnection, dash.data[0].dataset);

      // Add same filter with appscope to sheet 1 and save it
      functions.addFilter('cereal_name', 'samples', 'Cereals');
      // Set appscope for the filter
      filter.configureFilterScope(FilterVisibilityTypes.APP, FilterScopeTypes.CURRENT);

      cy.wait(1000);
      // Save Dashboard
      dashboard.saveDashboard();

      // Add sheet 2 and add 2 visuals
      addSheet();
      // Click on visuals tab on side panel
      cy.get("a[data-target='#dash-viz-tab']").click();
      functions.createNewVisual(dash.data[2].dataconnection, dash.data[2].dataset);
      functions.createNewVisual(dash.data[1].dataconnection, dash.data[1].dataset);
      cy.wait(2000);
      dashboard.saveDashboard();

      // Create another Dashboard with mutiple sheets without filter but same dataset for sheet 1 and 2
      functions.navigate_to_home();
      cy.get(CSSselector.home_new_dashboard).click();
      functions.createNewVisual(dash.data[2].dataconnection, dash.data[2].dataset);
      cy.get(CSSselector.Dashboard_title)
        .click()
        .type(DB4);
      cy.get(CSSselector.Dashboard_subtitle)
        .click()
        .type(DB4);
      functions.createNewVisual(dash.data[1].dataconnection, dash.data[1].dataset);
      cy.wait(1000);
      dashboard.saveDashboard();

      // Add sheet 2 and add 1 visual
      addSheet();
      // Click on visuals tab on side panel
      functions.createNewVisual(dash.data[2].dataconnection, dash.data[2].dataset);
      cy.wait(2000);
      dashboard.saveDashboard();

      // Create app and add DB3,DB4 provided DB3 has filter with app scope
      apps.createNewApp(APP, TAB1);
      apps.addDashboard(ConnectionType, Dataset, DB3);
      apps.addDashboard(ConnectionType, Dataset, DB4);
      apps.saveApp();

      // Assertion : Launch App, select a filter value and assert filter is applied to all Dashboards

      // Get app ID and visit the URL
      cy.url().then(($el2) => {
        appid = $el2.split('appedit/')[1];
        cy.visit(baseurl + '/arc/apps/appgroup/' + appid);
        waitForVisualsLoaded();

        // Select a filter
        cy.get(
          'div.picklist-list-wrapper.dropdown-menu ul.picklist-list li:nth-child(3) input[type=checkbox]'
        ).check({ force: true });

        // Assert 1: Assert filter on Sheet 1 of Dashboard 1
        arctest.verifyTableContents(visual, [
          [
            'All-Bran',
            'K',
            'C',
            '70',
            '4',
            '1',
            '260',
            '9',
            '7',
            '5',
            '3',
            '320',
            '25',
            '1',
            '0.33',
            'K',
            'Kelloggs',
          ],
        ]);
        // assert visual 2 is not filtered
        cy.get('#app-selected-list ul li:nth-child(2) div.dataTables_scrollBody tbody tr').then(
          ($row) => {
            expect($row).not.to.have.length(1);
          }
        );

        // Assert 2: Assert filter on Sheet 2 of Dashboard 1
        arctest.switchToSheet(2);
        arctest.verifyTableContents(visual, [
          [
            'All-Bran',
            'K',
            'C',
            '70',
            '4',
            '1',
            '260',
            '9',
            '7',
            '5',
            '3',
            '320',
            '25',
            '1',
            '0.33',
            'K',
            'Kelloggs',
          ],
        ]);

        // Assert 3: Assert filter on Sheet 1 of Dashboard 2
        // Click on 2nd Dashboard
        cy.get('.nav.navbar-nav').click();
        cy.get('.nav.navbar-nav ul.dropdown-menu li:nth-child(2) a').click({
          force: true,
        });
        cy.wait(5000);
        arctest.verifyTableContents(visual, [
          [
            'All-Bran',
            'K',
            'C',
            '70',
            '4',
            '1',
            '260',
            '9',
            '7',
            '5',
            '3',
            '320',
            '25',
            '1',
            '0.33',
            'K',
            'Kelloggs',
          ],
        ]);
        // assert visual 2 is not filtered
        cy.get('#app-selected-list ul li:nth-child(2) div.dataTables_scrollBody tbody tr').then(
          ($row) => {
            expect($row).not.to.have.length(1);
          }
        );

        // Assert 4: Assert filter on Sheet 2 of Dashboard 2
        arctest.switchToSheet(2);
        arctest.verifyTableContents(visual, [
          [
            'All-Bran',
            'K',
            'C',
            '70',
            '4',
            '1',
            '260',
            '9',
            '7',
            '5',
            '3',
            '320',
            '25',
            '1',
            '0.33',
            'K',
            'Kelloggs',
          ],
        ]);

        // Go back to home page to logout
        cy.visit(baseurl + '/arc/apps/');
      });
    });
  });
});
