import * as arctest from 'lib/arctest';
import { locators as csLocators } from 'lib/customsettings/locators';
import * as dashboard from 'lib/dashboard';
import { deleteSegment } from 'lib/datasetManagement';
/* import { ShelfTypes } from 'lib/dashboard';
import * as ddfunctions from 'lib/deriveddata';
import * as filter from 'lib/filter'; */
import * as functions from 'lib/functions';
import { waitForVisualsLoaded } from 'lib/functions';
import * as dsPermissions from 'lib/permissions';
import * as visuals from 'lib/visual';
import { CSSselector } from 'locators/common/CSSselector';

const timestamp = new Date().getTime();
const dashboardName = `dash_${timestamp}_ARC-8136`;
let totalVisualCount: number;
const labels = ['Text Color', 'Background Color', 'Page Background Color'];
const colors: any = ['#0000ff', '#fff2cc', '#741b47'];
const dashtitle: any = ['Text Color', 'Background Color'];
const dashtitlecolors: any = ['#ff0000', '#b6d7a8'];
const axeslabel = [
  'Axis Label Font Size',
  'Axis Ticks Font Size',
  'Label Font Size',
  'Label Text Color',
  'Trellis Font Size',
  'Legend Font Size',
];
const axeslabelvl = ['20', '20', '20', '#00ff00', '20', '20'];
const filtertitle: any = [
  'Text Color',
  'Selected Text Color',
  'Container Color',
  'Background Color',
];
const filterclr: any = ['#ff0000', '#ff00ff', '#ed8111', '#00ffff'];
const stylename: any = 'Style 1';
const visualName = `Bar Visual ${timestamp}`;
const visualSubtitle = `Subtitle for Bar Visual ${timestamp}`;
const dimX = 'cereal_name';
const dimY = 'calories';
const xExpr = '[cereal_name] As "Cereals"';
const yExpr = 'sum([calories]) As "Total Calories"';
const color = 'protein_grams';
const dimY2 = 'fat_grams';
const legendList = ['Value', 'sum(protein_grams)'];

const segment1 = 'Segment 1';
let segmentCreated = false;

function applyFilterToVisual(filterParam: string[]) {
  cy.get(CSSselector.filters.filter_dropdown).click();

  for (let i = 0, len = filterParam.length; i < len; i++) {
    cy.get(`input[data-value="${filterParam[i]}"]`)
      .scrollIntoView()
      .click();
  }
  waitForVisualsLoaded();

  cy.get(CSSselector.dashboard.alert_container).should('not.be.visible');
  cy.get(CSSselector.filters.filter_dropdown).click();
}

function createSpecificDashboard() {
  functions.navigate_to_home();
  cy.get(CSSselector.Visual_page).click();
  cy.get(CSSselector.apps.favourites_tab).click();
  cy.get(CSSselector.Visuals_all).click();
  functions.addNewDB(dashboardName);
  functions.createNewVisual('samples', 'Cereals');
  dashboard.saveDashboard();

  cy.get(CSSselector.visual_container).then(($visualcontainer) => {
    totalVisualCount = $visualcontainer.children().length;

    cy.intercept('POST', '/arc/reports/report/*').as('visualSubmitted');
    cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('visualUpdated');
    cy.intercept('POST', '/arc/reports/savedashboard').as('dashboardSaved');
    cy.get("div[class='gridster ready']>ul>li").then(($p2) => {
      expect($p2.children().length).to.equal(totalVisualCount);
    });
    cy.get("div[class='gridster ready']>ul>li:last").scrollIntoView();
    cy.get(CSSselector.visual_rows.replace('index', String(totalVisualCount))).should('be.visible');

    cy.get(CSSselector.visual_edit_specific(totalVisualCount)).click({
      force: true,
    });
    cy.get(CSSselector.bar_visual).click({ force: true });

    dashboard.activateBuildTab();

    cy.get(CSSselector.barVisual.x_Axis).click();
    cy.get(CSSselector.vizual_fields_tab.replace('column-name', dimX)).click();

    cy.get(dashboard.locators.dimension_first_select).click();
    visuals.verifySQLExpression(xExpr);
    cy.get(CSSselector.close_property_panel).click();

    cy.get(CSSselector.barVisual.y_Axis).click();
    cy.get(CSSselector.measure_field).click();
    cy.get(CSSselector.vizual_fields_tab.replace('column-name', dimY)).click();
    cy.get(dashboard.locators.measure_first_select).click();
    visuals.verifySQLExpression(yExpr);
    cy.get(CSSselector.measure_field).click();
    cy.get(CSSselector.vizual_fields_tab.replace('column-name', dimY2)).click();

    cy.get(CSSselector.color_field).click();
    cy.get(CSSselector.vizual_fields_tab.replace('column-name', color)).click();

    visuals.saveVisual();

    cy.get(CSSselector.visuals_title)
      .click()
      .type(visualName);

    cy.get(dashboard.locators.visual_subtitle)
      .click()
      .type(visualSubtitle);

    cy.get(csLocators.dashboard.visual_style_tab).click();
    cy.get(dashboard.locators.color_tab).click();
    cy.get('div.colorpalettes-setting > span:nth-child(8)').click();

    dashboard.closeVisual(false);
    dashboard.saveDashboard();
    cy.get(CSSselector.visual_bar_verify).should(($el) => {
      return expect($el).to.exist;
    });
    cy.get(CSSselector.visual_title.replace('index', String(totalVisualCount))).should(($el) => {
      assert.equal($el.text(), `Bar Visual ${timestamp}`, 'Visual name should be Bar Visual');
    });
    if (false) {
      // TODO: trellis table Measure label not displayed
      cy.get('div.viz-permanent table.viz-trellis-table tbody td > span').should(
        'have.text',
        'Measure'
      );
    }
    cy.get(
      'div.viz-permanent table.viz-trellis-table tbody tr:nth-child(2) > td.viz-td-header'
    ).should('have.text', 'Total Calories');
    cy.get(
      'div.viz-permanent table.viz-trellis-table tbody tr:nth-child(3) > td.viz-td-header'
    ).should('have.text', 'sum(fat_grams)');
    cy.get('div[class="gridster ready"]>ul li:nth-child(1) svg  > g.viz-bar-g >  text.x').each(
      ($el) => {
        expect($el.text()).to.eq('Cereals');
      }
    );
  });
}

// ARC-8136 : Dashboard with different settings
// Adding custom styles to dashboard on dashboard edit mode and asserting the setting changes to it accordingly
describe('ArcViz Automation - Multi Sheets Dashboard regression tests - task 6.', () => {
  before(() => {
    cy.fixture('users.json').then((data) => {
      arctest.logins.silent(data.admin.username, data.admin.password);
    });
    createSpecificDashboard();
  });

  after(() => {
    if (segmentCreated) {
      functions.navigate_to_data();
      dsPermissions.selectDataset('samples', 'Cereals');
      cy.get(dashboard.locators.segments).click();
      deleteSegment(segment1);
    }
  });

  context('Dashboard Settings', () => {
    it('1. Dashboard Basics Setting', () => {
      cy.get(dashboard.locators.dashboard_stylemenu).click();
      cy.get(dashboard.locators.dash_basics).click();
      labels.forEach((label, i) => {
        cy.get(`${dashboard.locators.dash_basics_labels}[title='${label}']`)
          .next()
          .click();
        cy.get(dashboard.locators.color_textbox)
          .click()
          .clear()
          .type(colors[i]);
        cy.get(dashboard.locators.color_ok_btn).click();
      });
      cy.get(dashboard.locators.dash_panel).should(
        'have.css',
        'background-color',
        'rgb(116, 27, 71)'
      );
      cy.get(dashboard.locators.dash_applist).should('have.css', 'color', 'rgb(0, 0, 255)');
      cy.get(dashboard.locators.dash_container).should(
        'have.css',
        'background-color',
        'rgb(255, 242, 204)'
      );
      dashboard.saveDashboard();
      cy.get(dashboard.locators.dashboard_stylemenu).click();
    });

    it('2. Dashboard Title Setting', () => {
      cy.get(dashboard.locators.dashboard_stylemenu).click();
      cy.get(dashboard.locators.dash_title).click();
      cy.get(dashboard.locators.dash_title_font)
        .next()
        .click({ force: true })
        .find(dashboard.locators.arial_black)
        .click({ force: true });
      cy.get(dashboard.locators.dash_title_font_size)
        .click({ force: true })
        .find('input')
        .type('25');
      cy.get(dashboard.locators.dash_title_font_weight)
        .click({ force: true })
        .find(dashboard.locators.dash_title_bold)
        .click({ force: true });
      cy.get(dashboard.locators.dash_title_font_class)
        .should('be.visible')
        .and('have.attr', 'style', 'text-align: left; font-size: 25px;');
      cy.get(dashboard.locators.dashboard_title).then((titleEl) => {
        const fontWeight = titleEl.css('font-weight');
        // 700 instead of bold on chrome; font-weight in numbers is better then then default bold
        if (['bold', '700'].indexOf(fontWeight) < 0) {
          assert.fail(`Unexpected font-weight:${fontWeight}`);
        }
      });

      for (let i = 0; i < 2; i++) {
        cy.get(dashboard.locators.dash_title_labels + "[title= '" + dashtitle[i] + "']")
          .next()
          .click();
        cy.get(dashboard.locators.color_textbox)
          .click()
          .clear()
          .type(dashtitlecolors[i]);
        cy.get(dashboard.locators.color_ok_btn).click();
      }
      cy.get(dashboard.locators.dashboard_container).should(
        'have.css',
        'background-color',
        'rgb(182, 215, 168)'
      );
      cy.get(dashboard.locators.dashboard_title).should('have.css', 'color', 'rgb(255, 0, 0)');

      cy.get(dashboard.locators.dash_title_center_align).check({ force: true });
      cy.get(dashboard.locators.dash_title_font_class).should(
        'have.attr',
        'style',
        'text-align: center; font-size: 25px;'
      );
      cy.get(dashboard.locators.dash_title_hidetitle_checkbox).check({ force: true });
      dashboard.saveDashboard();
      dashboard.viewDashboard();
      cy.get(CSSselector.apps.dashboardTitle).should('not.be.visible');
    });

    it('3. Dashboard SubTitle Setting', () => {
      dashboard.editDashboard();
      cy.get(dashboard.locators.dashboard_stylemenu).click();
      cy.get(dashboard.locators.dash_subtitle).click();
      cy.get(dashboard.locators.dash_subtitle_font)
        .next()
        .click({ force: true })
        .find(dashboard.locators.arial_black)
        .click({ force: true });
      cy.get(dashboard.locators.dash_subtitle_fontsize)
        .click({ force: true })
        .find('input')
        .type('20{enter}');
      cy.get(dashboard.locators.dash_subtitle_font_size_class)
        .should('be.visible')
        .and('have.attr', 'style', 'text-align: left; font-size: 20px;');

      for (let i = 0; i < 2; i++) {
        cy.get(dashboard.locators.dash_subtitle_labels + "[title= '" + dashtitle[i] + "']")
          .next()
          .click();
        cy.get(dashboard.locators.color_textbox)
          .click()
          .clear()
          .type(dashtitlecolors[i]);
        cy.get(dashboard.locators.color_ok_btn).click();
      }

      cy.get(dashboard.locators.dashboard_subtitle).should('have.css', 'color', 'rgb(255, 0, 0)');
      cy.get(dashboard.locators.dash_subtitle_container).should(
        'have.css',
        'background-color',
        'rgb(182, 215, 168)'
      );
      cy.get(dashboard.locators.dash_subtitle_center_align).check({ force: true });
      cy.get(dashboard.locators.dash_subtitle_font_size_class).should(
        'have.attr',
        'style',
        'text-align: center; font-size: 20px;'
      );
      dashboard.saveDashboard();
    });
  });

  context('Visual Settings', () => {
    it('1. Visual Basics Setting', () => {
      // focus the visual:
      cy.get(CSSselector.visual_container).click();

      dashboard.chooseVisualTab('style', false);

      cy.get(dashboard.locators.viz_basics).click();
      cy.get(dashboard.locators.viz_font)
        .next()
        .find('button')
        .click({ force: true });
      cy.get(dashboard.locators.viz_font)
        .next()
        .find(dashboard.locators.viz_arial)
        .click({ force: true });
      cy.get(dashboard.locators.viz_basics_fontsize)
        .click({ force: true })
        .find('input')
        .type('15{enter}');
      cy.get(dashboard.locators.viz_basics_bgcolor)
        .next()
        .find(dashboard.locators.color_dropdown)
        .click({ force: true });
      cy.get(dashboard.locators.color_textbox)
        .click()
        .clear()
        .type('#f4cccc');
      cy.get(dashboard.locators.color_ok_btn).click();
      cy.get(dashboard.locators.bar_visual).should('have.attr', 'data-fontclass', 'arc-font-arial');
      cy.get(
        'div[class="gridster ready"]>ul li:nth-child(1) svg > svg > g.x.arc-viz-axis > g.tick text'
      )
        .first()
        .should('have.attr', 'style', 'text-anchor: end; font-size: 15px;');
      cy.get(dashboard.locators.bar_visual).should(
        'have.css',
        'background-color',
        'rgb(244, 204, 204)'
      );
      dashboard.saveDashboard();
    });

    it('2. Visual Borders setting', () => {
      cy.reload();
      waitForVisualsLoaded();
      cy.get(dashboard.locators.dashboard_stylemenu).click();

      cy.get(dashboard.locators.viz_border).click();
      cy.get(dashboard.locators.viz_border_chk).check({ force: true });
      cy.get(dashboard.locators.viz_border_chk).should('be.checked');
      cy.get(dashboard.locators.viz_border_labels)
        .next()
        .find(dashboard.locators.color_dropdown)
        .click({ force: true });
      cy.get(dashboard.locators.color_textbox)
        .click()
        .clear()
        .type('#000000');
      cy.get(dashboard.locators.color_ok_btn).click();
      cy.get(dashboard.locators.viz_border_radius)
        .next()
        .find('input')
        .type('20{enter}');
      cy.get(dashboard.locators.bar_visual)
        .should('have.css', 'border-bottom-color', 'rgb(0, 0, 0)')
        .and('have.css', 'border-top-left-radius', '20px');
      dashboard.saveDashboard();
    });

    it('3. Visual Title Setting', () => {
      cy.reload();
      waitForVisualsLoaded();

      dashboard.editVisual();
      dashboard.chooseVisualTab('style', false);

      cy.get(dashboard.locators.viz_title).click();

      cy.get(dashboard.locators.viz_title_font_label)
        .next()
        .click({ force: true })
        .find(dashboard.locators.arial_black)
        .click({ force: true });
      cy.get(dashboard.locators.viz_title_font)
        .click({ force: true })
        .find('input')
        .type('25');
      cy.get(dashboard.locators.viz_title_fontweight)
        .click({ force: true })
        .find(dashboard.locators.dash_title_bold)
        .click({ force: true });
      cy.get(dashboard.locators.viz_title_class_span)
        .first()
        .should('have.attr', 'data-fontclass', 'arc-font-arial-black')
        .and('have.attr', 'style', 'font-size: 25px;');

      for (let i = 0; i < 2; i++) {
        cy.get(dashboard.locators.viz_title_label + "[title= '" + dashtitle[i] + "']")
          .next()
          .click();
        cy.get(dashboard.locators.color_textbox)
          .click()
          .clear()
          .type(dashtitlecolors[i]);
        cy.get(dashboard.locators.color_ok_btn).click();
      }
      cy.get(dashboard.locators.viz_title_center_align).click({ force: true });
      cy.get(dashboard.locators.viz_title_class_span)
        .should('have.css', 'font-size', '25px')
        .and('have.css', 'color', 'rgb(255, 0, 0)');
      cy.get(dashboard.locators.viz_title_class)
        .should('have.css', 'background-color', 'rgb(182, 215, 168)')
        .and('have.css', 'text-align', 'center');
      cy.get(dashboard.locators.viz_title_class).then((titleEl) => {
        const fontWeight = titleEl.css('font-weight');
        // 700 instead of bold on chrome; font-weight in numbers is better then then default bold
        if (['bold', '700'].indexOf(fontWeight) < 0) {
          assert.fail(`Unexpected font-weight:${fontWeight}`);
        }
      });
      dashboard.saveDashboard();
    });

    it('4. Visual SubTitle', () => {
      dashboard.editVisual();
      dashboard.chooseVisualTab('style', false);

      cy.get(dashboard.locators.visual_subtitle_style_setting).click();
      cy.get(dashboard.locators.viz_subtitle_fontlabel)
        .next()
        .click({ force: true })
        .find(dashboard.locators.arial_black)
        .click({ force: true });
      cy.get(dashboard.locators.viz_subtitle_fontsize)
        .click({ force: true })
        .find('input')
        .type('20{enter}');
      cy.get(dashboard.locators.viz_subtitle_class_span)
        .should('have.attr', 'style', 'font-size: 20px;')
        .and('have.attr', 'data-fontclass', 'arc-font-arial-black');
      cy.get(dashboard.locators.viz_subtitle_txtclr)
        .next()
        .find(dashboard.locators.color_dropdown)
        .click({ force: true });
      cy.get(dashboard.locators.color_textbox)
        .click()
        .clear()
        .type('#ff0000');
      cy.get(dashboard.locators.color_ok_btn).click();
      cy.get(dashboard.locators.viz_subtitle_bgclr)
        .next()
        .find(dashboard.locators.color_dropdown)
        .click({ force: true });
      cy.get(dashboard.locators.color_textbox)
        .click()
        .clear()
        .type('#7dcad2');
      cy.get(dashboard.locators.color_ok_btn).click();
      cy.get(dashboard.locators.viz_subtitle_container).should(
        'have.css',
        'background-color',
        'rgb(125, 202, 210)'
      );
      cy.get(dashboard.locators.viz_subtitle_class_span).should(
        'have.css',
        'color',
        'rgb(255, 0, 0)'
      );
      cy.get(dashboard.locators.viz_subtitle_centeralign).check({ force: true });
      cy.get(dashboard.locators.viz_subtitle_container).should('have.css', 'text-align', 'center');
      dashboard.saveDashboard();
    });

    it('5. Visual Axes and Labels Settings', () => {
      dashboard.editVisual();
      dashboard.chooseVisualTab('style', false);

      cy.get(dashboard.locators.viz_axes_label).click();
      for (let i = 0; i < 6; i++) {
        const labelLocator = dashboard.locators.viz_axes_labels(axeslabel[i]);
        const valueToType = axeslabelvl[i];
        if (valueToType.startsWith('#')) {
          cy.get(labelLocator)
            .next()
            .find(dashboard.locators.color_dropdown)
            .click({ force: true });
          cy.get(dashboard.locators.color_textbox)
            .click()
            .clear()
            .type(valueToType);
          cy.get(dashboard.locators.color_ok_btn).click();
        } else {
          cy.get(labelLocator)
            .next()
            .find('input')
            .type(valueToType + '{enter}');
        }
      }
      cy.get(
        'div[class="gridster ready"]>ul li:nth-child(1) svg > svg > g.x.arc-viz-axis > g.tick text'
      )
        .first()
        .should('have.attr', 'style', 'text-anchor: end; font-size: 20px;');
      cy.get(
        'div[class="gridster ready"]>ul li:nth-child(1) svg > g > g.y.arc-viz-axis >g.tick text'
      )
        .first()
        .should('have.attr', 'style', 'text-anchor: end; font-size: 20px;');
      cy.get('div[class="gridster ready"]>ul li:nth-child(1) svg  > g.viz-bar-g >  text.y')
        .first()
        .should('have.attr', 'style', 'font-size: 20px;');
      cy.get('div[class="gridster ready"]>ul li:nth-child(1) svg  > g.viz-bar-g >  text.x')
        .first()
        .should('have.attr', 'style', 'font-size: 20px;');
      cy.get(
        'div.viz-permanent table.viz-trellis-table tbody tr.viz-tr-header  td:nth-child(1)'
      ).should('have.attr', 'style', 'font-size: 20px;');
      cy.get(
        'div.viz-permanent div.arc-legend-container > svg.arc-viz-legend > g.small-legend > text.arc-legend-label'
      ).each(($el, i, $list) => {
        expect($el.text()).to.eq(legendList[i]);
      });
      dashboard.saveDashboard();
    });

    it('6. Tooltips Settings', () => {
      dashboard.editVisual();
      dashboard.chooseVisualTab('style', false);

      cy.get(dashboard.locators.tooltips).click();
      cy.get(dashboard.locators.tooltips_font)
        .next()
        .click({ force: true })
        .find(dashboard.locators.arial_black)
        .click({ force: true });
      cy.get(dashboard.locators.tooltips_font_size)
        .click({ force: true })
        .find('input')
        .type('20{enter}');
    });
  });

  context('Filters Settings', () => {
    it('1. Filter Settings', () => {
      // open dashboard style menu:
      cy.get(dashboard.locators.dashboard_stylemenu).click();

      cy.get(dashboard.locators.filters).click();
      for (let i = 0; i < 4; i++) {
        cy.get(dashboard.locators.filter_labels + "[title= '" + filtertitle[i] + "']")
          .next()
          .click();
        cy.get(dashboard.locators.color_textbox)
          .click()
          .clear()
          .type(filterclr[i]);
        cy.get(dashboard.locators.color_ok_btn).click();
      }
      cy.get(dashboard.locators.filter_checkbox).uncheck({ force: true });
      cy.get(dashboard.locators.filter_borders).should('not.be.checked');
      functions.addFilter('cereal_name', 'samples', 'Cereals');
      cy.get('div.control-container').should('have.css', 'background-color', 'rgb(237, 129, 17)');
      if (false) {
        // TODO VIZ-645: filter widget background color not set
        cy.get('div.control-container > ul > li').should(
          'have.css',
          'background-color',
          'rgb(0, 255, 255)'
        );
      }
      cy.get('div.control-container > ul > li div.selected-control-title').should(
        'have.css',
        'color',
        'rgb(255, 0, 0)'
      );
      cy.get('div.control-container > ul > li a.selected-picklist-link').should(
        'have.css',
        'color',
        'rgb(255, 0, 255)'
      );
      dashboard.saveDashboard();

      // Creating Custom style

      functions.goToSiteAdminSettingsMenu('manage_custom_styles');
      cy.get(dashboard.locators.new_custom_stylebtn).click();
      cy.get(dashboard.locators.custom_style_name)
        .clear()
        .type(stylename);
      cy.get(dashboard.locators.custom_style_desc)
        .clear()
        .type('Testing description of custom style');
      cy.get(
        dashboard.locators.custom_style_area
      ).type('th{{}background-color: #336699;color: #ffffff;}', { force: true });
      cy.intercept('POST', '/arc/reports/customcss').as('customcss');

      cy.get(dashboard.locators.save_btn).click();
      cy.wait('@customcss');
      functions.goToSiteAdminSettingsMenu('manage_custom_styles');

      cy.get(dashboard.locators.style_tbl).should('be.visible');
      cy.get(dashboard.locators.stlyle_tbl_name).should('contain', stylename);

      // Creating segment on dataset Cereals below

      functions.navigate_to_data();
      dsPermissions.selectDataset('samples', 'Cereals');
      // cy.get('span[class="dataset-title"]')
      //   .first()
      //   .click({ force: true });
      cy.get(dashboard.locators.segments).click();
      cy.get(dashboard.locators.new_segment).click();
      cy.get(dashboard.locators.segment_popup)
        .should('be.visible')
        .find(dashboard.locators.segment_name)
        .clear()
        .type(segment1);
      cy.get(dashboard.locators.segment_filter)
        .check({ force: true })
        .should('be.checked');
      cy.get(dashboard.locators.filter_expr).click();
      cy.wait(3000); // ace editor components need to be downloaded
      cy.get(dashboard.locators.segment_expr_popup).should('be.visible');
      cy.get(dashboard.locators.segment_filter_expr).type('[calories] = 70', { force: true });
      cy.get(dashboard.locators.validate_btn).click({ force: true });
      cy.get(dashboard.locators.apply_btn).click();
      cy.get(dashboard.locators.save_segment_button).click();
      cy.get(dashboard.locators.segment_row)
        .should('contain', segment1)
        .then(() => {
          segmentCreated = true;
        });

      dashboard.openDashboardforEdit(dashboardName);
      cy.get(dashboard.locators.visuals_side_tab).click();
      functions.createNewVisual('samples', 'Cereals');

      // Added wait till the time visual gets closed and loads on dashboard

      cy.wait(3000);
      visuals.editSpecificVisual(2);

      // Adding the created style in table visual and using segment as a filter in visual with explicit dataset scope

      cy.get(dashboard.locators.settings_tab).click();
      cy.get(dashboard.locators.parameters_tab).click({ force: true });
      cy.get(dashboard.locators.explicit_data_checkbox).check({ force: true });

      dashboard.chooseVisualTab('style', false, 2);

      cy.get(dashboard.locators.custom_style_tab).click();
      cy.get(dashboard.locators.add_custom_style).click();
      cy.get(dashboard.locators.pick_css_modal).should('be.visible');
      cy.get(dashboard.locators.pick_css_heading).should('have.text', 'Select Custom Styles');
      cy.get(dashboard.locators.custom_style_select)
        .parent()
        .find('.style-name-select')
        .contains(stylename)
        .click();
      cy.get(dashboard.locators.css_add_button).click();
      cy.get(dashboard.locators.style_table).should('be.visible');
      cy.get(dashboard.locators.style_table_name).should('have.text', stylename);

      // style tooltips for the second visual
      cy.get(dashboard.locators.tooltips).click();
      cy.get(dashboard.locators.tooltips_font)
        .next()
        .click({ force: true })
        .find(dashboard.locators.arial_black)
        .click({ force: true });
      cy.get(dashboard.locators.tooltips_font_size)
        .click({ force: true })
        .find('input')
        .type('20{enter}');

      dashboard.closeVisual(false);

      // Added wait till the time visual gets closed and loads on dashboard

      cy.wait(2000);

      cy.get(dashboard.locators.visuals_side_tab).click();
      cy.get('div.dataTables_scroll div.dataTables_scrollBody tbody tr:nth-child(1) > td')
        .first()
        .click('center', { force: true });
      cy.get(dashboard.locators.tooltip_popup)
        .should('be.visible')
        .and('have.attr', 'style', 'font-size:20px')
        .and('have.attr', 'data-fontclass', 'arc-font-arial-black');

      applyFilterToVisual(['100%_Bran']);

      const selector1 = '.main-app .gridster > ul li[data-arc-reportid]:nth-child(2)';
      arctest.screen.builder.verifyTableContents(selector1, [
        [
          '100%_Bran',
          'N',
          'C',
          '70',
          '4',
          '1',
          '130',
          '10',
          '5',
          '6',
          '3',
          '280',
          '25',
          '1',
          '0.33',
          'N',
          'Nabisco',
        ],
        [
          'All-Bran',
          'K',
          'C',
          '70',
          '4',
          '1',
          '260',
          '9',
          '7',
          '5',
          '3',
          '320',
          '25',
          '1',
          '0.33',
          'K',
          'Kelloggs',
        ],
      ]);
      dashboard.saveDashboard();
    });
  });
});
