import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import * as functions from 'lib/functions';
import { checkArcenginePresence } from 'lib/functions';
import * as visuals from 'lib/visual';
import { CSSselector } from 'locators/common/CSSselector';

describe('Create dashboard #1.', () => {
  before(function() {
    checkArcenginePresence(this);
  });

  // local method just for the test
  function get_time() {
    const a = new Date();
    const b = a.getTime();
    return b;
  }

  describe('ARC - 7383', () => {
    before(() => {
      cy.fixture('users.json').then((data) => {
        logins.silent(data.admin.username, data.admin.password);
      });
    });
    after(() => {
      dashboards.saveDashboard();
    });

    // It keep tracks of total created visual within dashboard
    let totalVisualCount: number;

    // It is used to initialize visual name that needs to be entered
    let visualName: string;

    it('dashboard: 1: create a new dashboard', () => {
      functions.addNewDB(`dashboard_${get_time()}`);
    });

    it('dashboard: 2: create a new visual and let the user edit visual to it', () => {
      cy.fixture('visuals.json').then((dash) => {
        functions.createNewVisual(dash.data[3].dataconnection, dash.data[3].dataset);
        dashboards.saveDashboard();
      });
    });

    it('dashboard: 3: convert table visual to Line visual', () => {
      visualName = 'Airplane Crash Incidents Per Year vs Casualties';
      cy.get(CSSselector.visual_container).then(($visualcontainer) => {
        totalVisualCount = $visualcontainer.children().length;
        visuals.editTableVSToLineVSCustom(totalVisualCount, visualName);
        dashboards.saveDashboard();
        // Verify Visual by its name
        cy.get(CSSselector.visual_title.replace('index', String(totalVisualCount))).should(
          'have.text',
          visualName
        );
        // Verify Expected Enter fields are displayed in visuals OR not
        cy.get(CSSselector.xAxis_field_name).should('have.text', 'Year');
        cy.get(CSSselector.yAxis_field_name).should('have.text', 'Incidents');
        cy.get(CSSselector.secondary_field).contains('Fatalities');
        cy.get(CSSselector.secondary_field).contains('Survivers');
      });
    });

    it('dashboard: 4: convert table visual to KPI visual', () => {
      visualName = 'Total Airplane Crash Incidents';
      cy.get(CSSselector.visual_container).then(($visualcontainer) => {
        totalVisualCount = $visualcontainer.children().length;
        visuals.editTableVSToKPIVSCustom(totalVisualCount + 1, visualName);
        dashboards.saveDashboard();
        // Verify Visual by its name
        cy.get(CSSselector.visual_title.replace('index', String(totalVisualCount + 1))).should(
          'have.text',
          visualName
        );
      });
      cy.get(CSSselector.visual_container).then(($visualcontainer) => {
        totalVisualCount = $visualcontainer.children().length;
        cy.get(CSSselector.kpi_value.replace('index', String(totalVisualCount + 1)))
          .should('contain', ',', 'KPI doesnt have commas')
          .should('be.gt', '0', 'KPI value is not greater than 0');
      });
    });

    it('dashboard: 5: convert table visual to Scatter visual', () => {
      visualName = 'Crashes By Operator';
      cy.get(CSSselector.visual_container).then(($visualcontainer) => {
        totalVisualCount = $visualcontainer.children().length;
        visuals.editTableVSToScatterVSCustom(totalVisualCount + 1, visualName);
        // Verify Visual by its name
        cy.get(CSSselector.visual_title.replace('index', String(totalVisualCount + 1))).should(
          'have.text',
          visualName
        );
        // Verify Expected Enter fields are displayed in visuals OR not
        // todo at dashlvel here
      });
    });

    it('dashboard: 6: convert table visual to second KPI visual', () => {
      visualName = 'Total People Dead in Airplane Crash Incidents';
      cy.get(CSSselector.visual_container).then(($visualcontainer) => {
        totalVisualCount = $visualcontainer.children().length;
        visuals.editTableVSToKPIVS2Custom(totalVisualCount + 1, visualName);
        // Verify Visual by its name
        cy.get(CSSselector.visual_title.replace('index', String(totalVisualCount + 1))).should(
          'have.text',
          visualName
        );
      });

      cy.get(CSSselector.visual_container).then(($visualcontainer) => {
        totalVisualCount = $visualcontainer.children().length;
        cy.get(CSSselector.kpi_value.replace('index', String(totalVisualCount + 1)))
          .should('contain', ',', 'KPI doesnt have commas')
          .should('be.gt', '0', 'KPI value is not greater than 0');
      });
    });

    it('dashboard: 7: convert table visual to third KPI visual', () => {
      visualName = 'Total People Survived in Airplane Crash Incidents';
      cy.get(CSSselector.visual_container).then(($visualcontainer) => {
        totalVisualCount = $visualcontainer.children().length;
        visuals.editTableVSToKPIVS3Custom(totalVisualCount + 1, visualName);
        // Verify Visual by its name
        cy.get(CSSselector.visual_title.replace('index', String(totalVisualCount + 1))).should(
          'have.text',
          visualName
        );

        // Verify expected data contains having number with comma
        // cy.get(CSSselector.kpi_value).then(($p) => {
        //     expect($p.text().indexOf(",") != -1)
        //     expect(Number.isInteger(Number($p.text().replace(",", "")))).to.be.true
        // })
      });

      cy.get(CSSselector.visual_container).then(($visualcontainer) => {
        totalVisualCount = $visualcontainer.children().length;
        cy.get(CSSselector.kpi_value.replace('index', String(totalVisualCount + 1)))
          .should('contain', ',', 'KPI doesnt have commas')
          .should('be.gt', '0', 'KPI value is not greater than 0');
      });
    });

    it('dashboard: 8: convert table visual to Radial visual', () => {
      visualName = '20 Airplane Operators With Most Crashes';
      cy.get(CSSselector.visual_container).then(($visualcontainer) => {
        totalVisualCount = $visualcontainer.children().length;
        visuals.editRadialVisual(totalVisualCount + 1, visualName);
        // Verify Visual by its name
        cy.get(CSSselector.visual_title.replace('index', String(totalVisualCount + 1))).should(
          'have.text',
          visualName
        );
        // Verify Expected Enter fields are displayed in visuals OR not
        cy.get(CSSselector.dimension_field_element).contains('operator');
        cy.get(CSSselector.measure_field_element).contains('Fatalities');
        cy.get(CSSselector.measure_field_element).contains('Incidents');
        cy.get(CSSselector.measure_field_element).contains('Survivers');
      });
    });
  });
});
