import { logins } from 'lib/arctest';
import * as functions from 'lib/functions';
import { checkArcenginePresence } from 'lib/functions';
import * as visuals from 'lib/visual';
import { CSSselector } from 'locators/common/CSSselector';

describe('Create dashboard #2.', () => {
  before(function() {
    checkArcenginePresence(this);
    cy.fixture('users.json').then((data) => {
      logins.silent(data.admin.username, data.admin.password);
    });
  });

  function get_time() {
    const a = new Date();
    return a.getTime();
  }

  describe('ARC ####', () => {
    let totalVisualCount: number;
    let visualName: string;
    const unixTimeAsInt = 'from_unixtime(unix_timestamp([date], "yyyy-MM-dd"), "yyyy") as int';

    it('1 : create a new dashboard', () => {
      functions.addNewDB(`dashboard_${get_time()}`);
    });

    it('2 : create a new visual and let the user edit visual to it', () => {
      cy.fixture('visuals.json').then((dash) => {
        functions.createNewVisual(dash.data[3].dataconnection, dash.data[3].dataset);
        cy.get(CSSselector.Save).click({ force: true });
      });
    });

    // Visual # 1: Map Visual
    it('3 : convert table visual to Map visual', () => {
      visualName = 'Aircraft Crash Map';
      const sqlExpr =
        'CASE WHEN instr([location],"Afghanstan")>0 THEN "Afghanistan" WHEN instr([location], "Brazil")>0 THEN "Brazil" WHEN instr([location], "California")>0 THEN "USA" WHEN instr([location], "Calilfornia")>0 THEN "USA" WHEN instr([location], "Arizona")>0 THEN "USA" WHEN instr([location], "Arkansas")>0 THEN "USA" WHEN instr([location], "Colorado")>0 THEN "USA" WHEN instr([location], "Connecticut")>0 THEN "USA" WHEN instr([location], "Idaho")>0 THEN "USA" WHEN instr([location], "Illinois")>0 THEN "USA" WHEN instr([location], "Indiana")>0 THEN "USA" WHEN instr([location], "Louisiana")>0 THEN "USA" WHEN instr([location], "New Mexico")>0 THEN "USA" WHEN instr([location], "New Hampshire")>0 THEN "USA" WHEN instr([location], "New Jersey")>0 THEN "USA" WHEN instr([location], "Nevada")>0 THEN "USA" WHEN instr([location], "Alabama")>0 THEN "USA" WHEN instr([location], "Kansas")>0 THEN "USA" WHEN instr([location], "Florida")>0 THEN "USA" WHEN instr([location], "Georgia")>0 THEN "USA" WHEN instr([location], "Alaska")>0 THEN "USA" WHEN instr([location], "Kentucky")>0 THEN "USA" WHEN instr([location], "Iowa")>0 THEN "USA" WHEN instr([location], "Maryland")>0 THEN "USA" WHEN instr([location], "Massachusetts")>0 THEN "USA" WHEN instr([location], "Minnesota")>0 THEN "USA" WHEN instr([location], "Missouri")>0 THEN "USA" WHEN instr([location], "Montana")>0 THEN "USA" WHEN instr([location], "North Carolina")>0 THEN "USA" WHEN instr([location], "Ohio")>0 THEN "USA" WHEN instr([location], "Pennsylvania")>0 THEN "USA" WHEN instr([location], "New York")>0 THEN "USA" WHEN instr([location], "Texas")>0 THEN "USA" WHEN instr([location], "Tennessee")>0 THEN "USA" WHEN instr([location], "South Dakota")>0 THEN "USA" WHEN instr([location], "South Carolina")>0 THEN "USA" WHEN instr([location], "Uta")>0 THEN "USA" WHEN instr([location], "Washington")>0 THEN "USA" WHEN instr([location], "Wisconsin")>0 THEN "USA" WHEN instr([location], "Wisconson")>0 THEN "USA" WHEN instr([location], "Wyoming")>0 THEN "USA" WHEN instr([location], "Virginia")>0 THEN "USA" WHEN instr([location], "Vermont")>0 THEN "USA" WHEN instr([location], "Hawaii")>0 THEN "USA" WHEN instr([location], "Congo")>0 THEN "Congo" WHEN instr([location], "Newfoundland")>0 THEN "Canada" WHEN instr([location], "Canada")>0 THEN "Canada" WHEN instr([location], "Chechnya")>0 THEN "Russia" ELSE ltrim(substr([location], CASE WHEN instr([location], ",")>0 AND locate(",", [location], instr([location], ",")+1)>0 THEN locate(",", [location], instr([location], ",")+1)+1 WHEN instr([location], ",")>0 THEN instr([location], ",")+1 ELSE 1 END)) END as "country"';
      cy.get(CSSselector.visual_container).then(($visualcontainer) => {
        totalVisualCount = $visualcontainer.children().length;

        cy.intercept('POST', '/arc/reports/report/*').as('visualSubmitted');
        cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('visualUpdated');
        cy.intercept('POST', '/arc/reports/savedashboard').as('dashboardSaved');
        cy.intercept('POST', '/arc/reports/create/*').as('visualCreated');
        cy.intercept('GET', '/arc/static/js/ext/maps/World/world-50m.json').as('worldMap');
        cy.intercept('GET', '/arc/static/js/ext/maps/World/country.codes.txt').as('countryCodes');

        cy.get(CSSselector.visual_edit_specific(totalVisualCount)).click({
          force: true,
        });
        cy.get(CSSselector.map_visual_detail.map_visual).click();
        cy.get(CSSselector.vizual_fields_tab.replace('column-name', 'location')).click();
        cy.get("li[class='dropdown app-shelf-item'] span[title='location']").click();
        visuals.verifySQLExpression(sqlExpr);
        cy.get(CSSselector.map_visual_detail.change_type_country).click({ force: true });
        cy.get(CSSselector.order_topK_list.order_topK).click();
        cy.get(CSSselector.order_topK_list.order_ascending).click();
        cy.get(CSSselector.measure_field_element).click();
        cy.get(CSSselector.vizual_fields_tab.replace('column-name', 'date')).click();
        cy.get(
          "#app-shelves #aggregates_shelf li[class='dropdown app-shelf-item']:nth-child(1)"
        ).click();
        visuals.verifySQLExpression('count([date]) as "Incidents"');
        cy.get(CSSselector.aggregates_list.aggregates).click();
        cy.get(CSSselector.aggregates_list.maximum).click();
        cy.get(CSSselector.visual_settings_tab).click();
        cy.get(CSSselector.visual_settings_parameter).click();
        // no longer a checkbox its a button dropdown instead
        cy.get(CSSselector.visual_settings_parameter_data_scope).should('be.visible');
        cy.get(CSSselector.visual_settings_parameter_data_scope).click();
        cy.get(CSSselector.visual_settings_parameter_data_scope)
          .next()
          .contains('Current Dataset')
          .click();
        cy.get(CSSselector.visual_settings_data).click();
        cy.get(CSSselector.visual_settings_downloading).type('-1');
        cy.get(CSSselector.visuals_title)
          .click()
          .type(visualName);
        cy.get(CSSselector.refresh_visual).click();
        cy.wait('@visualUpdated')
          .its('response.statusCode')
          .should('eq', 200);
        cy.wait('@worldMap');
        cy.wait('@countryCodes');
        cy.get(CSSselector.save_visual).click();

        cy.wait('@visualSubmitted')
          .its('response.statusCode')
          .should('eq', 200);

        cy.get(CSSselector.Save).click({ force: true });
        cy.wait('@dashboardSaved');
        cy.get(CSSselector.visual_title.replace('index', String(totalVisualCount))).should(
          ($el) => {
            assert.equal($el.text(), 'Aircraft Crash Map', 'visual name is expected.');
          }
        );
        cy.get(CSSselector.map_visual_detail.map_verify).should(($el) => {
          return expect($el).to.exist;
        });
      });
    });

    // Visual # 2 : Bar Visual
    it('4 : convert table visual to Bar visual', () => {
      visualName = '20 Airplanes With Most Crashes In XXI Century';

      const dimX = 'type';
      const dimY = 'fatalities';
      const yExpr = 'count(coalesce([fatalities],0)) as "Incidents"';
      const topOkValue = '20';
      const filterField = 'date';
      const filterExpr = 'cast(' + unixTimeAsInt + ') >= 2000';

      cy.get(CSSselector.visual_container).then(($visualcontainer) => {
        totalVisualCount = $visualcontainer.children().length;
        totalVisualCount = totalVisualCount + 1;
        cy.intercept('POST', '/arc/reports/report/*').as('visualSubmitted');
        cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('visualUpdated');
        cy.intercept('POST', '/arc/reports/savedashboard').as('dashboardSaved');
        cy.intercept('POST', '/arc/reports/create/*').as('visualCreated');
        cy.intercept('GET', '/arc/reports/report/*').as('visualGet');

        cy.get(CSSselector.selecting_visuals).click({ force: true });
        cy.wait('@visualCreated');
        cy.wait('@visualSubmitted')
          .its('response.statusCode')
          .should('eq', 200);
        cy.wait('@visualGet');

        cy.wait('@visualUpdated');
        cy.get("div[class='gridster ready']>ul>li").then(($p2) => {
          expect($p2.children().length).to.equal(totalVisualCount);
        });
        cy.get("div[class='gridster ready']>ul>li:last").scrollIntoView();
        cy.get(CSSselector.visual_rows.replace('index', String(totalVisualCount))).should(
          'be.visible'
        );
        cy.get(CSSselector.visual_edit_specific(totalVisualCount)).click({
          force: true,
        });
        cy.get(CSSselector.bar_visual).click();
        // cy.get(CSSselector.data_tab).click();
        cy.get(CSSselector.barVisual.x_Axis).click();
        cy.get(CSSselector.vizual_fields_tab.replace('column-name', dimX)).click();
        cy.get(CSSselector.barVisual.x_Axis).click();
        cy.get(CSSselector.measure_field).click();
        cy.get(CSSselector.vizual_fields_tab.replace('column-name', dimY)).click();
        cy.get("#app-shelves #y_shelf li[class='dropdown app-shelf-item']:nth-child(1)").click();
        visuals.verifySQLExpression(yExpr);
        cy.get("#properties-panel li[class*='order-functions']").click();
        cy.get("#properties-panel a[data-action='shelf-order-top'] input").type(topOkValue);
        cy.get("#properties-panel a[data-action='shelf-order-top']").click('left');
        cy.get(CSSselector.close_property_panel).click();
        cy.get(CSSselector.filter_field).click();
        cy.get(CSSselector.vizual_fields_tab.replace('column-name', filterField)).click();
        cy.get(
          "#app-shelves  #filters_shelf li[class='dropdown app-shelf-item']:nth-child(1)"
        ).click();
        visuals.verifySQLExpression(filterExpr);
        cy.get(CSSselector.visuals_title)
          .click()
          .type(visualName);
        cy.get(CSSselector.refresh_visual).click();
        cy.wait('@visualUpdated')
          .its('response.statusCode')
          .should('eq', 200);
        cy.get(CSSselector.save_visual).click();
        cy.wait('@visualSubmitted')
          .its('response.statusCode')
          .should('eq', 200);
        cy.get(CSSselector.Save).click({ force: true });
        cy.wait('@dashboardSaved');
        cy.get(CSSselector.map_visual_detail.map_verify).should(($el) => {
          return expect($el).to.exist;
        });
        cy.get(CSSselector.visual_title.replace('index', String(totalVisualCount))).should(
          ($el) => {
            assert.equal(
              $el.text(),
              '20 Airplanes With Most Crashes In XXI Century',
              'Visual name should be 20 Airplanes With Most Crashes In XXI Century'
            );
          }
        );
        cy.get(`#app-selected-list > ul > li:nth-child(2) [text-anchor='start']`).should(($el) => {
          assert.equal($el.text(), 'Incidents', 'Should be Incidents');
        });
        cy.get(`#app-selected-list > ul > li:nth-child(2) [text-anchor='middle']`).should(($el) => {
          assert.equal($el.text(), 'type', 'Should be type');
        });
      });
    });

    // Visual # 3 : Bar Visual
    it('5 : convert table visual to Bar visual', () => {
      visualName = '20 Airplanes With Most Casualties In XXI Century';

      const dimX = 'type';
      const dimY = 'fatalities';
      const yExpr = 'count(coalesce([fatalities],0)) as "Fatalities"';
      const topOkValue = '20';
      const filterField = 'date';
      const filterExpr = 'cast(' + unixTimeAsInt + ') >= 2000';
      cy.get(CSSselector.visual_container).then(($visualcontainer) => {
        totalVisualCount = $visualcontainer.children().length;
        totalVisualCount = totalVisualCount + 1;
        cy.intercept('POST', '/arc/reports/report/*').as('visualSubmitted');
        cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('visualUpdated');
        cy.intercept('POST', '/arc/reports/savedashboard').as('dashboardSaved');
        cy.intercept('POST', '/arc/reports/create/*').as('visualCreated');
        cy.intercept('GET', '/arc/reports/report/*').as('visualGet');

        cy.get(CSSselector.selecting_visuals).click({ force: true });
        cy.wait('@visualCreated');
        cy.wait('@visualSubmitted')
          .its('response.statusCode')
          .should('eq', 200);
        cy.wait('@visualGet');
        cy.wait('@visualUpdated');

        cy.get("div[class='gridster ready']>ul>li").then(($p2) => {
          expect($p2.children().length).to.equal(totalVisualCount);
        });
        cy.get("div[class='gridster ready']>ul>li:last").scrollIntoView();
        cy.get(CSSselector.visual_rows.replace('index', String(totalVisualCount))).should(
          'be.visible'
        );
        cy.get(CSSselector.visual_edit_specific(totalVisualCount)).click({
          force: true,
        });
        cy.get(CSSselector.bar_visual).click();
        // cy.get(CSSselector.data_tab).click();
        cy.get(CSSselector.barVisual.x_Axis).click();
        cy.get(CSSselector.vizual_fields_tab.replace('column-name', dimX)).click();
        cy.get(CSSselector.barVisual.x_Axis).click();
        cy.get(CSSselector.measure_field).click();
        cy.get(CSSselector.vizual_fields_tab.replace('column-name', dimY)).click();
        cy.get("#app-shelves #y_shelf li[class='dropdown app-shelf-item']:nth-child(1)").click();
        visuals.verifySQLExpression(yExpr);
        cy.get("#properties-panel li[class*='order-functions']").click();
        cy.get("#properties-panel a[data-action='shelf-order-top'] input").type(topOkValue);
        cy.get("#properties-panel a[data-action='shelf-order-top']").click('left');
        cy.get(CSSselector.close_property_panel).click();
        cy.get(CSSselector.filter_field).click();
        cy.get(CSSselector.vizual_fields_tab.replace('column-name', filterField)).click();
        cy.get(
          "#app-shelves  #filters_shelf li[class='dropdown app-shelf-item']:nth-child(1)"
        ).click();
        visuals.verifySQLExpression(filterExpr);
        cy.get(CSSselector.visuals_title)
          .click()
          .type(visualName);
        cy.get(CSSselector.refresh_visual).click();
        cy.wait('@visualUpdated')
          .its('response.statusCode')
          .should('eq', 200);
        cy.get(CSSselector.save_visual).click();
        cy.wait('@visualSubmitted')
          .its('response.statusCode')
          .should('eq', 200);
        cy.get(CSSselector.Save).click({ force: true });
        cy.wait('@dashboardSaved');
        cy.get(CSSselector.map_visual_detail.map_verify).should(($el) => {
          return expect($el).to.exist;
        });
        cy.get(CSSselector.visual_title.replace('index', String(totalVisualCount))).should(
          ($el) => {
            assert.equal(
              $el.text(),
              '20 Airplanes With Most Casualties In XXI Century',
              'Visual name should be 20 Airplanes With Most Casualties In XXI Century'
            );
          }
        );
        cy.get(
          `#app-selected-list > ul > li:nth-child(${totalVisualCount}) [text-anchor='start']`
        ).should(($el) => {
          assert.equal($el.text(), 'Fatalities', 'Should be Fatalities');
        });
        cy.get(
          `#app-selected-list > ul > li:nth-child(${totalVisualCount}) [text-anchor='middle']`
        ).should(($el) => {
          assert.equal($el.text(), 'type', 'Should be type');
        });
      });
    });
    // Visual # 4 : Radial Visual
    it('6 : convert table visual to Radial visual', () => {
      visualName = '20 Airplane Operators With Most Crashes';
      const Dim1 = 'type';
      const measure1 = 'fatalities';
      const measure2 = 'fatalities';
      const measure3 = 'fatalities';
      const measure1Expr = 'sum(coalesce([fatalities],0)) as "Fatalities"';
      const measure2Expr = 'count(coalesce([fatalities],0)) as "Incidents"';
      const measure3Expr =
        'sum(CASE [aboard] WHEN NULL THEN 0 ELSE [aboard]-coalesce([fatalities],0) END) as "Survivers"';
      const filterField1 = 'date';
      const filterField1Expr = 'cast(' + unixTimeAsInt + ') >= 2000';
      cy.get(CSSselector.visual_container).then(($visualcontainer) => {
        totalVisualCount = $visualcontainer.children().length;
        totalVisualCount = totalVisualCount + 1;
        cy.intercept('POST', '/arc/reports/report/*').as('visualSubmitted');
        cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('visualUpdated');
        cy.intercept('POST', '/arc/reports/savedashboard').as('dashboardSaved');
        cy.intercept('POST', '/arc/reports/create/*').as('visualCreated');
        cy.intercept('GET', '/arc/reports/report/*').as('visualGet');

        cy.get(CSSselector.selecting_visuals).click({ force: true });
        cy.wait('@visualCreated');
        cy.wait('@visualSubmitted')
          .its('response.statusCode')
          .should('eq', 200);
        cy.wait('@visualGet');
        cy.wait('@visualUpdated');

        cy.get("div[class='gridster ready']>ul>li").then(($p2) => {
          expect($p2.children().length).to.equal(totalVisualCount);
        });
        cy.get("div[class='gridster ready']>ul>li:last").scrollIntoView();
        cy.get(CSSselector.visual_rows.replace('index', String(totalVisualCount))).should(
          'be.visible'
        );
        cy.get(CSSselector.visual_edit_specific(totalVisualCount)).click({
          force: true,
        });
        cy.get(CSSselector.radial_visual).click();
        // cy.get(CSSselector.data_tab).click();
        cy.get(CSSselector.vizual_fields_tab.replace('column-name', Dim1)).click();
        cy.get("#app-shelves li[class='dropdown app-shelf-item'] span[title='type']").click();
        cy.get(CSSselector.measure_field_element).click();
        cy.get(CSSselector.vizual_fields_tab.replace('column-name', measure1)).click();
        cy.get(
          "#app-shelves #aggregates_shelf li[class='dropdown app-shelf-item']:nth-child(1)"
        ).click();
        visuals.verifySQLExpression(measure1Expr);
        cy.get(CSSselector.close_property_panel).click();
        cy.get(CSSselector.vizual_fields_tab.replace('column-name', measure2)).click();
        cy.get(
          "#app-shelves #aggregates_shelf li[class='dropdown app-shelf-item']:nth-child(2)"
        ).click();
        visuals.verifySQLExpression(measure2Expr);
        cy.get(CSSselector.close_property_panel).click();
        cy.get(CSSselector.vizual_fields_tab.replace('column-name', measure3)).click();
        cy.get(
          "#app-shelves #aggregates_shelf li[class='dropdown app-shelf-item']:nth-child(3)"
        ).click();
        visuals.verifySQLExpression(measure3Expr);
        cy.get(CSSselector.filter_field).click();
        cy.get(CSSselector.vizual_fields_tab.replace('column-name', filterField1)).click();
        cy.get(
          "#app-shelves  #filters_shelf li[class='dropdown app-shelf-item']:nth-child(1)"
        ).click();
        visuals.verifySQLExpression(filterField1Expr);
        cy.get(CSSselector.visuals_title)
          .click()
          .type(visualName);
        cy.get(CSSselector.refresh_visual).click();
        cy.wait('@visualUpdated')
          .its('response.statusCode')
          .should('eq', 200);
        cy.get(CSSselector.save_visual).click();
        cy.wait('@visualSubmitted')
          .its('response.statusCode')
          .should('eq', 200);
        cy.get(CSSselector.Save).click({ force: true });
        cy.wait('@dashboardSaved');
        cy.get(CSSselector.visual_title.replace('index', String(totalVisualCount))).should(
          ($el) => {
            assert.equal(
              $el.text(),
              '20 Airplane Operators With Most Crashes',
              'visual name is expected.'
            );
          }
        );
        cy.get(CSSselector.map_visual_detail.map_verify).should(($el) => {
          return expect($el).to.exist;
        });
        cy.get('#app-selected-list > ul > li:nth-child(4) .viz-chart .small-legend text').each(
          ($el, index, $list) => {
            if (index === 0) {
              assert.equal($el.text(), 'Fatalities', 'Should be Fatalities');
            }
            if (index === 1) {
              assert.equal($el.text(), 'Incidents', 'Should be Incidents');
            }
            if (index === 2) {
              assert.equal($el.text(), 'Survivers', 'Should be Survivers');
            }
          }
        );
      });
    });
    // Visual # 5 : Scatter Visual
    it('7 : convert table visual to Scatter visual', () => {
      visualName = 'Crashes By Operator in XXI Century';

      const xField = 'date';
      const yField = 'fatalities';
      const xExpr = 'from_unixtime(unix_timestamp([date] , "yyyy-MM-dd") , "yyyy") as "Year"';
      const yExpr = 'sum(coalesce([fatalities],0)) as "Fatalities"';
      const colorField = 'operator';
      const colorExpr = 'coalesce([operator], "N/A") as "Operator"';
      const sizeField = 'date';
      const sizeExpr = 'count([date]) as "Incidents"';
      const filterField1 = 'fatalities';
      const filterExpr1 = 'coalesce([fatalities],0)>0';
      const filterField2 = 'fatalities';
      const filterExpr2 = 'cast(' + unixTimeAsInt + ') >= 2000';

      cy.get(CSSselector.visual_container).then(($visualcontainer) => {
        totalVisualCount = $visualcontainer.children().length;
        totalVisualCount = totalVisualCount + 1;
        cy.intercept('POST', '/arc/reports/report/*').as('visualSubmitted');
        cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('visualUpdated');
        cy.intercept('POST', '/arc/reports/savedashboard').as('dashboardSaved');
        cy.intercept('POST', '/arc/reports/create/*').as('visualCreated');
        cy.intercept('GET', '/arc/reports/report/*').as('visualGet');

        cy.get(CSSselector.selecting_visuals).click({ force: true });
        cy.wait('@visualCreated');
        cy.wait('@visualSubmitted')
          .its('response.statusCode')
          .should('eq', 200);
        cy.wait('@visualGet');
        cy.wait('@visualUpdated');

        cy.get("div[class='gridster ready']>ul>li").then(($p2) => {
          expect($p2.children().length).to.equal(totalVisualCount);
        });
        cy.get("div[class='gridster ready']>ul>li:last").scrollIntoView();
        cy.get(CSSselector.visual_rows.replace('index', String(totalVisualCount))).should(
          'be.visible'
        );
        cy.get(CSSselector.visual_edit_specific(totalVisualCount)).click({
          force: true,
        });
        cy.get(CSSselector.scatter_visual).click();
        // cy.get(CSSselector.data_tab).click();
        cy.get(CSSselector.vizual_fields_tab.replace('column-name', xField)).click();
        cy.get("li[class='dropdown app-shelf-item'] span[title='date']").click();
        visuals.verifySQLExpression(xExpr);
        cy.get(CSSselector.yAxis_field).click();
        cy.get(CSSselector.vizual_fields_tab.replace('column-name', yField)).click();
        cy.get("#app-shelves #y_shelf li[class='dropdown app-shelf-item']:nth-child(1)").click();
        visuals.verifySQLExpression(yExpr);
        cy.get(CSSselector.color_field).click();
        cy.get(CSSselector.vizual_fields_tab.replace('column-name', colorField)).click();
        cy.get(CSSselector.color_field_first_element).click();
        visuals.verifySQLExpression(colorExpr);
        cy.get(CSSselector.close_property_panel).click();
        cy.get(CSSselector.size_field).click();
        cy.get(CSSselector.vizual_fields_tab.replace('column-name', sizeField)).click();
        cy.get("#app-shelves #size_shelf li[class='dropdown app-shelf-item']:nth-child(1)").click();
        visuals.verifySQLExpression(sizeExpr);
        cy.get(CSSselector.close_property_panel).click();
        cy.get(CSSselector.filter_field).click();
        cy.get(CSSselector.vizual_fields_tab.replace('column-name', filterField1)).click();
        cy.get(
          "#app-shelves #filters_shelf li[class='dropdown app-shelf-item']:nth-child(1)"
        ).click();
        visuals.verifySQLExpression(filterExpr1);
        cy.get(CSSselector.close_property_panel).click();
        cy.get(CSSselector.vizual_fields_tab.replace('column-name', filterField2)).click();
        cy.get(
          "#app-shelves #filters_shelf li[class='dropdown app-shelf-item']:nth-child(2)"
        ).click();
        visuals.verifySQLExpression(filterExpr2);
        cy.get(CSSselector.close_property_panel).click();
        cy.get(CSSselector.visual_settings_tab).click();
        cy.get(CSSselector.visual_settings_parameter).click();
        // no longer a checkbox its a button dropdown instead
        cy.get(CSSselector.visual_settings_parameter_data_scope).should('be.visible');
        cy.get(CSSselector.visual_settings_parameter_data_scope).click();
        cy.get(CSSselector.visual_settings_parameter_data_scope)
          .next()
          .contains('Current Dataset')
          .click();
        cy.get(CSSselector.visual_settings_parameter_aggregate).click();
        cy.get(CSSselector.visual_settings_data).click();
        cy.get(CSSselector.visual_settings_downloading).type('-1');
        cy.get(CSSselector.visuals_title)
          .click()
          .type(visualName);
        cy.get(CSSselector.refresh_visual).click();
        cy.wait('@visualUpdated')
          .its('response.statusCode')
          .should('eq', 200);
        cy.get(CSSselector.save_visual).click();
        cy.wait('@visualSubmitted')
          .its('response.statusCode')
          .should('eq', 200);
        cy.get(CSSselector.Save).click({ force: true });
        cy.wait('@dashboardSaved');
        // cy.get(CSSselector.visual_title.replace('index', String(totalVisualCount))).should(($el) => {
        //   assert.equal(
        //     $el.text(),
        //     'Crashes By Operator in XXI Century',
        //     'Should be Crashes By Operator in XXI Century'
        //   );
        // });
        // cy.get(CSSselector.map_visual_detail.map_verify).should(($el) => {
        //   return expect($el).to.exist;
        // });
        cy.get(CSSselector.visual_title.replace('index', String(totalVisualCount))).should(
          ($el) => {
            assert.equal(
              $el.text(),
              'Crashes By Operator in XXI Century',
              'Visual name should be Crashes By Operator in XXI Century'
            );
          }
        );
        cy.get(
          `ul li:nth-child(${totalVisualCount}) div[class='dash-viz-wrapper'] text[class='y arc-viz-axis-label']`
        ).should(($el) => {
          assert.equal($el.text(), 'Fatalities', 'Should be Fatalities');
        });
        cy.get(
          `ul li:nth-child(${totalVisualCount}) div[class='dash-viz-wrapper'] text[class='x arc-viz-axis-label']`
        ).should(($el) => {
          assert.equal($el.text(), 'Year', 'Should be Year');
        });
      });
    });
  });
});
