import { logins } from 'lib/arctest';
import * as functions from 'lib/functions';
import { checkArcenginePresence } from 'lib/functions';
import * as visuals from 'lib/visual';
import { CSSselector } from 'locators/common/CSSselector';

describe('Create dashboard #3.', () => {
  before(function() {
    checkArcenginePresence(this);
    cy.fixture('users.json').then((data) => {
      logins.silent(data.admin.username, data.admin.password);
    });
  });

  describe('ARC ##', () => {
    let totalVisualCount: number;
    let visualName: string;

    it('1 : create a new dashboard', () => {
      functions.addNewDB(`Elo Rating and Stats`);
    });

    it('2 : create a new visual and let the user edit visual to it', () => {
      cy.fixture('visuals.json').then((dash) => {
        functions.createNewVisual(dash.data[4].dataconnection, dash.data[4].dataset);
        cy.get(CSSselector.Save).click({ force: true });
      });
    });
    // Visual #1 - Extension Visual
    it('3 : convert table visual to Extension visual', () => {
      visualName = '';
      const text =
        "<h3>Another Way of Rating Team Performance: Elo</h3><div style='font-weight: 300; font-size: 14px; color: #999;'><p>Named after a Hungarian-born American physics professor Arpad Elo, is a method for calculating the relative skill levels of players in competitor-versus-competitor games. Originally invented to evaluate a chess player, today we can apply it to NBA teams also. Here are its essential features for evaluating an NBA team:</p><ul style='margin-top:10px; list-style-type: square;'><li>The ratings depend only on the final score of each game and where it was played (home-court advantage). They include both regular-season and playoff games.</li><li>Teams always earn Elo points after winning games and lose ground after losing them. They gain more points for upset wins and for winning by wider margins.</li><li>The system is zero-sum. When the Denver Nuggets gained 30 Elo points by defeating the No. 1 seed Seattle SuperSonics in the first round of the 1994 NBA playoffs, the Sonics lost 30 points.</li><li>Ratings are established on a game-by-game rather than a season-by-season basis, so changes in a team is “form” over the course of the year.</li><li>Portions of the rating for the last season is carried over to the next season. </li></ul><p style='margin-top:10px'>The long-term average Elo rating is 1500, although it can vary slightly in any particular year based on how recently the league has expanded (more about that below). More than 90 percent of team ratings are between 1300 (god awful) and 1700 (really good).</p><p style='margin-top:10px'>We take a look at the Elo rating for each team and how well it compares to teams' performance in playoff.<p></div>";

      cy.get(CSSselector.visual_container).then(($visualcontainer) => {
        totalVisualCount = $visualcontainer.children().length;
        cy.intercept('POST', '/arc/reports/report/*').as('visualSubmitted');
        cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('visualUpdated');

        cy.get(CSSselector.visual_edit_specific(1)).click({ force: true });
        cy.get(CSSselector.Extension_Visual_Details.extension_visual).click();
        cy.get(CSSselector.Extension_Visual_Details.html_textarea).type(text);
        cy.get(CSSselector.visualEdit.styleSettings).click();
        cy.get(CSSselector.visualEdit.overrideWarning);
        cy.get(CSSselector.refresh_visual).click();
        cy.get(CSSselector.save_visual).click();
        cy.wait('@visualSubmitted')
          .its('response.statusCode')
          .should('eq', 200);
        cy.get(CSSselector.Save).click({ force: true });
        cy.get(CSSselector.visual_extension_verify).should(($el) => {
          return expect($el).to.exist;
        });
        cy.get(CSSselector.visual_title.replace('index', String(totalVisualCount))).should(
          ($el) => {
            assert.equal($el.text(), visualName, 'visual name should be blank.');
          }
        );
      });
    });
    // Visual #2 - Radial Visual

    it('4 : convert table visual to Radial visual', () => {
      cy.intercept('POST', '/arc/reports/report/*').as('visualSubmitted');
      cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('visualUpdated');
      cy.get(CSSselector.visual_container).then(($visualcontainer) => {
        totalVisualCount = $visualcontainer.children().length;
        const dimension1 = 'year_id';
        const measure1 = 'is_playoffs';
        const measure2 = 'elo_i';
        const measure3 = 'game_result';
        const dimension1Expr = "[year_id] as 'Season'";
        const measure1Expr = "sum([is_playoffs]) as '# of Playoff Games'";
        const measure2Expr = "avg([elo_i]) as 'Elo i'";
        const measure3Expr = "sum(CASE [game_result] WHEN 'W' THEN 1 ELSE 0 END) as 'Wins'";
        const visualTitle = "<<Team:Lakers>>' Elo Difference, Games Won & Playoff Correlation";

        cy.intercept('POST', '/arc/reports/report/*').as('visualSubmitted');
        cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('visualUpdated');
        cy.intercept('POST', '/arc/reports/savedashboard').as('dashboardSaved');
        cy.intercept('POST', '/arc/reports/create/*').as('visualCreated');
        cy.intercept('GET', '/arc/reports/report/*').as('visualGet');

        cy.get(CSSselector.selecting_visuals).click({ force: true });
        cy.wait('@visualCreated');
        cy.wait('@visualSubmitted')
          .its('response.statusCode')
          .should('eq', 200);
        cy.wait('@visualGet');

        cy.wait('@visualUpdated');
        cy.get("div[class='gridster ready']>ul>li").then(($p2) => {
          expect($p2.children().length).to.equal(totalVisualCount + 1);
        });
        cy.get(CSSselector.visual_rows.replace('index', String(totalVisualCount + 1))).should(
          'be.visible'
        );
        cy.get(CSSselector.visual_edit_specific(totalVisualCount + 1)).click({
          force: true,
        });
        cy.get(CSSselector.radial_visual).click();
        cy.get(CSSselector.data_tab).click();
        cy.get(CSSselector.vizual_fields_tab.replace('column-name', dimension1)).click();
        cy.get("li[class='dropdown app-shelf-item'] span[title =" + dimension1 + ']').click();
        cy.get(CSSselector.order_topK_list.order_topK).click();
        cy.get(CSSselector.order_topK_list.order_descending).click();
        visuals.verifySQLExpression(dimension1Expr);
        cy.get(CSSselector.measure_field_element).click();
        cy.get(CSSselector.vizual_fields_tab.replace('column-name', measure1)).click();
        cy.get(
          "#app-shelves #aggregates_shelf li[class='dropdown app-shelf-item']:nth-child(1)"
        ).click();
        visuals.verifySQLExpression(measure1Expr);
        cy.get(CSSselector.close_property_panel).click();
        cy.get(CSSselector.vizual_fields_tab.replace('column-name', measure2)).click();
        cy.get(
          "#app-shelves #aggregates_shelf li[class='dropdown app-shelf-item']:nth-child(2)"
        ).click();
        visuals.verifySQLExpression(measure2Expr);
        cy.get(CSSselector.close_property_panel).click();
        cy.get(CSSselector.vizual_fields_tab.replace('column-name', measure3)).click();
        cy.get(
          "#app-shelves #aggregates_shelf li[class='dropdown app-shelf-item']:nth-child(3)"
        ).click();
        visuals.verifySQLExpression(measure3Expr);
        cy.get(CSSselector.visuals_title)
          .click()
          .type(visualTitle);
        cy.get(CSSselector.refresh_visual).click();
        cy.wait('@visualUpdated')
          .its('response.statusCode')
          .should('eq', 200);
        cy.get(CSSselector.save_visual).click();
        cy.wait('@visualSubmitted')
          .its('response.statusCode')
          .should('eq', 200);
        cy.get(CSSselector.Save).click({ force: true });
        cy.get(CSSselector.visual_radial_verify).should('be.visible');
        cy.get(CSSselector.visual_title.replace('index', String(totalVisualCount + 1))).should(
          ($el) => {
            assert.equal(
              $el.text(),
              "Lakers' Elo Difference, Games Won & Playoff Correlation",
              "Visual name should be Lakers' Elo Difference, Games Won & Playoff Correlation"
            );
          }
        );
        cy.get('#app-selected-list > ul > li:nth-child(2) .viz-chart .small-legend text').each(
          ($el, index, $list) => {
            if (index === 0) {
              assert.equal($el.text(), '# of Playoff Games', 'Should be # of Playoff Games');
            }
            if (index === 2) {
              assert.equal($el.text(), 'Wins', 'Should be Wins');
            }
          }
        );
      });
    });
    // Visual # 3 : Bar Visual
    it('5 : should convert  table visual to Bar visual', () => {
      visualName = 'Best ELO Ranking in Modern Era';

      const dimX = 'fran_id';
      const dimY = 'elo_i';
      const dimXExpr = "[fran_id] as 'Team'";
      const dimYExpr = "avg([elo_i]) as 'Average ELO Score'";
      const Colors = 'fran_id';
      const colorExpr = '[fran_id] as "Team"';
      const filterField = 'year_id';
      const filterExpr = '[year_id] > 1977';

      cy.get(CSSselector.visual_container).then(($visualcontainer) => {
        totalVisualCount = $visualcontainer.children().length;
        cy.intercept('POST', '/arc/reports/report/*').as('visualSubmitted');
        cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('visualUpdated');

        cy.intercept('POST', '/arc/reports/savedashboard').as('dashboardSaved');
        cy.intercept('POST', '/arc/reports/create/*').as('visualCreated');
        cy.intercept('GET', '/arc/reports/report/*').as('visualGet');

        cy.get(CSSselector.selecting_visuals).click({ force: true });
        cy.wait('@visualCreated');
        cy.wait('@visualSubmitted')
          .its('response.statusCode')
          .should('eq', 200);
        cy.wait('@visualGet');

        cy.wait('@visualUpdated');

        cy.get("div[class='gridster ready']>ul>li").then(($p2) => {
          expect($p2.children().length).to.equal(totalVisualCount + 1);
        });
        cy.get(CSSselector.visual_rows.replace('index', String(totalVisualCount + 1))).should(
          'be.visible'
        );
        cy.get(CSSselector.visual_edit_specific(totalVisualCount + 1)).click({
          force: true,
        });
        cy.get(CSSselector.bar_visual).click();
        cy.get(CSSselector.data_tab).click();
        cy.get(CSSselector.barVisual.x_Axis).click();

        cy.get(CSSselector.vizual_fields_tab.replace('column-name', dimX)).click();
        cy.get("#app-shelves #x_shelf li[class='dropdown app-shelf-item']:nth-child(1)").click();
        visuals.verifySQLExpression(dimXExpr);

        cy.get(CSSselector.barVisual.x_Axis).click();
        cy.get(CSSselector.measure_field).click();
        cy.get(CSSselector.vizual_fields_tab.replace('column-name', dimY)).click();
        cy.get("#app-shelves #y_shelf li[class='dropdown app-shelf-item']:nth-child(1)").click();
        visuals.verifySQLExpression(dimYExpr);

        cy.get(CSSselector.close_property_panel).click();
        cy.get(CSSselector.color_field).click();
        cy.get(CSSselector.vizual_fields_tab.replace('column-name', Colors)).click();
        cy.get(CSSselector.color_field_first_element).click();
        visuals.verifySQLExpression(colorExpr);
        cy.get(CSSselector.filter_field).click();
        cy.get(CSSselector.vizual_fields_tab.replace('column-name', filterField)).click();
        cy.get(
          "#app-shelves  #filters_shelf li[class='dropdown app-shelf-item']:nth-child(1)"
        ).click();
        visuals.verifySQLExpression(filterExpr);

        cy.get(CSSselector.visuals_title)
          .click()
          .type('Best ELO Ranking in Modern Era');
        cy.get(CSSselector.refresh_visual).click();

        cy.wait('@visualUpdated')
          .its('response.statusCode')
          .should('eq', 200);
        cy.get(CSSselector.save_visual).click();
        cy.wait('@visualSubmitted')
          .its('response.statusCode')
          .should('eq', 200);
        cy.get(CSSselector.Save).click({ force: true });

        cy.get(CSSselector.visual_bar_verify).should('be.visible');
        cy.get(CSSselector.visual_title.replace('index', String(totalVisualCount + 1))).should(
          ($el) => {
            assert.equal(
              $el.text(),
              'Best ELO Ranking in Modern Era',
              'Visual name should be Best ELO Ranking in Modern Era'
            );
          }
        );

        cy.get(
          `#app-selected-list > ul > li:nth-child(${totalVisualCount + 1}) [text-anchor='start']`
        ).should(($el) => {
          assert.equal($el.text(), 'Average ELO Score', 'Should be Average ELO Score');
        });
        cy.get(
          `#app-selected-list > ul > li:nth-child(${totalVisualCount + 1}) [text-anchor='middle']`
        ).should(($el) => {
          assert.equal($el.text(), 'Team', 'Should be Team');
        });
      });
    });
  });
});
