import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';
import * as visual from 'lib/visual';
import { areFilesEqual } from '../../csv_excel_download/utils';

const data = {
  connectionName: Cypress.env('smokeConnectionName'),
  datasetName: 'Cereals for Queries download',
  databaseName: 'main',
  tableName: 'cereals',
};

const tableDashboard: Dashboard = {
  title: 'Queries download test',
  subTitle: 'table visual test',
  sheets: [
    {
      sheet_name: 'Sheet 1',
      sheet_number: 1,
      visuals: [
        {
          name: 'table-visual',
          connection_name: data.connectionName,
          dataset_name: data.datasetName,
          visual_type: '.chart-icon-table',
          visual_title: 'simple_table',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.AGGREGATES,
                field_aggregate: 'Sum',
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Dashboard - Queries download', () => {
  before(() => {
    logins.silent();
    cy.visit('/');
    functions.deleteDatasetFromConnection(data.connectionName, data.datasetName);
    functions.createDatasetFromConnection(
      data.connectionName,
      data.databaseName,
      data.tableName,
      data.datasetName
    );
    dashboards.createDashboard(tableDashboard);
  });

  it('Test 1: Download Queries from the Show Usage Info modal', () => {
    cy.get(visual.locators.dash_ellipsis).click();
    cy.wait(500);
    cy.get(visual.locators.show_usage_info).click();
    cy.wait(500);
    cy.get(visual.locators.perf_modal.queries_tab).click();
    cy.wait(500);
    cy.get(visual.locators.perf_modal.downloadQueries_btn).click();

    cy.exec('ls -Art cypress/downloads | tail -n 1').then((results) => {
      const filename = results.stdout;
      const downloadedFile = `cypress/downloads/${filename}`;

      cy.readFile(`cypress/downloads/${filename}`, 'utf-8', { timeout: 10000 }).then(
        (downloadTxt) => {
          const htmlTagsRegex = /<\/?[a-z][\s\S]*>/i;

          cy.wrap(downloadTxt).should('not.match', htmlTagsRegex);

          const query =
            'SELECT TA_0.`cereal_name` as `cereal_name`, sum((TA_0.`calories`)) as `sum(calories)`\n' +
            '  FROM `main`.`cereals`  TA_0 \n' +
            '  GROUP BY 1\n' +
            '  LIMIT 100';

          expect(downloadTxt).to.include(query);
        }
      );
    });
  });
});
