// This script contains verification on test https://arcadiadata.atlassian.net/browse/ARC-7932
import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { openDashboardforEdit } from 'lib/dashboard';
import * as functions from 'lib/functions';
import * as workspaces from 'lib/workspace';
import { CSSselector } from 'locators/common/CSSselector';

describe('Dashboard sheet cloning.', () => {
  before(() => {
    cy.fixture('users.json').then((data) => {
      logins.silent(data.admin.username, data.admin.password);
    });
  });

  function get_time() {
    const a = new Date();
    return a.getTime();
  }

  function changeDBTitle(title: string) {
    cy.get(CSSselector.Dashboard_title)
      .click()
      .clear()
      .type(title);
  }
  describe('ARC-7932', () => {
    const dbName1 = `db1_${get_time()}`;
    const sheetName1 = `db1.s1${get_time()}`;
    const lvName1 = `lv1${get_time()}`;
    const vName1 = `v1${get_time()}`;
    const wsName1 = `w1_${get_time()}`;
    const f1Name1 = `f1_${get_time()}`;

    const dbName2 = `db2_${get_time()}`;
    const sheetName2 = `db2.s1${get_time()}`;
    const lv3Name2 = `lv3${get_time()}`;
    const wsName2 = `w2_${get_time()}`;
    const f1Name2 = `f2_${get_time()}`;

    const dbName3 = `db3_${get_time()}`;

    it('1: create a new dashboard', () => {
      functions.addNewDB(dbName1);
      dashboards.renameSheet(1, sheetName1);
      dashboards.createLinkedVisual(1, lvName1, Cypress.env('smokeConnectionName'), 'Cereals');
      dashboards.addVisual(2, vName1);
      cy.intercept('POST', '/arc/reports/savedashboard').as('savedashboard');
      cy.get(`${CSSselector.Save}`).click();
      cy.wait('@savedashboard')
        .its('response.statusCode')
        .should('eq', 200);
      functions.addFilter(f1Name1, Cypress.env('smokeConnectionName'), 'Cereals');
      workspaces.createWorkSpace(wsName1);
      cy.get(`${CSSselector.Visuals_all}`).click({
        force: true,
      });
      // functions.searchDashboard(dbName1);
      // cy.get(CSSselector.gridview_Visualspage).click();
      dashboards.openDashboard(dbName1);
      dashboards.dbMoveToWS(wsName1);

      // check the filter name:
      openDashboardforEdit(dbName1);
      cy.get(CSSselector.dashboardDetail.FilterName).should(($el) => {
        assert.equal($el.text(), f1Name1, 'Filter name is expected.');
      });
    });

    it('2: create a second dashboard', () => {
      functions.addNewDB(dbName2);
      dashboards.renameSheet(1, sheetName2);
      dashboards.createLinkedVisual(1, lv3Name2, Cypress.env('smokeConnectionName'), 'Cereals');
      dashboards.createLinkedVisual(2, '', Cypress.env('smokeConnectionName'), 'Cereals');
      cy.intercept('POST', '/arc/reports/savedashboard').as('savedashboard');
      cy.get(`${CSSselector.Save}`).click();
      cy.wait('@savedashboard')
        .its('response.statusCode')
        .should('eq', 200);
      functions.addFilter(f1Name2, Cypress.env('smokeConnectionName'), 'Cereals');
      workspaces.createWorkSpace(wsName2);
      cy.get(`${CSSselector.Visuals_all}`).click();
      dashboards.openDashboard(dbName2);
      dashboards.dbMoveToWS(wsName2);

      // functions.searchDashboard(dbName2);
      // cy.get('#report-table tr:nth-child(2)').should('not.exist');

      // cy.get(CSSselector.gridview_Visualspage).click();
      dashboards.openDashboardforEdit(dbName2);

      // cy.intercept('GET', '/arc/reports/customcss').as('newDashboard_customcss');
      // cy.intercept('GET', '/arc/datasets/dataconnection').as('newDashboard_dataconnection');
      // cy.intercept('GET', '/arc/workspace/workspace').as('newDashboard_workspace');
      // cy.intercept('GET', '/arc/reports/report?all=true').as('newDashboard_reportall');
      // cy.intercept('GET', '/arc/reports/report/*').as('newDashboard_report');
      // cy.get(`${CSSselector.dashboardDetail.menu} ${CSSselector.edit_link}`).click();

      // cy.wait('@newDashboard_report');
      // cy.wait('@newDashboard_workspace');
      // cy.wait('@newDashboard_customcss');
      // cy.wait('@newDashboard_dataconnection');
      // cy.wait('@newDashboard_reportall');

      dashboards.dbCloneMoveoptionSelection(
        CSSselector.dashboardDetail.workspaceMoveMaintainVisualRadio
      );

      changeDBTitle(dbName3);

      cy.intercept('POST', '/arc/reports/savedashboard').as('savedashboard');
      cy.get(`${CSSselector.Save}`).click();
      cy.wait('@savedashboard')
        .its('response.statusCode')
        .should('eq', 200);
      cy.get(CSSselector.Save).should('not.have.class', 'app-btn-highlight');

      // functions.searchDashboard(dbName3);

      // cy.get(CSSselector.gridview_Visualspage).click();

      dashboards.openDashboardforEdit(dbName3);

      // cy.intercept('GET', '/arc/reports/customcss').as('newDashboard_customcss');
      // cy.intercept('GET', '/arc/datasets/dataconnection').as('newDashboard_dataconnection');
      // cy.intercept('GET', '/arc/workspace/workspace').as('newDashboard_workspace');
      // cy.intercept('GET', '/arc/reports/report?all=true').as('newDashboard_reportall');
      // cy.intercept('GET', '/arc/reports/report/*').as('newDashboard_report');
      // cy.get(`${CSSselector.dashboardDetail.menu} ${CSSselector.edit_link}`).click();

      // cy.wait('@newDashboard_report');
      // cy.wait('@newDashboard_workspace');
      // cy.wait('@newDashboard_customcss');
      // cy.wait('@newDashboard_dataconnection');
      // cy.wait('@newDashboard_reportall');
      dashboards.dbMoveToWS(wsName1);
    });

    it('3: Clone/maintain first dashboard', () => {
      functions.searchDashboard(dbName1);
      cy.get('#report-table tr:nth-child(2)').should('not.exist');
      cy.get(CSSselector.gridview_Visualspage).click();
      dashboards.openDashboardforEdit(dbName1);

      // cy.intercept('GET', '/arc/reports/customcss').as('newDashboard_customcss');
      // cy.intercept('GET', '/arc/datasets/dataconnection').as('newDashboard_dataconnection');
      // cy.intercept('GET', '/arc/workspace/workspace').as('newDashboard_workspace');
      // cy.intercept('GET', '/arc/reports/report?all=true').as('newDashboard_reportall');
      // cy.intercept('GET', '/arc/reports/report/*').as('newDashboard_report');
      // cy.get(`${CSSselector.dashboardDetail.menu} ${CSSselector.edit_link}`).click();

      // cy.wait('@newDashboard_report');
      // cy.wait('@newDashboard_workspace');
      // cy.wait('@newDashboard_customcss');
      // cy.wait('@newDashboard_dataconnection');
      // cy.wait('@newDashboard_reportall');

      dashboards.dbCloneMoveoptionSelection(
        CSSselector.dashboardDetail.workspaceMoveCloneVisualRadio
      );

      changeDBTitle('DB1-cloned');

      cy.intercept('POST', '/arc/reports/savedashboard').as('savedashboard');
      cy.get(`${CSSselector.Save}`).click();
      cy.wait('@savedashboard')
        .its('response.statusCode')
        .should('eq', 200);

      cy.get(CSSselector.Save).should('not.have.class', 'app-btn-highlight');

      cy.get(`${CSSselector.dashboardDetail.FilterName}`).should(($el) => {
        assert.equal($el.text(), f1Name1, 'Filter name is expected.');
      });
      cy.get(CSSselector.visual_title.replace('index', '1')).should(($el) => {
        assert.equal($el.text(), lvName1, 'Linked visual name is expected.');
      });
      cy.get(CSSselector.visual_title.replace('index', '2')).should(($el) => {
        assert.equal($el.text(), vName1, 'visual name is expected.');
      });

      // functions.searchDashboard(dbName1);
      // cy.get('#report-table tr:nth-child(2)').should('not.exist');
      // cy.get(CSSselector.gridview_Visualspage).click();
      dashboards.openDashboardforEdit(dbName1);

      // cy.intercept('GET', '/arc/reports/customcss').as('newDashboard_customcss');
      // cy.intercept('GET', '/arc/datasets/dataconnection').as('newDashboard_dataconnection');
      // cy.intercept('GET', '/arc/workspace/workspace').as('newDashboard_workspace');
      // cy.intercept('GET', '/arc/reports/report?all=true').as('newDashboard_reportall');
      // cy.intercept('GET', '/arc/reports/report/*').as('newDashboard_report');
      // cy.get(`${CSSselector.dashboardDetail.menu} ${CSSselector.edit_link}`).click();

      // cy.wait('@newDashboard_report');
      // cy.wait('@newDashboard_workspace');
      // cy.wait('@newDashboard_customcss');
      // cy.wait('@newDashboard_dataconnection');
      // cy.wait('@newDashboard_reportall');

      dashboards.dbCloneMoveoptionSelection(
        CSSselector.dashboardDetail.workspaceMoveMaintainVisualRadio
      );

      changeDBTitle('DB1-Maintain');

      cy.intercept('POST', '/arc/reports/savedashboard').as('savedashboard');
      cy.get(`${CSSselector.Save}`).click();
      cy.wait('@savedashboard')
        .its('response.statusCode')
        .should('eq', 200);

      cy.get(CSSselector.Save).should('not.have.class', 'app-btn-highlight');

      cy.get(`${CSSselector.dashboardDetail.FilterName}`).should(($el) => {
        assert.equal($el.text(), f1Name1, 'Filter name is expected.');
      });
      cy.get(CSSselector.visual_title.replace('index', '1')).should(($el) => {
        assert.equal($el.text(), lvName1, 'Linked visual name is expected.');
      });
      cy.get(CSSselector.visual_title.replace('index', '2')).should(($el) => {
        assert.equal($el.text(), vName1, 'visual name is expected.');
      });

      cy.get(
        `${CSSselector.dashboardDetail.dashboardContainer} li.gs-w:first-child ${CSSselector.dashboardDetail.linkVisualIcon}`
      ).should(($el) => {
        return expect($el).to.exist;
      });
    });

    it('4: Move Dashboard ', () => {
      functions.searchDashboard(dbName1);
      cy.get('#report-table tr:nth-child(2)').should('not.exist');
      cy.get(CSSselector.gridview_Visualspage).click();
      dashboards.openDashboardforEdit(dbName1);

      // cy.intercept('GET', '/arc/reports/customcss').as('newDashboard_customcss');
      // cy.intercept('GET', '/arc/datasets/dataconnection').as('newDashboard_dataconnection');
      // cy.intercept('GET', '/arc/workspace/workspace').as('newDashboard_workspace');
      // cy.intercept('GET', '/arc/reports/report?all=true').as('newDashboard_reportall');
      // cy.intercept('GET', '/arc/reports/report/*').as('newDashboard_report');

      // cy.get(`${CSSselector.dashboardDetail.menu} ${CSSselector.edit_link}`).click();

      // cy.wait('@newDashboard_report');
      // cy.wait('@newDashboard_workspace');
      // cy.wait('@newDashboard_customcss');
      // cy.wait('@newDashboard_dataconnection');
      // cy.wait('@newDashboard_reportall');

      dashboards.dbMoveToWS(wsName2);

      cy.get(`${CSSselector.dashboardDetail.FilterName}`).should(($el) => {
        assert.equal($el.text(), f1Name1, 'Filter name is expected.');
      });
      cy.get(CSSselector.visual_title.replace('index', '1')).should(($el) => {
        assert.equal($el.text(), lvName1, 'Linked visual name is expected.');
      });
      cy.get(CSSselector.visual_title.replace('index', '2')).should(($el) => {
        assert.equal($el.text(), vName1, 'visual name is expected.');
      });
      cy.get(CSSselector.visual_title.replace('index', '1')).should(($el) => {
        return expect($el).to.exist;
      });

      // functions.searchDashboard(dbName1);
      // cy.get('#report-table tr:nth-child(2)').should('not.exist');
      // cy.get(CSSselector.gridview_Visualspage).click();
      dashboards.openDashboardforEdit(dbName1);

      // cy.intercept('GET', '/arc/reports/customcss').as('newDashboard_customcss');
      // cy.intercept('GET', '/arc/datasets/dataconnection').as('newDashboard_dataconnection');
      // cy.intercept('GET', '/arc/workspace/workspace').as('newDashboard_workspace');
      // cy.intercept('GET', '/arc/reports/report?all=true').as('newDashboard_reportall');
      // cy.intercept('GET', '/arc/reports/report/*').as('newDashboard_report');
      // cy.get(`${CSSselector.dashboardDetail.menu} ${CSSselector.edit_link}`).click();

      // cy.wait('@newDashboard_report');
      // cy.wait('@newDashboard_workspace');
      // cy.wait('@newDashboard_customcss');
      // cy.wait('@newDashboard_dataconnection');
      // cy.wait('@newDashboard_reportall');

      dashboards.dbMoveToWS(wsName1);
      dashboards.openDashboardforEdit(dbName1);

      dashboards.dbCloneToWS(wsName2);

      cy.get(`${CSSselector.dashboardDetail.FilterName}`).should(($el) => {
        assert.equal($el.text(), f1Name1, 'Filter name is expected.');
      });
      cy.get(CSSselector.visual_title.replace('index', '1')).should(($el) => {
        assert.equal($el.text(), lvName1, 'Linked visual name is expected.');
      });
      cy.get(CSSselector.visual_title.replace('index', '2')).should(($el) => {
        assert.equal($el.text(), vName1, 'visual name is expected.');
      });
    });
  });
});
