import { createConnection, selectConnectionByName } from 'lib/connections';
import { extractTargetDb } from 'lib/data/util';
import { silent } from 'lib/logins';

const loc = {
  ADD_DATA_BUTTON: '#import-data-templ',
  FILES_SELECTOR: 'input[type=file]:visible',
  GET_DATA_BUTTON: 'button[type=submit]:visible',
  IMPORTED_FILE_LABEL: '#datasource-src-file',
  TARGET_TABLE: '#datasource-tablename',
  CONFIRM_IMPORT: '#datasource-confirm-import-btn',
  CONFIRM_IMPORT_2: '#datasource-confirm-import-yes',
  DATABASE_TABLES_LIST: '.table-list-main',
};

describe('Data import tests', () => {
  before(() => {
    silent();
  });

  it('can import CSV file', () => {
    createConnection(extractTargetDb);
    selectConnectionByName(extractTargetDb.connection_name);

    cy.get(loc.ADD_DATA_BUTTON).click();

    const filename = 'earthquakes-100-rows.csv';
    cy.get(loc.FILES_SELECTOR).selectFile(`cypress/tests/data/${filename}`);
    cy.get(loc.GET_DATA_BUTTON).click();

    // preview page:
    // source file name should be displayed
    cy.get(loc.IMPORTED_FILE_LABEL).should('have.text', filename);

    // TODO: figure out what we need to wait for (clicking too fast results in error)
    cy.wait(3000);

    // type in target table name:
    const tableName = `earthquakes_100_rows_${new Date().getTime()}`;
    cy.get(loc.TARGET_TABLE).click();
    cy.get(loc.TARGET_TABLE).clear();
    cy.get(loc.TARGET_TABLE).type(tableName);

    cy.get(loc.CONFIRM_IMPORT).click();

    // second confirmation
    cy.get(loc.CONFIRM_IMPORT_2).click();

    // find the newly created table in the tables list:
    cy.get(loc.DATABASE_TABLES_LIST).contains(tableName);
  });
});
