import { dataset } from 'fixtures/locators/dataset';
import { notifications } from 'fixtures/locators/notifications';
import { goToDataPage, waitForAceEditorReady } from 'lib/functions';
import { silent } from 'lib/logins';
import { getRandomInt } from 'tests/jobs/utils';

const datasetName = 'Dataset deletion test ' + getRandomInt(0, 10000);

describe('Delete dataset from the Dataset Detail page', () => {
  before(() => {
    silent();
  });

  context('Testing if dataset deletion functions properly on the Dataset Detail page', () => {
    before(() => {
      goToDataPage();
    });
    it('Creates a new dataset and deletes it on the Dataset Detail page', () => {
      cy.get(dataset.samples_connection).click();
      cy.get(dataset.new_dataset_button).click();
      cy.get(dataset.dataset_title_input).type(datasetName);
      cy.get(dataset.sql_source).type('sql');
      cy.get(dataset.sql_option).click();
      cy.get(dataset.sql_editor_container)
        .find('.ace_editor')
        .then((aceEditor) => {
          waitForAceEditorReady(aceEditor);
        })
        .then((aceEditor) => cy.wrap(aceEditor).type(`SELECT "test"`));
      cy.get(dataset.create_button).click();
      cy.wait(1000);
      cy.get(dataset.dataset_title).each(($el) => {
        if ($el.text() === datasetName) {
          cy.wrap($el).click();
        }
      });
      cy.get(dataset.dataset_actions_dropdown).click();
      cy.get(dataset.delete_option).click();
      cy.get(dataset.confirmation_input).type('DELETE');
      cy.get(dataset.delete_dataset_button).click();
      cy.get(notifications.notification_message)
        .should('be.visible')
        .and('contain.text', `has been deleted`);
    });
  });
});
