import * as locators from 'lib/customsettings/locators';
import * as dashboards from 'lib/dashboard';
import * as dsManage from 'lib/datasetManagement';
import * as visual from 'lib/visual';
import { customColors } from '../../../../fixtures/locators/customColors';

const { useDatasetColorSettingCheckbox } = visual.locators.style.colors;
const { colorSettingSection } = customColors.stylePanel;
export const unCheckCustomColorSetting = () => {
  dashboards.chooseVisualTab('colors');
  cy.get(useDatasetColorSettingCheckbox).uncheck();
};

export const settingsShouldBeEnabled = (settingsLocator: string[]) => {
  settingsLocator.forEach((locator) => {
    cy.get(locator).should('be.enabled');
  });
};

export const settingsShouldBeDisabled = (settingsLocator: string[]) => {
  settingsLocator.forEach((locator) => {
    cy.get(locator).should('be.disabled');
  });
};

export const removeElementFromColorShelf = (name: string) => {
  dashboards.chooseVisualTab('data');
  cy.get(locators.locators.shelves.colorShelf).scrollIntoView();
  cy.get(
    locators.locators.removeButtonOnShelfItem(locators.locators.shelves.colorShelf, name)
  ).click({ force: true });
};

export const checkNthColorPaletteIsSelected = (index: number) => {
  cy.get(locators.locators.colorSettings.colorpaletteSelectionNthItem(index)).should(
    'have.class',
    'settings-color-selected'
  );
};

export const navigateToFieldDatasetMenuAndPressEdit = () => {
  cy.log('navigateToFieldDatasetMenuAndPressEdit start');
  cy.get(dsManage.locators.fields).click();
  cy.get(dsManage.locators.editField).click();
  cy.log('navigateToFieldDatasetMenuAndPressEdit end');
};

export const testColorAndOpacityValueOfNthBarOnBarVisual = (
  index: number,
  color: string | undefined,
  opacity: number
) => {
  if (color === undefined) {
    throw new Error(`Test style(${color}) value must not be undefined`);
  }
  testInlineStyleOfNthBar(index, 'fill', color);
  testInlineStyleOfNthBar(index, 'opacity', (opacity / 100).toString());
};

export const testInlineStyleOfNthBar = (index: number, styleProp: string, value: string) => {
  cy.get(locators.locators.visual.nthBarOnVisual(index)).should('have.css', styleProp, value);
};

export const testFixedMapColorAndOpacity = (index: number, color: string, opacity: number) => {
  cy.get(colorSettingSection.fixedDimensionToColorMapList)
    .scrollIntoView()
    .find(locators.locators.colorSettings.nthColorBoxWithStyle(index, `background-color: ${color}`))
    .should('exist');

  cy.get(colorSettingSection.fixedDimensionToColorMapList)
    .scrollIntoView()
    .find(locators.locators.colorSettings.nthColorBoxWithStyle(1, `opacity: ${opacity / 100}`))
    .should('exist');
};

export const hexToRgb = (hex: string) => {
  const hexWithoutHashTag = hex.replace('#', '');
  const bigint = parseInt(hexWithoutHashTag, 16);
  /* tslint:disable:no-bitwise */
  const r = (bigint >> 16) & 255;
  const g = (bigint >> 8) & 255;
  const b = bigint & 255;
  /* tslint:enable:no-bitwise */

  return `rgb(${r}, ${g}, ${b})`;
};
