import * as arctest from 'arctest';
import { CSSselector } from '../../locators';

describe('Custom Dateranges Test', () => {
  before(() => {
    arctest.logins.silent();
  });

  it('Check Custom Date Navigation', () => {
    // Tests to see if Dropdown Menu opens okay
    cy.visit('/');
    cy.get(CSSselector.site_admin_menus.site_admin_dropdown_toggle).click();

    // Dropdown menu opens okay
    cy.get(CSSselector.site_admin_menus.site_admin_dropdown_toggle)
      .next('.dropdown-menu')
      .should('be.visible');

    // Click on the calendar
    cy.get(CSSselector.site_admin_menus.site_admin_dropdown_toggle)
      .next('.dropdown-menu')
      .find('.fa-calendar')
      .parent()
      .click();

    cy.url().should('eq', `${Cypress.config().baseUrl}/arc/apps/customdates`);
  });

  it('Create a new Custom Date', () => {
    cy.visit('/arc/apps/customdates').getCookies();

    cy.get('.new-customstyle').click();

    cy.url().should('eq', `${Cypress.config().baseUrl}/arc/apps/customdates/0`);

    // Should have an empty name form, a table with one existing range on it with proper
    // defaults, and an "Add New Range Button"
    cy.get('#style-name').should('have.value', '');

    // Only one range in your daterange set at the start
    cy.get('.custom-daterange-table tbody')
      .find('tr')
      .should('have.length', 1)
      .first()
      .as('firstRow');

    // Correct default name
    cy.get('@firstRow')
      .find('.daterange-name-input')
      .first()
      .should('have.attr', 'placeholder', 'New Range');

    // Correct default start anchor
    cy.get('@firstRow')
      .find('.daterange-start-anchor-input')
      .first()
      .should('have.value', 'TODAY');

    // Correct default end anchor
    cy.get('@firstRow')
      .find('.daterange-end-anchor-input')
      .first()
      .should('have.value', 'TODAY');

    // Correct number of rules for start anchor
    cy.get('@firstRow')
      .find('.daterange-start-rule-input')
      .first()
      .find('.daterange-rule')
      .should('have.length', 0);

    // Correct number of rules for start anchor
    cy.get('@firstRow')
      .find('.daterange-end-anchor-input')
      .first()
      .find('.daterange-rule')
      .should('have.length', 0);

    cy.get('#add-range-btn');
  });

  it('Test Custom Date Functionality', () => {
    cy.visit('/arc/apps/customdates/0');

    cy.get('#add-range-btn').click();

    cy.get('.custom-daterange-table tbody')
      .find('tr')
      .should('have.length', 2)
      .first()
      .as('firstRow');

    // Click on the input field, check to see that the anchor appears
    cy.get('@firstRow')
      .find('.daterange-start-anchor-input')
      .first()
      .click();
    cy.get('@firstRow')
      .find('.daterange-start-anchor-dropdown')
      .first()
      .as('startAnchorDropdown')
      .should('be.visible');
  });
});
