import * as arctest from 'lib/arctest';
import * as dashboard from 'lib/dashboard';
import * as dsManage from 'lib/datasetManagement';
import * as dis_format from 'lib/display_format';
import * as functions from 'lib/functions';
import * as dataset from 'lib/permissions';
import { CSSselector } from 'locators/common/CSSselector';

function verifyTableData(selector: string, data: string) {
  cy.get(selector).should('have.text', data);
}
describe('Display Format.', () => {
  const connectionName = 'samples';
  const databaseName = 'main';
  const tableName = 'restaurant_scores_lives_standard';
  const datasetName = 'test_restaurant_scores_lives_standard';
  const dashTrIndexesList = ['2', '6', '7', '2', '6', '17', '1', '3', '4', '2', '6', '7'];
  const dashTdIndexesList = ['1', '1', '1', '2', '2', '2', '3', '3', '3', '4', '4', '4'];
  const dashDataList = [
    '37.7603',
    '37.7317',
    '37.7391',
    '14,155,863,507',
    '14,155,331,526',
    '14,155,793,534',
    '4/26/16 12:00:00 AM',
    '8/25/16 12:00:00 AM',
    '11/17/16 12:00:00 AM',
    '$94,114.00',
    '$94,124.00',
    '$94,116.00',
  ];

  before(() => {
    cy.fixture('users.json').then((data) => {
      arctest.logins.silent(data.admin.username, data.admin.password);
    });
  });

  context('ARC-10802: Automate smoke test for Display Format', () => {
    it('1 : Change display format of fields from dataset', () => {
      cy.visit('/');
      functions.createDatasetFromConnection(connectionName, databaseName, tableName, datasetName);
      functions.navigate_to_data();
      dataset.selectDataset(connectionName, datasetName);
      // cy.contains('test_restaurant_scores_lives_standard').click();
      cy.get(CSSselector.dataset_detail.fields).click();
      cy.get(CSSselector.dataset_detail.edit_field).click();

      // Change the format of Inspection Date
      dis_format.changeDisplayFormatOfdate(
        false,
        dis_format.locators.inspection_date_edit_field,
        'dddd, MMMM Do, YYYY'
      );
      dis_format.changeDisplayFormatOfnumber(
        false,
        dis_format.locators.business_latitude_edit_field
      );
      dis_format.changeDisplayFormatOfInteger(
        false,
        dis_format.locators.business_longitude_edit_field
      );
      dis_format.changeDisplayFormatOfCurrency(false, dis_format.locators.postal_code_edit_field);
      dsManage.saveEditField();
    });

    it('2 : Verify applied changes on Visual', () => {
      // check for number
      const trIndexesList = ['1', '2', '7', '2', '6', '7', '1', '2', '3', '1', '2', '7'];
      const tdIndexesList = ['7', '7', '7', '8', '8', '8', '12', '12', '12', '6', '6', '6'];
      const data = [
        '37.80',
        '37.76',
        '37.74',
        '-122',
        '-122',
        '-122',
        'Tuesday, April 26th, 2016',
        'Friday, August 26th, 2016',
        'Thursday, August 25th, 2016',
        '$94,133.00',
        '$94,114.00',
        '$94,116.00',
      ];
      functions.navigate_to_home();
      cy.get(CSSselector.home_new_dashboard).click();
      functions.createNewVisual('samples', 'test_restaurant_scores_lives_standard');
      cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('jsonselectparallel');
      // Coverage of bug ARC-10567: [Regression] Display Format is not respected for default visuals
      for (let index = 0; index < data.length; index++) {
        const selector = dis_format.locators.dash_fields
          .replace('trIndex', trIndexesList[index])
          .replace('tdIndex', tdIndexesList[index]);
        verifyTableData(selector, data[index]);
      }
    });

    it('3 : Change display format of fields from visual and verify the changes', () => {
      arctest.screen.builder.modal.addShelfColumn('dimensions_shelf', 'business_latitude');
      arctest.screen.builder.modal.addShelfColumn('dimensions_shelf', 'business_phone_number');
      arctest.screen.builder.modal.addShelfColumn('dimensions_shelf', 'inspection_date');
      arctest.screen.builder.modal.addShelfColumn('dimensions_shelf', 'business_postal_code');
      cy.get(CSSselector.refresh_visual).click();

      dis_format.changeDisplayFormatOfdate(
        true,
        dis_format.locators.select_insp_date,
        'M/D/YY h:mm:ss A'
      );
      dis_format.changeDisplayFormatOfnumber(true, dis_format.locators.select_busi_latitude, '4');
      dis_format.changeDisplayFormatOfInteger(true, dis_format.locators.select_busi_phone_number);
      dis_format.changeDisplayFormatOfCurrency(true, dis_format.locators.select_busi_postalcode);
      cy.get(CSSselector.refresh_visual).click();
      for (let index = 0; index < dashDataList.length; index++) {
        const selector = dis_format.locators.dash_fields
          .replace('trIndex', dashTrIndexesList[index])
          .replace('tdIndex', dashTdIndexesList[index]);
        verifyTableData(selector, dashDataList[index]);
      }
      dashboard.closeVisual(false);
      cy.get(dashboard.locators.dashboard_title)
        .click()
        .type('dis_format_dashboard');
      cy.get(CSSselector.Save).click();
    });

    it('4 : Verify the changes in dashboard edit and view mode', () => {
      for (let index = 0; index < dashDataList.length; index++) {
        const selector = dis_format.locators.dash_fields
          .replace('trIndex', dashTrIndexesList[index])
          .replace('tdIndex', dashTdIndexesList[index]);
        verifyTableData(selector, dashDataList[index]);
      }

      dashboard.viewDashboard();
      for (let index = 0; index < dashDataList.length; index++) {
        const selector = dis_format.locators.dash_fields
          .replace('trIndex', dashTrIndexesList[index])
          .replace('tdIndex', dashTdIndexesList[index]);
        verifyTableData(selector, dashDataList[index]);
      }
    });
  });
});
