import { jobs } from 'fixtures/locators/jobs';
import { logins } from 'lib/arctest';
import * as functions from 'lib/functions';
import { clickOnFirstJobName, createEmailJob, editEmailJob, JobType } from 'tests/jobs/utils';

const generalText = 'Job for testing editing';

function refreshJobsPage() {
  cy.get(jobs.job_overview.refresh).click();
}

function openEditEmailModal() {
  refreshJobsPage();
  cy.get(jobs.job_overview.refresh, { timeout: 10000 })
    .should('not.be.disabled')
    .then(() => {
      cy.get(`${jobs.job_overview.jobs_table} > tbody > tr`)
        .eq(0)
        .find(jobs.job_overview.jobs.actions)
        .click();
      cy.get(jobs.job_overview.jobs.edit).click();
      cy.wait(500);
    });
}

function openJobDetailModal() {
  cy.get(`${jobs.job_overview.jobs_table} > tbody > tr`)
    .eq(0)
    .find(jobs.job_overview.jobs.actions)
    .click();
  cy.get(jobs.job_overview.jobs.details).click();
}

describe('Edit Email', () => {
  before(() => {
    logins.silent();
  });

  context('Create and edit scheduled email', () => {
    before(() => {
      createEmailJob(JobType.Scheduled, generalText);
      functions.goToJobSchedulePage();
      openEditEmailModal();
      editEmailJob();
      openJobDetailModal();
    });

    it('Should display the changes', () => {
      cy.get(jobs.job_details_modal).within(() => {
        cy.get(jobs.job_type).should('contain', 'Email');
        cy.get(jobs.job_interval).should('contain', 'Scheduled');
        cy.get(jobs.job_name).should('contain', 'EDITED');
        cy.get(jobs.job_schedule).should('contain', 'Weekly');
        cy.get(jobs.email_subject).should('contain', 'EDITED');
        cy.get(jobs.email_message).should('contain', 'EDITED');
      });
    });
  });


  context('Create threshold email', () => {
    it('Create threshold email', () => {
      createEmailJob(JobType.Threshold);
    });
  });

  context('Edit threshold email', () => {
    before(() => {
      functions.goToJobSchedulePage();
      openEditEmailModal();
    });

    it('Email type should be threshold email', () => {
      cy.contains('Edit Job Details');
      cy.get(jobs.email_modal.threshold_tab.trigger);
    });

    it('Edit threshold email details', () => {
      cy.log('Edit job fields');

      cy.get(jobs.email_modal.form).within(() => {
        cy.get(jobs.email_modal.schedule_tab.trigger).click();
        cy.get(jobs.email_modal.schedule_tab.name).type(' EDITED');
        cy.get(jobs.email_modal.schedule_tab.schedule_group_id).click().type('Daily').type('{enter}');

        cy.get(jobs.email_modal.threshold_tab.trigger).click();
        cy.get(jobs.email_modal.threshold_tab.trigger_id).clear().type('1');
        cy.get(jobs.email_modal.threshold_tab.minimum_time_interval).click().type('Once a day{enter}');

        cy.get('[name="thresholdTarget.triggerConditions.0.equality"]').select('>');
        cy.get('[name="thresholdTarget.triggerConditions.0.value"]').clear().type('1');
        cy.get('[data-test-id="add-trigger-condition-button"]').click();
        cy.get('[name="thresholdTarget.triggerConditions.1.value"]').clear().type('3');
      });
      cy.get(jobs.email_modal.submit).click();
      cy.log('Save changes');

      openJobDetailModal();
      cy.get(jobs.job_details_modal).within(() => {
        cy.get(jobs.job_type).should('contain', 'Email');
        cy.get(jobs.job_name).should('contain', 'EDITED');
        cy.get(jobs.job_schedule).should('contain', 'Daily');
        cy.get(jobs.close_run_details).click();
      });

      clickOnFirstJobName('scheduledJobsTable');
      cy.get(jobs.details_toggle).click();
      cy.get(jobs.trigger_visual_id).should('contain', '1');
      cy.get(jobs.email_limit).should('contain', 'Once a day');
      cy.get(jobs.trigger_condition).eq(0).should('contain', '> 1');
      cy.get(jobs.trigger_condition).eq(1).should('contain', '< 3');
    });
  });
});
