import { jobs } from 'fixtures/locators/jobs';
import { notifications } from 'fixtures/locators/notifications';
import * as functions from 'lib/functions';
import * as moment from 'moment';
import { CSSselector } from '../../../locators';

export function addEmailTemplate(emailInfo: any) {
  functions.goToSiteAdminSettingsMenu('email_templates_site_admin');

  cy.get(CSSselector.email_templates.new_template_button).click();

  cy.get(CSSselector.email_templates.template_name).type(emailInfo.name);
  cy.get(CSSselector.email_templates.template_description).type(emailInfo.description);

  cy.get(CSSselector.email_templates.editor_top_toolbar)
    .find(CSSselector.email_templates.font_name)
    .click()
    .get(CSSselector.email_templates.font_drop_down)
    .contains('Helvetica')
    .click();

  addEmailTemplateContent(emailInfo);

  cy.get(CSSselector.email_templates.footer_text).then(($body) => {
    $body.text(emailInfo.email_footer_text);
  });

  cy.get(CSSselector.email_templates.wrapper_footer_text).then(($body) => {
    $body.text(emailInfo.email_wrapper_footer);
  });

  cy.get(CSSselector.email_templates.save_button).click();
  cy.wait(2000);
  cy.get(CSSselector.email_templates.back_breadcrumb).click();
}

export function addEmailTemplateContent(emailInfo: any) {
  const jobParameters = [
    {
      text: 'creator_username: ',
      selector: CSSselector.email_templates.job_parameters.username_sender,
    },
    {
      text: 'app_id: ',
      selector: CSSselector.email_templates.job_parameters.visual_dashboard_id,
    },
    {
      text: 'visual_name: ',
      selector: CSSselector.email_templates.job_parameters.visual_dashboard_name,
    },
    {
      text: 'visual_url: ',
      selector: CSSselector.email_templates.job_parameters.visual_dashboard_link,
    },
    {
      text: 'location_pathname: ',
      selector: CSSselector.email_templates.job_parameters.relative_link,
    },
    {
      text: 'start_time: ',
      selector: CSSselector.email_templates.job_parameters.job_start_time,
    },
    {
      text: 'url_params: ',
      selector: CSSselector.email_templates.job_parameters.page_parameters,
    },
    {
      text: 'run_time: ',
      selector: CSSselector.email_templates.job_parameters.total_job_run_time,
    },
  ];
  // fill message area
  cy.get(CSSselector.email_templates.email_message).then(($body) => {
    $body
      .text(`<u>${emailInfo.email_body_heading}</u>`)
      .append('<br>')
      .append('<br>')
      .append(emailInfo.email_body_message)
      .append('<br>')
      .append('<br>');
  });
  // add the jobParameters to the messagebody
  jobParameters.forEach(({ text, selector }) => {
    cy.get(CSSselector.email_templates.email_message)
      .click()
      .type(text);
    cy.get(CSSselector.email_templates.job_parameters.job_parameters_dropdown_btn).click();
    cy.get(selector).click();
    cy.wait(500);
    cy.get(CSSselector.email_templates.email_message).then(($body) => {
      $body.append('<br>').append('<br>');
    });
  });
}

export function deleteEmailTemplate(emailTemplateName: string) {
  const functionName = 'deleteEmailTemplate';
  cy.log(`${functionName} - started`);

  functions.goToSiteAdminSettingsMenu('email_templates_site_admin');
  cy.wait(2000);

  cy.get('#table-customemails').then(($elem) => {
    const emailTemplateTitles = $elem.find(
      'tr[data-test-id="custom-color-option"] td:nth-child(2)'
    );
    const index = Array.from(emailTemplateTitles).findIndex(
      (title) => title.textContent === emailTemplateName
    );

    if (index !== -1) {
      const templateTitle = emailTemplateTitles[index].textContent;
      cy.log(`${functionName} - ${templateTitle} found and will be deleted!`);

      const deleteButton = $elem.find(CSSselector.email_templates.email_trash)[index];
      deleteButton.click();

      cy.get('#delete-confirmation-text').type('DELETE');

      cy.get('#delete-confirmation-yes')
        .should('be.not.disabled')
        .then(() => {
          functions.modalClick('#delete-confirmation-yes');
        });
    }
  });
  cy.log(`${functionName} - finished`);
}

export function checkNewlyCreateEmailTemplateExists(emailTemplateName: string) {
  const functionName = 'checkNewlyCreateEmailTemplateExists';
  cy.log(`${functionName} - started`);

  functions.goToSiteAdminSettingsMenu('email_templates_site_admin');
  cy.wait(2000);

  cy.get('#table-customemails').then(($elem) => {
    const emailTemplateTitles = $elem.find(
      'tr[data-test-id="custom-color-option"] td:nth-child(2)'
    );
    const index = Array.from(emailTemplateTitles).findIndex(
      (title) => title.textContent === emailTemplateName
    );

    if (index !== -1) {
      const templateTitle = emailTemplateTitles[index].textContent;
      cy.log(`${functionName} - ${templateTitle} found!`);
    }
  });
  cy.log(`${functionName} - finished`);
}

export function fillAndSendEmailModal() {
    cy.get(jobs.email_modal.form).within(() => {
      cy.get(jobs.email_modal.email_tab.emailTo)
      .click()
      .type('viztest00@gmail.com')
      .type('{enter}');
    cy.get(jobs.email_modal.email_tab.template).click().type('Sample template made from automation{enter}');
  });
  cy.get(jobs.email_modal.submit).click();

  cy.get(notifications.notification_content, { timeout: 30000 }).should(
    'contain',
    'Email successfully sent'
  );
}

export function getLatestJobLog(emailTemplateName: string) {
  const expectedJobName = `Adhoc Email: [Cloudera Data Visualization] ${emailTemplateName}`;

  return cy
    .request('GET', 'arc/jobs/api/logs/latest')
    .its('body')
    .then((body) => {
      const extractedData = body.data
        .filter((item: any) => item.job.job_name === expectedJobName)
        .slice(0, 1)
        .map((item: any) => {
          const contentsArchiveObj = JSON.parse(item.log.contents_archive);
          const start_time = item.log.start_time;
          const formattedStartTime = moment.utc(start_time).format('MM/DD/YY HH:mm:ss [UTC]');
          const fullUrlParams = contentsArchiveObj[1].url_params;
          const urlParamsWithoutEmbedParam = fullUrlParams.substring(
            fullUrlParams.indexOf('&') + 1
          );
          return {
            creator_username: item.job.creator_username,
            app_id: contentsArchiveObj[1].app_id,
            visual_name: emailTemplateName,
            visual_url: `${contentsArchiveObj[1].visual_url}?${fullUrlParams}`,
            location_pathname: `${contentsArchiveObj[1].location_pathname}?${urlParamsWithoutEmbedParam}`,
            start_time: formattedStartTime,
            url_params: contentsArchiveObj[1].url_params,
            run_time: contentsArchiveObj.run_parameters.run_time,
          };
        });

      return extractedData;
    });
}

export function deleteLatestJob() {
  const functionName = 'deletePreviouslyCreatedJob';
  const jobTestName = 'Adhoc Email: [Cloudera Data Visualization] Email send out test';
  cy.log(`${functionName} - started`);

  functions.goto_job_status();
  cy.wait(2000);

  cy.get(jobs.job_overview.runs_tab).then(($elem) => {
    const jobTitles = $elem.find('tbody tr td.arc-job-name');

    const index = Array.from(jobTitles).findIndex((title) => title.textContent === jobTestName);

    if (index !== -1) {
      const jobTitle = jobTitles[index].textContent;
      cy.log(`${functionName} - ${jobTitle} found and will be deleted!`);

      const deleteInput = $elem.find('tbody tr td.arc-job-delete input')[index];
      deleteInput.click();

      cy.get('#arc-delete-button').should('not.be.disabled');
      cy.get('#arc-delete-button').click();

      cy.get('#delete-item-yes')
        .should('be.not.disabled')
        .then(() => {
          functions.modalClick('#delete-item-yes');
        });
    }
  });
  cy.log(`${functionName} - finished`);
}

export function verifyEmailInMailHog(emailSubject: string, receivedData: any) {
  const data = receivedData[0];

  cy.mhGetAllMails()
    .mhFirst()
    .mhGetSubject()
    .should('eq', `[Cloudera Data Visualization] ${emailSubject}`);

  cy.mhGetAllMails()
    .mhFirst()
    .then((mail) => {
      const emailContent = mail.MIME.Parts[0].MIME.Parts[0].Body;
      const cleanContent = cleanEmailContent(emailContent);
      expect(cleanContent).to.have.string(data.creator_username);
      expect(cleanContent).to.have.string(data.app_id);
      expect(cleanContent).to.have.string(data.visual_name);
      expect(cleanContent).to.have.string(data.visual_url);
      expect(cleanContent).to.have.string(data.location_pathname);
      expect(cleanContent).to.have.string(data.start_time);
      expect(cleanContent).to.have.string(data.url_params);
      expect(cleanContent).to.have.string(data.run_time);
    });
}

function cleanEmailContent(emailContent: string): string {
  return emailContent
    .replace(/=\r\n/g, '') // Remove =\r\n
    .replace(/<br\/?>/g, '\n') // Replace <br> or <br/> tags with newline
    .replace(/&lt;/g, '<') // Replace &lt; with <
    .replace(/&gt;/g, '>') // Replace &gt; with >
    .replace(/&amp;/g, '&') // Replace &amp; with &
    .replace(/&#x27;/g, "'") // Replace &#x27; with '
    .replace(/&nbsp;/g, ' ') // Replace &nbsp; with space
    .replace(/=C2=A0/g, ' ') // Replace =C2=A0 with space
    .replace(/=3D/g, '='); // Remove 3D
}
