import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { ShelfTypes } from 'lib/dashboard/dashboards_interfaces';
import * as ddfunctions from 'lib/deriveddata';
import * as expVisual from 'lib/explore_visual/functions';
import { locators } from 'lib/explore_visual/locators';
import * as functions from 'lib/functions';
import * as visuals from 'lib/visual';

const timestamp = new Date().getTime();
const dbName = `explore_visual_${timestamp}`;
const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'Cereals';

// for one field in both dimenstion shelf and aggregate shelf
const visualsList1 = [
  'Horizontal bars',
  'Vertical bar',
  'Scatter',
  'Packed bubbles',
  'Line dimensions',
  'Line dimensions with trends',
  'Scatter w/trendline',
  'Area',
  'Pie',
  'Crosstab',
  'Striped table',
  'Sparkline',
  'Crosstab, horizontal',
  'Trellised bullet',
  'Trellised gauge',
];

// for two fields in dimenstion shelf and one field in aggregate shelf
const visualsList2 = [
  'Vertical bar',
  'Horizontal stacked bars',
  'Dimension scatter',
  'Scatter',
  'Grouped bars w/dimensions',
  'Line dimensions',
  'Line dimensions with trends',
  'Scatter w/trendline',
  'Area',
  'Chord',
  'Treemap',
  'Network',
  'Correlation heatmap',
  'Dendrogram',
  'Correlation flow',
  'Crosstab',
  'Striped table',
  'Trellis bar',
  'Trellised sparklines',
];

// for two fields in dimenstion shelf and two fields in aggregate shelf
const visualsList3 = [
  'Vertical bar',
  'Horizontal stacked bars',
  'Dimension scatter',
  'Scatter',
  'Grouped bars w/dimensions',
  'Line dimensions',
  'Line dimensions with trends',
  'Scatter w/trendline',
  'Area',
  'Chord',
  'Treemap',
  'Network',
  'Correlation heatmap',
  'Correlation flow',
  'Crosstab',
  'Striped table',
  'Trellis bar',
];

// for one field in dimension shelf
const visualsList4 = ['Headline table', 'Striped table'];

// for two fields in dimension shelf
const visualsList5 = ['Dimension scatter', 'Striped table'];

// for one field in aggregate shelf
const visualsList6 = ['KPI', 'Metrics', 'Gauge', 'Bullet', 'Headline table', 'Striped table'];

// for two fields in aggregate shelf
const visualsList7 = ['Metrics', 'Gauge', 'Bullet', 'Striped table', 'Multi-aggregate KPI'];

const dimensions = ['cereal_name', 'manufacturer_code'];
const aggregates = ['calories', 'sum(calories)', 'protein_grams', 'sum(protein_grams)'];

function clickExploreVisual() {
  cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('jsonselect');
  cy.get(locators.visual.explore_icon).click();
  cy.wait('@jsonselect');
}
describe('ARC-10927: Automate Explore Visuals option in Visual.', () => {
  before(() => {
    logins.silent();
    functions.addNewDB(dbName);
    functions.createNewVisual(connectionName, datasetName);
    dashboards.editVisual();
  });

  context('Explore Visual Tests', () => {
    it('1: Add one field in both dimension shelf and aggregate shelf', () => {
      // Add Fields in shelves
      ddfunctions.addField(ShelfTypes.DIMENSIONS, dimensions[1]);
      ddfunctions.addField(ShelfTypes.AGGREGATES, aggregates[0]);
      cy.get('#viz-refresh').click();
      dashboards.saveDashboard();

      // Verify all visual options available after adding fields in shelf
      clickExploreVisual();
      cy.get(locators.showmeModal).should('be.visible');
      // put length+1 due to ARC-10944
      cy.get(locators.visual.container + ' > div').should('have.length', visualsList1.length);
      // tslint:disable-next-line:prefer-for-of
      for (let index = 0; index < visualsList1.length; index++) {
        cy.get(locators.visual.title).should('contain', visualsList1[index]);
      }

      // Verify selected visual type and its name after selecting visual from explore visual option
      cy.get(locators.visual.visual_position.replace('index', '1'))
        .find('.showme-visual-title')
        .should('have.text', visualsList1[0]);
      cy.get(locators.visual.visual_position.replace('index', '1')).click();
      cy.get(locators.showmeModal).should('not.be.visible');
      visuals.saveVisual();
      dashboards.saveDashboard();
      cy.get(locators.visual.selected_viz).should('contain', 'Bars');
      expVisual.verifyVisualHash('6aa5ffa6ecccb5a6f67493b0a5d58dee');

      // Removing fields from shelf and select table visual
      ddfunctions.removeField(ShelfTypes.Y, dimensions[1]);
      ddfunctions.removeField(ShelfTypes.X, aggregates[0]);
      cy.get(locators.visual.table_visual).click();
      cy.get('#viz-refresh').click();
      visuals.saveVisual();
      dashboards.saveDashboard();
    });

    it.skip('2: Add two fields in dimension shelf and one field in aggregate shelf- open bug ARC-11622', () => {
      // Add Fields in shelves
      ddfunctions.addField(ShelfTypes.DIMENSIONS, dimensions[0]);
      ddfunctions.addField(ShelfTypes.DIMENSIONS, dimensions[1]);
      ddfunctions.addField(ShelfTypes.AGGREGATES, aggregates[0]);
      cy.get('#viz-refresh').click();
      visuals.saveVisual();
      dashboards.saveDashboard();

      // Verify all visual options available after adding fields in shelf
      clickExploreVisual();
      cy.get(locators.showmeModal).should('be.visible');
      // put length+1 due to ARC-10944
      cy.get(locators.visual.container + ' > div').should('have.length', visualsList2.length);
      // tslint:disable-next-line:prefer-for-of
      for (let index = 0; index < visualsList2.length; index++) {
        cy.get(locators.visual.title).should('contain', visualsList2[index]);
      }

      // Verify selected visual type and its name after selecting visual from explore visual option
      cy.get(locators.visual.visual_position.replace('index', '3'))
        .find('.showme-visual-title')
        .should('have.text', visualsList2[2]);
      cy.get(locators.visual.visual_position.replace('index', '3')).click();
      cy.get(locators.showmeModal).should('not.be.visible');
      dashboards.saveDashboard();
      cy.get(locators.visual.selected_viz).should('contain', 'Scatter');
      expVisual.verifyVisualHash('1f37e460c4f248e9c3c39d02510b3f36');

      // Removing fields from shelf and select table visual
      ddfunctions.removeField(ShelfTypes.X, dimensions[0]);
      ddfunctions.removeField(ShelfTypes.Y, dimensions[1]);
      ddfunctions.removeField(ShelfTypes.COLOR, aggregates[1]);
      cy.get(locators.visual.table_visual).click();
      cy.get('#viz-refresh').click();
      visuals.saveVisual();
      dashboards.saveDashboard();
    });

    it.skip('3: Add two fields in dimension shelf and two fields in aggregate shelf- open bug ARC-11622 ', () => {
      // Add Fields in shelves
      ddfunctions.addField(ShelfTypes.DIMENSIONS, dimensions[0]);
      ddfunctions.addField(ShelfTypes.DIMENSIONS, dimensions[1]);
      ddfunctions.addField(ShelfTypes.AGGREGATES, aggregates[0]);
      ddfunctions.addField(ShelfTypes.AGGREGATES, aggregates[2]);
      cy.get('#viz-refresh').click();
      visuals.saveVisual();
      dashboards.saveDashboard();

      // Verify all visual options available after adding fields in shelf
      clickExploreVisual();
      cy.get(locators.showmeModal).should('be.visible');
      // put length+1 due to ARC-10944
      cy.get(locators.visual.container + ' > div').should('have.length', visualsList3.length);
      // tslint:disable-next-line:prefer-for-of
      for (let index = 0; index < visualsList3.length; index++) {
        cy.get(locators.visual.title).should('contain', visualsList3[index]);
      }

      // Verify selected visual type and its name after selecting visual from explore visual option
      cy.get(locators.visual.visual_position.replace('index', '2'))
        .find('.showme-visual-title')
        .should('have.text', visualsList3[1]);
      cy.get(locators.visual.visual_position.replace('index', '2')).click();
      cy.get(locators.showmeModal).should('not.be.visible');
      visuals.saveVisual();
      dashboards.saveDashboard();
      cy.get(locators.visual.selected_viz).should('contain', 'Bars');
      expVisual.verifyVisualHash('7f33df26b53d6ccd9e7b605099e33bf5');

      // Removing fields from shelf and select table visual
      ddfunctions.removeField(ShelfTypes.COLOR, dimensions[0]);
      ddfunctions.removeField(ShelfTypes.Y, dimensions[1]);
      ddfunctions.removeField(ShelfTypes.X, aggregates[1]);
      ddfunctions.removeField(ShelfTypes.TOOLTIP, aggregates[3]);
      cy.get(locators.visual.table_visual).click();
      cy.get('#viz-refresh').click();
      visuals.saveVisual();
      dashboards.saveDashboard();
    });

    it('4: Add one field in dimension shelf', () => {
      // Add Fields in shelves
      ddfunctions.addField(ShelfTypes.DIMENSIONS, dimensions[0]);
      cy.get('#viz-refresh').click();
      dashboards.saveDashboard();

      // Verify all visual options available after adding fields in shelf
      clickExploreVisual();
      cy.get(locators.showmeModal).should('be.visible');
      cy.get(locators.visual.container + ' > div').should('have.length', visualsList4.length);
      // tslint:disable-next-line:prefer-for-of
      for (let index = 0; index < visualsList4.length; index++) {
        cy.get(locators.visual.title).should('contain', visualsList4[index]);
      }

      // Verify selected visual type and its name after selecting visual from explore visual option
      cy.get(locators.visual.visual_position.replace('index', '1'))
        .find('.showme-visual-title')
        .should('have.text', visualsList4[0]);
      cy.get(locators.visual.visual_position.replace('index', '1')).should('be.visible');
      cy.get(locators.visual.visual_position.replace('index', '1')).click();
      cy.get(locators.showmeModal).should('not.be.visible');
      dashboards.saveDashboard();
      cy.get(locators.visual.selected_viz).should('contain', 'Table');
      cy.get('div.dash-viz-wrapper table.table-headline td').should(
        'have.css',
        'text-align',
        'center'
      );

      // Removing fields from shelf and select table visual
      ddfunctions.removeField(ShelfTypes.DIMENSIONS, dimensions[0]);
      cy.get(locators.visual.table_visual).click();
      cy.get('#viz-refresh').click();
      dashboards.saveDashboard();
    });

    it('5: Add two fields in dimension shelf', () => {
      // Add Fields in shelves
      ddfunctions.addField(ShelfTypes.DIMENSIONS, dimensions[0]);
      ddfunctions.addField(ShelfTypes.DIMENSIONS, dimensions[1]);
      cy.get('#viz-refresh').click();
      dashboards.saveDashboard();

      // Verify all visual options available after adding fields in shelf
      clickExploreVisual();
      cy.get(locators.showmeModal).should('be.visible');
      cy.get(locators.visual.container + ' > div').should('have.length', visualsList5.length);
      // tslint:disable-next-line:prefer-for-of
      for (let index = 0; index < visualsList5.length; index++) {
        cy.get(locators.visual.title).should('contain', visualsList5[index]);
      }

      // Verify selected visual type and its name after selecting visual from explore visual option
      cy.get(locators.visual.visual_position.replace('index', '1'))
        .find('.showme-visual-title')
        .should('have.text', visualsList5[0]);
      cy.get(locators.visual.visual_position.replace('index', '1')).click();
      cy.get(locators.showmeModal).should('not.be.visible');
      dashboards.saveDashboard();
      cy.get(locators.visual.selected_viz).should('contain', 'Scatter');
      expVisual.verifyVisualHash('6acd7e7d523905501a9825a457435a6d');

      // Removing fields from shelf and select table visual
      ddfunctions.removeField(ShelfTypes.X, dimensions[0]);
      ddfunctions.removeField(ShelfTypes.Y, dimensions[1]);
      cy.get(locators.visual.table_visual).click();
      cy.get('#viz-refresh').click();
      dashboards.saveDashboard();
    });

    it('6: Add one field in aggregate shelf', () => {
      // Add Fields in shelves
      ddfunctions.addField(ShelfTypes.AGGREGATES, aggregates[0]);
      cy.get('#viz-refresh').click();
      dashboards.saveDashboard();

      // Verify all visual options available after adding fields in shelf
      clickExploreVisual();
      cy.get(locators.showmeModal).should('be.visible');
      cy.get(locators.visual.container + ' > div').should('have.length', visualsList6.length);
      // tslint:disable-next-line:prefer-for-of
      for (let index = 0; index < visualsList6.length; index++) {
        cy.get(locators.visual.title).should('contain', visualsList6[index]);
      }

      // Verify selected visual type and its name after selecting visual from explore visual option
      cy.get(locators.visual.visual_position.replace('index', '3'))
        .find('.showme-visual-title')
        .should('have.text', visualsList6[2]);
      cy.get(locators.visual.visual_position.replace('index', '3')).click();
      cy.get(locators.showmeModal).should('not.be.visible');
      dashboards.saveDashboard();
      cy.get(locators.visual.selected_viz).should('contain', 'Gauge');
      cy.get(locators.visual.visual_container).should('have.class', 'viz-gauge-container');

      // Removing fields from shelf and select table visual
      ddfunctions.removeField(ShelfTypes.AGGREGATES, aggregates[1]);
      cy.get(locators.visual.table_visual).click();
      cy.get('#viz-refresh').click();
      dashboards.saveDashboard();
    });

    it('7: Add two fields in aggregate shelf', () => {
      // Add Fields in shelves
      ddfunctions.addField(ShelfTypes.AGGREGATES, aggregates[0]);
      ddfunctions.addField(ShelfTypes.AGGREGATES, aggregates[2]);
      cy.get('#viz-refresh').click();
      dashboards.saveDashboard();

      // Verify all visual options available after adding fields in shelf
      clickExploreVisual();
      cy.get(locators.showmeModal).should('be.visible');
      cy.get(locators.visual.container + ' > div').should('have.length', visualsList7.length);
      // tslint:disable-next-line:prefer-for-of
      for (let index = 0; index < visualsList7.length; index++) {
        cy.get(locators.visual.title).should('contain', visualsList7[index]);
      }

      // Verify selected visual type and its name after selecting visual from explore visual option
      cy.get(locators.visual.visual_position.replace('index', '3'))
        .find('.showme-visual-title')
        .should('have.text', visualsList7[2]);
      cy.get(locators.visual.visual_position.replace('index', '3')).click();
      cy.get(locators.showmeModal).should('not.be.visible');
      dashboards.saveDashboard();
      cy.get(locators.visual.selected_viz).should('contain', 'Bullet');
      cy.get(locators.visual.visual_container).should('have.class', 'viz-bullet-container');

      // Removing fields from shelf and select table visual
      ddfunctions.removeField(ShelfTypes.AGGREGATES, aggregates[1]);
      ddfunctions.removeField(ShelfTypes.COMPARE, aggregates[3]);
      cy.get(locators.visual.table_visual).click();
      cy.get('#viz-refresh').click();
      dashboards.saveDashboard();
    });
  });
});
