import { logins } from 'lib/arctest';
import * as arctest from 'lib/arctest/screen/builder';
import { addShelfColumn } from 'lib/arctest/screen/data/modal';
import * as connections from 'lib/connections';
import * as context_menu from 'lib/context_menus/locators';
import * as dashboard from 'lib/dashboard';
import { Dashboard, ShelfTypes } from 'lib/dashboard/dashboards_interfaces';
import * as filter from 'lib/filter';
import * as functions from 'lib/functions';
import * as dataset from 'lib/permissions';
import { editSpecificVisual } from 'lib/visual';
import { CSSselector } from 'locators/common/CSSselector';

function get_time() {
  const a = new Date();
  return a.getTime();
}

// fetch url from GET URL popup in dashboard edit mode and visit the extracted URL
function getURL() {
  let url: string = '';

  functions.waitForVisualsLoaded();

  // get filtered url from GET URL
  cy.get(CSSselector.dashboard.more_options).click();
  cy.get(dashboard.locators.get_url).click({ force: true });
  cy.get(dashboard.locators.dash_url_modal).should('be.visible');

  cy.get('#link-modal-link').should('contain.value', 'http');

  cy.get('#link-modal-link')
    .invoke('val')
    .then(($url) => {
      cy.log('test selected AppId : ' + $url);
      url = String($url).split(`${Cypress.config().baseUrl}`)[1];

      functions.modalClick(`${dashboard.locators.dash_url_modal_buttons}:contains("Close")`);
      cy.visit(url);

      functions.waitForVisualsLoaded();
    });
}

function goToHomePage() {
  cy.get('.navbar-brand').click({ force: true });
  cy.wait(1500);
}

describe('Filter Bugs Verification.', () => {
  const connectionName = Cypress.env('smokeConnectionName');
  const datasetName = 'Cereals';
  const filterName = 'cereal_name';
  const filterValue1 = '100%_Bran';
  const filterValue2 = 'All-Bran';
  const dashName = 'filter_db_' + get_time();

  before(() => {
    logins.silent();
  });

  context(
    'ARC-8768: Dashboard name entered by user disappears after adding values in the filter on a dashboard',
    () => {
      it('1 : Create Dashboard and add dashboard filter', () => {
        functions.addNewDB(dashName);
        functions.createNewVisual(connectionName, datasetName);
        arctest.addFilter([filterName]);
        cy.get(CSSselector.Dashboard_title).should('have.text', dashName);
        cy.get(CSSselector.Dashboard_subtitle).should('have.text', dashName + ' subtitle');
        dashboard.saveDashboard();
        goToHomePage();
      });
    }
  );

  context(
    "ARC-9515: Filter exclude doesn't work in Dashboard view mode but works in Edit mode.",
    () => {
      it('1 : Create Dashboard, add dashbaord filter, apply filter value and verify it in edit mode.', () => {
        // Create Dashboard and Add filter
        functions.addNewDB(dashName);
        functions.createNewVisual(connectionName, datasetName);
        functions.createNewVisual(connectionName, datasetName);
        dashboard.saveDashboard();
        arctest.addFilter([filterName]);
        cy.get('a[data-target="#dash-filters-tab"]').click();

        // Verify filter before and after excluding filter value
        filter.applyDropdownFilterToVisual([filterValue1]);
        cy.get('.dataTables_scrollBody tbody tr').contains(filterValue1);
        filter.verifyCurrentParameters([filterName]);
        filter.verifyCurrentParameters([filterValue1]);
        filter.excludeFilterCheck();
        cy.get(dashboard.locators.dashboard_title).click();
        cy.get('.dataTables_scrollBody tbody tr').should('not.contain', filterValue1);
        filter.verifyCurrentParameters([filterName]);
        filter.verifyCurrentParameters(['not in']);

        // remove the applied filter and verify it
        filter.excludeFilterUncheck();
        filter.removeFilterValues(1, [filterValue1]);
        cy.get(CSSselector.filters.filter_dropdown_specific(1)).click({ force: true });
        cy.get(CSSselector.filters.exclude_filter).should('not.be.checked');
        cy.get(CSSselector.filters.filter_dropdown_specific(1)).click({ force: true });
        const selector1 =
          '.main-app .gridster > ul li[data-arc-reportid]:nth-child(1) tr:nth-child(1) > td:nth-child(1)';
        const selector2 =
          '.main-app .gridster > ul li[data-arc-reportid]:nth-child(1) tr:nth-child(3) > td:nth-child(1)';
        cy.get(selector1).should('have.text', filterValue1);
        cy.get(selector2).should('have.text', filterValue2);

        // Apply filter again after remove applied filter and verify it
        filter.applyDropdownFilterToVisual([filterValue2]);
        cy.get('.dataTables_scrollBody tbody tr').contains(filterValue2);
        cy.get(dashboard.locators.dashboard_title).click();
        filter.verifyCurrentParameters([filterValue2]);
        filter.verifyCurrentParameters(['in']);

        // Remove applied filter and make it as default
        filter.removeFilterValues(1, [filterValue2]);
        dashboard.saveDashboard();
        cy.get(selector1).should('have.text', filterValue1);
        cy.get(selector2).should('have.text', filterValue2);
      });

      it('2 : Filter value, exclude it and verify it in view mode.', () => {
        dashboard.viewDashboard();

        // Verify applied filter
        filter.applyDropdownFilterToVisual([filterValue1]);
        cy.get('.dataTables_scrollBody tbody tr').contains(filterValue1);
        filter.verifyCurrentParameters([filterName]);
        filter.verifyCurrentParameters([filterValue1]);

        // Exclude applied filter and verify it
        filter.excludeFilterCheck();
        cy.get('.dataTables_scrollBody tbody tr').should('not.contain', filterValue1);
        filter.verifyCurrentParameters([filterName]);
        filter.verifyCurrentParameters(['not in']);

        // remove applied filter and verify it
        filter.removeFilterValues(1, [filterValue1]);
        const selector1 =
          '.main-app .gridster > ul li[data-arc-reportid]:nth-child(1) tr:nth-child(1) > td:nth-child(1)';
        const selector2 =
          '.main-app .gridster > ul li[data-arc-reportid]:nth-child(1) tr:nth-child(3) > td:nth-child(1)';
        cy.get(selector1).should('have.text', filterValue1);
        cy.get(selector2).should('have.text', filterValue2);

        // Apply filter with exclude checked and verify it
        filter.applyDropdownFilterToVisual([filterValue2]);
        cy.get(selector1).should('not.have.text', filterValue2);
        filter.verifyCurrentParameters([filterValue2]);
        filter.verifyCurrentParameters(['not in']);

        // Remove filter and verify it
        filter.removeFilterValues(1, [filterValue2]);
        cy.get(selector1).should('have.text', filterValue1);
        cy.get(selector2).should('have.text', filterValue2);
      });
    }
  );

  // Skipping the test due to open bug ARC-11870
  context.skip('ARC-10008: Filter dropdown menu appears broken with MySQL connection.', () => {
    const mysql = {
      connection_type: 'MySQL',
      connection_name: 'mysql',
      hostname: '10.10.15.34',
      port: '3306',
      database: 'employees',
      username: 'root',
      password: 'p@ssw0rd',
    };
    const sqlTableName = 'america';
    const sqlDatasetName = 'America';
    const sqlFilterName = 'fname';
    const sqlFilterValue = 'Aaron';
    it('1 : Create Dashboard, add dashbaord filter, apply filter value and verify it in edit mode using MySQL connection.', () => {
      cy.fixture('configs/connections/connections').then((connectionData) => {
        // connections.createConnection(connectionData.mysql);
        connections.createConnection(mysql);
      });
      functions.createDatasetFromConnection(
        mysql.connection_name,
        mysql.database,
        sqlTableName,
        sqlDatasetName
      );
      functions.addNewDB(dashName);
      functions.createNewVisual(mysql.connection_name, sqlDatasetName);
      dashboard.saveDashboard();
      arctest.addFilter([sqlFilterName]);
      cy.get('a[data-target="#dash-filters-tab"]').click();

      // Verify filter before and after excluding filter value
      filter.applyFilterToTelcoDashboard([sqlFilterValue]);
      cy.get('.dataTables_scrollBody tbody tr').contains(sqlFilterValue);
      filter.verifyCurrentParameters([sqlFilterName]);
      filter.verifyCurrentParameters([sqlFilterValue]);
      dashboard.saveDashboard();
    });
    it('2 : Verify the same in view mode .', () => {
      dashboard.viewDashboard();
      cy.get('.dataTables_scrollBody tbody tr').contains(sqlFilterValue);
      filter.verifyCurrentParameters([sqlFilterName]);
      filter.verifyCurrentParameters([sqlFilterValue]);
    });
  });

  context('ARC-8772: Dashboard back button seems broken', () => {
    const filterList = ['cold_or_hot', 'manufacturer_code'];
    const filterValues = ['H', 'Q'];
    const visualIndxes = [
      arctest.locators.visuals + ':nth-child(1)',
      arctest.locators.visuals + ':nth-child(2)',
    ];
    it('1 : Create Dashboard, add dashbaord filters and Verify filter back button in view mode', () => {
      cy.intercept('GET', '/arc/searches/search').as('searchLoaded');
      functions.addNewDB(dashName);
      functions.createNewVisual(connectionName, datasetName);
      functions.createNewVisual(connectionName, datasetName);
      arctest.addFilter(filterList[0]);
      cy.get('a[data-target="#dash-filters-tab"]').click();
      arctest.addFilter(filterList[1]);
      // wait to setup filters
      functions.waitForVisualsLoaded();

      dashboard.saveDashboard();
      dashboard.viewDashboard();
      filter.applyDropdownFilterToVisual([filterValues[0]]);
      filter.applyDropdownFilterToVisual([filterValues[1]], null, 2);
      for (let index = 0; index < filterList.length; index++) {
        filter.verifyCurrentParameters([filterList[index]]);
        filter.verifyCurrentParameters([filterValues[index]]);
      }
      // tslint:disable-next-line:prefer-for-of
      for (let index = 0; index < visualIndxes.length; index++) {
        arctest.verifyTableContents(visualIndxes[index], [
          [
            'Quaker_Oatmeal',
            'Q',
            'H',
            '100',
            '5',
            '2',
            '0',
            '',
            '-1',
            '-1',
            '1',
            '110',
            '0',
            '1',
            '0.67',
            'Q',
            'Quaker Oats',
          ],
        ]);
      }
      filter.filterControlBack();
      // tslint:disable-next-line:prefer-for-of
      for (let index = 0; index < visualIndxes.length; index++) {
        arctest.verifyTableContents(visualIndxes[index], [
          [
            'Cream_of_Wheat_(Quick)',
            'N',
            'H',
            '100',
            '3',
            '0',
            '80',
            '1',
            '21',
            '0',
            '2',
            '-1',
            '0',
            '1',
            '1',
            'N',
            'Nabisco',
          ],
          [
            'Maypo',
            'A',
            'H',
            '100',
            '4',
            '1',
            '0',
            '0',
            '16',
            '3',
            '2',
            '95',
            '25',
            '1',
            '-1',
            'A',
            'American Home Food Products',
          ],
          [
            'Quaker_Oatmeal',
            'Q',
            'H',
            '100',
            '5',
            '2',
            '0',
            '',
            '-1',
            '-1',
            '1',
            '110',
            '0',
            '1',
            '0.67',
            'Q',
            'Quaker Oats',
          ],
        ]);

        cy.get(CSSselector.filters.filter_selected_value_specific(2)).should('have.text', '(All)');
      }
    });
    it('2 : Verify filter forward button in view mode with filter value', () => {
      filter.filterControlForward();
      // tslint:disable-next-line:prefer-for-of
      for (let index = 0; index < visualIndxes.length; index++) {
        arctest.verifyTableContents(visualIndxes[index], [
          [
            'Quaker_Oatmeal',
            'Q',
            'H',
            '100',
            '5',
            '2',
            '0',
            '',
            '-1',
            '-1',
            '1',
            '110',
            '0',
            '1',
            '0.67',
            'Q',
            'Quaker Oats',
          ],
        ]);
      }
      cy.get(CSSselector.filters.filter_selected_names_specific(2)).should('have.text', ' Q ');
      goToHomePage();
    });
  });

  context(
    'ARC-8732: Unselecting filter options in multi-sheet dashboard do not retain when switching sheets',
    () => {
      it('1 : Create Multisheet Dashboard, add dashbaord filter and Verify filter value back button in view mode', () => {
        cy.intercept('GET', '/arc/searches/search').as('searchLoaded');
        functions.addNewDB(dashName);
        functions.createNewVisual(connectionName, datasetName);
        arctest.addFilter([filterName]);
        filter.applyDropdownFilterToVisual([filterValue1]);
        filter.applyDropdownFilterToVisual([filterValue2]);

        cy.get(CSSselector.filters.configure_filter_specific(1)).click({ force: true });
        cy.get(CSSselector.filters.settings_navtab)
          .contains('Values')
          .click();
        cy.get(CSSselector.filters.values.remember_in_viewmode_checkbox).uncheck();
        functions.modalClick(CSSselector.filters.save_settings);

        dashboard.saveDashboard();
        dashboard.addSheet();
        cy.get(dashboard.locators.visuals_side_tab).click();
        // wait to open visuals tab
        cy.wait(1000);
        functions.createNewVisual(connectionName, datasetName);
        dashboard.saveDashboard();
        dashboard.viewDashboard();
        cy.get('.sheet-title')
          .contains('Sheet 1')
          .click();
        filter.removeFilterValues(1, [filterValue1, filterValue2]);

        cy.get('.sheet-title')
          .contains('Sheet 2')
          .click();
        functions.waitForVisualsLoaded();

        cy.get('.sheet-title')
          .contains('Sheet 1')
          .click();
        functions.waitForVisualsLoaded();

        cy.get(CSSselector.filters.filter_selected_value_specific(1)).should('have.text', '(All)');
      });
    }
  );

  context(
    'ARC-6273: Filter widget defaults do not override defaults in filter shelf of visual',
    () => {
      const dash: Dashboard = {
        title: 'filter_widget_db_' + get_time(),
        subTitle: 'simple_scatter_visual_dashboard_subtitle',
        sheets: [
          {
            sheet_name: 'sheet1',
            sheet_number: 1,
            visuals: [
              {
                name: 'tableVisual',
                connection_name: connectionName,
                dataset_name: 'US State Populations Over Time',
                visual_type: '.chart-icon-table',
                visual_title: 'simple_table_visual1',
                linked: 'false',
                data: {
                  fields: [
                    {
                      field_name: 'state',
                      field_shelf: ShelfTypes.FILTERS,
                      field_expression: "[state] <<state.exclude:in>> (<<state.data:'AZ'>>)",
                    },
                  ],
                },
              },
            ],
          },
        ],
      };

      const filterValue = ['CA'];
      const fName = ['state'];
      const visualIndex = arctest.locators.visuals + ':nth-child(1)';
      const populationDataDefault = [
        ['1960', 'AZ', '1,302,161'],
        ['1940', 'AZ', '499,261'],
        ['1970', 'AZ', '1,775,399'],
        ['2000', 'AZ', '5,130,632'],
        ['1950', 'AZ', '749,587'],
        ['1900', 'AZ', '122,931'],
        ['1920', 'AZ', '334,162'],
        ['1800', 'AZ', '0'],
        ['1790', 'AZ', '0'],
        ['1910', 'AZ', '204,354'],
        ['2010', 'AZ', '6,392,017'],
        ['1980', 'AZ', '2,716,546'],
        ['1990', 'AZ', '3,665,228'],
        ['1930', 'AZ', '435,573'],
      ];
      const populationDataAfterFilterChange = [
        ['1960', 'CA', '15,717,204'],
        ['1940', 'CA', '6,907,387'],
        ['1970', 'CA', '19,971,069'],
        ['2000', 'CA', '33,871,648'],
        ['1950', 'CA', '10,586,223'],
        ['1900', 'CA', '1,485,053'],
        ['1920', 'CA', '3,426,861'],
        ['1800', 'CA', '0'],
        ['1790', 'CA', '0'],
        ['1910', 'CA', '2,377,549'],
        ['2010', 'CA', '37,253,956'],
        ['1980', 'CA', '23,667,764'],
        ['1990', 'CA', '29,760,021'],
        ['1930', 'CA', '5,677,251'],
      ];

      before(() => {
        dashboard.createDashboard(dash);
        arctest.addFilter(fName);
        dashboard.saveDashboard();
        arctest.verifyTableContents(visualIndex, populationDataDefault);
      });

      it('1 : Create Dashboard, add dashbaord filter and visual with custom filter expression in filter shelf', () => {
        filter.applyDropdownFilterToVisual(filterValue);
        dashboard.saveDashboard();
        filter.verifyCurrentParameters(filterValue);
        filter.verifyCurrentParameters(fName);
        arctest.verifyTableContents(visualIndex, populationDataAfterFilterChange);
      });

      it('2 : Changing value in filter widget should not modify the default filter on the filter shelf', () => {
        filter.applyDropdownFilterToVisual(filterValue);
        dashboard.saveDashboard();
        filter.removeFilterValues(1, filterValue);
        arctest.verifyTableContents(visualIndex, populationDataDefault);
      });
    }
  );

  context(
    'ARC-10972: Navigate to dashboard parameters does not render to the new dashboard',
    () => {
      const targetVisual = arctest.locators.visuals + ':nth-child(1)';
      const url = `${Cypress.config().baseUrl}` + '/arc/apps/app/';
      let targetDashId: number;
      let datasetID: number;
      let vizID: number;
      const targetDashboard = 'arc-10972-target_db' + get_time();
      const filterValues = ['100%_Bran', '100%_Natural_Bran'];
      const sourceDashboard: Dashboard = {
        title: 'arc-10972-source_db' + get_time(),
        subTitle: 'arc-10972-source_db_subtitle' + get_time(),
        sheets: [
          {
            sheet_name: 'sheet1',
            sheet_number: 1,
            visuals: [
              {
                name: 'source visual',
                connection_name: connectionName,
                dataset_name: datasetName,
                visual_type: '.chart-icon-table',
                visual_title: 'source_visual',
                linked: 'false',
                data: {
                  fields: [
                    {
                      field_name: 'cereal_name',
                      field_shelf: ShelfTypes.DIMENSIONS,
                    },
                    {
                      field_name: 'manufacturer_code',
                      field_shelf: ShelfTypes.DIMENSIONS,
                    },
                    {
                      field_name: 'calories',
                      field_shelf: ShelfTypes.AGGREGATES,
                    },
                    {
                      field_name: 'protein_grams',
                      field_shelf: ShelfTypes.AGGREGATES,
                    },
                    {
                      field_name: 'fat_grams',
                      field_shelf: ShelfTypes.AGGREGATES,
                    },
                  ],
                },
              },
            ],
          },
        ],
      };
      it('1 : Create target and source dashboards, add dashboard filter and configure navigate to dashboard feature', () => {
        // get dataset id
        goToHomePage();
        functions.navigate_to_data();
        cy.get(CSSselector.data.connection_list).each(($el) => {
          if ($el.text() === connectionName) {
            cy.wrap($el).click();
          }
        });
        cy.intercept('GET', '/arc/roles/roles').as('rolesLoaded');
        dataset.selectDataset(connectionName, datasetName);
        cy.get('#dataset-id')
          .then(($id) => {
            datasetID = Number($id.text().trim());
          })
          .then(() => {
            cy.log('datasetId------------>', datasetID);

            // Create target dashboard and add filter widget
            functions.addNewDB(targetDashboard);
            functions.createNewVisual(connectionName, datasetName);
            functions.addFilter(filterName, connectionName, datasetName);
            dashboard.saveDashboard();

            // Create source dashboard, add filter widget and apply filter
            dashboard.createDashboard(sourceDashboard);
            dashboard.editVisual();
            // get visual id
            cy.get(CSSselector.visualEdit.ellipsis).click();
            cy.get('div.visual-id span')
              .then(($id) => {
                vizID = Number($id.text().trim());
              })
              .then(() => {
                cy.log('visualId------------>', vizID);
                dashboard.closeVisual(false);
                functions.addFilter(filterName, connectionName, datasetName);
                filter.applyDropdownFilterToVisual(filterValues);
                dashboard.saveDashboard();
                cy.get(
                  context_menu.locators.table_data
                    .replace('visualIndex', '1')
                    .replace('row', '1')
                    .replace('column', '1')
                ).as('clickedRow');
                cy.get('@clickedRow').click({ force: true });

                // open context menu and click on navigate to dashboard
                cy.get(context_menu.locators.context_menu)
                  .find('.cm-navigateclick')
                  .as('navigateTo');
                cy.get('@navigateTo').click({ force: true });

                // Configure navigate to dashboard by searching target db, get its id, click on it
                cy.get(context_menu.locators.pick_dashboard).should('be.visible');
                cy.get(context_menu.locators.pick_dashboard)
                  .find('input.pass-params')
                  .check();
                cy.get(context_menu.locators.pick_dashboard)
                  .find('input#searchbox')
                  .type('"' + targetDashboard + '"');
                cy.get(context_menu.locators.select_dashboard)
                  .contains(targetDashboard)
                  .parent()
                  .as('targetDB');

                // Get id of target dashboard
                cy.get('@targetDB')
                  .invoke('attr', 'arc-report-id')
                  .then(($id) => {
                    targetDashId = Number($id);
                  })
                  .then(() => {
                    cy.get('@targetDB').click();
                    dashboard.saveDashboard();
                    cy.get('@clickedRow').click({ force: true });
                    cy.get('@navigateTo').should('have.class', 'selected');
                    cy.get(context_menu.locators.nav_link)
                      .should('be.visible')
                      .should('have.text', 'Navigate to ID: ' + targetDashId);
                    dashboard.viewDashboard();

                    // Click on first cell of table to check the navigation
                    cy.get('@clickedRow').click({ force: true });
                    functions.waitForVisualsLoaded();

                    // Verify the URL after navigation
                    const url1 =
                      url +
                      targetDashId +
                      `?param.cereal_name=100%25_Bran&param.cereal_name.include_nulls=false&scope=[{%22scopeType%22:%22global%22,%22isAppScope%22:false,%22filterName%22:%22cereal_name%22,%22filterId%22:${vizID}}]`;
                    cy.url().should('eq', url1);
                  });
              });
          });
      });

      it('2 : Verify target dashboard', () => {
        filter.verifyCurrentParameters([filterValues[0], filterName]);
        filter.verifyFilterContent(arctest.locators.filter_widgets + ':nth-child(1)', [
          filterValues[0],
        ]);
        arctest.verifyTableContents(targetVisual, [
          [
            '100%_Bran',
            'N',
            'C',
            '70',
            '4',
            '1',
            '130',
            '10',
            '5',
            '6',
            '3',
            '280',
            '25',
            '1',
            '0.33',
            'N',
            'Nabisco',
          ],
        ]);
      });
    }
  );

  context(
    'ARC-8397: Parameters substituting in a visual is happening even when explicit scope is set for a visual',
    () => {
      const dbName = 'explicit_scope_filter_check_' + get_time();
      const visual = arctest.locators.visuals + ':nth-child(index)';
      it('1 : Create dashboard with explicit scope filter', () => {
        goToHomePage();
        functions.addNewDB(dbName);
        for (let index = 0; index < 2; index++) {
          functions.createNewVisual(connectionName, datasetName);
          editSpecificVisual(index + 1);
          addShelfColumn(ShelfTypes.DIMENSIONS, 'cereal_name');
          addShelfColumn(ShelfTypes.AGGREGATES, 'fat_grams');
          cy.get(dashboard.locators.limit)
            .clear()
            .type('5');
          cy.get('#viz-refresh').click();
          functions.waitForVisualsLoaded();
          dashboard.closeVisual(false);
          dashboard.saveDashboard();
        }

        // workaround for stuck filter popup
        cy.reload();
        functions.waitForVisualsLoaded();

        arctest.addFilter([filterName]);
        filter.configureNewFilterScope('Sheet', 'Explicit');
        filter.applyDropdownFilterToVisual([filterValue1, filterValue2]);
        filter.verifyCurrentParameters([filterName, filterValue1, filterValue2]);
        dashboard.saveDashboard();
        for (let index = 1; index < 3; index++) {
          arctest.verifyTableContents(visual.replace('index', '' + index + ''), [
            ['100%_Bran', '1'],
            ['100%_Natural_Bran', '5'],
            ['All-Bran', '1'],
            ['All-Bran_with_Extra_Fiber', '0'],
            ['Almond_Delight', '2'],
          ]);
        }
      });

      it('2 : Go to view dashboard and verify the explicit scope filter', () => {
        let alreadyAddedFilterCount = '';
        const filterValues = ['Almond_Delight', 'All-Bran_with_Extra_Fiber'];
        cy.get(CSSselector.filters.filter_selected_count_specific(1)).then(($counter) => {
          alreadyAddedFilterCount = $counter.text();
          dashboard.viewDashboard();
          filter.applyDropdownFilterToVisual(
            filterValues,
            null,
            1,
            Number(alreadyAddedFilterCount)
          );
        });

        filter.verifyCurrentParameters([filterName, filterValue1, filterValue2]);
        filter.verifyCurrentParameters(filterValues);
        for (let index = 1; index < 3; index++) {
          arctest.verifyTableContents(visual.replace('index', '' + index + ''), [
            ['100%_Bran', '1'],
            ['100%_Natural_Bran', '5'],
            ['All-Bran', '1'],
            ['All-Bran_with_Extra_Fiber', '0'],
            ['Almond_Delight', '2'],
          ]);
        }
      });
    }
  );

  context(
    "ARC-8796: Visiting URL obtained after Filter All doesn't take user to same state as before",
    () => {
      const dbName = 'filter_all_check_' + get_time();
      const visual = arctest.locators.visuals + ':nth-child(index)';
      it('1 : Create dashboard and click on Filter All from context menu', () => {
        goToHomePage();
        functions.addNewDB(dbName);
        for (let index = 0; index < 3; index++) {
          functions.createNewVisual(connectionName, datasetName);
        }
        cy.get('#main').scrollTo('top');
        // Remove this line when ARC-11945 get resolved.
        arctest.addFilter([filterName]);
        dashboard.saveDashboard();
        // click on filter all context menu
        cy.get(
          context_menu.locators.table_data
            .replace('visualIndex', '1')
            .replace('row', '1')
            .replace('column', '1')
        ).click({ force: true });
        cy.get(context_menu.locators.context_menu)
          .find('div.cm-filterall')
          .click({
            force: true,
          });
        dashboard.saveDashboard();
      });

      it('2 : Get url from GET URL and visit the url', () => {
        // Close filters tab
        cy.get(CSSselector.filters_tab).click();
        // get filtered url from GET URL
        getURL();
        filter.verifyCurrentParameters(['cereal_name', '100%_Bran']);
        for (let index = 2; index < 4; index++) {
          arctest.verifyTableContents(visual.replace('index', String(index)), [
            [
              '100%_Bran',
              'N',
              'C',
              '70',
              '4',
              '1',
              '130',
              '10',
              '5',
              '6',
              '3',
              '280',
              '25',
              '1',
              '0.33',
              'N',
              'Nabisco',
            ],
          ]);
        }
      });

      // Skipping this test due to open bug ARC-11945
      it.skip('3 : Verify Source visual after visiting filer url', () => {
        cy.get(visual.replace('index', '1'))
          .find('.dataTables_scrollBody tbody tr')
          .should('have.length.greaterThan', 1);
        cy.get(visual.replace('index', '1'))
          .find('.paging_full_numbers')
          .should('be.visible');
      });
    }
  );

  context(
    'ARC-9329: Regression - Explicit filters are handled incorrectly in email schedules (snapshot pdf/png)',
    () => {
      const dbName = 'explicit_filter_' + get_time();
      const visual = arctest.locators.visuals + ':nth-child(1)';
      const visualType = '.chart-icon-trellis-bars';
      const hash = '228a69c342f079d2506538d2ab162dde';
      const stack = [
        'setTransform',
        'translate',
        'globalAlpha',
        'font',
        'setTransform',
        'translate',
        'clearRect',
        'fillStyle',
        'beginPath',
        'rect',
        'fill',
        'fillStyle',
        'beginPath',
        'rect',
        'fill',
      ];
      const dsName = 'Restaurant Inspection SF';
      const filters = ['business_id', 'inspection_date'];
      it('1 : Create dashboard with two visuals and add two dashboard filters', () => {
        // Create dashboard and visuals
        goToHomePage();
        functions.addNewDB(dbName);
        for (let index = 0; index < 2; index++) {
          functions.createNewVisual(connectionName, dsName);
        }

        // Add fields in visual
        for (let index = 1; index < 3; index++) {
          editSpecificVisual(index);
          if (index === 2) {
            addShelfColumn(ShelfTypes.DIMENSIONS, 'business_name');
            addShelfColumn(ShelfTypes.AGGREGATES, 'Record Count');
            cy.get('#chart-type-menu ' + visualType).click();
          } else {
            arctest.modal.addShelfColumn(ShelfTypes.DIMENSIONS, 'business_id');
            arctest.modal.addShelfColumn(ShelfTypes.DIMENSIONS, 'business_name');
            arctest.modal.addShelfColumn(ShelfTypes.DIMENSIONS, 'business_city');
          }
          cy.get('#viz-refresh').click();
          functions.waitForVisualsLoaded();
          dashboard.closeVisual(false);
        }

        // Add dashboard filter and apply values to filter
        arctest.addFilter(filters);
        filter.applyDropdownFilterToVisual(['31', '903']);
        dashboard.saveDashboard();
      });

      it('2 : Apply fixed date filter and verify visuals', () => {
        // Apply date range to date filter
        cy.get(filter.locators.fixed_date_range).click();
        cy.get(filter.locators.calender).should('be.visible');
        cy.get(filter.locators.start_date_input)
          .clear()
          .type('2016-10-09');
        cy.get(filter.locators.apply_date).click();
        functions.waitForVisualsLoaded();
        dashboard.saveDashboard();

        // Verify applied filter values, filter params and visuals
        filter.verifyCurrentParameters([
          'business_id',
          '31',
          '903',
          'inspection_date.start',
          'inspection_date.end',
          '2016-10-09',
          '2017-01-09',
        ]);
        cy.get('div.control-container li:nth-child(1) span.selected-names').should(
          'have.text',
          ' 31, 903 '
        );
        cy.get(filter.locators.fixed_date_range)
          .invoke('val')
          .should('be.eq', '2016-10-09 - 2017-01-09');
        arctest.verifyTableContents(visual, [
          ['903', 'Sabra Restaurant', 'San Francisco'],
          ['31', "Norman's Ice Cream and Freezes", 'San Francisco'],
        ]);
      });

      it('3 : Verify filters and visual after visiting filtered URL', () => {
        cy.get(CSSselector.filters_tab).click();
        cy.get('div#dash-filters-tab').should('not.be.visible');
        getURL();
        filter.verifyCurrentParameters([
          'business_id',
          '31',
          '903',
          'inspection_date.start',
          'inspection_date.end',
          '2016-10-09',
          '2017-01-09',
        ]);
        cy.get('div.control-container li:nth-child(1) span.selected-names').should(
          'have.text',
          ' 31, 903 '
        );
        cy.get(filter.locators.fixed_date_range)
          .invoke('val')
          .should('be.eq', '2016-10-09 - 2017-01-09');
        arctest.verifyTableContents(visual, [
          ['903', 'Sabra Restaurant', 'San Francisco'],
          ['31', "Norman's Ice Cream and Freezes", 'San Francisco'],
        ]);
        let dashboard1 = '';
        let dashboard2 = '';
        cy.get('#main').then(($dashboard1) => {
          dashboard1 = String($dashboard1);
          getURL();

          cy.get('#main').then(($dashboard2) => {
            dashboard2 = String($dashboard2);
            expect(dashboard1).to.eq(dashboard2);
          });
        });
      });
    }
  );
});
