import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';
import { dragAndDrop } from 'lib/util/drag_and_drop';
import { CSSselector } from 'locators/common/CSSselector';

const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'Cereals';

const dashboard: Dashboard = {
  title: 'simple_bar_visual_dashboard',
  subTitle: 'simple_bar_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'calorie sheet',
      sheet_number: 1,
      visuals: [
        {
          name: 'tablevisual',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-table',
          visual_title: 'simple_table_visual2',
          linked: 'false',
        },
      ],
    },
  ],
};

describe('Drag and drop filter item suite:', () => {
  before(() => {
    logins.silent();
    cy.visit('/arc/apps/home');
  });

  it('Test dragging and dropping filter items to the dashboard', () => {
    dashboards.createDashboard(dashboard);

    // click the filter tab
    cy.get(CSSselector.side_panel)
      .find(CSSselector.filters_tab)
      .click();

    // move an item into the dashboard's filter
    dragAndDrop(
      `${CSSselector.filters_tab_selection_item}[data-test-item='cereal_name']`,
      CSSselector.Dashboard_filters
    );
    // check the changes and the dropdown content
    cy.get(`${CSSselector.Dashboard_filters} :nth-child(1)`).should('contain', 'cereal_name');
    cy.get(CSSselector.filters.filter_dropdown).click();
    cy.get(CSSselector.filters.filter_picklist).should('have.length', 77);
    cy.get(CSSselector.filters.filter_dropdown).click();

    // move an item into the dashboard's filter and check the changes
    dragAndDrop(
      `${CSSselector.filters_tab_selection_item}[data-test-item='calories']`,
      CSSselector.Dashboard_filters
    );
    cy.get(`${CSSselector.Dashboard_filters} :nth-child(2)`).should('contain', 'calories');

    // save changes
    cy.get(CSSselector.Save).click();

    // refresh the page and check the changes again
    cy.reload(true);
    cy.get(`${CSSselector.Dashboard_filters} :nth-child(1)`).should('contain', 'cereal_name');
    cy.get(`${CSSselector.Dashboard_filters} :nth-child(2)`).should('contain', 'calories');
  });
});
