import { logins } from 'lib/arctest';
import * as arctest from 'lib/arctest/screen/builder';
import { BackendAPI } from 'lib/backend-api';
import * as dashboards from 'lib/dashboard';
import * as filter from 'lib/filter';
import { locators } from 'lib/filter/locators';
import { CSSselector } from 'locators/common/CSSselector';

const filterList1 = ['100%_Bran', 'Basic_4', 'Maypo'];
const filterList2 = ['A', 'G', 'K'];
const pillTooltip1 = encodeURIComponent(
  "(([cereal_name] in ('100%_Bran', 'Basic_4', 'Maypo')) or ([cereal_name] is NULL))"
);
const pillTooltip2 = encodeURIComponent(
  "(([manufacturer_code] in ('A', 'G', 'K')) or ([manufacturer_code] is NULL))"
);
const targetVisual = arctest.locators.visuals + ':nth-child(1)';

describe('Filter Breadcrumbs/Pills-open.', () => {
  let visualBuilderUrl: string;

  before(() => {
    logins.silent();
    BackendAPI.dashboards
      .import({
        fileName: 'cypress_filter_pills_prettyfied.json',
        dataconnectionName: 'samples',
        workspaceName: 'Public',
      })
      .then(({ dashboard }) => {
        visualBuilderUrl = `/arc/apps/builder/${dashboard.id}`;
        cy.visit(visualBuilderUrl);
      });
  });

  context('ARC-11344: Automate Filters - breadcrumbs (filter pills) - Smoke', () => {
    it('1: Apply filter at dashboard level with filter breadcrumb/pills settings', () => {
      cy.wait(3000);
      arctest.addFilter(['cereal_name', 'manufacturer_code']);
      dashboards.saveDashboard();
      filter.applyDropdownFilterToVisual(filterList1, visualBuilderUrl, 1);
      dashboards.saveDashboard();
      filter.applyDropdownFilterToVisual(filterList2, visualBuilderUrl, 2);
      dashboards.saveDashboard();

      // Apply filter pills settings
      cy.get(`${CSSselector.settings_tab}`).click();
      cy.get('#dash-settings-tab div[data-tabname="Filter Settings"] h5').click();
      cy.get(locators.display_filters).check();
      cy.get(`${CSSselector.settings_tab}`).click();
      dashboards.saveDashboard();
    });

    it('2: Verify visual with filter breadcrumb/pills settings applied in dashboard edit mode as well as view mode', () => {
      for (let i = 0; i < 2; i++) {
        if (i === 1) {
          dashboards.viewDashboard();
        }
        filter.verifyCurrentParameters(['cereal_name']);
        filter.verifyCurrentParameters(['manufacturer_code']);
        // tslint:disable-next-line:prefer-for-of
        for (let index = 0; index < filterList1.length; index++) {
          filter.verifyCurrentParameters([filterList1[index]]);
          filter.verifyCurrentParameters([filterList2[index]]);
        }
        cy.get(locators.filter_pills.replace('vizIndex', '1').replace('pilIndex', '2'))
          .should('be.visible')
          .should('have.text', 'cereal_name')
          .invoke('attr', 'data-tooltip-contents')
          .then(($val) => {
            expect($val).to.eq(pillTooltip1);
          });
        cy.get(locators.filter_pills.replace('vizIndex', '1').replace('pilIndex', '3'))
          .should('be.visible')
          .should('have.text', 'manufacturer_code')
          .invoke('attr', 'data-tooltip-contents')
          .then(($val) => {
            expect($val).to.eq(pillTooltip2);
          });
        arctest.verifyTableContents(targetVisual, [
          ['Maypo', 'A', 'H'],
          ['Basic_4', 'G', 'C'],
        ]);
      }
    });

    it('3: Verify visual without filter breadcrumb/pills settings applied in dashboard edit mode as well as view mode', () => {
      dashboards.editDashboard();
      // Remove filter pills settings
      cy.get(`${CSSselector.settings_tab}`).click();
      cy.get('#dash-settings-tab div[data-tabname="Filter Settings"] h5').click();
      cy.get(locators.display_filters).uncheck();
      dashboards.saveDashboard();
      for (let i = 0; i < 2; i++) {
        if (i === 1) {
          dashboards.viewDashboard();
        }
        filter.verifyCurrentParameters(['cereal_name']);
        filter.verifyCurrentParameters(['manufacturer_code']);
        // tslint:disable-next-line:prefer-for-of
        for (let index = 0; index < filterList1.length; index++) {
          filter.verifyCurrentParameters([filterList1[index]]);
          filter.verifyCurrentParameters([filterList2[index]]);
        }
        cy.get(locators.filter_pills.replace('vizIndex', '1').replace('pilIndex', '2')).should(
          'not.exist'
        );
        arctest.verifyTableContents(targetVisual, [
          ['Maypo', 'A', 'H'],
          ['Basic_4', 'G', 'C'],
        ]);
      }
    });
  });
});
