import { addShelfColumn } from 'lib/arctest/screen/builder/modal';
import { BackendAPI } from 'lib/backend-api';
import { createDashboardFilter, refreshVisual } from 'lib/dashboard';
import {
  FilterScopeTypes,
  FilterTypes,
  FilterVisibilityTypes,
  ShelfTypes,
} from 'lib/dashboard/dashboards_interfaces';
import { modalClick } from 'lib/functions';

import * as logins from 'lib/logins';
import { saveVisual } from 'lib/visual';

describe('Special characters in filters tests', () => {
  before(() => {
    logins.silent();
    BackendAPI.dashboards
      .import({
        fileName: 'special-filters_prettyfied.json',
        dataconnectionName: 'samples',
        workspaceName: 'Public',
      })
      .then(({ dashboard }) => cy.visit(`/arc/apps/builder/${dashboard.id}`));
  });

  it('Test 1: Create dashboard filter with spaces in the column name', () => {
    createDashboardFilter({
      shelfType: 'dim',
      fieldName: 'state name',
      filterOptions: {
        title: 'state name',
        visibility: FilterVisibilityTypes.SHEET,
        scope: FilterScopeTypes.CURRENT,
        values: ['AK'],
        allowTimeRange: false,
      },
    });
  });

  it('Test 2: Create visual filter with spaces in the column name', () => {
    cy.get(
      `.main-app.arc-app-container.edit-mode [data-test-id="visual-list-item"]:nth-child(1)`
    ).click();
    cy.get(`#visual-data-tab > a`).click();
    cy.get(`#${ShelfTypes.FILTERS}`).click();
    addShelfColumn(ShelfTypes.FILTERS, 'state name');
    cy.get(`#${ShelfTypes.FILTERS}:nth-child(1)`)
      .first()
      .click();
    // This part is coming from visual_functions.ts file (we should refactor here and there)
    cy.get(
      '[data-test-id="properties-panel"] [data-test-id="properties-menu-select-values"]'
    ).click({ force: true });

    cy.get('[data-test-id="filter-modal-apply-btn"]:visible', { timeout: 30000 });
    cy.get('#filter-modal .modal-body-loading:hidden', { timeout: 30000 });

    cy.get(`[data-test-id="filter-input-checkbox-AK"]`).click();

    modalClick('[data-test-id="filter-modal-apply-btn"]');

    cy.get('[data-test-id="filter-modal-apply-btn"]').should('not.exist');
    refreshVisual();
    saveVisual();
  });

  it('Test 3: Create dashboard filter with spaces + special characters in the column name', () => {
    createDashboardFilter({
      shelfType: 'agg',
      fieldName: '# population',
      filterType: FilterTypes.RANGE,
      filterOptions: {
        title: '# population',
        visibility: FilterVisibilityTypes.SHEET,
        scope: FilterScopeTypes.CURRENT,
        values: [0],
        allowTimeRange: false,
      },
    });
  });

  it('Test 4: Create visual filter with spaces + special characters in the column name', () => {
    cy.get(
      `.main-app.arc-app-container.edit-mode [data-test-id="visual-list-item"]:nth-child(1)`
    ).click();
    cy.get(`#visual-data-tab > a`).click();
    cy.get(`#${ShelfTypes.FILTERS}`)
      .should('be.visible')
      .click();
    addShelfColumn(ShelfTypes.FILTERS, '# population');
    cy.get(`#${ShelfTypes.FILTERS} > :nth-child(2)`).click();
    // This part is coming from visual_functions.ts file (we should refactor here and there)
    cy.get(
      '[data-test-id="properties-panel"] [data-test-id="properties-menu-select-values"]'
    ).click({ force: true });

    cy.get('[data-test-id="filter-modal-apply-btn"]:visible', { timeout: 30000 });
    cy.get('#filter-modal .modal-body-loading:hidden', { timeout: 30000 });

    cy.get(`[data-test-id="filter-input-checkbox-1250"]`).click();

    modalClick('[data-test-id="filter-modal-apply-btn"]');

    cy.get('[data-test-id="filter-modal-apply-btn"]').should('not.exist');
    refreshVisual();
    saveVisual();
  });
});
