import * as apps_locators from 'lib/apps/apps_locators';
import * as arctest from 'lib/arctest';
import * as dashboard from 'lib/dashboard';
import * as functions from 'lib/functions';
import { locators } from 'lib/homepage/locators';
import { CSSselector } from 'locators/common/CSSselector';

/**
 * get the sum of elements of an array
 */
function getSum(total: number, num: number) {
  return total + num;
}

describe('Homepage.', () => {
  const learnLinks = ['Get Started', "What's New in", 'Documentation'];
  let dashCount1: string;
  let dashCount2: string;
  let appCount1: string;
  let appCount2: string;
  let datasetCount1: number;
  const datasetCount2: number[] = [];

  function getContentsNonEmpty(locator: string) {
    cy.get(locator).contains(/[^\s]/);
    return cy.get(locator).then(($e) => {
      return $e.text().trim();
    });
  }

  function getContainedInt(locator: string) {
    return getContentsNonEmpty(locator).then((text) => {
      return parseInt(text, 10);
    });
  }

  before(() => {
    arctest.logins.silent();
    functions.navigate_to_home();

    getContentsNonEmpty(locators.dashboards_number).then((text) => {
      dashCount1 = text;
      cy.log('Dash count = ' + dashCount1);
    });

    getContentsNonEmpty(locators.apps_number).then((text) => {
      appCount1 = text;
    });

    getContainedInt(locators.datasets_number).then((someNumber) => {
      datasetCount1 = someNumber;
      cy.log('Dataset count = ' + datasetCount1);
    });
  });

  context('ARC-11298: Automate smoke test for Homepage', () => {
    it('1: Verify Homepage elements in left panel', () => {
      // dashboards count verification
      cy.get(locators.dashboards).click();
      cy.get(CSSselector.Visuals_all)
        .click()
        .should('have.attr', 'data-test-active', 'true');
      cy.get(locators.list_view).click();

      // wait for the table to get populated
      const tableRows = 'table:visible tr[data-test-report-id]';
      cy.get(tableRows).should('exist');
      cy.get(tableRows).should('not.be.empty');

      getContentsNonEmpty(locators.tableinfo_visual).then((text) => {
        cy.log('dashCount1-->', dashCount1);
        dashCount2 = text;
        cy.log('dashCount2-->', dashCount2);
        expect(dashCount1).to.eq(dashCount2);
      });

      // apps count verification
      functions.navigate_to_home();
      cy.get(locators.apps).click();
      cy.wait(2000);
      cy.get(CSSselector.Visuals_all).click();
      cy.get(CSSselector.Visuals_all).should('have.attr', 'data-test-active', 'true');

      cy.get(locators.tableinfo_visual).scrollIntoView();

      getContentsNonEmpty(locators.tableinfo_visual).then((text) => {
        appCount2 = text;
        console.log(appCount2);
        expect(appCount1).to.eq(appCount2);
      });

      // datasets count verification
      functions.navigate_to_home();
      cy.get(locators.datasets).should('be.visible');
      functions.navigate_to_data();
      cy.get(locators.connection_list)
        .each(($el, index) => {
          cy.wrap($el).click();
          cy.get(locators.connection_count).then(($text4) => {
            datasetCount2.push(Number($text4.text()));
            console.log('datasetCount--->before', datasetCount2);
          });
        })
        .then(() => {
          console.log('datasetCount--->after', datasetCount2);
          const count = datasetCount2.reduce(getSum);
          console.log(count);
          // since "All Connections" is in the connections list
          // the calculated dataset count is the double of the actual
          expect(count / 2).to.eq(datasetCount1);
        });

      // total views
      functions.navigate_to_home();
      cy.get(locators.total_views).should('be.visible');
    });

    it('2: Verify Homepage elements in right panel', () => {
      functions.navigate_to_home();
      cy.get(CSSselector.home_new_dashboard).click();
      cy.get(dashboard.locators.dash_panel).should('be.visible');
      cy.get(CSSselector.Save).should('be.visible');
      functions.navigate_to_home();
      cy.get('button:contains("New App")').click();
      cy.get(locators.app_panel).should('be.visible');
      cy.get(apps_locators.locators.appsave).should('be.visible');
      functions.navigate_to_home();
      cy.get(locators.dashboard_created_text).should('contain', 'Dashboards were created');
      cy.get(locators.apps_created_text).should('contain', 'Apps were created');
      cy.get(locators.datasets_created_text).should('contain', 'Datasets were created');
      cy.get(locators.learn).each(($el, index) => {
        cy.wrap($el.text().replace('\n', '')).should('contain', learnLinks[index]);
        cy.get(locators.last_login).should('contain', 'Your last login was on:');
      });
    });
  });
});
