import * as arctest from 'lib/arctest';
import * as dashboard from 'lib/dashboard';
import * as filters from 'lib/filter';
import * as functions from 'lib/functions';
import * as imp_exp from 'lib/import_export';
import * as visuals from 'lib/visual';
import { CSSselector } from 'locators/common/CSSselector';

/**
 * This function will verify filter content.
 * @param locator : Css locator of filter
 * @param data : filter data to verify
 */
function verifyFilterContent(locator: string, data: any[] = []) {
  const selectedNamesLabel = (() => {
    if (!data.length) {
      return 'All';
    } else if (data.length < 3) {
      return data.join(', ');
    } else {
      return '';
    }
  })();

  expect(
    cy
      .get(locator)
      .find('.selected-names')
      .then(($selectedElem) => $selectedElem.text().trim()),
    selectedNamesLabel
  );

  // if (data.length > 2) {
  //     expect(cy.get(locator).find('.selected-count-wrapper')).to.be.visible;
  // } else {
  //     expect(cy.get(locator).find('.selected-count-wrapper')).to.be.not.visible;
  // }

  cy.get(locator)
    .find('.picklist-list li input:checked')
    .should('have.length', data.length);
  let value;
  cy.get(locator)
    .find('.picklist-list li input:checked')
    .each(($val, i, $list) => {
      value = $val.data('value');
      expect(value).to.be.equal(data[i]);
    });
}

// const DashboardName = 'Dashboard to export visual';
const dataconnection = 'samples';
const dashboardName = 'Cereal Comparisons';
const jsonFiles = [
  'DB_with_multiple_sheets.json',
  'DB_explicitscope_filter_grid_area.json',
  'DB_navigate_to_sample_dashboards.json',
  'DB_datasetscope_filter_grid_area.json',
];
const multiSheetDashName = 'DB_with_multiple_sheets';
const explicitDashboard = 'DB_explicitscope_filter_grid_area';
const navigateDashboard = 'DB_navigate_to_sample_dashboards';
const datasetScopeDashboard = 'DB_datasetscope_filter_grid_area';
const visualName = ['linked_VS', 'VS2'];
const filterName = ['filter_1', 'filter_2'];
describe('import and export dashboard test: ARC-8019 and ARC-8015', () => {
  before(() => {
    arctest.logins.silent();
  });

  it('Test 1: Import a dashboard ARC-8019', () => {
    imp_exp.import_artifacts(dataconnection, jsonFiles[0]);
    imp_exp.import_artifacts(dataconnection, jsonFiles[1]);
    imp_exp.import_artifacts(dataconnection, jsonFiles[2]);
    imp_exp.import_artifacts(dataconnection, jsonFiles[3]);
    functions.navigate_to_visuals();
    cy.get(`${CSSselector.Visuals_all}`).click({ force: true });
    // const dashboardName = simpleDashFile.replace('.json', '');
    cy.contains(multiSheetDashName);

    // verify sheets name in the dashboard
    dashboard.openDashboardforEdit(multiSheetDashName);
    dashboard.verifyDashboardSheetName(1, 'Sheet 1');
    dashboard.verifyDashboardSheetName(2, 'Sheet 2');

    // verify linked visual spread accross multiple sheets and app scope of dashboard filter
    for (let i = 1; i < 3; i++) {
      cy.get(`${CSSselector.renameSheet.sheet}:nth-child(${i})`).click({ force: true });
      cy.get(CSSselector.first_visual)
        .find('span.linked-visual-icon')
        .should('be.visible');
      cy.get(CSSselector.visual_edit_first).click({ force: true });
      cy.get(imp_exp.locators.linked_visual)
        .find('span.linked-visual-icon')
        .should('be.visible')
        .parent()
        .should('have.text', ' Linked');
    }
    filters.verifyFilterScope(true);

    // verify visuals and filters positions in the dashboard
    let index = 0;
    for (let i = 1; i < 3; i++) {
      cy.get(`${CSSselector.renameSheet.sheet}:nth-child(1)`).click();
      cy.get(imp_exp.locators.specific_visual.replace('index', '' + i + ''))
        .find(CSSselector.visual_title_Check)
        .should('contain', visualName[index]);
      const selector =
        arctest.screen.builder.locators.filter_widgets + ':nth-child(' + (i + 1) + ')';
      arctest.screen.builder.verifyTitleFilter(selector, filterName[index]);
      index++;
    }
    // to make it run in headless mode.
    dashboard.saveDashboard();

    // verify explicit scope filter of visual
    dashboard.openDashboardforEdit(explicitDashboard);
    for (let i = 1; i < 3; i++) {
      const tableSelector = arctest.screen.builder.locators.visuals + ':nth-child(' + i + ')';
      switch (i) {
        case 1:
          cy.get(CSSselector.visual_edit_specific(1)).click({
            force: true,
          });
          cy.get(CSSselector.visual_settings_tab)
            .click()
            .parent()
            .should('have.class', 'active');
          cy.get(CSSselector.visual_settings_parameter)
            .click()
            .should('have.class', 'open');
          cy.get(CSSselector.visual_settings_parameter_explicit_scope).should('be.checked');
          verifyFilterContent(arctest.screen.builder.locators.filter_widgets, [45, 70]);
          const selector1 =
            arctest.screen.builder.locators.filter_widgets + ':nth-child(' + i + ')';
          arctest.screen.builder.verifyTitleFilter(selector1, 'sodium_mg');
          arctest.screen.builder.verifyTableContents(tableSelector, [
            ['100%_Natural_Bran', '120', '15'],
            ['Golden_Crisp', '100', '45'],
            ['Great_Grains_Pecan', '120', '75'],
            ['Smacks', '110', '70'],
            ['Strawberry_Fruit_Wheats', '90', '15'],
          ]);
          break;
        case 2:
          verifyFilterContent(arctest.screen.builder.locators.filter_widgets, [45, 70]);
          arctest.screen.builder.verifyGridFilterContent(
            imp_exp.locators.specific_visual.replace('index', '3'),
            ['P']
          );
          arctest.screen.builder.verifyTableContents(tableSelector, [
            [
              'Golden_Crisp',
              'P',
              'C',
              '100',
              '2',
              '0',
              '45',
              '0',
              '11',
              '15',
              '1',
              '40',
              '25',
              '1',
              '0.88',
              'P',
              'Post',
            ],
          ]);
          break;
      }
    }

    // verify Navigate to Dashboard functionality
    let id;
    dashboard.openDashboardforEdit(navigateDashboard);
    cy.get(imp_exp.locators.navigate_dashboard).then(($el) => {
      id = $el
        .text()
        .split(':')[1]
        .trim();
      cy.visit('/arc/apps/app/' + id);
      cy.get(CSSselector.apps.dashboardTitle).should('have.text', 'Cereal Comparisons');
    });

    // verify dataset scope filter of dashboard
    cy.visit('/');
    dashboard.openDashboardforEdit(datasetScopeDashboard);
    filters.verifyFilterScope(false, true);
    verifyFilterContent(arctest.screen.builder.locators.filter_widgets, [45, 70, 75, 80]);
    arctest.screen.builder.verifyGridFilterContent(
      imp_exp.locators.specific_visual.replace('index', '3'),
      [100]
    );
    for (let i = 1; i < 3; i++) {
      const tableSelector1 = arctest.screen.builder.locators.visuals + ':nth-child(' + i + ')';
      switch (i) {
        case 1:
          arctest.screen.builder.verifyTableContents(tableSelector1, [
            ['Cream_of_Wheat_(Quick)', '100', '80'],
            ['Golden_Crisp', '100', '45'],
          ]);
          break;
        case 2:
          arctest.screen.builder.verifyTableContents(tableSelector1, [
            [
              'Cream_of_Wheat_(Quick)',
              'N',
              'H',
              '100',
              '3',
              '0',
              '80',
              '1',
              '21',
              '0',
              '2',
              '-1',
              '0',
              '1',
              '1',
              'N',
              'Nabisco',
            ],
            [
              'Golden_Crisp',
              'P',
              'C',
              '100',
              '2',
              '0',
              '45',
              '0',
              '11',
              '15',
              '1',
              '40',
              '25',
              '1',
              '0.88',
              'P',
              'Post',
            ],
          ]);
          break;
      }
    }
  });

  it('Test 2: Export simple dashboard ARC-8015', () => {
    imp_exp.export_dashboard(dashboardName);
  });
});
