import { jobs } from 'fixtures/locators/jobs';
import { goto_job_status } from 'lib/functions';
import { silent } from 'lib/logins';
import { clickOnFirstJobName, configureEmailSettings, createEmailJob, JobType } from './utils';

describe('Job Page - Job details', () => {
  before(() => {
    silent();
  });

  context('Check Adhoc Email Job on Job Detail Page', () => {
    before(() => {
      configureEmailSettings();
      createEmailJob(JobType.Adhoc);
      goto_job_status();
      clickOnFirstJobName('runsTable');
    });

    it('Should display information about the Adhoc Email Job', () => {
      cy.get(jobs.job_name).should(
        'have.text',
        'Adhoc Email: This is an email job created by Cypress'
      );

      cy.get(jobs.job_type).should('have.text', 'Email');

      cy.get(jobs.job_interval).should('have.text', 'Adhoc');

      cy.get(jobs.job_owner).should('have.text', 'vizapps_admin');

      cy.get(jobs.details_toggle).click();

      cy.get(jobs.email_subject).should('contain', 'This is an email job created by Cypress');

      cy.get(jobs.email_message).should('contain', 'This is an email job created by Cypress');

      cy.get(jobs.email_template).should('contain.text', 'Default Email Template');

      cy.get(jobs.email_attachments).should('contain.text', 'Embedded image, CSV');

      cy.fixture('configs/email/emails.json').then((email) => {
        cy.get(`[data-test-id="to-${email.email_settings.gmail.address}"]`).should(
          'contain',
          email.email_settings.gmail.address
        );

        cy.get(`[data-test-id="cc-${email.email_settings.yahoo.username}"]`).should(
          'contain',
          email.email_settings.yahoo.username
        );
      });

      cy.get(`${jobs.previous_runs_table} tbody tr`)
        .first()
        .should('contain.text', 'Running')
        .get(jobs.run_details)
        .click();
      cy.contains('Run Details');

      cy.get(jobs.close_run_details).click();
    });
  });
});
