import { jobs } from 'fixtures/locators/jobs';
import { BackendAPI, DefaultRoleIds } from 'lib/backend-api';
import { goto_job_status } from 'lib/functions';
import { silent } from 'lib/logins';
import {
  checkFirstJobOwner,
  checkSuccessfulNotification,
  clickOnFirstJobName,
  configureEmailSettings,
  createEmailJob,
  getRandomInt,
  JobType,
} from './utils';

const adminUser = 'vizapps_admin';
const adminUserPass = 'vizapps_admin';
const nonAdminUser = 'new_user_' + getRandomInt(0, 10000);
const nonAdminUserPass = nonAdminUser;

describe('Job Details Page - Change Job Owner Modal', () => {
  before(() => {
    BackendAPI.users.create({
      username: nonAdminUser,
      password: nonAdminUserPass,
      roleIds: [DefaultRoleIds.Analyst],
    });
    silent(adminUser, adminUserPass);
    configureEmailSettings();
  });

  context('Check Change Job Owner Modal on Job Detail Page', () => {
    before(() => {
      createEmailJob(JobType.Adhoc);

      goto_job_status();
      clickOnFirstJobName('runsTable');
    });

    beforeEach(() => {
      cy.get(jobs.job_actions).click();
      cy.get(jobs.job_actions_change_owner).click();
      cy.get(jobs.change_job_owner_modal).should('be.visible');
    });

    it('Changes current owner and modal reflects new current owner', () => {
      cy.get(jobs.change_job_owner_current_owner).then(($input) => {
        expect($input.val()).to.equal(adminUser);
      });

      cy.get(jobs.change_job_owner_select).type('new_user');

      cy.get(jobs.change_job_owner_options)
        .contains(nonAdminUser)
        .click();

      cy.wait(1000);
      cy.get(jobs.change_job_owner_save).click();

      checkSuccessfulNotification('Successfully changed job owner');

      cy.get(jobs.job_owner).should('have.text', nonAdminUser);
    });

    it('It saves the modal with the same owner and checks if the correct notification message is displayed.', () => {
      cy.get(jobs.change_job_owner_current_owner).then(($input) => {
        expect($input.val()).to.equal(nonAdminUser);
      });

      cy.get(jobs.change_job_owner_select).type('new_user');
      cy.get(jobs.change_job_owner_options)
        .contains(nonAdminUser)
        .click();

      cy.wait(1000);
      cy.get(jobs.change_job_owner_save).click();

      checkSuccessfulNotification('No owner change made');
      cy.get(jobs.job_owner).should('have.text', nonAdminUser);
    });

    it('Changes current owner and the job runs as new owner. Jobs page and single job page both reflect change both immediately and upon refresh', () => {
      cy.get(jobs.change_job_owner_current_owner).then(($input) => {
        expect($input.val()).to.equal(nonAdminUser);
      });

      cy.get(jobs.change_job_owner_select).type('viza');

      cy.get(jobs.change_job_owner_options)
        .contains(adminUser)
        .click();

      cy.wait(1000);
      cy.get(jobs.change_job_owner_save).click();
      cy.get(jobs.job_owner).should('have.text', adminUser);

      cy.get(jobs.go_back_to_jobs_button).click();

      checkFirstJobOwner('runsTable', adminUser);
      clickOnFirstJobName('runsTable');
    });
  });
});
