import { jobs } from 'fixtures/locators/jobs';
import { goToJobSchedulePage } from 'lib/functions';
import { silent } from 'lib/logins';
import { clickOnFirstJobName, configureEmailSettings, createEmailJob, JobType } from './utils';

describe('Job Page - Job details', () => {
  before(() => {
    silent();
  });

  context('Check Threshold Email Job on Job Detail Page', () => {
    before(() => {
      configureEmailSettings();
      createEmailJob(JobType.Threshold);
      goToJobSchedulePage();
      clickOnFirstJobName('scheduledJobsTable');
    });

    it('Should display information about the Threshold Email Job', () => {
      cy.get(jobs.job_name).should('have.text', 'This is an email job created by Cypress');

      cy.get(jobs.job_interval).should('have.text', 'Scheduled');

      cy.get(jobs.job_type).should('have.text', 'Email');

      cy.get(jobs.job_threshold).should('have.text', 'Yes');

      cy.get(jobs.job_owner).should('have.text', 'vizapps_admin');

      cy.get(jobs.details_toggle).click();

      cy.get(jobs.email_subject).should('contain', 'This is an email job created by Cypress');

      cy.get(jobs.email_message).should('contain', 'This is an email job created by Cypress');

      cy.get(jobs.email_template).should('contain.text', 'Default Email Template');

      cy.fixture('configs/email/emails.json').then((email) => {
        cy.get(`[data-test-id="to-${email.email_settings.gmail.address}"]`).should(
          'contain',
          email.email_settings.gmail.address
        );

        cy.get(`[data-test-id="cc-${email.email_settings.yahoo.username}"]`).should(
          'contain',
          email.email_settings.yahoo.username
        );
      });
    });
  });
});
