/*
 * To run this test, set the ENABLE_LARRY_TESTS environment variable in Cypress.
 * One way to do that is to start cypress using:
 *   CYPRESS_ENABLE_LARRY_TESTS=True npm run cy-open
 */
import * as arctest from 'lib/arctest';
describe('Basic Demo', () => {
  function getRandomInt(max: number) {
    return Math.floor(Math.random() * Math.floor(max));
  }

  it('TV Data Demo', () => {
    if (Cypress.env('ENABLE_LARRY_TESTS') !== 'True') {
      return;
    }

    arctest.logins.larry('9999');
    arctest.screen.data.connectionNavigate('Arcadia Enterprise');

    const dsName = 'auto-cypress-' + getRandomInt(1000000000);
    arctest.screen.data.createDatasetFromPredefinedTable('tvdata', 'tv_viewing_data', dsName);

    arctest.screen.data.newDashboard(dsName);

    cy.get(arctest.screen.builder.locators.tablevisual).dblclick();

    cy.get(arctest.screen.builder.modal.locators.table_table);
    cy.wait(2000);

    // Create calendar heatmap
    arctest.screen.builder.modal.addShelfColumn('dimensions_shelf', ['datestring', 'Datestring']);
    arctest.screen.builder.modal.addShelfColumn('aggregates_shelf', 'Record Count');
    arctest.screen.builder.modal.refreshVisual();

    cy.get('.chart-icon-calendar-heatmap').click();

    arctest.screen.builder.modal.saveVisual();

    // Create stacked bar
    cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('jsonselect');
    cy.get('.btn-new-visual').click();
    cy.wait('@jsonselect');
    cy.get('.gridster .arc-viz-type-table .dataTables_scroll').dblclick();

    cy.get('.chart-icon-trellis-bars').click();

    arctest.screen.builder.modal.addShelfColumn('x_shelf', 'Record Count');
    arctest.screen.builder.modal.topk('x_shelf', 50);

    arctest.screen.builder.modal.addShelfColumn('y_shelf', ['channel', 'Channel']);
    arctest.screen.builder.modal.addShelfColumn('color_shelf', ['program', 'Program']);
    arctest.screen.builder.modal.refreshVisual();
    arctest.screen.builder.modal.saveVisual();

    cy.get('a[href="#dash-filters-tab"], a[data-target="#dash-filters-tab"]').click();

    cy.get(
      ".dash-settings-content .selection_field[title='channel'], .dash-settings-content .selection_field[title='Channel']"
    ).click();

    cy.get('.control-container .selected-control-title', {
      timeout: 20000,
    });
  });
});
