import { logins } from 'lib/arctest';
import { CSSselector } from 'locators';
import { configureEmailSettings, createEmailJob, JobType } from 'tests/jobs/utils';

const MAIN_MENUS = {
  HOME: CSSselector.homepage,
  SQL: CSSselector.sql_tab,
  VISUALS: CSSselector.Visual_page,
  DATA: CSSselector.data_menu,
  SETTINGS: CSSselector.site_admin_menus.site_admin_dropdown_toggle,
};

const SUB_MENUS = {
  SET_HOME_PAGE: CSSselector.site_admin_menus.set_home_page,
  JOB_STATUS: CSSselector.site_admin_menus.job_status,
  ACTIVITY_LOG: CSSselector.site_admin_menus.activity_log,
  USERS_AND_GROUPS: CSSselector.site_admin_menus.manage_user_and_group_site_admin,
  MANAGE_ROLES: CSSselector.site_admin_menus.manage_roles_site_admin,
  URL_ALIASES: CSSselector.site_admin_menus.manage_url_aliases,
  MANAGE_API_KEYS: CSSselector.site_admin_menus.manage_api_keys,
  EMAIL_TEMPLATES: CSSselector.site_admin_menus.email_templates_site_admin,
  CUSTOM_STYLES: CSSselector.site_admin_menus.manage_custom_styles,
  CUSTOM_COLORS: CSSselector.site_admin_menus.manage_custom_colors,
  CUSTOM_DATES: CSSselector.site_admin_menus.manage_custom_dates,
  STATIC_ASSETS: CSSselector.site_admin_menus.static_assets,
  SITE_SETTINGS: CSSselector.site_admin_menus.site_settings_site_admin,
};

const ACTIVE_MENUS_BY_URL = {
  '/arc/jobs/customemails': {
    mainMenu: MAIN_MENUS.SETTINGS,
    subMenu: SUB_MENUS.EMAIL_TEMPLATES,
  },
  '/arc/apps/app': {
    mainMenu: MAIN_MENUS.VISUALS,
    subMenu: null,
  },
  '/arc/apps/builder': {
    mainMenu: MAIN_MENUS.VISUALS,
    subMenu: null,
  },
  '/arc/apps/appedit': {
    mainMenu: MAIN_MENUS.VISUALS,
    subMenu: null,
  },
  '/arc/apps/customcolors': {
    mainMenu: MAIN_MENUS.SETTINGS,
    subMenu: SUB_MENUS.CUSTOM_COLORS,
  },
  '/arc/apps/customdates': {
    mainMenu: MAIN_MENUS.SETTINGS,
    subMenu: SUB_MENUS.CUSTOM_DATES,
  },
  '/arc/apps/customstyles': {
    mainMenu: MAIN_MENUS.SETTINGS,
    subMenu: SUB_MENUS.CUSTOM_STYLES,
  },
  '/arc/apps/datasql': {
    mainMenu: MAIN_MENUS.SQL,
    subMenu: null,
  },
  '/arc/apps/staticassets': {
    mainMenu: MAIN_MENUS.SETTINGS,
    subMenu: SUB_MENUS.STATIC_ASSETS,
  },
  '/arc/apps/activity': {
    mainMenu: MAIN_MENUS.SETTINGS,
    subMenu: SUB_MENUS.ACTIVITY_LOG,
  },
  '/arc/apps/aliases': {
    mainMenu: MAIN_MENUS.SETTINGS,
    subMenu: SUB_MENUS.URL_ALIASES,
  },
  '/arc/apps/apikeys': {
    mainMenu: MAIN_MENUS.SETTINGS,
    subMenu: SUB_MENUS.MANAGE_API_KEYS,
  },
  '/arc/apps/data': {
    mainMenu: MAIN_MENUS.DATA,
    subMenu: null,
  },
  '/arc/apps/dataset': {
    mainMenu: MAIN_MENUS.DATA,
    subMenu: null,
  },
  '/arc/apps': {
    mainMenu: MAIN_MENUS.HOME,
    subMenu: null,
  },
  '/arc/apps/users': {
    mainMenu: MAIN_MENUS.SETTINGS,
    subMenu: SUB_MENUS.USERS_AND_GROUPS,
  },
  '/arc/jobs/adminlogs': {
    mainMenu: MAIN_MENUS.SETTINGS,
    subMenu: SUB_MENUS.JOB_STATUS,
  },
  '/arc/apps/roles': {
    mainMenu: MAIN_MENUS.SETTINGS,
    subMenu: SUB_MENUS.MANAGE_ROLES,
  },
  '/arc/apps/homepage': {
    mainMenu: MAIN_MENUS.SETTINGS,
    subMenu: SUB_MENUS.SET_HOME_PAGE,
  },
  '/arc/apps/settings': {
    mainMenu: MAIN_MENUS.SETTINGS,
    subMenu: SUB_MENUS.SITE_SETTINGS,
  },
  '/arc/apps/apps': {
    mainMenu: MAIN_MENUS.VISUALS,
    subMenu: null,
  },
  '/arc/apps/apidemo': {
    mainMenu: MAIN_MENUS.SETTINGS,
    subMenu: null,
  },
  '/arc/apps/restart': {
    mainMenu: MAIN_MENUS.SETTINGS,
    subMenu: null,
  },
};

const ACTIVE_COLOR = 'rgb(46, 137, 197)';

describe('Each page should set the correct navigation bar link to active', () => {
  before(() => {
    logins.silent();
  });

  Object.entries(ACTIVE_MENUS_BY_URL).forEach(([url, { mainMenu, subMenu }]) => {
    it(`should activate ${mainMenu}${subMenu ? ` ${subMenu}` : ''} on ${url}`, () => {
      cy.visit(url);

      cy.get(mainMenu).should('have.class', 'active');

      if (subMenu) {
        cy.get(MAIN_MENUS.SETTINGS).click();

        cy.get(subMenu).should('have.css', 'color', ACTIVE_COLOR);
      }
    });
  });

  it(`should activate ${MAIN_MENUS.SETTINGS} ${SUB_MENUS.JOB_STATUS} on /arc/jobs/job/<job-id>`, () => {
    configureEmailSettings();

    createEmailJob(JobType.Adhoc);

    cy.visit('/arc/jobs/adminlogs');

    cy.contains('Adhoc Email: This is an email job created by Cypress').click();

    cy.url().should('include', '/arc/jobs/job/');

    cy.get(MAIN_MENUS.SETTINGS).should('have.class', 'active');

    cy.get(MAIN_MENUS.SETTINGS).click();

    cy.get(SUB_MENUS.JOB_STATUS).should('have.css', 'color', ACTIVE_COLOR);
  });
});
