/*
 * The test creates a dataset and a SQL visual out of it
 * Saves table from the SQL visual and creates a dataset out of the query
 * clones a field in the dataset and in the expression editor of the cloned field does a validation
 * it is supposed to be successful
 */

import { logins } from 'lib/arctest';
import { createConnection } from 'lib/connections';
import * as dashboards from 'lib/dashboard';
import { extractTargetDb } from 'lib/data/util';
import * as dataset from 'lib/dataset';
import * as functions from 'lib/functions';
import * as permission from 'lib/permissions';
import { setSiteSettingCheckbox } from 'lib/settings';
import * as visuals from 'lib/visual';
import { CSSselector } from 'locators/common/CSSselector';
import { copyDataViaExtract } from 'tests/extracts/util';

describe('Dataset Edit - Cloning field', () => {
  const connectionName = Cypress.env('smokeConnectionName');
  const datasetName = 'Cereals Dataset123';
  const visualTitle = 'SQL Query Visual';
  const fieldName = 'cold_or_hot';
  const viewName = `arc_7473_view_from_ds123_${get_time()}`;
  const newDatasetName = `arc_7473_table_from_ds_table123_${get_time()}`;

  function get_time() {
    return new Date().getTime();
  }

  const extract = {
    srcConnName: 'samples',
    srcDsName: 'Cereals',
    tgtConnName: extractTargetDb.connection_name,
    tgtDbName: extractTargetDb.test_db_name,
    tgtTableName: `cdv_e2etest_table_${get_time()}`,
  };

  context(
    'Test case ## Clone a field and validate from dataset created from SQL Visual save table',
    () => {
      before(() => {
        cy.fixture('users.json').then((data) => {
          logins.silent(data.admin.username, data.admin.password);
        });

        // populate data in target data connection
        setSiteSettingCheckbox('Enable Data Extracts', true);
        createConnection(extractTargetDb);
        functions.navigate_to_data();
        copyDataViaExtract(extract);

        // create a base dataset if it doesn't exist
        functions.createDatasetFromConnection(
          extractTargetDb.connection_name,
          extractTargetDb.test_db_name,
          extract.tgtTableName,
          datasetName
        );

        // create a dashboard with default table visual
        functions.addNewDB(`dashboard_${get_time()}`);
        functions.createNewVisual(extractTargetDb.connection_name, datasetName);
      });

      let totalVisualCount;
      it('1 : Convert table visual to SQL visual, keep the visual in edit mode', () => {
        cy.get(CSSselector.visual_container).then(($visualcontainer) => {
          totalVisualCount = $visualcontainer.children().length;
          visuals.editTableVSToSQLQueryVS(totalVisualCount, visualTitle, true);
          cy.get(CSSselector.save_visual).click();
        });
      });

      it('2 : Save Table and create dataset directly from it', () => {
        cy.get(CSSselector.visualEdit.ellipsis).click();
        cy.get(
          `${CSSselector.visualEdit.moreOptions.submenu} ${CSSselector.visualEdit.moreOptions.options.saveAsTableOrDataset}`
        ).click();
        cy.get('#typeOfView')
          .find(`#logicalView`)
          .click();
        const viewNameInputField = '#newt_view_name';
        cy.get(viewNameInputField).type(viewName);
        const createNewDatasetBasedOnNewTableOrViewCheckbox = "input[value='newt_newds']";
        cy.get(createNewDatasetBasedOnNewTableOrViewCheckbox).check();
        const datasetNameInputField = '#newt_ds_name';
        cy.get(datasetNameInputField).type(newDatasetName);

        cy.get(CSSselector.visualEdit.newtable_from_visual.dbname)
          .invoke('val')
          .should('not.be.empty');

        cy.get('#create-newtable-form')
          .find("button[type='submit']")
          .click();

        cy.get(CSSselector.visualEdit.newtable_from_visual.modal, { timeout: 20_000 }).should(
          'not.be.visible'
        );

        dashboards.saveDashboard();
      });

      it('3 : Edit the dataset and clone a field and validate', () => {
        functions.navigate_to_data();
        permission.selectDataset(extractTargetDb.connection_name, newDatasetName);
        cy.get(CSSselector.dataset_detail.fields).click();
        cy.get(CSSselector.dataset_detail.edit_field).click();
        dataset.cloneField(fieldName);
        cy.get(CSSselector.dataset_detail.edit_field).click();
        dataset.editField('Copy of ' + fieldName);
      });
    }
  );
});
