import * as functions from 'lib/functions';
import * as logins from 'lib/logins';
import * as nl from 'lib/search';

const datasetDetailUrlCereal = 'arc/apps/dataset/94#tab_dataset-search';
const connnectionNameSamples = 'samples';
const datasetNameCereals = 'CerealsWithSearch';

const datasetDetailUrlTradeds = 'arc/apps/dataset/138#tab_dataset-search';
const connnectionNameCurly = 'curly';
const datasetNameTradeds = 'TradeDS';

function visitUrl(datasetUrl: string) {
  cy.visit('/arc/apps/home');
  cy.intercept('GET', '/arc/datasets/dataset/*?detail=true').as('getDatasetdetail');
  cy.intercept('GET', '/arc/apps/users_api?summary=true').as('getUserSummary');
  cy.intercept('GET', '/arc/apps/groups_api').as('getGroupsApi');
  cy.intercept('GET', '/arc/datasets/filterassociation?dataset_id=*').as('filterAssociations');
  cy.intercept('GET', '/arc/roles/roles').as('getRoles');
  cy.visit(datasetUrl);
  cy.wait('@getDatasetdetail');
  cy.wait('@getUserSummary');
  cy.wait('@getGroupsApi');
  cy.wait('@filterAssociations');
  cy.wait('@getRoles');
}

describe('Natural language search test suite', () => {
  context('search_aggregation_tests', () => {
    before(() => {
      logins.silent();
      visitUrl(datasetDetailUrlCereal);
      nl.enableSearchonThisDataset(true);
      nl.setEveryColumnPriority('Low');
    });

    beforeEach(() => {
      cy.visit('/arc/apps/home');
    });

    it('1: simple search using measure field aggregation as sum', () => {
      const searchPhrase = 'sum of calories by manufacturer';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'manufacturer');
      nl.assertFieldMultiple('#x_shelf', 'SUM(calories) [limit 5000]');
    });

    it('2: simple search using measure field aggregation as total', () => {
      const searchPhrase = 'total of calories by manufacturer';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'manufacturer');
      nl.assertFieldMultiple('#x_shelf', 'SUM(calories) [limit 5000]');
    });

    it('3: simple search using measure field aggregation as avg', () => {
      const searchPhrase = 'avg calories by manufacturer';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'manufacturer');
      nl.assertFieldMultiple('#x_shelf', 'AVG(calories) [limit 5000]');
    });
    it('4: simple search using measure field aggregation as average', () => {
      const searchPhrase = 'average calories by manufacturer';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'manufacturer');
      nl.assertFieldMultiple('#x_shelf', 'AVG(calories) [limit 5000]');
    });
    it('5: simple search using measure field aggregation as mean', () => {
      const searchPhrase = 'mean calories by manufacturer';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'manufacturer');
      nl.assertFieldMultiple('#x_shelf', 'AVG(calories) [limit 5000]');
    });

    it('6: simple search using count of all - ARC-11597', () => {
      // the query formed is not correct - file a bug
      const searchPhrase = 'count of all cereal_name by manufacturer';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'manufacturer');
      nl.assertFieldMultiple('#x_shelf', 'AVG(calories) [limit 5000]');
    });

    it('7: simple search using measure field aggregation as max', () => {
      const searchPhrase = 'max calories by manufacturer';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'manufacturer');
      nl.assertFieldMultiple('#x_shelf', 'MAX(calories) [limit 5000]');
    });

    it('8: simple search using measure field aggregation as min', () => {
      const searchPhrase = 'min calories by manufacturer';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'manufacturer');
      nl.assertFieldMultiple('#x_shelf', 'MIN(calories) [limit 5000]');
    });
  });

  context('natural language aggregation tests Trade DS', () => {
    before(() => {
      logins.silent();
      visitUrl(datasetDetailUrlTradeds);
      nl.enableSearchonThisDataset(true);
      nl.setEveryColumnPriority('Low');
      visitUrl(datasetDetailUrlCereal);
      nl.enableSearchonThisDataset(false);
    });

    beforeEach(() => {
      cy.visit('/arc/apps/home');
    });

    after(() => {
      visitUrl(datasetDetailUrlTradeds);
      nl.enableSearchonThisDataset(false);
      visitUrl(datasetDetailUrlCereal);
      nl.enableSearchonThisDataset(true);
    });

    it('1: simple search using measure field aggregation as standard deviation', () => {
      // use trade dataset, since stddev is supported only in arcadia connection
      const searchPhrase = 'standard deviation quantity businessdate';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'businessdate');
      nl.assertFieldMultiple('#x_shelf', 'STDDEV(quantity) [limit 5000]');
    });

    it('2: simple search using measure field aggregation as variance', () => {
      // use trade dataset, since stddev is supported only in arcadia connection
      const searchPhrase = 'variance of price by businessdate';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'businessdate');
      nl.assertFieldMultiple('#x_shelf', 'VARIANCE(price) [limit 5000]');
    });
  });
});
