import * as logins from 'lib/logins';
import * as nl from 'lib/search';
import { setSearchSetting } from 'lib/settings';

const datasetDetailUrlCereal = 'arc/apps/dataset/10#tab_dataset-search';

describe('Natural language search visual type test suite ', () => {
  context('natural language visual type search tests', () => {
    before(() => {
      logins.silent();

      setSearchSetting(true);

      cy.intercept('GET', '/arc/datasets/dataset/*?detail=true').as('getDatasetdetail');
      cy.intercept('GET', '/arc/apps/users_api?summary=true').as('getUserSummary');
      cy.intercept('GET', '/arc/apps/groups_api').as('getGroupsApi');
      cy.intercept('GET', '/arc/datasets/filterassociation?dataset_id=*').as('filterAssociations');
      cy.intercept('GET', '/arc/roles/roles').as('getRoles');
      cy.visit(datasetDetailUrlCereal);
      cy.wait('@getDatasetdetail');
      cy.wait('@getUserSummary');
      cy.wait('@getGroupsApi');
      cy.wait('@filterAssociations');
      cy.wait('@getRoles');

      nl.enableSearchonThisDataset(true);
      nl.setEveryColumnPriority('Low');
    });

    after(() => {
      setSearchSetting(false);
    });

    it('1: simple search using measure field aggregation as table', () => {
      const searchPhrase = 'cereal_name calories as table';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#dimensions_shelf', 'cereal_name');
      nl.assertFieldMultiple('#aggregates_shelf', 'SUM(calories) [limit 5000]');
      cy.get('[data-test-id="visual-data-tab"]').click();
      cy.get('div.selected-viz .chart-icon.chart-icon-table').should('be.visible');
    });

    it('2: simple search using measure field aggregation as pie', () => {
      const searchPhrase = 'cereal_name calories as pie';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#dimensions_shelf', 'cereal_name');
      nl.assertFieldMultiple('#aggregates_shelf', 'SUM(calories) [limit 5000]');
      cy.get('[data-test-id="visual-data-tab"]').click();
      cy.get('div.selected-viz .chart-icon.chart-icon-pie').should('be.visible');
    });

    it('3: simple search using measure field aggregation as crosstab', () => {
      const searchPhrase = 'cereal_name calories as crosstab';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_dimensions_shelf', 'cereal_name');
      nl.assertFieldMultiple('#aggregates_shelf', 'SUM(calories) [limit 5000]');
      cy.get('[data-test-id="visual-data-tab"]').click();
      cy.get('div.selected-viz .chart-icon.chart-icon-crosstab').should('be.visible');
    });
  });
});
