import * as dsManagement from 'lib/datasetManagement';
import * as logins from 'lib/logins';
import * as nl from 'lib/search';

const datasetDetailUrlTradeds = 'arc/apps/dataset/138#tab_dataset-search';

const datasetDetailUrlCereal = 'arc/apps/dataset/94#tab_dataset-search';
const columnNames = [
  'tradingsystem',
  'businessdate',
  'tradingdesk',
  'price',
  'quantity',
  'limitprice',
  'stopprice',
  'averageprice',
  'orderstatus',
  'ordercreationdatetime',
  'executiondatetime',
  'numofexecutions',
  'originatingorderdatetime',
  'orderexecutiondatetime',
];

describe('Natural language serch test suite', () => {
  context('natural language filter tests with date fields', () => {
    before(() => {
      logins.silent();
      cy.visit('/arc/apps/home');
      visitUrl(datasetDetailUrlTradeds);
      nl.enableSearchonThisDataset(true);

      columnNames.forEach((column) => {
        cy.get(dsManagement.locators.columnsTableRow)
          .contains(`[${column}]`)
          .parent()
          .find('select.user-for-search:not([disabled])')
          .each(($el) => cy.wrap($el).select('Low'));
      });
      cy.get(dsManagement.locators.saveSearchModelBtn).click();

      visitUrl(datasetDetailUrlCereal);
      nl.enableSearchonThisDataset(false);
    });

    beforeEach(() => {
      cy.visit('/arc/apps/home');
    });

    after(() => {
      visitUrl(datasetDetailUrlTradeds);
      nl.enableSearchonThisDataset(false);
      visitUrl(datasetDetailUrlCereal);
      nl.enableSearchonThisDataset(true);
    });

    function visitUrl(datasetUrl: string) {
      cy.visit('/arc/apps/home');
      cy.intercept('GET', '/arc/datasets/dataset/*?detail=true').as('getDatasetdetail');
      cy.intercept('GET', '/arc/apps/users_api?summary=true').as('getUserSummary');
      cy.intercept('GET', '/arc/apps/groups_api').as('getGroupsApi');
      cy.intercept('GET', '/arc/datasets/filterassociation?dataset_id=*').as('filterAssociations');
      cy.intercept('GET', '/arc/roles/roles').as('getRoles');
      cy.visit(datasetUrl);
      cy.wait('@getDatasetdetail');
      cy.wait('@getUserSummary');
      cy.wait('@getGroupsApi');
      cy.wait('@filterAssociations');
      cy.wait('@getRoles');
    }

    it('1: filter using date field starts with year', () => {
      const searchPhrase =
        'price  orderexecutiondatetime  where orderexecutiondatetime starts with 2016';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'orderexecutiondatetime');
      nl.assertFieldMultiple('#x_shelf', 'SUM(price) [limit 5000]');
      nl.assertFieldMultiple(
        '#filters_shelf',
        '((lower(cast(orderexecutiondatetime as string)) like "2016%"))'
      );
    });

    it('2: filter using date field ends with date', () => {
      const searchPhrase =
        'price  orderexecutiondatetime  where orderexecutiondatetime ends with 29';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'orderexecutiondatetime');
      nl.assertFieldMultiple('#x_shelf', 'SUM(price) [limit 5000]');
      nl.assertFieldMultiple(
        '#filters_shelf',
        '((lower(cast(orderexecutiondatetime as string)) like "%29"))'
      );
    });

    it('3: filter using date field where date is', () => {
      const searchPhrase =
        'price  orderexecutiondatetime  where orderexecutiondatetime is 2016-01-26';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'orderexecutiondatetime');
      nl.assertFieldMultiple('#x_shelf', 'SUM(price) [limit 5000]');
      nl.assertFieldMultiple(
        '#filters_shelf',
        '(lower(cast(orderexecutiondatetime as string)) like "%2016-01-26%")'
      );
    });

    it('4: filter using date field where date is equal to', () => {
      const searchPhrase =
        'price  orderexecutiondatetime  where orderexecutiondatetime is equal to 2016-01-27';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'orderexecutiondatetime');
      nl.assertFieldMultiple('#x_shelf', 'SUM(price) [limit 5000]');
      nl.assertFieldMultiple(
        '#filters_shelf',
        '((lower(cast(orderexecutiondatetime as string)) in ("to")) OR (lower(cast(orderexecutiondatetime as string)) like "%2016-01-27%"))'
      );
    });

    it('5: filter using date field where date between date1 and date2 - ARC-11723', () => {
      const searchPhrase =
        'price  orderexecutiondatetime  where orderexecutiondatetime year between 2016 and 2017';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#x_shelf', 'orderexecutiondatetime');
      nl.assertFieldMultiple('#y_shelf', 'SUM(price) [limit 5000]');
      nl.assertFieldMultiple('#filters_shelf', '(lower(orderexecutiondatetime) like "2016%")');
    });

    it('6: filter using date field where date from year to year  ARC-11723', () => {
      const searchPhrase =
        'price  orderexecutiondatetime  where orderexecutiondatetime from 2016 to 2017';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#x_shelf', 'orderexecutiondatetime');
      nl.assertFieldMultiple('#y_shelf', 'SUM(price) [limit 5000]');
      nl.assertFieldMultiple('#filters_shelf', '(lower(orderexecutiondatetime) like "2016%")');
    });

    it('7: filter using date field where date between date1 and date2', () => {
      const searchPhrase =
        'price  orderexecutiondatetime  where orderexecutiondatetime between 2016-01-26 and 2016-01-27';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'orderexecutiondatetime');
      nl.assertFieldMultiple('#x_shelf', 'SUM(price) [limit 5000]');
      nl.assertFieldMultiple(
        '#filters_shelf',
        'orderexecutiondatetime between "2016-01-26 00:00:00" and "2016-01-27 00:00:00"'
      );
    });

    it('8: filter using date field where date from date1 to date2', () => {
      const searchPhrase =
        'price  orderexecutiondatetime  where orderexecutiondatetime between 2016-01-26 and 2016-01-27';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'orderexecutiondatetime');
      nl.assertFieldMultiple('#x_shelf', 'SUM(price) [limit 5000]');
      nl.assertFieldMultiple(
        '#filters_shelf',
        'orderexecutiondatetime between "2016-01-26 00:00:00" and "2016-01-27 00:00:00"'
      );
    });

    it('9: filter using date field keyword this year', () => {
      const searchPhrase = 'price  orderexecutiondatetime  where orderexecutiondatetime this year';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'orderexecutiondatetime');
      nl.assertFieldMultiple('#x_shelf', 'SUM(price) [limit 5000]');
      nl.assertFieldMultiple(
        '#filters_shelf',
        'orderexecutiondatetime between "' +
          new Date().getFullYear() +
          '-01-01 00:00:00" and "' +
          new Date().getFullYear() +
          '-12-31 23:59:59"'
      );
    });

    it('10: filter using date field keyword this month', () => {
      const searchPhrase = 'price  orderexecutiondatetime  where orderexecutiondatetime this month';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'orderexecutiondatetime');
      nl.assertFieldMultiple('#x_shelf', 'SUM(price) [limit 5000]');
      const date = new Date();
      cy.log(
        'orderexecutiondatetime between "' +
          date.getFullYear() +
          '-' +
          getCurrentMonth() +
          '-01 00:00:00" and "' +
          date.getFullYear() +
          '-' +
          getCurrentMonth() +
          '-' +
          getDaysinCurrentMonth() +
          ' 23:59:59"'
      );
      nl.assertFieldMultiple(
        '#filters_shelf',
        'orderexecutiondatetime between "' +
          date.getFullYear() +
          '-' +
          getCurrentMonth() +
          '-01 00:00:00" and "' +
          date.getFullYear() +
          '-' +
          getCurrentMonth() +
          '-' +
          getDaysinCurrentMonth() +
          ' 23:59:59"'
      );
    });

    it.skip('11: filter using date field keyword this week', () => {
      const searchPhrase = 'price  orderexecutiondatetime  where orderexecutiondatetime this week';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'orderexecutiondatetime');
      nl.assertFieldMultiple('#x_shelf', 'SUM(price) [limit 5000]');
      nl.assertFieldMultiple('#filters_shelf', '(lower(orderexecutiondatetime) like "2016%")');
    });

    it('12: filter using date field keyword last year', () => {
      const searchPhrase = 'price  orderexecutiondatetime  where orderexecutiondatetime last year';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'orderexecutiondatetime');
      nl.assertFieldMultiple('#x_shelf', 'SUM(price) [limit 5000]');
      nl.assertFieldMultiple(
        '#filters_shelf',
        'orderexecutiondatetime between "' +
          (new Date().getFullYear() - 1) +
          '-01-01 00:00:00" and "' +
          (new Date().getFullYear() - 1) +
          '-12-31 23:59:59"'
      );
    });

    it.skip('13: filter using date field keyword last month', () => {
      const searchPhrase = 'price  orderexecutiondatetime  where orderexecutiondatetime last month';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'orderexecutiondatetime');
      nl.assertFieldMultiple('#x_shelf', 'SUM(price) [limit 5000]');
      const date = new Date();
      cy.log(
        'orderexecutiondatetime between "' +
          date.getFullYear() +
          '-' +
          getLastMonth() +
          '-01 00:00:00" and "' +
          date.getFullYear() +
          '-' +
          getLastMonth() +
          '-' +
          getDaysinCurrentMonth() +
          ' 23:59:59"'
      );
      nl.assertFieldMultiple(
        '#filters_shelf',
        'orderexecutiondatetime between "' +
          date.getFullYear() +
          '-' +
          getLastMonth() +
          '-01 00:00:00" and "' +
          date.getFullYear() +
          '-' +
          getLastMonth() +
          '-' +
          getDaysinCurrentMonth() +
          ' 23:59:59"'
      );
    });

    it.skip('14: filter using date field keyword last week', () => {
      const searchPhrase = 'price  orderexecutiondatetime  where orderexecutiondatetime last week';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'orderexecutiondatetime');
      nl.assertFieldMultiple('#x_shelf', 'SUM(price) [limit 5000]');
      nl.assertFieldMultiple('#filters_shelf', '(lower(orderexecutiondatetime) like "2016%")');
    });

    it('15: filter using date field keyword previous year', () => {
      const searchPhrase =
        'price  orderexecutiondatetime  where orderexecutiondatetime previous year';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'orderexecutiondatetime');
      nl.assertFieldMultiple('#x_shelf', 'SUM(price) [limit 5000]');
      nl.assertFieldMultiple(
        '#filters_shelf',
        'orderexecutiondatetime between "' +
          (new Date().getFullYear() - 1) +
          '-01-01 00:00:00" and "' +
          (new Date().getFullYear() - 1) +
          '-12-31 23:59:59"'
      );
    });

    it.skip('16: filter using date field keyword previous month', () => {
      const searchPhrase =
        'price  orderexecutiondatetime  where orderexecutiondatetime previous month';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#x_shelf', 'orderexecutiondatetime');
      nl.assertFieldMultiple('#y_shelf', 'SUM(price) [limit 5000]');
      nl.assertFieldMultiple('#filters_shelf', '(lower(orderexecutiondatetime) like "2016%")');
    });

    it.skip('17: filter using date field keyword previous week', () => {
      const searchPhrase =
        'price orderexecutiondatetime  where orderexecutiondatetime previous week';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#x_shelf', 'orderexecutiondatetime');
      nl.assertFieldMultiple('#y_shelf', 'SUM(price) [limit 5000]');
      nl.assertFieldMultiple('#filters_shelf', '(lower(orderexecutiondatetime) like "2016%")');
    });

    it('18: filter using aggregate field real value', () => {
      const searchPhrase =
        'price on businessdate with price greater than 1.3 or price is equal  to 1.3';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'businessdate');
      nl.assertFieldMultiple('#x_shelf', 'SUM(price) [limit 5000]');
      nl.assertFieldMultiple('#filters_shelf', '((SUM(price) > 1.3) OR (SUM(price) = 1.3))');
    });
  });
});

function getDaysinCurrentMonth() {
  const today = new Date();
  const month = today.getMonth();
  cy.log('Month :' + today.getMonth());
  const days = daysInMonth(month + 1, today.getFullYear());
  if (days < 10) {
    return '0' + days;
  } else {
    return days;
  }
}

function daysInMonth(month: number, year: number) {
  cy.log('Date :' + new Date(year, month, 0).getDate());
  return new Date(year, month, 0).getDate();
}

function getCurrentMonth() {
  const today = new Date();
  const month = today.getMonth() + 1;
  if (month < 10) {
    return '0' + month;
  } else {
    return month;
  }
}

function getLastMonth() {
  const today = new Date();
  const month = today.getMonth();
  if (month < 10) {
    return '0' + month;
  } else {
    return month;
  }
}

function getDaysinLastMonth(month: number, year: number) {
  cy.log('Date :' + new Date(year, month, 0).getDate());
  return new Date(year, month, 0).getDate();
}
