import * as logins from 'lib/logins';
import * as nl from 'lib/search';

const datasetDetailUrlCereal = 'arc/apps/dataset/94#tab_dataset-search';

describe('Natural language search  test suite ', () => {
  context('search_dim_mea_tests', () => {
    before(() => {
      logins.silent();
      cy.visit('/arc/apps/home');
      cy.intercept('GET', '/arc/datasets/dataset/*?detail=true').as('getDatasetdetail');
      cy.intercept('GET', '/arc/apps/users_api?summary=true').as('getUserSummary');
      cy.intercept('GET', '/arc/apps/groups_api').as('getGroupsApi');
      cy.intercept('GET', '/arc/datasets/filterassociation?dataset_id=*').as('filterAssociations');
      cy.intercept('GET', '/arc/roles/roles').as('getRoles');
      cy.visit(datasetDetailUrlCereal);
      cy.wait('@getDatasetdetail');
      cy.wait('@getUserSummary');
      cy.wait('@getGroupsApi');
      cy.wait('@filterAssociations');
      cy.wait('@getRoles');
      nl.enableSearchonThisDataset(true);
      nl.setEveryColumnPriority('Low');
    });

    beforeEach(() => {
      cy.visit('/arc/apps/home');
    });

    it('1: simple search using dimension and measure field', () => {
      const searchPhrase = 'cereal_name calories';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'cereal_name');
      nl.assertFieldMultiple('#x_shelf', 'SUM(calories) [limit 5000]');
    });

    it('2: filter using between keyword', () => {
      const searchPhrase = 'cereal_name with calories between 50 and 100';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'cereal_name');
      nl.assertFieldMultiple('#x_shelf', 'SUM(calories) [limit 5000]');
      nl.assertFieldMultiple('#filters_shelf', '(SUM(calories) >= 50) and (SUM(calories) <= 100)');
    });

    it('3: comparison using vs', () => {
      const searchPhrase = 'manufacturer vs average calories';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'manufacturer');
      nl.assertFieldMultiple('#x_shelf', 'AVG(calories) [limit 5000]');
    });

    it('4: filter using = operator', () => {
      const searchPhrase = 'count cereal_name by manufacturer with calories = 100';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#x_shelf', 'cereal_name');
      nl.assertFieldMultiple('#y_shelf', 'sum(1) [limit 5000]');
      nl.assertFieldMultiple('#color_shelf', 'manufacturer');
      nl.assertFieldMultiple('#filters_shelf', '(SUM(calories) = 100)');
    });

    it('5: filter using != operator', () => {
      const searchPhrase = 'count cereal_name by manufacturer with calories != 100';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#x_shelf', 'cereal_name');
      nl.assertFieldMultiple('#y_shelf', 'sum(1) [limit 5000]');
      nl.assertFieldMultiple('#color_shelf', 'manufacturer');
      nl.assertFieldMultiple('#filters_shelf', '(SUM(calories) != 100)');
    });

    it('6: filter using < operator', () => {
      const searchPhrase = 'count cereal_name by manufacturer with calories < 100';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#x_shelf', 'cereal_name');
      nl.assertFieldMultiple('#y_shelf', 'sum(1) [limit 5000]');
      nl.assertFieldMultiple('#color_shelf', 'manufacturer');
      nl.assertFieldMultiple('#filters_shelf', '(SUM(calories) < 100)');
    });

    it('7: filter using > operator', () => {
      const searchPhrase = 'count cereal_name by manufacturer with calories > 100';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#x_shelf', 'cereal_name');
      nl.assertFieldMultiple('#y_shelf', 'sum(1) [limit 5000]');
      nl.assertFieldMultiple('#color_shelf', 'manufacturer');
      nl.assertFieldMultiple('#filters_shelf', '(SUM(calories) > 100)');
    });

    it('8: filter using <= operator', () => {
      const searchPhrase = 'count cereal_name by manufacturer with calories <= 100';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#x_shelf', 'cereal_name');
      nl.assertFieldMultiple('#y_shelf', 'sum(1) [limit 5000]');
      nl.assertFieldMultiple('#color_shelf', 'manufacturer');
      nl.assertFieldMultiple('#filters_shelf', '(SUM(calories) <= 100)');
    });

    it('9: filter using >= operator', () => {
      const searchPhrase = 'count cereal_name by manufacturer with calories >= 100';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#x_shelf', 'cereal_name');
      nl.assertFieldMultiple('#y_shelf', 'sum(1) [limit 5000]');
      nl.assertFieldMultiple('#color_shelf', 'manufacturer');
      nl.assertFieldMultiple('#filters_shelf', '(SUM(calories) >= 100)');
    });

    it('10: count by dimension', () => {
      const searchPhrase = 'count cereal_name by cold_or_hot';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#x_shelf', 'cereal_name');
      nl.assertFieldMultiple('#y_shelf', 'sum(1) [limit 5000]');
      nl.assertFieldMultiple('#color_shelf', 'cold_or_hot');
    });

    it('11: filter using begins with keyword', () => {
      const searchPhrase = 'cereal_name begins with A';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#x_shelf', 'sum(1) [limit 5000]');
      nl.assertFieldMultiple('#y_shelf', 'cereal_name');
      nl.assertFieldMultiple('#filters_shelf', '((lower(cereal_name) like "a%"))');
    });

    it('12: filter using contains keyword', () => {
      const searchPhrase = 'cereal_name contains ee';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'cereal_name');
      nl.assertFieldMultiple('#x_shelf', 'sum(1) [limit 5000]');
      nl.assertFieldMultiple('#filters_shelf', '((lower(cereal_name) like "%ee%"))');
    });

    it('13: filter using multiple filter keywords joined using and', () => {
      const searchPhrase =
        'cereal_name calories with cereal_name ends with a and calories between 10 and 150';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'cereal_name');
      nl.assertFieldMultiple('#x_shelf', 'SUM(calories) [limit 5000]');
      nl.assertFieldMultiple('#filters_shelf', '((lower(cereal_name) like "%a"))');
      nl.assertFieldMultiple('#filters_shelf', '(SUM(calories) >= 10) and (SUM(calories) <= 150)');
    });

    it('14: filter using not begins with keyword', () => {
      const searchPhrase = 'cereal_name not begins with A';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'cereal_name');
      nl.assertFieldMultiple('#x_shelf', 'sum(1) [limit 5000]');
      nl.assertFieldMultiple('#filters_shelf', '(not (lower(cereal_name) like "a%"))');
    });

    it('15: filter using not contains with keyword', () => {
      const searchPhrase = 'cereal_name not contains Cheerios';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'cereal_name');
      nl.assertFieldMultiple('#x_shelf', 'sum(1) [limit 5000]');
      nl.assertFieldMultiple('#filters_shelf', '(not ((lower(cereal_name) like "%cheerios%")))');
    });

    it('16: filter using similar to keyword', () => {
      const searchPhrase = 'manufacturer similar to loggs';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'manufacturer');
      nl.assertFieldMultiple('#x_shelf', 'sum(1) [limit 5000]');
      nl.assertFieldMultiple('#filters_shelf', '((lower(manufacturer) like "%loggs%"))');
    });

    it('17: filter using not similar to keyword', () => {
      const searchPhrase = 'manufacturer not similar to loggs';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'manufacturer');
      nl.assertFieldMultiple('#x_shelf', 'sum(1) [limit 5000]');
      nl.assertFieldMultiple('#filters_shelf', '(not ((lower(manufacturer) like "%loggs%")))');
    });

    it('18: filter using excluding keyword', () => {
      const searchPhrase = 'calories by manufacturer excluding Kelloggs';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#y_shelf', 'manufacturer');
      nl.assertFieldMultiple('#x_shelf', 'SUM(calories) [limit 5000]');
      nl.assertFieldMultiple('#filters_shelf', '(lower(manufacturer) not in ("kelloggs"))');
    });

    it('19: filter using dimension field value - file bug', () => {
      const searchPhrase = 'cereal_name calories fat sugars of Apple_Jacks as table';
      // search
      nl.openGlobalSearchModal();
      //  search and assert for result
      nl.searchGlobally(searchPhrase);
      // create a dashboard from first seach result
      nl.createDashboardFromSearch();
      // edit visual and assert
      nl.assertFieldMultiple('#dimensions_shelf', 'cereal_name');
      nl.assertFieldMultiple('#aggregates_shelf', 'SUM(calories)');
      nl.assertFieldMultiple('#aggregates_shelf', 'SUM(fat)');
      nl.assertFieldMultiple('#aggregates_shelf', 'SUM(sugars) [limit 5000]');
      nl.assertFieldMultiple('#filters_shelf', '(lower(cereal_name) like "%apple_jacks%")');
    });
  });
});
