import * as logins from 'lib/logins';
import * as nl from 'lib/search';
import { setSearchSetting } from 'lib/settings';

const datasetDetailUrlCereal = 'arc/apps/dataset/10#tab_dataset-search';

describe('Natural language search sort test suite ', () => {
  context('natural language sort search tests', () => {
    before(() => {
      logins.silent();

      setSearchSetting(true);

      cy.intercept('GET', '/arc/datasets/dataset/*?detail=true').as('getDatasetdetail');
      cy.intercept('GET', '/arc/apps/users_api?summary=true').as('getUserSummary');
      cy.intercept('GET', '/arc/apps/groups_api').as('getGroupsApi');
      cy.intercept('GET', '/arc/datasets/filterassociation?dataset_id=*').as('filterAssociations');
      cy.intercept('GET', '/arc/roles/roles').as('getRoles');
      cy.visit(datasetDetailUrlCereal);
      cy.wait('@getDatasetdetail');
      cy.wait('@getUserSummary');
      cy.wait('@getGroupsApi');
      cy.wait('@filterAssociations');
      cy.wait('@getRoles');

      nl.enableSearchonThisDataset(true);
      nl.setEveryColumnPriority('Low');
    });

    after(() => {
      setSearchSetting(false);
    });

    it('1: sort dimension field in ascending order', () => {
      nl.openGlobalSearchModal();
      nl.searchGlobally('manufacturer_code, cereal_name sort by calories in ascending');
      nl.createDashboardFromSearch();
      cy.get('[data-test-id="visual-data-tab"]').click();
      cy.get('[data-test-id="loading-bar"]').should('not.be.visible');

      nl.assertFieldMultiple('#x_shelf', 'SUM(calories) [limit 5000]');
      nl.assertFieldMultiple('#y_shelf', 'cereal_name');
      nl.assertFieldMultiple('#color_shelf', 'manufacturer_code');
      nl.assertFieldSorting('#x_shelf', 'asc');
    });

    it('2: sort dimension field in descending order', () => {
      nl.openGlobalSearchModal();
      nl.searchGlobally('manufacturer_code, cereal_name sort by calories in descending');
      nl.createDashboardFromSearch();
      cy.get('[data-test-id="visual-data-tab"]').click();
      cy.get('[data-test-id="loading-bar"]').should('not.be.visible');

      nl.assertFieldMultiple('#x_shelf', 'SUM(calories) [limit 5000]');
      nl.assertFieldMultiple('#y_shelf', 'cereal_name');
      nl.assertFieldMultiple('#color_shelf', 'manufacturer_code');
      nl.assertFieldSorting('#x_shelf', 'desc');
    });

    it('3: sort dimension field in descending order using keyword ordered by', () => {
      nl.openGlobalSearchModal();
      nl.searchGlobally('cereal_name calories ordered by calories');
      nl.createDashboardFromSearch();
      cy.get('[data-test-id="visual-data-tab"]').click();
      cy.get('[data-test-id="loading-bar"]').should('not.be.visible');

      nl.assertFieldMultiple('#x_shelf', 'SUM(calories) [limit 5000]');
      nl.assertFieldMultiple('#y_shelf', 'cereal_name');
      nl.assertFieldSorting('#x_shelf', 'asc');
    });

    it('4: sort dimension field in ascending order and apply top 3', () => {
      nl.openGlobalSearchModal();
      nl.searchGlobally('manufacturer_code, cereal_name sort by calories top 3');
      nl.createDashboardFromSearch();
      cy.get('[data-test-id="visual-data-tab"]').click();
      cy.get('[data-test-id="loading-bar"]').should('not.be.visible');

      nl.assertFieldMultiple('#x_shelf', 'SUM(calories) [limit 3]');
      nl.assertFieldMultiple('#y_shelf', 'cereal_name');
      nl.assertFieldMultiple('#color_shelf', 'manufacturer_code');
      nl.assertFieldSorting('#x_shelf', 'desc');
    });
  });
});
