import * as logins from 'lib/logins';
import * as nl from 'lib/search';
import { setSearchSetting } from 'lib/settings';

/*
 * Verify that the SQL builder visualization works
 * Assumes the existence of the 'US State Populations Over Time' dataset
 *   in the Cypress.env("smokeConnectionName") data connection
 */

describe('Search', () => {
  describe('search model tests', () => {
    before(() => {
      logins.silent();
      setSearchSetting(true);
    });

    beforeEach(() => {
      cy.intercept('GET', '/arc/datasets/dataset/*?detail=true').as('getDatasetdetail');
      cy.intercept('GET', '/arc/apps/users_api?summary=true').as('getUserSummary');
      cy.intercept('GET', '/arc/apps/groups_api').as('getGroupsApi');
      cy.intercept('GET', '/arc/datasets/filterassociation?dataset_id=*').as('filterAssociations');
      cy.intercept('GET', '/arc/roles/roles').as('getRoles');
      cy.visit(datasetDetailUrl);
      cy.wait('@getDatasetdetail');
      cy.wait('@getUserSummary');
      cy.wait('@getGroupsApi');
      cy.wait('@filterAssociations');
      cy.wait('@getRoles');
    });

    after(() => {
      setSearchSetting(false);
    });

    const datasetDetailUrl = 'arc/apps/dataset/6#tab_dataset-search';
    const connnectionName = 'samples';
    const datasetName = 'US State Populations Over Time';
    const searchGroupName = 'Population Stats_1'; // bug that save button shows disabled even though page contents changed -  so have to pass {enter}
    const sampleSearchQuery: string = 'state with the most population';
    const suggestedQuestions: string[] = ['question 1 ', 'question 2'];
    const wordExpansions: nl.WordExpansion[] = [
      { inputWord: 'trip', outputWord: 'trip_distance' },
      { inputWord: 'pickup', outputWord: 'pickup_neighborhood' },
    ];

    it('1: enable search on dataset', () => {
      nl.enableSearchonThisDataset(true);
      nl.setEveryColumnPriority('Low');
      nl.verifySearchEnabledOnDataset(connnectionName, datasetName, sampleSearchQuery);
    });

    it('2: set a search group', () => {
      nl.enterSearchGroup(searchGroupName);
      nl.verifySearchGroup(searchGroupName, datasetName);
    });

    it('3: add suggested questions', () => {
      nl.addSuggestedQuestions(suggestedQuestions);
      nl.verifySuggestedQuestions(suggestedQuestions);
    });

    it('4: add word expansions', () => {
      nl.addWordExpansions(wordExpansions);
      nl.verifyWordExpansions(wordExpansions);
    });

    it('5: disable search on dataset', () => {
      nl.setEveryColumnPriority('None');
      nl.enableSearchonThisDataset(false);
      nl.verifySerchDisabledOnDataset(connnectionName, datasetName);
    });
  });
});
