import { logins } from 'lib/arctest';
import { BackendAPI } from 'lib/backend-api';
import { createConnection } from 'lib/connections';
import { TEST_CONNECTIONS } from 'lib/connections/connections';
import { getErrorIndicators, viewDashboardById } from 'lib/dashboard';
import { getTableVisualColumn, getVisualContent } from 'lib/visual';

describe('Solr dashboard with daterange filter and Solr query', () => {
  before(() => {
    logins.silent();
    cy.visit('/');

    const solrConn = TEST_CONNECTIONS.test_solr_standalone;
    createConnection(solrConn);

    BackendAPI.dashboards
      .import({
        fileName: 'solr/solr_hr_data_daterange_solr_query.json',
        dataconnectionName: solrConn.connection_name,
        workspaceName: 'Public',
      })
      .then(({ dashboard }) => viewDashboardById(dashboard.id));
  });

  it('Loads dashboard without errors', () => {
    getErrorIndicators().should('have.length', 0);
  });

  it('filters table correctly', () => {
    // select the 3rd column from a table:
    const visualTitle = 'Active only, uses Solr Query';
    getTableVisualColumn(visualTitle, 3).should('have.length', 4);
  });

  it('filters KPI', () => {
    const visualTitle = 'Rows in selected range';
    getVisualContent(visualTitle).then((contents) => {
      expect(contents.text()).to.equal('7');
    });
  });
});
