import * as arctest from 'lib/arctest';
import { createConnection } from 'lib/connections';
import { createNewDataset, navigate_to_data } from 'lib/functions';

const solrConn = {
  connection_type: 'Solr (Tech Preview)',
  connection_name: 'Solr-Connection-202009',
  hostname: Cypress.env('SOLR_HOSTNAME'),
  port: '8983',
  solr_type: 'solr_type_standalone',
  advanced: {
    sock_type: 'normal_sock_type',
  },
};

describe('Solr search highlight test', () => {
  before(() => {
    arctest.logins.silent();

    navigate_to_data();

    cy.get('#dataconnections-list').then(() => {
      createConnection(solrConn);
      createNewDataset('Solr', 'books', 'books');
    });
  });

  it('Highlight test', () => {
    cy.get('a[data-title="New Dashboard"]').click();
    cy.get('.progress.viz-permanent').should('not.be.visible');
    cy.get('#visual-settings-tab').click();
    cy.get('.settings-tab-holder[data-tabname="Marks"]').click();
    cy.get('[name="CELLS_ARE_HTML"]').check();
    cy.get('.progress.viz-permanent').should('not.be.visible');
    cy.get('[name="ENABLE_HIGHLIGHTING"]').check();
    cy.get('.progress.viz-permanent').should('not.be.visible');
    cy.get('#visual-data-tab').click();

    cy.get('#app-shelves').within(() => {
      cy.get('input').type('Author:John', { force: true });
    });

    cy.get('#viz-refresh').click({ force: true });
    cy.get('.progress.viz-permanent').should('not.be.visible');

    cy.get('.arc-viz-container.arc-viz-type-table').within(() => {
      cy.get('mark').should('exist');
    });

    cy.get('#app-shelves').within(() => {
      cy.get('input')
        .clear({ force: true })
        .type('Author:NotAnAuthor', { force: true });
    });

    cy.get('#viz-refresh').click({ force: true });
    cy.get('.progress.viz-permanent').should('not.be.visible');

    cy.get('.arc-viz-container.arc-viz-type-table').within(() => {
      cy.get('mark').should('not.exist');
    });
  });
});
