import { logins } from 'lib/arctest';
import { BackendAPI } from 'lib/backend-api';
import { createConnection } from 'lib/connections';
import { TEST_CONNECTIONS } from 'lib/connections/connections';
import { getErrorIndicators, viewDashboardById } from 'lib/dashboard';
import { getTableVisualColumn } from 'lib/visual';

describe('Solr dashboard with table visuals', () => {
  before(() => {
    logins.silent();
    cy.visit('/');

    const solrConn = TEST_CONNECTIONS.test_solr_standalone;
    createConnection(solrConn);

    BackendAPI.dashboards
      .import({
        fileName: 'solr/solr_hr_data_tables.json',
        dataconnectionName: solrConn.connection_name,
        workspaceName: 'Public',
      })
      .then(({ dashboard }) => viewDashboardById(dashboard.id));
  });

  it('Loads dashboard without errors', () => {
    getErrorIndicators().should('have.length', 0);
  });

  it('filters table correctly', () => {
    const visualTitle = 'No aggregates, various filters';
    // third column of the table:
    getTableVisualColumn(visualTitle, 3).should('have.length', 13);
  });

  it('sorts by sum aggregate', () => {
    const visualTitle = 'All aggregates, ordered by the first aggregate';
    // first column of the table:
    getTableVisualColumn(visualTitle, 1)
      .should('have.length', 3)
      .then((cells) => {
        const cellTexts = cells.map((_, e) => Cypress.$(e).text()).toArray();
        expect(cellTexts).to.deep.equal([
          'Terminated for Cause',
          'Voluntarily Terminated',
          'Active',
        ]);
      });
  });
});
