import * as arctest from 'lib/arctest/screen/builder';
import * as dashboard from 'lib/dashboard';
import { addNewDB, createNewVisual } from 'lib/functions';

export function createDBWithTwoSheetsAndFilters(data: any) {
  // create dashboard
  addNewDB('URL Shortener');
  // create new visual with filter
  // select filter values
  createNewVisual(data.connectionName, data.datasetName);
  arctest.addFilter([data.filterName1]);
  arctest.chooseOneOrMultiplePicklistItem(data.filterValue1);
  // save
  dashboard.saveDashboard();

  // add sheet
  dashboard.addSheet();
  // create new visual with filter
  // select filter values
  createNewVisual(data.connectionName, data.datasetName);
  arctest.addFilter([data.filterName2]);
  arctest.chooseOneOrMultiplePicklistItem(data.filterValue2);
  // save
  dashboard.saveDashboard();
}

export function existsInDOM(element: string) {
  cy.get('body').then(($body) => {
    const selector = $body.find(`${element}:visible`).length > 0;
    console.log('selector: ', selector);
    if (!selector) {
      console.log('No element in DOM');
    } else {
      cy.get(`${element}`)
        .eq(0)
        .click();
    }
  });
}

export function existingURLListControlsCheck(alias: string) {
  cy.get('.nav-tab-aliases').click();
  // check the alias URL exists
  cy.get('.flex-grow-1').should('contain', `/arc/short/${alias}\n`);
  // check the a tag href
  cy.location('origin').then((origin) => {
    const hostname = origin;
    cy.get('.existing-url').should('have.attr', 'href', `${hostname}/arc/short/${alias}`);
  });
  // check the filter icon exists
  existsInDOM(`#btn-${alias}`);
  // delete the
  cy.intercept('DELETE', '/arc/short').as('aliasDeleted');
  cy.get(`.existing-urls-control-${alias} .delete-alias`).click();
  cy.wait('@aliasDeleted');
  cy.get('#existing-short-urls');
  // check the alias doesn't exist after deletion
  existsInDOM('#existing-short-urls');
}

export function grantClipboardPermissions() {
  cy.wrap(
    Cypress.automation('remote:debugger:protocol', {
      command: 'Browser.grantPermissions',
      params: {
        permissions: ['clipboardReadWrite', 'clipboardSanitizedWrite'],
        origin: window.location.origin,
      },
    })
  );
}

export function checkDefaultCopyToURLButton() {
  grantClipboardPermissions();

  cy.get('#link-modal-link')
    .invoke('val')
    .should('not.be.empty');

  cy.get('#link-modal-link').then(($textarea) => {
    cy.get('.origin.url-copy-btn')
      .first()
      .click();

    const defaultURL = $textarea.val() as string;
    cy.log(`defaultURL='${defaultURL}'`);

    cy.window()
      .its('navigator.clipboard')
      .invoke('readText')
      .then((text) => cy.wrap(text).should('equal', defaultURL));
  });
}

export function checkGeneratedCopyToClipboard() {
  grantClipboardPermissions();

  cy.get('.short-url-slug')
    .invoke('val')
    .should('not.be.empty');

  cy.get('.short-url-slug').then(($input) => {
    cy.get('.short.url-copy-btn')
      .first()
      .click();

    const customURL = $input.val() as string;
    cy.log(`customURL='${customURL}'`);

    cy.window()
      .its('navigator.clipboard')
      .invoke('readText')
      .then((text) => cy.wrap(text).should('include', `/arc/short/${customURL}`));
  });
}

export function openURLModal() {
  cy.get('.viz-params-more')
    .eq(1)
    .click();
  cy.get('.viz-params-link')
    .eq(1)
    .click();
}

export function closeURLModal() {
  cy.get('#link-modal .modal-footer .btn.btn-secondary').click();
  cy.wait(500);
}
