/*
 * Verify that the SQL builder visualization works
 * Assumes the existence of the 'US State Populations Over Time' dataset
 *   in the Cypress.env("smokeConnectionName") data connection
 */
import * as arctest from 'lib/arctest';
import { refreshVisual } from 'lib/arctest/screen/data/modal';
import { createTableVisual } from 'lib/dashboard';
import * as functions from 'lib/functions';

import {
  addNewColorRage,
  enableBackgroundColorPropertiesOfField,
  getAggregateTitle,
  openColorSetting,
  selectTheBlackGradedColorPalette,
  testColorSettings,
  testFixedDimensionMapHasColorsFromActivePalette,
} from './utility';
const connectionName = Cypress.env('smokeConnectionName');
const databaseName = 'main';
const sourceTable = 'cereals';
const datasetName = 'Table color tests dataset';
describe('VIZ-1060', () => {
  before(() => {
    arctest.logins.silent();
    cy.visit('/arc/apps/home');
    functions.deleteDatasetFromConnection(connectionName, datasetName);
    functions.createDatasetFromConnection(connectionName, databaseName, sourceTable, datasetName);
  });

  it("Tests color of Table visual's dimension column's can be fixed and customized", () => {
    // GIVEN
    const dimensionShelfItem = 'cold_or_hot';
    createTableVisual(
      // TABLE
      'E2E custom color on table visual 3',
      connectionName,
      datasetName,
      {
        x_shelfs: [dimensionShelfItem],
        y_shelfs: ['protein_grams'],
      }
    );

    refreshVisual();

    // WHEN
    enableBackgroundColorPropertiesOfField(dimensionShelfItem, '#dimensions_shelf');
    openColorSetting();

    // THEN
    const defaultGradedBluePaletteIndex = 3;
    testColorSettings({ activeColorPaletteIndex: defaultGradedBluePaletteIndex, aggregate: false });
    testFixedDimensionMapHasColorsFromActivePalette();

    // WHEN
    selectTheBlackGradedColorPalette();
    testFixedDimensionMapHasColorsFromActivePalette();

    // The test checks the existence of the setting, and the user can use it
    // However the effect on the visual is not tested yet.
    // TODO: Visually compare the expected and actual table visual - see if the color change had any effect on the visual
  });

  it("Tests color of Table visual's measure column can be customized by range", () => {
    // GIVEN
    const dimensionShelfItem = 'cold_or_hot';
    const measureShelfItem = 'protein_grams';

    createTableVisual(
      // TABLE
      'E2E custom color on table visual 3',
      connectionName,
      datasetName,
      {
        x_shelfs: [dimensionShelfItem],
        y_shelfs: [measureShelfItem],
      }
    );

    refreshVisual();

    // WHEN
    enableBackgroundColorPropertiesOfField(
      getAggregateTitle(measureShelfItem),
      '#aggregates_shelf'
    );
    openColorSetting();

    // THEN
    const defaultGradedBluePaletteIndex = 3;
    testColorSettings({ activeColorPaletteIndex: defaultGradedBluePaletteIndex, aggregate: true });

    // WHEN
    selectTheBlackGradedColorPalette();
    addNewColorRage({ lower: undefined, upper: '50%' }, 1);
    addNewColorRage({ lower: '50%', upper: '101%' }, 2);

    // The test checks the existence of the setting, and the user can use it
    // However the effect on the visual is not tested yet.
    // TODO: Visually compare the expected and actual table visual - see if the color change had any effect on the visual
  });
});
