import { notifications } from 'fixtures/locators/notifications';
import { logins } from 'lib/arctest';
import { BackendAPI, DefaultRoleIds } from 'lib/backend-api';
import * as dashboards from 'lib/dashboard';
import * as functions from 'lib/functions';
import * as imp_exp from 'lib/import_export';
import * as visual from 'lib/visual';
import { CSSselector } from 'locators/common/CSSselector';

// rewrite the existing fuction because we simply need to change a workspace, nothing else.
function dbMoveToWS(workspaceName: string) {
  cy.get(
    `${CSSselector.dashboardDetail.menu} ${CSSselector.dashboardDetail.workspaceMenu}`
  ).click();
  cy.get(`${CSSselector.dashboardDetail.menu} [data-workspace-name="${workspaceName}"]`).click();
  cy.get(
    `${CSSselector.dashboardDetail.menu} ${CSSselector.dashboardDetail.workspaceMove}`
  ).click();
  cy.wait(2000);
  cy.get(notifications.alert_success).should('be.visible');
  cy.get(notifications.notification_close).click();
}

function makeLinkedVisual() {
  cy.get(CSSselector.visualEdit.ellipsis).click();
  cy.get(CSSselector.visualEdit.linkUnlink).click();
  cy.intercept('POST', 'arc/reports/report/*').as('visualSaved');
  cy.get(`${CSSselector.visualEdit.linkUnlinkConfirm}`).click();
  cy.wait('@visualSaved');

  dashboards.saveDashboard();
}

// skip because of VIZ-1423
describe.skip('ARC-10799: Customer Issue: Workspace name on linked visual displayed wrong ', () => {
  const connectionName = Cypress.env('smokeConnectionName');
  const datasetName = 'Cereals';
  const dashboardName = 'arc_10653_db';

  before(() => {
    logins.silent();
  });

  it('1 : Verify workspace of linked visual', () => {
    cy.fixture('users.json').then((data) => {
      BackendAPI.users.create({
        username: data.visualconsumer.username,
        password: data.visualconsumer.password,
        roleIds: [DefaultRoleIds.VisualConsumer],
      });
      functions.addNewDB(dashboardName);
      functions.createNewVisual(connectionName, datasetName);
      dashboards.saveDashboard();
      dbMoveToWS('Public');
      dashboards.editVisual();
      makeLinkedVisual();
      dashboards.viewDashboard();
      cy.get(visual.locators.full_size).click({ force: true });
      cy.get(visual.locators.workspace).should('have.text', ' Public');
      cy.get('.report-menu-list')
        .contains('Close')
        .click();
      logins.silent(data.visualconsumer.username, data.visualconsumer.password);
      dashboards.openDashboard(dashboardName);
      cy.get(visual.locators.full_size).click({ force: true });
      cy.get(visual.locators.workspace).should('have.text', ' Public');
      cy.get('.report-menu-list')
        .contains('Close')
        .click();
    });
  });

  it('2 : Verify workspace of unlinked visual', () => {
    dashboards.openDashboardforEdit(dashboardName);
    dashboards.editVisual();
    cy.get(CSSselector.visualEdit.ellipsis).click();
    cy.get(CSSselector.visualEdit.linkUnlink).click();
    cy.get(CSSselector.visualEdit.linkUnlinkConfirm).click();
    cy.get('div.workspace-visual-wrapper').should('be.not.visible');
    dashboards.closeVisual(false);
    dbMoveToWS('Private');
    cy.get('a.viz-workspace-wrapper').should('have.text', ' Private');
    dashboards.viewDashboard();
    cy.get(visual.locators.full_size).click({ force: true });
    cy.get('div.workspace-visual-wrapper').should('be.not.visible');
    cy.get('.report-menu-list')
      .contains('Close')
      .click();
  });

  it('3 : Verify workspace of Move/clone linked visuals', () => {
    dashboards.openDashboardforEdit(dashboardName);
    makeLinkedVisual();
    cy.intercept('POST', '/arc/reports/savedashboard').as('dashboardCloned');
    // Clone Db with maintain linked visual option
    cy.get(visual.locators.dash_ellipsis).click();
    cy.get(CSSselector.workspace_regression.Dashboard_clone).click();
    cy.get(CSSselector.dashboardDetail.workspaceMoveCloneModel).should('be.visible');
    cy.get('.radio-inline.clone-only-text').click();
    cy.get(CSSselector.dashboardDetail.workspaceMoveMoveNow).click();
    cy.wait('@dashboardCloned');
    dashboards.editVisual();
    cy.get('a.viz-workspace-wrapper').should('have.text', ' Private');
    cy.get(imp_exp.locators.linked_visual).should('have.text', ' Linked');
    dashboards.closeVisual(false);

    // Clone Db with clone visual option
    cy.get(visual.locators.dash_ellipsis).click();
    cy.get(CSSselector.workspace_regression.Dashboard_clone).click();
    cy.get(CSSselector.dashboardDetail.workspaceMoveCloneModel).should('be.visible');
    cy.get('.linked-visual-table .clone-option input').click();
    cy.get(CSSselector.dashboardDetail.workspaceMoveMoveNow).click();
    cy.wait('@dashboardCloned');
    dashboards.editVisual();
    cy.get('div.workspace-visual-wrapper').should('have.class', 'arc-invisible');
    dashboards.closeVisual(false);
  });
});
