import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';

const dashboardName = 'viz_784_grobar_visual_dashsboard';

const dashboard: Dashboard = {
  title: dashboardName,
  subTitle: 'viz_784_grobar_visual_dashsboard_subtitle',
  sheets: [
    {
      sheet_name: 'inspection sheet',
      sheet_number: 1,
      visuals: [
        {
          name: 'grobarvisual1',
          connection_name: 'samples',
          dataset_name: 'Restaurant Inspection SF',
          visual_type: '.chart-icon-trellis-groupedbars',
          visual_title: 'grobar_visual',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'inspection_date',
                field_shelf: ShelfTypes.X,
              },
              {
                field_name: 'inspection_score',
                field_shelf: ShelfTypes.Y,
                field_aggregate: 'Average',
              },
              {
                field_name: 'inspection_type',
                field_shelf: ShelfTypes.COLOR,
              },
            ],
          },
        },
      ],
    },
  ],
};

describe("VIZ-784: Customer Issue: Grouped Bar chart doesn't show Display formats properly on x-axis", () => {
  before(() => {
    cy.fixture('users.json').then((data) => {
      logins.silent(data.admin.username, data.admin.password);
    });
  });

  it('Test 1: Verify application of display formats on x-axis if we add any dimension to the Color shelf in grouped bar visuals', () => {
    dashboards.createDashboard(dashboard);
    dashboards.openDashboardforEdit(dashboardName);
    dashboards.editVisual();

    dashboards.chooseVisualTab('data');
    // Template issue.
    // Used "parent child" selector to differentiate the one we need and the one in template
    cy.get('[data-test-id="visual-shelves-panel"] [data-test-id="x_shelf_list"]')
      .find('.dropdown-toggle')
      .click();
    cy.get('[data-test-id="properties-panel"]')
      .find('[data-test-id="date-functions"]')
      .click();
    cy.get('[data-test-id="properties-panel"]')
      .find('[data-test-id="shelf-fn-month"]')
      .click();
    cy.get('[data-test-id="viz-refresh"]').click({ force: true });
    // Template issue.
    // Used "parent child" selector to differentiate the one we need and the one in template
    cy.get('[data-test-id="visual-shelves-panel"] [data-test-id="x_shelf_list"]')
      .find('.dropdown-toggle')
      .click();
    cy.get('[data-test-id="properties-panel"]')
      .find('[data-test-id="display-format-link"]')
      .click();
    cy.get('[data-test-id="display-format-type"]').select('date');
    cy.get('[data-test-id="custom-display-format-dateformat"]')
      .clear()
      .type('MMM');
    cy.get('#display-format-form')
      .find('[data-test-id="display-format-save"]')
      .click();

    cy.get('[data-test-id="visual-list-item"]:nth-child(1)')
      .find('text')
      .contains('Jan');
  });
});
