import * as dashboards from 'lib/dashboard';
import {
  Dashboard,
  ShelfTypes,
  SortOptions,
  VisualType,
} from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';
import * as logins from 'lib/logins';
import * as combo_functions from 'lib/visual/visual_bar_line_combo_area_group';
import * as visual_common_test from 'lib/visual/visual_common_tests';

const connectionName = 'samples';
const datasetName1 = 'Cereals';
const dashboardName = 'Area Visual Dashboard';

const dashboard1: Dashboard = {
  title: dashboardName,
  subTitle: 'simple_area_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'sheet1',
      sheet_number: 1,
      visuals: [
        {
          name: 'areavisual1',
          connection_name: connectionName,
          dataset_name: datasetName1,
          visual_type: VisualType.area,
          visual_title: 'area visual',
          linked: 'false',
          // prettier-ignore
          is: ['setTransform', 'translate', 'globalAlpha', 'lineWidth', 'font', 'clearRect', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill'],
          hash: '8739d8c2b18d8a246c0bd95de5f02ae5',
          data: {
            fields: [
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.X,
              },
              {
                field_name: 'manufacturer',
                field_shelf: ShelfTypes.X,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.Y,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.COLOR,
              },
              {
                field_name: 'cold_or_hot',
                field_shelf: ShelfTypes.TOOLTIP,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'manufacturer',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['American Home Food Products', 'Ralston Purina'],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};

let sourceDashboardID = '';
let clonedDashboardId = '';

/**
 * Create a clone used by a single test case.
 *
 * @param originalDashboardName
 * @param clonedDashboardName
 */
function cloneAndOpenVisual(originalDashboardName: string, clonedDashboardName: string) {
  const useFast = true;
  if (useFast) {
    // we execute a bit faster if we do not rename the clone
    cloneAndOpenVisual_fast();
  } else {
    cloneAndOpenVisual_UI(originalDashboardName, clonedDashboardName);
  }
}

/**
 * A (hopefully) more performant cloning method.
 * Does not search dashboards by name, but uses the IDs stored in global variables.
 *
 * It stores the clone result and deletes it on the next run
 *
 * @param clonedDashboardName
 */
function cloneAndOpenVisual_fast(clonedDashboardName: string | null = null) {
  if (clonedDashboardId) {
    // delete previous clone
    deleteDashboard(clonedDashboardId).then(() => {
      clonedDashboardId = '';
    });
  }

  dashboards.editDashboardById(sourceDashboardID);

  dashboards.dbClone().then((dashboardId) => {
    clonedDashboardId = dashboardId;
  });

  if (clonedDashboardName) {
    cy.get('[data-test-id="dashboard-title"]:visible')
      .clear()
      .type(clonedDashboardName);
    dashboards.saveDashboard();
  }

  // the stock editVisual function seems to cause issues with the side bar
  const useStockEditVisual = false;
  if (useStockEditVisual) {
    dashboards.editVisual();
  } else {
    editVisual();
  }
}

/**
 * Unlike dashboards.editVisual this method does not use force clicking.
 */
function editVisual() {
  cy.get('[data-test-id="visual-list-item"]:nth-child(1):visible').click();
}

/**
 * Delete dashboard using direct request.
 * @param dashboardId
 * @returns
 */
function deleteDashboard(dashboardId: string) {
  return cy
    .window()
    .its('arc.frontend.globals.authentication.csrfToken')
    .then((csrftoken) => {
      return cy.request({
        followRedirect: false,
        form: true,
        log: true,
        method: 'DELETE',
        url: `/arc/reports/report/${dashboardId}`,
        headers: {
          'X-Csrftoken': csrftoken,
          Origin: Cypress.config().baseUrl,
        },
      });
    });
}

/**
 * UI-based cloning. Slow.
 * @param originalDashboardName
 * @param clonedDashboardName
 */
function cloneAndOpenVisual_UI(originalDashboardName: string, clonedDashboardName: string) {
  dashboards.cloneDashboard(originalDashboardName, clonedDashboardName);
  dashboards.openDashboardforEdit(clonedDashboardName);

  dashboards.editVisual();
}

describe('Arcviz Area visuals tests.', () => {
  context('Area visual tests settings tab tests', () => {
    const cloneDashboardName = 'area_settings_dashboard';

    before(() => {
      logins.silent();

      if (!sourceDashboardID) {
        dashboards.createDashboard(dashboard1).then((dashboard1Id) => {
          sourceDashboardID = dashboard1Id;
        });
      }
    });

    it('Test 2: Set Download as CSV locale', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.enableDisableCSVDownload();
    });

    it('Test 3: Enable Animation', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.enableDisableAnimation();
    });

    it('Test 4: Enable/Disable Drill into context menu', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      visual_common_test.enableDrillIntoFunctionality();
      visual_common_test.testDrillIntoOptionIsAvailableInContextMenu();
    });

    it('Test 5: Display trellis borders', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.displayTrelisBorder();
    });

    it('Test 6: Independent dimension scale', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      dashboards.chooseVisualTab('style');
      combo_functions.setinpdDimScaleTrellis();
    });

    it('Test 7: Independent aggregate scale', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      dashboards.chooseVisualTab('style');
      combo_functions.setinpdAggrScaleTrellis();
    });

    it.skip('Test 8: Display tooltips', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      // getting the tooltip is not working
      combo_functions.setEnableTooltipsSetting(cloneDashboardName, 'checked');
      dashboards.editVisual();
      combo_functions.setEnableTooltipsSetting(cloneDashboardName, 'unchecked');
    });

    it.skip('Test 9: Show detailed data button', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      // getting the context menu is not working
    });

    it('Test 11: Legend style', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      visual_common_test.testLegendPositionChangeBySwitchingTo('none');
      visual_common_test.testLegendPositionChangeBySwitchingTo('right');
      visual_common_test.testLegendPositionChangeBySwitchingTo('bottom');
      visual_common_test.testLegendPositionChangeBySwitchingTo('above');
    });

    it('Test 14: Display linear trendline', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setlineraTrendline();
    });

    it('Test 16: Show selection brush', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setSelectionBrush();
    });

    it('Test 17: Show measure values next to points', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      visual_common_test.testShowMeasureValuesSetting();
    });

    it('Test 19: Ignore negative measure values', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setIgnoreNegativeData();
    });

    it('Test 20: Show data with null values', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setShowNullValues('Default');
      dashboards.editVisual();
      combo_functions.setShowNullValues('First');
      dashboards.editVisual();
      combo_functions.setShowNullValues('Last');
    });

    it('Test 21: Number of rows to download in CSV', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setNumberofRowsinCSV('75');
    });

    it('Test 22: Maximum number of rows to fetch', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setNumberofRowsToFetch('10');
    });

    it('Test 24: Auto detect time columns', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setAutoDetectTimeColumn();
    });

    it('Test 25: Axis scale log10', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setAxisScale('LOG10');
    });

    it('Test 26: Axis scale Linear', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setAxisScale('LINEAR');
    });

    it('Test 27: Axis scale percentage of dimension', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setAxisScale('PERCENTAGE');
    });

    it('Test 28: Axis scale percentage of color', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setAxisScale('PERCENTAGE_COLOR');
    });

    it('Test 29: Gridlines enabled', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setGridLines();
    });

    it('Test 30: Positive aggregate scale start at 0', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setAggrScaleStart();
    });

    it('Test 31: Aggregate scale minimum', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setMinAggrScale();
    });

    it('Test 32: Aggregate scale maximum', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setMaxAggrScale();
    });

    it('Test 33: Include X-Axis label', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setIncludeXAxisLabel();
    });

    it('Test 34: Include Y-Axis label', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setIncludeYAxisLabel();
    });

    it.skip('Test 35: Maximum tick length', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
    });

    it('Test 36: Show X-Axis', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setShowXAXis();
    });

    it('Test 37: Show Y-Axis ', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setShowYAXis();
    });
  });
});
