import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';
import { CSSselector } from 'locators';

const connectionName = 'samples';
const datasetName = 'Global Information Security Threats';

const dashboard: Dashboard = {
  title: 'Stacks should follow the priority and ascending prop of color fields',
  subTitle: 'Dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'Sheet 1',
      sheet_number: 1,
      visuals: [
        {
          name: 'Stacked bars sorting',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-trellis-bars',
          visual_title: 'Stacked bars sorting, THREAT is ascending, YEAR is ascending',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'industry',
                field_shelf: ShelfTypes.X,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'count',
                field_shelf: ShelfTypes.Y,
              },
              {
                field_name: 'threat',
                field_shelf: ShelfTypes.COLOR,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'year',
                field_shelf: ShelfTypes.COLOR,
                sort: SortOptions.ASCENDING,
                field_expression: '[year]',
              },
              {
                field_name: 'industry',
                field_shelf: ShelfTypes.FILTERS,
                field_expression:
                  "[industry] in ('Accounting', 'Auto', 'Banking', 'Communications')",
              },
              {
                field_name: 'threat',
                field_shelf: ShelfTypes.FILTERS,
                field_expression: "[threat] in ('botnet', 'malicious_ip', 'phishing', 'scanning')",
              },
              {
                field_name: 'threat',
                field_shelf: ShelfTypes.LABEL,
                field_expression: "substr([threat], 0 , 3) || '-' || substr([year], 3, 4)",
              },
            ],
          },
        },
        {
          name: 'Stacked bars sorting',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-trellis-bars',
          visual_title: 'Stacked bars sorting, THREAT is ascending, YEAR is descending',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'industry',
                field_shelf: ShelfTypes.X,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'count',
                field_shelf: ShelfTypes.Y,
              },
              {
                field_name: 'threat',
                field_shelf: ShelfTypes.COLOR,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'year',
                field_shelf: ShelfTypes.COLOR,
                sort: SortOptions.DESCENDING,
                field_expression: '[year]',
              },
              {
                field_name: 'industry',
                field_shelf: ShelfTypes.FILTERS,
                field_expression:
                  "[industry] in ('Accounting', 'Auto', 'Banking', 'Communications')",
              },
              {
                field_name: 'threat',
                field_shelf: ShelfTypes.FILTERS,
                field_expression: "[threat] in ('botnet', 'malicious_ip', 'phishing', 'scanning')",
              },
              {
                field_name: 'threat',
                field_shelf: ShelfTypes.LABEL,
                field_expression: "substr([threat], 0 , 3) || '-' || substr([year], 3, 4)",
              },
            ],
          },
        },
        {
          name: 'Stacked bars sorting',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-trellis-bars',
          visual_title: 'Stacked bars sorting, THREAT is descending, YEAR is ascending',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'industry',
                field_shelf: ShelfTypes.X,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'count',
                field_shelf: ShelfTypes.Y,
              },
              {
                field_name: 'threat',
                field_shelf: ShelfTypes.COLOR,
                sort: SortOptions.DESCENDING,
              },
              {
                field_name: 'year',
                field_shelf: ShelfTypes.COLOR,
                sort: SortOptions.ASCENDING,
                field_expression: '[year]',
              },
              {
                field_name: 'industry',
                field_shelf: ShelfTypes.FILTERS,
                field_expression:
                  "[industry] in ('Accounting', 'Auto', 'Banking', 'Communications')",
              },
              {
                field_name: 'threat',
                field_shelf: ShelfTypes.FILTERS,
                field_expression: "[threat] in ('botnet', 'malicious_ip', 'phishing', 'scanning')",
              },
              {
                field_name: 'threat',
                field_shelf: ShelfTypes.LABEL,
                field_expression: "substr([threat], 0 , 3) || '-' || substr([year], 3, 4)",
              },
            ],
          },
        },
      ],
    },
  ],
};

describe("On the stacked bar visual the stack should reflect the color priority and it's it is ascending prop", () => {
  before(() => {
    logins.silent();
    cy.visit('/arc/apps/home');
  });
  afterEach(() => {
    cy.window().then(
      (win: Window) =>
        (win.onbeforeunload = () => {
          return;
        })
    );
  });

  context('Stacked Bar visual tests', () => {
    // In case multiple color field is present and they are ordered,
    // the order should be reflected layer by layer like in table visual.
    // For example: color by year(1th) and threat(2nd). They should displayed in a grouped manner
    //  Asc-Asc     Asc-Desc   Desc-Asc
    //  Y1 - T1     Y1 - T3    Y2 - T3
    //  Y1 - T2     Y1 - T2    Y2 - T2
    //  Y1 - T3     Y1 - T1    Y2 - T1
    //  Y2 - T1     Y2 - T3    Y1 - T3
    //  Y2 - T2     Y2 - T2    Y1 - T2
    //  Y2 - T3     Y2 - T1    Y1 - T1
    it('Test 1: Create stacked graph visuals, with multiple color field, and check stack order is correct', () => {
      // The test scenarios are actually the visuals definition in the dashboard Object.
      // Their title describes what should be seen in the images
      dashboards.createDashboard(dashboard);
      cy.get(CSSselector.close_dashboard_setting).click();
      cy.get(
        `${CSSselector.dashboardDetail.dashboardContainer} [data-test-id="visual-list-item"]:nth-of-type(1)`
      )
        .scrollIntoView()
        .matchImage();
      cy.get(
        `${CSSselector.dashboardDetail.dashboardContainer} [data-test-id="visual-list-item"]:nth-of-type(2)`
      )
        .scrollIntoView()
        .matchImage();
      cy.get(
        `${CSSselector.dashboardDetail.dashboardContainer} [data-test-id="visual-list-item"]:nth-of-type(3)`
      )
        .scrollIntoView()
        .matchImage();
    });
  });
});
