import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';

const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'Cereals';

const dashboard: Dashboard = {
  title: 'simple_barline_visual_dashboard',
  subTitle: 'simple_barline_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'calorie sheet',
      sheet_number: 1,
      visuals: [
        {
          name: 'barlinevisual1',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-combo',
          visual_title: 'simple_barline_visual',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.X,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.BAR_MEASURE,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'sugars_grams',
                field_shelf: ShelfTypes.BAR_MEASURE,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'cups_per_serving',
                field_shelf: ShelfTypes.TOOLTIP,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['Cheerios', 'Cocoa_Puffs'],
                },
              },
            ],
          },
        },
        {
          name: 'barlinevisual2',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-combo',
          visual_title: 'simple_barline_visual',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.X,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.BAR_MEASURE,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Average',
              },
              {
                field_name: 'sugars_grams',
                field_shelf: ShelfTypes.LINE_MEASURE,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Maximum',
              },
              {
                field_name: 'sugars_grams',
                field_shelf: ShelfTypes.LINE_MEASURE,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Maximum',
              },
              {
                field_name: 'fat_grams',
                field_shelf: ShelfTypes.LINE_MEASURE,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'fat_grams',
                field_shelf: ShelfTypes.HIGHLIGHTER_MEASURE,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'cups_per_serving',
                field_shelf: ShelfTypes.TOOLTIP,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['Cheerios', 'Cocoa_Puffs'],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Arcviz Bar Line combo visuals test suite.', () => {
  before(() => {
    logins.silent();
    cy.visit('/arc/apps/home');
  });

  context('Bar Line graph visual tests', () => {
    it('Test 1: Create basic bar line graph visuals', () => {
      dashboards.createDashboard(dashboard);
    });
  });
});
