import * as dashboards from 'lib/dashboard';
import { getVersion } from 'lib/functions';
import * as logins from 'lib/logins';
import * as visuals from 'lib/visual';
import * as combo_functions from 'lib/visual/visual_bar_line_combo_area_group';

const datasetName2 = 'Iris';

const testDashbboardName = 'E2e Bar visual test dashboard';

describe('Arcviz Bar visuals test suite.', () => {
  context('Bar graph visual tests data tab tests', () => {
    before(() => {
      logins.silent();
    });

    beforeEach(() => {
      dashboards.deleteDashboard(testDashbboardName);
      dashboards.createBarVisual(testDashbboardName);
      dashboards.chooseVisualTab('data');
    });

    it('Test 1: A visualunderlying dataset can be edited via the (pencil) edit button', () => {
      // test opens dataset in separate tab, which cypress cannot handle
      const datasetEditSelector = '#app-dataset-edit .fa-pencil';

      getVersion().then((versionNumber) => {
        const vNum = Number.parseInt(versionNumber, 10);
        // version 7 does not need two clicks, 6 does
        if (vNum < 7) {
          cy.get(datasetEditSelector).click();
        }
      });

      cy.window().then((win) => {
        cy.stub(win, 'open').as('open');
      });
      cy.get(datasetEditSelector).click();
      cy.get<{ getCall: (index: number) => { args: string[] } }>('@open').should((subject) => {
        const [url, arg] = subject.getCall(0).args;
        expect(String(url)).to.contain('arc/apps/dataset/');
        expect(arg).to.equal('_blank');
      });
      visuals.closeVisual(false);
    });

    it('Test 2: a visual dataset can be switched to another one', () => {
      combo_functions.switchDataSet(datasetName2);
      // visuals.closeVisual(false);
      dashboards.editVisual();
      cy.get('[data-test-id="app-dataset"]').should('have.text', datasetName2);
      visuals.closeVisual(false);
    });

    it('Test 3: Searching fields on data panel (dimesion and measures)', () => {
      const dataPanelLocator = '[data-test-id="visual-data-panel"]';
      const searchInputLocator = `${dataPanelLocator} input`;
      const fieldsLocator = '.field-list-wrapper li';
      const dimensionsListLocator = `${dataPanelLocator} [data-test-id="dim-app-params"] ${fieldsLocator}`;
      const measuresListLocator = `${dataPanelLocator} [data-test-id="agg-app-params"] ${fieldsLocator}`;
      cy.get(searchInputLocator).type('cereal_name');
      cy.get(dimensionsListLocator).should('have.length', 1);
      cy.get(searchInputLocator).clear();
      cy.get(searchInputLocator).type('calories');
      cy.get(measuresListLocator).should('have.length', 1);
      visuals.closeVisual(false);
    });
  });
});
