import * as dashboards from 'lib/dashboard';
import * as logins from 'lib/logins';
import * as combo_functions from 'lib/visual/visual_bar_line_combo_area_group';
import * as visual_common_test from 'lib/visual/visual_common_tests';

const testDashbboardName = 'bar_settings_dashboard';

function createVisualWithTrellisAndOpenStylSetting() {
  dashboards.createBarVisual(testDashbboardName, 'samples', 'Cereals', {
    x_shelfs: ['manufacturer'],
    y_shelfs: ['protein_grams', 'sodium_mg'],
    color_shelfs: ['fat_grams', 'sugars_grams'],
  });
  dashboards.editVisual();
  dashboards.chooseVisualTab('style');
}

function createSampleBarVisualAndOpenStylSetting() {
  dashboards.createBarVisual(testDashbboardName);
  dashboards.editVisual();
  dashboards.chooseVisualTab('style');
}

describe('Arcviz Bar visuals test suite.', () => {
  context('Bar graph visual tests settings tab tests', () => {
    before(() => {
      logins.silent();
      cy.visit('/arc/apps/home');
    });

    beforeEach(() => {
      dashboards.deleteDashboard(testDashbboardName);
    });

    it('Test 2: Set Download as CSV locale', () => {
      createSampleBarVisualAndOpenStylSetting();
      combo_functions.enableDisableCSVDownload();
    });

    it('Test 3: Enable Animation', () => {
      createSampleBarVisualAndOpenStylSetting();
      combo_functions.enableDisableAnimation();
    });

    it('Test 4: Enable/Disable Drill into context menu', () => {
      createSampleBarVisualAndOpenStylSetting();
      visual_common_test.enableDrillIntoFunctionality();
      visual_common_test.testDrillIntoOptionIsAvailableInContextMenu();
    });

    it('Test 5: Display trellis borders', () => {
      createVisualWithTrellisAndOpenStylSetting();
      combo_functions.displayTrelisBorder();
    });

    it('Test 6: Independent dimension scale', () => {
      createVisualWithTrellisAndOpenStylSetting();
      combo_functions.setinpdDimScaleTrellis();
    });

    it('Test 7.a: Enable Independent aggregate scale if bar visual has trellis', () => {
      createVisualWithTrellisAndOpenStylSetting();
      combo_functions.setinpdAggrScaleTrellis();
    });

    it('Test 6 & 7 disabled scenario: Trellis cell dimension, and border setting, Axes`s dimension settings should be disabled if no trellis present', () => {
      createSampleBarVisualAndOpenStylSetting();

      const settingsShouldbeDisabled = [
        {
          settingGroup: 'Trellis',
          settinglocators: [
            '[data-setting-id="trellis_borders"]',
            '[data_setting_id="trellis_cell_minimum_height"]',
            '[data_setting_id="trellis_cell_minimum_width"]',
          ],
        },
        {
          settingGroup: 'Axes',
          settinglocators: [
            '[data-setting-id="scale_dim_independent"]',
            '[data-setting-id="scale_agg_independent"]',
            '[data-setting-id="dim_is_continuous"]',
            '[data-setting-id="dim_is_continuous"]',
            '[data-setting-id="max_dim_axis"]',
            '[data-setting-id="max_dim_axis"]',
          ],
        },
      ];
      combo_functions.verifySettingsAreDisabled(settingsShouldbeDisabled);
    });

    it('Test 8: Enable display tooltips', () => {
      // getting the tooltip is not working
      createSampleBarVisualAndOpenStylSetting();
      combo_functions.setEnableTooltipsSetting(testDashbboardName, 'checked');
      dashboards.editVisual();
      combo_functions.setEnableTooltipsSetting(testDashbboardName, 'unchecked');
    });

    it.skip('Test 9: Show detailed data button', () => {
      // getting the context menu is not working
    });

    // Skip because setBarSizeRange doesn't work (and I couldn't fix it)
    it.skip('Test 10: Set bar size range', () => {
      createSampleBarVisualAndOpenStylSetting();
      // combo_functions.setBarSizeRange('2-10');
    });

    it('Test 11: Legend style', () => {
      createSampleBarVisualAndOpenStylSetting();
      visual_common_test.testLegendPositionChangeBySwitchingTo('none');
      visual_common_test.testLegendPositionChangeBySwitchingTo('right');
      visual_common_test.testLegendPositionChangeBySwitchingTo('bottom');
      visual_common_test.testLegendPositionChangeBySwitchingTo('above');
    });

    it('Test 12: Legend item width', () => {
      createSampleBarVisualAndOpenStylSetting();
      combo_functions.setLegendWidth(testDashbboardName, '100');
    });

    it('Test 13: Reverse legend order', () => {
      createSampleBarVisualAndOpenStylSetting();
      combo_functions.reverseLegendOrder();
    });

    it('Test 14: Display linear trendline', () => {
      createSampleBarVisualAndOpenStylSetting();
      combo_functions.setlineraTrendline();
    });

    it('Test 15: Trendline decimals', () => {
      // slider not working
    });

    it('Test 16: Show selection brush', () => {
      createSampleBarVisualAndOpenStylSetting();
      combo_functions.setSelectionBrush();
    });

    it('Test 17: Show measure values inside bars', () => {
      createSampleBarVisualAndOpenStylSetting();
      visual_common_test.testShowMeasureValuesSetting();
    });

    it('Test 18: Minimum cell height and width', () => {
      createSampleBarVisualAndOpenStylSetting();
      visual_common_test.testCellHeightWidthSetting();
    });

    it('Test 19: Ignore negative measure values', () => {
      createSampleBarVisualAndOpenStylSetting();
      combo_functions.setIgnoreNegativeData();
    });

    it('Test 20: Show data with null values', () => {
      createSampleBarVisualAndOpenStylSetting();
      combo_functions.setShowNullValues('First');
      dashboards.editVisual();
      combo_functions.setShowNullValues('Default');
      dashboards.editVisual();
      combo_functions.setShowNullValues('Last');
    });

    it('Test 21: Number of rows to download in CSV', () => {
      createSampleBarVisualAndOpenStylSetting();
      combo_functions.setNumberofRowsinCSV('75');
    });

    it('Test 22: Maximum number of rows to fetch', () => {
      createSampleBarVisualAndOpenStylSetting();
      combo_functions.setNumberofRowsToFetch('10');
    });

    it('Test 23: Disable incremental queries', () => {
      createSampleBarVisualAndOpenStylSetting();
      combo_functions.setDisableIncrementalQueries();
    });

    it('Test 24: Auto detect time columns', () => {
      createSampleBarVisualAndOpenStylSetting();
      combo_functions.setAutoDetectTimeColumn();
    });

    it('Test 25: Axis scale log10', () => {
      createSampleBarVisualAndOpenStylSetting();
      combo_functions.setAxisScale('LOG10');
    });

    it('Test 26: Axis scale Linear', () => {
      createSampleBarVisualAndOpenStylSetting();
      combo_functions.setAxisScale('LINEAR');
    });

    it('Test 27: Axis scale percentage of dimension', () => {
      createSampleBarVisualAndOpenStylSetting();
      combo_functions.setAxisScale('PERCENTAGE');
    });

    it('Test 28: Axis scale percentage of color', () => {
      createSampleBarVisualAndOpenStylSetting();
      combo_functions.setAxisScale('PERCENTAGE_COLOR');
    });

    it('Test 29: Gridlines enabled', () => {
      createSampleBarVisualAndOpenStylSetting();
      combo_functions.setGridLines();
    });

    it('Test 30: Positive aggregate scale start at 0', () => {
      createSampleBarVisualAndOpenStylSetting();
      combo_functions.setAggrScaleStart();
    });

    it('Test 31: Aggregate scale minimum', () => {
      createSampleBarVisualAndOpenStylSetting();
      combo_functions.setMinAggrScale();
    });

    it('Test 32: Aggregate scale maximum', () => {
      createSampleBarVisualAndOpenStylSetting();
      combo_functions.setMaxAggrScale();
    });

    it('Test 33: Include X-Axis label', () => {
      createSampleBarVisualAndOpenStylSetting();
      combo_functions.setIncludeXAxisLabel();
    });

    it('Test 34: Include Y-Axis label', () => {
      createSampleBarVisualAndOpenStylSetting();
      combo_functions.setIncludeYAxisLabel();
    });

    it('Test 36: Show X-Axis', () => {
      createSampleBarVisualAndOpenStylSetting();
      combo_functions.setShowXAXis();
    });

    it('Test 37: Show Y-Axis ', () => {
      createSampleBarVisualAndOpenStylSetting();
      combo_functions.setShowYAXis();
    });

    it('Test 38: Dynamically size graph to window size', () => {
      createSampleBarVisualAndOpenStylSetting();
      combo_functions.dynamicallyResizeGraph();
    });
  });
});
