import * as dashboards from 'lib/dashboard';
import * as logins from 'lib/logins';

describe('Arcviz Bar visuals test suite.', () => {
  context('Bar graph visual tests style tab tests', () => {
    const testDashbboardName = 'bar_style_dashboard';
    before(() => {
      logins.silent();
    });

    beforeEach(() => {
      // visit is done in the cloneDashboard call
      dashboards.deleteDashboard(testDashbboardName);
      dashboards.createBarVisual(testDashbboardName);
      dashboards.editVisual();
      dashboards.chooseVisualTab('style');
    });

    // Partial html issue.
    // Used "parent child" selector in the following 4 lines to differentiate style tab in dashboard and visual style tabs.
    // Style tab is created from a partial html.
    const visualBasicSelector =
      '[data-test-id="dash-visual-style-tab"] [data-test-id="visual-style-tab-Visual Basics"]';
    const visualTitleSelector =
      '[data-test-id="dash-visual-style-tab"] [data-test-id="visual-style-tab-Visual Title"]';
    const visualSubtitleSelector =
      '[data-test-id="dash-visual-style-tab"] [data-test-id="visual-style-tab-Visual Subtitle"]';
    const visualTooltipStyleSelector =
      '[data-test-id="dash-visual-style-tab"] [data-test-id="visual-style-tab-Tooltip"]';
    const dropDownSelector = '[data-test-id="dropdown-btn"]';

    it('Test 1: Visual basic font type', () => {
      const visualBasicFontTypeSelector = '[data-test-id="setting-id-visual_font"]';
      const testName = 'setVisualBasicFontType';
      console.log('Starting test ' + testName);
      cy.get(visualBasicSelector).within(() => {
        cy.get(visualBasicFontTypeSelector).within(() => {
          cy.get(dropDownSelector).click();
          cy.get('[data-test-id="value-arc-font-comic-sans-ms"]').click();
        });
      });
      dashboards.closeVisual(false);
      dashboards.editVisual();
      dashboards.chooseVisualTab('style');
      cy.get(visualBasicSelector).within(() => {
        cy.get(visualBasicFontTypeSelector).within(() => {
          cy.get(dropDownSelector).should('have.attr', 'data-value', 'arc-font-comic-sans-ms');
        });
      });
      dashboards.closeVisual(false);
    });

    it('Test 2: Visual basic font size', () => {
      const visualBasicFontSizeSelector = '[data-test-id="setting-id-visual_font_size"]';
      const testName = 'setVisualBasicFontSize';
      console.log('Starting test ' + testName);
      cy.get(visualBasicSelector).within(() => {
        cy.get(visualBasicFontSizeSelector).within(() => {
          cy.get('[data-test-id="TITLE_FONT_SIZE"]').clear();
          cy.get('[data-test-id="TITLE_FONT_SIZE"]').type('20{enter}');
        });
      });
      dashboards.closeVisual(false);
      dashboards.openDashboardforEdit(testDashbboardName);
      dashboards.editVisual();
      dashboards.chooseVisualTab('style');
      cy.get(visualBasicSelector).within(() => {
        cy.get(visualBasicFontSizeSelector).within(() => {
          cy.get('[data-test-id="TITLE_FONT_SIZE"]').should('have.attr', 'value', '20');
        });
      });
      dashboards.closeVisual(false);
    });

    it('Test 3: Visual title font type', () => {
      const visualTitleFontSelector = '[data-test-id="setting-id-title_font"]';
      const testName = 'setVisualTitleFontType';
      console.log('Starting test ' + testName);
      // set the visual title font to be Comic Sans MS
      cy.get(visualTitleSelector).within(() => {
        cy.get(visualTitleFontSelector).within(() => {
          cy.get(dropDownSelector).click();
          cy.get('[data-test-id="value-arc-font-comic-sans-ms"]').click();
        });
      });
      // save close and reopen the visual
      dashboards.closeVisual(false);
      dashboards.editVisual();
      dashboards.chooseVisualTab('style');
      // verify the font is set right
      cy.get(visualTitleSelector).within(() => {
        cy.get(visualTitleFontSelector).within(() => {
          cy.get(dropDownSelector).should('have.attr', 'data-value', 'arc-font-comic-sans-ms');
        });
      });
      dashboards.closeVisual(false);
    });

    it('Test 4: Visual title font size', () => {
      const visualTitleFontSizeSelector = '[data-test-id="setting-id-title_font_size"]';
      const testName = 'setVisualTitleFontSize';
      console.log('Starting test ' + testName);
      // set the visual title font to be Comic Sans MS
      cy.get(visualTitleSelector).within(() => {
        cy.get(visualTitleFontSizeSelector).within(() => {
          cy.get('[data-test-id="TITLE_FONT_SIZE"]').clear();
          cy.get('[data-test-id="TITLE_FONT_SIZE"]').type('40{enter}');
        });
      });
      // save close and reopen the visual
      dashboards.closeVisual(false);
      dashboards.openDashboardforEdit(testDashbboardName);
      dashboards.editVisual();
      dashboards.chooseVisualTab('style');
      // dashboards.openDashboardforEdit(dashboardName);
      // editVisual();
      // verify the font is set right
      cy.get(visualTitleSelector).within(() => {
        cy.get(visualTitleFontSizeSelector).within(() => {
          cy.get('[data-test-id="TITLE_FONT_SIZE"]').should('have.attr', 'value', '40');
        });
      });
      dashboards.closeVisual(false);
    });

    it('Test 5: Visual title font weight', () => {
      const visualTitleFontWeightSelector = '[data-test-id="setting-id-visual_title_font_weight"]';
      const testName = 'setVisualTitleFontWeight';
      console.log('Starting test ' + testName);
      // set the visual title font to be Comic Sans MS
      cy.get(visualTitleSelector).within(() => {
        cy.get(visualTitleFontWeightSelector).within(() => {
          cy.get(dropDownSelector).click();
          cy.get('[data-test-id="value-bold"]').click();
        });
      });
      // save close and reopen the visual
      dashboards.closeVisual(false);
      dashboards.editVisual();
      dashboards.chooseVisualTab('style');
      // verify the font is set right
      cy.get(visualTitleSelector).within(() => {
        cy.get(visualTitleFontWeightSelector).within(() => {
          cy.get(dropDownSelector).should('have.attr', 'data-value', 'bold');
        });
      });
      dashboards.closeVisual(false);
    });

    it('Test 6: Visual title alignment', () => {
      const visualTitleFontAlignmentSelector = '[data-test-id="title_alignment"]';
      const testName = 'setVisualTitleFontAlignment';
      console.log('Starting test ' + testName);
      // set the visual title font to be Comic Sans MS
      cy.get(visualTitleSelector).within(() => {
        cy.get(visualTitleFontAlignmentSelector).within(() => {
          cy.get('[data-test-id="radio-Center"]').click();
        });
      });
      // save close and reopen the visual
      dashboards.closeVisual(false);
      dashboards.openDashboardforEdit(testDashbboardName);
      dashboards.editVisual();
      dashboards.chooseVisualTab('style');
      // verify the font is set right
      cy.get(visualTitleSelector).within(() => {
        cy.get(visualTitleFontAlignmentSelector).within(() => {
          cy.get('[data-test-id="radio-Center"]').should('be.checked');
        });
      });
      dashboards.closeVisual(false);
    });

    it('Test 7: Visual title hide in view mode', () => {
      const visualTitleHideSelector = '[data-test-id="setting-id-visual_hide_title"]';
      const testName = 'setVisualTitleHide';
      console.log('Starting test ' + testName);
      // set the visual title font to be Comic Sans MS
      cy.get(visualTitleSelector).within(() => {
        cy.get(visualTitleHideSelector).within(() => {
          cy.get('[data-test-id="checkbox-VISUAL_HIDE_TITLE"]').click();
        });
      });
      // save close and reopen the visual
      dashboards.closeVisual(false);
      dashboards.openDashboardforEdit(testDashbboardName);
      dashboards.editVisual();
      dashboards.chooseVisualTab('style');
      // verify the font is set right
      cy.get(visualTitleSelector).within(() => {
        cy.get(visualTitleHideSelector).within(() => {
          cy.get('[data-test-id="checkbox-VISUAL_HIDE_TITLE"]').should('be.checked');
        });
      });
      dashboards.closeVisual(false);
    });

    it('Test 8: Visual subtitle font', () => {
      const visualSubtitleFontSelector = '[data-test-id="setting-id-subtitle_font"]';
      const testName = 'setVisualSubtitleFontType';
      console.log('Starting test ' + testName);
      cy.get(visualSubtitleSelector).within(() => {
        cy.get(visualSubtitleFontSelector).within(() => {
          cy.get(dropDownSelector).click();
          cy.get('[data-test-id="value-arc-font-helvetica"]').click();
        });
      });
      dashboards.closeVisual(false);
      dashboards.openDashboardforEdit(testDashbboardName);
      dashboards.editVisual();
      dashboards.chooseVisualTab('style');
      cy.get(visualSubtitleSelector).within(() => {
        cy.get(visualSubtitleFontSelector).within(() => {
          cy.get(dropDownSelector).should('have.attr', 'data-value', 'arc-font-helvetica');
        });
      });
      dashboards.closeVisual(false);
    });

    it('Test 9: Visual subtitle font size', () => {
      const visualSubtitleFontSizeSelector = '[data-test-id="setting-id-subtitle_font_size"]';
      const testName = 'setVisualSubtitleFontSize';
      console.log('Starting test ' + testName);
      // set the visual title font to be Comic Sans MS
      cy.get(visualSubtitleSelector).within(() => {
        cy.get(visualSubtitleFontSizeSelector).within(() => {
          cy.get('[data-test-id="TITLE_FONT_SIZE"]').clear();
          cy.get('[data-test-id="TITLE_FONT_SIZE"]').type('50{enter}');
        });
      });
      // save close and reopen the visual
      dashboards.closeVisual(false);
      dashboards.openDashboardforEdit(testDashbboardName);
      dashboards.editVisual();
      dashboards.chooseVisualTab('style');
      // dashboards.openDashboardforEdit(dashboardName);
      // editVisual();
      // verify the font is set right
      cy.get(visualSubtitleSelector).within(() => {
        cy.get(visualSubtitleFontSizeSelector).within(() => {
          cy.get('[data-test-id="TITLE_FONT_SIZE"]').should('have.attr', 'value', '50');
        });
      });
      dashboards.closeVisual(false);
    });

    it('Test 10: Visual subtitle alignment', () => {
      const visualSubtitleFontAlignmentSelector = '[data-test-id="subtitle_alignment"]';
      const testName = 'setVisualSubtitleFontAlignment';
      console.log('Starting test ' + testName);
      // set the visual title font to be Comic Sans MS
      cy.get(visualSubtitleSelector).within(() => {
        cy.get(visualSubtitleFontAlignmentSelector).within(() => {
          cy.get('[data-test-id="radio-Right"]').click();
        });
      });
      // save close and reopen the visual
      dashboards.closeVisual(false);
      dashboards.openDashboardforEdit(testDashbboardName);
      dashboards.editVisual();
      dashboards.chooseVisualTab('style');
      // verify the font is set right
      cy.get(visualSubtitleSelector).within(() => {
        cy.get(visualSubtitleFontAlignmentSelector).within(() => {
          cy.get('[data-test-id="radio-Right"]').then(($el) => {
            const isChecked = $el.attr('checked');
            assert.isTrue(typeof isChecked !== typeof undefined);
            // assert.isTrue($el.attr("checked"));
          });
        });
      });
      dashboards.closeVisual(false);
    });

    it('Test 11: Visual style tool tip  font', () => {
      const visualTooltipFontSelector = '[data-test-id="setting-id-tooltip_font"]';
      const testName = 'setVisualSubtitleTTFontType';
      console.log('Starting test ' + testName);
      cy.get(visualTooltipStyleSelector).within(() => {
        cy.get(visualTooltipFontSelector).within(() => {
          cy.get(dropDownSelector).click();
          cy.get('[data-test-id="value-arc-font-tahoma"]').click();
        });
      });
      dashboards.closeVisual(false);
      dashboards.editVisual();
      dashboards.chooseVisualTab('style');
      cy.get(visualTooltipStyleSelector).within(() => {
        cy.get(visualTooltipFontSelector).within(() => {
          cy.get(dropDownSelector).should('have.attr', 'data-value', 'arc-font-tahoma');
        });
      });
      dashboards.closeVisual(false);
    });

    it('Test 12: Visual style tooltip size ', () => {
      const visualTooltipFontSizeSelector = '[data-test-id="setting-id-tooltip_font_size"]';
      const testName = 'setVisualSubtitleTTFontSize';
      console.log('Starting test ' + testName);
      // set the visual title font to be Comic Sans MS
      cy.get(visualTooltipStyleSelector).within(() => {
        cy.get(visualTooltipFontSizeSelector).within(() => {
          cy.get('[data-test-id="TITLE_FONT_SIZE"]').clear();
          cy.get('[data-test-id="TITLE_FONT_SIZE"]').type('60{enter}');
        });
      });
      // save close and reopen the visual
      dashboards.closeVisual(false);
      dashboards.openDashboardforEdit(testDashbboardName);
      dashboards.editVisual();
      dashboards.chooseVisualTab('style');
      // dashboards.openDashboardforEdit(dashboardName);
      // editVisual();
      // verify the font is set right
      cy.get(visualTooltipStyleSelector).within(() => {
        cy.get(visualTooltipFontSizeSelector).within(() => {
          cy.get('[data-test-id="TITLE_FONT_SIZE"]').should('have.attr', 'value', '60');
        });
      });
      dashboards.closeVisual(false);
    });
  });
});
