import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';

const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'Cereals';

const dashboard: Dashboard = {
  title: 'simple_bar_visual_dashboard',
  subTitle: 'simple_bar_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'calorie sheet',
      sheet_number: 1,
      visuals: [
        {
          name: 'barvisual1',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-trellis-bars',
          visual_title: 'simple_bar_visual1',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.X,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.Y,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'sugars_grams',
                field_shelf: ShelfTypes.Y,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.COLOR,
              },
              {
                field_name: 'cups_per_serving',
                field_shelf: ShelfTypes.TOOLTIP,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.DRILL,
                alias: 'CEREALS',
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['Cheerios', 'Cocoa_Puffs'],
                },
              },
            ],
          },
        },
        {
          name: 'tablevisual1',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-table',
          visual_title: 'simple_table_visual1',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.DIMENSIONS,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.AGGREGATES,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Average',
              },
              {
                field_name: 'sugars_grams',
                field_shelf: ShelfTypes.AGGREGATES,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Maximum',
              },
              {
                field_name: 'cups_per_serving',
                field_shelf: ShelfTypes.TOOLTIP,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['Cheerios', 'Cocoa_Puffs'],
                },
              },
            ],
          },
        },
      ],
    },
    {
      sheet_name: 'sugar sheet',
      sheet_number: 2,
      visuals: [
        {
          name: 'barvisual3',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-trellis-bars',
          visual_title: 'simple_bar_visual3',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.X,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.Y,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'sugars_grams',
                field_shelf: ShelfTypes.Y,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.COLOR,
              },
              {
                field_name: 'cups_per_serving',
                field_shelf: ShelfTypes.TOOLTIP,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.DRILL,
                alias: 'CEREALS',
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['Cheerios', 'Cocoa_Puffs'],
                },
              },
            ],
          },
        },
        {
          name: 'tablevisual4',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-table',
          visual_title: 'simple_table_visual2',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.DIMENSIONS,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.AGGREGATES,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Average',
              },
              {
                field_name: 'sugars_grams',
                field_shelf: ShelfTypes.AGGREGATES,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Maximum',
              },
              {
                field_name: 'cups_per_serving',
                field_shelf: ShelfTypes.TOOLTIP,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['Cheerios', 'Cocoa_Puffs'],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Arcviz Bar visuals test suite.', () => {
  before(() => {
    logins.silent();
    cy.visit('/arc/apps/home');
  });

  context('Bar graph visual tests', () => {
    it('Test 1: Create basic bar graph visual', () => {
      dashboards.createDashboard(dashboard);
    });
  });
});
