import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';

const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'Cereals';

const dashboard: Dashboard = {
  title: 'simple_groupedbar_visual_dashboard',
  subTitle: 'simple_groupedbar_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'calorie sheet',
      sheet_number: 1,
      visuals: [
        {
          name: 'barvisual1',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-trellis-groupedbars',
          visual_title: 'simple_groupedbar_visual',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.X,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.Y,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'sugars_grams',
                field_shelf: ShelfTypes.Y,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'sugars_grams',
                field_shelf: ShelfTypes.COLOR,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'sodium_mg',
                field_shelf: ShelfTypes.COLOR,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.COLOR,
                field_aggregate: 'Average',
              },
              {
                field_name: 'cups_per_serving',
                field_shelf: ShelfTypes.TOOLTIP,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'manufacturer',
                field_shelf: ShelfTypes.DRILL,
              },
              {
                field_name: 'manufacturer',
                field_shelf: ShelfTypes.LABEL,
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['Cheerios', 'Cocoa_Puffs'],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Arcviz Grouped Bar visuals test suite.', () => {
  before(() => {
    logins.silent();
    cy.visit('/arc/apps/home');
  });
  afterEach(() => {
    cy.window().then(
      (win: Window) =>
        (win.onbeforeunload = () => {
          return;
        })
    );
  });

  context('Grouped Bar visual tests', () => {
    it('Test 1: Create basic grouped bar visuals', () => {
      dashboards.createDashboard(dashboard);
    });
  });
});
