import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';

const connectionName = 'samples';
const databaseName = 'main';
const tableName = 'world_life_expectancy';
const datasetName = 'World Life Expectancy';

const dashboard: Dashboard = {
  title: 'boxplot_visual_dashboard',
  subTitle: 'simple_boxplot_visual_dashboard_ARC-9077,9371',
  sheets: [
    {
      sheet_name: 'Sheet 1',
      sheet_number: 1,
      visuals: [
        {
          name: 'boxplot visuals',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-boxplot',
          visual_title: 'simple_boxplot_visual1',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'country',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'life_expectancy',
                field_shelf: ShelfTypes.AGGREGATES,
              },
              {
                field_name: 'life_expectancy',
                field_shelf: ShelfTypes.FILTERS,
                field_expression: '[life_expectancy] IS NOT NULL',
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Boxplot visual tests.', () => {
  context('ARC-9077, ARC-9371', () => {
    before(() => {
      logins.silent();
      cy.visit('/');
      // create dataset if it doesn't exist
      functions.createDatasetFromConnection(connectionName, databaseName, tableName, datasetName);
    });

    it('1: Create boxplot visual', { retries: 2 }, () => {
      dashboards.createDashboard(dashboard);
      // dashboards.openDashboard(dashboard.title);
      dashboards.openDashboardforEdit(dashboard.title);
      cy.get('g.y.arc-viz-axis g.tick').should('exist');
      const life: string[] = [];
      cy.get('g.y.arc-viz-axis g.tick').each(($el, index, $list) => {
        if ($el.text().length > 0) {
          life.push($el.text());
        }
        if (index === $list.length - 1) {
          if (life.length === 0) {
            assert.fail('No Y ticks were found');
          } else {
            console.log('These ticks were displayed:' + life.toString());
          }
        }
      });

      cy.get('g.x.arc-viz-axis g.tick').should('exist');
      const countries: string[] = [];
      cy.get('g.x.arc-viz-axis g.tick').each(($el, index, $list) => {
        if ($el.text().length > 0) {
          countries.push($el.text());
        }
        if (index === $list.length - 1) {
          if (countries.length === 0) {
            assert.fail('No X ticks were found');
          } else {
            console.log('These ticks were displayed:' + countries.toString());
          }
        }
      });
    });
  });
});
