import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';
import * as logins from 'lib/logins';
import * as visuals from 'lib/visual';
import { reverseColorPalette } from 'lib/visual';

const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'Cereals';
const dashboardName = 'Cereal Scatters DB';
const dashboard: Dashboard = {
  title: dashboardName,
  subTitle: 'simple_boxplot_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'sheet1',
      sheet_number: 1,
      visuals: [
        {
          name: 'boxplotvisual1',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-boxplot',
          visual_title: 'simple_boxplot_visual1',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'manufacturer_code',
                field_shelf: ShelfTypes.DIMENSIONS,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.AGGREGATES,
              },
              {
                field_name: 'manufacturer_code_15',
                field_shelf: ShelfTypes.X,
              },
              {
                field_name: 'cold_or_hot',
                field_shelf: ShelfTypes.Y,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['110', '120', '130', '140', '150', '160'],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Boxplot visual tests.', () => {
  context('Boxplot Plot visual tests colors tab tests', () => {
    before(() => {
      logins.silent();
      dashboards.createDashboard(dashboard);
    });

    beforeEach(() => {
      // visit is done in the cloneDashboard call
      dashboards.editVisual();
      dashboards.chooseVisualTab('colors');
    });

    it.skip('Test 1: Reverse order of color palettes', () => {
      reverseColorPalette();
      // The functionality doesn't work, need a fix!
      cy.get(visuals.locators.boxplot.boxes).each(($el) => {
        cy.wrap($el)
          .should('have.css', 'fill', 'rgb(176, 203, 115)')
          .should('have.css', 'fill-opacity', '0.8');
      });
    });

    it('Test 2: Choose color palettes, double click select the inital color', () => {
      const colorPaletteLocator =
        '[data-test-id="colorpalettes-setting"] .settings-palette:nth-child(2) .settings-swatch:nth-child(4)';
      cy.get(colorPaletteLocator)
        .dblclick()
        .then(($color) => {
          const selectedColor = $color.css('background-color');
          cy.get(visuals.locators.boxplot.boxes).each(($el) => {
            cy.wrap($el)
              .should('have.css', 'fill', `${selectedColor}`)
              .should('have.css', 'fill-opacity', '0.8');
          });
        });
    });

    it('Test 3: Color opacity', () => {
      // Slider is not working
      cy.get(".settings-tab-holder[data-tabname='Colors']").within(($colorsTab) => {
        cy.get('[data_setting_id="color_opacity"] .settings-slider-input').click(20, 7);
      });
    });
  });
});
