import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';
import * as visual from 'lib/visual';

const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'Cereals';
const datasetName2 = 'Iris';
const dashboardName = 'Cereal Scatters DB';
const dashboard: Dashboard = {
  title: dashboardName,
  subTitle: 'simple_boxplot_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'sheet1',
      sheet_number: 1,
      visuals: [
        {
          name: 'boxplotvisual1',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-boxplot',
          visual_title: 'simple_boxplot_visual1',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'manufacturer_code',
                field_shelf: ShelfTypes.DIMENSIONS,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.AGGREGATES,
              },
              {
                field_name: 'manufacturer_code_15',
                field_shelf: ShelfTypes.X,
              },
              {
                field_name: 'cold_or_hot',
                field_shelf: ShelfTypes.Y,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['110', '120', '130', '140', '150', '160'],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Boxplot visual tests.', () => {
  context('Boxplot visual tests data tab tests', () => {
    before(() => {
      cy.fixture('users.json').then((data) => {
        logins.silent(data.admin.username, data.admin.password);
      });

      dashboards.createDashboard(dashboard);
    });

    beforeEach(() => {
      dashboards.editVisual();
      dashboards.chooseVisualTab('data');
    });

    it('Test 1: Switch data set', () => {
      visual.switchVisualDataSet(datasetName2);
      dashboards.editVisual();
      cy.get('#app-dataset').should('have.text', datasetName2);
      dashboards.closeVisual(false);
    });
  });
});
