import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';
import * as dsManage from 'lib/datasetManagement';
import * as functions from 'lib/functions';
import * as visuals from 'lib/visual';
import * as scatter_functions from 'lib/visual/visual_scatter_boxplot';

const connectionName = Cypress.env('smokeConnectionName');
const databaseName = 'main';
const tableName = 'cereals';
const datasetName = 'E2E test dataset Cereals';
const dashboardName = 'E2E Cereal Boxplot dashboard';
const dashboard: Dashboard = {
  title: dashboardName,
  subTitle: 'simple_boxplot_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'sheet1',
      sheet_number: 1,
      visuals: [
        {
          name: 'boxplotvisual1',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-boxplot',
          visual_title: 'simple_boxplot_visual1',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'Segment',
                field_shelf: ShelfTypes.X,
              },
              {
                field_name: 'manufacturer_code',
                field_shelf: ShelfTypes.DIMENSIONS,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.AGGREGATES,
              },
              {
                field_name: 'manufacturer_code_15',
                field_shelf: ShelfTypes.X,
              },
              {
                field_name: 'cold_or_hot',
                field_shelf: ShelfTypes.Y,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['90', '100', '110', '120', '130'],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Boxplot visual tests.', () => {
  context('Segment tab tests', () => {
    const segmentName = 'SegGrpT1_' + Math.floor(Math.random() * 1000);
    const segmentGroup = 'SegmentGroup1';
    const filterExpression = '[calories] < 120';
    before(() => {
      logins.silent();
      cy.visit('/');
    });

    beforeEach(() => {
      // Because we modify the dataset (adding segments), I rather create a new one
      // not to mess up local db!
      functions.deleteDatasetFromConnection(connectionName, datasetName);
      functions.createDatasetFromConnection(connectionName, databaseName, tableName, datasetName);
      dsManage.createNewSegment(
        connectionName,
        datasetName,
        segmentName,
        filterExpression,
        segmentGroup
      );
      dashboards.createDashboard(dashboard);
    });

    it('1: Choose segment is', () => {
      dashboards.editVisual();
      scatter_functions.setSegment(segmentName);
      cy.get(visuals.locators.visual.y_axis + ' > g.tick text').each(($el) => {
        cy.wrap(parseInt($el.text(), 10)).should('be.lessThan', 120);
      });
    });

    // Due to ARC-11671 skipping it. Will remove it in future
    it.skip('2: Choose segment All', () => {
      dashboards.editVisual();
      dashboards.chooseVisualTab('segments');
      cy.get('#apps-segments-container input.all').click();
      cy.get(visuals.locators.visual.y_axis + ' > g.tick text').each(($el) => {
        cy.wrap(parseInt($el.text(), 10)).should('be.lessThan', 120);
      });
      visuals.closeVisual(false);
    });
  });
});
