import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';

const connectionName = Cypress.env('smokeConnectionName');
const databaseName = 'main';
const tableName = 'cereals';
const datasetName = 'Cereals';

const dashboard: Dashboard = {
  title: 'simple_boxplot_visual_dashboard',
  subTitle: 'simple_boxplot_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'Cereal sheet',
      sheet_number: 1,
      visuals: [
        {
          name: 'boxplot visuals',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-boxplot',
          visual_title: 'simple_boxplot_visual1',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'manufacturer',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.AGGREGATES,
              },
              {
                field_name: 'cold_or_hot',
                field_shelf: ShelfTypes.FILTERS,
              },
              {
                field_name: 'cold_or_hot',
                field_shelf: ShelfTypes.X,
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Boxplot visual tests.', () => {
  context('Basic boxplot dashboard', () => {
    before(() => {
      logins.silent();
      cy.visit('/');
      // create dataset if it doesn't exist
      functions.createDatasetFromConnection(connectionName, databaseName, tableName, datasetName);
    });

    it('Test 1: Create boxplot visual', () => {
      dashboards.createDashboard(dashboard);
    });
  });
});
