import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';

const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'Cereals';

const dashboard: Dashboard = {
  title: 'simple_bullet_visual_dashboard',
  subTitle: 'simple_bullet_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'calories',
      sheet_number: 1,
      visuals: [
        {
          name: 'Bullet visuals',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-bullet',
          visual_title: 'simple_bullet_visual1',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.AGGREGATES,
                sort: SortOptions.ASCENDING,
                field_expression: 'avg(calories) - 60',
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.COMPARE,
                field_expression: 'sum(calories)',
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.QUALITATIVE,
                field_expression: 'sum(calories) + 50',
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.PROJECTION,
                field_expression: 'sum(calories) + 40',
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.LABEL,
              },
              {
                field_name: 'cold_or_hot',
                field_shelf: ShelfTypes.TOOLTIP,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'manufacturer',
                field_shelf: ShelfTypes.X,
                alias: 'STATE',
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.Y,
              },
              {
                field_name: 'manufacturer',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['General Mills'],
                },
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['Cheerios', 'Cinnamon_Toast_Crunch', 'Clusters', 'Cocoa_Puffs'],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Arcviz Bullet visuals test suite.', () => {
  before(() => {
    logins.silent();
    cy.visit('/arc/apps/home');
  });
  afterEach(() => {
    cy.window().then(
      (win: Window) =>
        (win.onbeforeunload = () => {
          return;
        })
    );
  });

  context('Bullet visual tests', () => {
    it('Test 1: Create bullet visual', () => {
      dashboards.createDashboard(dashboard);
    });
  });
});
