import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';

const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'Food Stores Inspection in NYC';

const dashboard: Dashboard = {
  title: 'simple_cal_heatmap_visual_dashboard',
  subTitle: 'simple_calendar_heatmap_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'inspection sheet',
      sheet_number: 1,
      visuals: [
        {
          name: 'cal_heatmapvisual1',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-calendar-heatmap',
          visual_title: 'simple_cal_heatmap_visual',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'inspection_date',
                field_shelf: ShelfTypes.DIMENSIONS,
                alias: 'Inspection Date',
              },
              {
                field_name: 'Record Count',
                field_shelf: ShelfTypes.AGGREGATES,
                alias: 'Deficiency Code',
              },
              {
                field_name: 'owner_name',
                field_shelf: ShelfTypes.TOOLTIP,
                alias: 'Business Owner',
              },
              {
                field_name: 'trade_name',
                field_shelf: ShelfTypes.TOOLTIP,
                alias: 'Business Name',
              },
              {
                field_name: 'county',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: [],
                },
              },
              {
                field_name: 'deficiency_code',
                field_shelf: ShelfTypes.FILTERS,
                field_expression: "[deficiency_code] not in ('null', '-  ')",
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Arcviz calendar heatmap visuals test suite.', () => {
  before(() => {
    logins.silent();
    cy.visit('/arc/apps/home');
  });
  afterEach(() => {
    cy.window().then(
      (win: Window) =>
        (win.onbeforeunload = () => {
          return;
        })
    );
  });

  context('calendar heatmap visual tests', () => {
    it('Test 1: Create basic calendar heatmap visuals', () => {
      dashboards.createDashboard(dashboard);
    });
  });
});
