import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';

const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'InfoSeсGlobal';

const dashboard: Dashboard = {
  title: 'simple_chord_visual_dashboard',
  subTitle: 'simple_chord_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'Threats sheet',
      sheet_number: 1,
      visuals: [
        {
          name: 'chord visuals',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-chord',
          visual_title: 'simple_chord_visual',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'source_country',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'country',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'count',
                field_shelf: ShelfTypes.AGGREGATES,
                sort: SortOptions.ASCENDING,
                field_expression: "sum([count]) as 'Threat Count'",
              },
              {
                field_name: 'threat',
                field_shelf: ShelfTypes.TOOLTIP,
              },
              {
                field_name: 'industry',
                field_shelf: ShelfTypes.TOOLTIP,
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Arcviz Chord visuals test suite.', () => {
  before(() => {
    logins.silent();
    cy.visit('/arc/apps/home');
  });
  afterEach(() => {
    cy.window().then(
      (win: Window) =>
        (win.onbeforeunload = () => {
          return;
        })
    );
  });

  context('Chord visual tests', () => {
    it('Test 1: Create chord visual', () => {
      dashboards.createDashboard(dashboard);
    });
  });
});
