import { logins } from 'lib/arctest';
import { BackendAPI } from 'lib/backend-api';
import * as dashboards from 'lib/dashboard';
import { ShelfTypes } from 'lib/dashboard/dashboards_interfaces';
import {
  combinatedBarLineDashboard,
  combinedLineAndBarVisual,
} from 'lib/dashboard/default_dashboard';
import * as combo_functions from 'lib/visual/visual_bar_line_combo_area_group';
import * as visual_common_tests from 'lib/visual/visual_common_tests';

describe('Arcviz Combined Bar Line visuals tests.', () => {
  context('Bar/Line visual tests settings tab tests', () => {
    before(() => {
      logins.silent();
    });

    beforeEach(() => {
      BackendAPI.dashboards
        .import({
          fileName: 'combinated_bar_and_line_e2e_prettified.json',
          dataconnectionName: 'samples',
          workspaceName: 'Public',
        })
        .then(({ dashboard }) => {
          cy.visit(`/arc/apps/builder/${dashboard.id}`);
          cy.get('.progress.viz-permanent').should('be.visible');
          cy.get('.progress.viz-permanent').should('not.be.visible');
          cy.get('[data-test-id="visual-list-item"]')
            .first()
            .click();
        });
    });

    it('Test 1: Set Download as CSV locale', () => {
      combo_functions.enableDisableCSVDownload();
    });

    it('Test 2: Enable Animation', () => {
      combo_functions.enableDisableAnimation();
    });

    it('Test 3: Enable/Disable Drill into context menu', () => {
      visual_common_tests.enableDrillIntoFunctionality();
      const expectedVisibility = false;
      visual_common_tests.testDrillIntoOptionIsAvailableInContextMenu(expectedVisibility);
    });

    it('Test 4: Enable/Disable Tooltips', () => {
      combo_functions.setEnableTooltipsSetting(combinatedBarLineDashboard.title, 'checked');
      visual_common_tests.testTooltipIsVisibleOnHover();
      dashboards.editVisual();
      combo_functions.setEnableTooltipsSetting(combinatedBarLineDashboard.title, 'unchecked');
    });

    // Skip because no idea how to actually trigger it.
    it.skip('Test 5: Bar size range', () => {
      cy.document().then(($doc) => {
        combo_functions.setBarSizeRange('2-10');
      });
    });

    // Skip
    it.skip('Test 6: Legend style is not an option', () => {
      // Fill it again after fix: https://jira.cloudera.com/browse/VIZ-1076
    });

    it('Test 8: Show selection brush', () => {
      combo_functions.setSelectionBrush();
      // Todo: detect selection brush ?!
    });

    it('Test 9: Label measure values next to point', () => {
      visual_common_tests.testShowMeasureValuesSetting();
    });

    it('Test 10: Minimum cell height and width', () => {
      visual_common_tests.testCellHeightWidthSetting();
    });

    it('Test 11: Ignore negative measure values', () => {
      combo_functions.setIgnoreNegativeData();
      // TODO fill this text with actual assertation.
    });

    it('Test 12: set highlight mark type', () => {
      // I think the functionality does not working
      combo_functions.setHighLightMarkType('CIRCLE');
      dashboards.editVisual();
      combo_functions.setHighLightMarkType('DOT');
      dashboards.editVisual();
      combo_functions.setHighLightMarkType('STAR');
    });

    it('Test 13: Convert Line to Area Visual', () => {
      visual_common_tests.convertLineToArea();
    });

    it('Test 14: Show Points', () => {
      const locator = '.arc-viz-overlay-container svg+canvas.maincanvas';
      visual_common_tests.getCanvasStack(locator).then((beforeStack) => {
        expect((beforeStack as any[]).filter((item) => item.method === 'arc').length).to.be.eq(0);
        return combo_functions.setShowPoints();
      });
      // TODO scanpshot comparison!
      visual_common_tests.getCanvasStack(locator).then((stack) => {
        const numberOfLineTo = (stack as any[]).filter((item) => item.method === 'lineTo').length;
        const numberOfLineMeaseFields =
          combinedLineAndBarVisual.data?.fields.filter(
            (field) => field.field_shelf === ShelfTypes.LINE_MEASURE
          ).length || 0;
        const numberOfPoint = (stack as any[]).filter((item) => item.method === 'arc').length;
        expect(numberOfPoint).to.eq(numberOfLineTo + numberOfLineMeaseFields);
      });
    });

    it('Test 15: Ignore missing dates', () => {
      combo_functions.setIgnoreMissingDates();
      // TODO check the functionality once...
    });

    it('Test 16: Set Line Style', () => {
      combo_functions.setLineStyle('basis');
      dashboards.editVisual();
      combo_functions.setLineStyle('linear');
      dashboards.editVisual();
      combo_functions.setLineStyle('bundle');
      dashboards.editVisual();
      combo_functions.setLineStyle('cardinal');
      dashboards.editVisual();
      combo_functions.setLineStyle('monotone');
      dashboards.editVisual();
      combo_functions.setLineStyle('step');
      dashboards.editVisual();
      combo_functions.setLineStyle('step-before');
      // TODO check the functionality once...
    });

    // TODO the following tests are not really testng the feautre
    // Add expectations - what should change on the visual
    it('Test 17: Show data with null values', () => {
      combo_functions.setShowNullValues('Default');
      dashboards.editVisual();
      combo_functions.setShowNullValues('First');
      dashboards.editVisual();
      combo_functions.setShowNullValues('Last');
    });

    it('Test 18: Number of rows to download in CSV', () => {
      combo_functions.setNumberofRowsinCSV('75');
    });

    it('Test 19: Maximum number of rows to fetch', () => {
      combo_functions.setNumberofRowsToFetch('10');
    });

    it('Test 20: Disable incremental queries', () => {
      combo_functions.setDisableIncrementalQueries();
    });

    it('Test 21: Auto detect time columns', () => {
      combo_functions.setAutoDetectTimeColumn();
    });

    it('Test 22: Axis scale log10', () => {
      combo_functions.setAxisScale('LOG10');
    });

    it('Test 23: Axis scale Linear', () => {
      combo_functions.setAxisScale('LINEAR');
    });

    it('Test 24: Gridlines enabled', () => {
      combo_functions.setGridLines();
    });

    it('Test 25: Positive aggregate scale start at 0', () => {
      combo_functions.setAggrScaleStart();
    });

    it('Test 26: Aggregate scale minimum', () => {
      combo_functions.setMinAggrScale();
    });

    it('Test 27: Aggregate scale maximum', () => {
      combo_functions.setMaxAggrScale();
    });

    it('Test 28: Include X-Axis label', () => {
      combo_functions.setIncludeXAxisLabel();
    });

    it('Test 29: Include Y-Axis label', () => {
      combo_functions.setIncludeYAxisLabel();
    });

    it('Test 30: Show X-Axis', () => {
      combo_functions.setShowXAXis();
    });

    it('Test 31: Show Y-Axis ', () => {
      combo_functions.setShowYAXis();
    });
  });
});
