import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';

const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'Food Stores Inspection in NYC';

const dashboard: Dashboard = {
  title: 'simple_cor_flow_visual_dashboard',
  subTitle: 'simple_correlation-heatflow_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'inspection sheet',
      sheet_number: 1,
      visuals: [
        {
          name: 'cor_flow_visual1',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-corelation-flow',
          visual_title: 'simple_cor_flow_visual',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'county',
                field_shelf: ShelfTypes.DIMENSIONS,
                alias: 'Country',
              },
              {
                field_name: 'deficiency_code',
                field_shelf: ShelfTypes.DIMENSIONS,
                alias: 'Deficiency Code',
              },
              {
                field_name: 'Record Count',
                field_shelf: ShelfTypes.AGGREGATES,
              },
              {
                field_name: 'deficiency_description',
                field_shelf: ShelfTypes.TOOLTIP,
                field_expression: "group_concat([deficiency_description]) as 'Deficiency'",
              },
              {
                field_name: 'county',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: [],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Arcviz correlation heatflow visuals tests.', () => {
  before(() => {
    logins.silent();
    cy.visit('/arc/apps/home');
  });
  afterEach(() => {
    cy.window().then(
      (win: Window) =>
        (win.onbeforeunload = () => {
          return;
        })
    );
  });

  context('correlation heatflow visual tests', () => {
    it('Test 1: Create basic correlation heatflow visuals', () => {
      dashboards.createDashboard(dashboard);
    });
  });
});
