import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';

const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'Food Stores Inspection in NYC';

const dashboard: Dashboard = {
  title: 'simple_cor_heatmap_visual_dashboard',
  subTitle: 'simple_correlation-heatmap_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'inspection sheet',
      sheet_number: 1,
      visuals: [
        {
          name: 'cor_heatmapvisual1',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-corelation',
          visual_title: 'simple_cor_heatmap_visual',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'inspection_date',
                field_shelf: ShelfTypes.DIMENSIONS,
                sort: SortOptions.ASCENDING,
                alias: 'Inspection Date',
              },
              {
                field_name: 'deficiency_code',
                field_shelf: ShelfTypes.DIMENSIONS,
                alias: 'Deficiency Code',
              },
              {
                field_name: 'Record Count',
                field_shelf: ShelfTypes.AGGREGATES,
              },
              {
                field_name: 'owner_name',
                field_shelf: ShelfTypes.TOOLTIP,
              },
              {
                field_name: 'trade_name',
                field_shelf: ShelfTypes.TOOLTIP,
              },
              {
                field_name: 'deficiency_code',
                field_shelf: ShelfTypes.FILTERS,
                field_expression: "[deficiency_code] not in ('null', '-  ')",
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Arcviz correlation-heatmap visuals tests.', () => {
  before(() => {
    logins.silent();
    cy.visit('/arc/apps/home');
  });
  afterEach(() => {
    cy.window().then(
      (win: Window) =>
        (win.onbeforeunload = () => {
          return;
        })
    );
  });

  context('correlation-heatmap visual tests', () => {
    it('Test 1: Create basic correlation-heatmap visuals', () => {
      dashboards.createDashboard(dashboard);
    });
  });
});
