import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';

const connectionName = Cypress.env('smokeConnectionName');
const datasetName2 = 'Iris';
const datasetName1 = 'Cereals';

const dashboard: Dashboard = {
  title: 'simple_crosstab_visuals_dashboard',
  subTitle: 'simple_crosstab_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'iris details',
      sheet_number: 1,
      visuals: [
        {
          name: 'crosstablevisual1',
          connection_name: connectionName,
          dataset_name: datasetName1,
          visual_type: '.chart-icon-crosstab',
          visual_title: 'simple_crosstab_visual1',
          linked: 'true',
          data: {
            fields: [
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.X_DIMENSIONS,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'manufacturer',
                field_shelf: ShelfTypes.Y_DIMENSIONS,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.AGGREGATES,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'fat_grams',
                field_shelf: ShelfTypes.AGGREGATES,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'sodium_mg',
                field_shelf: ShelfTypes.TOOLTIP,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'manufacturer',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['Kelloggs', 'Post'],
                },
              },
            ],
          },
        },
        {
          name: 'tablevisual2',
          connection_name: connectionName,
          dataset_name: datasetName2,
          visual_type: '.chart-icon-table',
          visual_title: 'simple_table_visual1',
          linked: 'true',
          data: {
            fields: [
              {
                field_name: 'species',
                field_shelf: ShelfTypes.DIMENSIONS,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'sepal_length',
                field_shelf: ShelfTypes.AGGREGATES,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Sum',
              },

              {
                field_name: 'petal_length',
                field_shelf: ShelfTypes.AGGREGATES,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'sepal_ratio',
                field_shelf: ShelfTypes.TOOLTIP,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'species',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['setosa'],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Arcviz crosstab visuals tests.', () => {
  before(() => {
    logins.silent();
    cy.visit('/arc/apps/home');
  });
  afterEach(() => {
    cy.window().then(
      (win: Window) =>
        (win.onbeforeunload = () => {
          return;
        })
    );
  });

  context('cross tab visual tests', () => {
    it('Test 1: Create basic cross tab visual', () => {
      dashboards.createDashboard(dashboard);
    });
  });
});
