import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';

const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'Cereals';

const dashboard: Dashboard = {
  title: 'simple_dendrogram_visual_dashboard',
  subTitle: 'simple_dendrogram_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'cereal details',
      sheet_number: 1,
      visuals: [
        {
          name: 'dendrogram visuals',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-dendrogram',
          visual_title: 'simple_dendrogram_visual1',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'manufacturer',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.AGGREGATES,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'fat_grams',
                field_shelf: ShelfTypes.TOOLTIP,
              },
              {
                field_name: 'sugars_grams',
                field_shelf: ShelfTypes.TOOLTIP,
              },
              {
                field_name: 'manufacturer',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['Kelloggs', 'General Mills'],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Arcviz dendrogram visuals tests.', () => {
  before(() => {
    logins.silent();
    cy.visit('/');
  });

  context('dendrogram visual tests', () => {
    it('Test 1: Create dendrogram visual', () => {
      dashboards.createDashboard(dashboard);
    });
  });
});
