import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';

const connectionName = Cypress.env('smokeConnectionName');
const databaseName = 'main';
const tableName = 'census_pop';
const datasetName = 'US State Populations Over Time';

const dashboard: Dashboard = {
  title: 'simple_fieldstats_visual_dashboard',
  subTitle: 'simple_fieldstats_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'population sheet',
      sheet_number: 1,
      visuals: [
        {
          name: 'fieldstatsvisual1',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-column-stats',
          visual_title: 'simple_fieldstats_visual',
          linked: 'false',
        },
      ],
    },
  ],
};

describe('Arcviz fieldstats visuals tests.', () => {
  before(() => {
    logins.silent();
    cy.visit('/arc/apps/home');
  });
  afterEach(() => {
    cy.window().then(
      (win: Window) =>
        (win.onbeforeunload = () => {
          return;
        })
    );
  });

  // local method just for the test
  function get_time() {
    return new Date().getTime();
  }

  context('Field stats visual tests', () => {
    before(() => {
      // create dataset if it doesn't exist
      functions.createDatasetFromConnection(connectionName, databaseName, tableName, datasetName);
    });
    it('Test 1: Create basic fieldstats visuals', () => {
      dashboards.createDashboard(dashboard);
    });
  });
});
