import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';

const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'Cereals';

const dashboard: Dashboard = {
  title: 'simple_gauge_visual_dashboard',
  subTitle: 'simple_gauge_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'calories',
      sheet_number: 1,
      visuals: [
        {
          name: 'Gauge visuals',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-gauge',
          visual_title: 'simple_gauge_visual1',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.AGGREGATES,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Average',
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.COMPARE,
                field_expression: 'avg([calories]) + random() * avg([calories])',
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.QUALITATIVE,
                field_expression: 'avg([calories]) + (random()/2) * avg([calories])',
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.PROJECTION,
                field_expression: 'avg([calories]) + (random()*2) * avg([calories])',
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.LABEL,
              },
              {
                field_name: 'cold_or_hot',
                field_shelf: ShelfTypes.TOOLTIP,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.X,
                alias: 'STATE',
              },
              {
                field_name: 'manufacturer',
                field_shelf: ShelfTypes.Y,
              },
              {
                field_name: 'manufacturer',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['Post'],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Arcviz Gauge visuals tests.', () => {
  before(() => {
    logins.silent();
    cy.visit('/arc/apps/home');
  });
  afterEach(() => {
    cy.window().then(
      (win: Window) =>
        (win.onbeforeunload = () => {
          return;
        })
    );
  });


  context('Hauge visual tests', () => {
    it('Test 1: Create gauge visual', () => {
      dashboards.createDashboard(dashboard);
    });
  });
});
