import { logins } from 'lib/arctest';
import { BackendAPI } from 'lib/backend-api';
import * as dashboards from 'lib/dashboard';
import * as combo_functions from 'lib/visual/visual_bar_line_combo_area_group';

const dashboardName = 'Grouped bar visual dashboard';

describe('Arcviz Grouped Bar visuals tests.', () => {
  context('Grouped bar visual tests colors tab tests', () => {
    const cloneDashboardName = 'grobar_visual_colors_dashboard';

    before(() => {
      cy.fixture('users.json').then((data) => {
        logins.silent(data.admin.username, data.admin.password);
      });
      BackendAPI.dashboards.import({
        fileName: 'grouped_bars_dashboard.json',
        dataconnectionName: 'samples',
        workspaceName: 'Public',
        randomised: false,
      });
    });

    beforeEach(() => {
      dashboards.cloneDashboard(dashboardName, cloneDashboardName);
      dashboards.openDashboardforEdit(cloneDashboardName);
      dashboards.editVisual();
    });

    afterEach(() => {
      BackendAPI.dashboards.getByName(cloneDashboardName).then((dashboard) => {
        BackendAPI.dashboards.delete(dashboard.id);
      });
    });

    after(() => {
      BackendAPI.dashboards.getByName(dashboardName).then((dashboard) => {
        BackendAPI.dashboards.delete(dashboard.id);
      });
    });

    it('Test 1: Reverse order of color palettes', () => {
      combo_functions.reverseColorPalette();
      dashboards.closeVisual(false);
    });

    it('Test 2: Choose color palettes', () => {
      combo_functions.chooseColorPalette();
      dashboards.closeVisual(false);
    });

    it('Test 3: Color opacity', () => {
      combo_functions.setColorOpacity();
      dashboards.closeVisual(false);
    });

    it('Test 4: Base colors on dimension values', () => {
      combo_functions.setFixedColorLegend();
      dashboards.closeVisual(false);
    });
  });
});
