import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import * as combo_functions from 'lib/visual/visual_bar_line_combo_area_group';
import * as visual_common_tests from 'lib/visual/visual_common_tests';

const dashboardName = 'Grouped bar visual dashboard';

function cloneAndOpenVisual(originalDashboardName: string, clonedDashboardName: string) {
  dashboards.cloneDashboard(originalDashboardName, clonedDashboardName);
  dashboards.openDashboardforEdit(clonedDashboardName);
  dashboards.editVisual();
}
describe('Arcviz Grouped Bar visuals tests.', () => {
  context('Grouped bar visual tests settings tab tests', () => {
    const cloneDashboardName = 'grobar_settings_dashboard';

    before(() => {
      cy.fixture('users.json').then((data) => {
        logins.silent(data.admin.username, data.admin.password);
      });
    });

    it('Test 1: Set Download as CSV locale', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.enableDisableCSVDownload();
    });
    it('Test 2: Enable Animation', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.enableDisableAnimation();
    });

    it.skip('Test 3: Enable/Disable Drill into context menu', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      visual_common_tests.enableDrillIntoFunctionality();
      visual_common_tests.testDrillIntoOptionIsAvailableInContextMenu();
    });

    it('Test 4: Enable/Disable Tooltips', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setEnableTooltipsSetting(cloneDashboardName, 'checked');
      dashboards.editVisual();
      combo_functions.setEnableTooltipsSetting(cloneDashboardName, 'unchecked');
    });

    it.skip('Test 5: Bar size range', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setBarSizeRange('2-10');
    });

    it('Test 6: Legend style', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      visual_common_tests.testLegendPositionChangeBySwitchingTo('none');
      visual_common_tests.testLegendPositionChangeBySwitchingTo('right');
      visual_common_tests.testLegendPositionChangeBySwitchingTo('bottom');
      visual_common_tests.testLegendPositionChangeBySwitchingTo('above');
    });

    it('Test 7: Reverse legend order', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.reverseLegendOrder();
    });

    it('Test 8: Show selection brush', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setSelectionBrush();
    });

    it('Test 9: Label measure values next to point', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      visual_common_tests.testShowMeasureValuesSetting();
    });

    it('Test 10: Minimum cell height and width', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      visual_common_tests.testCellHeightWidthSetting();
    });

    it('Test 11: Ignore negative measure values', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setIgnoreNegativeData();
    });

    it('Test 17: Show data with null values', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setShowNullValues('Default');
      dashboards.editVisual();
      combo_functions.setShowNullValues('First');
      dashboards.editVisual();
      combo_functions.setShowNullValues('Last');
    });

    it('Test 18: Number of rows to download in CSV', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setNumberofRowsinCSV('75');
    });

    it('Test 19: Maximum number of rows to fetch', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setNumberofRowsToFetch('10');
    });

    it('Test 20: Disable incremental queries', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setDisableIncrementalQueries();
    });

    it('Test 21: Auto detect time columns', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setAutoDetectTimeColumn();
    });

    it('Test 22: Axis scale log10', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setAxisScale('LOG10');
    });

    it('Test 23: Axis scale Linear', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setAxisScale('LINEAR');
    });

    it('Test 24: Gridlines enabled', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setGridLines();
    });

    it('Test 25: Positive aggregate scale start at 0', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setAggrScaleStart();
    });

    it('Test 26: Aggregate scale minimum', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setMinAggrScale();
    });

    it('Test 27: Aggregate scale maximum', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setMaxAggrScale();
    });

    it('Test 28: Include X-Axis label', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setIncludeXAxisLabel();
    });

    it('Test 29: Include Y-Axis label', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setIncludeYAxisLabel();
    });

    it('Test 30: Show X-Axis', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setShowXAXis();
    });

    it('Test 31: Show Y-Axis ', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setShowYAXis();
    });

    it('Test 32: Sec. aggregate scale minimum', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setSecMinAggrScale();
    });

    it('Test 33: Sec. aggregate scale maximum', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      combo_functions.setSecMaxAggrScale();
    });
  });
});
