import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';
import * as drillinto from 'lib/drillinto';
import * as functions from 'lib/functions';
import * as logins from 'lib/logins';
import { verifyVisual } from 'lib/visual';
import * as combo_functions from 'lib/visual/visual_bar_line_combo_area_group';

const connectionName = 'samples';
const databaseName = 'main';
const tableName = 'cereals';
const datasetName1 = 'CerealsQA1';
const datasetName2 = 'CerealsQA2';
const dashboardName = 'Histogram Visual Dashboard';
const trellisDashboardName = 'Trellis Histogram Visual Dashboard';

const dashboard1: Dashboard = {
  title: dashboardName,
  subTitle: 'simple_histogram_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'sheet1',
      sheet_number: 1,
      visuals: [
        {
          name: 'histogram visual',
          connection_name: connectionName,
          dataset_name: datasetName1,
          visual_type: '.chart-icon-histogram',
          visual_title: 'histogram visual visual',
          linked: 'false',
          // prettier-ignore
          is: ['setTransform', 'translate', 'globalAlpha', 'lineWidth', 'font', 'clearRect', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill'],
          hash: '8739d8c2b18d8a246c0bd95de5f02ae5',
          data: {
            fields: [
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.AGGREGATES,
              },
              {
                field_name: 'manufacturer',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: [
                    'American Home Food Products',
                    'General Mills',
                    'Kelloggs',
                    'Nabisco',
                    'Post',
                    'Quaker Oats',
                    'Ralston Purina',
                  ],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};

const dashboard2: Dashboard = {
  title: trellisDashboardName,
  subTitle: 'simple_histogram_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'sheet1',
      sheet_number: 1,
      visuals: [
        {
          name: 'trellis histogram visual',
          connection_name: connectionName,
          dataset_name: datasetName1,
          visual_type: '.chart-icon-histogram',
          visual_title: 'histogram visual visual',
          linked: 'false',
          // prettier-ignore
          is: ['setTransform', 'translate', 'globalAlpha', 'lineWidth', 'font', 'clearRect', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill', 'fillStyle', 'strokeStyle', 'beginPath', 'moveTo', 'lineTo', 'lineTo', 'lineTo', 'closePath', 'fill'],
          hash: '8739d8c2b18d8a246c0bd95de5f02ae5',
          data: {
            fields: [
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.AGGREGATES,
              },
              {
                field_name: 'manufacturer',
                field_shelf: ShelfTypes.X,
              },
              {
                field_name: 'cold_or_hot',
                field_shelf: ShelfTypes.Y,
              },
              {
                field_name: 'manufacturer',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: [
                    'American Home Food Products',
                    'General Mills',
                    'Kelloggs',
                    'Nabisco',
                    'Post',
                    'Quaker Oats',
                    'Ralston Purina',
                  ],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};

function cloneAndOpenVisual(originalDashboardName: string, clonedDashboardName: string) {
  dashboards.cloneDashboard(originalDashboardName, clonedDashboardName);
  dashboards.openDashboardforEdit(clonedDashboardName);
  dashboards.editVisual();
}
describe('Arcviz Histogram visuals tests.', () => {
  context('Histogram visual tests settings tab tests', () => {
    const cloneDashboardName = 'histogram_settings_dashboard';
    const trelliscloneDashboardName = 'trellis_histogram_settings_dashboard';
    let cloneDB = false;

    before(() => {
      logins.silent();
      cy.visit('/arc/apps/home');
    });
    afterEach(() => {
      cloneDB = false;
      cy.window().then(
        (win: Window) =>
          (win.onbeforeunload = () => {
            return;
          })
      );
    });
    it('Test 1: Create Histogram visual', () => {
      dashboards.createDashboard(dashboard1);
    });
    it('Test 2: Create Histogram Trellis visual', () => {
      dashboards.createDashboard(dashboard2);
    });

    it('Test 3: Set Download as CSV locale', () => {
      cloneAndOpenVisual(dashboardName, cloneDashboardName);
      cloneDB = true;
      const testName = 'permitCSVDownload';
      console.log('Starting test ' + testName);
      combo_functions.enableDisableCSVDownload();
    });

    it('Test 4: Show data with null values', () => {
      cloneAndOpenVisual(trellisDashboardName, trelliscloneDashboardName);
      cloneDB = true;
      const testName = 'showNullValues';
      console.log('Starting test ' + testName);
      combo_functions.setShowNullValues('Default');
      dashboards.editVisual();
      combo_functions.setShowNullValues('First');
      dashboards.editVisual();
      combo_functions.setShowNullValues('Last');
    });

    it('Test 5: Display trellis borders', () => {
      cloneAndOpenVisual(trellisDashboardName, trelliscloneDashboardName);
      const testName = 'displayTrelisBorder';
      console.log('Starting test ' + testName);
      combo_functions.displayTrelisBorder();
    });

    it.skip('Test 6: Independent dimension scale', () => {
      cloneAndOpenVisual(trellisDashboardName, trelliscloneDashboardName);
      const testName = 'inpdDimScale';
      console.log('Starting test ' + testName);
      combo_functions.setinpdDimScaleTrellis();
    });

    it('Test 7: Independent aggregate scale', () => {
      cloneAndOpenVisual(trellisDashboardName, trelliscloneDashboardName);
      const testName = 'inpdAggrScale';
      console.log('Starting test ' + testName);
      combo_functions.setinpdAggrScaleTrellis();
    });

    it.skip('Test 8: Bar size range', () => {
      cloneAndOpenVisual(trellisDashboardName, trelliscloneDashboardName);
      cloneDB = true;
      const testName = 'barSizeRange';
      console.log('Starting test ' + testName);
      combo_functions.setBarSizeRange('10-50');
    });

    it('Test 9: Bucket Count', () => {
      cloneAndOpenVisual(trellisDashboardName, trelliscloneDashboardName);
      cloneDB = true;
      const testName = 'bucketCount';
      console.log('Starting test ' + testName);
      combo_functions.setBucketCount('30');
    });

    it('Test 10: Normalize Histogram', () => {
      cloneAndOpenVisual(trellisDashboardName, trelliscloneDashboardName);
      cloneDB = true;
      const testName = 'bucketCount';
      console.log('Starting test ' + testName);
      combo_functions.setNormalizeHistogram();
    });

    it('Test 11: Cumulative Histogram', () => {
      cloneAndOpenVisual(trellisDashboardName, trelliscloneDashboardName);
      cloneDB = true;
      const testName = 'bucketCount';
      console.log('Starting test ' + testName);
      combo_functions.setCumulativeHistogram();
    });

    it('Test 12: Positive aggregate scale start at 0', () => {
      cloneAndOpenVisual(trellisDashboardName, trelliscloneDashboardName);
      cloneDB = true;
      const testName = 'aggrScaleStart';
      console.log('Starting test ' + testName);
      combo_functions.setAggrScaleStart();
    });
  });
});
