import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';
import * as visuals from 'lib/visual';

const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'Iris';

const dashboard: Dashboard = {
  title: 'simple_histogram_visual_dashboard',
  subTitle: 'simple_histogram_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'population sheet',
      sheet_number: 1,
      visuals: [
        {
          name: 'histogramvisual1',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-histogram',
          visual_title: 'simple_histogram_visual',
          linked: 'false',
          trellis: true,
          // prettier-ignore
          is:[
            ['setTransform','translate','globalAlpha','font','setTransform','translate','clearRect','fillStyle','beginPath','rect','fill','fillStyle','beginPath','rect','fill','fillStyle','beginPath','rect','fill','fillStyle','beginPath','rect','fill','fillStyle','beginPath','rect','fill'],
            ['setTransform', 'translate','font','setTransform','translate','clearRect','fillStyle','beginPath','rect','fill','fillStyle','beginPath','rect','fill','fillStyle','beginPath','rect','fill','fillStyle','beginPath','rect','fill','fillStyle','beginPath','rect','fill',],
            ['setTransform','translate','lineWidth','strokeStyle','font','setTransform','translate','clearRect',]
          ],
          hash: [
            '4b061d87b4a269f6fbe17697a86fcb9a',
            'e93621555808de0131047cc520a899e8',
            '220201d1229a447eaf5768cdc9b56f85',
          ],
          data: {
            fields: [
              // {
              //   field_name: 'species',
              //   field_shelf: ShelfTypes.X,
              // },
              {
                field_name: 'species',
                field_shelf: ShelfTypes.Y,
              },
              {
                field_name: 'sepal_length',
                field_shelf: ShelfTypes.AGGREGATES,
              },
              {
                field_name: 'species',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['setosa', 'versicolor', 'virginica'],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Arcviz histogram visuals tests.', () => {
  before(() => {
    logins.silent();
    cy.visit('/arc/apps/home');
  });
  afterEach(() => {
    cy.window().then(
      (win: Window) =>
        (win.onbeforeunload = () => {
          return;
        })
    );
  });


  context('Histogram visual tests', () => {
    it('Test 1: Create basic histogram visuals', () => {
      dashboards.createDashboard(dashboard);
      visuals.verifyVisual(dashboard);
    });
  });
});
