import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';

const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'Restaurant Inspection SF';

const dashboard: Dashboard = {
  title: 'simple_int_map_visual_dashboard',
  subTitle: 'simple_intmap_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'inspection sheet',
      sheet_number: 1,
      visuals: [
        {
          name: 'int_heatmapvisual1',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-leaflet',
          visual_title: 'simple_int_heatmap_visual',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'business_longitude',
                field_shelf: ShelfTypes.GEO,
                change: 'Longitude',
                alias: 'Business Location Longitude',
              },
              {
                field_name: 'business_latitude',
                field_shelf: ShelfTypes.GEO,
                change: 'Latitude',
                alias: 'Business Location Latitude',
              },
              {
                field_name: 'Record Count',
                field_shelf: ShelfTypes.AGGREGATES,
                alias: 'Deficiency Code',
              },
              {
                field_name: 'business_state',
                field_shelf: ShelfTypes.COLOR,
                alias: 'Business State',
              },
              {
                field_name: 'business_name',
                field_shelf: ShelfTypes.TOOLTIP,
                alias: 'Business Name',
              },
              {
                field_name: 'inspection_date',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['2017-01-09 00:00:00'],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Arcviz interactive map visuals tests.', () => {
  before(() => {
    logins.silent();
    cy.visit('/arc/apps/home');
  });
  afterEach(() => {
    cy.window().then(
      (win: Window) =>
        (win.onbeforeunload = () => {
          return;
        })
    );
  });


  context('interactive map visual tests', () => {
    it('Test 1: Create basic interactive map visuals', () => {
      dashboards.createDashboard(dashboard);
    });
  });
});
