import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import * as combo_functions from 'lib/visual/visual_bar_line_combo_area_group';

import { dashboardWithLineVisual } from '../../../../lib/dashboard/default_dashboard';

describe('Arcviz Line visuals tests.', () => {
  context('Line visual tests colors tab tests', () => {
    before(() => {
      logins.silent();
      dashboards.createDashboard(dashboardWithLineVisual);
    });

    it('Test 1: Reverse order of color palettes', () => {
      combo_functions.reverseColorPalette();
      dashboards.closeVisual(false);
    });

    it('Test 2: Choose color palettes', () => {
      combo_functions.chooseColorPalette();

      cy.get(
        `${dashboards.locators.color_palette_10th} ${dashboards.locators.color_palette_color_boxes}`
      ).then(($boxes) => {
        const paletteColors: string[] = [];
        for (const $box of $boxes) {
          const color = $box.getAttribute('data-color');
          if (color) {
            paletteColors.push(color);
          }
        }

        cy.document().then(($doc) => {
          const canvas = '.arc-viz-overlay-container canvas.maincanvas';
          const context = ($doc.querySelector(canvas) as HTMLCanvasElement).getContext('2d');
          const linecolors = (context as any)._stack
            .filter((obj: { attr: string; val: string }) => obj?.attr === 'strokeStyle')
            .map((obj: { attr: string; val: string }) => obj.val) as string[];
          for (const linecolor of linecolors) {
            cy.log(`${paletteColors.toString()}, ${linecolor}`);
            if (!paletteColors.some((x) => x === linecolor)) {
              console.log(`${paletteColors.toString()}, ${linecolor}`);
            }
            expect(paletteColors.some((x) => x === linecolor)).to.be.true;
          }
        });
      });
      dashboards.closeVisual(false);
    });

    it('Test 3: Color opacity', () => {
      combo_functions.setColorOpacity();
      dashboards.closeVisual(false);
    });

    it('Test 4: Base colors on dimension values', () => {
      combo_functions.setFixedColorLegend();
      dashboards.closeVisual(false);
    });
  });
});
