import { logins } from 'lib/arctest';
import * as customsettings from 'lib/customsettings';
import * as dashboards from 'lib/dashboard';
import * as dsManage from 'lib/datasetManagement';
import * as functions from 'lib/functions';
import * as visual from 'lib/visual';
import * as combo_functions from 'lib/visual/visual_bar_line_combo_area_group';

import { dashboardWithLineVisual } from '../../../../lib/dashboard/default_dashboard';

const connectionName = 'samples';
const databaseName = 'main';
const tableName = 'cereals';

describe('Arcviz Line visuals tests.', () => {
  context('Line visual tests segments tab tests', () => {
    const segmentName = 'SegGrpT1_' + Math.floor(Math.random() * 1000);
    const segmentGroup = 'SegmentGroup1';
    const segmentMax = 20;
    const filterExpression = `[sodium_mg] < ${segmentMax}`;
    const datasetName = 'Cereals e2e';

    before(() => {
      logins.silent();
      cy.visit('/');
      // to avoid manipulating the existing sample Cereals dataset!
      functions.deleteDatasetFromConnection(connectionName, datasetName);
      functions.createDatasetFromConnection(connectionName, databaseName, tableName, datasetName);
      dsManage.createNewSegment(
        connectionName,
        datasetName,
        segmentName,
        filterExpression,
        segmentGroup
      );

      const dashboard2 = { ...dashboardWithLineVisual };
      dashboard2.sheets.forEach((s) => s.visuals.forEach((v) => (v.dataset_name = datasetName)));
      dashboards.createDashboard(dashboardWithLineVisual);
    });

    beforeEach(() => {
      dashboards.editVisual();
    });

    it('Test 1: Choose segment is', () => {
      const testName = 'segmentIs';
      console.log('Starting test ' + testName);
      combo_functions.addSegmentTo(customsettings.locators.shelves.appShelves.x_shelf);
      combo_functions.addSegmentToXShelf(segmentName);
      cy.get(visual.locators.y_axis_values).each(($el) => {
        cy.wrap(parseInt($el.text(), 10)).should('be.lessThan', segmentMax);
      });
    });
  });
});
