import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import * as combo_functions from 'lib/visual/visual_bar_line_combo_area_group';
import * as visual_common_test from 'lib/visual/visual_common_tests';

import { dashboardWithLineVisual } from '../../../../lib/dashboard/default_dashboard';

describe('Arcviz Line visuals tests.', () => {
  context('Line visual tests settings tab tests', () => {
    before(() => {
      logins.silent();
      dashboards.createDashboard(dashboardWithLineVisual);
    });

    it('Test 1: Set Download as CSV locale', () => {
      combo_functions.enableDisableCSVDownload();
    });

    it('Test 2: Enable Animation', () => {
      combo_functions.enableDisableAnimation();
    });

    it('Test 3: Enable/Disable Drill into context menu', () => {
      visual_common_test.enableDrillIntoFunctionality();
      visual_common_test.testDrillIntoOptionIsAvailableInContextMenu();
    });

    it('Test 4: Enable/Disable Tooltips', () => {
      combo_functions.setEnableTooltipsSetting(dashboardWithLineVisual.title, 'checked');
      dashboards.editVisual();
      combo_functions.setEnableTooltipsSetting(dashboardWithLineVisual.title, 'unchecked');
    });

    it('Test 6: Legend style', () => {
      // dashboards.createDashboard(dashboard);
      visual_common_test.testLegendPositionChangeBySwitchingTo('none');
      visual_common_test.testLegendPositionChangeBySwitchingTo('right');
      visual_common_test.testLegendPositionChangeBySwitchingTo('bottom');
      visual_common_test.testLegendPositionChangeBySwitchingTo('above');
    });

    it('Test 7: Reverse legend order', () => {
      combo_functions.reverseLegendOrder();
    });

    it('Test 8: Show selection brush', () => {
      combo_functions.setSelectionBrush();
    });

    it('Test 9: Label measure values next to point', () => {
      visual_common_test.testShowMeasureValuesSetting();
    });

    it('Test 10: Minimum cell height and width', () => {
      visual_common_test.testCellHeightWidthSetting();
    });

    it('Test 11: Ignore negative measure values', () => {
      combo_functions.setIgnoreNegativeData();
    });

    it('Test 14: Show Points', () => {
      combo_functions.setShowPoints();
    });

    it('Test 15: Ignore missing dates', () => {
      combo_functions.setIgnoreMissingDates();
    });

    it('Test 16: Set Line Style', () => {
      combo_functions.setLineStyle('basis');
      dashboards.editVisual();
      combo_functions.setLineStyle('linear');
      dashboards.editVisual();
      combo_functions.setLineStyle('bundle');
      dashboards.editVisual();
      combo_functions.setLineStyle('cardinal');
      dashboards.editVisual();
      combo_functions.setLineStyle('monotone');
      dashboards.editVisual();
      combo_functions.setLineStyle('step');
      dashboards.editVisual();
      combo_functions.setLineStyle('step-before');
    });

    it('Test 17: Show data with null values', () => {
      combo_functions.setShowNullValues('Default');
      dashboards.editVisual();
      combo_functions.setShowNullValues('First');
      dashboards.editVisual();
      combo_functions.setShowNullValues('Last');
    });

    it('Test 18: Number of rows to download in CSV', () => {
      combo_functions.setNumberofRowsinCSV('75');
    });

    it('Test 19: Maximum number of rows to fetch', () => {
      combo_functions.setNumberofRowsToFetch('10');
    });

    it('Test 20: Disable incremental queries', () => {
      combo_functions.setDisableIncrementalQueries();
    });

    it('Test 21: Auto detect time columns', () => {
      combo_functions.setAutoDetectTimeColumn();
    });

    it('Test 22: Axis scale log10', () => {
      combo_functions.setAxisScale('LOG10');
    });

    it('Test 23: Axis scale Linear', () => {
      combo_functions.setAxisScale('LINEAR');
    });

    it('Test 24: Gridlines enabled', () => {
      combo_functions.setGridLines();
    });

    it('Test 25: Positive aggregate scale start at 0', () => {
      combo_functions.setAggrScaleStart();
    });

    it('Test 26: Aggregate scale minimum', () => {
      combo_functions.setMinAggrScale();
    });

    it('Test 27: Aggregate scale maximum', () => {
      combo_functions.setMaxAggrScale();
    });

    it('Test 28: Include X-Axis label', () => {
      combo_functions.setIncludeXAxisLabel();
    });

    it('Test 29: Include Y-Axis label', () => {
      combo_functions.setIncludeYAxisLabel();
    });

    it('Test 30: Show X-Axis', () => {
      combo_functions.setShowXAXis();
    });

    it('Test 31: Show Y-Axis ', () => {
      combo_functions.setShowYAXis();
    });
  });
});
