import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';

const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'InfoSeсGlobal';

const dashboard: Dashboard = {
  title: 'simple_map_visual_dashboard',
  subTitle: 'simple_map_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'threat sheet',
      sheet_number: 1,
      visuals: [
        {
          name: 'mapvisual1',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-map',
          visual_title: 'simple_map_visual1',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'country',
                field_shelf: ShelfTypes.GEO,
                field_change: 'Country',
              },
              {
                field_name: 'count',
                field_shelf: ShelfTypes.AGGREGATES,
                sort: SortOptions.DESCENDING,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'date',
                field_shelf: ShelfTypes.TOOLTIP,
              },
              {
                field_name: 'country',
                field_shelf: ShelfTypes.DRILL,
                alias: 'Country',
              },
              {
                field_name: 'threat',
                field_shelf: ShelfTypes.DRILL,
                alias: 'Threat',
              },
              {
                field_name: 'industry',
                field_shelf: ShelfTypes.DRILL,
                alias: 'Industry',
              },
              {
                field_name: 'threat',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['scanning', 'phishing', 'spamming'],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Arcviz Map visuals tests.', () => {
  before(() => {
    logins.silent();
    cy.visit('/arc/apps/home');
  });
  afterEach(() => {
    cy.window().then(
      (win: Window) =>
        (win.onbeforeunload = () => {
          return;
        })
    );
  });


  context('Map visual tests', () => {
    it('Test 1: Create basic map visual', () => {
      dashboards.createDashboard(dashboard);
    });
  });
});
