import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';

const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'Cereals';

const dashboard: Dashboard = {
  title: 'simple_network_visual_dashboard',
  subTitle: 'simple_network_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'calories sheet',
      sheet_number: 1,
      visuals: [
        {
          name: 'barvisual1',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-network',
          visual_title: 'simple_network_visual',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'manufacturer',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.AGGREGATES,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.SOURCE_NODE_SIZE,
              },
              {
                field_name: 'cold_or_hot',
                field_shelf: ShelfTypes.TARGET_NODE_SIZE,
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.LINK_COLOR,
              },
              {
                field_name: 'manufacturer',
                field_shelf: ShelfTypes.SOURCE_NODE_COLOR,
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.TARGET_NODE_COLOR,
              },
              {
                field_name: 'cups_per_serving',
                field_shelf: ShelfTypes.TOOLTIP,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'manufacturer',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: [],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Arcviz Network visuals tests.', () => {
  before(() => {
    logins.silent();
    cy.visit('/arc/apps/home');
  });

  context('Grouped Network visual tests', () => {
    it('Test 1: Create basic network visuals', () => {
      dashboards.createDashboard(dashboard);
    });
  });
});
