import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';

const connectionName = Cypress.env('smokeConnectionName');
const databaseName = 'main';
const tableName = 'cereals';
const datasetName = 'Cereals';

const dashboard: Dashboard = {
  title: 'simple_radial_visual_dashboard',
  subTitle: 'simple_radial_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'cereal sheet',
      sheet_number: 1,
      visuals: [
        {
          name: 'radialvisual1',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-radial',
          visual_title: 'simple_radial_visual',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.AGGREGATES,
                field_aggregate: 'Sum',
                alias: 'Calories',
              },
              {
                field_name: 'complex_carbohydrates_grams',
                field_shelf: ShelfTypes.AGGREGATES,
                field_aggregate: 'Sum',
                alias: 'Carbohydrates',
              },
              {
                field_name: 'fat_grams',
                field_shelf: ShelfTypes.AGGREGATES,
                field_aggregate: 'Sum',
                alias: 'Fat',
              },
              {
                field_name: 'dietary_fiber_grams',
                field_shelf: ShelfTypes.AGGREGATES,
                field_aggregate: 'Sum',
                alias: 'Fiber',
              },
              {
                field_name: 'cups_per_serving',
                field_shelf: ShelfTypes.TOOLTIP,
              },
              {
                field_name: 'manufacturer',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: [],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Arcviz radial visuals tests.', () => {
  before(() => {
    logins.silent();
    cy.visit('/arc/apps/home');
  });
  afterEach(() => {
    cy.window().then(
      (win: Window) =>
        (win.onbeforeunload = () => {
          return;
        })
    );
  });

  // local method just for the test
  function get_time() {
    return new Date().getTime();
  }

  context('radial visual tests', () => {
    before(() => {
      // create dataset if it doesn't exist
      functions.createDatasetFromConnection(connectionName, databaseName, tableName, datasetName);
    });
    it('Test 1: Create basic radial visuals', () => {
      dashboards.createDashboard(dashboard);
    });
  });
});
