import { BackendAPI } from 'lib/backend-api';
import * as dashboards from 'lib/dashboard';
import { VisualType } from 'lib/dashboard/dashboards_interfaces';
import * as logins from 'lib/logins';
import * as visuals from 'lib/visual';
import { reverseColorPalette } from 'lib/visual';
import { testColorOpacity, testColorPaletteSelection } from 'lib/visual/visual_common_tests';

describe('Scatter Visual tests.', () => {
  context('Colors tab tests - scatter_visual_colors_tests_risk1.ts', () => {
    let visualBuilderUrl;
    before(() => {
      logins.silent();
      // I exported the result of this process
      // dashboards.createDashboard(dashboard);

      BackendAPI.dashboards
        .import({
          fileName: 'scatter_plot_prettified.json',
          dataconnectionName: 'samples',
          workspaceName: 'Public',
          randomised: false,
        })
        .then(({ dashboard }) => {
          visualBuilderUrl = `/arc/apps/builder/${dashboard.id}`;
          cy.visit(visualBuilderUrl);
        });
    });

    beforeEach(() => {
      // visit is done in the cloneDashboard call
      dashboards.editVisual();
      dashboards.chooseVisualTab('colors');
    });

    it('1: Reverse order of color palettes', () => {
      const testName = 'reverseColorPalettes';
      console.log('Starting test ' + testName);
      reverseColorPalette();
      // add reverseColorDetection!
    });

    it('2: Choose color palettes', () => {
      const testName = 'chooseColorPalette';
      console.log('Starting test ' + testName);
      testColorPaletteSelection(VisualType.scatter);
      visuals.closeVisual(false);
    });

    it('3: Color opacity', () => {
      const testName = 'setColorOpacity';
      console.log('Starting test ' + testName);
      testColorOpacity();
      visuals.closeVisual(false);
    });

    // This test does not do anything useful actually
    it('4: Base colors on dimension values', () => {
      const testName = 'fixedColorLegend';
      console.log('Starting test ' + testName);
      cy.get('[data-test-id="checkbox-fixed_color_legend"]').check();
      visuals.closeVisual(false);
    });
  });
});
