import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';
import * as drillinto from 'lib/drillinto';
import * as functions from 'lib/functions';
import * as visual from 'lib/visual';
import * as combo_functions from 'lib/visual/visual_bar_line_combo_area_group';

const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'Cereals';
const datasetName2 = 'Iris';
const dashboardName = 'Cereal Scatters DB';
const dashboard: Dashboard = {
  title: dashboardName,
  subTitle: 'simple_scatter_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'sheet1',
      sheet_number: 1,
      visuals: [
        {
          name: 'scattervisual1',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-scatter',
          visual_title: 'simple_scatter_visual1',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'manufacturer_code',
                field_shelf: ShelfTypes.X,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.X,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'cold_or_hot',
                field_shelf: ShelfTypes.Y,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.Y,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'protein_grams',
                field_shelf: ShelfTypes.SIZE,
              },
              {
                field_name: 'sugars_grams',
                field_shelf: ShelfTypes.TOOLTIP,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['50', '70', '80', '90', '100', '110'],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Scatter Visual tests.', () => {
  context('Data tab tests', () => {
    before(() => {
      logins.silent();
      dashboards.createDashboard(dashboard);
    });

    beforeEach(() => {
      dashboards.editVisual();
      dashboards.chooseVisualTab('data');
    });

    it('1: Switch data set', () => {
      visual.switchVisualDataSet(datasetName2);
      dashboards.editVisual();
      cy.get('[data-test-id="app-dataset"]').should('have.text', datasetName2);
      dashboards.closeVisual(false);
    });
  });
});
