/* This script contains verification of following tests
  https://arcadiadata.atlassian.net/browse/ARC-10087
*/
import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes } from 'lib/dashboard/dashboards_interfaces';
import { verifyVisualHash } from 'lib/explore_visual/functions';
import * as functions from 'lib/functions';
import * as visual from 'lib/visual';
import { CSSselector } from 'locators/common/CSSselector';

const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'World Life Expectancy';
const transitionValues = ['1900', '1901', '1902', '1903', '2010'];
const scatterVisual = {
  type: '.chart-icon-scatter',
  initial_hash: 'eab97a3acc8ad58104d6f38c0c1589e9',
  hash: 'a4fe03f145222ea8d9b8cf41704ab80a',
};
const dashboard: Dashboard = {
  title: 'transition_scatter_visual_dashboard',
  subTitle: 'transition_scatter_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'sheet1',
      sheet_number: 1,
      visuals: [
        {
          name: 'scattervisual1',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-scatter',
          visual_title: 'simple_scatter_visual1',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'gdp_per_capita',
                field_shelf: ShelfTypes.X,
                alias: 'GDP',
              },
              {
                field_name: 'life_expectancy',
                field_shelf: ShelfTypes.Y,
                alias: 'Life Expectancy',
              },
              {
                field_name: 'country',
                field_shelf: ShelfTypes.COLOR,
              },
              {
                field_name: 'year',
                field_shelf: ShelfTypes.TRANSITION,
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Scatter Visual tests.', () => {
  before(() => {
    logins.silent();
    cy.visit('/arc/apps/home');
  });

  context('ARC-10087', () => {
    it('1: Create scatter visual with transition field and assert the visual with transitions', () => {
      dashboards.createDashboard(dashboard);
      visual.editSpecificVisual(1);

      cy.get(CSSselector.xAxis_field_name).should('have.text', 'GDP');
      cy.get(CSSselector.yAxis_field_name).should('have.text', 'Life Expectancy');
      cy.get(visual.locators.x_axis_values)
        .should('have.length.greaterThan', 0)
        .and('be.visible');
      cy.get(visual.locators.y_axis_values)
        .should('have.length.greaterThan', 0)
        .and('be.visible');
      cy.get(visual.locators.animation.controller)
        .should('be.visible')
        .find('span')
        .should('have.length', 6);
      dashboards.closeVisual(false);
      cy.wait(2000); // wait to close the visual properly

      cy.get(CSSselector.dashboard.visual.x_axis)
        .should('have.length.greaterThan', 0)
        .and('be.visible');
      cy.get(visual.locators.y_axis_values.replace('chart-container', 'app-selected-list'))
        .should('have.length.greaterThan', 0)
        .and('be.visible');

      cy.log('Verify inital visual');
      verifyVisualHash(
        scatterVisual.initial_hash,
        '[data-test-id="main-app-arc-app-container"] [data-test-id="visual-list-item"]:nth-child(1) canvas.maincanvas'
      );

      cy.get(visual.locators.animation.value).should('have.text', transitionValues[0]);
      cy.get(visual.locators.animation.play)
        .should('be.visible')
        .click({ force: true });
      cy.wait(200); // wait to change the value of animation controller
      cy.get(visual.locators.animation.pause)
        .should('be.visible')
        .click({ force: true });
      cy.get(visual.locators.animation.value).should('have.text', transitionValues[1]);
      cy.get(visual.locators.animation.forward)
        .should('be.visible')
        .click({ force: true });
      cy.get(visual.locators.animation.value).should('have.text', transitionValues[3]);
      cy.get(visual.locators.animation.back)
        .should('be.visible')
        .click({ force: true });
      cy.get(visual.locators.animation.value).should('have.text', transitionValues[2]);
      cy.get(visual.locators.animation.fast_forward)
        .should('be.visible')
        .click({ force: true });
      cy.get(visual.locators.animation.value).should('have.text', transitionValues[4]);
      cy.wait(1000); // wait to change the scatters of visual
      cy.get(visual.locators.animation.fast_back)
        .should('be.visible')
        .click({ force: true });
      cy.get(visual.locators.animation.value).should('have.text', transitionValues[0]);
      cy.wait(1000); // wait to change the scatters of visual

      cy.log('Verify animated visual');
      verifyVisualHash(
        scatterVisual.hash,
        '[data-test-id="main-app-arc-app-container"] [data-test-id="visual-list-item"]:nth-child(1) canvas.maincanvas'
      );
    });
  });
});
