import { logins } from 'lib/arctest';
import * as customsettings from 'lib/customsettings';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';
import * as dsManage from 'lib/datasetManagement';
import * as functions from 'lib/functions';
import * as dsPerm from 'lib/permissions';
import * as visuals from 'lib/visual';
import * as combo_functions from 'lib/visual/visual_bar_line_combo_area_group';
import * as scatter_functions from 'lib/visual/visual_scatter_boxplot';

const databaseName = 'main';
const tableName = 'cereals';

const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'Cereals e2e';
const dashboardName = 'Cereal Scatters DB';
const dashboard: Dashboard = {
  title: dashboardName,
  subTitle: 'simple_scatter_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'sheet1',
      sheet_number: 1,
      visuals: [
        {
          name: 'scattervisual1',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-scatter',
          visual_title: 'simple_scatter_visual1',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'Segment',
                field_shelf: ShelfTypes.X,
              },
              {
                field_name: 'manufacturer_code',
                field_shelf: ShelfTypes.X,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.X,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'cold_or_hot',
                field_shelf: ShelfTypes.Y,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.Y,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'protein_grams',
                field_shelf: ShelfTypes.SIZE,
              },
              {
                field_name: 'sugars_grams',
                field_shelf: ShelfTypes.TOOLTIP,
                sort: SortOptions.ASCENDING,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['50', '70', '80', '90', '100', '110'],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Scatter Visual tests.', () => {
  context('Segment tab tests', () => {
    const segmentName = 'SegGrpT1_' + Math.floor(Math.random() * 1000);
    const segmentGroup = 'SegmentGroup1';
    const filterExpression = '[calories] < 120';

    before(() => {
      logins.silent();
      cy.visit('/');
      // to avoid manipulating the existing sample Cereals dataset!
      functions.deleteDatasetFromConnection(connectionName, datasetName);
      functions.createDatasetFromConnection(connectionName, databaseName, tableName, datasetName);
      functions.navigate_to_data();
      dsPerm.selectDataset(connectionName, datasetName);
      dsManage.createNewSegment(
        connectionName,
        datasetName,
        segmentName,
        filterExpression,
        segmentGroup
      );

      dashboards.createDashboard(dashboard);
    });

    it('1: Choose segment is', () => {
      dashboards.editVisual();
      const testName = 'segmentIs';
      cy.log('Starting test ' + testName);
      scatter_functions.setSegment(segmentName);
      cy.get(visuals.locators.visual.y_axis + ' > g.tick text').each(($el) => {
        cy.wrap(parseInt($el.text(), 10)).should('be.lessThan', 120);
      });
    });

    // Due to ARC-11671 skipping it. Will remove it in future
    it.skip('2: Choose segment All', () => {
      const testName = 'segmentAll';
      cy.log('Starting test ' + testName);
      dashboards.editVisual();
      dashboards.chooseVisualTab('segments');

      combo_functions.addSegmentTo(customsettings.locators.app_shelves.x_shelf);
      combo_functions.addSegmentToXShelf(segmentName);
      cy.get(visuals.locators.y_axis_values).each(($el) => {
        cy.wrap(parseInt($el.text(), 10)).should('be.lessThan', 120);
      });
      visuals.closeVisual(false);
    });
  });
});
