import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';
import * as visual from 'lib/visual';

const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'Iris';

const dashboard: Dashboard = {
  title: 'simple_scatter_visual_dashboard',
  subTitle: 'simple_scatter_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'species sheet',
      sheet_number: 1,
      visuals: [
        {
          name: 'scattervisual1',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-scatter',
          visual_title: 'simple_scatter_visual1',
          linked: 'false',
          // prettier-ignore
          is: ['setTransform', 'translate', 'font', 'textBaseline', 'textAlign', 'globalAlpha', 'clearRect', 'beginPath', 'arc', 'fillStyle', 'fill', 'closePath', 'fillStyle', 'measureText', 'beginPath', 'arc', 'fillStyle', 'fill', 'closePath', 'fillStyle', 'measureText', 'beginPath', 'arc', 'fillStyle', 'fill', 'closePath', 'fillStyle', 'measureText'],
          hash: '78399dc3501a0b0ddaae1dcb43370871',
          data: {
            fields: [
              {
                field_name: 'petal_area',
                field_shelf: ShelfTypes.X,
              },
              {
                field_name: 'sepal_area',
                field_shelf: ShelfTypes.Y,
              },
              {
                field_name: 'species',
                field_shelf: ShelfTypes.COLOR,
              },
              {
                field_name: 'id',
                field_shelf: ShelfTypes.TOOLTIP,
              },
              {
                field_name: 'species',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['setosa', 'versicolor', 'virginica'],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Scatter Visual tests.', () => {
  before(() => {
    logins.silent();
    cy.visit('/arc/apps/home');
  });
  afterEach(() => {
    cy.window().then(
      (win: Window) =>
        (win.onbeforeunload = () => {
          return;
        })
    );
  });

  context('Basic dashboard creation', () => {
    it('1: Create scatter visual', () => {
      dashboards.createDashboard(dashboard);
      visual.verifyVisual(dashboard);
    });
  });
});
