import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';

const connectionName = Cypress.env('smokeConnectionName');
const databaseName = 'main';
const tableName = 'cereals';
const datasetName = 'Cereals';

const dashboard: Dashboard = {
  title: 'simple_sparkline_visual_dashboard',
  subTitle: 'simple_sparkline_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'cereals',
      sheet_number: 1,
      visuals: [
        {
          name: 'sparklinevisual1',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-sparklines',
          visual_title: 'simple_sparkline_visual',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.X,
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.Y,
              },
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: [
                    'Apple_Jacks',
                    'Bran_Flakes',
                    'Cheerios',
                    'Cinnamon_Toast_Crunch',
                    'Clusters',
                  ],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Arcviz sparkline visuals tests.', () => {
  before(() => {
    logins.silent();
    cy.visit('/arc/apps/home');
  });
  afterEach(() => {
    cy.window().then(
      (win: Window) =>
        (win.onbeforeunload = () => {
          return;
        })
    );
  });

  // local method just for the test
  function get_time() {
    return new Date().getTime();
  }

  context('Sparkline visual tests', () => {
    before(() => {
      // create dataset if it doesn't exist
      functions.createDatasetFromConnection(connectionName, databaseName, tableName, datasetName);
    });
    it('Test 1: Create basic sparkline visuals', () => {
      dashboards.createDashboard(dashboard);
    });
  });
});
