import { logins } from 'lib/arctest';
import { createDashboard } from 'lib/dashboard';
import { Dashboard, ShelfTypes, SortOptions } from 'lib/dashboard/dashboards_interfaces';

const connectionName = Cypress.env('smokeConnectionName');
const datasetName1 = 'Iris';
const datasetName2 = 'Cereals';

const dashboard: Dashboard = {
  title: 'simple_table_visuals_dashboard',
  subTitle: 'simple_table_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'iris details sheet',
      sheet_number: 1,
      visuals: [
        {
          name: 'tablevisual1',
          connection_name: connectionName,
          dataset_name: datasetName1,
          visual_type: '.chart-icon-table',
          visual_title: 'simple_table_visual1',
          linked: 'true',
          data: {
            fields: [
              {
                field_name: 'species',
                field_shelf: ShelfTypes.DIMENSIONS,
                sort: SortOptions.DESCENDING,
              },
              {
                field_name: 'sepal_length',
                field_shelf: ShelfTypes.AGGREGATES,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Average',
              },
              {
                field_name: 'petal_ratio',
                field_shelf: ShelfTypes.TOOLTIP,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'species',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['virginica', 'versicolor'],
                },
              },
            ],
          },
        },
        {
          name: 'tablevisual2',
          connection_name: connectionName,
          dataset_name: datasetName2,
          visual_type: '.chart-icon-table',
          visual_title: 'simple_table_visual2',
          linked: 'true',
          data: {
            fields: [
              {
                field_name: 'cereal_name',
                field_shelf: ShelfTypes.DIMENSIONS,
                sort: SortOptions.DESCENDING,
              },
              {
                field_name: 'manufacturer',
                field_shelf: ShelfTypes.DIMENSIONS,
                sort: SortOptions.ASCENDING,
                alias: 'MANUFACTURE',
              },
              {
                field_name: 'calories',
                field_shelf: ShelfTypes.AGGREGATES,
                sort: SortOptions.ASCENDING,
                field_aggregate: 'Sum',
              },
              // {
              //   field_name: 'species',
              //   field_shelf: ShelfTypes.FILTERS,
              //   field_pick_filter: {
              //     values: ['virginica', 'versicolor'],
              //   },
              // },
            ],
          },
        },
      ],
    },
  ],
};

describe('Arcviz Table visuals tests.', () => {
  before(() => {
    logins.silent();
    cy.visit('/arc/apps/home');
  });

  context('Table visual tests', () => {
    it('Test 1: Create basic table visual', () => {
      createDashboard(dashboard);
    });
  });
});
