import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes } from 'lib/dashboard/dashboards_interfaces';

const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'US State Populations Over Time';

const dashboard: Dashboard = {
  title: 'simple_wordcloud_visual_dashboard',
  subTitle: 'simple_wordcloud_visual_dashboard_subtitle',
  sheets: [
    {
      sheet_name: 'population',
      sheet_number: 1,
      visuals: [
        {
          name: 'wordcloudvisual1',
          connection_name: connectionName,
          dataset_name: datasetName,
          visual_type: '.chart-icon-wordcloud',
          visual_title: 'simple_wordcloud_visual',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'state',
                field_shelf: ShelfTypes.DIMENSIONS,
              },
              {
                field_name: 'population',
                field_shelf: ShelfTypes.AGGREGATES,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'year',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['2010'],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Arcviz Word Cloud visuals tests.', () => {
  before(() => {
    logins.silent();
    cy.visit('/');
  });

  context('Word Cloud visual tests', () => {
    it('Test 1: Create basic word cloud visuals', () => {
      dashboards.createDashboard(dashboard);
    });
  });
});
