import { logins } from 'lib/arctest';
import { BackendAPI } from 'lib/backend-api';
import { createConnection } from 'lib/connections';
import { TEST_CONNECTIONS } from 'lib/connections/connections';
import * as dashboards from 'lib/dashboard';
import { datasetName } from 'lib/dashboard/default_dashboard';
import * as functions from 'lib/functions';
import * as dsPerm from 'lib/permissions';
import { CSSselector } from 'locators';

const dataset = {
  database: 'mrf',
  table: 'mrf_ent_report',
  datasetName: 'mrf_ent_report',
};

describe('VIZ-650: on Hive connection date format requires from_timestamp should work', () => {
  beforeEach(() => {
    cy.window().then(
      (win: Window) =>
        (win.onbeforeunload = () => {
          return;
        })
    );
    cy.fixture('users.json').then((data) => {
      logins.silent(data.admin.username, data.admin.password);
    });
  });
  afterEach(() => {
    logins.logout();
  });

  const getDimensionShelfFirstItem = () =>
    cy.get(`[data-test-id*="dimensions_shelf-app-shelf-item"]:first:visible`).scrollIntoView();

  const setDateFormat = (dateFormat: string) => {
    getDimensionShelfFirstItem().click(); // this opens up the properties panel
    cy.get('[data-test-id="properties-panel"]')
      .scrollIntoView()
      .find('[data-test-id="date-functions"]')
      .click();

    cy.get('[data-test-id="properties-panel"]')
      .scrollIntoView()
      .find(`ul li a:contains(${dateFormat})${dateFormat === 'Year' ? ':first' : ''}`)
      .click();
    cy.get('[data-test-id="viz-refresh"]').click({ force: true });
    functions.waitForVisualsLoaded(60000);
  };

  it('Date format requires from_timestamp(Year, Year and Month Year and Week, Year and Day) works on Hive connection', () => {
    createConnection(TEST_CONNECTIONS.test_hive);
    functions.createDatasetFromConnection(
      TEST_CONNECTIONS.test_hive.connection_name,
      dataset.database,
      dataset.table,
      dataset.datasetName
    );
    functions.navigate_to_data();
    dsPerm.selectDataset(TEST_CONNECTIONS.test_hive.connection_name, datasetName);
    BackendAPI.dashboards
      .import({
        fileName: 'hive-date-format-pretty.json',
        dataconnectionName: TEST_CONNECTIONS.test_hive.connection_name,
        workspaceName: 'Public',
      })
      .then(({ dashboard }) => {
        const visualBuilderUrl = `/arc/apps/builder/${dashboard.id}`;
        cy.visit(visualBuilderUrl);
      });
    dashboards.chooseVisualTab('data');

    setDateFormat('Year and Week');
    cy.get('tbody td:contains(2023-W21)').should('exist');
    cy.get('tbody td:contains(2023-W22)').should('exist');
    cy.get('tbody td:contains(2023-W23)').should('exist');

    setDateFormat('Year and Month');
    cy.get('tbody td:contains(2023-M05)').should('exist');
    cy.get('tbody td:contains(2023-M06)').should('exist');

    setDateFormat('Year and Quarter');
    cy.get('tbody td:contains(2023-Q2)').should('exist');

    setDateFormat('Year');
    cy.get('tbody td:contains(2,023)').should('exist');
  });
});

// If the data ever disappears in hive, this is the underlying data, to import
// "report_dt","app_id"
// 2023-05-31,7
// 2023-06-02,5
// 2023-06-04,3
// 2023-05-28,10
// 2023-06-01,6
// 2023-06-06,1
// 2023-05-30,8
// 2023-06-03,4
// 2023-05-29,9
// 2023-06-05,2
