import * as arctest from 'lib/arctest';
import * as visuals from 'lib/visual';

/*
 * Verify that the SQL builder visualization works
 * Assumes the existence of the 'US State Populations Over Time' dataset
 *   in the Cypress.env("smokeConnectionName") data connection
 */

describe('barlinearea measure trellis Viz', () => {
  before(() => {
    arctest.logins.silent();
    cy.visit('/');
  });

  function drawIt(chartIcon: string, includeColor?: boolean) {
    measuresInSingleShelf(chartIcon, includeColor);
    measuresInTwoShelves(chartIcon, includeColor);
  }

  // If measuresInSingleShelf (all measures on Y axis) then the trellis values
  // ordered by their order in Y axis shelf
  function verifyTrellisMeasuresInSingleShelf() {
    cy.get('.viz-td-header').should('have.length', 3);

    cy.get('.viz-td-header').each(($el, i) => {
      if (i === 0) {
        expect($el.text()).to.eq('');
      }

      if (i === 1) {
        expect($el.text()).to.eq('Record Count');
      }

      if (i === 2) {
        expect($el.text()).to.eq('sum(population)');
      }
    });
  }

  // If measuresInTwoShelves then the trellis values ordered reverse alphabetically
  function verifyTrellisMeasuresInTwoShelves() {
    cy.get('.viz-td-header').should('have.length', 3);

    cy.get('.viz-td-header').each(($el, i) => {
      if (i === 0) {
        expect($el.text()).to.eq('');
      }

      if (i === 1) {
        expect($el.text()).to.eq('sum(population)');
      }

      if (i === 2) {
        expect($el.text()).to.eq('Record Count');
      }
    });
  }

  function measuresInTwoShelves(chartIcon: string, includeColor?: boolean) {
    arctest.logins.silent();
    arctest.misc.newDashboardEnterModal(
      Cypress.env('smokeConnectionName'),
      'US State Populations Over Time'
    );

    arctest.screen.builder.modal.switchToVisualType(chartIcon);

    // Create a chart
    arctest.screen.builder.modal.addShelfColumn('x_shelf', 'Record Count');
    arctest.screen.builder.modal.addShelfColumn('x_shelf', 'year');
    arctest.screen.builder.modal.addShelfColumn('y_shelf', 'population');
    if (includeColor) {
      arctest.screen.builder.modal.addShelfColumn('color_shelf', 'state');
    }

    arctest.screen.builder.modal.refreshVisual();
    cy.get(arctest.screen.builder.modal.locators.refresh_icon).should('be.visible');

    verifyTrellisMeasuresInTwoShelves();

    cy.get(arctest.screen.builder.modal.locators.switch_xy).click();
    cy.get(arctest.screen.builder.modal.locators.refresh_icon).should('be.visible');

    verifyTrellisMeasuresInTwoShelves();
    visuals.saveVisual();
  }

  function measuresInSingleShelf(chartIcon: string, includeColor?: boolean) {
    arctest.misc.newDashboardEnterModal(
      Cypress.env('smokeConnectionName'),
      'US State Populations Over Time'
    );

    arctest.screen.builder.modal.switchToVisualType(chartIcon);

    // Create a chart
    arctest.screen.builder.modal.addShelfColumn('x_shelf', 'year');
    arctest.screen.builder.modal.addShelfColumn('y_shelf', 'Record Count');
    arctest.screen.builder.modal.addShelfColumn('y_shelf', 'population');

    if (includeColor) {
      arctest.screen.builder.modal.addShelfColumn('color_shelf', 'state');
    }

    arctest.screen.builder.modal.refreshVisual();
    cy.get(arctest.screen.builder.modal.locators.refresh_icon).should('be.visible');

    verifyTrellisMeasuresInSingleShelf();

    cy.get(arctest.screen.builder.modal.locators.switch_xy).click();
    cy.get(arctest.screen.builder.modal.locators.refresh_icon).should('be.visible');

    verifyTrellisMeasuresInSingleShelf();

    visuals.saveVisual();
  }

  it('visual: 1: bar Simple', () => {
    drawIt('.chart-icon-trellis-bars');
  });

  it('visual: 2: line Simple', () => {
    drawIt('.chart-icon-trellis-lines');
  });

  it('visual: 3: area Simple', () => {
    drawIt('.chart-icon-trellis-areas');
  });

  it('visual: 4: stacked Bar', () => {
    drawIt('.chart-icon-trellis-bars', true);
  });

  it('visual: 5: stacked Line', () => {
    drawIt('.chart-icon-trellis-lines', true);
  });

  it('visual: 6: stacked Area', () => {
    drawIt('.chart-icon-trellis-areas', true);
  });
});
