import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import { Dashboard, ShelfTypes } from 'lib/dashboard/dashboards_interfaces';
import * as functions from 'lib/functions';
import { setNumberofRowsToFetch } from 'lib/visual/visual_bar_line_combo_area_group';
import * as visual_functions from 'lib/visual/visual_functions';

const data = {
  connectionName: Cypress.env('smokeConnectionName'),
  datasetName: 'Global Information Security Threats',
};

const trellisDashboard: Dashboard = {
  title: 'trellis_title',
  subTitle: 'trellis_subtitle',
  sheets: [
    {
      sheet_name: 'Sheet 1',
      sheet_number: 1,
      visuals: [
        {
          name: 'trellisvisual',
          connection_name: data.connectionName,
          dataset_name: data.datasetName,
          visual_type: '.chart-icon-trellis-bars',
          visual_title: 'simple_trellis',
          linked: 'false',
          data: {
            fields: [
              {
                field_name: 'country',
                field_shelf: ShelfTypes.X,
              },
              {
                field_name: 'industry',
                field_shelf: ShelfTypes.Y,
              },
              {
                field_name: 'Record Count',
                field_shelf: ShelfTypes.Y,
                field_aggregate: 'Sum',
              },
              {
                field_name: 'industry',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['Communications', 'Web Services'],
                },
              },
              {
                field_name: 'country',
                field_shelf: ShelfTypes.FILTERS,
                field_pick_filter: {
                  values: ['ABW', 'AFG', 'ARE', 'AUS'],
                },
              },
            ],
          },
        },
      ],
    },
  ],
};

describe('Trellis test', () => {
  before(() => {
    logins.silent();
    cy.visit('/');
    dashboards.createDashboard(trellisDashboard);
    setNumberofRowsToFetch('100');
  });

  it('Should take screenshot of the original trellis', () => {
    cy.get('.dash-viz-wrapper').scrollIntoView();
    cy.get('.dash-viz-wrapper .viz-chart').matchImage();
  });

  it('Should turn the Independent aggregate scale across trellis settings on', () => {
    visual_functions.openSection('settings', 'Trellis');
    const independentAggrScaleLocator = '[data-test-id="checkbox-scale_agg_independent"]';
    cy.get(independentAggrScaleLocator)
      .scrollIntoView()
      .check();

    cy.get('.dash-viz-wrapper').scrollIntoView();
    cy.get('.dash-viz-wrapper .viz-chart').matchImage();
  });

  it('Should turn the Independent dimension scale across trellis settings on', () => {
    visual_functions.openSection('settings', 'Trellis');
    cy.get('[data-test-id="checkbox-scale_dim_independent"]')
      .scrollIntoView()
      .check();

    cy.get('.dash-viz-wrapper').scrollIntoView();
    cy.get('.dash-viz-wrapper .viz-chart').matchImage();
  });
});
