/*
 * Verify that the SQL builder visualization works
 * Assumes the existence of the 'US State Populations Over Time' dataset
 *   in the Cypress.env("smokeConnectionName") data connection
 */
import * as arctest from 'lib/arctest';

describe('Column stats Viz', () => {
  before(() => {
    arctest.logins.silent();
    cy.visit('/');
  });

  it('simple Column stats', () => {
    arctest.misc.newDashboardEnterModal(
      Cypress.env('smokeConnectionName'),
      'US State Populations Over Time'
    );

    // Click over to the SQL type visual, and verify it displays a table
    cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('jsonselect');
    arctest.screen.builder.modal.switchToVisualType('.chart-icon-column-stats');

    arctest.screen.builder.modal.refreshVisual();
    // One for min/max
    cy.wait('@jsonselect');

    // One for year
    cy.wait('@jsonselect');

    // One for state
    cy.wait('@jsonselect');

    // One for population
    cy.wait('@jsonselect');
    cy.get(arctest.screen.builder.modal.locators.refresh_icon).should('be.visible');

    // Wait for the draw to complete
    cy.wait(2000);

    // Verify
    cy.get('.arc-viz-type-column-stats td').should('have.length', 24);

    cy.get('.arc-viz-type-column-stats rect').should('have.length', 37);
  });

  it('Column stats popup should not have NaN values', () => {
    cy.get('.arc-viz-type-column-stats')
      .find('.column-data td svg rect:first')
      .trigger('mouseover');

    cy.get('#tooltip').should('be.visible');
    cy.get('#tooltip')
      .find('table tbody tr')
      .each(($el) => {
        cy.wrap($el)
          .find('td')
          .eq(1)
          .should('not.have.text', 'NaN');
      });
  });
});
