/*
 * Verify that the SQL builder visualization works
 * Assumes the existence of the 'US State Populations Over Time' dataset
 *   in the Cypress.env("smokeConnectionName") data connection
 */
import * as arctest from 'lib/arctest';
import * as visuals from 'lib/visual';

describe('combo Viz', () => {
  before(() => {
    arctest.logins.silent();
    cy.visit('/');
  });

  it('visual: simple combo', () => {
    arctest.misc.newDashboardEnterModal(
      Cypress.env('smokeConnectionName'),
      'US State Populations Over Time'
    );

    // Click over to the SQL type visual, and verify it displays a table
    arctest.screen.builder.modal.switchToVisualType('.chart-icon-combo');

    // Create a chart
    arctest.screen.builder.modal.addShelfColumn('x_shelf', 'year');
    arctest.screen.builder.modal.addShelfColumn('bar_measure', 'population');
    arctest.screen.builder.modal.addShelfColumn('line_measure', 'Record Count');

    cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('jsonselect');
    arctest.screen.builder.modal.refreshVisual();
    cy.wait('@jsonselect');
    cy.get(arctest.screen.builder.modal.locators.refresh_icon).should('be.visible');

    cy.get(arctest.screen.builder.modal.locators.axis_labels).each(($el, i) => {
      if (i === 0) {
        expect($el.text()).to.eq('year');
      }

      if (i === 1) {
        expect($el.text()).to.eq('sum(population), Record Count');
      }
    });

    arctest.screen.builder.modal.setSecondaryAxis('Record Count');

    arctest.screen.builder.modal.refreshVisual();

    // Wait for the visual to refresh
    cy.wait(2000);

    cy.get(arctest.screen.builder.modal.locators.axis_labels).each(($el, i) => {
      if (i === 0) {
        expect($el.text()).to.eq('year');
      }

      if (i === 1) {
        expect($el.text()).to.eq('sum(population)');
      }

      if (i === 2) {
        expect($el.text()).to.eq('Record Count');
      }
    });

    visuals.saveVisual();

    // save dash - since we dont show a view link and save button remains highlighted on discarding visual changes
    cy.intercept('POST', '/arc/reports/savedashboard').as('saveClicked');
    cy.get('#app-save-link').click();
    cy.wait('@saveClicked');
  });
});
