/*
 * Verify that the SQL builder visualization works
 * Assumes the existence of the 'US State Populations Over Time' dataset
 *   in the Cypress.env("smokeConnectionName") data connection
 */
import * as arctest from 'lib/arctest';
import * as visuals from 'lib/visual';

describe('correlation Viz', () => {
  before(() => {
    arctest.logins.silent();
    cy.visit('/');
  });

  it('visual: simple corelation', () => {
    arctest.misc.newDashboardEnterModal(
      Cypress.env('smokeConnectionName'),
      'US State Populations Over Time'
    );

    // Click over to the SQL type visual, and verify it displays a table
    arctest.screen.builder.modal.switchToVisualType('.chart-icon-corelation');

    // Create a chart
    arctest.screen.builder.modal.addShelfColumn('dimensions_shelf', 'year');
    arctest.screen.builder.modal.addShelfColumn('dimensions_shelf', 'state');
    arctest.screen.builder.modal.addShelfColumn('aggregates_shelf', 'population');

    cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('jsonselect');
    arctest.screen.builder.modal.refreshVisual();
    cy.wait('@jsonselect');
    cy.get(arctest.screen.builder.modal.locators.refresh_icon).should('be.visible');

    cy.get('.arc-viz-type-corelation canvas').should('have.length', 3);
    cy.get('.arc-viz-type-corelation canvas.maincanvas').then(($c) => {
      const w = $c.attr('width');
      const h = $c.attr('height');
      console.log('dims', w, h);

      if (w && h) {
        expect(+w / +h).to.be.greaterThan(3);
        expect(+w / +h).to.be.lessThan(4);
      }
    });

    cy.get("#dimensions_shelf span[title='year']").then(($year) => {
      const yearPos = $year.offset();
      cy.get("#dimensions_shelf span[title='state']").then(($state) => {
        const statePos = $state.offset();
        if (yearPos && statePos) {
          cy.get("#dimensions_shelf span[title='year']")
            .trigger('mousedown', {
              which: 1,
              pageX: yearPos.left + 1,
              pageY: yearPos.top + 1,
            })
            .trigger('mousemove', {
              which: 1,
              pageX: statePos.left + 1,
              pageY: statePos.top + 1,
            })
            .trigger('mouseup');
        }
      });
    });

    cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('jsonselect');
    arctest.screen.builder.modal.refreshVisual();
    cy.wait('@jsonselect');
    cy.get(arctest.screen.builder.modal.locators.refresh_icon).should('be.visible');

    // Wait for the draw to complete
    cy.wait(2000);

    cy.get('.arc-viz-type-corelation canvas').should('have.length', 3);

    cy.get('.arc-viz-type-corelation canvas.maincanvas').then(($c) => {
      const w = $c.attr('width');
      const h = $c.attr('height');
      console.log('dims', w, h);

      if (w && h) {
        expect(+h / +w).to.be.greaterThan(3);
        expect(+h / +w).to.be.lessThan(4);
      }
    });

    visuals.saveVisual();

    // save dash - since we dont show a view link and save button remains highlighted on discarding visual changes
    cy.intercept('POST', '/arc/reports/savedashboard').as('saveClicked');
    cy.get('#app-save-link').click();
    cy.wait('@saveClicked');
  });
});
