/*
 * Verify that the SQL builder visualization works
 * Assumes the existence of the 'US State Populations Over Time' dataset
 *   in the Cypress.env("smokeConnectionName") data connection
 */
import * as arctest from 'lib/arctest';
import * as visuals from 'lib/visual';

describe('crosstab Viz', () => {
  afterEach(() => {
    cy.window().then(
      (win: Window) =>
        (win.onbeforeunload = () => {
          return null;
        })
    );
  });

  it('visual: simple crosstab', () => {
    arctest.logins.silent();
    arctest.misc.newDashboardEnterModal(
      Cypress.env('smokeConnectionName'),
      'US State Populations Over Time'
    );

    // Click over to the SQL type visual, and verify it displays a table
    arctest.screen.builder.modal.switchToVisualType('.chart-icon-crosstab');

    // Create a chart
    arctest.screen.builder.modal.addShelfColumn('y_dimensions_shelf', 'year');
    arctest.screen.builder.modal.addShelfColumn('y_dimensions_shelf', 'state');
    arctest.screen.builder.modal.addShelfColumn('aggregates_shelf', 'population');

    arctest.screen.builder.modal.setExpansion('year');

    cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('jsonselect');
    arctest.screen.builder.modal.refreshVisual();
    cy.wait('@jsonselect');
    cy.get(arctest.screen.builder.modal.locators.refresh_icon).should('be.visible');

    cy.get('.viz-crosstab-table .slick-row').should('have.length', 2);

    cy.get(".viz-crosstab-table .slick-row span[data-dimidx='0'] .fa-plus-square-o").click();
    cy.wait('@jsonselect');

    cy.get('.viz-crosstab-table .slick-row').should('have.length', 15);
    cy.get('.slick-row:nth-child(3) .slick-cell:nth-child(1)').then(($e) => {
      assert.equal($e.text(), '1790');
    });

    cy.get('.slick-row:nth-child(4) .slick-cell:nth-child(1)').then(($e) => {
      assert.equal($e.text(), '1800');
    });

    cy.get(".viz-crosstab-table .slick-row span[data-dimidx='0'] .fa-minus-square-o").click();

    cy.get('.viz-crosstab-table .slick-row').should('have.length', 2);

    cy.get("span[data-dim='1'][data-dimidx='1']").click();

    cy.get('.viz-crosstab-table .slick-row').should('have.length', 15);
    cy.get('.slick-row:nth-child(3) .slick-cell:nth-child(1)').then(($e) => {
      assert.equal($e.text(), '1790');
    });

    cy.get('.slick-row:nth-child(3) .slick-cell:nth-child(2)').then(($e) => {
      assert.equal($e.text(), 'AK');
    });

    // save dash - since we dont show a view link and save button remains highlighted on discarding visual changes
    cy.intercept('POST', '/arc/reports/savedashboard').as('saveClicked');
    cy.get('#app-save-link').click();
    cy.wait('@saveClicked');
  });
});
