/*
 * Verify that the SQL builder visualization works
 * Assumes the existence of the 'US State Populations Over Time' dataset
 *   in the Cypress.env("smokeConnectionName") data connection
 */
import * as arctest from 'lib/arctest';
import * as visuals from 'lib/visual';

describe('dendrogram Viz', () => {
  afterEach(() => {
    cy.window().then(
      (win: Window) =>
        (win.onbeforeunload = () => {
          return null;
        })
    );
  });

  it('visual: simple dendrogram', () => {
    arctest.logins.silent();
    arctest.misc.newDashboardEnterModal(
      Cypress.env('smokeConnectionName'),
      'US State Populations Over Time'
    );

    // Click over to the SQL type visual, and verify it displays a table
    arctest.screen.builder.modal.switchToVisualType('.chart-icon-dendrogram');

    // Create a chart
    arctest.screen.builder.modal.addShelfColumn('dimensions_shelf', 'year');
    arctest.screen.builder.modal.orderAsc('year', true);
    arctest.screen.builder.modal.addShelfColumn('dimensions_shelf', 'state');
    arctest.screen.builder.modal.orderAsc('state', true);
    arctest.screen.builder.modal.addShelfColumn('aggregates_shelf', 'population');

    cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('jsonselect');
    arctest.screen.builder.modal.refreshVisual();
    cy.wait('@jsonselect');
    cy.get(arctest.screen.builder.modal.locators.refresh_icon).should('be.visible');

    // Wait for the draw to complete
    cy.wait(2000);

    // Verify
    cy.get('.arc-viz-type-dendrogram text').should('have.length', 728);

    cy.get('.arc-viz-type-dendrogram text').each(($el, i) => {
      if (i === 0) {
        expect($el.text()).to.equal('1790');
      }

      if (i === 1) {
        expect($el.text()).to.equal('AK');
      }
    });

    // TODO: re-enable this test after getting the drag event right
    if (false) {
      cy.get("#dimensions_shelf span[title='year']").then(($year) => {
        const yearPos = $year.offset();
        cy.get("#dimensions_shelf span[title='state']").then(($state) => {
          const statePos = $state.offset();
          if (yearPos) {
            cy.get("#dimensions_shelf span[title='state']")
              .trigger('mousedown', {
                which: 1,
                pageX: yearPos.left + 1,
                pageY: yearPos.top + 1,
              })
              .trigger('mousemove', {
                which: 1,
                pageX: yearPos.left + 1,
                pageY: yearPos.top + 10,
              })
              .trigger('mouseup');
          }
        });
      });

      cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('jsonselect');
      arctest.screen.builder.modal.refreshVisual();
      cy.wait('@jsonselect');

      // Verify
      cy.get('.arc-viz-type-dendrogram text').should('have.length', 728);

      cy.get('.arc-viz-type-dendrogram text').each(($el, i) => {
        if (i === 0) {
          expect($el.text()).to.equal('AZ');
        }

        if (i === 1) {
          expect($el.text()).to.equal('1790');
        }
      });

      visuals.saveVisual();
    }

    // save dash - since we dont show a view link and save button remains highlighted on discarding visual changes
    cy.intercept('POST', '/arc/reports/savedashboard').as('saveClicked');
    cy.get('#app-save-link').click();
    cy.wait('@saveClicked');
  });
});
