/*
 * Verify that the SQL builder visualization works
 * Assumes the existence of the 'US State Populations Over Time' dataset
 *   in the Cypress.env("smokeConnectionName") data connection
 */
import * as arctest from 'lib/arctest';
import * as visuals from 'lib/visual';

describe('network Viz', () => {
  it('visual: simple network', () => {
    arctest.logins.silent();
    cy.visit('/');
    arctest.misc.newDashboardEnterModal(Cypress.env('smokeConnectionName'), 'Cereals');

    // Click over to the SQL type visual, and verify it displays a table
    arctest.screen.builder.modal.switchToVisualType('.chart-icon-network');

    // Create a chart
    arctest.screen.builder.modal.addShelfColumn('dimensions_shelf', 'cereal_name');
    arctest.screen.builder.modal.addShelfColumn('dimensions_shelf', 'manufacturer_code');
    arctest.screen.builder.modal.addShelfColumn('aggregates_shelf', 'calories');
    arctest.screen.builder.modal.addShelfColumn('link_color_shelf', 'manufacturer_code');

    cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('jsonselect');
    arctest.screen.builder.modal.refreshVisual();
    cy.wait('@jsonselect');
    cy.get(arctest.screen.builder.modal.locators.refresh_icon).should('be.visible');

    // Wait for visual render to complete
    cy.wait(2000);

    // verify
    cy.get('.arc-viz-type-network line').should('have.length', 77);

    cy.get('.arc-viz-type-network path').should('have.length', 86);

    cy.get("#dimensions_shelf span[title='cereal_name']").then(($year) => {
      const yearPos = $year.offset();
      cy.get("#dimensions_shelf span[title='manufacturer_code']").then(($state) => {
        const statePos = $state.offset();
        if (yearPos) {
          cy.get("#dimensions_shelf span[title='cereal_name']")
            .trigger('mousedown', {
              which: 1,
              pageX: yearPos.left + 1,
              pageY: yearPos.top + 1,
            })
            .trigger('mousemove', {
              which: 1,
              pageX: yearPos.left + 1,
              pageY: yearPos.top + 10,
            })
            .trigger('mouseup');
        }
      });
    });

    cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('jsonselect');
    arctest.screen.builder.modal.refreshVisual();
    cy.wait('@jsonselect');
    cy.get(arctest.screen.builder.modal.locators.refresh_icon).should('be.visible');

    // Wait for the draw to complete
    cy.wait(2000);

    // verify
    cy.get('.arc-viz-type-network line').should('have.length', 77);

    cy.get('.arc-viz-type-network path').should('have.length', 86);

    visuals.saveVisual();

    // save dash - since we dont show a view link and save button remains highlighted on discarding visual changes
    cy.intercept('POST', '/arc/reports/savedashboard').as('saveClicked');
    cy.get('#app-save-link').click();
    cy.wait('@saveClicked');
  });
});
