/*
 * Verify that the SQL builder visualization works
 * Assumes the existence of the 'US State Populations Over Time' dataset
 *   in the Cypress.env("smokeConnectionName") data connection
 */
import * as arctest from 'lib/arctest';

describe('pie Viz', () => {
  before(() => {
    arctest.logins.silent();
    cy.visit('/');
  });

  it('visual: simple pie', () => {
    arctest.misc.newDashboardEnterModal(
      Cypress.env('smokeConnectionName'),
      'US State Populations Over Time'
    );

    // Click over to the SQL type visual, and verify it displays a table
    arctest.screen.builder.modal.switchToVisualType('.chart-icon-pie');

    // Create a chart
    arctest.screen.builder.modal.addShelfColumn('dimensions_shelf', 'state');
    arctest.screen.builder.modal.addShelfColumn('y_shelf', 'year');
    arctest.screen.builder.modal.addShelfColumn('aggregates_shelf', 'population');

    cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('jsonselect');
    arctest.screen.builder.modal.refreshVisual();
    cy.wait('@jsonselect');
    cy.get(arctest.screen.builder.modal.locators.refresh_icon).should('be.visible');

    // Wait for the draw to complete
    cy.wait(2000);

    // Verify. Note that there are differences in rendering across browsers, so the
    // check below is not exact
    cy.get('.arc-viz-type-pie text').should('have.length.of.at.least', 74);

    // these are the US state names displayed as a color legend
    // VIZ 7 displayes 4 states more than VIZ 6
    // VIZ 6 stops at ME, VIZ 7 at MS
    cy.get('.arc-viz-type-pie text').should('have.length.of.at.most', 80);

    // save dash - since we dont show a view link and save button remains highlighted on discarding visual changes
    cy.intercept('POST', '/arc/reports/savedashboard').as('saveClicked');
    cy.get('#app-save-link').click();
    cy.wait('@saveClicked');
  });
});
