/*
 * Verify that the SQL builder visualization works
 * Assumes the existence of the 'US State Populations Over Time' dataset
 *   in the Cypress.env("smokeConnectionName") data connection
 */
import * as arctest from 'lib/arctest';

describe('rawtable sql Viz', () => {
  before(() => {
    arctest.logins.silent();
    cy.visit('/');
  });

  it('visual: simple rawtable', () => {
    arctest.misc.newDashboardEnterModal(
      Cypress.env('smokeConnectionName'),
      'US State Populations Over Time'
    );

    arctest.screen.builder.modal.switchToVisualType('.chart-icon-rawtable');

    // 1. Verify "Cancel" button
    openRawtableEditModal();
    editSqlExpression();
    cy.get('[data-test-id="super-modal"] [data-test-id="modal-cancel-btn"]').click();
    cy.get('[data-test-id="super-modal"]').should('not.be.visible', 1000);
    verifyRawtableNotUpdated();

    // 2. Verify "X" button
    openRawtableEditModal();
    editSqlExpression();
    cy.get('[data-test-id="super-modal"] [data-test-id="close-btn"]').click();
    cy.get('[data-test-id="super-modal"]').should('not.be.visible', 1000);
    verifyRawtableNotUpdated();

    // 3. Verify "Validate & Refresh" button
    openRawtableEditModal();
    editSqlExpression();

    cy.get('[data-test-id="super-modal"] [data-test-id="validate-and-save"]').click();
    cy.get('[data-test-id="edit-expression-textbox"] [data-test-id="validate-spinner"]', {
      timeout: 30000,
    }).should('not.be.visible');
    cy.get('[data-test-id="edit-expression-textbox"] [data-test-id="alert-dialog"]', {
      timeout: 30000,
    }).should('not.exist');

    // ... and verify the output on preview table
    cy.get('[data-test-id="super-modal"] .dataTables_scroll')
      .find('td')
      .should('have.length', 4);

    cy.get('[data-test-id="super-modal"] .dataTables_scroll')
      .find('td')
      .each(($el) => {
        expect($el.text()).to.eq('2010');
      });

    cy.get('[data-test-id="super-modal"] [data-test-id="modal-cancel-btn"]').click();
    cy.get('[data-test-id="super-modal"]').should('not.be.visible', 1000);

    verifyRawtableNotUpdated();

    // 4. Verify "Apply" button
    openRawtableEditModal();
    editSqlExpression();

    cy.get('[data-test-id="super-modal"] [data-test-id="apply-button"]').click();

    cy.get('[data-test-id="super-modal"]').should('not.be.visible', 1000);

    // VIZ-1025: SQL visual no longer has a refresh button:
    // cy.get(arctest.screen.builder.modal.locators.refresh_icon).should('be.visible', 1000);

    // The problem here is that we have a table on screen, and another very
    // similar one is going to be drawn a short time after the jsonselect
    // completes. For now, simply wait for a while and then test the contents
    // of the table
    cy.wait(2000);

    // ... and verify the output
    cy.get(arctest.screen.builder.modal.locators.rawtable_table)
      .find('td')
      .should('have.length', 4);

    cy.get(arctest.screen.builder.modal.locators.rawtable_table)
      .find('td')
      .each(($el) => {
        expect($el.text()).to.eq('2010');
      });

    // save dash - since we dont show a view link and save button remains highlighted on discarding visual changes
    cy.intercept('POST', '/arc/reports/savedashboard').as('saveClicked');
    cy.get('#app-save-link').click();
    cy.wait('@saveClicked');
  });
});

function openRawtableEditModal() {
  cy.get('[data-test-id="app-shelves"] [data-test-id="viz-sql-view"]').click();
}

function verifyRawtableNotUpdated() {
  cy.get(arctest.screen.builder.modal.locators.rawtable_table)
    .find('td')
    .should('have.length.gt', 4);

  cy.get(arctest.screen.builder.modal.locators.rawtable_table)
    .find('td')
    .each(($el) => {
      expect($el.text()).not.to.eq('2010');
    });
}

function editSqlExpression() {
  // Change to now use ace editor
  cy.get('[data-test-id="super-modal"] [data-test-id="edit-expression-textbox"]')
    .find('[data-test-id="expression-data"]')
    .then(($elem) => {
      cy.window()
        .its('ace' as any)
        .invoke('edit', $elem.attr('id'))
        .invoke('setValue', 'SELECT year FROM main.census_pop ORDER BY year desc LIMIT 4');
    });

  // Give ace.js some time to finish processing, otherwise we get a getTokens exception
  // from that library
  cy.wait(2000);
}
