import { logins } from 'lib/arctest';
import * as arctest from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import * as functions from 'lib/functions';
import { locators } from 'lib/visual/locators';
import { CSSselector } from 'locators/common/CSSselector';

const connectionName = Cypress.env('smokeConnectionName');
const datasetName = 'Cereals';
const timeStamp = new Date().getTime();
const dashboardName = `richtext_db_${timeStamp}`;
const text = 'Lorem Ipsum is simply dummy text of the printing and typesetting industry.';
const styleTags = ['b', 'i', 'u', 'strike', 'sup', 'sub'];
const codeViewText =
  '<p><b>Lorem Ipsum is simply dummy text of the printing and typesetting industry.</b></p><p><i>Lorem Ipsum is simply dummy text of the printing and typesetting industry.</i></p><p><u>Lorem Ipsum is simply dummy text of the printing and typesetting industry.</u></p><p><strike>Lorem Ipsum is simply dummy text of the printing and typesetting industry.</strike></p><p><sup>Lorem Ipsum is simply dummy text of the printing and typesetting industry.</sup></p><p><sub>Lorem Ipsum is simply dummy text of the printing and typesetting industry.</sub></p><p><sub><br></sub></p>';
describe('Richtext Visual.', () => {
  before(() => {
    logins.silent();
    cy.visit('/');
  });

  context('ARC-11089: Automate Rich Text Visual - Smoke', () => {
    it('1: Create richtext visuals', () => {
      functions.addNewDB(dashboardName);
      functions.createNewVisual(connectionName, datasetName);
      cy.wait(1000);

      arctest.screen.builder.modal.switchToVisualType('#chart-type-menu .chart-icon-html');

      cy.get('.arc-viz-type-html .viz-modal-link').click();

      // Wait a bit for the handlers to be setup
      cy.get('.builder-container-modal .viz-htmleditor').should('be.visible');
    });

    it('2: Verify Bold, Italic and Underline styles', () => {
      // Bold Text
      cy.get(locators.richText.bold).click();
      cy.get(locators.richText.body).type(text + '{enter}');
      cy.get(locators.richText.body)
        .find('b')
        .should('be.visible')
        .should('have.text', text);
      cy.get(locators.richText.bold).click();

      // Italic Text
      cy.get(locators.richText.italic).click();
      cy.get(locators.richText.body).type(text + '{enter}');
      cy.get(locators.richText.body)
        .find('i')
        .should('be.visible')
        .should('have.text', text);
      cy.get(locators.richText.italic).click();

      // Italic Text
      cy.get(locators.richText.underline).click();
      cy.get(locators.richText.body).type(text + '{enter}');
      cy.get(locators.richText.body)
        .find('u')
        .should('be.visible')
        .should('have.text', text);
      cy.get(locators.richText.underline).click();
    });

    it('3: Verify Strikethrough, Superscript and Subscript styles', () => {
      // Strikethrough Text
      cy.get(locators.richText.strikethrough).click();
      cy.get(locators.richText.body).type(text + '{enter}');
      cy.get(locators.richText.body)
        .find('strike')
        .should('be.visible')
        .should('have.text', text);
      cy.get(locators.richText.strikethrough).click();

      // Superscript Text
      cy.get(locators.richText.superscript).click();
      cy.get(locators.richText.body).type(text + '{enter}');
      cy.get(locators.richText.body)
        .find('sup')
        .should('be.visible')
        .should('have.text', text);
      cy.get(locators.richText.superscript).click();

      // Subscript Text
      cy.get(locators.richText.subscript).click();
      cy.get(locators.richText.body).type(text + '{enter}');
      cy.get(locators.richText.body)
        .find('sub')
        .should('be.visible')
        .should('have.text', text);
      cy.get(locators.richText.subscript).click();

      // Verify all applied styles
      cy.get(locators.richText.body + ' p').should('have.length', 7);
      for (let index = 0; index < styleTags.length - 1; index++) {
        const element = styleTags[index];
        cy.get('div.note-editable p:nth-child(' + (index + 1) + ')')
          .find(element)
          .should('be.visible')
          .should('have.text', text);
      }

      // Verify code view style
      cy.get(locators.richText.codeView).click();
      cy.get('div.note-editor').should('have.class', 'codeview');

      cy.get('.note-codable').should('have.value', codeViewText);

      cy.get('#apply-button').click();
    });

    it('4: Verify all applied styles in dashboard edit mode', () => {
      cy.get('.viz-chart .viz-htmleditor-predecessor p').should('have.length', 7);
      for (let index = 0; index < styleTags.length - 1; index++) {
        const element = styleTags[index];
        cy.get('.viz-chart .viz-htmleditor-predecessor p:nth-child(' + (index + 1) + ')')
          .find(element)
          .should('be.visible')
          .should('have.text', text);
      }
      dashboards.saveDashboard();
    });

    it('5: Verify all applied styles in dashboard view mode', () => {
      cy.get(CSSselector.view_mode).click({ force: true });

      // wait for the HTML visual to appear:
      cy.get('.view-mode .viz-htmleditor').should('be.visible');

      cy.get('.viz-chart .viz-htmleditor-predecessor p').should('have.length', 7);
      for (let index = 0; index < styleTags.length - 1; index++) {
        const element = styleTags[index];
        cy.get('.viz-chart .viz-htmleditor-predecessor p:nth-child(' + (index + 1) + ')')
          .find(element)
          .should('be.visible')
          .should('have.text', text);
      }
    });
  });
});
