/*
 * Verify that the SQL builder visualization works
 * Assumes the existence of the 'US State Populations Over Time' dataset
 *   in the Cypress.env("smokeConnectionName") data connection
 */
import * as arctest from 'lib/arctest';
import * as visuals from 'lib/visual';

describe('sparklines Viz', () => {
  before(() => {
    arctest.logins.silent();
    cy.visit('/');
  });

  it('visual: simple sparklines', () => {
    arctest.misc.newDashboardEnterModal(
      Cypress.env('smokeConnectionName'),
      'US State Populations Over Time'
    );

    // Click over to the SQL type visual, and verify it displays a table
    arctest.screen.builder.modal.switchToVisualType('.chart-icon-sparklines');

    // Create a chart
    arctest.screen.builder.modal.addShelfColumn('x_shelf', 'state');
    arctest.screen.builder.modal.addShelfColumn('y_shelf', 'year');
    arctest.screen.builder.modal.addShelfColumn('aggregates_shelf', 'population');

    cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('jsonselect');
    arctest.screen.builder.modal.refreshVisual();
    cy.wait('@jsonselect');
    cy.get(arctest.screen.builder.modal.locators.refresh_icon).should('be.visible');

    // Wait for the draw to complete
    cy.wait(2000);

    // Verify
    cy.get('.arc-viz-type-sparklines canvas').should('have.length', 14);

    visuals.saveVisual();

    // save dash - since we dont show a view link and save button remains highlighted on discarding visual changes
    cy.intercept('POST', '/arc/reports/savedashboard').as('saveClicked');
    cy.get('#app-save-link').click();
    cy.wait('@saveClicked');
  });
});
