/*
 * Verify that the SQL builder visualization works
 * Assumes the existence of the 'US State Populations Over Time' dataset
 *   in the Cypress.env("smokeConnectionName") data connection
 */
import * as arctest from 'lib/arctest';
import { editVisual } from 'lib/dashboard';
import * as visuals from 'lib/visual';

describe('Visual smoke tests.', () => {
  describe('Wordcloud.', () => {
    before(() => {
      arctest.logins.silent();
      cy.visit('/');
    });

    it('Simple wordcloud', () => {
      arctest.misc.newDashboardEnterModal(
        Cypress.env('smokeConnectionName'),
        'Cereals',
        'Dashboard for arc_visual_wordcloud'
      );

      editVisual();
      // Click over to the SQL type visual, and verify it displays a table
      arctest.screen.builder.modal.switchToVisualType('.chart-icon-wordcloud');

      // Create a chart
      arctest.screen.builder.modal.addShelfColumn('dimensions_shelf', 'manufacturer_code');
      arctest.screen.builder.modal.addShelfColumn('aggregates_shelf', 'Record Count');

      cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('jsonselect');
      arctest.screen.builder.modal.refreshVisual();
      cy.wait('@jsonselect');
      cy.get(arctest.screen.builder.modal.locators.refresh_icon).should('be.visible');

      // Wait for the draw to complete
      cy.wait(2000);

      visuals.saveVisual();

      // Verify
      cy.get('.arc-viz-type-wordcloud text').should('have.length', 6);

      cy.get('.arc-viz-type-wordcloud text').then((textElements) => {
        const texts = textElements.toArray().map((e) => Cypress.$(e).text());
        expect(texts).to.have.members(['G', 'K', 'N', 'P', 'Q', 'R']);
      });
    });
  });
});
