// This script contains verification on test https://arcadiadata.atlassian.net/browse/ARC-8154
import * as arctest from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import * as functions from 'lib/functions';
import * as workspace from 'lib/workspace';
import { CSSselector } from 'locators/common/CSSselector';

describe('Workspace tests.', () => {
  const timestamp = new Date().getTime();
  const dashboardName = `dash_${timestamp}_ARC`;

  context('regression tests 1', () => {
    // Test Case ARC-8146 and ARC-8147

    it('1: ARC-8146 ARC-8147: Verify users read only access with public workspace-- Visual Consumer', () => {
      cy.fixture('users.json').then((data) => {
        arctest.logins.silent(data.visualconsumer.username, data.visualconsumer.password);
      });
      cy.get(CSSselector.Visual_page).click();
      cy.get(CSSselector.apps.favourites_tab).click();
      cy.get(CSSselector.Visuals_all).click();
      cy.get(CSSselector.workspace_regression.Public_workspace).click();
      cy.get(CSSselector.apps.applist_btn).click();

      cy.get(CSSselector.apps.dashboardcheckbox)
        .first()
        .parents('tr')
        .click();
      cy.get(CSSselector.edit_link).then(($el) => {
        expect($el).to.be.hidden;
      });
      cy.get(CSSselector.workspace_regression.Dashboard_edit_ellipsis)
        .click()
        .parents('li')
        .find(CSSselector.workspace_regression.Dashboardedit_dropdown)
        .then(($el) => {
          expect($el).to.be.visible;
          let isCloneAvailable = false;
          cy.wrap($el)
            .find('li:not(.arc-invisible)')
            .each(($liel) => {
              if ($liel.text().trim() === 'Clone') {
                isCloneAvailable = true;
              }
              assert.isNotOk(isCloneAvailable, 'this will pass');
            });
          let isDeleteDashboardAvailable = false;
          cy.wrap($el)
            .find('li:not(.arc-invisible)')
            .each(($liel) => {
              if ($liel.text().trim() === 'Delete Dashboard') {
                isDeleteDashboardAvailable = true;
              }
              assert.isNotOk(isDeleteDashboardAvailable, 'this will pass');
            });
        });
    });

    // Test Case ARC-8148

    it('2: ARC-8148: Non-admin user with manage workspace permissions', () => {
      cy.fixture('users.json').then((data) => {
        arctest.logins.silent(data.sysadmin.username, data.sysadmin.password);
      });
      const workspaceName = 'System Admin Workspace';
      const workspaceDesc = 'This is a description for user defined workspace';

      workspace.createWorkSpace(workspaceName, workspaceDesc);

      cy.get(CSSselector.Visual_page).click();
      functions.addNewDB(dashboardName);
      functions.createNewVisual('samples', 'Cereals');

      cy.intercept('POST', '/arc/reports/savedashboard').as('savedashboard');
      cy.get(CSSselector.Save).click();
      cy.wait('@savedashboard')
        .its('response.statusCode')
        .should('eq', 200);
      cy.get(CSSselector.homepage).click();
      cy.get(CSSselector.Visual_page).click();
      cy.get(CSSselector.workspace_regression.workspace_menulist)
        .find('> li [title="Private"]')
        .should(($el) => {
          expect($el).to.exist;
        })
        .parent()
        .find('.badge')
        .should(($el) => {
          let numerOfDashboard: number;
          numerOfDashboard = Number($el.text());
          assert.isAbove(numerOfDashboard, 0, 'Dashboard is added in private workspace');
        })
        .parents('ul')
        .find('> li [title="Private"]')
        .click();

      cy.get(CSSselector.apps.applist_btn).click();
      cy.intercept('GET', '/arc/workspace/workspace').as('movews');
      cy.intercept('POST', '/arc/reports/move').as('movews2');

      cy.get(CSSselector.apps.dashboardcheckbox)
        .first()
        .click()
        .parents('tr')
        .find('.table-app-title')
        .then(($el) => {
          cy.get('li.move-viz-btn')
            .click()
            .next('div.workspace-move-popover')
            .should(($elm) => {
              expect($elm).to.exist;
            })
            .find('[data-workspace-name="System Admin Workspace"]')
            .click({ force: true });

          // Moving the dashboard from Private workspace to System Admin Workspace

          cy.get('.popover-content .save-workspace-move').click({ force: true });
          cy.wait('@movews2')
            .its('response.statusCode')
            .should('eq', 200);
          cy.wait('@movews')
            .its('response.statusCode')
            .should('eq', 200);

          // Wait is added so that the dashboard can be moved to system Admin workspace from Private workspace
          cy.wait(4000);
          cy.get(CSSselector.Visual_page).click();
          cy.get(CSSselector.workspace_regression.workspace_menulist)
            .find('> li [title="System Admin Workspace"]')
            .should(($el2) => {
              expect($el2).to.exist;
            })
            .parent()
            .find('.badge')
            .should(($elm) => {
              let numberOfDashboard: number;
              numberOfDashboard = Number($elm.text());
              assert.isAbove(
                numberOfDashboard,
                0,
                'Dashboard is added in system user defined workspace'
              );
            })
            .parents('ul')
            .find('> li [title="System Admin Workspace"]')
            .click();
          cy.get(CSSselector.workspace_regression.Applist_btn).click();

          // Moving the dashboard from System Admin defined workspace to Private workspace
          cy.intercept('GET', '/arc/workspace/workspace').as('movews');
          cy.intercept('POST', '/arc/reports/move').as('movews2');
          cy.get(CSSselector.apps.dashboardcheckbox)
            .first()
            .click()
            .parents('tr')
            .find('.table-app-title')
            .then(($el1) => {
              cy.get('li.move-viz-btn')
                .click()
                .next('div.workspace-move-popover')
                .should(($elm2) => {
                  expect($elm2).to.exist;
                })
                .find('[data-workspace-name="Private"]')
                .click()
                .parents('.popover-content')
                .find('.popover-btns .save-workspace-move')
                .click({ force: true });
              cy.wait('@movews2')
                .its('response.statusCode')
                .should('eq', 200);
              cy.wait('@movews')
                .its('response.statusCode')
                .should('eq', 200);

              // Wait is added so that the dashboard can be moved to system Admin workspace from Private workspace
              cy.wait(4000);
              cy.get(CSSselector.homepage).click();
              cy.get(CSSselector.Visual_page).click();
              cy.get(CSSselector.workspace_regression.workspace_menulist)
                .find('> li [title="Private"]')
                .should(($el3) => {
                  expect($el3).to.exist;
                })
                .parent()
                .find('.badge')
                .should(($el3) => {
                  let numberOfDashboard: number;
                  numberOfDashboard = Number($el3.text());
                  assert.isAbove(
                    numberOfDashboard,
                    0,
                    'Dashboard is again moved in private workspace from user defined'
                  );
                })
                .parents('ul')
                .find('> li [title="Private"]')
                .click();
              cy.get(CSSselector.workspace_regression.Applist_btn).click();
              cy.get(CSSselector.apps.dashboardcheckbox)
                .first()
                .click()
                .parents('tr')
                .find('.table-app-title')
                .then(($el4) => {
                  cy.get('li.move-viz-btn')
                    .click()
                    .next('div.workspace-move-popover')
                    .should(($elm) => {
                      expect($elm).to.exist;
                    })
                    .find('[data-workspace-name="Public"]')
                    .click();

                  // Moving the dashboard from Private workspace to Public  Workspace

                  cy.get('.popover-content .save-workspace-move').click({ force: true });
                  cy.wait('@movews2')
                    .its('response.statusCode')
                    .should('eq', 200);
                  cy.wait('@movews')
                    .its('response.statusCode')
                    .should('eq', 200);

                  // Wait is added so that the dashboard can be moved to system Admin workspace from Private workspace

                  cy.wait(4000);
                  cy.get(CSSselector.Visual_page).click();
                  cy.get(CSSselector.workspace_regression.workspace_menulist)
                    .find('> li [title="Public"]')
                    .click();
                  cy.get(CSSselector.workspace_regression.workspace_menulist)
                    .find('> li [title="Public"]')
                    .should(($el2) => {
                      expect($el2).to.exist;
                    })
                    .parent()
                    .find('.badge')
                    .should(($elm) => {
                      let numerOfDashboard: number;
                      numerOfDashboard = Number($elm.text());
                      assert.isAbove(numerOfDashboard, 0, 'Dashboard is added in public workspace');
                    })
                    .parents('ul')
                    .find('> li [title="Public"]')
                    .click();

                  // Analyst user logs in to view the dashboard moved by System-Admin in Public Workspace

                  cy.fixture('users.json').then((data) => {
                    arctest.logins.silent(data.analyst.username, data.analyst.password);
                  });
                  cy.get(CSSselector.Visual_page).click();
                  cy.get(CSSselector.workspace_regression.workspace_menulist)
                    .find('> li [title="Public"]')
                    .click();
                  functions.searchDashboard(dashboardName);
                  cy.get('#report-table tbody tr:first-child td.table-app-title').then(($el9) => {
                    const dbid = $el9.next('td').text();
                  });
                  cy.get(CSSselector.gridview_Visualspage).click();
                  dashboards.openDashboardforEdit(dashboardName);
                  // clickEditdashboard();
                  // cy.get(CSSselector.edit_dashboard).click();
                });
            });
        });
    });

    //  Test-case ARC-8149

    // Verifying the ACL of System Admin user-defined workspace by adding a user with manage and another with edit permission

    it('3: ARC-8149: User with user-defined workspace permissions', () => {
      cy.fixture('users.json').then((data) => {
        arctest.logins.silent(data.sysadmin.username, data.sysadmin.password);
        cy.get(CSSselector.homepage).click();
        cy.get(CSSselector.Visual_page).click();
        cy.get(CSSselector.workspace_regression.workspace_menulist)
          .find('> li [title="System Admin Workspace"]')
          .click();
        cy.get('.ws-menu-list div[title="System Admin Workspace"] span.edit-icon').click();
        // wait to pop-up worksapce modal
        cy.wait(1000);
        cy.get(CSSselector.workspace_regression.WSEditModal)
          .should('be.visible')
          .then(($el5) => {
            cy.wrap($el5)
              .find('.ws-delete-button')
              .then(($elm) => {
                expect($elm).to.be.visible;
              });
            cy.get(CSSselector.workspace_regression.username_textbox).type('analyst{enter}');
            cy.get('.add-more-acl').click({ force: true });
            cy.get('.table.form-horizontal tbody tr:nth-child(2) input[value="VIEW"]').click({
              force: true,
            });
            cy.get(CSSselector.workspace_regression.username_textbox).type('dbadmin{enter}');
            cy.get('.add-more-acl').click({ force: true });
            cy.get('.table.form-horizontal tbody tr:nth-child(3) input[value="MANAGE"]').click({
              force: true,
            });
            workspace.saveWorkSpace();
          });

        arctest.logins.silent(data.analyst.username, data.analyst.password);

        cy.get(CSSselector.homepage).click();
        cy.get(CSSselector.Visual_page).click();
        cy.reload();
        cy.get('.ws-menu-list [title="System Admin Workspace"]')
          .click()
          .within(() => {
            cy.get('.edit-icon').should('not.be.visible');
          });
        // login dbadmin
        arctest.logins.silent(data.dbadmin.username, data.dbadmin.password);
        cy.get(CSSselector.homepage).click();
        cy.get(CSSselector.Visual_page).click();
        cy.get('.ws-menu-list [title="System Admin Workspace"]')
          .click()
          .within(() => {
            cy.get('.edit-icon').should('be.visible');
          });

        cy.get('.ws-menu-list div[title="System Admin Workspace"] span.edit-icon').click({
          force: true,
        });
        cy.get(CSSselector.workspace_regression.WSEditModal).then(($el) => {
          const workspaceNameedit = 'Editing System Admin Workspace';
          const workspaceDescedit = 'Editing the description for user defined workspace';
          cy.wrap($el)
            .find(CSSselector.workspace_regression.wsname)
            .clear()
            .type(workspaceNameedit);
          cy.wrap($el)
            .find(CSSselector.workspace_regression.wsdesc)
            .clear()
            .type(workspaceDescedit);
          cy.get('.table.form-horizontal tbody tr:nth-child(2) input[value="MANAGE"]').click({
            force: true,
          });
          workspace.saveWorkSpace();

          cy.get(CSSselector.homepage).click();
          cy.get(CSSselector.Visual_page).click();

          cy.get(CSSselector.workspace_regression.workspace_menulist)
            .find('> li [title="Public"]')
            .click();
        });
        cy.get(CSSselector.apps.applist_btn).click();
        cy.intercept('GET', '/arc/workspace/workspace').as('movews');
        cy.intercept('POST', '/arc/reports/move').as('movews2');
        let Dashboardname = '';
        cy.get(CSSselector.apps.dashboardcheckbox)
          .first()
          .click()
          .parents('tr')
          .find('.table-app-title')
          .then(($el1) => {
            Dashboardname = $el1.text();
            cy.get('li.move-viz-btn')
              .click()
              .next('div.workspace-move-popover')
              .should(($elm2) => {
                expect($elm2).to.exist;
              })
              .find('[data-workspace-name="Private"]')
              .click()
              .parents('.popover-content')
              .find('.popover-btns .save-workspace-move')
              .click({ force: true });
            cy.wait('@movews2')
              .its('response.statusCode')
              .should('eq', 200);
            cy.wait('@movews')
              .its('response.statusCode')
              .should('eq', 200);

            // Wait is added so that the dashboard can be moved to system Admin workspace from Private workspace
            cy.wait(4000);
            cy.get(CSSselector.homepage).click();
            cy.get(CSSselector.Visual_page).click();
            cy.get(CSSselector.workspace_regression.workspace_menulist)
              .find('> li [title="Private"]')
              .should(($el3) => {
                expect($el3).to.exist;
              })
              .parent()
              .find('.badge')
              .should(($el3) => {
                let numberOfDashboard: number;
                numberOfDashboard = Number($el3.text());
                assert.isAbove(
                  numberOfDashboard,
                  0,
                  'Dashboard is moved in private workspace from public workspace by DBadmin'
                );
              })
              .parents('ul')
              .find('> li [title="Private"]')
              .click();
          });
      });
    });
  });
});
