// This script contains verification on test https://arcadiadata.atlassian.net/browse/ARC-8155
import { notifications } from 'fixtures/locators/notifications';
import * as apps from 'lib/apps';
import { logins } from 'lib/arctest';
import * as connections from 'lib/connections';
import * as dashboards from 'lib/dashboard';
import * as functions from 'lib/functions';
import * as imp_exp from 'lib/import_export';
import * as workspace from 'lib/workspace';
import { CSSselector } from 'locators/common/CSSselector';

/**
 * Type dataset and dataconnection to create new visual
 * @param dataconnection: connection name
 * @param dataset: dataset name
 */
function typeDatasetandDataCon(dataconnection: string, dataset: string) {
  cy.get(CSSselector.selecting_dataconnection)
    .click()
    .find('input')
    .type(dataconnection);
  cy.get(CSSselector.selecting_dataconnection)
    .find('.chosen-results')
    .contains(dataconnection)
    .click({ force: true });
  cy.get(CSSselector.selecting_datasets)
    .click()
    .find('input')
    .type(dataset);
  cy.get(CSSselector.selecting_datasets)
    .find('.chosen-results')
    .contains(dataset)
    .click({ force: true });
}

function get_time() {
  const a = new Date();
  return a.getTime();
}

describe('Workspace tests.', () => {
  const DBList = [
    'Dashboard1',
    'Dashboard2',
    'Dashboard3',
    'Dashbord4',
    'Dashboard_with_single_viz',
    'Dashboard_with_multiple_single_viz1',
    'Dashboard_with_multiple_single_viz2',
    'Dashboard_with_multiple_single_viz3',
    'Dashboard_with_multiple_single_viz4',
  ];
  const singleVizDashboard = ['Single_Visual3', 'Single_Visual4'];
  const linkedVizDashboards = [
    'Dashboard_with_multiple_single_viz1',
    'Dashboard_with_multiple_single_viz2',
    'Dashboard_with_multiple_single_viz3',
    'Dashboard_with_multiple_single_viz4',
  ];
  const linkedVisual = 'Single_Visual2';
  const appList = ['test_app1', 'test_app2'];
  const appWithSubGrps = [' test_app2 ', ' sub_group1 ', ' sub_group2 '];
  const dataconnection = 'samples';
  const dataset = 'Chicago Gov Pay';
  const lvName1 = `lv1${get_time()}`;
  const vName1 = `v1${get_time()}`;
  // let linkedVizDB:any = [];
  context('regression tests 2', () => {
    before(() => {
      logins.silent();
    });

    it.skip('1: ARC-8102: Checks for workspace and visuals after migration', () => {
      cy.intercept('GET', '/arc/workspace/workspace').as('workspaceLoaded');
      cy.intercept('PUT', '/arc/migration/migrate?dry_run=false').as('importDash');
      cy.intercept('PUT', '/arc/sqlrun/jsonselect_parallel_async/*').as('jsonselect');
      // verify that all the dashboards are present in Public workspace
      for (const DB of DBList) {
        workspace.verifyUserDefinedWorkspace('Public', DB, true);
      }

      // verify that all the single visuals are present as a dashboard in Public workspace
      for (const viz of singleVizDashboard) {
        workspace.verifyUserDefinedWorkspace('Public', viz, true);
      }

      // verify that single visual used in single dashboard should be unlinked and present in a dashboard
      dashboards.openDashboardforEdit(DBList[4]);
      cy.get(CSSselector.visual_title_Check).should('have.text', 'Single_Visual1');
      cy.get(CSSselector.dashboardDetail.linkVisualIcon).should('not.be.visible');
      cy.get(CSSselector.visual_edit_first).click({ force: true });
      cy.get('div.linked-text-wrapper').should('not.be.visible');
      dashboards.closeVisual(false);

      // verify that single visual used in multiple dashboards should be linked and present in dashboards
      functions.navigate_to_visuals();
      cy.get('#app-grid input[value="visuals"]').click({ force: true });
      cy.contains(linkedVisual);
      for (const DB of linkedVizDashboards) {
        dashboards.openDashboardforEdit(DB);
        cy.get(CSSselector.visual_title_Check).should('have.text', linkedVisual);
        cy.get(CSSselector.dashboardDetail.linkVisualIcon).should('be.visible');
        cy.get(CSSselector.visual_edit_first).click({ force: true });
        cy.get('div.linked-text-wrapper').should('be.visible');
        dashboards.closeVisual(false);
      }

      // verify that app with single visual should be converted in a app with single visual dashboard
      workspace.verifyUserDefinedWorkspace('Public', 'Single_Viz_for_app', true);
      apps.launchApp('single_viz_app');
      cy.get(apps.locators.app_tab.replace('index', '1') + ' > a')
        .should('have.text', ' single_viz_app ')
        .click();
      cy.get(workspace.locators.single_viz_app_title).should('have.text', 'Single_Viz_for_app');
      cy.visit('/arc/apps/apps');

      // verify that single visual dashboard can be cloned from view dashboard
      functions.navigate_to_visuals();
      dashboards.openDashboard(singleVizDashboard[0]);
      cy.get(CSSselector.workspace_regression.Dashboard_edit_ellipsis).click();
      cy.get(CSSselector.workspace_regression.Dashboard_clone).click();
      cy.wait('@jsonselect');
      cy.get(CSSselector.Dashboard_title).should('have.text', 'Clone of ' + singleVizDashboard[0]);

      // verify that multi-visual dashboard can be cloned from view dashboard
      functions.navigate_to_visuals();
      dashboards.openDashboard(DBList[0]);
      cy.get(CSSselector.workspace_regression.Dashboard_edit_ellipsis).click();
      cy.get(CSSselector.workspace_regression.Dashboard_clone).click();
      cy.get(workspace.locators.clone_move).click();
      cy.wait('@jsonselect');
      cy.get(CSSselector.Dashboard_title).should('have.text', 'Clone of ' + DBList[0]);

      // verify that visual can be cloned from dashboard and user should be able to save it. Also, verify cloned visual name
      functions.navigate_to_visuals();
      dashboards.openDashboardforEdit(DBList[4]);
      cy.get(CSSselector.visual_edit_first).click({ force: true });
      cy.get(CSSselector.visualEdit.ellipsis).click();
      cy.get(workspace.locators.visual_clone).click();
      cy.wait('@newDashboard_jsonselect');
      cy.get(notifications.alert_success)
        .should('be.visible')
        .find(' p.message')
        .should('have.text', 'Successfully cloned and added visual to dashboard');
      dashboards.closeVisual(false);
      cy.get(CSSselector.visual_title.replace('index', '2')).should(
        'have.text',
        'Clone of Single_Visual1'
      );
      dashboards.saveDashboard();

      // verify that after launching the apps, sungroups(if available) and dashboards should be displayed properly
      apps.launchApp(appList[0]);
      cy.get(apps.locators.app_tab.replace('index', '1') + ' > a')
        .should('have.text', ' ' + appList[0] + ' ')
        .click();
      cy.get(apps.locators.app_tab.replace('index', '1'))
        .find(' ul li')
        .each(($el, i, $list) => {
          expect($el.text().trim()).to.eq(DBList[i]);
          cy.wrap($el)
            .find(' a')
            .click({ force: true });
          cy.get(CSSselector.Dashboard_title).should('have.text', DBList[i]);
          cy.get(apps.locators.app_tab.replace('index', '1')).click();
        });
      cy.visit('/arc/apps/apps');
      apps.launchApp(appList[1]);
      const appDBs = [
        ['US State Population Trends Dark', 'Iris species w/ images'],
        ['Taxi rides application', 'Dashbord4'],
        ['World Population & GDP Trends', 'Cereal Comparisons'],
      ];
      cy.get(apps.locators.app_tab.replace(':nth-child(index)', '')).each(($el, i, $list) => {
        cy.get(apps.locators.app_tab.replace('index', '' + (i + 1) + '') + ' > a')
          .should('have.text', appWithSubGrps[i])
          .click();
        cy.get(apps.locators.app_tab.replace('index', '' + (i + 1) + ''))
          .find(' ul li')
          .each(($ele, index, $lst) => {
            expect($ele.text().trim()).to.eq(appDBs[i][index]);
            cy.wrap($ele)
              .find(' a')
              .click();
            cy.get(CSSselector.Dashboard_title).should('have.text', appDBs[i][index]);
            cy.get(apps.locators.app_tab.replace('index', '' + (i + 1) + '')).click();
          });
      });
    });

    it('2: ARC-8110: Checks for Linked/Unlinked visuals in workspaces', () => {
      // verify that clicking on clone visuals while cloning dashboard, visuals should be clonned and added into the new dashboard
      cy.visit('/arc/apps/apps');
      dashboards.openDashboard(DBList[0]);
      cy.get(CSSselector.workspace_regression.Dashboard_edit_ellipsis).click();
      cy.get(CSSselector.workspace_regression.Dashboard_clone).click();
      cy.get('label.clone-option input').each(($el) => {
        expect($el).to.be.checked;
      });
      cy.get(workspace.locators.clone_move).click();
      cy.wait('@jsonselect');
      cy.get(CSSselector.Dashboard_title).should('have.text', 'Clone of ' + DBList[0]);
      cy.get(CSSselector.visual_title.replace('index', '1')).should(
        'have.text',
        'Avg quantity per nutritional category'
      );
      cy.get(CSSselector.visual_title.replace('index', '2')).should(
        'have.text',
        'Comparison of average calories per manufacturer to overall average'
      );
      cy.get('div#app-selected-list > ul > li').each(($e) => {
        cy.wrap($e)
          .find(CSSselector.dashboardDetail.linkVisualIcon)
          .should('not.be.visible');
      });

      dashboards.openDashboard(DBList[1]);
      cy.get(CSSselector.workspace_regression.Dashboard_edit_ellipsis).click();
      cy.get(CSSselector.workspace_regression.Dashboard_clone).click();

      cy.get('label.clone-only-text input').each(($el) => {
        cy.wrap($el)
          .click()
          .then(() => {
            expect($el).to.be.checked;
          });
      });
      cy.get(workspace.locators.clone_move).click();
      cy.wait('@jsonselect');
      cy.get(CSSselector.Dashboard_title).should('have.text', 'Clone of ' + DBList[1]);
      cy.get(CSSselector.visual_title.replace('index', '1')).should(
        'have.text',
        'Taxi cabs rides by hour of day and pick-up neighborhood'
      );
      cy.get(CSSselector.visual_title.replace('index', '2')).should(
        'have.text',
        'Top pick-up neighborhoods'
      );
      cy.get('div#app-selected-list > ul > li').each(($e) => {
        cy.wrap($e)
          .find(CSSselector.dashboardDetail.linkVisualIcon)
          .should('be.visible');
      });

      // verify that visual should get cloned and added to the dashboard, after creating new datasets and dashboards in it.
      functions.navigate_to_data();
      functions.createNewDataset('main', 'chicago_govt_pay', dataset);
      for (let i = 1; i < 3; i++) {
        functions.addNewDB('Chicago Dashboard' + i);
        functions.createNewVisual(dataconnection, dataset);
        cy.get(CSSselector.visual_edit_specific(1)).click({ force: true });
        cy.get(CSSselector.visuals_title)
          .click({ force: true })
          .type('Visual' + i);
        dashboards.saveVisual();
        dashboards.closeVisual(false);
        dashboards.saveDashboard();
      }
      functions.navigate_to_visuals();
      cy.get('a.new-dashboard-btn').click();
      cy.get(CSSselector.Dashboard_title)
        .click()
        .type('Cloned Viz Dashboard');
      typeDatasetandDataCon(dataconnection, dataset);
      cy.get('ul.dataset-app-list')
        .scrollIntoView()
        .find('.app-title')
        .each(($el) => {
          if ($el.text().trim() === 'Visual2') {
            cy.wrap($el).click();
          }
        });
      cy.get(notifications.notification_message).should('be.visible');
      cy.wait('@visualCreated');
      cy.wait('@jsonselect');
      cy.get(CSSselector.visual_title_Check).should('have.text', 'Clone of Visual2');
      dashboards.saveDashboard();

      // verify that linked visual should get displayed with the link and added to the dashboard, after creating new datasets and dashboards in it.
      functions.navigate_to_visuals();
      cy.get('a.new-dashboard-btn').click();
      typeDatasetandDataCon(dataconnection, 'NYC Taxicab Rides');
      cy.get('ul.dataset-app-list')
        .scrollIntoView()
        .find('.app-title')
        .each(($el) => {
          if ($el.text().trim() === 'Top pick-up neighborhoods') {
            cy.wrap($el)
              .next()
              .should('have.class', 'linked-visual-icon')
              .click({ force: true });
            cy.get('#add-linked-modal')
              .should('be.visible')
              .find('label.link-option input')
              .check();
            cy.get('button.add-visual-btn').click();
            cy.wait('@visualCreated');
            cy.wait('@jsonselect');
            cy.get(CSSselector.visual_title_Check).should('have.text', 'Top pick-up neighborhoods');
            cy.get(CSSselector.dashboardDetail.linkVisualIcon).should('be.visible');
            dashboards.saveDashboard();
          }
        });

      // verify that after unlinking a visual from all the dashbaords, visual should not be displayed as linked in all the dashboards
      for (let i = 1; i < 5; i++) {
        dashboards.openDashboardforEdit(DBList[i + 4]);
        cy.get(CSSselector.visual_edit_first).click({ force: true });
        cy.get(CSSselector.visualEdit.ellipsis).click();
        cy.get(CSSselector.visualEdit.linkUnlink)
          .should('have.text', ' Unlink')
          .click();
        if (i === 4) {
          cy.get(CSSselector.visualEdit.linkUnlinkConfirm)
            .should('have.text', 'Unlink')
            .click();
          cy.get('div.linked-text-wrapper').should('not.be.visible');
        } else {
          cy.get(CSSselector.visualEdit.linkUnlinkConfirm)
            .should('have.text', 'Clone')
            .click();
          cy.get('div.linked-text-wrapper').should('not.be.visible');
          cy.get(CSSselector.visuals_title).should('have.text', 'Clone of Single_Visual2');
        }
        dashboards.closeVisual(false);
        dashboards.saveDashboard();
      }

      // verify that after creating a dashboard with linked and unlinked visual, dashboard should move to public WS.
      functions.addNewDB('Link_Unlink_Viz_Dashboard');
      dashboards.createLinkedVisual(1, lvName1, dataconnection, 'Cereals');
      dashboards.addVisual(2, vName1);
      dashboards.saveDashboard();
      cy.get('a.view-link').click();
      cy.wait('@visualUpdated');
      cy.get(CSSselector.dashboardDetail.workspaceMenu).click({ force: true });
      cy.get(CSSselector.dashboardDetail.menu + ' [data-workspace-name="Public"]').click({
        force: true,
      });
      cy.get(
        CSSselector.dashboardDetail.menu + ' ' + CSSselector.dashboardDetail.workspaceMove
      ).click({ force: true });
      cy.get(CSSselector.dashboardDetail.workspaceMoveCloneModel).should('be.visible');
      cy.get(
        CSSselector.dashboardDetail.workspaceMoveCloneModel +
          ' ' +
          CSSselector.dashboardDetail.workspaceMoveCloneVisualRadio
      ).should('be.checked');
      cy.get(CSSselector.dashboardDetail.workspaceMoveMoveNow).click();
      // dashboards.saveDashboard();
      workspace.verifyUserDefinedWorkspace('Public', 'Link_Unlink_Viz_Dashboard', true);
      cy.fixture('users.json').then((data) => {
        logins.silent(data.analyst.username, data.analyst.password);
        workspace.verifyUserDefinedWorkspace('Public', 'Link_Unlink_Viz_Dashboard', true);
      });
      logins.silent();
      functions.addNewDB('Link_Unlink_Viz_Dashboard1');
      dashboards.createLinkedVisual(1, lvName1, dataconnection, 'Cereals');
      dashboards.addVisual(2, vName1);
      dashboards.saveDashboard();
      cy.get('a.view-link').click();
      cy.wait('@visualUpdated');
      cy.get(CSSselector.dashboardDetail.workspaceMenu).click({ force: true });
      cy.get(CSSselector.dashboardDetail.menu + ' [data-workspace-name="Public"]').click({
        force: true,
      });
      cy.get(
        CSSselector.dashboardDetail.menu + ' ' + CSSselector.dashboardDetail.workspaceMove
      ).click({ force: true });
      cy.get(CSSselector.dashboardDetail.workspaceMoveCloneModel).should('be.visible');
      cy.get(
        CSSselector.dashboardDetail.workspaceMoveCloneModel +
          ' ' +
          CSSselector.dashboardDetail.workspaceMoveMoveVisualRadio +
          ' input'
      ).check();
      cy.get(CSSselector.dashboardDetail.workspaceMoveMoveNow).click();
      // dashboards.saveDashboard();
      cy.get('ul#report-menu-list a.editapp-link').click();
      cy.wait('@visualUpdated');
      cy.get(CSSselector.first_visual)
        .find(CSSselector.dashboardDetail.linkVisualIcon)
        .should('be.visible');

      // verify link/unlink button in visual edit modal
      cy.get(CSSselector.visual_edit_first).click({ force: true });
      cy.get(CSSselector.visualEdit.ellipsis)
        .click()
        .find(CSSselector.visualEdit.linkUnlink)
        .should('have.text', ' Unlink');
      dashboards.closeVisual(false);

      cy.get("a[data-target='#dash-viz-tab']").click();
      cy.get(CSSselector.visual_edit_specific(2)).click({ force: true });
      cy.get(CSSselector.visualEdit.ellipsis)
        .click()
        .find(CSSselector.visualEdit.linkUnlink)
        .should('have.text', ' Make Linked');
      dashboards.closeVisual(false);
    });
  });
});
