// This script contains verification on test https://arcadiadata.atlassian.net/browse/ARC-8156
import * as apps from 'lib/apps';
import { logins } from 'lib/arctest';
import * as connections from 'lib/connections';
import * as dashboards from 'lib/dashboard';
import * as functions from 'lib/functions';
import * as imp_exp from 'lib/import_export';
import * as workspace from 'lib/workspace';

/** write the content of json file to import it.
 * @param dashboardFile: name of dashboard (it must be an exact json filename)
 */
function importFile(dashboardFile: string) {
  const dt = new DataTransfer();
  const files: any[] = [];
  const jsonFolderPath = './cypress/test_resources/';

  cy.readFile(jsonFolderPath + dashboardFile, 'ascii').then((str) => {
    const jsonData: string = JSON.stringify(str);
    console.log('this is the file:' + jsonData);
    files.push(
      new File([jsonData], dashboardFile, {
        type: 'text/plain',
        lastModified: new Date().getTime(),
      })
    );
    console.log('this is the files:' + files[0]);

    dt.items.add(files[0] as File);
  });

  cy.get(imp_exp.locators.import_input_file).then(($inputFile) => {
    ($inputFile[0] as HTMLInputElement).files = dt.files;
  });
}

describe('Workspace tests.', () => {
  const DBName = 'Imp_Exp Dashboard for WS';
  const DBName1 = 'Imp_Exp Dashboard for WS1';
  const conName = 'samples';
  const userDefinedWs = ['db_admin_workspace', 'sys_admin_workspace'];
  const DbForUserDefineWs = ['workspace_a_dashboard', 'workspace_b_dashboard'];
  context('regression tests 3', () => {
    before(() => {
      logins.silent();
    });

    it('1: # ARC-8106: Checks for Import/export visuals in workspaces', () => {
      cy.intercept('GET', '/arc/workspace/workspace').as('workspaceGet');
      cy.intercept('PUT', '/arc/migration/migrate?dry_run=false').as('importDash');
      cy.fixture('users.json').then((data) => {
        // verify the Private WS is selected while importing dashboard using admin user.Also, Import dashbaord should be in Private WS of admin user.
        connections.selectConnectionByName(conName);
        cy.get(imp_exp.locators.import_toggle).click();
        cy.get(imp_exp.locators.import_artifacts).click();
        cy.wait('@workspaceGet')
          .its('response.statusCode')
          .should('eq', 200);
        cy.get(workspace.locators.selected_ws_on_import).should('have.text', 'Private');
        importFile(DBName + '.json');
        cy.get(imp_exp.locators.import_button).click();
        cy.get(imp_exp.locators.import_confirmation).click();
        cy.wait('@importDash')
          .its('response.statusCode')
          .should('eq', 200);
        cy.contains('Success! The import was successful.');
        cy.wait(2000);
        workspace.verifyUserDefinedWorkspace('Private', DBName, true);

        // verify that after changing WS from private to public, imported dashboard should be in public workspace
        connections.selectConnectionByName(conName);
        cy.get(imp_exp.locators.import_toggle).click();
        cy.get(imp_exp.locators.import_artifacts).click();
        cy.wait('@workspaceLoaded')
          .its('response.statusCode')
          .should('eq', 200);
        cy.get(workspace.locators.selected_ws_on_import).should('have.text', 'Private');
        cy.get("select[name='workspace_id']").select('Public', { force: true });
        importFile(DBName + '.json');
        cy.get(imp_exp.locators.import_button).click();
        cy.get(workspace.locators.planned_action).should('have.text', 'Move Workspace');
        cy.get(imp_exp.locators.import_confirmation).click();
        cy.wait('@importDash')
          .its('response.statusCode')
          .should('eq', 200);
        cy.wait(2000);
        workspace.verifyUserDefinedWorkspace('Public', DBName, true);
        workspace.verifyUserDefinedWorkspace('Private', DBName, false);

        // verify that after importing dashboard into user-defined WS from database admin user
        // it should be placed in respective WS
        logins.silent(data.dbadmin.username, data.dbadmin.password);
        imp_exp.import_artifacts(conName, DBName + '.json');
        cy.wait(3000);
        workspace.verifyUserDefinedWorkspace('Private', DBName, true);

        // import dashboard into user defined workspace and verify that imported dashboard
        connections.selectConnectionByName(conName);
        cy.get(imp_exp.locators.import_toggle).click();
        cy.get(imp_exp.locators.import_artifacts).click();
        cy.wait('@workspaceLoaded')
          .its('response.statusCode')
          .should('eq', 200);
        cy.get("select[name='workspace_id']").select(userDefinedWs[0], { force: true });
        importFile(DbForUserDefineWs[0] + '.json');
        cy.get(imp_exp.locators.import_button).click();
        cy.get(imp_exp.locators.import_confirmation).click();
        cy.wait('@importDash')
          .its('response.statusCode')
          .should('eq', 200);
        cy.wait(2000);
        workspace.verifyUserDefinedWorkspace(userDefinedWs[0], DbForUserDefineWs[0], true);

        // verify that after importing dashboard into user-defined WS from system admin user
        // it should be placed in respective WS
        logins.silent(data.sysadmin.username, data.sysadmin.password);
        imp_exp.import_artifacts(conName, DBName1 + '.json');
        cy.wait(3000);
        workspace.verifyUserDefinedWorkspace('Private', DBName, true);

        // import dashboard into user defined workspace and verify that imported dashboard
        connections.selectConnectionByName(conName);
        cy.get(imp_exp.locators.import_toggle).click();
        cy.get(imp_exp.locators.import_artifacts).click();
        cy.wait('@workspaceLoaded')
          .its('response.statusCode')
          .should('eq', 200);
        cy.get("select[name='workspace_id']").select(userDefinedWs[1], { force: true });
        importFile(DbForUserDefineWs[1] + '.json');
        cy.get(imp_exp.locators.import_button).click();
        cy.get(imp_exp.locators.import_confirmation).click();
        cy.wait('@importDash')
          .its('response.statusCode')
          .should('eq', 200);
        cy.wait(2000);
        workspace.verifyUserDefinedWorkspace(userDefinedWs[1], DbForUserDefineWs[1], true);
      });
    });
  });
});
