// This script contains verification on test https://arcadiadata.atlassian.net/browse/ARC-8157
import { notifications } from 'fixtures/locators/notifications';
import * as apps from 'lib/apps';
import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import * as functions from 'lib/functions';
import * as workspace from 'lib/workspace';
import { CSSselector } from 'locators/common/CSSselector';

let appId: number;
//   verifies the dashbaord of public workspace.
// If appname is specified at the time of calling this, it will verifies the app in public as well.
function verifyPublicWorkspace(dashname: string, shouldContain: boolean, appName?: string) {
  // Set aliases to validate or wait on
  functions.navigate_to_visuals();
  cy.get(CSSselector.workspace_regression.Public_workspace).click();
  cy.get(workspace.locators.dashboard_radio_btn).click();
  if (shouldContain === true) {
    cy.contains(dashname);
    if (appName) {
      cy.get(workspace.locators.app_radio_btn).click();
      cy.contains(appName);
    }
  } else {
    cy.get('body').should('not.contain', dashname);
    if (appName) {
      cy.get(workspace.locators.app_radio_btn).click();
      cy.get('body').should('not.contain', appName);
    }
  }
}

//   verifies the dashbaord of private workspace.
// If appname is specified at the time of calling this, it will verifies the app in private  as well.
function verifyPrivateWorkspace(dashname: string, shouldContain: boolean, appName?: string) {
  // Set aliases to validate or wait on
  functions.navigate_to_visuals();
  cy.get(CSSselector.Private_WS).click();
  cy.get(workspace.locators.dashboard_radio_btn).click();
  if (shouldContain === true) {
    cy.contains(dashname);
    if (appName) {
      cy.get(workspace.locators.app_radio_btn).click();
      cy.contains(appName);
    }
  } else {
    cy.get('body').should('not.contain', dashname);
    if (appName) {
      cy.get(workspace.locators.app_radio_btn).click();
      cy.get('body').should('not.contain', appName);
    }
  }
}

// rewrite deleteDashboard function and put more routes as well as waits, remove few clicks as per requirement.
function deleteDashboard(dashname: string) {
  functions.typeInSearchField('"' + dashname + '"');
  cy.wait(1000);
  cy.get(workspace.locators.grid_dashboard_list).each(($el) => {
    if ($el.text().trim() === dashname) {
      console.log('title was found');
      cy.wrap($el)
        .parent()
        .within(() => {
          cy.get('span.btn-sel').should('have.class', 'btn-off');
          cy.get('span.btn-sel')
            .invoke('trigger', 'onpointerenter')
            .click({
              force: true,
            });
        });
      cy.get('li#delete-apps-btn').click();

      cy.get('#delete-confirmation-modal div.modal-body').should(
        'contain',
        'To confirm, type the word DELETE in the field below and then click the Delete button.'
      );
      cy.get('#delete-confirmation-text').type('DELETE');
      cy.intercept('DELETE', '/arc/reports/report/*').as('reportDeleted');
      cy.intercept('GET', '/arc/reports/report').as('reportLoaded');
      cy.intercept('GET', '/arc/workspace/workspace').as('workspaceLoaded');
      cy.get('#delete-confirmation-yes').click();
      cy.wait('@reportDeleted')
        .its('response.statusCode')
        .should('be.eq', 200);
      cy.wait('@reportLoaded')
        .its('response.statusCode')
        .should('be.eq', 200);
      cy.wait('@workspaceLoaded')
        .its('response.statusCode')
        .should('be.eq', 200);
    }
  });
}

// rewrite the existing fuction because we simply need to clone a dashbaord, nothing else.
function dbMoveToWS(workspaceName: string) {
  cy.get(
    `${CSSselector.dashboardDetail.menu} ${CSSselector.dashboardDetail.workspaceMenu}`
  ).click();
  cy.get(`${CSSselector.dashboardDetail.menu} [data-workspace-name="${workspaceName}"]`).click();
  cy.get(
    `${CSSselector.dashboardDetail.menu} ${CSSselector.dashboardDetail.workspaceMove}`
  ).click();
  cy.wait(2000);
  cy.get(notifications.alert_success).should('be.visible');
  cy.get(notifications.notification_close).click();
}

describe('Workspace tests.', () => {
  before(() => {
    logins.silent();
  });

  const timestamp = new Date().getTime();
  const user1Dash = `user1_${timestamp}_dashboard`;
  const user1App = `user1_${timestamp}_app`;

  const visualDetail = {
    connection_name: 'samples',
    dataset_name: 'Cereals',
    dataset_name1: 'InfoSecGlobal',
  };

  context('regression tests 4', () => {
    // Test Case ARC-8100

    it('1: ARC-8100: Checks for single visual changes', () => {
      functions.navigate_to_data();

      // Verify that visual icon should not exist in dataset tab
      cy.get(CSSselector.data.connection_list)
        .find('a[title="' + visualDetail.connection_name + '"]')
        .click();
      cy.get(CSSselector.data.dataset_table_row).each(($el, index, $list) => {
        cy.wrap($el)
          .find('[data-title="New Visual"]')
          .should('not.exist');
      });

      // Verify that visual icon should not exist in dataset page of first dataset in sample dataconnection
      cy.intercept('GET', '/arc/datasets/dataconnection').as('datasetPage');
      cy.get(CSSselector.data.dataset_table_row + ':first-child').click('left');
      cy.wait('@datasetPage')
        .its('response.statusCode')
        .should('be.eq', 200);
      cy.get(workspace.locators.dataset_page_header)
        .find('span.fa.sym-visual')
        .should('not.exist');

      // Verify that home stats does not contain visual test on home stats
      functions.navigate_to_home();
      cy.get(workspace.locators.home_stats).should('not.contain', 'Visuals were created');
    });

    // Test Case ARC-8093

    it('2: ARC-8093: Verify users access checked with private workspace', () => {
      // let Dashboardname = '';
      cy.intercept('POST', '/arc/reports/savedashboard').as('savedashboard');
      // Verify that dashboard and app created by admin user are displayed in admin private workspace.
      functions.navigate_to_visuals();
      cy.get(CSSselector.Private_WS).click();
      cy.get(workspace.locators.dashboard_radio_btn).click();
      cy.contains('admin_user_dashboard');
      cy.get(workspace.locators.app_radio_btn).click();
      cy.contains('admin_user_app');

      // Create a dashboard from user1 (non-admin user) and verify that dashboard and app created by admin user is not visible to user1.
      cy.fixture('users.json').then((data) => {
        logins.silent(data.user1.username, data.user1.password);
      });

      functions.addNewDB(user1Dash);
      functions.createNewVisual(visualDetail.connection_name, visualDetail.dataset_name);
      cy.get(CSSselector.Save).click();
      cy.wait('@savedashboard')
        .its('response.statusCode')
        .should('eq', 200);
      apps.createNewApp(user1App);
      apps.addDashboard(visualDetail.connection_name, visualDetail.dataset_name, user1Dash);
      apps.saveApp();
      verifyPublicWorkspace('admin_user_dashboard', false, 'admin_user_app');
      verifyPrivateWorkspace('admin_user_dashboard', false, 'admin_user_app');

      // verify that user1(non-admin user) should not see the app created by admin user.
      logins.silent();
      functions.navigate_to_visuals();
      cy.get(`${CSSselector.Visuals_all}`).click();
      cy.get(`${CSSselector.listview_Visualspage}`).click();
      cy.get(apps.locators.apps_list).click({ force: true });

      functions.typeInSearchField('"admin_user_app"');
      cy.wait(2000);
      cy.get(apps.locators.latest_created_app).then(($el) => {
        appId = $el.data('app');
        cy.fixture('users.json').then((data) => {
          logins.silent(data.user1.username, data.user1.password);
        });
        cy.visit('/arc/apps/appgroup/' + appId, { failOnStatusCode: false });
        cy.get('body').should(
          'contain',
          'You do not have the necessary permission on this workspace'
        );
      });

      // verify that admin can see user1's private workspace and dashboards.
      cy.visit('/');
      cy.visit('/');
      logins.silent();
      functions.navigate_to_visuals();
      cy.get(workspace.locators.all_private_ws).click();
      cy.get(workspace.locators.private_ws_list)
        .should('contain', 'user1 private workspace')
        .find('div[title="user1 private workspace"]')
        .click();
      cy.get(workspace.locators.dashboard_radio_btn).click();
      cy.contains(user1Dash);
      cy.get(workspace.locators.app_radio_btn).click();
      cy.contains(user1App);

      // verify that when non-admin user clone any public dashboard then clonned dashboard should be placed in user's private workspace.
      cy.fixture('users.json').then((data) => {
        logins.silent(data.user1.username, data.user1.password);
      });
      functions.navigate_to_visuals();
      cy.get(CSSselector.workspace_regression.Public_workspace).click();
      cy.get(workspace.locators.dashboard_radio_btn).click();
      dashboards.openDashboard('State of NYC');
      dashboards.dbClone();
      verifyPrivateWorkspace('Clone of State of NYC', true);

      // verify that when admin user clone user1's private WS dashboard then clonned dashboard should be placed in admin's private workspace
      logins.silent();
      functions.navigate_to_visuals();
      cy.get(workspace.locators.all_private_ws).click();
      cy.get(workspace.locators.private_ws_list)
        .should('contain', 'user1 private workspace')
        .find('div[title="user1 private workspace"]')
        .click();
      cy.get(workspace.locators.dashboard_radio_btn).click();
      cy.contains(user1Dash);
      dashboards.openDashboard(user1Dash);
      dashboards.dbClone();
      verifyPrivateWorkspace('Clone of ' + user1Dash, true);

      // verify that admin can delete dashboard of user1's private workspace.
      cy.get(workspace.locators.all_private_ws).click();
      cy.get(workspace.locators.private_ws_list)
        .should('contain', 'user1 private workspace')
        .find('div[title="user1 private workspace"]')
        .click();
      cy.get(workspace.locators.dashboard_radio_btn).click();
      deleteDashboard(user1Dash);
      functions.navigate_to_visuals();
      cy.get(workspace.locators.private_ws_list)
        .should('contain', 'user1 private workspace')
        .find('div[title="user1 private workspace"]')
        .click();
      cy.get('div#app-grid .appgrid-li').each(($name) => {
        cy.wrap($name).should('not.contain', user1Dash);
      });

      // verify that after moving a dashboard from user1's private workspace to public workspace, dashboard should be displayed to everyone.
      cy.fixture('users.json').then((data) => {
        logins.silent(data.user1.username, data.user1.password);
      });
      dashboards.openDashboard('Clone of State of NYC');
      dbMoveToWS('Public');
      verifyPublicWorkspace('Clone of State of NYC', true);

      logins.silent();
      verifyPublicWorkspace('Clone of State of NYC', true);

      // This code is not working as it is regression. Admin is unable to move dashboard from user's private workspave to any workspace from dashboard view/edit mode.
      // ARC- 10508
      // // verify that after moving a dashboard from user1's private workspace to public workspace using admin user, dashboard should be displayed to everyone.
      // cy.fixture('users.json').then((data) => {
      //   logins.silent(data.user1.username, data.user1.password);
      // });
      // functions.navigate_to_visuals();
      // cy.get(CSSselector.workspace_regression.Public_workspace).click();
      // cy.get(workspace.locators.dashboard_radio_btn).click();
      // dashboards.openDashboard('Cereal Comparisons');
      // dashboards.dbClone();
      // logins.silent();
      // dashboards.openDashboard('Clone of Cereal Comparisons');
      // dbMoveToWS('Public');
      // verifyPublicWorkspace('Clone of Cereal Comparisons', true);

      // cy.fixture('users.json').then((data) => {
      //   logins.silent(data.user2.username, data.user2.password);
      // });

      // });
      // verifyPublicWorkspace('Clone of Cereal Comparisons', true);

      // verify that admin user cannot move a public dashboard to private user's workspace.

      dashboards.openDashboard('Taxi rides application');
      cy.get(
        `${CSSselector.dashboardDetail.menu} ${CSSselector.dashboardDetail.workspaceMenu}`
      ).click();
      cy.get(
        `${CSSselector.dashboardDetail.menu} [data-workspace-name="user1 private workspace"]`
      ).should('not.be.exist');
      functions.navigate_to_visuals();

      // verify that user2 can move the public dashboard to it's private dashboard originally moved from user1's private workspace to public workspace.
      cy.fixture('users.json').then((data) => {
        logins.silent(data.user1.username, data.user1.password);
      });
      functions.navigate_to_visuals();
      cy.get(CSSselector.workspace_regression.Public_workspace).click();
      cy.get(workspace.locators.dashboard_radio_btn).click();
      dashboards.openDashboard('Global Threats');
      dashboards.dbClone();
      dashboards.openDashboard('Clone of Global Threats');
      dbMoveToWS('Public');
      cy.fixture('users.json').then((data) => {
        logins.silent(data.user2.username, data.user2.password);
      });
      dashboards.openDashboard('Clone of Global Threats');
      dbMoveToWS('Private');
      verifyPrivateWorkspace('Clone of Global Threats', true);

      // verify that after deleting user1 from system, its private workspace should be accessible by admin user
      logins.silent();
      functions.deleteUser('user1');
      functions.navigate_to_visuals();
      cy.get(".ws-menu-list [title='user1 private workspace']")
        .should('be.exist')
        .find('span.edit-icon')
        .should('be.exist');
    });

    //  Test-case ARC-8095

    it('3: ARC-8095: Verify users access checked with user-defined workspace', () => {
      cy.fixture('users.json').then((data) => {
        // verify that without "create workspace" permission, user can not create workspace
        logins.silent(data.user2.username, data.user2.password);
        functions.navigate_to_visuals();
        cy.get(workspace.locators.create_workspace).should('not.be.visible');

        // verify that user with workspace access can create/edit/delete user defined workspace.
        logins.silent(data.wk_user.username, data.wk_user.password);

        functions.navigate_to_visuals();
        cy.get(workspace.locators.create_workspace).should('be.visible');
        workspace.createWorkSpace('User Defined Workspace');
        cy.get(
          CSSselector.workspace_regression.workspace_menulist +
            ' li div[title="User Defined Workspace"]'
        ).should('be.visible');
        cy.get(CSSselector.workspace_regression.workspace_menulist)
          .find('div[title="User Defined Workspace"] span.edit-icon')
          .click();
        cy.get(CSSselector.workspace_regression.WSEditModal).should('be.visible');
        cy.get(`${CSSselector.workspace.wsEditModal} ${CSSselector.workspace.wsName}`)
          .clear()
          .type('User Defined Workspace Modified', { force: true });
        workspace.saveWorkSpace();
        workspace.deleteWorkSpace('User Defined Workspace Modified');

        // verify that admin can move the dashboard from user defined workspace to private workspace
        workspace.createWorkSpace('User Defined Workspace');
        dashboards.openDashboard('Census Dashboard');
        dbMoveToWS('User Defined Workspace');
        // This code is not working as it is regression. Admin is unable to move dashboard from user's private workspave to any workspace from dashboard view/edit mode.
        // ARC- 10508
        // logins.silent();
        // dashboards.openDashboard('Census Dashboard');
        // dbMoveToWS('Private');
        // verifyPrivateWorkspace('Census Dashboard', true);

        // verify that after moving dashboard from user defined workspace to public workspace as non-admin user, everyone can access that dashboard.
        logins.silent(data.wk_user.username, data.wk_user.password);
        dashboards.openDashboard('Life Expectancy Dashboard');
        dbMoveToWS('User Defined Workspace');
        dashboards.openDashboard('Life Expectancy Dashboard');
        dbMoveToWS('Public');
        logins.silent(data.user2.username, data.user2.password);
        verifyPublicWorkspace('Life Expectancy Dashboard', true);
        logins.silent();
        verifyPublicWorkspace('Life Expectancy Dashboard', true);

        // verify that user with 'manage workspace' access can add another user and check the feature by logging through added user.
        logins.silent(data.wk_user.username, data.wk_user.password);
        functions.navigate_to_visuals();
        cy.get(CSSselector.workspace_regression.workspace_menulist)
          .find('div[title="User Defined Workspace"] span.edit-icon')
          .click();
        cy.get(CSSselector.workspace_regression.username_textbox).type('user2{enter}');
        cy.get('.add-more-acl').click({ force: true });
        cy.get('.table.form-horizontal tbody tr:nth-child(2) input[value="VIEW"]').click({
          force: true,
        });
        workspace.saveWorkSpace();
        logins.silent(data.user2.username, data.user2.password);
        workspace.verifyWorkSpaceVisibility('User Defined Workspace', false);

        logins.silent(data.wk_user.username, data.wk_user.password);
        functions.navigate_to_visuals();
        cy.get(CSSselector.workspace_regression.workspace_menulist)
          .find('div[title="User Defined Workspace"] span.edit-icon')
          .click();
        cy.get('.table.form-horizontal tbody tr:nth-child(2) input[value="EDIT"]').click({
          force: true,
        });
        workspace.saveWorkSpace();
        logins.silent(data.user2.username, data.user2.password);
        workspace.verifyWorkSpaceVisibility('User Defined Workspace', true);
        cy.wait(1000); // put wait to close the 'Move To Workspace' popup
        // cy.get(workspace.locators.move_to_ws_button).should('contain', 'Move to Workspace').click({ force: true });
        cy.get('div.popover-content li[data-workspace-name="Private"]').click();
        cy.get(workspace.locators.save_workspace_move).click();
        cy.wait('@workspaceLoaded')
          .its('response.statusCode')
          .should('eq', 200);
        verifyPrivateWorkspace('User Defined Workspace', true);

        logins.silent(data.wk_user.username, data.wk_user.password);
        functions.navigate_to_visuals();
        cy.get(CSSselector.workspace_regression.workspace_menulist)
          .find('div[title="User Defined Workspace"] span.edit-icon')
          .click();
        cy.get('.table.form-horizontal tbody tr:nth-child(2) input[value="MANAGE"]').click({
          force: true,
        });
        workspace.saveWorkSpace();
        logins.silent(data.user2.username, data.user2.password);
        functions.navigate_to_visuals();
        cy.get(CSSselector.workspace_regression.workspace_menulist)
          .find('div[title="User Defined Workspace"] span:nth-child(2)')
          .should('have.not.class', 'arc-invisible')
          .should('have.class', 'edit-icon')
          .click();
        cy.get(CSSselector.workspace_regression.WSEditModal).should('be.visible');
        cy.get(CSSselector.workspace_regression.WSEditModal)
          .find("button[class='btn btn-default']")
          .should('have.text', 'Close')
          .click();

        // verify that if user with 'manage workspace' access tries to delete the workspace which has dashboard in it then workspace should not get deleted and warning message should appear.
        dashboards.openDashboard('Clone of State of NYC');
        dbMoveToWS('User Defined Workspace');
        functions.navigate_to_visuals();
        cy.get(
          CSSselector.workspace_regression.workspace_menulist +
            ' li div[title="User Defined Workspace"]'
        ).should('be.visible');
        cy.get(CSSselector.workspace_regression.workspace_menulist)
          .find('div[title="User Defined Workspace"] span.edit-icon')
          .click();
        cy.get(CSSselector.workspace_regression.WSEditModal)
          .should('be.visible')
          .find('.ws-delete-button')
          .should('be.visible')
          .click();
        cy.get('#delete-confirmation-modal div.modal-body').should(
          'contain',
          'To confirm, type the word DELETE in the field below and then click the Delete button.'
        );
        cy.get('#delete-confirmation-text').type('DELETE');
        cy.get('#delete-confirmation-yes').click();
        cy.intercept('DELETE', '/arc/workspace/workspace/*').as('workspaceDeleted');
        cy.wait('@workspaceDeleted')
          .its('response.statusCode')
          .should('be.eq', 500);
        cy.contains('Failed to delete workspace: Unable to remove non-empty workspace');

        // verify that either user with 'manage workspace' access or admin can delete empty workspace.
        deleteDashboard('Clone of State of NYC');

        // need to delete dashboard due to regression error of moving dashboard from user's private ws to any ws.
        deleteDashboard('Census Dashboard');

        workspace.deleteWorkSpace('User Defined Workspace');
        logins.silent(data.wk_user.username, data.wk_user.password);
        workspace.createWorkSpace('User Defined Workspace');
        logins.silent();
        workspace.deleteWorkSpace('User Defined Workspace');

        // verify that after clicking on cancel button on UI modal, it should disappeared.
        // check on Create Workspace Modal
        cy.get(`${CSSselector.workspace.createWorkspaceIcon}`).click();
        cy.get(CSSselector.workspace_regression.WSEditModal).should('be.visible');
        cy.get(CSSselector.workspace_regression.WSEditModal)
          .find("button[class='btn btn-default']")
          .should('have.text', 'Close')
          .click();
        cy.get(CSSselector.workspace_regression.WSEditModal).should('not.be.visible');
        // check on Public Workspace Modal
        cy.get(CSSselector.workspace_regression.workspace_menulist)
          .find('div[title="Public"] span.edit-icon')
          .click();
        cy.get(CSSselector.workspace_regression.WSEditModal).should('be.visible');
        cy.get(CSSselector.workspace_regression.WSEditModal)
          .find("button[class='btn btn-default']")
          .should('have.text', 'Close')
          .click();
        cy.get(CSSselector.workspace_regression.WSEditModal).should('not.be.visible');

        // verify that user cannot create duplicate workspace
        cy.get(`${CSSselector.workspace.createWorkspaceIcon}`).click();
        cy.get(
          `${CSSselector.workspace.wsEditModal} ${CSSselector.workspace.wsName}`
        ).type('Public', { force: true });
        workspace.saveWorkSpace();
        cy.get(`${CSSselector.workspace.wsEditModal} ${CSSselector.workspace.wsName}`)
          .next()
          .should('have.class', 'with-errors')
          .should('not.have.class', 'arc-invisible')
          .should('have.text', 'Duplicate workspace name not allowed');
        cy.get(CSSselector.workspace_regression.WSEditModal)
          .find("button[class='btn btn-default']")
          .should('have.text', 'Close')
          .click();
      });
    });
  });
});
