// This script contains verification on test https://arcadiadata.atlassian.net/browse/ARC-8158
import { logins } from 'lib/arctest';
import * as dashboards from 'lib/dashboard';
import * as functions from 'lib/functions';
import * as workspace from 'lib/workspace';
import { CSSselector } from 'locators/common/CSSselector';

describe('Workspace tests.', () => {
  const timestamp = new Date().getTime();
  const dashboardName = `dash_${timestamp}_ARC`;

  function clickEditdashboard() {
    cy.intercept('GET', '/arc/reports/customcss').as('newDashboard_customcss');
    cy.intercept('GET', '/arc/datasets/dataconnection').as('newDashboard_dataconnection');
    // cy.intercept('GET', '/arc/workspace/workspace').as('newDashboard_workspace');
    cy.intercept('GET', '/arc/reports/report/*').as('newDashboard_report');
    cy.intercept('POST', '/arc/sqlrun/jsonselect_parallel_async/*').as('newDashboard_jsonselect');

    cy.get('#report-menu-list .editapp-link').click();

    cy.wait('@newDashboard_customcss');
    cy.wait('@newDashboard_dataconnection');
    // removed in https://bitbucket.org/arcadiadata/arcviz/commits/cbf059d5a1d01a35ec50c875144f14666f940235
    // cy.wait('@newDashboard_workspace');
    cy.wait('@newDashboard_report');
    cy.wait('@newDashboard_jsonselect');
  }
  describe('Workspace regression tests', () => {
    // Test Case ARC-8082

    it('ARC-8082: Check Default Public Workspace-- Admin user', () => {
      cy.fixture('users.json').then((data) => {
        logins.silent(data.admin2.username, data.admin2.password);
      });
      let Dashboardname = '';
      functions.addNewDB(dashboardName);
      functions.createNewVisual('samples', 'Cereals');
      dashboards.saveDashboard();
      functions.navigate_to_visuals();

      cy.get(CSSselector.workspace_regression.workspace_menulist)
        .find('> li span[title="Private"]')
        .should(($el) => {
          expect($el).to.exist;
        })
        .parent()
        .find('.badge')
        .should(($el) => {
          let numerOfDashboard: number;
          numerOfDashboard = Number($el.text());
          assert.isAbove(numerOfDashboard, 0, 'Dashboard is added in private workspace');
        })
        .parents('ul')
        .find('> li span[title="Public"]')
        .click();

      cy.get(CSSselector.apps.applist_btn).click();
      cy.intercept('GET', '/arc/workspace/workspace').as('movews');
      cy.intercept('POST', '/arc/reports/move').as('movews2');

      cy.get(CSSselector.apps.dashboardcheckbox)
        .first()
        .click()
        .parents('tr')
        .find('.table-app-title')
        .then(($el) => {
          Dashboardname = $el.text();
          cy.get('li.move-viz-btn')
            .click()
            .next('div.workspace-move-popover')
            .should(($elm) => {
              expect($elm).to.exist;
            })
            .find('[data-workspace-name="Private"]')
            .click();

          cy.get('.popover-content .save-workspace-move').click();
          cy.wait('@movews2')
            .its('response.statusCode')
            .should('eq', 200);
          cy.wait('@movews')
            .its('response.statusCode')
            .should('eq', 200);

          // functions.navigate_to_visuals();
          // cy.get(CSSselector.homepage).click();
          cy.reload();
          // we had to put explicit wait so that the dashboard can be moved to Private workspace
          // cy.wait(3000);
          functions.navigate_to_visuals();
          cy.get(CSSselector.workspace_regression.workspace_menulist)
            .find('> li span[title="Private"]')
            .click();
          cy.get(CSSselector.workspace_regression.Applist_btn).click();

          cy.get(CSSselector.apps.dashboardcheckbox)
            .parents('tr')
            .find(`[title="${Dashboardname}"]`)
            .should(($elm) => {
              expect($elm).to.exist;
            });
        });
    });

    // Test Case ARC-8090

    it('ARC-8090: Verify users access with public workspace-- Visual Consumer', () => {
      cy.fixture('users.json').then((data) => {
        logins.silent(data.visualconsumer.username, data.visualconsumer.password);
      });
      // cy.get(CSSselector.workspace_regression.WhatsNew_popup).then(($el) => {
      //   if ($el.is(':visible')) {
      //     cy.wrap($el)
      //       .find(CSSselector.workspace_regression.Gotit_button)
      //       .click();
      //   }
      // });

      // functions.navigate_to_visuals(); cannot use this as it looks for dashboard button

      cy.intercept('GET', '/arc/reports/report').as('reportLoaded');
      cy.intercept('GET', '/arc/reports/report_usage').as('report_usageLoaded');
      cy.intercept('GET', '/arc/reports/appgroup').as('appgroupReportLoaded');
      cy.intercept('GET', '/arc/workspace/workspace').as('workspaceLoaded');

      cy.visit('/arc/apps/apps');

      cy.wait('@reportLoaded');

      cy.wait('@report_usageLoaded');
      cy.wait('@appgroupReportLoaded');
      cy.wait('@workspaceLoaded');

      // click all

      cy.get('div.side-menu-wrapper span.app-group')
        .should('be.visible')
        .contains('All')
        .click();

      cy.get(CSSselector.Visuals_all)
        .parent()
        .parent()
        .parent()
        .should('have.class', 'active');

      cy.get(CSSselector.apps.favourites_tab).click();
      cy.get(CSSselector.Visuals_all).click();
      cy.get(CSSselector.workspace_regression.Public_workspace).click();
      cy.get(CSSselector.apps.applist_btn).click();

      cy.get(CSSselector.apps.dashboardcheckbox)
        .first()
        .parents('tr')
        .click();
      cy.get(CSSselector.edit_link).then(($el) => {
        expect($el).to.be.hidden;
      });
      cy.get(CSSselector.workspace_regression.Dashboard_edit_ellipsis)
        .click()
        .parents('li')
        .find(CSSselector.workspace_regression.Dashboardedit_dropdown)
        .then(($el) => {
          expect($el).to.be.visible;
          let isCloneAvailable = false;
          cy.wrap($el)
            .find('li:not(.arc-invisible)')
            .each(($liel, index, $list) => {
              if ($liel.text().trim() === 'Clone') {
                isCloneAvailable = true;
              }
              assert.isNotOk(isCloneAvailable, 'this will pass');
            });
          let isDeleteDashboardAvailable = false;
          cy.wrap($el)
            .find('li:not(.arc-invisible)')
            .each(($liel, index, $list) => {
              if ($liel.text().trim() === 'Delete Dashboard') {
                isDeleteDashboardAvailable = true;
              }
              assert.isNotOk(isDeleteDashboardAvailable, 'this will pass');
            });
        });
    });

    it('ARC-8090: Verify users access with public workspace-- Analyst', () => {
      cy.fixture('users.json').then((data) => {
        logins.silent(data.analyst.username, data.analyst.password);
      });
      // cy.get(CSSselector.workspace_regression.WhatsNew_popup).then(($el) => {
      //   if ($el.is(':visible')) {
      //     cy.wrap($el)
      //       .find(CSSselector.workspace_regression.Gotit_button)
      //       .click();
      //   }
      // });
      functions.navigate_to_visuals();
      cy.get(CSSselector.apps.favourites_tab).click();
      cy.get(CSSselector.Visuals_all).click();
      cy.get(CSSselector.workspace_regression.Public_workspace).click();
      cy.get(CSSselector.apps.applist_btn).click();

      let DashboardName = '';
      cy.get(CSSselector.apps.dashboardcheckbox)
        .first()
        .parents('tr')
        .find('[title]')
        .then(($el) => {
          DashboardName = $el.text();
          cy.wrap($el)
            .parents('tr')
            .click();
          cy.get(CSSselector.edit_link).then(($elm) => {
            expect($elm).to.be.visible;
          });
          clickEditdashboard();
          // cy.get(CSSselector.edit_dashboard).click();

          cy.get(CSSselector.workspace_regression.Dashboard_edit_ellipsis)
            .click()
            .parents('li')
            .find(CSSselector.workspace_regression.Dashboardedit_dropdown)
            .then(($elm) => {
              expect($elm).to.be.visible;
              let isDeleteDashboardAvailable = false;
              cy.wrap($elm)
                .find('a[class="delete-db-link"]')
                .each(($liel, index, $list) => {
                  if ($liel.text().trim() === 'Delete Dashboard') {
                    isDeleteDashboardAvailable = true;
                  }
                  assert.isOk(isDeleteDashboardAvailable, 'this will pass');
                });
              cy.get(CSSselector.workspace_regression.Deletedashboard_option).click();
              cy.get(CSSselector.workspace_regression.Deletedashboardconfirm).click();
            });

          cy.get(CSSselector.apps.dashboardcheckbox)
            .first()
            .parents('tr')
            .find('[title]')
            .then(($elm) => {
              let isDashboardAvailable = true;
              if ($elm.text() === DashboardName) {
                isDashboardAvailable = false;
              }
              assert.isOk(isDashboardAvailable, 'this will pass');
            });
        });
    });

    it('ARC-8090: Verify users access with public workspace-- DBAdmin', () => {
      cy.fixture('users.json').then((data) => {
        logins.silent(data.dbadmin.username, data.dbadmin.password);
      });
      // cy.get(CSSselector.workspace_regression.WhatsNew_popup).then(($el) => {
      //   if ($el.is(':visible')) {
      //     cy.wrap($el)
      //       .find(CSSselector.workspace_regression.Gotit_button)
      //       .click();
      //   }
      // });
      functions.navigate_to_visuals();
      cy.get(CSSselector.apps.favourites_tab).click();
      cy.get(CSSselector.Visuals_all).click();
      cy.get(CSSselector.workspace_regression.Public_workspace).click();
      cy.get(CSSselector.apps.applist_btn).click();

      let DashboardName = '';
      cy.get(CSSselector.apps.dashboardcheckbox)
        .first()
        .parents('tr')
        .find('[title]')
        .then(($el) => {
          DashboardName = $el.text();
          cy.wrap($el)
            .parents('tr')
            .click();
          cy.get(CSSselector.edit_link).then(($elm) => {
            expect($elm).to.be.visible;
          });
          // cy.get(CSSselector.edit_dashboard).click();
          clickEditdashboard();
          cy.get(CSSselector.workspace_regression.Dashboard_edit_ellipsis)
            .click()
            .parents('li')
            .find(CSSselector.workspace_regression.Dashboardedit_dropdown)
            .then(($elm) => {
              expect($elm).to.be.visible;
              let isDeleteDashboardAvailable = false;
              cy.wrap($elm)
                .find('a[class="delete-db-link"]')
                .each(($liel, index, $list) => {
                  if ($liel.text().trim() === 'Delete Dashboard') {
                    isDeleteDashboardAvailable = true;
                  }
                  assert.isOk(isDeleteDashboardAvailable, 'this will pass');
                });

              cy.get(CSSselector.workspace_regression.Deletedashboard_option).click();
              cy.get(CSSselector.workspace_regression.Deletedashboardconfirm).click();
            });
          cy.get(CSSselector.apps.dashboardcheckbox)
            .first()
            .parents('tr')
            .find('[title]')
            .then(($elm) => {
              let isDashboardAvailable = true;
              if ($elm.text() === DashboardName) {
                isDashboardAvailable = false;
              }
              assert.isOk(isDashboardAvailable, 'this will pass');
            });
        });
    });

    it('ARC-8090: Verify users access with public workspace-- SysAdmin', () => {
      cy.fixture('users.json').then((data) => {
        logins.silent(data.sysadmin.username, data.sysadmin.password);
      });
      // cy.get(CSSselector.workspace_regression.WhatsNew_popup).then(($el) => {
      //   if ($el.is(':visible')) {
      //     cy.wrap($el)
      //       .find(CSSselector.workspace_regression.Gotit_button)
      //       .click();
      //   }
      // });
      functions.navigate_to_visuals();
      cy.get(CSSselector.apps.favourites_tab).click();
      cy.get(CSSselector.Visuals_all).click();
      cy.get(CSSselector.workspace_regression.Public_workspace).click();
      cy.get(CSSselector.apps.applist_btn).click();

      let DashboardName = '';
      cy.get(CSSselector.apps.dashboardcheckbox)
        .first()
        .parents('tr')
        .find('[title]')
        .then(($el) => {
          DashboardName = $el.text();
          cy.wrap($el)
            .parents('tr')
            .click();
          cy.get(CSSselector.edit_link).then(($elm) => {
            expect($elm).to.be.visible;
          });
          // cy.get(CSSselector.edit_dashboard).click();
          clickEditdashboard();
          cy.get(CSSselector.workspace_regression.Dashboard_edit_ellipsis)
            .click()
            .parents('li')
            .find(CSSselector.workspace_regression.Dashboardedit_dropdown)
            .then(($elm) => {
              expect($elm).to.be.visible;
              let isDeleteDashboardAvailable = false;
              cy.wrap($elm)
                .find('a[class="delete-db-link"]')
                .each(($liel, index, $list) => {
                  if ($liel.text().trim() === 'Delete Dashboard') {
                    isDeleteDashboardAvailable = true;
                  }
                  assert.isOk(isDeleteDashboardAvailable, 'this will pass');
                });

              cy.get(CSSselector.workspace_regression.Deletedashboard_option).click();
              cy.get(CSSselector.workspace_regression.Deletedashboardconfirm).click();
            });
          cy.get(CSSselector.apps.dashboardcheckbox)
            .first()
            .parents('tr')
            .find('[title]')
            .then(($elm) => {
              let isDashboardAvailable = true;
              if ($elm.text() === DashboardName) {
                isDashboardAvailable = false;
              }
              assert.isOk(isDashboardAvailable, 'this will pass');
            });
        });
    });

    // Test Case ARC-8092

    it('ARC-8092: Verify ACL of Public Workspace for Admin user-- Admin user', () => {
      // Admin
      cy.fixture('users.json').then((data) => {
        logins.silent(data.admin.username, data.admin.password);
      });
      const Dashboardname = '';
      functions.navigate_to_visuals();
      cy.get(CSSselector.apps.favourites_tab).click();
      cy.get(`${CSSselector.Visuals_all}`).click();

      cy.get(CSSselector.workspace_regression.public_gear_icon).then(($el) => {
        expect($el).to.be.visible;
      });
      cy.get(CSSselector.workspace_regression.public_gear_icon).click({ force: true });
      cy.get(CSSselector.workspace_regression.view_permission).click({ force: true });
      workspace.saveWorkSpace();

      cy.get(CSSselector.apps.favourites_tab).click();
      cy.get(`${CSSselector.Visuals_all}`).click();
      cy.get(CSSselector.workspace_regression.public_gear_icon).click({ force: true });
      cy.get(CSSselector.workspace_regression.WSEditModal)
        .should('be.visible')
        .then(($el) => {
          cy.wrap($el)
            .find(CSSselector.workspace_regression.wsname)
            .then(($elm) => {
              expect($elm).to.have.prop('disabled', true);
            });
          cy.wrap($el)
            .find(CSSselector.workspace_regression.wsdesc)
            .then(($elm) => {
              expect($elm).to.have.prop('disabled', true);
            });
          cy.wrap($el)
            .find('.ws-delete-button')
            .then(($elm) => {
              expect($elm).to.be.hidden;
            });
          cy.wrap($el)
            .find('.close')
            .click();
        });

      // Analyst
      cy.fixture('users.json').then((data) => {
        logins.silent(data.analyst.username, data.analyst.password);
      });
      functions.navigate_to_visuals();
      cy.get(CSSselector.apps.favourites_tab).click();
      cy.get(CSSselector.Visuals_all).click();
      cy.get(CSSselector.workspace_regression.Public_workspace)
        .click()
        .within(($publicTab) => {
          cy.get('.edit-icon').should('not.be.visible');
        });
      // .find('.edit-icon ')
      // .then(($el) => {
      //   expect($el).to.be.hidden;
      // });
      cy.get('.app-grid-list li')
        .first()
        .click();
      cy.get(CSSselector.edit_link).then(($el) => {
        expect($el).to.be.hidden;
      });

      cy.fixture('users.json').then((data) => {
        logins.silent(data.dbadmin.username, data.dbadmin.password);
      });
      functions.navigate_to_visuals();
      cy.get(CSSselector.apps.favourites_tab).click();
      cy.get(CSSselector.Visuals_all).click();
      cy.get(CSSselector.workspace_regression.Public_workspace)
        .click()
        .within(($publicTab) => {
          cy.get('.edit-icon').should('not.be.visible');
        });

      // .find('.edit-icon ')
      // .then(($el) => {
      //   expect($el).to.be.hidden;
      // });
      cy.get('.app-grid-list li')
        .first()
        .click();
      cy.get(CSSselector.edit_link).then(($el) => {
        expect($el).to.be.hidden;
      });

      cy.fixture('users.json').then((data) => {
        logins.silent(data.admin.username, data.admin.password);
      });
      functions.navigate_to_visuals();
      cy.get(CSSselector.apps.favourites_tab).click();
      cy.get(`${CSSselector.Visuals_all}`).click();
      cy.get(CSSselector.workspace_regression.public_gear_icon, { timeout: 10000 }).click();
      cy.get(CSSselector.workspace_regression.WSEditModal)
        .should('be.visible')
        .then(($el) => {
          cy.get(CSSselector.workspace_regression.manage_permission).click({ force: true });
          cy.intercept('POST', '/arc/workspace/workspace').as('jsonselect');

          workspace.saveWorkSpace();
        });

      cy.fixture('users.json').then((data) => {
        logins.silent(data.admin.username, data.admin.password);
      });

      functions.navigate_to_visuals();
      cy.get(CSSselector.apps.favourites_tab).click();
      cy.get(CSSselector.Visuals_all).click();

      cy.get(CSSselector.workspace_regression.Public_workspace)
        .click()
        .find('.edit-icon')
        .then(($el) => {
          expect($el).to.be.visible;
          cy.get(CSSselector.workspace_regression.public_gear_icon, {
            timeout: 10000,
          }).click();
          cy.get(CSSselector.workspace_regression.WSEditModal)
            .should('be.visible')
            .then(($elm) => {
              cy.wrap($elm)
                .find('.close')
                .click();
            });
        });
      cy.get('.app-grid-list li')
        .first()
        .click();
      cy.get(CSSselector.edit_link).then(($el) => {
        expect($el).to.be.visible;
      });
      clickEditdashboard();

      cy.fixture('users.json').then((data) => {
        logins.silent(data.dbadmin.username, data.dbadmin.password);
      });

      // cy.get(CSSselector.workspace_regression.WhatsNew_popup).then(($el) => {
      //   if ($el.is(':visible')) {
      //     cy.wrap($el)
      //       .find(CSSselector.workspace_regression.Gotit_button)
      //       .click();
      //   }
      // });
      functions.navigate_to_visuals();
      cy.get(CSSselector.apps.favourites_tab).click();
      cy.get(CSSselector.Visuals_all).click();
      cy.get(CSSselector.workspace_regression.Public_workspace)
        .click()
        .find('.edit-icon ')
        .then(($el) => {
          expect($el).to.be.visible;
          cy.get(CSSselector.workspace_regression.public_gear_icon, {
            timeout: 10000,
          }).click();
          cy.get(CSSselector.workspace_regression.WSEditModal)
            .should('be.visible')
            .then(($elm) => {
              cy.wrap($elm)
                .find('.close')
                .click();
            });
        });
      // cy.get(CSSselector.workspace_regression.Public_workspace)
      //   .click()
      //   .within(($publicTab) => {cy.get('.edit-icon').should('not.be.visible');});
      cy.get('.app-grid-list li')
        .first()
        .click();
      cy.get(CSSselector.edit_link).then(($el) => {
        expect($el).to.be.visible;
      });
      clickEditdashboard();

      cy.fixture('users.json').then((data) => {
        logins.silent(data.admin.username, data.admin.password);
      });

      functions.navigate_to_visuals();
      cy.get(CSSselector.apps.favourites_tab).click();
      cy.get(`${CSSselector.Visuals_all}`).click();

      cy.get(CSSselector.workspace_regression.public_gear_icon, { timeout: 10000 }).click();
      cy.get(CSSselector.workspace_regression.WSEditModal)
        .should('be.visible')
        .then(($el) => {
          cy.wrap($el)
            .find(CSSselector.workspace_regression.wspermissions_remove)
            .click();
          cy.intercept('POST', '/arc/workspace/workspace').as('jsonselect');

          workspace.saveWorkSpace();
        });

      cy.fixture('users.json').then((data) => {
        logins.silent(data.analyst.username, data.analyst.password);
      });

      // cy.get(CSSselector.workspace_regression.WhatsNew_popup).then(($el) => {
      //   if ($el.is(':visible')) {
      //     cy.wrap($el)
      //       .find(CSSselector.workspace_regression.Gotit_button)
      //       .click();
      //   }
      // });
      functions.navigate_to_visuals();
      cy.get(CSSselector.apps.favourites_tab).click();
      cy.get(CSSselector.Visuals_all).click();
      cy.get(CSSselector.workspace_regression.workspace_menulist).then(($el) => {
        expect($el.find(CSSselector.workspace_regression.Public_workspace)).not.to.exist;
      });

      cy.fixture('users.json').then((data) => {
        logins.silent(data.dbadmin.username, data.dbadmin.password);
      });
      // cy.get(CSSselector.workspace_regression.WhatsNew_popup).then(($el) => {
      //   if ($el.is(':visible')) {
      //     cy.wrap($el)
      //       .find(CSSselector.workspace_regression.Gotit_button)
      //       .click();
      //   }
      // });
      functions.navigate_to_visuals();
      cy.get(CSSselector.apps.favourites_tab).click();
      cy.get(CSSselector.Visuals_all).click();

      cy.get(CSSselector.workspace_regression.workspace_menulist).then(($el) => {
        expect($el.find(CSSselector.workspace_regression.Public_workspace)).not.to.exist;
      });

      const workspaceNameForAdmin = 'User Defined Workspace';
      const workspaceDescForAdmin = 'This is a description for user defined workspace';

      cy.fixture('users.json').then((data) => {
        logins.silent(data.admin.username, data.admin.password);
      });

      functions.navigate_to_visuals();
      cy.get(CSSselector.apps.favourites_tab).click();
      cy.get(`${CSSselector.Visuals_all}`).click();

      cy.get(`${CSSselector.workspace_regression.workspace_menulist}`)
        .prev()
        .find('.ws-list-edit')
        .click();
      cy.get(CSSselector.workspace_regression.WSEditModal).then(($el) => {
        cy.wrap($el)
          .find(CSSselector.workspace_regression.wsname)
          .clear()
          .type(workspaceNameForAdmin);
        cy.wrap($el)
          .find(CSSselector.workspace_regression.wsdesc)
          .clear()
          .type(workspaceDescForAdmin);
        cy.intercept('POST', '/arc/workspace/workspace').as('jsonselect');

        workspace.saveWorkSpace();
      });

      cy.get(
        'ul[class="app-menu ws-menu-list"] > li:nth-child(3) span[class="edit-workspace edit-icon "]'
      ).click();
      cy.get('.ws-delete-button').click();

      cy.get('#delete-confirmation-modal')
        .should('be.visible')
        .then(($el) => {
          cy.wrap($el)
            .find('#delete-confirmation-text')
            .clear()
            .type('DELETE')
            .parents('.modal-body')
            .next()
            .find('#delete-confirmation-yes')
            .click();
        });

      cy.get(`.ws-menu-list span[title='User Defined Workspace'] + span`).should(($el) => {
        expect($el).not.to.exist;
      });
    });
  });
});
