import { logins } from 'lib/arctest';

describe('Multiple reflected Cross-Site scripting', () => {
  before(() => {
    logins.silent();
  });

  context('Testing multiple reflected Cross-Site scripting', () => {
    it('VIZ-2648: Middleware ExceptionHandler class => process_exception() => general exception response returned xss', () => {
        // script that adds p with class xss to the dom
        const script = `var p = document.createElement("p"); var text = document.createTextNode("Reflected XSS!"); p.appendChild(text); p.className = "xss"; document.body.appendChild(p);`;
        const xssUrls = [
            {
                vulnerable_parameter: 'days',
                payload: `"/><script>${script}</script>`,
                url: function () {
                    return `/arc/apps/activeusers?days=30${this.payload}`;
                }
            },
            {
                vulnerable_parameter: 'dataconnection_id',
                payload: `"<script>${script}</script>`,
                url: function () {
                    return `/arc/sql/query?dataconnection_id=14${this.payload}&favorite=true`;
                }
            },
            {
                vulnerable_parameter: 'limit',
                payload: `'<script>${script}</script>`,
                url: function () {
                    return `/arc/apps/homepage_datasets?limit=30${this.payload}`;
                }
            },
            {
                vulnerable_parameter: 'limit',
                payload: `"<script>${script}</script>`,
                url: function () {
                    return `/arc/sql/query?get_type=all&limit=30${this.payload}&should_sort=false`;
                }
            },
            {
                vulnerable_parameter: 'columns[1][name]',
                payload: `usernameka1sf<script>${script}</script>py5lz`,
                url: function () {
                    return `/arc/apps/users_api?draw=5&columns[0][data]=0&columns[0][name]=&columns[0][searchable]=true&columns[0][orderable]=false&columns[0][search][value]=&columns[0][search][regex]=false&columns[1][data]=1&columns[1][name]=${this.payload}&columns[1][searchable]=true&columns[1][orderable]=true&columns[1][search][value]=&columns[1][search][regex]=false&columns[2][data]=2&columns[2][name]=is_superuser&columns[2][searchable]=true&columns[2][orderable]=true&columns[2][search][value]=&columns[2][search][regex]=false&columns[3][data]=3&columns[3][name]=date_joined&columns[3][searchable]=true&columns[3][orderable]=true&columns[3][search][value]=&columns[3][search][regex]=false&columns[4][data]=4&columns[4][name]=last_login&columns[4][searchable]=true&columns[4][orderable]=true&columns[4][search][value]=&columns[4][search][regex]=false&columns[5][data]=5&columns[5][name]=groups&columns[5][searchable]=true&columns[5][orderable]=true&columns[5][search][value]=&columns[5][search][regex]=false&columns[6][data]=6&columns[6][name]=roles&columns[6][searchable]=true&columns[6][orderable]=true&columns[6][search][value]=&columns[6][search][regex]=false&columns[7][data]=7&columns[7][name]=is_active&columns[7][searchable]=true&columns[7][orderable]=true&columns[7][search][value]=&columns[7][search][regex]=false&columns[8][data]=8&columns[8][name]=&columns[8][searchable]=true&columns[8][orderable]=false&columns[8][search][value]=&columns[8][search][regex]=false&order[0][column]=1&order[0][dir]=asc&start=0&length=10&search[value]=&search[regex]=false`;
                }
            },

        ];

        xssUrls.forEach((xssUrl) => {
            cy.visit(encodeURI(xssUrl.url()), {failOnStatusCode: false});
            // ensure the script didn't run
            cy.get('p.xss').should('not.exist');
        });
    });
  });
});