import { dataset } from 'fixtures/locators/dataset';
import { logins } from 'lib/arctest';
import { navigate_to_data, waitForAceEditorReady } from 'lib/functions';
import { getRandomInt } from 'tests/jobs/utils';

const datasetName = 'Stored XSS test ' + getRandomInt(0, 10000);

describe('Stored cross-site scripting', () => {
  before(() => {
    logins.silent();
  });

  context('Testing stored cross-site scripting', () => {
    it('creates a new dataset object using an SQL query containing an XSS payload', () => {
      const script = 'alert(2)';

      navigate_to_data();
      cy.get(dataset.samples_connection).click();
      cy.get(dataset.new_dataset_button).click();
      cy.get(dataset.dataset_title_input).type(datasetName);
      cy.get(dataset.dataset_source_select).click();
      cy.get(dataset.sql_source).type('sql');
      cy.get(dataset.sql_option).click();
      cy.get(dataset.sql_editor_container)
        .find('.ace_editor')
        .then((aceEditor) => {
          waitForAceEditorReady(aceEditor);
        })
        .then((aceEditor) => cy.wrap(aceEditor).type(`SELECT "<script>${script}</script>"`));
      cy.get(dataset.create_button).click();
      cy.get(dataset.dataset_title)
        .contains(datasetName)
        .click();
      cy.on('window:alert', (text) => {
        throw new Error(`Unexpected alert with message: ${text}`);
      });
      cy.reload();
      cy.on('window:alert', (text) => {
        throw new Error(`Unexpected alert with message: ${text}`);
      });
    });
  });
});
