from itertools import count
import os
import time
import re
import json
import logging
from typing import List
from unicodedata import name

from locust import HttpUser, TaskSet, task, between
from locust import events

import urllib3
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

import locust.stats
locust.stats.CURRENT_RESPONSE_TIME_PERCENTILE_WINDOW = 50 # default is 10
locust.stats.PERCENTILES_TO_REPORT = [0.50, 0.95]
# locust.stats.CSV_STATS_FLUSH_INTERVAL_SEC = 60
locust.stats.CSV_STATS_INTERVAL_SEC = 10


opts_string =  os.getenv('LOAD_TEST_OPTIONS','')
opts = opts_string.split('/')

def is_unset(opts, key):
  return False if key in opts else True

is_static_on=is_unset(opts, 'STATIC_OFF')
is_async_data_on=is_unset(opts, 'ASYNC_DATA_OFF')
is_other_on=is_unset(opts, 'OTHER_OFF')

print(f"LOAD_TEST_OPTIONS={opts_string}")
print(f"is_static_on={is_static_on}")
print(f"is_async_data_on={is_async_data_on}")
print(f"is_other_on={is_other_on}")

LOG = logging.getLogger('')

re1 = re.compile('csrfToken: "(.*?)"');
re_url = re.compile(r".*?://.*?(/.*)$")
re_v = re.compile(r"\?v=[0-9\.]+$")
# re_static_asset = re.compile('\.(js|css|png|jpeg|jpg|gif|svg|ico|ttf|woff|woff2|html|htm)$')
re_static_asset = re.compile('\/arc\/static\/')

def findToken(response):
  m = re1.search(response.text)
  if m:
    token = m.groups()[0]
    return token

  return None
class QuickstartUser(HttpUser):
  # @task
  def homepage(self):
    with open("locust_template_home.txt", "r") as f:
      for line in f:
        j = json.loads(line)
        method = j["method"]
        url = j["url"]
        m = re_url.search(url)
        url = m.groups(0)[0]

        if method == "GET":
          self.client.get(url, verify=False)

  def login(self):
    response = self.client.get('/arc/apps/login', verify=False)
    token = findToken(response)
    if not token:
      # print('here', response.text, 'done')
      raise Exception('no csrf token in login process')

    response = self.client.post('/arc/apps/login', verify=False,
        data={
          "csrfmiddlewaretoken": token,
          "next": "",
          "username": "vizapps_admin",
          "password": "vizapps_admin"
        })

  def on_start(self):
    self.login()

def handleAsync(self: TaskSet, headers, url, response):
  ITERATION_TIME = 0.1

  while True:
    try:
      # print('resp', response.text)
      jr = json.loads(response.text)
      qhandles = []
      if type(jr) == type([]):
        for item in jr:
          itemresponse = item.get("response")
          if itemresponse:
            qhandle = itemresponse.get('qhandle')
            if qhandle:
              qhandles.append(qhandle)

        # print('qhandles', qhandles)
        if qhandles:
          time.sleep(ITERATION_TIME)
          response = self.client.post('/arc/sqlrun/p1/asyncstatus_async',
            verify=False,
            headers=headers,
            data={
              "qhandles": ','.join(qhandles)
            })
          # print('here3', response.text)
          continue

        # print('done4', jr)
        return

      if type(jr) != type({}):
        # print('done1', jr)
        return

      if jr.get("type") == "async":
        time.sleep(ITERATION_TIME)
        response = self.client.post(url, verify=False,
            headers=headers,
            data={
              "async": json.dumps(jr)
            })
        continue

      # print('done2', jr)
      return

    except Exception as e:
      # print('done3', e)
      return

def create_task(dashboard_name: str, lines: List[str]):

  def dashboard(self: TaskSet):

    LOG.info('Starting test for dashboard %s', dashboard_name)

    storedToken = None
    response = None

    async_count = 0
    static_asset_time = 0
    static_asset_time2 = 0
    db_start_time = time.time()
    for line in lines:
      j = json.loads(line)
      method = j["method"]
      url = j["url"]
      m = re_url.search(url)
      url = m.groups(0)[0]
      url_name = re_v.sub("", url)

      url_name = f"{dashboard_name}_{url_name}"
      is_static = re_static_asset.search(url_name) != None

      if method == "GET":
        if is_static:
          if is_static_on:
            s_start=time.time()
            response = self.client.get(url, verify=False, name=url_name)
            static_asset_time2 += (time.time()-s_start)
            # this below does not seem to work
            # static_asset_time += response.elapsed.total_seconds()
        elif is_other_on:
            response = self.client.get(url, verify=False, name=url_name)

        if response:
          token = findToken(response)
          if token:
            storedToken = token

        continue

      if method == "POST":
        postData = j.get("post_data")
        headers = {
          'x-csrftoken': storedToken
        }

        if url.find('jsonselect_parallel_async') < 0:
          # print('ignoring', url)
          continue

        asyncCall = postData.get('async') is not None
        if not asyncCall:
          # print('processing', url)
          if not is_async_data_on:
            continue

          start_time = time.time()

          response = self.client.post(url, verify=False,
              headers=headers,
              data=postData)

          handleAsync(self, headers, url, response)

          total_time = int((time.time() - start_time) * 1000)
          events.request_success.fire(
              request_type="ASYNC",
              name=f"{dashboard_name}_jsonselect_parallel_async_{async_count}",
              response_time=total_time,
              response_length=0
          )
          async_count += 1

        else:
          # print('ignoring async', url)
          pass

        continue

      raise Exception(f"unhandled method {method} called")

    total_time = int((time.time() - db_start_time) * 1000)
    events.request_success.fire(
        request_type="DBLOAD",
        name=f"{dashboard_name}_TOTAL",
        response_time=total_time,
        response_length=0
    )

    static_asset_time = static_asset_time2*1000
    events.request_success.fire(
        request_type="DBLOAD",
        name=f"{dashboard_name}_TOTAL_STATIC",
        response_time=static_asset_time,
        response_length=0
    )

    # assume a browser will download 6 static assets in parallel:
    # total_estimated = total_time - static_asset_time + static_asset_time/6
    # events.request_success.fire(
    #     request_type="DBLOAD",
    #     name=f"{dashboard_name}_TOTAL_ESTIMATED",
    #     response_time=total_estimated,
    #     response_length=0
    # )


  return dashboard

def add_task(dashboard_file: str):
  with open(f"{dashboard_file}", "r") as f:
    lines = f.readlines()
    dashboard_name = dashboard_file.replace('.txt', '')
    dashboard_name = dashboard_name.replace('locust_template_', '')
    print(f"adding task for {dashboard_name}")
    QuickstartUser.tasks.append(create_task(dashboard_name, lines))


def create_tasks():
  print('looking for dashboard files')
  counter = 0
  for f in os.listdir():
      if f.endswith(".txt"):
          dashboard_file = f
          counter += 1
          add_task(dashboard_file)

  print(f"found {counter} files")


create_tasks()